/*!
 *                     GNU GENERAL PUBLIC LICENSE
 *                        Version 3, 29 June 2007
 * 
 *  Copyright (C) 2007 Free Software Foundation, Inc. <https://fsf.org/>
 *  Everyone is permitted to copy and distribute verbatim copies
 *  of this license document, but changing it is not allowed.
 * 
 *                             Preamble
 * 
 *   The GNU General Public License is a free, copyleft license for
 * software and other kinds of works.
 * 
 *   The licenses for most software and other practical works are designed
 * to take away your freedom to share and change the works.  By contrast,
 * the GNU General Public License is intended to guarantee your freedom to
 * share and change all versions of a program--to make sure it remains free
 * software for all its users.  We, the Free Software Foundation, use the
 * GNU General Public License for most of our software; it applies also to
 * any other work released this way by its authors.  You can apply it to
 * your programs, too.
 * 
 *   When we speak of free software, we are referring to freedom, not
 * price.  Our General Public Licenses are designed to make sure that you
 * have the freedom to distribute copies of free software (and charge for
 * them if you wish), that you receive source code or can get it if you
 * want it, that you can change the software or use pieces of it in new
 * free programs, and that you know you can do these things.
 * 
 *   To protect your rights, we need to prevent others from denying you
 * these rights or asking you to surrender the rights.  Therefore, you have
 * certain responsibilities if you distribute copies of the software, or if
 * you modify it: responsibilities to respect the freedom of others.
 * 
 *   For example, if you distribute copies of such a program, whether
 * gratis or for a fee, you must pass on to the recipients the same
 * freedoms that you received.  You must make sure that they, too, receive
 * or can get the source code.  And you must show them these terms so they
 * know their rights.
 * 
 *   Developers that use the GNU GPL protect your rights with two steps:
 * (1) assert copyright on the software, and (2) offer you this License
 * giving you legal permission to copy, distribute and/or modify it.
 * 
 *   For the developers' and authors' protection, the GPL clearly explains
 * that there is no warranty for this free software.  For both users' and
 * authors' sake, the GPL requires that modified versions be marked as
 * changed, so that their problems will not be attributed erroneously to
 * authors of previous versions.
 * 
 *   Some devices are designed to deny users access to install or run
 * modified versions of the software inside them, although the manufacturer
 * can do so.  This is fundamentally incompatible with the aim of
 * protecting users' freedom to change the software.  The systematic
 * pattern of such abuse occurs in the area of products for individuals to
 * use, which is precisely where it is most unacceptable.  Therefore, we
 * have designed this version of the GPL to prohibit the practice for those
 * products.  If such problems arise substantially in other domains, we
 * stand ready to extend this provision to those domains in future versions
 * of the GPL, as needed to protect the freedom of users.
 * 
 *   Finally, every program is threatened constantly by software patents.
 * States should not allow patents to restrict development and use of
 * software on general-purpose computers, but in those that do, we wish to
 * avoid the special danger that patents applied to a free program could
 * make it effectively proprietary.  To prevent this, the GPL assures that
 * patents cannot be used to render the program non-free.
 * 
 *   The precise terms and conditions for copying, distribution and
 * modification follow.
 * 
 *                        TERMS AND CONDITIONS
 * 
 *   0. Definitions.
 * 
 *   "This License" refers to version 3 of the GNU General Public License.
 * 
 *   "Copyright" also means copyright-like laws that apply to other kinds of
 * works, such as semiconductor masks.
 * 
 *   "The Program" refers to any copyrightable work licensed under this
 * License.  Each licensee is addressed as "you".  "Licensees" and
 * "recipients" may be individuals or organizations.
 * 
 *   To "modify" a work means to copy from or adapt all or part of the work
 * in a fashion requiring copyright permission, other than the making of an
 * exact copy.  The resulting work is called a "modified version" of the
 * earlier work or a work "based on" the earlier work.
 * 
 *   A "covered work" means either the unmodified Program or a work based
 * on the Program.
 * 
 *   To "propagate" a work means to do anything with it that, without
 * permission, would make you directly or secondarily liable for
 * infringement under applicable copyright law, except executing it on a
 * computer or modifying a private copy.  Propagation includes copying,
 * distribution (with or without modification), making available to the
 * public, and in some countries other activities as well.
 * 
 *   To "convey" a work means any kind of propagation that enables other
 * parties to make or receive copies.  Mere interaction with a user through
 * a computer network, with no transfer of a copy, is not conveying.
 * 
 *   An interactive user interface displays "Appropriate Legal Notices"
 * to the extent that it includes a convenient and prominently visible
 * feature that (1) displays an appropriate copyright notice, and (2)
 * tells the user that there is no warranty for the work (except to the
 * extent that warranties are provided), that licensees may convey the
 * work under this License, and how to view a copy of this License.  If
 * the interface presents a list of user commands or options, such as a
 * menu, a prominent item in the list meets this criterion.
 * 
 *   1. Source Code.
 * 
 *   The "source code" for a work means the preferred form of the work
 * for making modifications to it.  "Object code" means any non-source
 * form of a work.
 * 
 *   A "Standard Interface" means an interface that either is an official
 * standard defined by a recognized standards body, or, in the case of
 * interfaces specified for a particular programming language, one that
 * is widely used among developers working in that language.
 * 
 *   The "System Libraries" of an executable work include anything, other
 * than the work as a whole, that (a) is included in the normal form of
 * packaging a Major Component, but which is not part of that Major
 * Component, and (b) serves only to enable use of the work with that
 * Major Component, or to implement a Standard Interface for which an
 * implementation is available to the public in source code form.  A
 * "Major Component", in this context, means a major essential component
 * (kernel, window system, and so on) of the specific operating system
 * (if any) on which the executable work runs, or a compiler used to
 * produce the work, or an object code interpreter used to run it.
 * 
 *   The "Corresponding Source" for a work in object code form means all
 * the source code needed to generate, install, and (for an executable
 * work) run the object code and to modify the work, including scripts to
 * control those activities.  However, it does not include the work's
 * System Libraries, or general-purpose tools or generally available free
 * programs which are used unmodified in performing those activities but
 * which are not part of the work.  For example, Corresponding Source
 * includes interface definition files associated with source files for
 * the work, and the source code for shared libraries and dynamically
 * linked subprograms that the work is specifically designed to require,
 * such as by intimate data communication or control flow between those
 * subprograms and other parts of the work.
 * 
 *   The Corresponding Source need not include anything that users
 * can regenerate automatically from other parts of the Corresponding
 * Source.
 * 
 *   The Corresponding Source for a work in source code form is that
 * same work.
 * 
 *   2. Basic Permissions.
 * 
 *   All rights granted under this License are granted for the term of
 * copyright on the Program, and are irrevocable provided the stated
 * conditions are met.  This License explicitly affirms your unlimited
 * permission to run the unmodified Program.  The output from running a
 * covered work is covered by this License only if the output, given its
 * content, constitutes a covered work.  This License acknowledges your
 * rights of fair use or other equivalent, as provided by copyright law.
 * 
 *   You may make, run and propagate covered works that you do not
 * convey, without conditions so long as your license otherwise remains
 * in force.  You may convey covered works to others for the sole purpose
 * of having them make modifications exclusively for you, or provide you
 * with facilities for running those works, provided that you comply with
 * the terms of this License in conveying all material for which you do
 * not control copyright.  Those thus making or running the covered works
 * for you must do so exclusively on your behalf, under your direction
 * and control, on terms that prohibit them from making any copies of
 * your copyrighted material outside their relationship with you.
 * 
 *   Conveying under any other circumstances is permitted solely under
 * the conditions stated below.  Sublicensing is not allowed; section 10
 * makes it unnecessary.
 * 
 *   3. Protecting Users' Legal Rights From Anti-Circumvention Law.
 * 
 *   No covered work shall be deemed part of an effective technological
 * measure under any applicable law fulfilling obligations under article
 * 11 of the WIPO copyright treaty adopted on 20 December 1996, or
 * similar laws prohibiting or restricting circumvention of such
 * measures.
 * 
 *   When you convey a covered work, you waive any legal power to forbid
 * circumvention of technological measures to the extent such circumvention
 * is effected by exercising rights under this License with respect to
 * the covered work, and you disclaim any intention to limit operation or
 * modification of the work as a means of enforcing, against the work's
 * users, your or third parties' legal rights to forbid circumvention of
 * technological measures.
 * 
 *   4. Conveying Verbatim Copies.
 * 
 *   You may convey verbatim copies of the Program's source code as you
 * receive it, in any medium, provided that you conspicuously and
 * appropriately publish on each copy an appropriate copyright notice;
 * keep intact all notices stating that this License and any
 * non-permissive terms added in accord with section 7 apply to the code;
 * keep intact all notices of the absence of any warranty; and give all
 * recipients a copy of this License along with the Program.
 * 
 *   You may charge any price or no price for each copy that you convey,
 * and you may offer support or warranty protection for a fee.
 * 
 *   5. Conveying Modified Source Versions.
 * 
 *   You may convey a work based on the Program, or the modifications to
 * produce it from the Program, in the form of source code under the
 * terms of section 4, provided that you also meet all of these conditions:
 * 
 *     a) The work must carry prominent notices stating that you modified
 *     it, and giving a relevant date.
 * 
 *     b) The work must carry prominent notices stating that it is
 *     released under this License and any conditions added under section
 *     7.  This requirement modifies the requirement in section 4 to
 *     "keep intact all notices".
 * 
 *     c) You must license the entire work, as a whole, under this
 *     License to anyone who comes into possession of a copy.  This
 *     License will therefore apply, along with any applicable section 7
 *     additional terms, to the whole of the work, and all its parts,
 *     regardless of how they are packaged.  This License gives no
 *     permission to license the work in any other way, but it does not
 *     invalidate such permission if you have separately received it.
 * 
 *     d) If the work has interactive user interfaces, each must display
 *     Appropriate Legal Notices; however, if the Program has interactive
 *     interfaces that do not display Appropriate Legal Notices, your
 *     work need not make them do so.
 * 
 *   A compilation of a covered work with other separate and independent
 * works, which are not by their nature extensions of the covered work,
 * and which are not combined with it such as to form a larger program,
 * in or on a volume of a storage or distribution medium, is called an
 * "aggregate" if the compilation and its resulting copyright are not
 * used to limit the access or legal rights of the compilation's users
 * beyond what the individual works permit.  Inclusion of a covered work
 * in an aggregate does not cause this License to apply to the other
 * parts of the aggregate.
 * 
 *   6. Conveying Non-Source Forms.
 * 
 *   You may convey a covered work in object code form under the terms
 * of sections 4 and 5, provided that you also convey the
 * machine-readable Corresponding Source under the terms of this License,
 * in one of these ways:
 * 
 *     a) Convey the object code in, or embodied in, a physical product
 *     (including a physical distribution medium), accompanied by the
 *     Corresponding Source fixed on a durable physical medium
 *     customarily used for software interchange.
 * 
 *     b) Convey the object code in, or embodied in, a physical product
 *     (including a physical distribution medium), accompanied by a
 *     written offer, valid for at least three years and valid for as
 *     long as you offer spare parts or customer support for that product
 *     model, to give anyone who possesses the object code either (1) a
 *     copy of the Corresponding Source for all the software in the
 *     product that is covered by this License, on a durable physical
 *     medium customarily used for software interchange, for a price no
 *     more than your reasonable cost of physically performing this
 *     conveying of source, or (2) access to copy the
 *     Corresponding Source from a network server at no charge.
 * 
 *     c) Convey individual copies of the object code with a copy of the
 *     written offer to provide the Corresponding Source.  This
 *     alternative is allowed only occasionally and noncommercially, and
 *     only if you received the object code with such an offer, in accord
 *     with subsection 6b.
 * 
 *     d) Convey the object code by offering access from a designated
 *     place (gratis or for a charge), and offer equivalent access to the
 *     Corresponding Source in the same way through the same place at no
 *     further charge.  You need not require recipients to copy the
 *     Corresponding Source along with the object code.  If the place to
 *     copy the object code is a network server, the Corresponding Source
 *     may be on a different server (operated by you or a third party)
 *     that supports equivalent copying facilities, provided you maintain
 *     clear directions next to the object code saying where to find the
 *     Corresponding Source.  Regardless of what server hosts the
 *     Corresponding Source, you remain obligated to ensure that it is
 *     available for as long as needed to satisfy these requirements.
 * 
 *     e) Convey the object code using peer-to-peer transmission, provided
 *     you inform other peers where the object code and Corresponding
 *     Source of the work are being offered to the general public at no
 *     charge under subsection 6d.
 * 
 *   A separable portion of the object code, whose source code is excluded
 * from the Corresponding Source as a System Library, need not be
 * included in conveying the object code work.
 * 
 *   A "User Product" is either (1) a "consumer product", which means any
 * tangible personal property which is normally used for personal, family,
 * or household purposes, or (2) anything designed or sold for incorporation
 * into a dwelling.  In determining whether a product is a consumer product,
 * doubtful cases shall be resolved in favor of coverage.  For a particular
 * product received by a particular user, "normally used" refers to a
 * typical or common use of that class of product, regardless of the status
 * of the particular user or of the way in which the particular user
 * actually uses, or expects or is expected to use, the product.  A product
 * is a consumer product regardless of whether the product has substantial
 * commercial, industrial or non-consumer uses, unless such uses represent
 * the only significant mode of use of the product.
 * 
 *   "Installation Information" for a User Product means any methods,
 * procedures, authorization keys, or other information required to install
 * and execute modified versions of a covered work in that User Product from
 * a modified version of its Corresponding Source.  The information must
 * suffice to ensure that the continued functioning of the modified object
 * code is in no case prevented or interfered with solely because
 * modification has been made.
 * 
 *   If you convey an object code work under this section in, or with, or
 * specifically for use in, a User Product, and the conveying occurs as
 * part of a transaction in which the right of possession and use of the
 * User Product is transferred to the recipient in perpetuity or for a
 * fixed term (regardless of how the transaction is characterized), the
 * Corresponding Source conveyed under this section must be accompanied
 * by the Installation Information.  But this requirement does not apply
 * if neither you nor any third party retains the ability to install
 * modified object code on the User Product (for example, the work has
 * been installed in ROM).
 * 
 *   The requirement to provide Installation Information does not include a
 * requirement to continue to provide support service, warranty, or updates
 * for a work that has been modified or installed by the recipient, or for
 * the User Product in which it has been modified or installed.  Access to a
 * network may be denied when the modification itself materially and
 * adversely affects the operation of the network or violates the rules and
 * protocols for communication across the network.
 * 
 *   Corresponding Source conveyed, and Installation Information provided,
 * in accord with this section must be in a format that is publicly
 * documented (and with an implementation available to the public in
 * source code form), and must require no special password or key for
 * unpacking, reading or copying.
 * 
 *   7. Additional Terms.
 * 
 *   "Additional permissions" are terms that supplement the terms of this
 * License by making exceptions from one or more of its conditions.
 * Additional permissions that are applicable to the entire Program shall
 * be treated as though they were included in this License, to the extent
 * that they are valid under applicable law.  If additional permissions
 * apply only to part of the Program, that part may be used separately
 * under those permissions, but the entire Program remains governed by
 * this License without regard to the additional permissions.
 * 
 *   When you convey a copy of a covered work, you may at your option
 * remove any additional permissions from that copy, or from any part of
 * it.  (Additional permissions may be written to require their own
 * removal in certain cases when you modify the work.)  You may place
 * additional permissions on material, added by you to a covered work,
 * for which you have or can give appropriate copyright permission.
 * 
 *   Notwithstanding any other provision of this License, for material you
 * add to a covered work, you may (if authorized by the copyright holders of
 * that material) supplement the terms of this License with terms:
 * 
 *     a) Disclaiming warranty or limiting liability differently from the
 *     terms of sections 15 and 16 of this License; or
 * 
 *     b) Requiring preservation of specified reasonable legal notices or
 *     author attributions in that material or in the Appropriate Legal
 *     Notices displayed by works containing it; or
 * 
 *     c) Prohibiting misrepresentation of the origin of that material, or
 *     requiring that modified versions of such material be marked in
 *     reasonable ways as different from the original version; or
 * 
 *     d) Limiting the use for publicity purposes of names of licensors or
 *     authors of the material; or
 * 
 *     e) Declining to grant rights under trademark law for use of some
 *     trade names, trademarks, or service marks; or
 * 
 *     f) Requiring indemnification of licensors and authors of that
 *     material by anyone who conveys the material (or modified versions of
 *     it) with contractual assumptions of liability to the recipient, for
 *     any liability that these contractual assumptions directly impose on
 *     those licensors and authors.
 * 
 *   All other non-permissive additional terms are considered "further
 * restrictions" within the meaning of section 10.  If the Program as you
 * received it, or any part of it, contains a notice stating that it is
 * governed by this License along with a term that is a further
 * restriction, you may remove that term.  If a license document contains
 * a further restriction but permits relicensing or conveying under this
 * License, you may add to a covered work material governed by the terms
 * of that license document, provided that the further restriction does
 * not survive such relicensing or conveying.
 * 
 *   If you add terms to a covered work in accord with this section, you
 * must place, in the relevant source files, a statement of the
 * additional terms that apply to those files, or a notice indicating
 * where to find the applicable terms.
 * 
 *   Additional terms, permissive or non-permissive, may be stated in the
 * form of a separately written license, or stated as exceptions;
 * the above requirements apply either way.
 * 
 *   8. Termination.
 * 
 *   You may not propagate or modify a covered work except as expressly
 * provided under this License.  Any attempt otherwise to propagate or
 * modify it is void, and will automatically terminate your rights under
 * this License (including any patent licenses granted under the third
 * paragraph of section 11).
 * 
 *   However, if you cease all violation of this License, then your
 * license from a particular copyright holder is reinstated (a)
 * provisionally, unless and until the copyright holder explicitly and
 * finally terminates your license, and (b) permanently, if the copyright
 * holder fails to notify you of the violation by some reasonable means
 * prior to 60 days after the cessation.
 * 
 *   Moreover, your license from a particular copyright holder is
 * reinstated permanently if the copyright holder notifies you of the
 * violation by some reasonable means, this is the first time you have
 * received notice of violation of this License (for any work) from that
 * copyright holder, and you cure the violation prior to 30 days after
 * your receipt of the notice.
 * 
 *   Termination of your rights under this section does not terminate the
 * licenses of parties who have received copies or rights from you under
 * this License.  If your rights have been terminated and not permanently
 * reinstated, you do not qualify to receive new licenses for the same
 * material under section 10.
 * 
 *   9. Acceptance Not Required for Having Copies.
 * 
 *   You are not required to accept this License in order to receive or
 * run a copy of the Program.  Ancillary propagation of a covered work
 * occurring solely as a consequence of using peer-to-peer transmission
 * to receive a copy likewise does not require acceptance.  However,
 * nothing other than this License grants you permission to propagate or
 * modify any covered work.  These actions infringe copyright if you do
 * not accept this License.  Therefore, by modifying or propagating a
 * covered work, you indicate your acceptance of this License to do so.
 * 
 *   10. Automatic Licensing of Downstream Recipients.
 * 
 *   Each time you convey a covered work, the recipient automatically
 * receives a license from the original licensors, to run, modify and
 * propagate that work, subject to this License.  You are not responsible
 * for enforcing compliance by third parties with this License.
 * 
 *   An "entity transaction" is a transaction transferring control of an
 * organization, or substantially all assets of one, or subdividing an
 * organization, or merging organizations.  If propagation of a covered
 * work results from an entity transaction, each party to that
 * transaction who receives a copy of the work also receives whatever
 * licenses to the work the party's predecessor in interest had or could
 * give under the previous paragraph, plus a right to possession of the
 * Corresponding Source of the work from the predecessor in interest, if
 * the predecessor has it or can get it with reasonable efforts.
 * 
 *   You may not impose any further restrictions on the exercise of the
 * rights granted or affirmed under this License.  For example, you may
 * not impose a license fee, royalty, or other charge for exercise of
 * rights granted under this License, and you may not initiate litigation
 * (including a cross-claim or counterclaim in a lawsuit) alleging that
 * any patent claim is infringed by making, using, selling, offering for
 * sale, or importing the Program or any portion of it.
 * 
 *   11. Patents.
 * 
 *   A "contributor" is a copyright holder who authorizes use under this
 * License of the Program or a work on which the Program is based.  The
 * work thus licensed is called the contributor's "contributor version".
 * 
 *   A contributor's "essential patent claims" are all patent claims
 * owned or controlled by the contributor, whether already acquired or
 * hereafter acquired, that would be infringed by some manner, permitted
 * by this License, of making, using, or selling its contributor version,
 * but do not include claims that would be infringed only as a
 * consequence of further modification of the contributor version.  For
 * purposes of this definition, "control" includes the right to grant
 * patent sublicenses in a manner consistent with the requirements of
 * this License.
 * 
 *   Each contributor grants you a non-exclusive, worldwide, royalty-free
 * patent license under the contributor's essential patent claims, to
 * make, use, sell, offer for sale, import and otherwise run, modify and
 * propagate the contents of its contributor version.
 * 
 *   In the following three paragraphs, a "patent license" is any express
 * agreement or commitment, however denominated, not to enforce a patent
 * (such as an express permission to practice a patent or covenant not to
 * sue for patent infringement).  To "grant" such a patent license to a
 * party means to make such an agreement or commitment not to enforce a
 * patent against the party.
 * 
 *   If you convey a covered work, knowingly relying on a patent license,
 * and the Corresponding Source of the work is not available for anyone
 * to copy, free of charge and under the terms of this License, through a
 * publicly available network server or other readily accessible means,
 * then you must either (1) cause the Corresponding Source to be so
 * available, or (2) arrange to deprive yourself of the benefit of the
 * patent license for this particular work, or (3) arrange, in a manner
 * consistent with the requirements of this License, to extend the patent
 * license to downstream recipients.  "Knowingly relying" means you have
 * actual knowledge that, but for the patent license, your conveying the
 * covered work in a country, or your recipient's use of the covered work
 * in a country, would infringe one or more identifiable patents in that
 * country that you have reason to believe are valid.
 * 
 *   If, pursuant to or in connection with a single transaction or
 * arrangement, you convey, or propagate by procuring conveyance of, a
 * covered work, and grant a patent license to some of the parties
 * receiving the covered work authorizing them to use, propagate, modify
 * or convey a specific copy of the covered work, then the patent license
 * you grant is automatically extended to all recipients of the covered
 * work and works based on it.
 * 
 *   A patent license is "discriminatory" if it does not include within
 * the scope of its coverage, prohibits the exercise of, or is
 * conditioned on the non-exercise of one or more of the rights that are
 * specifically granted under this License.  You may not convey a covered
 * work if you are a party to an arrangement with a third party that is
 * in the business of distributing software, under which you make payment
 * to the third party based on the extent of your activity of conveying
 * the work, and under which the third party grants, to any of the
 * parties who would receive the covered work from you, a discriminatory
 * patent license (a) in connection with copies of the covered work
 * conveyed by you (or copies made from those copies), or (b) primarily
 * for and in connection with specific products or compilations that
 * contain the covered work, unless you entered into that arrangement,
 * or that patent license was granted, prior to 28 March 2007.
 * 
 *   Nothing in this License shall be construed as excluding or limiting
 * any implied license or other defenses to infringement that may
 * otherwise be available to you under applicable patent law.
 * 
 *   12. No Surrender of Others' Freedom.
 * 
 *   If conditions are imposed on you (whether by court order, agreement or
 * otherwise) that contradict the conditions of this License, they do not
 * excuse you from the conditions of this License.  If you cannot convey a
 * covered work so as to satisfy simultaneously your obligations under this
 * License and any other pertinent obligations, then as a consequence you may
 * not convey it at all.  For example, if you agree to terms that obligate you
 * to collect a royalty for further conveying from those to whom you convey
 * the Program, the only way you could satisfy both those terms and this
 * License would be to refrain entirely from conveying the Program.
 * 
 *   13. Use with the GNU Affero General Public License.
 * 
 *   Notwithstanding any other provision of this License, you have
 * permission to link or combine any covered work with a work licensed
 * under version 3 of the GNU Affero General Public License into a single
 * combined work, and to convey the resulting work.  The terms of this
 * License will continue to apply to the part which is the covered work,
 * but the special requirements of the GNU Affero General Public License,
 * section 13, concerning interaction through a network will apply to the
 * combination as such.
 * 
 *   14. Revised Versions of this License.
 * 
 *   The Free Software Foundation may publish revised and/or new versions of
 * the GNU General Public License from time to time.  Such new versions will
 * be similar in spirit to the present version, but may differ in detail to
 * address new problems or concerns.
 * 
 *   Each version is given a distinguishing version number.  If the
 * Program specifies that a certain numbered version of the GNU General
 * Public License "or any later version" applies to it, you have the
 * option of following the terms and conditions either of that numbered
 * version or of any later version published by the Free Software
 * Foundation.  If the Program does not specify a version number of the
 * GNU General Public License, you may choose any version ever published
 * by the Free Software Foundation.
 * 
 *   If the Program specifies that a proxy can decide which future
 * versions of the GNU General Public License can be used, that proxy's
 * public statement of acceptance of a version permanently authorizes you
 * to choose that version for the Program.
 * 
 *   Later license versions may give you additional or different
 * permissions.  However, no additional obligations are imposed on any
 * author or copyright holder as a result of your choosing to follow a
 * later version.
 * 
 *   15. Disclaimer of Warranty.
 * 
 *   THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
 * APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
 * IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
 * ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
 * 
 *   16. Limitation of Liability.
 * 
 *   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
 * THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
 * GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
 * USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
 * DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
 * PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
 * EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES.
 * 
 *   17. Interpretation of Sections 15 and 16.
 * 
 *   If the disclaimer of warranty and limitation of liability provided
 * above cannot be given local legal effect according to their terms,
 * reviewing courts shall apply local law that most closely approximates
 * an absolute waiver of all civil liability in connection with the
 * Program, unless a warranty or assumption of liability accompanies a
 * copy of the Program in return for a fee.
 * 
 *                      END OF TERMS AND CONDITIONS
 * 
 *             How to Apply These Terms to Your New Programs
 * 
 *   If you develop a new program, and you want it to be of the greatest
 * possible use to the public, the best way to achieve this is to make it
 * free software which everyone can redistribute and change under these terms.
 * 
 *   To do so, attach the following notices to the program.  It is safest
 * to attach them to the start of each source file to most effectively
 * state the exclusion of warranty; and each file should have at least
 * the "copyright" line and a pointer to where the full notice is found.
 * 
 *     <one line to give the program's name and a brief idea of what it does.>
 *     Copyright (C) <year>  <name of author>
 * 
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * Also add information on how to contact you by electronic and paper mail.
 * 
 *   If the program does terminal interaction, make it output a short
 * notice like this when it starts in an interactive mode:
 * 
 *     <program>  Copyright (C) <year>  <name of author>
 *     This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 *     This is free software, and you are welcome to redistribute it
 *     under certain conditions; type `show c' for details.
 * 
 * The hypothetical commands `show w' and `show c' should show the appropriate
 * parts of the General Public License.  Of course, your program's commands
 * might be different; for a GUI interface, you would use an "about box".
 * 
 *   You should also get your employer (if you work as a programmer) or school,
 * if any, to sign a "copyright disclaimer" for the program, if necessary.
 * For more information on this, and how to apply and follow the GNU GPL, see
 * <https://www.gnu.org/licenses/>.
 * 
 *   The GNU General Public License does not permit incorporating your program
 * into proprietary programs.  If your program is a subroutine library, you
 * may consider it more useful to permit linking proprietary applications with
 * the library.  If this is what you want to do, use the GNU Lesser General
 * Public License instead of this License.  But first, please read
 * <https://www.gnu.org/licenses/why-not-lgpl.html>.
 * 
 * Version: 2.5.0
 * Release date: 29/05/2023 (built at 29/05/2023 09:32:45)
 */
!function(e,t){"object"==typeof exports&&"object"==typeof module?module.exports=t():"function"==typeof define&&define.amd?define("HyperFormula",[],t):"object"==typeof exports?exports.HyperFormula=t():e.HyperFormula=t()}("undefined"!=typeof self?self:this,(function(){return function(e){var t={};function r(n){if(t[n])return t[n].exports;var o=t[n]={i:n,l:!1,exports:{}};return e[n].call(o.exports,o,o.exports,r),o.l=!0,o.exports}return r.m=e,r.c=t,r.d=function(e,t,n){r.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:n})},r.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},r.t=function(e,t){if(1&t&&(e=r(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var n=Object.create(null);if(r.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var o in e)r.d(n,o,function(t){return e[t]}.bind(null,o));return n},r.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return r.d(t,"a",t),t},r.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},r.p="",r(r.s=0)}([function(e,t,r){"use strict";t.__esModule=!0,t.default=void 0;var n=r(1);t.ArraySize=n.ArraySize;var o=r(74);t.CellError=o.CellError,t.CellType=o.CellType,t.CellValueDetailedType=o.CellValueDetailedType,t.CellValueType=o.CellValueType,t.ErrorType=o.ErrorType;var s=r(169);t.DetailedCellError=s.DetailedCellError;var i=r(170),a=r(174);t.AlwaysDense=a.AlwaysDense,t.AlwaysSparse=a.AlwaysSparse,t.DenseSparseChooseBasedOnThreshold=a.DenseSparseChooseBasedOnThreshold;var u=r(148);t.ConfigValueTooBigError=u.ConfigValueTooBigError,t.ConfigValueTooSmallError=u.ConfigValueTooSmallError,t.EvaluationSuspendedError=u.EvaluationSuspendedError,t.ExpectedOneOfValuesError=u.ExpectedOneOfValuesError,t.ExpectedValueOfTypeError=u.ExpectedValueOfTypeError,t.FunctionPluginValidationError=u.FunctionPluginValidationError,t.InvalidAddressError=u.InvalidAddressError,t.InvalidArgumentsError=u.InvalidArgumentsError,t.LanguageAlreadyRegisteredError=u.LanguageAlreadyRegisteredError,t.LanguageNotRegisteredError=u.LanguageNotRegisteredError,t.MissingTranslationError=u.MissingTranslationError,t.NamedExpressionDoesNotExistError=u.NamedExpressionDoesNotExistError,t.NamedExpressionNameIsAlreadyTakenError=u.NamedExpressionNameIsAlreadyTakenError,t.NamedExpressionNameIsInvalidError=u.NamedExpressionNameIsInvalidError,t.NoOperationToRedoError=u.NoOperationToRedoError,t.NoOperationToUndoError=u.NoOperationToUndoError,t.NoRelativeAddressesAllowedError=u.NoRelativeAddressesAllowedError,t.NoSheetWithIdError=u.NoSheetWithIdError,t.NoSheetWithNameError=u.NoSheetWithNameError,t.NotAFormulaError=u.NotAFormulaError,t.NothingToPasteError=u.NothingToPasteError,t.ProtectedFunctionTranslationError=u.ProtectedFunctionTranslationError,t.SheetNameAlreadyTakenError=u.SheetNameAlreadyTakenError,t.SheetSizeLimitExceededError=u.SheetSizeLimitExceededError,t.SourceLocationHasArrayError=u.SourceLocationHasArrayError,t.TargetLocationHasArrayError=u.TargetLocationHasArrayError,t.UnableToParseError=u.UnableToParseError;var c=r(197);t.ExportedCellChange=c.ExportedCellChange,t.ExportedNamedExpressionChange=c.ExportedNamedExpressionChange;var l=r(179);t.HyperFormula=l.HyperFormula;var h,d=(h=r(215))&&h.__esModule?h:{default:h},p=r(216);t.FunctionPlugin=p.FunctionPlugin,t.FunctionArgumentType=p.FunctionArgumentType;var m=function(e,t){if(!t&&e&&e.__esModule)return e;if(null===e||"object"!=typeof e&&"function"!=typeof e)return{default:e};var r=f(t);if(r&&r.has(e))return r.get(e);var n={},o=Object.defineProperty&&Object.getOwnPropertyDescriptor;for(var s in e)if("default"!==s&&Object.prototype.hasOwnProperty.call(e,s)){var i=o?Object.getOwnPropertyDescriptor(e,s):null;i&&(i.get||i.set)?Object.defineProperty(n,s,i):n[s]=e[s]}n.default=e,r&&r.set(e,n);return n}(r(217)),g=r(140);function f(e){if("function"!=typeof WeakMap)return null;var t=new WeakMap,r=new WeakMap;return(f=function(e){return e?r:t})(e)}t.SimpleRangeValue=g.SimpleRangeValue;
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class E extends l.HyperFormula{}E.HyperFormula=l.HyperFormula,E.ErrorType=o.ErrorType,E.CellError=o.CellError,E.CellType=o.CellType,E.CellValueType=o.CellValueType,E.CellValueDetailedType=o.CellValueDetailedType,E.DetailedCellError=s.DetailedCellError,E.ExportedCellChange=c.ExportedCellChange,E.ExportedNamedExpressionChange=c.ExportedNamedExpressionChange,E.ConfigValueTooBigError=u.ConfigValueTooBigError,E.ConfigValueTooSmallError=u.ConfigValueTooSmallError,E.EvaluationSuspendedError=u.EvaluationSuspendedError,E.ExpectedOneOfValuesError=u.ExpectedOneOfValuesError,E.ExpectedValueOfTypeError=u.ExpectedValueOfTypeError,E.ArraySize=n.ArraySize,E.SimpleRangeValue=g.SimpleRangeValue,E.FunctionPlugin=p.FunctionPlugin,E.FunctionArgumentType=p.FunctionArgumentType,E.FunctionPluginValidationError=u.FunctionPluginValidationError,E.InvalidAddressError=u.InvalidAddressError,E.InvalidArgumentsError=u.InvalidArgumentsError,E.LanguageNotRegisteredError=u.LanguageNotRegisteredError,E.LanguageAlreadyRegisteredError=u.LanguageAlreadyRegisteredError,E.MissingTranslationError=u.MissingTranslationError,E.NamedExpressionDoesNotExistError=u.NamedExpressionDoesNotExistError,E.NamedExpressionNameIsAlreadyTakenError=u.NamedExpressionNameIsAlreadyTakenError,E.NamedExpressionNameIsInvalidError=u.NamedExpressionNameIsInvalidError,E.NoOperationToRedoError=u.NoOperationToRedoError,E.NoOperationToUndoError=u.NoOperationToUndoError,E.NoRelativeAddressesAllowedError=u.NoRelativeAddressesAllowedError,E.NoSheetWithIdError=u.NoSheetWithIdError,E.NoSheetWithNameError=u.NoSheetWithNameError,E.NotAFormulaError=u.NotAFormulaError,E.NothingToPasteError=u.NothingToPasteError,E.ProtectedFunctionTranslationError=u.ProtectedFunctionTranslationError,E.SheetNameAlreadyTakenError=u.SheetNameAlreadyTakenError,E.SheetSizeLimitExceededError=u.SheetSizeLimitExceededError,E.SourceLocationHasArrayError=u.SourceLocationHasArrayError,E.TargetLocationHasArrayError=u.TargetLocationHasArrayError,E.UnableToParseError=u.UnableToParseError;const T=i.Config.defaultConfig.language;l.HyperFormula.registerLanguage(T,d.default),l.HyperFormula.languages[d.default.langCode]=d.default;for(const e of Object.getOwnPropertyNames(m))e.startsWith("_")||l.HyperFormula.registerFunctionPlugin(m[e]);var y=E;t.default=y},function(e,t,r){"use strict";t.__esModule=!0,t.ArraySizePredictor=t.ArraySize=void 0,r(2);var n=r(73),o=r(164),s=r(165),i=r(78);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a{constructor(e,t){let r=arguments.length>2&&void 0!==arguments[2]&&arguments[2];if(this.width=e,this.height=t,this.isRef=r,e<=0||t<=0)throw Error("Incorrect array size")}static fromArray(e){return new a(e.length>0?e[0].length:0,e.length)}static error(){return new a(1,1,!0)}static scalar(){return new a(1,1,!1)}isScalar(){return this.width<=1&&this.height<=1||this.isRef}}t.ArraySize=a;t.ArraySizePredictor=class{constructor(e,t){this.config=e,this.functionRegistry=t}checkArraySize(e,t){return this.checkArraySizeForAst(e,{formulaAddress:t,arraysFlag:this.config.useArrayArithmetic})}checkArraySizeForAst(e,t){switch(e.type){case i.AstNodeType.FUNCTION_CALL:return this.checkArraySizeForFunction(e,t);case i.AstNodeType.COLUMN_RANGE:case i.AstNodeType.ROW_RANGE:case i.AstNodeType.CELL_RANGE:{const r=n.AbsoluteCellRange.fromAstOrUndef(e,t.formulaAddress);return void 0===r?a.error():new a(r.width(),r.height(),!0)}case i.AstNodeType.ARRAY:{const r=[],n=[];for(const o of e.args){const e=o.map(e=>this.checkArraySizeForAst(e,t)),s=Math.min(...e.map(e=>e.height)),i=e.reduce((e,t)=>e+t.width,0);r.push(s),n.push(i)}const o=r.reduce((e,t)=>e+t,0),s=Math.min(...n);return new a(s,o)}case i.AstNodeType.STRING:case i.AstNodeType.NUMBER:return a.scalar();case i.AstNodeType.CELL_REFERENCE:return new a(1,1,!0);case i.AstNodeType.DIV_OP:case i.AstNodeType.CONCATENATE_OP:case i.AstNodeType.EQUALS_OP:case i.AstNodeType.GREATER_THAN_OP:case i.AstNodeType.GREATER_THAN_OR_EQUAL_OP:case i.AstNodeType.LESS_THAN_OP:case i.AstNodeType.LESS_THAN_OR_EQUAL_OP:case i.AstNodeType.MINUS_OP:case i.AstNodeType.NOT_EQUAL_OP:case i.AstNodeType.PLUS_OP:case i.AstNodeType.POWER_OP:case i.AstNodeType.TIMES_OP:{const r=this.checkArraySizeForAst(e.left,t),n=this.checkArraySizeForAst(e.right,t);return!t.arraysFlag&&(r.height>1||r.width>1||n.height>1||n.width>1)?a.error():(o=r,s=n,new a(Math.max(o.width,s.width),Math.max(o.height,s.height)))}case i.AstNodeType.MINUS_UNARY_OP:case i.AstNodeType.PLUS_UNARY_OP:case i.AstNodeType.PERCENT_OP:{const n=this.checkArraySizeForAst(e.value,t);return!t.arraysFlag&&(n.height>1||n.width>1)?a.error():new a((r=n).width,r.height)}case i.AstNodeType.PARENTHESIS:return this.checkArraySizeForAst(e.expression,t);case i.AstNodeType.EMPTY:default:return a.error()}var r,o,s}checkArraySizeForFunction(e,t){const r=this.functionRegistry.getMetadata(e.procedureName),n=this.functionRegistry.getArraySizeFunction(e.procedureName);if(void 0!==n)return n(e,t);const i=e.args.map(e=>{var n;return this.checkArraySizeForAst(e,new o.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))});if(void 0===r||r.expandRanges||!t.arraysFlag||r.vectorizationForbidden||void 0===r.parameters)return new a(1,1);const u=[...r.parameters];if(void 0===r.repeatLastArgs&&u.length<i.length)return a.error();if(void 0!==r.repeatLastArgs&&u.length<i.length&&(i.length-u.length)%r.repeatLastArgs!=0)return a.error();for(;u.length<i.length;)u.push(...u.slice(u.length-r.repeatLastArgs));let c=1,l=1;for(let e=0;e<i.length;e++)u[e].argumentType!==s.FunctionArgumentType.RANGE&&u[e].argumentType!==s.FunctionArgumentType.ANY&&(l=Math.max(l,i[e].height),c=Math.max(c,i[e].width));return new a(c,l)}}},function(e,t,r){"use strict";var n=r(3),o=r(41),s=r(65),i=r(70),a=r(72);n({target:"Array",proto:!0,arity:1,forced:r(8)((function(){return 4294967297!==[].push.call({length:4294967296},1)}))||!function(){try{Object.defineProperty([],"length",{writable:!1}).push()}catch(e){return e instanceof TypeError}}()},{push:function(e){var t=o(this),r=s(t),n=arguments.length;a(r+n);for(var u=0;u<n;u++)t[r]=arguments[u],r++;return i(t,r),r}})},function(e,t,r){var n=r(4),o=r(6).f,s=r(45),i=r(49),a=r(39),u=r(57),c=r(69);e.exports=function(e,t){var r,l,h,d,p,m=e.target,g=e.global,f=e.stat;if(r=g?n:f?n[m]||a(m,{}):(n[m]||{}).prototype)for(l in t){if(d=t[l],h=e.dontCallGetSet?(p=o(r,l))&&p.value:r[l],!c(g?l:m+(f?".":"#")+l,e.forced)&&void 0!==h){if(typeof d==typeof h)continue;u(d,h)}(e.sham||h&&h.sham)&&s(d,"sham",!0),i(r,l,d,e)}}},function(e,t,r){(function(t){var r=function(e){return e&&e.Math==Math&&e};e.exports=r("object"==typeof globalThis&&globalThis)||r("object"==typeof window&&window)||r("object"==typeof self&&self)||r("object"==typeof t&&t)||function(){return this}()||this||Function("return this")()}).call(this,r(5))},function(e,t){var r;r=function(){return this}();try{r=r||new Function("return this")()}catch(e){"object"==typeof window&&(r=window)}e.exports=r},function(e,t,r){var n=r(7),o=r(9),s=r(11),i=r(12),a=r(13),u=r(19),c=r(40),l=r(43),h=Object.getOwnPropertyDescriptor;t.f=n?h:function(e,t){if(e=a(e),t=u(t),l)try{return h(e,t)}catch(e){}if(c(e,t))return i(!o(s.f,e,t),e[t])}},function(e,t,r){var n=r(8);e.exports=!n((function(){return 7!=Object.defineProperty({},1,{get:function(){return 7}})[1]}))},function(e,t){e.exports=function(e){try{return!!e()}catch(e){return!0}}},function(e,t,r){var n=r(10),o=Function.prototype.call;e.exports=n?o.bind(o):function(){return o.apply(o,arguments)}},function(e,t,r){var n=r(8);e.exports=!n((function(){var e=function(){}.bind();return"function"!=typeof e||e.hasOwnProperty("prototype")}))},function(e,t,r){"use strict";var n={}.propertyIsEnumerable,o=Object.getOwnPropertyDescriptor,s=o&&!n.call({1:2},1);t.f=s?function(e){var t=o(this,e);return!!t&&t.enumerable}:n},function(e,t){e.exports=function(e,t){return{enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}},function(e,t,r){var n=r(14),o=r(17);e.exports=function(e){return n(o(e))}},function(e,t,r){var n=r(15),o=r(8),s=r(16),i=Object,a=n("".split);e.exports=o((function(){return!i("z").propertyIsEnumerable(0)}))?function(e){return"String"==s(e)?a(e,""):i(e)}:i},function(e,t,r){var n=r(10),o=Function.prototype,s=o.call,i=n&&o.bind.bind(s,s);e.exports=n?i:function(e){return function(){return s.apply(e,arguments)}}},function(e,t,r){var n=r(15),o=n({}.toString),s=n("".slice);e.exports=function(e){return s(o(e),8,-1)}},function(e,t,r){var n=r(18),o=TypeError;e.exports=function(e){if(n(e))throw o("Can't call method on "+e);return e}},function(e,t){e.exports=function(e){return null==e}},function(e,t,r){var n=r(20),o=r(24);e.exports=function(e){var t=n(e,"string");return o(t)?t:t+""}},function(e,t,r){var n=r(9),o=r(21),s=r(24),i=r(31),a=r(34),u=r(35),c=TypeError,l=u("toPrimitive");e.exports=function(e,t){if(!o(e)||s(e))return e;var r,u=i(e,l);if(u){if(void 0===t&&(t="default"),r=n(u,e,t),!o(r)||s(r))return r;throw c("Can't convert object to primitive value")}return void 0===t&&(t="number"),a(e,t)}},function(e,t,r){var n=r(22),o=r(23),s=o.all;e.exports=o.IS_HTMLDDA?function(e){return"object"==typeof e?null!==e:n(e)||e===s}:function(e){return"object"==typeof e?null!==e:n(e)}},function(e,t,r){var n=r(23),o=n.all;e.exports=n.IS_HTMLDDA?function(e){return"function"==typeof e||e===o}:function(e){return"function"==typeof e}},function(e,t){var r="object"==typeof document&&document.all,n=void 0===r&&void 0!==r;e.exports={all:r,IS_HTMLDDA:n}},function(e,t,r){var n=r(25),o=r(22),s=r(26),i=r(27),a=Object;e.exports=i?function(e){return"symbol"==typeof e}:function(e){var t=n("Symbol");return o(t)&&s(t.prototype,a(e))}},function(e,t,r){var n=r(4),o=r(22),s=function(e){return o(e)?e:void 0};e.exports=function(e,t){return arguments.length<2?s(n[e]):n[e]&&n[e][t]}},function(e,t,r){var n=r(15);e.exports=n({}.isPrototypeOf)},function(e,t,r){var n=r(28);e.exports=n&&!Symbol.sham&&"symbol"==typeof Symbol.iterator},function(e,t,r){var n=r(29),o=r(8),s=r(4).String;e.exports=!!Object.getOwnPropertySymbols&&!o((function(){var e=Symbol();return!s(e)||!(Object(e)instanceof Symbol)||!Symbol.sham&&n&&n<41}))},function(e,t,r){var n,o,s=r(4),i=r(30),a=s.process,u=s.Deno,c=a&&a.versions||u&&u.version,l=c&&c.v8;l&&(o=(n=l.split("."))[0]>0&&n[0]<4?1:+(n[0]+n[1])),!o&&i&&(!(n=i.match(/Edge\/(\d+)/))||n[1]>=74)&&(n=i.match(/Chrome\/(\d+)/))&&(o=+n[1]),e.exports=o},function(e,t){e.exports="undefined"!=typeof navigator&&String(navigator.userAgent)||""},function(e,t,r){var n=r(32),o=r(18);e.exports=function(e,t){var r=e[t];return o(r)?void 0:n(r)}},function(e,t,r){var n=r(22),o=r(33),s=TypeError;e.exports=function(e){if(n(e))return e;throw s(o(e)+" is not a function")}},function(e,t){var r=String;e.exports=function(e){try{return r(e)}catch(e){return"Object"}}},function(e,t,r){var n=r(9),o=r(22),s=r(21),i=TypeError;e.exports=function(e,t){var r,a;if("string"===t&&o(r=e.toString)&&!s(a=n(r,e)))return a;if(o(r=e.valueOf)&&!s(a=n(r,e)))return a;if("string"!==t&&o(r=e.toString)&&!s(a=n(r,e)))return a;throw i("Can't convert object to primitive value")}},function(e,t,r){var n=r(4),o=r(36),s=r(40),i=r(42),a=r(28),u=r(27),c=n.Symbol,l=o("wks"),h=u?c.for||c:c&&c.withoutSetter||i;e.exports=function(e){return s(l,e)||(l[e]=a&&s(c,e)?c[e]:h("Symbol."+e)),l[e]}},function(e,t,r){var n=r(37),o=r(38);(e.exports=function(e,t){return o[e]||(o[e]=void 0!==t?t:{})})("versions",[]).push({version:"3.30.2",mode:n?"pure":"global",copyright:"© 2014-2023 Denis Pushkarev (zloirock.ru)",license:"https://github.com/zloirock/core-js/blob/v3.30.2/LICENSE",source:"https://github.com/zloirock/core-js"})},function(e,t){e.exports=!1},function(e,t,r){var n=r(4),o=r(39),s=n["__core-js_shared__"]||o("__core-js_shared__",{});e.exports=s},function(e,t,r){var n=r(4),o=Object.defineProperty;e.exports=function(e,t){try{o(n,e,{value:t,configurable:!0,writable:!0})}catch(r){n[e]=t}return t}},function(e,t,r){var n=r(15),o=r(41),s=n({}.hasOwnProperty);e.exports=Object.hasOwn||function(e,t){return s(o(e),t)}},function(e,t,r){var n=r(17),o=Object;e.exports=function(e){return o(n(e))}},function(e,t,r){var n=r(15),o=0,s=Math.random(),i=n(1..toString);e.exports=function(e){return"Symbol("+(void 0===e?"":e)+")_"+i(++o+s,36)}},function(e,t,r){var n=r(7),o=r(8),s=r(44);e.exports=!n&&!o((function(){return 7!=Object.defineProperty(s("div"),"a",{get:function(){return 7}}).a}))},function(e,t,r){var n=r(4),o=r(21),s=n.document,i=o(s)&&o(s.createElement);e.exports=function(e){return i?s.createElement(e):{}}},function(e,t,r){var n=r(7),o=r(46),s=r(12);e.exports=n?function(e,t,r){return o.f(e,t,s(1,r))}:function(e,t,r){return e[t]=r,e}},function(e,t,r){var n=r(7),o=r(43),s=r(47),i=r(48),a=r(19),u=TypeError,c=Object.defineProperty,l=Object.getOwnPropertyDescriptor;t.f=n?s?function(e,t,r){if(i(e),t=a(t),i(r),"function"==typeof e&&"prototype"===t&&"value"in r&&"writable"in r&&!r.writable){var n=l(e,t);n&&n.writable&&(e[t]=r.value,r={configurable:"configurable"in r?r.configurable:n.configurable,enumerable:"enumerable"in r?r.enumerable:n.enumerable,writable:!1})}return c(e,t,r)}:c:function(e,t,r){if(i(e),t=a(t),i(r),o)try{return c(e,t,r)}catch(e){}if("get"in r||"set"in r)throw u("Accessors not supported");return"value"in r&&(e[t]=r.value),e}},function(e,t,r){var n=r(7),o=r(8);e.exports=n&&o((function(){return 42!=Object.defineProperty((function(){}),"prototype",{value:42,writable:!1}).prototype}))},function(e,t,r){var n=r(21),o=String,s=TypeError;e.exports=function(e){if(n(e))return e;throw s(o(e)+" is not an object")}},function(e,t,r){var n=r(22),o=r(46),s=r(50),i=r(39);e.exports=function(e,t,r,a){a||(a={});var u=a.enumerable,c=void 0!==a.name?a.name:t;if(n(r)&&s(r,c,a),a.global)u?e[t]=r:i(t,r);else{try{a.unsafe?e[t]&&(u=!0):delete e[t]}catch(e){}u?e[t]=r:o.f(e,t,{value:r,enumerable:!1,configurable:!a.nonConfigurable,writable:!a.nonWritable})}return e}},function(e,t,r){var n=r(15),o=r(8),s=r(22),i=r(40),a=r(7),u=r(51).CONFIGURABLE,c=r(52),l=r(53),h=l.enforce,d=l.get,p=String,m=Object.defineProperty,g=n("".slice),f=n("".replace),E=n([].join),T=a&&!o((function(){return 8!==m((function(){}),"length",{value:8}).length})),y=String(String).split("String"),A=e.exports=function(e,t,r){"Symbol("===g(p(t),0,7)&&(t="["+f(p(t),/^Symbol\(([^)]*)\)/,"$1")+"]"),r&&r.getter&&(t="get "+t),r&&r.setter&&(t="set "+t),(!i(e,"name")||u&&e.name!==t)&&(a?m(e,"name",{value:t,configurable:!0}):e.name=t),T&&r&&i(r,"arity")&&e.length!==r.arity&&m(e,"length",{value:r.arity});try{r&&i(r,"constructor")&&r.constructor?a&&m(e,"prototype",{writable:!1}):e.prototype&&(e.prototype=void 0)}catch(e){}var n=h(e);return i(n,"source")||(n.source=E(y,"string"==typeof t?t:"")),e};Function.prototype.toString=A((function(){return s(this)&&d(this).source||c(this)}),"toString")},function(e,t,r){var n=r(7),o=r(40),s=Function.prototype,i=n&&Object.getOwnPropertyDescriptor,a=o(s,"name"),u=a&&"something"===function(){}.name,c=a&&(!n||n&&i(s,"name").configurable);e.exports={EXISTS:a,PROPER:u,CONFIGURABLE:c}},function(e,t,r){var n=r(15),o=r(22),s=r(38),i=n(Function.toString);o(s.inspectSource)||(s.inspectSource=function(e){return i(e)}),e.exports=s.inspectSource},function(e,t,r){var n,o,s,i=r(54),a=r(4),u=r(21),c=r(45),l=r(40),h=r(38),d=r(55),p=r(56),m=a.TypeError,g=a.WeakMap;if(i||h.state){var f=h.state||(h.state=new g);f.get=f.get,f.has=f.has,f.set=f.set,n=function(e,t){if(f.has(e))throw m("Object already initialized");return t.facade=e,f.set(e,t),t},o=function(e){return f.get(e)||{}},s=function(e){return f.has(e)}}else{var E=d("state");p[E]=!0,n=function(e,t){if(l(e,E))throw m("Object already initialized");return t.facade=e,c(e,E,t),t},o=function(e){return l(e,E)?e[E]:{}},s=function(e){return l(e,E)}}e.exports={set:n,get:o,has:s,enforce:function(e){return s(e)?o(e):n(e,{})},getterFor:function(e){return function(t){var r;if(!u(t)||(r=o(t)).type!==e)throw m("Incompatible receiver, "+e+" required");return r}}}},function(e,t,r){var n=r(4),o=r(22),s=n.WeakMap;e.exports=o(s)&&/native code/.test(String(s))},function(e,t,r){var n=r(36),o=r(42),s=n("keys");e.exports=function(e){return s[e]||(s[e]=o(e))}},function(e,t){e.exports={}},function(e,t,r){var n=r(40),o=r(58),s=r(6),i=r(46);e.exports=function(e,t,r){for(var a=o(t),u=i.f,c=s.f,l=0;l<a.length;l++){var h=a[l];n(e,h)||r&&n(r,h)||u(e,h,c(t,h))}}},function(e,t,r){var n=r(25),o=r(15),s=r(59),i=r(68),a=r(48),u=o([].concat);e.exports=n("Reflect","ownKeys")||function(e){var t=s.f(a(e)),r=i.f;return r?u(t,r(e)):t}},function(e,t,r){var n=r(60),o=r(67).concat("length","prototype");t.f=Object.getOwnPropertyNames||function(e){return n(e,o)}},function(e,t,r){var n=r(15),o=r(40),s=r(13),i=r(61).indexOf,a=r(56),u=n([].push);e.exports=function(e,t){var r,n=s(e),c=0,l=[];for(r in n)!o(a,r)&&o(n,r)&&u(l,r);for(;t.length>c;)o(n,r=t[c++])&&(~i(l,r)||u(l,r));return l}},function(e,t,r){var n=r(13),o=r(62),s=r(65),i=function(e){return function(t,r,i){var a,u=n(t),c=s(u),l=o(i,c);if(e&&r!=r){for(;c>l;)if((a=u[l++])!=a)return!0}else for(;c>l;l++)if((e||l in u)&&u[l]===r)return e||l||0;return!e&&-1}};e.exports={includes:i(!0),indexOf:i(!1)}},function(e,t,r){var n=r(63),o=Math.max,s=Math.min;e.exports=function(e,t){var r=n(e);return r<0?o(r+t,0):s(r,t)}},function(e,t,r){var n=r(64);e.exports=function(e){var t=+e;return t!=t||0===t?0:n(t)}},function(e,t){var r=Math.ceil,n=Math.floor;e.exports=Math.trunc||function(e){var t=+e;return(t>0?n:r)(t)}},function(e,t,r){var n=r(66);e.exports=function(e){return n(e.length)}},function(e,t,r){var n=r(63),o=Math.min;e.exports=function(e){return e>0?o(n(e),9007199254740991):0}},function(e,t){e.exports=["constructor","hasOwnProperty","isPrototypeOf","propertyIsEnumerable","toLocaleString","toString","valueOf"]},function(e,t){t.f=Object.getOwnPropertySymbols},function(e,t,r){var n=r(8),o=r(22),s=/#|\.prototype\./,i=function(e,t){var r=u[a(e)];return r==l||r!=c&&(o(t)?n(t):!!t)},a=i.normalize=function(e){return String(e).replace(s,".").toLowerCase()},u=i.data={},c=i.NATIVE="N",l=i.POLYFILL="P";e.exports=i},function(e,t,r){"use strict";var n=r(7),o=r(71),s=TypeError,i=Object.getOwnPropertyDescriptor,a=n&&!function(){if(void 0!==this)return!0;try{Object.defineProperty([],"length",{writable:!1}).length=1}catch(e){return e instanceof TypeError}}();e.exports=a?function(e,t){if(o(e)&&!i(e,"length").writable)throw s("Cannot set read only .length");return e.length=t}:function(e,t){return e.length=t}},function(e,t,r){var n=r(16);e.exports=Array.isArray||function(e){return"Array"==n(e)}},function(e,t){var r=TypeError;e.exports=function(e){if(e>9007199254740991)throw r("Maximum allowed index exceeded");return e}},function(e,t,r){"use strict";t.__esModule=!0,t.WRONG_RANGE_SIZE=t.AbsoluteRowRange=t.AbsoluteColumnRange=t.AbsoluteCellRange=void 0,t.isSimpleCellRange=function(e){return!(!e||"object"!=typeof e&&"function"!=typeof e)&&("start"in e&&(0,n.isSimpleCellAddress)(e.start)&&"end"in e&&(0,n.isSimpleCellAddress)(e.end))},t.simpleCellRange=void 0,r(2);var n=r(74),o=r(148),s=r(78),i=r(142);t.WRONG_RANGE_SIZE="AbsoluteCellRange: Wrong range size";t.simpleCellRange=(e,t)=>({start:e,end:t});class a{constructor(e,t){if(e.sheet!==t.sheet)throw new o.SheetsNotEqual(e.sheet,t.sheet);this.start=(0,n.simpleCellAddress)(e.sheet,e.col,e.row),this.end=(0,n.simpleCellAddress)(t.sheet,t.col,t.row)}get sheet(){return this.start.sheet}static fromSimpleCellAddresses(e,t){if(e.sheet!==t.sheet)throw new o.SheetsNotEqual(e.sheet,t.sheet);const r=t.col-e.col,n=t.row-e.row;return Number.isFinite(n)&&Number.isFinite(r)?new a(e,t):Number.isFinite(n)?new c(e.sheet,e.row,t.row):new u(e.sheet,e.col,t.col)}static fromAst(e,t){return e.type===s.AstNodeType.CELL_RANGE?a.fromCellRange(e,t):e.type===s.AstNodeType.COLUMN_RANGE?u.fromColumnRange(e,t):c.fromRowRangeAst(e,t)}static fromAstOrUndef(e,t){try{return a.fromAst(e,t)}catch(e){return}}static fromCellRange(e,t){return new a(e.start.toSimpleCellAddress(t),e.end.toSimpleCellAddress(t))}static spanFrom(e,t,r){const n=a.spanFromOrUndef(e,t,r);if(void 0===n)throw new Error("AbsoluteCellRange: Wrong range size");return n}static spanFromOrUndef(e,t,r){if(!Number.isFinite(t)&&Number.isFinite(r)){if(0!==e.col)return;return new c(e.sheet,e.row,e.row+r-1)}if(!Number.isFinite(r)&&Number.isFinite(t)){if(0!==e.row)return;return new u(e.sheet,e.col,e.col+t-1)}if(Number.isFinite(r)&&Number.isFinite(t))return new a(e,(0,n.simpleCellAddress)(e.sheet,e.col+t-1,e.row+r-1))}static fromCoordinates(e,t,r,o,s){return new a((0,n.simpleCellAddress)(e,t,r),(0,n.simpleCellAddress)(e,o,s))}isFinite(){return Number.isFinite(this.size())}doesOverlap(e){return this.start.sheet==e.start.sheet&&(!(this.end.row<e.start.row||this.start.row>e.end.row)&&!(this.end.col<e.start.col||this.start.col>e.end.col))}addressInRange(e){return this.sheet===e.sheet&&(this.start.row<=e.row&&this.end.row>=e.row&&this.start.col<=e.col&&this.end.col>=e.col)}columnInRange(e){return this.sheet===e.sheet&&(this.start.col<=e.col&&this.end.col>=e.col)}rowInRange(e){return this.sheet===e.sheet&&(this.start.row<=e.row&&this.end.row>=e.row)}containsRange(e){return this.addressInRange(e.start)&&this.addressInRange(e.end)}intersectionWith(e){if(this.sheet!==e.start.sheet)return;const t=Math.max(this.start.row,e.start.row),r=Math.min(this.end.row,e.end.row),o=Math.max(this.start.col,e.start.col),s=Math.min(this.end.col,e.end.col);return t>r||o>s?void 0:new a((0,n.simpleCellAddress)(this.sheet,o,t),(0,n.simpleCellAddress)(this.sheet,s,r))}includesRow(e){return this.start.row<e&&this.end.row>=e}includesColumn(e){return this.start.col<e&&this.end.col>=e}shiftByRows(e){this.start.row+=e,this.end.row+=e}expandByRows(e){this.end.row+=e}shiftByColumns(e){this.start.col+=e,this.end.col+=e}shifted(e,t){return a.spanFrom((0,n.simpleCellAddress)(this.sheet,this.start.col+e,this.start.row+t),this.width(),this.height())}expandByColumns(e){this.end.col+=e}moveToSheet(e){this.start.sheet=e,this.end.sheet=e}removeSpan(e){e instanceof i.RowsSpan?this.removeRows(e.start,e.end):this.removeColumns(e.start,e.end)}shouldBeRemoved(){return this.width()<=0||this.height()<=0}rangeWithSameWidth(e,t){return a.spanFrom((0,n.simpleCellAddress)(this.sheet,this.start.col,e),this.width(),t)}rangeWithSameHeight(e,t){return a.spanFrom((0,n.simpleCellAddress)(this.sheet,e,this.start.row),t,this.height())}toString(){return`${this.start.sheet},${this.start.col},${this.start.row},${this.end.col},${this.end.row}`}width(){return this.end.col-this.start.col+1}height(){return this.end.row-this.start.row+1}size(){return this.height()*this.width()}arrayOfAddressesInRange(){const e=[];for(let t=0;t<this.height();++t){e[t]=[];for(let r=0;r<this.width();++r){const o=(0,n.simpleCellAddress)(this.sheet,this.start.col+r,this.start.row+t);e[t].push(o)}}return e}withStart(e){return new a(e,this.end)}sameDimensionsAs(e){return this.width()===e.width()&&this.height()===e.height()}sameAs(e){return(0,n.equalSimpleCellAddress)(this.start,e.start)&&(0,n.equalSimpleCellAddress)(this.end,e.end)}addressesArrayMap(e,t){const r=[];let o=this.start.row;for(;o<=this.effectiveEndRow(e);){let s=this.start.col;const i=[];for(;s<=this.effectiveEndColumn(e);)i.push(t((0,n.simpleCellAddress)(this.start.sheet,s,o))),s++;r.push(i),o++}return r}addresses(e){const t=[];let r=this.start.row;const o=this.effectiveEndRow(e),s=this.effectiveEndColumn(e);for(;r<=o;){let e=this.start.col;for(;e<=s;)t.push((0,n.simpleCellAddress)(this.start.sheet,e,r)),e++;r++}return t}*addressesWithDirection(e,t,r){if(e>0)if(t>0){let e=this.effectiveEndRow(r);for(;e>=this.start.row;){let t=this.effectiveEndColumn(r);for(;t>=this.start.col;)yield(0,n.simpleCellAddress)(this.start.sheet,t,e),t-=1;e-=1}}else{let e=this.start.row;for(;e<=this.effectiveEndRow(r);){let t=this.effectiveEndColumn(r);for(;t>=this.start.col;)yield(0,n.simpleCellAddress)(this.start.sheet,t,e),t-=1;e+=1}}else if(t>0){let e=this.effectiveEndRow(r);for(;e>=this.start.row;){let t=this.start.col;for(;t<=this.effectiveEndColumn(r);)yield(0,n.simpleCellAddress)(this.start.sheet,t,e),t+=1;e-=1}}else{let e=this.start.row;for(;e<=this.effectiveEndRow(r);){let t=this.start.col;for(;t<=this.effectiveEndColumn(r);)yield(0,n.simpleCellAddress)(this.start.sheet,t,e),t+=1;e+=1}}}getAddress(e,t){if(e<0||t<0||t>this.height()-1||e>this.width()-1)throw Error("Index out of bound");return(0,n.simpleCellAddress)(this.start.sheet,this.start.col+e,this.start.row+t)}exceedsSheetSizeLimits(e,t){return this.end.col>=e||this.end.row>=t}effectiveEndColumn(e){return this.end.col}effectiveEndRow(e){return this.end.row}effectiveWidth(e){return this.width()}effectiveHeight(e){return this.height()}removeRows(e,t){if(!(e>this.end.row)){if(t<this.start.row){const r=t-e+1;return this.shiftByRows(-r)}e<=this.start.row&&(this.start.row=e),this.end.row-=Math.min(t,this.end.row)-e+1}}removeColumns(e,t){if(!(e>this.end.col)){if(t<this.start.col){const r=t-e+1;return this.shiftByColumns(-r)}e<=this.start.col&&(this.start.col=e),this.end.col-=Math.min(t,this.end.col)-e+1}}}t.AbsoluteCellRange=a;class u extends a{constructor(e,t,r){super((0,n.simpleCellAddress)(e,t,0),(0,n.simpleCellAddress)(e,r,Number.POSITIVE_INFINITY))}static fromColumnRange(e,t){const r=e.start.toSimpleColumnAddress(t),n=e.end.toSimpleColumnAddress(t);if(r.sheet!==n.sheet)throw new o.SheetsNotEqual(r.sheet,n.sheet);return new u(r.sheet,r.col,n.col)}shouldBeRemoved(){return this.width()<=0}shiftByRows(e){}expandByRows(e){}shifted(e,t){return new u(this.sheet,this.start.col+e,this.end.col+e)}rangeWithSameHeight(e,t){return new u(this.sheet,e,e+t-1)}exceedsSheetSizeLimits(e,t){return this.end.col>=e}effectiveEndRow(e){return this.effectiveHeight(e)-1}effectiveHeight(e){return e.getSheetHeight(this.sheet)}removeRows(e,t){}}t.AbsoluteColumnRange=u;class c extends a{constructor(e,t,r){super((0,n.simpleCellAddress)(e,0,t),(0,n.simpleCellAddress)(e,Number.POSITIVE_INFINITY,r))}static fromRowRangeAst(e,t){const r=e.start.toSimpleRowAddress(t),n=e.end.toSimpleRowAddress(t);if(r.sheet!==n.sheet)throw new o.SheetsNotEqual(r.sheet,n.sheet);return new c(r.sheet,r.row,n.row)}shouldBeRemoved(){return this.height()<=0}shiftByColumns(e){}expandByColumns(e){}shifted(e,t){return new c(this.sheet,this.start.row+t,this.end.row+t)}rangeWithSameWidth(e,t){return new c(this.sheet,e,e+t-1)}exceedsSheetSizeLimits(e,t){return this.end.row>=t}effectiveEndColumn(e){return this.effectiveWidth(e)-1}effectiveWidth(e){return e.getSheetWidth(this.sheet)}removeColumns(e,t){}}t.AbsoluteRowRange=c},function(e,t,r){"use strict";t.__esModule=!0,t.invalidSimpleRowAddress=t.invalidSimpleColumnAddress=t.invalidSimpleCellAddress=t.getCellValueType=t.getCellValueFormat=t.getCellValueDetailedType=t.getCellType=t.equalSimpleCellAddress=t.addressKey=t.absoluteSheetReference=t.ErrorType=t.CellValueTypeOrd=t.CellValueType=t.CellValueNoNumber=t.CellValueJustNumber=t.CellValueDetailedType=t.CellType=t.CellError=void 0,t.isSimpleCellAddress=function(e){return!(!e||"object"!=typeof e&&"function"!=typeof e)&&("col"in e&&"number"==typeof e.col&&"row"in e&&"number"==typeof e.row&&"sheet"in e&&"number"==typeof e.sheet)},t.simpleRowAddress=t.simpleColumnAddress=t.simpleCellAddress=t.movedSimpleCellAddress=void 0;var n,o,s=r(75),i=r(132),a=r(141),u=r(140);t.ErrorType=n,function(e){e.DIV_BY_ZERO="DIV_BY_ZERO",e.NAME="NAME",e.VALUE="VALUE",e.NUM="NUM",e.NA="NA",e.CYCLE="CYCLE",e.REF="REF",e.SPILL="SPILL",e.LIC="LIC",e.ERROR="ERROR"}(n||(t.ErrorType=n={})),t.CellType=o,function(e){e.FORMULA="FORMULA",e.VALUE="VALUE",e.ARRAY="ARRAY",e.EMPTY="EMPTY",e.ARRAYFORMULA="ARRAYFORMULA"}(o||(t.CellType=o={}));var c,l;t.getCellType=(e,t)=>e instanceof s.ArrayVertex?e.isLeftCorner(t)?o.ARRAYFORMULA:o.ARRAY:e instanceof s.FormulaCellVertex||e instanceof s.ParsingErrorVertex?o.FORMULA:e instanceof s.ValueCellVertex?o.VALUE:o.EMPTY,t.CellValueNoNumber=c,function(e){e.EMPTY="EMPTY",e.NUMBER="NUMBER",e.STRING="STRING",e.BOOLEAN="BOOLEAN",e.ERROR="ERROR"}(c||(t.CellValueNoNumber=c={})),t.CellValueJustNumber=l,function(e){e.NUMBER="NUMBER"}(l||(t.CellValueJustNumber=l={}));const h=Object.assign(Object.assign({},c),l);t.CellValueType=h;const d=Object.assign(Object.assign({},c),a.NumberType);t.CellValueDetailedType=d;t.CellValueTypeOrd=e=>{switch(e){case h.EMPTY:return 0;case h.NUMBER:return 1;case h.STRING:return 2;case h.BOOLEAN:return 3;case h.ERROR:return 4}throw new Error("Cell value not computed")};const p=e=>{if(e===a.EmptyValue)return h.EMPTY;if(e instanceof m||e instanceof u.SimpleRangeValue)return h.ERROR;if("string"==typeof e)return h.STRING;if((0,a.isExtendedNumber)(e))return h.NUMBER;if("boolean"==typeof e)return h.BOOLEAN;throw new Error("Cell value not computed")};t.getCellValueType=p;t.getCellValueDetailedType=e=>(0,a.isExtendedNumber)(e)?(0,a.getTypeOfExtendedNumber)(e):p(e);t.getCellValueFormat=e=>(0,a.isExtendedNumber)(e)?(0,a.getFormatOfExtendedNumber)(e):void 0;class m{constructor(e,t,r){this.type=e,this.message=t,this.root=r}static parsingError(){return new m(n.ERROR,i.ErrorMessage.ParseError)}attachRootVertex(e){return void 0===this.root?new m(this.type,this.message,e):this}}t.CellError=m;t.simpleRowAddress=(e,t)=>({sheet:e,row:t});t.invalidSimpleRowAddress=e=>e.row<0;t.simpleColumnAddress=(e,t)=>({sheet:e,col:t});t.invalidSimpleColumnAddress=e=>e.col<0;const g=(e,t,r)=>({sheet:e,col:t,row:r});t.simpleCellAddress=g;t.invalidSimpleCellAddress=e=>e.col<0||e.row<0;t.movedSimpleCellAddress=(e,t,r,n)=>g(t,e.col+r,e.row+n);t.addressKey=e=>`${e.sheet},${e.row},${e.col}`;t.absoluteSheetReference=(e,t)=>{var r;return null!==(r=e.sheet)&&void 0!==r?r:t.sheet};t.equalSimpleCellAddress=(e,t)=>e.sheet===t.sheet&&e.col===t.col&&e.row===t.row},function(e,t,r){"use strict";t.__esModule=!0,t.ValueCellVertex=t.SparseStrategy=t.SheetMapping=t.RangeVertex=t.RangeMapping=t.ParsingErrorVertex=t.Graph=t.FormulaCellVertex=t.EmptyCellVertex=t.DependencyGraph=t.DenseStrategy=t.ArrayVertex=t.ArrayMapping=t.AddressMapping=void 0;var n=r(76);t.DependencyGraph=n.DependencyGraph;var o=r(147);t.AddressMapping=o.AddressMapping;var s=r(154);t.Graph=s.Graph;var i=r(155);t.RangeMapping=i.RangeMapping;var a=r(156);t.SheetMapping=a.SheetMapping;var u=r(149);t.ArrayMapping=u.ArrayMapping;var c=r(151);t.FormulaCellVertex=c.FormulaCellVertex,t.ArrayVertex=c.ArrayVertex;var l=r(159);t.EmptyCellVertex=l.EmptyCellVertex;var h=r(160);t.ValueCellVertex=h.ValueCellVertex;var d=r(161);t.ParsingErrorVertex=d.ParsingErrorVertex;var p=r(153);t.RangeVertex=p.RangeVertex;var m=r(162);t.SparseStrategy=m.SparseStrategy;var g=r(163);t.DenseStrategy=g.DenseStrategy},function(e,t,r){"use strict";t.__esModule=!0,t.DependencyGraph=void 0,r(2);var n=r(73),o=r(77),s=r(74),i=r(139),a=r(132),u=r(141),c=r(140),l=r(78),h=r(142),d=r(143),p=r(75),m=r(147),g=r(149),f=r(150),E=r(151),T=r(154),y=r(155),A=r(156);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class R{constructor(e,t,r,a,u,c,h,d){this.addressMapping=e,this.rangeMapping=t,this.sheetMapping=r,this.arrayMapping=a,this.stats=u,this.lazilyTransformingAstService=c,this.functionRegistry=h,this.namedExpressions=d,this.changes=i.ContentChanges.empty(),this.dependencyQueryAddresses=e=>{if(e instanceof p.RangeVertex)return this.rangeDependencyQuery(e).map(e=>{let[t,r]=e;return t});{const t=this.formulaDependencyQuery(e);if(void 0!==t){const[e,r]=t;return r.map(t=>t instanceof l.NamedExpressionDependency?this.namedExpressions.namedExpressionOrPlaceholder(t.name,e.sheet).address:(0,s.isSimpleCellAddress)(t)?t:(0,n.simpleCellRange)(t.start,t.end))}return[]}},this.dependencyQueryVertices=e=>{if(e instanceof p.RangeVertex)return this.rangeDependencyQuery(e);{const t=this.formulaDependencyQuery(e);if(void 0!==t){const[e,r]=t;return r.map(t=>{if(t instanceof n.AbsoluteCellRange)return[t.start,this.rangeMapping.fetchRange(t.start,t.end)];if(t instanceof l.NamedExpressionDependency){const r=this.namedExpressions.namedExpressionOrPlaceholder(t.name,e.sheet);return[r.address,this.addressMapping.fetchCell(r.address)]}return[t,this.addressMapping.fetchCell(t)]})}return[]}},this.rangeDependencyQuery=e=>{const t=[],{smallerRangeVertex:r,restRange:o}=this.rangeMapping.findSmallerRange(e.range);let s;void 0!==r&&this.graph.adjacentNodes(r).has(e)?(s=o,t.push([new n.AbsoluteCellRange(r.start,r.end),r])):s=e.range;for(const e of s.addresses(this)){const r=this.addressMapping.getCell(e);void 0!==r&&t.push([e,r])}return t},this.formulaDependencyQuery=e=>{let t,r;if(!(e instanceof E.FormulaVertex))return;r=e.getAddress(this.lazilyTransformingAstService),t=e.getFormula(this.lazilyTransformingAstService);const n=(0,l.collectDependencies)(t,this.functionRegistry);return[r,(0,o.absolutizeDependencies)(n,r)]},this.graph=new T.Graph(this.dependencyQueryVertices)}static buildEmpty(e,t,r,n,o){return new R(new m.AddressMapping(t.chooseAddressMappingPolicy),new y.RangeMapping,new A.SheetMapping(t.translationPackage),new g.ArrayMapping,o,e,r,n)}setFormulaToCell(e,t,r,n,o,s){const i=E.FormulaVertex.fromAst(t,e,n,this.lazilyTransformingAstService.version());return this.exchangeOrAddFormulaVertex(i),this.processCellDependencies(r,i),this.graph.markNodeAsSpecialRecentlyChanged(i),o&&this.markAsVolatile(i),s&&this.markAsDependentOnStructureChange(i),this.correctInfiniteRangesDependency(e),this.getAndClearContentChanges()}setParsingErrorToCell(e,t){const r=this.shrinkPossibleArrayAndGetCell(e);return this.exchangeOrAddGraphNode(r,t),this.addressMapping.setCell(e,t),this.graph.markNodeAsSpecialRecentlyChanged(t),this.correctInfiniteRangesDependency(e),this.getAndClearContentChanges()}setValueToCell(e,t){const r=this.shrinkPossibleArrayAndGetCell(e);if(r instanceof p.ArrayVertex&&this.arrayMapping.removeArray(r.getRange()),r instanceof p.ValueCellVertex){r.getValues().rawValue!==t.rawValue&&(r.setValues(t),this.graph.markNodeAsSpecialRecentlyChanged(r))}else{const n=new p.ValueCellVertex(t.parsedValue,t.rawValue);this.exchangeOrAddGraphNode(r,n),this.addressMapping.setCell(e,n),this.graph.markNodeAsSpecialRecentlyChanged(n)}return this.correctInfiniteRangesDependency(e),this.getAndClearContentChanges()}setCellEmpty(e){const t=this.shrinkPossibleArrayAndGetCell(e);if(void 0===t)return i.ContentChanges.empty();if(this.graph.adjacentNodes(t).size>0){const r=new p.EmptyCellVertex;this.exchangeGraphNode(t,r),0===this.graph.adjacentNodesCount(r)?(this.removeVertex(r),this.addressMapping.removeCell(e)):(this.graph.markNodeAsSpecialRecentlyChanged(r),this.addressMapping.setCell(e,r))}else this.removeVertex(t),this.addressMapping.removeCell(e);return this.getAndClearContentChanges()}ensureThatVertexIsNonArrayCellVertex(e){if(e instanceof p.ArrayVertex)throw new Error("Illegal operation")}clearRecentlyChangedVertices(){this.graph.clearSpecialNodesRecentlyChanged()}verticesToRecompute(){return new Set([...this.graph.specialNodesRecentlyChanged,...this.volatileVertices()])}processCellDependencies(e,t){e.forEach(e=>{if(e instanceof n.AbsoluteCellRange){const r=e;let n=this.getRange(r.start,r.end);void 0===n&&(n=new p.RangeVertex(r),this.rangeMapping.setRange(n)),this.graph.addNode(n),r.isFinite()||this.graph.markNodeAsInfiniteRange(n);const{smallerRangeVertex:o,restRange:s}=this.rangeMapping.findSmallerRange(r);if(void 0!==o){if(this.graph.addEdge(o,n),n.bruteForce){n.bruteForce=!1;for(const e of r.addresses(this))this.graph.removeEdge(this.fetchCell(e),n)}}else n.bruteForce=!0;const i=this.arrayMapping.getArray(s);if(void 0!==i)this.graph.addEdge(i,n);else for(const e of s.addresses(this))this.graph.addEdge(this.fetchCellOrCreateEmpty(e),n);this.graph.addEdge(n,t),r.isFinite()&&this.correctInfiniteRangesDependenciesByRangeVertex(n)}else if(e instanceof l.NamedExpressionDependency){const r=t.getAddress(this.lazilyTransformingAstService).sheet,n=this.fetchNamedExpressionVertex(e.name,r);this.graph.addEdge(n,t)}else this.graph.addEdge(this.fetchCellOrCreateEmpty(e),t)})}fetchNamedExpressionVertex(e,t){const r=this.namedExpressions.namedExpressionOrPlaceholder(e,t);return this.fetchCellOrCreateEmpty(r.address)}exchangeNode(e,t){const r=this.fetchCellOrCreateEmpty(e),n=this.fetchCellOrCreateEmpty(t);this.addressMapping.removeCell(e),this.exchangeGraphNode(r,n)}correctInfiniteRangesDependency(e){let t=void 0;for(const r of this.graph.infiniteRanges){const n=r;n.range.addressInRange(e)&&(t=null!=t?t:this.fetchCellOrCreateEmpty(e),this.graph.addEdge(t,n))}}fetchCellOrCreateEmpty(e){let t=this.addressMapping.getCell(e);return void 0===t&&(t=new p.EmptyCellVertex,this.graph.addNode(t),this.addressMapping.setCell(e,t)),t}removeRows(e){this.stats.measure(d.StatType.ADJUSTING_GRAPH,()=>{for(const[t,r]of this.addressMapping.entriesFromRowsSpan(e)){for(const e of this.graph.adjacentNodes(r))this.graph.markNodeAsSpecialRecentlyChanged(e);if(r instanceof p.ArrayVertex){if(!r.isLeftCorner(t))continue;this.shrinkArrayToCorner(r),this.arrayMapping.removeArray(r.getRange())}this.removeVertex(r)}}),this.stats.measure(d.StatType.ADJUSTING_ADDRESS_MAPPING,()=>{this.addressMapping.removeRows(e)});const t=this.stats.measure(d.StatType.ADJUSTING_RANGES,()=>{const t=this.truncateRanges(e,e=>e.row);return this.getArrayVerticesRelatedToRanges(t)});return this.stats.measure(d.StatType.ADJUSTING_ARRAY_MAPPING,()=>{this.fixArraysAfterRemovingRows(e.sheet,e.rowStart,e.numberOfRows)}),this.addStructuralNodesToChangeSet(),{affectedArrays:t,contentChanges:this.getAndClearContentChanges()}}removeSheet(e){this.clearSheet(e);for(const[t,r]of this.addressMapping.sheetEntries(e)){for(const e of this.graph.adjacentNodes(r))this.graph.markNodeAsSpecialRecentlyChanged(e);this.removeVertex(r),this.addressMapping.removeCell(t)}this.stats.measure(d.StatType.ADJUSTING_RANGES,()=>{const t=this.rangeMapping.removeRangesInSheet(e);for(const e of t)this.removeVertex(e);this.stats.measure(d.StatType.ADJUSTING_ADDRESS_MAPPING,()=>{this.addressMapping.removeSheet(e)})})}clearSheet(e){const t=new Set;for(const[r,n]of this.addressMapping.sheetEntries(e))n instanceof p.ArrayVertex?t.add(n):this.setCellEmpty(r);for(const e of t.values())this.setArrayEmpty(e);this.addStructuralNodesToChangeSet()}removeColumns(e){this.stats.measure(d.StatType.ADJUSTING_GRAPH,()=>{for(const[t,r]of this.addressMapping.entriesFromColumnsSpan(e)){for(const e of this.graph.adjacentNodes(r))this.graph.markNodeAsSpecialRecentlyChanged(e);if(r instanceof p.ArrayVertex){if(!r.isLeftCorner(t))continue;this.shrinkArrayToCorner(r),this.arrayMapping.removeArray(r.getRange())}this.removeVertex(r)}}),this.stats.measure(d.StatType.ADJUSTING_ADDRESS_MAPPING,()=>{this.addressMapping.removeColumns(e)});const t=this.stats.measure(d.StatType.ADJUSTING_RANGES,()=>{const t=this.truncateRanges(e,e=>e.col);return this.getArrayVerticesRelatedToRanges(t)});return this.stats.measure(d.StatType.ADJUSTING_ARRAY_MAPPING,()=>this.fixArraysAfterRemovingColumns(e.sheet,e.columnStart,e.numberOfColumns)),this.addStructuralNodesToChangeSet(),{affectedArrays:t,contentChanges:this.getAndClearContentChanges()}}addRows(e){this.stats.measure(d.StatType.ADJUSTING_ADDRESS_MAPPING,()=>{this.addressMapping.addRows(e.sheet,e.rowStart,e.numberOfRows)});const t=this.stats.measure(d.StatType.ADJUSTING_RANGES,()=>{const t=this.rangeMapping.moveAllRangesInSheetAfterRowByRows(e.sheet,e.rowStart,e.numberOfRows);return this.fixRangesWhenAddingRows(e.sheet,e.rowStart,e.numberOfRows),this.getArrayVerticesRelatedToRanges(t.verticesWithChangedSize)});this.stats.measure(d.StatType.ADJUSTING_ARRAY_MAPPING,()=>{this.fixArraysAfterAddingRow(e.sheet,e.rowStart,e.numberOfRows)});for(const t of this.addressMapping.verticesFromRowsSpan(e))this.graph.markNodeAsSpecialRecentlyChanged(t);return this.addStructuralNodesToChangeSet(),{affectedArrays:t}}addColumns(e){this.stats.measure(d.StatType.ADJUSTING_ADDRESS_MAPPING,()=>{this.addressMapping.addColumns(e.sheet,e.columnStart,e.numberOfColumns)});const t=this.stats.measure(d.StatType.ADJUSTING_RANGES,()=>{const t=this.rangeMapping.moveAllRangesInSheetAfterColumnByColumns(e.sheet,e.columnStart,e.numberOfColumns);return this.fixRangesWhenAddingColumns(e.sheet,e.columnStart,e.numberOfColumns),this.getArrayVerticesRelatedToRanges(t.verticesWithChangedSize)});this.stats.measure(d.StatType.ADJUSTING_ARRAY_MAPPING,()=>this.fixArraysAfterAddingColumn(e.sheet,e.columnStart,e.numberOfColumns));for(const t of this.addressMapping.verticesFromColumnsSpan(e))this.graph.markNodeAsSpecialRecentlyChanged(t);return this.addStructuralNodesToChangeSet(),{affectedArrays:t,contentChanges:this.getAndClearContentChanges()}}ensureNoArrayInRange(e){if(this.arrayMapping.isFormulaArrayInRange(e))throw Error("It is not possible to move / replace cells with array")}isThereSpaceForArray(e){const t=e.getRangeOrUndef();if(void 0===t)return!1;for(const r of t.addresses(this)){const t=this.addressMapping.getCell(r);if(void 0!==t&&!(t instanceof p.EmptyCellVertex)&&t!==e)return!1}return!0}moveCells(e,t,r,n){for(const o of e.addressesWithDirection(t,r,this)){const i=(0,s.simpleCellAddress)(n,o.col+t,o.row+r);let a=this.addressMapping.getCell(o);const u=this.addressMapping.getCell(i);if(this.addressMapping.removeCell(o),void 0!==a){this.graph.markNodeAsSpecialRecentlyChanged(a),this.addressMapping.setCell(i,a);let t=void 0;for(const r of this.graph.adjacentNodes(a))r instanceof p.RangeVertex&&!e.containsRange(r.range)&&(t=null!=t?t:this.fetchCellOrCreateEmpty(o),this.graph.addEdge(t,r),this.graph.removeEdge(a,r));t&&(this.graph.markNodeAsSpecialRecentlyChanged(t),this.addressMapping.setCell(o,t))}if(void 0!==u){void 0===a&&this.addressMapping.removeCell(i);for(const e of this.graph.adjacentNodes(u))a=null!=a?a:this.fetchCellOrCreateEmpty(i),this.graph.addEdge(a,e),this.graph.markNodeAsSpecialRecentlyChanged(a);this.removeVertex(u)}}for(const t of this.rangeMapping.rangeVerticesContainedInRange(e))for(const r of this.graph.adjacentNodes(t))if(r instanceof p.RangeVertex&&!e.containsRange(r.range)){this.graph.removeEdge(t,r);for(const e of t.range.addresses(this)){const t=this.fetchCellOrCreateEmpty(e);this.graph.addEdge(t,r),this.addressMapping.setCell(e,t),this.graph.markNodeAsSpecialRecentlyChanged(t)}}this.rangeMapping.moveRangesInsideSourceRange(e,t,r,n)}setArrayEmpty(e){const t=n.AbsoluteCellRange.spanFrom(e.getAddress(this.lazilyTransformingAstService),e.width,e.height),r=this.graph.adjacentNodes(e);for(const e of t.addresses(this))this.addressMapping.removeCell(e);for(const t of r.values()){const r=(0,f.collectAddressesDependentToRange)(this.functionRegistry,t,e.getRange(),this.lazilyTransformingAstService,this);for(const e of r){const r=this.fetchCellOrCreateEmpty(e);this.graph.addEdge(r,t)}r.length>0&&this.graph.markNodeAsSpecialRecentlyChanged(t)}this.removeVertex(e),this.arrayMapping.removeArray(e.getRange())}addVertex(e,t){this.graph.addNode(t),this.addressMapping.setCell(e,t)}addArrayVertex(e,t){this.graph.addNode(t),this.setAddressMappingForArrayVertex(t,e)}*arrayFormulaNodes(){for(const e of this.graph.nodes)e instanceof p.ArrayVertex&&(yield e)}*entriesFromRowsSpan(e){yield*this.addressMapping.entriesFromRowsSpan(e)}*entriesFromColumnsSpan(e){yield*this.addressMapping.entriesFromColumnsSpan(e)}existsVertex(e){return this.addressMapping.has(e)}fetchCell(e){return this.addressMapping.fetchCell(e)}getCell(e){return this.addressMapping.getCell(e)}getCellValue(e){return this.addressMapping.getCellValue(e)}getRawValue(e){return this.addressMapping.getRawValue(e)}getScalarValue(e){const t=this.addressMapping.getCellValue(e);return t instanceof c.SimpleRangeValue?new s.CellError(s.ErrorType.VALUE,a.ErrorMessage.ScalarExpected):t}existsEdge(e,t){return this.graph.existsEdge(e,t)}getSheetId(e){return this.sheetMapping.fetch(e)}getSheetHeight(e){return this.addressMapping.getHeight(e)}getSheetWidth(e){return this.addressMapping.getWidth(e)}getArray(e){return this.arrayMapping.getArray(e)}setArray(e,t){this.arrayMapping.setArray(e,t)}getRange(e,t){return this.rangeMapping.getRange(e,t)}topSortWithScc(){return this.graph.topSortWithScc()}markAsVolatile(e){this.graph.markNodeAsSpecial(e)}markAsDependentOnStructureChange(e){this.graph.markNodeAsChangingWithStructure(e)}forceApplyPostponedTransformations(){for(const e of this.graph.nodes.values())e instanceof p.FormulaCellVertex&&e.ensureRecentData(this.lazilyTransformingAstService)}volatileVertices(){return this.graph.specialNodes}getArrayVerticesRelatedToRanges(e){const t=e.map(e=>this.graph.hasNode(e)?Array.from(this.graph.adjacentNodes(e)).filter(e=>e instanceof p.ArrayVertex):[]);return new Set(...t)}*rawValuesFromRange(e){for(const t of e.addresses(this)){const e=this.getScalarValue(t);e!==u.EmptyValue&&(yield[(0,u.getRawValue)(e),t])}}*entriesFromRange(e){for(const t of e.addresses(this))yield[t,this.getCell(t)]}exchangeGraphNode(e,t){this.graph.addNode(t);const r=this.graph.adjacentNodes(e);this.removeVertex(e),r.forEach(e=>{this.graph.hasNode(e)&&this.graph.addEdge(t,e)})}exchangeOrAddGraphNode(e,t){e?this.exchangeGraphNode(e,t):this.graph.addNode(t)}computeListOfValuesInRange(e){const t=[];for(const r of e.addresses(this)){const e=this.getScalarValue(r);t.push(e)}return t}shrinkArrayToCorner(e){this.cleanAddressMappingUnderArray(e);for(const t of this.adjacentArrayVertices(e)){let r;r=t instanceof E.FormulaVertex?this.formulaDirectDependenciesToArray(t,e):this.rangeDirectDependenciesToArray(t,e);let n=!1;for(const[o,s]of r)e.isLeftCorner(o)&&(n=!0),this.graph.addEdge(s,t),this.graph.markNodeAsSpecialRecentlyChanged(s);n||this.graph.removeEdge(e,t)}this.graph.markNodeAsSpecialRecentlyChanged(e)}isArrayInternalCell(e){const t=this.getCell(e);return t instanceof p.ArrayVertex&&!t.isLeftCorner(e)}getAndClearContentChanges(){const e=this.changes;return this.changes=i.ContentChanges.empty(),e}getAdjacentNodesAddresses(e){const t=this.graph.adjacentNodes(e),r=[];return t.forEach(e=>{const t=e;t instanceof p.RangeVertex?r.push((0,n.simpleCellRange)(t.start,t.end)):r.push(t.getAddress(this.lazilyTransformingAstService))}),r}correctInfiniteRangesDependenciesByRangeVertex(e){for(const t of this.graph.infiniteRanges){const r=t,n=e.range.intersectionWith(r.range);if(void 0!==n)for(const e of n.addresses(this))this.graph.addEdge(this.fetchCellOrCreateEmpty(e),t)}}cleanAddressMappingUnderArray(e){const t=e.getRange();for(const r of t.addresses(this)){const t=e.getArrayCellValue(r);this.getCell(r)===e?e.isLeftCorner(r)?this.changes.addChange(new s.CellError(s.ErrorType.REF),r,t):(this.addressMapping.removeCell(r),this.changes.addChange(u.EmptyValue,r,t)):this.changes.addChange(u.EmptyValue,r,t)}}*formulaDirectDependenciesToArray(e,t){var r;const[,o]=null!==(r=this.formulaDependencyQuery(e))&&void 0!==r?r:[];if(void 0!==o)for(const e of o)if(!(e instanceof l.NamedExpressionDependency||e instanceof n.AbsoluteCellRange)&&t.getRange().addressInRange(e)){const t=this.fetchCellOrCreateEmpty(e);yield[e,t]}}*rangeDirectDependenciesToArray(e,t){const{restRange:r}=this.rangeMapping.findSmallerRange(e.range);for(const e of r.addresses(this))if(t.getRange().addressInRange(e)){const t=this.fetchCellOrCreateEmpty(e);yield[e,t]}}*adjacentArrayVertices(e){const t=this.graph.adjacentNodes(e);for(const e of t)(e instanceof E.FormulaVertex||e instanceof p.RangeVertex)&&(yield e)}addStructuralNodesToChangeSet(){for(const e of this.graph.specialNodesStructuralChanges)this.graph.markNodeAsSpecialRecentlyChanged(e)}fixRangesWhenAddingRows(e,t,r){const o=Array.from(this.rangeMapping.rangesInSheet(e));for(const e of o)if(e.range.includesRow(t+r))if(e.bruteForce){const n=e.range.rangeWithSameWidth(t,r);for(const t of n.addresses(this))this.graph.addEdge(this.fetchCellOrCreateEmpty(t),e)}else{let t=e,r=this.rangeMapping.findSmallerRange(t.range);if(void 0!==r.smallerRangeVertex)continue;for(;void 0===r.smallerRangeVertex;){const e=new p.RangeVertex(n.AbsoluteCellRange.spanFrom(t.range.start,t.range.width(),t.range.height()-1));this.rangeMapping.setRange(e),this.graph.addNode(e);const o=new n.AbsoluteCellRange((0,s.simpleCellAddress)(t.range.start.sheet,t.range.start.col,t.range.end.row),t.range.end);this.addAllFromRange(o,t),this.graph.addEdge(e,t),t=e,r=this.rangeMapping.findSmallerRange(t.range)}this.graph.addEdge(r.smallerRangeVertex,t),this.addAllFromRange(r.restRange,t),this.graph.removeEdge(r.smallerRangeVertex,e)}}addAllFromRange(e,t){for(const r of e.addresses(this))this.graph.addEdge(this.fetchCellOrCreateEmpty(r),t)}fixRangesWhenAddingColumns(e,t,r){for(const o of this.rangeMapping.rangesInSheet(e))if(o.range.includesColumn(t+r)){let i;i=o.bruteForce?o.range.rangeWithSameHeight(t,r):n.AbsoluteCellRange.spanFrom((0,s.simpleCellAddress)(e,t,o.range.end.row),r,1);for(const e of i.addresses(this))this.graph.addEdge(this.fetchCellOrCreateEmpty(e),o)}}exchangeOrAddFormulaVertex(e){const t=e.getAddress(this.lazilyTransformingAstService),r=n.AbsoluteCellRange.spanFrom(t,e.width,e.height),o=this.shrinkPossibleArrayAndGetCell(t);if(e instanceof p.ArrayVertex&&this.setArray(r,e),this.exchangeOrAddGraphNode(o,e),this.addressMapping.setCell(t,e),e instanceof p.ArrayVertex){if(!this.isThereSpaceForArray(e))return;for(const t of r.addresses(this)){if(e.isLeftCorner(t))continue;const r=this.getCell(t);this.exchangeOrAddGraphNode(r,e)}}for(const t of r.addresses(this))this.addressMapping.setCell(t,e)}setAddressMappingForArrayVertex(e,t){if(this.addressMapping.setCell(t,e),!(e instanceof p.ArrayVertex))return;const r=n.AbsoluteCellRange.spanFromOrUndef(t,e.width,e.height);if(void 0!==r&&(this.setArray(r,e),this.isThereSpaceForArray(e)))for(const t of r.addresses(this))this.addressMapping.setCell(t,e)}truncateRanges(e,t){const{verticesToRemove:r,verticesToMerge:n,verticesWithChangedSize:o}=this.rangeMapping.truncateRanges(e,t);for(const[e,t]of n)this.mergeRangeVertices(e,t);for(const e of r)this.removeVertexAndCleanupDependencies(e);return o}fixArraysAfterAddingRow(e,t,r){if(this.arrayMapping.moveArrayVerticesAfterRowByRows(e,t,r),!(t<=0))for(const[,n]of this.arrayMapping.arraysInRows(h.RowsSpan.fromRowStartAndEnd(e,t-1,t-1))){const o=n.getRange();for(let i=o.start.col;i<=o.end.col;++i)for(let a=t;a<=o.end.row;++a){const t=(0,s.simpleCellAddress)(e,i,a),o=(0,s.simpleCellAddress)(e,i,a+r),c=n.getArrayCellValue(t);this.addressMapping.moveCell(o,t),this.changes.addChange(u.EmptyValue,o,c)}}}fixArraysAfterRemovingRows(e,t,r){if(this.arrayMapping.moveArrayVerticesAfterRowByRows(e,t,-r),!(t<=0))for(const[,r]of this.arrayMapping.arraysInRows(h.RowsSpan.fromRowStartAndEnd(e,t-1,t-1)))if(this.isThereSpaceForArray(r))for(const e of r.getRange().addresses(this))this.addressMapping.setCell(e,r);else this.setNoSpaceIfArray(r)}fixArraysAfterAddingColumn(e,t,r){if(this.arrayMapping.moveArrayVerticesAfterColumnByColumns(e,t,r),!(t<=0))for(const[,n]of this.arrayMapping.arraysInCols(h.ColumnsSpan.fromColumnStartAndEnd(e,t-1,t-1))){const o=n.getRange();for(let i=o.start.row;i<=o.end.row;++i)for(let a=t;a<=o.end.col;++a){const t=(0,s.simpleCellAddress)(e,a,i),o=(0,s.simpleCellAddress)(e,a+r,i),c=n.getArrayCellValue(t);this.addressMapping.moveCell(o,t),this.changes.addChange(u.EmptyValue,o,c)}}}fixArraysAfterRemovingColumns(e,t,r){if(this.arrayMapping.moveArrayVerticesAfterColumnByColumns(e,t,-r),!(t<=0))for(const[,r]of this.arrayMapping.arraysInCols(h.ColumnsSpan.fromColumnStartAndEnd(e,t-1,t-1)))if(this.isThereSpaceForArray(r))for(const e of r.getRange().addresses(this))this.addressMapping.setCell(e,r);else this.setNoSpaceIfArray(r)}shrinkPossibleArrayAndGetCell(e){const t=this.getCell(e);return t instanceof p.ArrayVertex?(this.setNoSpaceIfArray(t),this.getCell(e)):t}setNoSpaceIfArray(e){e instanceof p.ArrayVertex&&(this.shrinkArrayToCorner(e),e.setNoSpace())}removeVertex(e){this.removeVertexAndCleanupDependencies(e),e instanceof p.RangeVertex&&this.rangeMapping.removeRange(e)}mergeRangeVertices(e,t){const r=this.graph.adjacentNodes(t);this.removeVertexAndCleanupDependencies(t),this.graph.softRemoveEdge(e,t),r.forEach(t=>{this.graph.hasNode(t)&&this.graph.addEdge(e,t)})}removeVertexAndCleanupDependencies(e){const t=new Set(this.graph.removeNode(e));for(;t.size>0;){const e=t.values().next().value;t.delete(e);const[r,n]=e;this.graph.hasNode(n)&&0===this.graph.adjacentNodesCount(n)&&((n instanceof p.RangeVertex||n instanceof p.EmptyCellVertex)&&this.graph.removeNode(n).forEach(e=>t.add(e)),n instanceof p.RangeVertex?this.rangeMapping.removeRange(n):n instanceof p.EmptyCellVertex&&this.addressMapping.removeCell(r))}}}t.DependencyGraph=R},function(e,t,r){"use strict";t.__esModule=!0,t.filterDependenciesOutOfScope=t.absolutizeDependencies=void 0;var n=r(73),o=r(74),s=r(78);t.absolutizeDependencies=(e,t)=>e.map(e=>e.absolutize(t));t.filterDependenciesOutOfScope=e=>e.filter(e=>e instanceof s.NamedExpressionDependency||(e instanceof n.AbsoluteCellRange?!((0,o.invalidSimpleCellAddress)(e.start)||(0,o.invalidSimpleCellAddress)(e.end)):!(0,o.invalidSimpleCellAddress)(e)))},function(e,t,r){"use strict";t.__esModule=!0,t.simpleCellRangeToString=t.simpleCellRangeFromString=t.simpleCellAddressToString=t.simpleCellAddressFromString=t.collectDependencies=t.cellAddressFromString=t.buildProcedureAst=t.buildParsingErrorAst=t.buildLexerConfig=t.buildCellRangeAst=t.buildCellErrorAst=t.Unparser=t.RowRangeDependency=t.ParsingErrorType=t.ParserWithCaching=t.NamedExpressionDependency=t.FormulaLexer=t.ColumnRangeDependency=t.CellRangeDependency=t.CellAddress=t.AstNodeType=t.AddressDependency=void 0;var n=r(79);t.cellAddressFromString=n.cellAddressFromString,t.simpleCellAddressFromString=n.simpleCellAddressFromString,t.simpleCellAddressToString=n.simpleCellAddressToString,t.simpleCellRangeFromString=n.simpleCellRangeFromString,t.simpleCellRangeToString=n.simpleCellRangeToString;var o=r(80);t.CellAddress=o.CellAddress;var s=r(84);t.ParserWithCaching=s.ParserWithCaching;var i=r(137);t.collectDependencies=i.collectDependencies;var a=r(133);t.buildLexerConfig=a.buildLexerConfig;var u=r(131);t.FormulaLexer=u.FormulaLexer;var c=r(128);t.AstNodeType=c.AstNodeType,t.ParsingErrorType=c.ParsingErrorType,t.buildProcedureAst=c.buildProcedureAst,t.buildCellRangeAst=c.buildCellRangeAst,t.buildParsingErrorAst=c.buildParsingErrorAst,t.buildCellErrorAst=c.buildCellErrorAst;var l=r(136);t.Unparser=l.Unparser;var h=r(138);t.AddressDependency=h.AddressDependency,t.CellRangeDependency=h.CellRangeDependency,t.ColumnRangeDependency=h.ColumnRangeDependency,t.RowRangeDependency=h.RowRangeDependency,t.NamedExpressionDependency=h.NamedExpressionDependency},function(e,t,r){"use strict";t.__esModule=!0,t.columnAddressFromString=t.cellAddressFromString=void 0,t.columnIndexToLabel=f,t.rowAddressFromString=void 0,t.sheetIndexToString=E,t.simpleCellRangeToString=t.simpleCellRangeFromString=t.simpleCellAddressToString=t.simpleCellAddressFromString=void 0;var n=r(73),o=r(74),s=r(80),i=r(81),a=r(83),u=r(82);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const c=new RegExp(`^(${a.SHEET_NAME_PATTERN})?(\\${a.ABSOLUTE_OPERATOR}?)([A-Za-z]+)(\\${a.ABSOLUTE_OPERATOR}?)([0-9]+)$`),l=new RegExp(`^(${a.SHEET_NAME_PATTERN})?(\\${a.ABSOLUTE_OPERATOR}?)([A-Za-z]+)$`),h=new RegExp(`^(${a.SHEET_NAME_PATTERN})?(\\${a.ABSOLUTE_OPERATOR}?)([0-9]+)$`),d=new RegExp(`^${a.UNQUOTED_SHEET_NAME_PATTERN}$`);t.cellAddressFromString=(e,t,r)=>{const n=c.exec(t),o=g(n[6]);let i=T(n,e);if(void 0===i)return;null===i&&(i=void 0);const u=Number(n[8])-1;return n[5]===a.ABSOLUTE_OPERATOR&&n[7]===a.ABSOLUTE_OPERATOR?s.CellAddress.absolute(o,u,i):n[5]===a.ABSOLUTE_OPERATOR?s.CellAddress.absoluteCol(o,u-r.row,i):n[7]===a.ABSOLUTE_OPERATOR?s.CellAddress.absoluteRow(o-r.col,u,i):s.CellAddress.relative(o-r.col,u-r.row,i)};t.columnAddressFromString=(e,t,r)=>{const n=l.exec(t);let o=T(n,e);if(void 0===o)return;null===o&&(o=void 0);const s=g(n[6]);return n[5]===a.ABSOLUTE_OPERATOR?i.ColumnAddress.absolute(s,o):i.ColumnAddress.relative(s-r.col,o)};t.rowAddressFromString=(e,t,r)=>{const n=h.exec(t);let o=T(n,e);if(void 0===o)return;null===o&&(o=void 0);const s=Number(n[6])-1;return n[5]===a.ABSOLUTE_OPERATOR?u.RowAddress.absolute(s,o):u.RowAddress.relative(s-r.row,o)};const p=(e,t,r)=>{const n=c.exec(t),s=g(n[6]);let i=T(n,e);if(void 0===i)return;null===i&&(i=r);const a=Number(n[8])-1;return(0,o.simpleCellAddress)(i,s,a)};t.simpleCellAddressFromString=p;t.simpleCellRangeFromString=(e,t,r)=>{const o=t.split(a.RANGE_OPERATOR);if(2!==o.length)return;const[s,i]=o,u=p(e,s,r);if(void 0===u)return;const c=p(e,i,u.sheet);return void 0!==c&&u.sheet===c.sheet?(0,n.simpleCellRange)(u,c):void 0};const m=(e,t,r)=>{const n=f(t.col),o=E(t.sheet,e);if(void 0!==o)return r!==t.sheet?`${o}!${n}${t.row+1}`:`${n}${t.row+1}`};t.simpleCellAddressToString=m;function g(e){return 1===e.length?e.toUpperCase().charCodeAt(0)-65:e.split("").reduce((e,t)=>26*e+(t.toUpperCase().charCodeAt(0)-64),0)-1}function f(e){let t="";for(;e>=0;)t=String.fromCharCode(e%26+97)+t,e=Math.floor(e/26)-1;return t.toUpperCase()}function E(e,t){let r=t(e);if(void 0!==r)return d.test(r)?r:(r=r.replace(/'/g,"''"),`'${r}'`)}function T(e,t){var r;let n=null!==(r=e[3])&&void 0!==r?r:e[2];return n?(n=n.replace(/''/g,"'"),t(n)):null}t.simpleCellRangeToString=(e,t,r)=>{const n=m(e,t.start,r),o=m(e,t.end,t.start.sheet);return void 0===n||void 0===o?void 0:`${n}${a.RANGE_OPERATOR}${o}`}},function(e,t,r){"use strict";t.__esModule=!0,t.CellReferenceType=t.CellAddress=void 0;var n,o=r(74),s=r(79),i=r(81),a=r(82);t.CellReferenceType=n,function(e){e.CELL_REFERENCE_RELATIVE="CELL_REFERENCE",e.CELL_REFERENCE_ABSOLUTE="CELL_REFERENCE_ABSOLUTE",e.CELL_REFERENCE_ABSOLUTE_COL="CELL_REFERENCE_ABSOLUTE_COL",e.CELL_REFERENCE_ABSOLUTE_ROW="CELL_REFERENCE_ABSOLUTE_ROW"}(n||(t.CellReferenceType=n={}));class u{constructor(e,t,r,n){this.col=e,this.row=t,this.type=r,this.sheet=n}static fromColAndRow(e,t,r){return(e.isColumnAbsolute()&&t.isRowAbsolute()?u.absolute.bind(this):e.isColumnAbsolute()?u.absoluteCol.bind(this):t.isRowAbsolute()?u.absoluteRow.bind(this):u.relative.bind(this))(e.col,t.row,r)}static relative(e,t,r){return new u(e,t,n.CELL_REFERENCE_RELATIVE,r)}static absolute(e,t,r){return new u(e,t,n.CELL_REFERENCE_ABSOLUTE,r)}static absoluteCol(e,t,r){return new u(e,t,n.CELL_REFERENCE_ABSOLUTE_COL,r)}static absoluteRow(e,t,r){return new u(e,t,n.CELL_REFERENCE_ABSOLUTE_ROW,r)}toSimpleCellAddress(e){const t=(0,o.absoluteSheetReference)(this,e);return this.type===n.CELL_REFERENCE_ABSOLUTE?(0,o.simpleCellAddress)(t,this.col,this.row):this.type===n.CELL_REFERENCE_ABSOLUTE_ROW?(0,o.simpleCellAddress)(t,e.col+this.col,this.row):this.type===n.CELL_REFERENCE_ABSOLUTE_COL?(0,o.simpleCellAddress)(t,this.col,e.row+this.row):(0,o.simpleCellAddress)(t,e.col+this.col,e.row+this.row)}toColumnAddress(){const e=this.isColumnRelative()?i.ReferenceType.RELATIVE:i.ReferenceType.ABSOLUTE;return new i.ColumnAddress(e,this.col,this.sheet)}toRowAddress(){const e=this.isRowRelative()?i.ReferenceType.RELATIVE:i.ReferenceType.ABSOLUTE;return new a.RowAddress(e,this.row,this.sheet)}toSimpleColumnAddress(e){const t=(0,o.absoluteSheetReference)(this,e);let r=this.col;return this.isColumnRelative()&&(r+=e.col),(0,o.simpleColumnAddress)(t,r)}toSimpleRowAddress(e){const t=(0,o.absoluteSheetReference)(this,e);let r=this.row;return this.isRowRelative()&&(r+=e.row),(0,o.simpleRowAddress)(t,r)}isRowAbsolute(){return this.type===n.CELL_REFERENCE_ABSOLUTE||this.type===n.CELL_REFERENCE_ABSOLUTE_ROW}isColumnAbsolute(){return this.type===n.CELL_REFERENCE_ABSOLUTE||this.type===n.CELL_REFERENCE_ABSOLUTE_COL}isColumnRelative(){return this.type===n.CELL_REFERENCE_RELATIVE||this.type===n.CELL_REFERENCE_ABSOLUTE_ROW}isRowRelative(){return this.type===n.CELL_REFERENCE_RELATIVE||this.type===n.CELL_REFERENCE_ABSOLUTE_COL}isAbsolute(){return this.type===n.CELL_REFERENCE_ABSOLUTE&&void 0!==this.sheet}shiftedByRows(e){return new u(this.col,this.row+e,this.type,this.sheet)}shiftedByColumns(e){return new u(this.col+e,this.row,this.type,this.sheet)}moved(e,t,r){const n=void 0===this.sheet?void 0:e;return new u(this.col+t,this.row+r,this.type,n)}withSheet(e){return new u(this.col,this.row,this.type,e)}isInvalid(e){return(0,o.invalidSimpleCellAddress)(this.toSimpleCellAddress(e))}shiftRelativeDimensions(e,t){const r=this.isColumnAbsolute()?this.col:this.col+e,n=this.isRowAbsolute()?this.row:this.row+t;return new u(r,n,this.type,this.sheet)}shiftAbsoluteDimensions(e,t){const r=this.isColumnRelative()?this.col:this.col+e,n=this.isRowRelative()?this.row:this.row+t;return new u(r,n,this.type,this.sheet)}hash(e){const t=e&&void 0!==this.sheet?"#"+this.sheet:"";switch(this.type){case n.CELL_REFERENCE_RELATIVE:return`${t}#${this.row}R${this.col}`;case n.CELL_REFERENCE_ABSOLUTE:return`${t}#${this.row}A${this.col}`;case n.CELL_REFERENCE_ABSOLUTE_COL:return`${t}#${this.row}AC${this.col}`;case n.CELL_REFERENCE_ABSOLUTE_ROW:return`${t}#${this.row}AR${this.col}`}}unparse(e){const t=this.toSimpleCellAddress(e);if((0,o.invalidSimpleCellAddress)(t))return;const r=(0,s.columnIndexToLabel)(t.col),i=this.type===n.CELL_REFERENCE_ABSOLUTE||this.type===n.CELL_REFERENCE_ABSOLUTE_ROW?"$":"";return`${this.type===n.CELL_REFERENCE_ABSOLUTE||this.type===n.CELL_REFERENCE_ABSOLUTE_COL?"$":""}${r}${i}${t.row+1}`}exceedsSheetSizeLimits(e,t){return this.row>=t||this.col>=e}}t.CellAddress=u},function(e,t,r){"use strict";t.__esModule=!0,t.ReferenceType=t.ColumnAddress=void 0;var n,o=r(74),s=r(79);t.ReferenceType=n,function(e){e.RELATIVE="RELATIVE",e.ABSOLUTE="ABSOLUTE"}(n||(t.ReferenceType=n={}));class i{constructor(e,t,r){this.type=e,this.col=t,this.sheet=r}static absolute(e,t){return new i(n.ABSOLUTE,e,t)}static relative(e,t){return new i(n.RELATIVE,e,t)}static compareByAbsoluteAddress(e){return(t,r)=>t.toSimpleColumnAddress(e).col-r.toSimpleColumnAddress(e).col}isColumnAbsolute(){return this.type===n.ABSOLUTE}isColumnRelative(){return this.type===n.RELATIVE}isAbsolute(){return this.type===n.ABSOLUTE&&void 0!==this.sheet}moved(e,t,r){const n=void 0===this.sheet?void 0:e;return new i(this.type,this.col+t,n)}shiftedByColumns(e){return new i(this.type,this.col+e,this.sheet)}toSimpleColumnAddress(e){const t=(0,o.absoluteSheetReference)(this,e);let r=this.col;return this.isColumnRelative()&&(r=e.col+this.col),(0,o.simpleColumnAddress)(t,r)}shiftRelativeDimensions(e,t){const r=this.isColumnRelative()?this.col+e:this.col;return new i(this.type,r,this.sheet)}shiftAbsoluteDimensions(e,t){const r=this.isColumnAbsolute()?this.col+e:this.col;return new i(this.type,r,this.sheet)}withSheet(e){return new i(this.type,this.col,e)}isInvalid(e){return this.toSimpleColumnAddress(e).col<0}hash(e){const t=e&&void 0!==this.sheet?"#"+this.sheet:"";switch(this.type){case n.RELATIVE:return`${t}#COLR${this.col}`;case n.ABSOLUTE:return`${t}#COLA${this.col}`}}unparse(e){const t=this.toSimpleColumnAddress(e);if((0,o.invalidSimpleColumnAddress)(t))return;const r=(0,s.columnIndexToLabel)(t.col);return`${this.type===n.ABSOLUTE?"$":""}${r}`}exceedsSheetSizeLimits(e){return this.col>=e}}t.ColumnAddress=i},function(e,t,r){"use strict";t.__esModule=!0,t.RowAddress=void 0;var n=r(74),o=r(81);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s{constructor(e,t,r){this.type=e,this.row=t,this.sheet=r}static absolute(e,t){return new s(o.ReferenceType.ABSOLUTE,e,t)}static relative(e,t){return new s(o.ReferenceType.RELATIVE,e,t)}static compareByAbsoluteAddress(e){return(t,r)=>t.toSimpleRowAddress(e).row-r.toSimpleRowAddress(e).row}isRowAbsolute(){return this.type===o.ReferenceType.ABSOLUTE}isRowRelative(){return this.type===o.ReferenceType.RELATIVE}isAbsolute(){return this.type===o.ReferenceType.ABSOLUTE&&void 0!==this.sheet}moved(e,t,r){const n=void 0===this.sheet?void 0:e;return new s(this.type,this.row+r,n)}shiftedByRows(e){return new s(this.type,this.row+e,this.sheet)}toSimpleRowAddress(e){const t=(0,n.absoluteSheetReference)(this,e);let r=this.row;return this.isRowRelative()&&(r=e.row+this.row),(0,n.simpleRowAddress)(t,r)}shiftRelativeDimensions(e,t){const r=this.isRowRelative()?this.row+t:this.row;return new s(this.type,r,this.sheet)}shiftAbsoluteDimensions(e,t){const r=this.isRowAbsolute()?this.row+t:this.row;return new s(this.type,r,this.sheet)}withSheet(e){return new s(this.type,this.row,e)}isInvalid(e){return this.toSimpleRowAddress(e).row<0}hash(e){const t=e&&void 0!==this.sheet?"#"+this.sheet:"";switch(this.type){case o.ReferenceType.RELATIVE:return`${t}#ROWR${this.row}`;case o.ReferenceType.ABSOLUTE:return`${t}#ROWA${this.row}`}}unparse(e){const t=this.toSimpleRowAddress(e);if((0,n.invalidSimpleRowAddress)(t))return;return`${this.type===o.ReferenceType.ABSOLUTE?"$":""}${t.row+1}`}exceedsSheetSizeLimits(e){return this.row>=e}}t.RowAddress=s},function(e,t,r){"use strict";t.__esModule=!0,t.UNQUOTED_SHEET_NAME_PATTERN=t.UNICODE_LETTER_PATTERN=t.SHEET_NAME_PATTERN=t.ROW_REFERENCE_PATTERN=t.RANGE_OPERATOR=t.R1C1_CELL_REFERENCE_PATTERN=t.QUOTED_SHEET_NAME_PATTERN=t.ODFF_WHITESPACE_PATTERN=t.NON_RESERVED_CHARACTER_PATTERN=t.NAMED_EXPRESSION_PATTERN=t.COLUMN_REFERENCE_PATTERN=t.CELL_REFERENCE_WITH_NEXT_CHARACTER_PATTERN=t.CELL_REFERENCE_PATTERN=t.ALL_WHITESPACE_PATTERN=t.ALL_UNICODE_LETTERS_ARRAY=t.ALL_DIGITS_ARRAY=t.ABSOLUTE_OPERATOR=void 0;t.RANGE_OPERATOR=":";t.ABSOLUTE_OPERATOR="$";t.ALL_WHITESPACE_PATTERN="\\s+";t.ODFF_WHITESPACE_PATTERN="[ \\t\\n\\r]+";t.UNICODE_LETTER_PATTERN="A-Za-zÀ-ʯ";t.NON_RESERVED_CHARACTER_PATTERN="A-Za-zÀ-ʯ0-9_.";t.UNQUOTED_SHEET_NAME_PATTERN="[A-Za-zÀ-ʯ0-9_]+";t.QUOTED_SHEET_NAME_PATTERN="'(((?!').|'')*)'";const n="([A-Za-zÀ-ʯ0-9_]+|'(((?!').|'')*)')!";t.SHEET_NAME_PATTERN=n;const o=`(${n})?\\$?[A-Za-z]+\\$?[0-9]+`;t.CELL_REFERENCE_PATTERN=o;const s=`(${n})?\\$?[A-Za-z]+`;t.COLUMN_REFERENCE_PATTERN=s;const i=`(${n})?\\$?[0-9]+`;t.ROW_REFERENCE_PATTERN=i;t.R1C1_CELL_REFERENCE_PATTERN="[rR][0-9]*[cC][0-9]*";const a=`(${o})[^A-Za-zÀ-ʯ0-9_.]`;t.CELL_REFERENCE_WITH_NEXT_CHARACTER_PATTERN=a;t.NAMED_EXPRESSION_PATTERN="[A-Za-zÀ-ʯ_][A-Za-zÀ-ʯ0-9_.]*";t.ALL_DIGITS_ARRAY=["0","1","2","3","4","5","6","7","8","9"];const u=[...Array.from(Array(26)).map((e,t)=>t+"A".charCodeAt(0)),...Array.from(Array(26)).map((e,t)=>t+"a".charCodeAt(0)),...Array.from(Array(496)).map((e,t)=>t+192)].map(e=>String.fromCharCode(e));t.ALL_UNICODE_LETTERS_ARRAY=u},function(e,t,r){"use strict";t.__esModule=!0,t.ParserWithCaching=void 0,r(2);var n=r(85),o=r(74),s=r(78),i=r(79),a=r(128),u=r(129),c=r(130),l=r(131),h=r(133),d=r(136),p=r(81),m=r(82);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class g{constructor(e,t,r){this.config=e,this.functionRegistry=t,this.sheetMapping=r,this.statsCacheUsed=0,this.lexerConfig=(0,h.buildLexerConfig)(e),this.lexer=new l.FormulaLexer(this.lexerConfig),this.formulaParser=new l.FormulaParser(this.lexerConfig,this.sheetMapping),this.cache=new c.Cache(this.functionRegistry)}parse(e,t){this.formulaAddress=t;const r=this.tokenizeFormula(e);if(r.errors.length>0){const e=r.errors.map(e=>({type:a.ParsingErrorType.LexingError,message:e.message}));return{ast:(0,s.buildParsingErrorAst)(),errors:e,hasVolatileFunction:!1,hasStructuralChangeFunction:!1,dependencies:[]}}const n=this.computeHashFromTokens(r.tokens,t);let o=this.cache.get(n);if(void 0!==o)++this.statsCacheUsed;else{const e=this.bindWhitespacesToTokens(r.tokens),s=this.formulaParser.parseFromTokens(e,t);if(s.errors.length>0)return Object.assign(Object.assign({},s),{hasVolatileFunction:!1,hasStructuralChangeFunction:!1,dependencies:[]});o=this.cache.set(n,s.ast)}const{ast:i,hasVolatileFunction:u,hasStructuralChangeFunction:c}=o,l=this.convertReversedRangesToRegularRanges(i);return{ast:l,errors:[],hasVolatileFunction:u,hasStructuralChangeFunction:c,dependencies:(0,s.collectDependencies)(l,this.functionRegistry)}}convertReversedRangesToRegularRanges(e){switch(e.type){case s.AstNodeType.EMPTY:case s.AstNodeType.NUMBER:case s.AstNodeType.STRING:case s.AstNodeType.ERROR:case s.AstNodeType.ERROR_WITH_RAW_INPUT:case s.AstNodeType.CELL_REFERENCE:case s.AstNodeType.NAMED_EXPRESSION:return e;case s.AstNodeType.CELL_RANGE:{const{start:t,end:r}=e,n=this.orderCellRangeEnds(t,r);return Object.assign(Object.assign({},e),{start:n.start,end:n.end})}case s.AstNodeType.COLUMN_RANGE:{const{start:t,end:r}=e,n=this.orderColumnRangeEnds(t,r);return Object.assign(Object.assign({},e),{start:n.start,end:n.end})}case s.AstNodeType.ROW_RANGE:{const{start:t,end:r}=e,n=this.orderRowRangeEnds(t,r);return Object.assign(Object.assign({},e),{start:n.start,end:n.end})}case s.AstNodeType.PERCENT_OP:case s.AstNodeType.PLUS_UNARY_OP:case s.AstNodeType.MINUS_UNARY_OP:{const t=this.convertReversedRangesToRegularRanges(e.value);return Object.assign(Object.assign({},e),{value:t})}case s.AstNodeType.CONCATENATE_OP:case s.AstNodeType.EQUALS_OP:case s.AstNodeType.NOT_EQUAL_OP:case s.AstNodeType.LESS_THAN_OP:case s.AstNodeType.GREATER_THAN_OP:case s.AstNodeType.LESS_THAN_OR_EQUAL_OP:case s.AstNodeType.GREATER_THAN_OR_EQUAL_OP:case s.AstNodeType.MINUS_OP:case s.AstNodeType.PLUS_OP:case s.AstNodeType.TIMES_OP:case s.AstNodeType.DIV_OP:case s.AstNodeType.POWER_OP:{const t=this.convertReversedRangesToRegularRanges(e.left),r=this.convertReversedRangesToRegularRanges(e.right);return Object.assign(Object.assign({},e),{left:t,right:r})}case s.AstNodeType.PARENTHESIS:{const t=this.convertReversedRangesToRegularRanges(e.expression);return Object.assign(Object.assign({},e),{expression:t})}case s.AstNodeType.FUNCTION_CALL:{const t=e.args.map(e=>this.convertReversedRangesToRegularRanges(e));return Object.assign(Object.assign({},e),{args:t})}case s.AstNodeType.ARRAY:{const t=e.args.map(e=>e.map(e=>this.convertReversedRangesToRegularRanges(e)));return Object.assign(Object.assign({},e),{args:t})}}}orderCellRangeEnds(e,t){const r=[e,t],[n,o]=r.map(e=>e.toColumnAddress()).sort(p.ColumnAddress.compareByAbsoluteAddress(this.formulaAddress)),[i,a]=r.map(e=>e.toRowAddress()).sort(m.RowAddress.compareByAbsoluteAddress(this.formulaAddress)),[u,c]=r.map(e=>e.sheet).sort(g.compareSheetIds.bind(this));return{start:s.CellAddress.fromColAndRow(n,i,u),end:s.CellAddress.fromColAndRow(o,a,c)}}orderColumnRangeEnds(e,t){const r=[e,t],[n,o]=r.sort(p.ColumnAddress.compareByAbsoluteAddress(this.formulaAddress)),[s,i]=r.map(e=>e.sheet).sort(g.compareSheetIds.bind(this));return{start:new p.ColumnAddress(n.type,n.col,s),end:new p.ColumnAddress(o.type,o.col,i)}}orderRowRangeEnds(e,t){const r=[e,t],[n,o]=r.sort(m.RowAddress.compareByAbsoluteAddress(this.formulaAddress)),[s,i]=r.map(e=>e.sheet).sort(g.compareSheetIds.bind(this));return{start:new m.RowAddress(n.type,n.row,s),end:new m.RowAddress(o.type,o.row,i)}}static compareSheetIds(e,t){return(e=null!=e?e:1/0)-(t=null!=t?t:1/0)}fetchCachedResultForAst(e){const t=this.computeHashFromAst(e);return this.fetchCachedResult(t)}fetchCachedResult(e){const t=this.cache.get(e);if(void 0===t)throw new Error("There is no AST with such key in the cache");{const{ast:e,hasVolatileFunction:r,hasStructuralChangeFunction:n,relativeDependencies:o}=t;return{ast:e,errors:[],hasVolatileFunction:r,hasStructuralChangeFunction:n,dependencies:o}}}computeHashFromTokens(e,t){var r;let o="",s=0;for(;s<e.length;){const a=e[s];if((0,n.tokenMatcher)(a,h.CellReference)){const e=(0,i.cellAddressFromString)(this.sheetMapping,a.image,t);o=void 0===e?o.concat(a.image):o.concat(e.hash(!0))}else if((0,n.tokenMatcher)(a,h.ProcedureName)){const e=a.image.toUpperCase().slice(0,-1),t=null!==(r=this.lexerConfig.functionMapping[e])&&void 0!==r?r:e;o=o.concat(t,"(")}else if((0,n.tokenMatcher)(a,h.ColumnRange)){const[e,r]=a.image.split(":"),n=(0,i.columnAddressFromString)(this.sheetMapping,e,t),s=(0,i.columnAddressFromString)(this.sheetMapping,r,t);o=void 0===n||void 0===s?o.concat("!REF"):o.concat(n.hash(!0),":",s.hash(!0))}else if((0,n.tokenMatcher)(a,h.RowRange)){const[e,r]=a.image.split(":"),n=(0,i.rowAddressFromString)(this.sheetMapping,e,t),s=(0,i.rowAddressFromString)(this.sheetMapping,r,t);o=void 0===n||void 0===s?o.concat("!REF"):o.concat(n.hash(!0),":",s.hash(!0))}else o=o.concat(a.image);s++}return o}rememberNewAst(e){const t=this.computeHashFromAst(e);return this.cache.maybeSetAndThenGet(t,e)}computeHashFromAst(e){return"="+this.computeHashOfAstNode(e)}computeHashOfAstNode(e){switch(e.type){case s.AstNodeType.EMPTY:return e.leadingWhitespace||"";case s.AstNodeType.NUMBER:return(0,a.imageWithWhitespace)((0,d.formatNumber)(e.value,this.config.decimalSeparator),e.leadingWhitespace);case s.AstNodeType.STRING:return(0,a.imageWithWhitespace)('"'+e.value+'"',e.leadingWhitespace);case s.AstNodeType.NAMED_EXPRESSION:return(0,a.imageWithWhitespace)(e.expressionName,e.leadingWhitespace);case s.AstNodeType.FUNCTION_CALL:{const t=e.args.map(e=>this.computeHashOfAstNode(e)).join(this.config.functionArgSeparator),r=e.procedureName+"("+t+(0,a.imageWithWhitespace)(")",e.internalWhitespace);return(0,a.imageWithWhitespace)(r,e.leadingWhitespace)}case s.AstNodeType.CELL_REFERENCE:return(0,a.imageWithWhitespace)(e.reference.hash(!0),e.leadingWhitespace);case s.AstNodeType.COLUMN_RANGE:case s.AstNodeType.ROW_RANGE:case s.AstNodeType.CELL_RANGE:{const t=e.start.hash(e.sheetReferenceType!==a.RangeSheetReferenceType.RELATIVE),r=e.end.hash(e.sheetReferenceType===a.RangeSheetReferenceType.BOTH_ABSOLUTE);return(0,a.imageWithWhitespace)(t+":"+r,e.leadingWhitespace)}case s.AstNodeType.MINUS_UNARY_OP:return(0,a.imageWithWhitespace)("-"+this.computeHashOfAstNode(e.value),e.leadingWhitespace);case s.AstNodeType.PLUS_UNARY_OP:return(0,a.imageWithWhitespace)("+"+this.computeHashOfAstNode(e.value),e.leadingWhitespace);case s.AstNodeType.PERCENT_OP:return this.computeHashOfAstNode(e.value)+(0,a.imageWithWhitespace)("%",e.leadingWhitespace);case s.AstNodeType.ERROR:{const t=this.config.translationPackage.getErrorTranslation(e.error?e.error.type:o.ErrorType.ERROR);return(0,a.imageWithWhitespace)(t,e.leadingWhitespace)}case s.AstNodeType.ERROR_WITH_RAW_INPUT:return(0,a.imageWithWhitespace)(e.rawInput,e.leadingWhitespace);case s.AstNodeType.ARRAY:{const t=e.args.map(e=>e.map(e=>this.computeHashOfAstNode(e)).join(",")).join(";");return(0,a.imageWithWhitespace)("{"+t+(0,a.imageWithWhitespace)("}",e.internalWhitespace),e.leadingWhitespace)}case s.AstNodeType.PARENTHESIS:{const t="("+this.computeHashOfAstNode(e.expression)+(0,a.imageWithWhitespace)(")",e.internalWhitespace);return(0,a.imageWithWhitespace)(t,e.leadingWhitespace)}default:return this.computeHashOfAstNode(e.left)+(0,a.imageWithWhitespace)(u.binaryOpTokenMap[e.type],e.leadingWhitespace)+this.computeHashOfAstNode(e.right)}}bindWhitespacesToTokens(e){const t=[],r=e[0];(0,n.tokenMatcher)(r,this.lexerConfig.WhiteSpace)||t.push(r);for(let r=1;r<e.length;++r){const o=e[r];if((0,n.tokenMatcher)(o,this.lexerConfig.WhiteSpace))continue;const s=e[r-1];(0,n.tokenMatcher)(s,this.lexerConfig.WhiteSpace)&&(o.leadingWhitespace=s),t.push(o)}return t}tokenizeFormula(e){return this.lexer.tokenizeFormula(e)}}t.ParserWithCaching=g},function(e,t,r){"use strict";r.r(t),r.d(t,"clearCache",(function(){return g}));var n=r(86);r.d(t,"VERSION",(function(){return n.VERSION}));var o=r(87);r.d(t,"Parser",(function(){return o.Parser})),r.d(t,"CstParser",(function(){return o.CstParser})),r.d(t,"EmbeddedActionsParser",(function(){return o.EmbeddedActionsParser})),r.d(t,"ParserDefinitionErrorType",(function(){return o.ParserDefinitionErrorType})),r.d(t,"EMPTY_ALT",(function(){return o.EMPTY_ALT}));var s=r(93);r.d(t,"Lexer",(function(){return s.Lexer})),r.d(t,"LexerDefinitionErrorType",(function(){return s.LexerDefinitionErrorType}));var i=r(92);r.d(t,"createToken",(function(){return i.createToken})),r.d(t,"createTokenInstance",(function(){return i.createTokenInstance})),r.d(t,"EOF",(function(){return i.EOF})),r.d(t,"tokenLabel",(function(){return i.tokenLabel})),r.d(t,"tokenMatcher",(function(){return i.tokenMatcher})),r.d(t,"tokenName",(function(){return i.tokenName}));var a=r(106);r.d(t,"defaultGrammarResolverErrorProvider",(function(){return a.defaultGrammarResolverErrorProvider})),r.d(t,"defaultGrammarValidatorErrorProvider",(function(){return a.defaultGrammarValidatorErrorProvider})),r.d(t,"defaultParserErrorProvider",(function(){return a.defaultParserErrorProvider}));var u=r(113);r.d(t,"EarlyExitException",(function(){return u.EarlyExitException})),r.d(t,"isRecognitionException",(function(){return u.isRecognitionException})),r.d(t,"MismatchedTokenException",(function(){return u.MismatchedTokenException})),r.d(t,"NotAllInputParsedException",(function(){return u.NotAllInputParsedException})),r.d(t,"NoViableAltException",(function(){return u.NoViableAltException}));var c=r(99);r.d(t,"defaultLexerErrorProvider",(function(){return c.defaultLexerErrorProvider}));var l=r(91);r.d(t,"Alternation",(function(){return l.Alternation})),r.d(t,"Flat",(function(){return l.Flat})),r.d(t,"NonTerminal",(function(){return l.NonTerminal})),r.d(t,"Option",(function(){return l.Option})),r.d(t,"Repetition",(function(){return l.Repetition})),r.d(t,"RepetitionMandatory",(function(){return l.RepetitionMandatory})),r.d(t,"RepetitionMandatoryWithSeparator",(function(){return l.RepetitionMandatoryWithSeparator})),r.d(t,"RepetitionWithSeparator",(function(){return l.RepetitionWithSeparator})),r.d(t,"Rule",(function(){return l.Rule})),r.d(t,"Terminal",(function(){return l.Terminal})),r.d(t,"serializeGrammar",(function(){return l.serializeGrammar})),r.d(t,"serializeProduction",(function(){return l.serializeProduction}));var h=r(102);r.d(t,"GAstVisitor",(function(){return h.GAstVisitor}));var d=r(110);r.d(t,"assignOccurrenceIndices",(function(){return d.assignOccurrenceIndices})),r.d(t,"resolveGrammar",(function(){return d.resolveGrammar})),r.d(t,"validateGrammar",(function(){return d.validateGrammar}));var p=r(125);r.d(t,"createSyntaxDiagramsCode",(function(){return p.createSyntaxDiagramsCode}));var m=r(126);function g(){console.warn("The clearCache function was 'soft' removed from the Chevrotain API.\n\t It performs no action other than printing this message.\n\t Please avoid using it as it will be completely removed in the future")}r.d(t,"generateParserFactory",(function(){return m.generateParserFactory})),r.d(t,"generateParserModule",(function(){return m.generateParserModule}))},function(e,t,r){"use strict";r.r(t),r.d(t,"VERSION",(function(){return n}));var n="6.5.0"},function(e,t,r){"use strict";r.r(t),r.d(t,"END_OF_FILE",(function(){return R})),r.d(t,"DEFAULT_PARSER_CONFIG",(function(){return S})),r.d(t,"DEFAULT_RULE_CONFIG",(function(){return C})),r.d(t,"ParserDefinitionErrorType",(function(){return N})),r.d(t,"EMPTY_ALT",(function(){return v})),r.d(t,"Parser",(function(){return O})),r.d(t,"CstParser",(function(){return M})),r.d(t,"EmbeddedActionsParser",(function(){return I}));var n,o=r(88),s=r(89),i=r(92),a=r(104),u=r(106),c=r(110),l=r(112),h=r(114),d=r(115),p=r(118),m=r(119),g=r(120),f=r(121),E=r(122),T=r(123),y=r(124),A=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}),R=Object(i.createTokenInstance)(i.EOF,"",NaN,NaN,NaN,NaN,NaN,NaN);Object.freeze(R);var N,S=Object.freeze({recoveryEnabled:!1,maxLookahead:4,ignoredIssues:{},dynamicTokensEnabled:!1,outputCst:!0,errorMessageProvider:u.defaultParserErrorProvider,nodeLocationTracking:"none",traceInitPerf:!1,skipValidations:!1}),C=Object.freeze({recoveryValueFunc:function(){},resyncEnabled:!0});function v(e){return void 0===e&&(e=void 0),function(){return e}}!function(e){e[e.INVALID_RULE_NAME=0]="INVALID_RULE_NAME",e[e.DUPLICATE_RULE_NAME=1]="DUPLICATE_RULE_NAME",e[e.INVALID_RULE_OVERRIDE=2]="INVALID_RULE_OVERRIDE",e[e.DUPLICATE_PRODUCTIONS=3]="DUPLICATE_PRODUCTIONS",e[e.UNRESOLVED_SUBRULE_REF=4]="UNRESOLVED_SUBRULE_REF",e[e.LEFT_RECURSION=5]="LEFT_RECURSION",e[e.NONE_LAST_EMPTY_ALT=6]="NONE_LAST_EMPTY_ALT",e[e.AMBIGUOUS_ALTS=7]="AMBIGUOUS_ALTS",e[e.CONFLICT_TOKENS_RULES_NAMESPACE=8]="CONFLICT_TOKENS_RULES_NAMESPACE",e[e.INVALID_TOKEN_NAME=9]="INVALID_TOKEN_NAME",e[e.INVALID_NESTED_RULE_NAME=10]="INVALID_NESTED_RULE_NAME",e[e.DUPLICATE_NESTED_NAME=11]="DUPLICATE_NESTED_NAME",e[e.NO_NON_EMPTY_LOOKAHEAD=12]="NO_NON_EMPTY_LOOKAHEAD",e[e.AMBIGUOUS_PREFIX_ALTS=13]="AMBIGUOUS_PREFIX_ALTS",e[e.TOO_MANY_ALTS=14]="TOO_MANY_ALTS"}(N||(N={}));var O=function(){function e(e,t){void 0===t&&(t=S),this.ignoredIssues=S.ignoredIssues,this.definitionErrors=[],this.selfAnalysisDone=!1;this.initErrorHandler(t),this.initLexerAdapter(),this.initLooksAhead(t),this.initRecognizerEngine(e,t),this.initRecoverable(t),this.initTreeBuilder(t),this.initContentAssist(),this.initGastRecorder(t),this.initPerformanceTracer(t),Object(o.has)(t,"ignoredIssues")&&t.ignoredIssues!==S.ignoredIssues&&Object(o.PRINT_WARNING)("The <ignoredIssues> IParserConfig property is soft-deprecated and will be removed in future versions.\n\tPlease use the <IGNORE_AMBIGUITIES> flag on the relevant DSL method instead."),this.ignoredIssues=Object(o.has)(t,"ignoredIssues")?t.ignoredIssues:S.ignoredIssues,this.skipValidations=Object(o.has)(t,"skipValidations")?t.skipValidations:S.skipValidations}return e.performSelfAnalysis=function(e){e.performSelfAnalysis()},e.prototype.performSelfAnalysis=function(){var t=this;this.TRACE_INIT("performSelfAnalysis",(function(){var r;t.selfAnalysisDone=!0;var n=t.className;t.TRACE_INIT("toFastProps",(function(){Object(o.toFastProperties)(t)})),t.TRACE_INIT("Grammar Recording",(function(){try{t.enableRecording(),Object(o.forEach)(t.definedRulesNames,(function(e){var r=t[e].originalGrammarAction,n=void 0;t.TRACE_INIT(e+" Rule",(function(){n=t.topLevelRuleRecord(e,r)})),t.gastProductionsCache[e]=n}))}finally{t.disableRecording()}}));var i=[];if(t.TRACE_INIT("Grammar Resolving",(function(){i=Object(c.resolveGrammar)({rules:Object(o.values)(t.gastProductionsCache)}),t.definitionErrors.push.apply(t.definitionErrors,i)})),t.TRACE_INIT("Grammar Validations",(function(){if(Object(o.isEmpty)(i)&&!1===t.skipValidations){var e=Object(c.validateGrammar)({rules:Object(o.values)(t.gastProductionsCache),maxLookahead:t.maxLookahead,tokenTypes:Object(o.values)(t.tokensMap),ignoredIssues:t.ignoredIssues,errMsgProvider:u.defaultGrammarValidatorErrorProvider,grammarName:n});t.definitionErrors.push.apply(t.definitionErrors,e)}})),Object(o.isEmpty)(t.definitionErrors)&&(t.recoveryEnabled&&t.TRACE_INIT("computeAllProdsFollows",(function(){var e=Object(s.computeAllProdsFollows)(Object(o.values)(t.gastProductionsCache));t.resyncFollows=e})),t.TRACE_INIT("ComputeLookaheadFunctions",(function(){t.preComputeLookaheadFunctions(Object(o.values)(t.gastProductionsCache))}))),t.TRACE_INIT("expandAllNestedRuleNames",(function(){var e=Object(a.expandAllNestedRuleNames)(Object(o.values)(t.gastProductionsCache),t.fullRuleNameToShort);t.allRuleNames=e.allRuleNames})),!e.DEFER_DEFINITION_ERRORS_HANDLING&&!Object(o.isEmpty)(t.definitionErrors))throw r=Object(o.map)(t.definitionErrors,(function(e){return e.message})),new Error("Parser Definition Errors detected:\n "+r.join("\n-------------------------------\n"))}))},e.DEFER_DEFINITION_ERRORS_HANDLING=!1,e}();Object(o.applyMixins)(O,[l.Recoverable,h.LooksAhead,d.TreeBuilder,p.LexerAdapter,g.RecognizerEngine,m.RecognizerApi,f.ErrorHandler,E.ContentAssist,T.GastRecorder,y.PerformanceTracer]);var M=function(e){function t(t,r){void 0===r&&(r=S);var n=Object(o.cloneObj)(r);return n.outputCst=!0,e.call(this,t,n)||this}return A(t,e),t}(O),I=function(e){function t(t,r){void 0===r&&(r=S);var n=Object(o.cloneObj)(r);return n.outputCst=!1,e.call(this,t,n)||this}return A(t,e),t}(O)},function(e,t,r){"use strict";function n(e){return e&&0===e.length}function o(e){return null==e?[]:Object.keys(e)}function s(e){for(var t=[],r=Object.keys(e),n=0;n<r.length;n++)t.push(e[r[n]]);return t}function i(e,t){for(var r=[],n=o(e),s=0;s<n.length;s++){var i=n[s];r.push(t.call(null,e[i],i))}return r}function a(e,t){for(var r=[],n=0;n<e.length;n++)r.push(t.call(null,e[n],n));return r}function u(e){for(var t=[],r=0;r<e.length;r++){var n=e[r];Array.isArray(n)?t=t.concat(u(n)):t.push(n)}return t}function c(e){return n(e)?void 0:e[0]}function l(e){var t=e&&e.length;return t?e[t-1]:void 0}function h(e,t){if(Array.isArray(e))for(var r=0;r<e.length;r++)t.call(null,e[r],r);else{if(!_(e))throw Error("non exhaustive match");var n=o(e);for(r=0;r<n.length;r++){var s=n[r],i=e[s];t.call(null,i,s)}}}function d(e){return"string"==typeof e}function p(e){return void 0===e}function m(e){return e instanceof Function}function g(e,t){return void 0===t&&(t=1),e.slice(t,e.length)}function f(e,t){return void 0===t&&(t=1),e.slice(0,e.length-t)}function E(e,t){var r=[];if(Array.isArray(e))for(var n=0;n<e.length;n++){var o=e[n];t.call(null,o)&&r.push(o)}return r}function T(e,t){return E(e,(function(e){return!t(e)}))}function y(e,t){for(var r=Object.keys(e),n={},o=0;o<r.length;o++){var s=r[o],i=e[s];t(i)&&(n[s]=i)}return n}function A(e,t){return!!_(e)&&e.hasOwnProperty(t)}function R(e,t){return void 0!==C(e,(function(e){return e===t}))}function N(e){for(var t=[],r=0;r<e.length;r++)t.push(e[r]);return t}function S(e){var t={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&(t[r]=e[r]);return t}function C(e,t){for(var r=0;r<e.length;r++){var n=e[r];if(t.call(null,n))return n}}function v(e,t){for(var r=[],n=0;n<e.length;n++){var o=e[n];t.call(null,o)&&r.push(o)}return r}function O(e,t,r){for(var n=Array.isArray(e),i=n?e:s(e),a=n?[]:o(e),u=r,c=0;c<i.length;c++)u=t.call(null,u,i[c],n?c:a[c]);return u}function M(e){return T(e,(function(e){return null==e}))}function I(e,t){void 0===t&&(t=function(e){return e});var r=[];return O(e,(function(e,n){var o=t(n);return R(r,o)?e:(r.push(o),e.concat(n))}),[])}function w(e){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];var n=[null],o=n.concat(t);return Function.bind.apply(e,o)}function F(e){return Array.isArray(e)}function b(e){return e instanceof RegExp}function _(e){return e instanceof Object}function L(e,t){for(var r=0;r<e.length;r++)if(!t(e[r],r))return!1;return!0}function x(e,t){return T(e,(function(e){return R(t,e)}))}function P(e,t){for(var r=0;r<e.length;r++)if(t(e[r]))return!0;return!1}function U(e,t){for(var r=0;r<e.length;r++)if(e[r]===t)return r;return-1}function V(e,t){var r=N(e);return r.sort((function(e,r){return t(e)-t(r)})),r}function D(e,t){if(e.length!==t.length)throw Error("can't zipObject with different number of keys and values!");for(var r={},n=0;n<e.length;n++)r[e[n]]=t[n];return r}function B(e){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];for(var n=0;n<t.length;n++)for(var s=t[n],i=o(s),a=0;a<i.length;a++){var u=i[a];e[u]=s[u]}return e}function k(e){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];for(var n=0;n<t.length;n++){var s=t[n];if(!p(s))for(var i=o(s),a=0;a<i.length;a++){var u=i[a];A(e,u)||(e[u]=s[u])}}return e}function G(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return k.apply(null,[{}].concat(e))}function j(e,t){var r={};return h(e,(function(e){var n=t(e),o=r[n];o?o.push(e):r[n]=[e]})),r}function W(e,t){for(var r=S(e),n=o(t),s=0;s<n.length;s++){var i=n[s],a=t[i];r[i]=a}return r}function H(){}function z(e){return e}function Y(e){for(var t=[],r=0;r<e.length;r++){var n=e[r];t.push(void 0!==n?n:void 0)}return t}function X(e){console&&console.error&&console.error("Error: "+e)}function K(e){console&&console.warn&&console.warn("Warning: "+e)}function $(){return"function"==typeof Map}function q(e,t){t.forEach((function(t){var r=t.prototype;Object.getOwnPropertyNames(r).forEach((function(n){if("constructor"!==n){var o=Object.getOwnPropertyDescriptor(r,n);o&&(o.get||o.set)?Object.defineProperty(e.prototype,n,o):e.prototype[n]=t.prototype[n]}}))}))}function Q(e){function t(){}t.prototype=e;var r=new t;function n(){return typeof r.bar}return n(),n(),e}function Z(e){return e[e.length-1]}function J(e){var t=(new Date).getTime(),r=e();return{time:(new Date).getTime()-t,value:r}}r.r(t),r.d(t,"isEmpty",(function(){return n})),r.d(t,"keys",(function(){return o})),r.d(t,"values",(function(){return s})),r.d(t,"mapValues",(function(){return i})),r.d(t,"map",(function(){return a})),r.d(t,"flatten",(function(){return u})),r.d(t,"first",(function(){return c})),r.d(t,"last",(function(){return l})),r.d(t,"forEach",(function(){return h})),r.d(t,"isString",(function(){return d})),r.d(t,"isUndefined",(function(){return p})),r.d(t,"isFunction",(function(){return m})),r.d(t,"drop",(function(){return g})),r.d(t,"dropRight",(function(){return f})),r.d(t,"filter",(function(){return E})),r.d(t,"reject",(function(){return T})),r.d(t,"pick",(function(){return y})),r.d(t,"has",(function(){return A})),r.d(t,"contains",(function(){return R})),r.d(t,"cloneArr",(function(){return N})),r.d(t,"cloneObj",(function(){return S})),r.d(t,"find",(function(){return C})),r.d(t,"findAll",(function(){return v})),r.d(t,"reduce",(function(){return O})),r.d(t,"compact",(function(){return M})),r.d(t,"uniq",(function(){return I})),r.d(t,"partial",(function(){return w})),r.d(t,"isArray",(function(){return F})),r.d(t,"isRegExp",(function(){return b})),r.d(t,"isObject",(function(){return _})),r.d(t,"every",(function(){return L})),r.d(t,"difference",(function(){return x})),r.d(t,"some",(function(){return P})),r.d(t,"indexOf",(function(){return U})),r.d(t,"sortBy",(function(){return V})),r.d(t,"zipObject",(function(){return D})),r.d(t,"assign",(function(){return B})),r.d(t,"assignNoOverwrite",(function(){return k})),r.d(t,"defaults",(function(){return G})),r.d(t,"groupBy",(function(){return j})),r.d(t,"merge",(function(){return W})),r.d(t,"NOOP",(function(){return H})),r.d(t,"IDENTITY",(function(){return z})),r.d(t,"packArray",(function(){return Y})),r.d(t,"PRINT_ERROR",(function(){return X})),r.d(t,"PRINT_WARNING",(function(){return K})),r.d(t,"isES2015MapSupported",(function(){return $})),r.d(t,"applyMixins",(function(){return q})),r.d(t,"toFastProperties",(function(){return Q})),r.d(t,"peek",(function(){return Z})),r.d(t,"timer",(function(){return J}))},function(e,t,r){"use strict";r.r(t),r.d(t,"ResyncFollowsWalker",(function(){return l})),r.d(t,"computeAllProdsFollows",(function(){return h})),r.d(t,"buildBetweenProdsFollowPrefix",(function(){return d})),r.d(t,"buildInProdFollowPrefix",(function(){return p}));var n,o=r(90),s=r(100),i=r(88),a=r(103),u=r(91),c=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}),l=function(e){function t(t){var r=e.call(this)||this;return r.topProd=t,r.follows={},r}return c(t,e),t.prototype.startWalking=function(){return this.walk(this.topProd),this.follows},t.prototype.walkTerminal=function(e,t,r){},t.prototype.walkProdRef=function(e,t,r){var n=d(e.referencedRule,e.idx)+this.topProd.name,o=t.concat(r),i=new u.Flat({definition:o}),a=Object(s.first)(i);this.follows[n]=a},t}(o.RestWalker);function h(e){var t={};return Object(i.forEach)(e,(function(e){var r=new l(e).startWalking();Object(i.assign)(t,r)})),t}function d(e,t){return e.name+t+a.IN}function p(e){return e.terminalType.name+e.idx+a.IN}},function(e,t,r){"use strict";r.r(t),r.d(t,"RestWalker",(function(){return s}));var n=r(88),o=r(91),s=function(){function e(){}return e.prototype.walk=function(e,t){var r=this;void 0===t&&(t=[]),Object(n.forEach)(e.definition,(function(s,i){var a=Object(n.drop)(e.definition,i+1);if(s instanceof o.NonTerminal)r.walkProdRef(s,a,t);else if(s instanceof o.Terminal)r.walkTerminal(s,a,t);else if(s instanceof o.Flat)r.walkFlat(s,a,t);else if(s instanceof o.Option)r.walkOption(s,a,t);else if(s instanceof o.RepetitionMandatory)r.walkAtLeastOne(s,a,t);else if(s instanceof o.RepetitionMandatoryWithSeparator)r.walkAtLeastOneSep(s,a,t);else if(s instanceof o.RepetitionWithSeparator)r.walkManySep(s,a,t);else if(s instanceof o.Repetition)r.walkMany(s,a,t);else{if(!(s instanceof o.Alternation))throw Error("non exhaustive match");r.walkOr(s,a,t)}}))},e.prototype.walkTerminal=function(e,t,r){},e.prototype.walkProdRef=function(e,t,r){},e.prototype.walkFlat=function(e,t,r){var n=t.concat(r);this.walk(e,n)},e.prototype.walkOption=function(e,t,r){var n=t.concat(r);this.walk(e,n)},e.prototype.walkAtLeastOne=function(e,t,r){var n=[new o.Option({definition:e.definition})].concat(t,r);this.walk(e,n)},e.prototype.walkAtLeastOneSep=function(e,t,r){var n=i(e,t,r);this.walk(e,n)},e.prototype.walkMany=function(e,t,r){var n=[new o.Option({definition:e.definition})].concat(t,r);this.walk(e,n)},e.prototype.walkManySep=function(e,t,r){var n=i(e,t,r);this.walk(e,n)},e.prototype.walkOr=function(e,t,r){var s=this,i=t.concat(r);Object(n.forEach)(e.definition,(function(e){var t=new o.Flat({definition:[e]});s.walk(t,i)}))},e}();function i(e,t,r){return[new o.Option({definition:[new o.Terminal({terminalType:e.separator})].concat(e.definition)})].concat(t,r)}},function(e,t,r){"use strict";r.r(t),r.d(t,"AbstractProduction",(function(){return a})),r.d(t,"NonTerminal",(function(){return u})),r.d(t,"Rule",(function(){return c})),r.d(t,"Flat",(function(){return l})),r.d(t,"Option",(function(){return h})),r.d(t,"RepetitionMandatory",(function(){return d})),r.d(t,"RepetitionMandatoryWithSeparator",(function(){return p})),r.d(t,"Repetition",(function(){return m})),r.d(t,"RepetitionWithSeparator",(function(){return g})),r.d(t,"Alternation",(function(){return f})),r.d(t,"Terminal",(function(){return E})),r.d(t,"serializeGrammar",(function(){return T})),r.d(t,"serializeProduction",(function(){return y}));var n,o=r(88),s=r(92),i=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}),a=function(){function e(e){this.definition=e}return e.prototype.accept=function(e){e.visit(this),Object(o.forEach)(this.definition,(function(t){t.accept(e)}))},e}(),u=function(e){function t(t){var r=e.call(this,[])||this;return r.idx=1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),Object.defineProperty(t.prototype,"definition",{get:function(){return void 0!==this.referencedRule?this.referencedRule.definition:[]},set:function(e){},enumerable:!0,configurable:!0}),t.prototype.accept=function(e){e.visit(this)},t}(a),c=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.orgText="",Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),l=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.ignoreAmbiguities=!1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),h=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.idx=1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),d=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.idx=1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),p=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.idx=1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),m=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.idx=1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),g=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.idx=1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),f=function(e){function t(t){var r=e.call(this,t.definition)||this;return r.idx=1,r.ignoreAmbiguities=!1,r.hasPredicates=!1,Object(o.assign)(r,Object(o.pick)(t,(function(e){return void 0!==e}))),r}return i(t,e),t}(a),E=function(){function e(e){this.idx=1,Object(o.assign)(this,Object(o.pick)(e,(function(e){return void 0!==e})))}return e.prototype.accept=function(e){e.visit(this)},e}();function T(e){return Object(o.map)(e,y)}function y(e){function t(e){return Object(o.map)(e,y)}if(e instanceof u)return{type:"NonTerminal",name:e.nonTerminalName,idx:e.idx};if(e instanceof l)return{type:"Flat",definition:t(e.definition)};if(e instanceof h)return{type:"Option",idx:e.idx,definition:t(e.definition)};if(e instanceof d)return{type:"RepetitionMandatory",name:e.name,idx:e.idx,definition:t(e.definition)};if(e instanceof p)return{type:"RepetitionMandatoryWithSeparator",name:e.name,idx:e.idx,separator:y(new E({terminalType:e.separator})),definition:t(e.definition)};if(e instanceof g)return{type:"RepetitionWithSeparator",name:e.name,idx:e.idx,separator:y(new E({terminalType:e.separator})),definition:t(e.definition)};if(e instanceof m)return{type:"Repetition",name:e.name,idx:e.idx,definition:t(e.definition)};if(e instanceof f)return{type:"Alternation",name:e.name,idx:e.idx,definition:t(e.definition)};if(e instanceof E){var r={type:"Terminal",name:e.terminalType.name,label:Object(s.tokenLabel)(e.terminalType),idx:e.idx},n=e.terminalType.PATTERN;return e.terminalType.PATTERN&&(r.pattern=Object(o.isRegExp)(n)?n.source:n),r}if(e instanceof c)return{type:"Rule",name:e.name,orgText:e.orgText,definition:t(e.definition)};throw Error("non exhaustive match")}},function(e,t,r){"use strict";r.r(t),r.d(t,"tokenLabel",(function(){return i})),r.d(t,"tokenName",(function(){return a})),r.d(t,"hasTokenLabel",(function(){return u})),r.d(t,"createToken",(function(){return c})),r.d(t,"EOF",(function(){return l})),r.d(t,"createTokenInstance",(function(){return h})),r.d(t,"tokenMatcher",(function(){return d}));var n=r(88),o=r(93),s=r(98);function i(e){return u(e)?e.LABEL:e.name}function a(e){return e.name}function u(e){return Object(n.isString)(e.LABEL)&&""!==e.LABEL}function c(e){return function(e){var t=e.pattern,r={};r.name=e.name,Object(n.isUndefined)(t)||(r.PATTERN=t);if(Object(n.has)(e,"parent"))throw"The parent property is no longer supported.\nSee: https://github.com/SAP/chevrotain/issues/564#issuecomment-349062346 for details.";Object(n.has)(e,"categories")&&(r.CATEGORIES=e.categories);Object(s.augmentTokenTypes)([r]),Object(n.has)(e,"label")&&(r.LABEL=e.label);Object(n.has)(e,"group")&&(r.GROUP=e.group);Object(n.has)(e,"pop_mode")&&(r.POP_MODE=e.pop_mode);Object(n.has)(e,"push_mode")&&(r.PUSH_MODE=e.push_mode);Object(n.has)(e,"longer_alt")&&(r.LONGER_ALT=e.longer_alt);Object(n.has)(e,"line_breaks")&&(r.LINE_BREAKS=e.line_breaks);Object(n.has)(e,"start_chars_hint")&&(r.START_CHARS_HINT=e.start_chars_hint);return r}(e)}var l=c({name:"EOF",pattern:o.Lexer.NA});function h(e,t,r,n,o,s,i,a){return{image:t,startOffset:r,endOffset:n,startLine:o,endLine:s,startColumn:i,endColumn:a,tokenTypeIdx:e.tokenTypeIdx,tokenType:e}}function d(e,t){return Object(s.tokenStructuredMatcher)(e,t)}Object(s.augmentTokenTypes)([l])},function(e,t,r){"use strict";r.r(t),r.d(t,"LexerDefinitionErrorType",(function(){return n})),r.d(t,"Lexer",(function(){return l}));var n,o=r(94),s=r(88),i=r(98),a=r(99),u=r(97);!function(e){e[e.MISSING_PATTERN=0]="MISSING_PATTERN",e[e.INVALID_PATTERN=1]="INVALID_PATTERN",e[e.EOI_ANCHOR_FOUND=2]="EOI_ANCHOR_FOUND",e[e.UNSUPPORTED_FLAGS_FOUND=3]="UNSUPPORTED_FLAGS_FOUND",e[e.DUPLICATE_PATTERNS_FOUND=4]="DUPLICATE_PATTERNS_FOUND",e[e.INVALID_GROUP_TYPE_FOUND=5]="INVALID_GROUP_TYPE_FOUND",e[e.PUSH_MODE_DOES_NOT_EXIST=6]="PUSH_MODE_DOES_NOT_EXIST",e[e.MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE=7]="MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE",e[e.MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY=8]="MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY",e[e.MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST=9]="MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST",e[e.LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED=10]="LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED",e[e.SOI_ANCHOR_FOUND=11]="SOI_ANCHOR_FOUND",e[e.EMPTY_MATCH_PATTERN=12]="EMPTY_MATCH_PATTERN",e[e.NO_LINE_BREAKS_FLAGS=13]="NO_LINE_BREAKS_FLAGS",e[e.UNREACHABLE_PATTERN=14]="UNREACHABLE_PATTERN",e[e.IDENTIFY_TERMINATOR=15]="IDENTIFY_TERMINATOR",e[e.CUSTOM_LINE_BREAK=16]="CUSTOM_LINE_BREAK"}(n||(n={}));var c={deferDefinitionErrorsHandling:!1,positionTracking:"full",lineTerminatorsPattern:/\n|\r\n?/g,lineTerminatorCharacters:["\n","\r"],ensureOptimizations:!1,safeMode:!1,errorMessageProvider:a.defaultLexerErrorProvider,traceInitPerf:!1,skipValidations:!1};Object.freeze(c);var l=function(){function e(e,t){var r=this;if(void 0===t&&(t=c),this.lexerDefinition=e,this.lexerDefinitionErrors=[],this.lexerDefinitionWarning=[],this.patternIdxToConfig={},this.charCodeToPatternIdxToConfig={},this.modes=[],this.emptyGroups={},this.config=void 0,this.trackStartLines=!0,this.trackEndLines=!0,this.hasCustom=!1,this.canModeBeOptimized={},"boolean"==typeof t)throw Error("The second argument to the Lexer constructor is now an ILexerConfig Object.\na boolean 2nd argument is no longer supported");this.config=Object(s.merge)(c,t);var n=this.config.traceInitPerf;!0===n?(this.traceInitMaxIdent=1/0,this.traceInitPerf=!0):"number"==typeof n&&(this.traceInitMaxIdent=n,this.traceInitPerf=!0),this.traceInitIndent=-1,this.TRACE_INIT("Lexer Constructor",(function(){var n,a=!0;r.TRACE_INIT("Lexer Config handling",(function(){if(r.config.lineTerminatorsPattern===c.lineTerminatorsPattern)r.config.lineTerminatorsPattern=o.LineTerminatorOptimizedTester;else if(r.config.lineTerminatorCharacters===c.lineTerminatorCharacters)throw Error("Error: Missing <lineTerminatorCharacters> property on the Lexer config.\n\tFor details See: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#MISSING_LINE_TERM_CHARS");if(t.safeMode&&t.ensureOptimizations)throw Error('"safeMode" and "ensureOptimizations" flags are mutually exclusive.');r.trackStartLines=/full|onlyStart/i.test(r.config.positionTracking),r.trackEndLines=/full/i.test(r.config.positionTracking),Object(s.isArray)(e)?((n={modes:{}}).modes[o.DEFAULT_MODE]=Object(s.cloneArr)(e),n[o.DEFAULT_MODE]=o.DEFAULT_MODE):(a=!1,n=Object(s.cloneObj)(e))})),!1===r.config.skipValidations&&(r.TRACE_INIT("performRuntimeChecks",(function(){r.lexerDefinitionErrors=r.lexerDefinitionErrors.concat(Object(o.performRuntimeChecks)(n,r.trackStartLines,r.config.lineTerminatorCharacters))})),r.TRACE_INIT("performWarningRuntimeChecks",(function(){r.lexerDefinitionWarning=r.lexerDefinitionWarning.concat(Object(o.performWarningRuntimeChecks)(n,r.trackStartLines,r.config.lineTerminatorCharacters))}))),n.modes=n.modes?n.modes:{},Object(s.forEach)(n.modes,(function(e,t){n.modes[t]=Object(s.reject)(e,(function(e){return Object(s.isUndefined)(e)}))}));var l=Object(s.keys)(n.modes);if(Object(s.forEach)(n.modes,(function(e,n){r.TRACE_INIT("Mode: <"+n+"> processing",(function(){var a;(r.modes.push(n),!1===r.config.skipValidations&&r.TRACE_INIT("validatePatterns",(function(){r.lexerDefinitionErrors=r.lexerDefinitionErrors.concat(Object(o.validatePatterns)(e,l))})),Object(s.isEmpty)(r.lexerDefinitionErrors))&&(Object(i.augmentTokenTypes)(e),r.TRACE_INIT("analyzeTokenTypes",(function(){a=Object(o.analyzeTokenTypes)(e,{lineTerminatorCharacters:r.config.lineTerminatorCharacters,positionTracking:t.positionTracking,ensureOptimizations:t.ensureOptimizations,safeMode:t.safeMode,tracer:r.TRACE_INIT.bind(r)})})),r.patternIdxToConfig[n]=a.patternIdxToConfig,r.charCodeToPatternIdxToConfig[n]=a.charCodeToPatternIdxToConfig,r.emptyGroups=Object(s.merge)(r.emptyGroups,a.emptyGroups),r.hasCustom=a.hasCustom||r.hasCustom,r.canModeBeOptimized[n]=a.canBeOptimized)}))})),r.defaultMode=n.defaultMode,!Object(s.isEmpty)(r.lexerDefinitionErrors)&&!r.config.deferDefinitionErrorsHandling){var h=Object(s.map)(r.lexerDefinitionErrors,(function(e){return e.message})).join("-----------------------\n");throw new Error("Errors detected in definition of Lexer:\n"+h)}Object(s.forEach)(r.lexerDefinitionWarning,(function(e){Object(s.PRINT_WARNING)(e.message)})),r.TRACE_INIT("Choosing sub-methods implementations",(function(){if(o.SUPPORT_STICKY?(r.chopInput=s.IDENTITY,r.match=r.matchWithTest):(r.updateLastIndex=s.NOOP,r.match=r.matchWithExec),a&&(r.handleModes=s.NOOP),!1===r.trackStartLines&&(r.computeNewColumn=s.IDENTITY),!1===r.trackEndLines&&(r.updateTokenEndLineColumnLocation=s.NOOP),/full/i.test(r.config.positionTracking))r.createTokenInstance=r.createFullToken;else if(/onlyStart/i.test(r.config.positionTracking))r.createTokenInstance=r.createStartOnlyToken;else{if(!/onlyOffset/i.test(r.config.positionTracking))throw Error('Invalid <positionTracking> config option: "'+r.config.positionTracking+'"');r.createTokenInstance=r.createOffsetOnlyToken}r.hasCustom?(r.addToken=r.addTokenUsingPush,r.handlePayload=r.handlePayloadWithCustom):(r.addToken=r.addTokenUsingMemberAccess,r.handlePayload=r.handlePayloadNoCustom)})),r.TRACE_INIT("Failed Optimization Warnings",(function(){var e=Object(s.reduce)(r.canModeBeOptimized,(function(e,t,r){return!1===t&&e.push(r),e}),[]);if(t.ensureOptimizations&&!Object(s.isEmpty)(e))throw Error("Lexer Modes: < "+e.join(", ")+' > cannot be optimized.\n\t Disable the "ensureOptimizations" lexer config flag to silently ignore this and run the lexer in an un-optimized mode.\n\t Or inspect the console log for details on how to resolve these issues.')})),r.TRACE_INIT("clearRegExpParserCache",(function(){Object(u.clearRegExpParserCache)()})),r.TRACE_INIT("toFastProperties",(function(){Object(s.toFastProperties)(r)}))}))}return e.prototype.tokenize=function(e,t){if(void 0===t&&(t=this.defaultMode),!Object(s.isEmpty)(this.lexerDefinitionErrors)){var r=Object(s.map)(this.lexerDefinitionErrors,(function(e){return e.message})).join("-----------------------\n");throw new Error("Unable to Tokenize because Errors detected in definition of Lexer:\n"+r)}return this.tokenizeInternal(e,t)},e.prototype.tokenizeInternal=function(e,t){var r,n,i,a,u,c,l,h,d,p,m,g,f,E,T=this,y=e,A=y.length,R=0,N=0,S=this.hasCustom?0:Math.floor(e.length/10),C=new Array(S),v=[],O=this.trackStartLines?1:void 0,M=this.trackStartLines?1:void 0,I=Object(o.cloneEmptyGroups)(this.emptyGroups),w=this.trackStartLines,F=this.config.lineTerminatorsPattern,b=0,_=[],L=[],x=[],P=[];Object.freeze(P);var U=void 0;function V(){return _}function D(e){var t=Object(o.charCodeToOptimizedIndex)(e),r=L[t];return void 0===r?P:r}var B,k=function(e){if(1===x.length&&void 0===e.tokenType.PUSH_MODE){var t=T.config.errorMessageProvider.buildUnableToPopLexerModeMessage(e);v.push({offset:e.startOffset,line:void 0!==e.startLine?e.startLine:void 0,column:void 0!==e.startColumn?e.startColumn:void 0,length:e.image.length,message:t})}else{x.pop();var r=Object(s.last)(x);_=T.patternIdxToConfig[r],L=T.charCodeToPatternIdxToConfig[r],b=_.length;var n=T.canModeBeOptimized[r]&&!1===T.config.safeMode;U=L&&n?D:V}};function G(e){x.push(e),L=this.charCodeToPatternIdxToConfig[e],_=this.patternIdxToConfig[e],b=_.length,b=_.length;var t=this.canModeBeOptimized[e]&&!1===this.config.safeMode;U=L&&t?D:V}for(G.call(this,t);R<A;){u=null;var j=y.charCodeAt(R),W=U(j),H=W.length;for(r=0;r<H;r++){var z=(B=W[r]).pattern;if(c=null,!1!==(te=B.short)?j===te&&(u=z):!0===B.isCustom?null!==(E=z.exec(y,R,C,I))?(u=E[0],void 0!==E.payload&&(c=E.payload)):u=null:(this.updateLastIndex(z,R),u=this.match(z,e,R)),null!==u){if(void 0!==(a=B.longerAlt)){var Y=_[a],X=Y.pattern;l=null,!0===Y.isCustom?null!==(E=X.exec(y,R,C,I))?(i=E[0],void 0!==E.payload&&(l=E.payload)):i=null:(this.updateLastIndex(X,R),i=this.match(X,e,R)),i&&i.length>u.length&&(u=i,c=l,B=Y)}break}}if(null!==u){if(h=u.length,void 0!==(d=B.group)&&(p=B.tokenTypeIdx,m=this.createTokenInstance(u,R,p,B.tokenType,O,M,h),this.handlePayload(m,c),!1===d?N=this.addToken(C,N,m):I[d].push(m)),e=this.chopInput(e,h),R+=h,M=this.computeNewColumn(M,h),!0===w&&!0===B.canLineTerminator){var K=0,$=void 0,q=void 0;F.lastIndex=0;do{!0===($=F.test(u))&&(q=F.lastIndex-1,K++)}while(!0===$);0!==K&&(O+=K,M=h-q,this.updateTokenEndLineColumnLocation(m,d,q,K,O,M,h))}this.handleModes(B,k,G,m)}else{for(var Q=R,Z=O,J=M,ee=!1;!ee&&R<A;)for(y.charCodeAt(R),e=this.chopInput(e,1),R++,n=0;n<b;n++){var te,re=_[n];z=re.pattern;if(!1!==(te=re.short)?y.charCodeAt(R)===te&&(ee=!0):!0===re.isCustom?ee=null!==z.exec(y,R,C,I):(this.updateLastIndex(z,R),ee=null!==z.exec(e)),!0===ee)break}g=R-Q,f=this.config.errorMessageProvider.buildUnexpectedCharactersMessage(y,Q,g,Z,J),v.push({offset:Q,line:Z,column:J,length:g,message:f})}}return this.hasCustom||(C.length=N),{tokens:C,groups:I,errors:v}},e.prototype.handleModes=function(e,t,r,n){if(!0===e.pop){var o=e.push;t(n),void 0!==o&&r.call(this,o)}else void 0!==e.push&&r.call(this,e.push)},e.prototype.chopInput=function(e,t){return e.substring(t)},e.prototype.updateLastIndex=function(e,t){e.lastIndex=t},e.prototype.updateTokenEndLineColumnLocation=function(e,t,r,n,o,s,i){var a,u;void 0!==t&&(u=(a=r===i-1)?-1:0,1===n&&!0===a||(e.endLine=o+u,e.endColumn=s-1-u))},e.prototype.computeNewColumn=function(e,t){return e+t},e.prototype.createTokenInstance=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return null},e.prototype.createOffsetOnlyToken=function(e,t,r,n){return{image:e,startOffset:t,tokenTypeIdx:r,tokenType:n}},e.prototype.createStartOnlyToken=function(e,t,r,n,o,s){return{image:e,startOffset:t,startLine:o,startColumn:s,tokenTypeIdx:r,tokenType:n}},e.prototype.createFullToken=function(e,t,r,n,o,s,i){return{image:e,startOffset:t,endOffset:t+i-1,startLine:o,endLine:o,startColumn:s,endColumn:s+i-1,tokenTypeIdx:r,tokenType:n}},e.prototype.addToken=function(e,t,r){return 666},e.prototype.addTokenUsingPush=function(e,t,r){return e.push(r),t},e.prototype.addTokenUsingMemberAccess=function(e,t,r){return e[t]=r,++t},e.prototype.handlePayload=function(e,t){},e.prototype.handlePayloadNoCustom=function(e,t){},e.prototype.handlePayloadWithCustom=function(e,t){null!==t&&(e.payload=t)},e.prototype.match=function(e,t,r){return null},e.prototype.matchWithTest=function(e,t,r){return!0===e.test(t)?t.substring(r,e.lastIndex):null},e.prototype.matchWithExec=function(e,t){var r=e.exec(t);return null!==r?r[0]:r},e.prototype.TRACE_INIT=function(e,t){if(!0===this.traceInitPerf){this.traceInitIndent++;var r=new Array(this.traceInitIndent+1).join("\t");this.traceInitIndent<this.traceInitMaxIdent&&console.log(r+"--\x3e <"+e+">");var n=Object(s.timer)(t),o=n.time,i=n.value,a=o>10?console.warn:console.log;return this.traceInitIndent<this.traceInitMaxIdent&&a(r+"<-- <"+e+"> time: "+o+"ms"),this.traceInitIndent--,i}return t()},e.SKIPPED="This marks a skipped Token pattern, this means each token identified by it willbe consumed and then thrown into oblivion, this can be used to for example to completely ignore whitespace.",e.NA=/NOT_APPLICABLE/,e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"DEFAULT_MODE",(function(){return h})),r.d(t,"MODES",(function(){return d})),r.d(t,"SUPPORT_STICKY",(function(){return p})),r.d(t,"disableSticky",(function(){return m})),r.d(t,"enableSticky",(function(){return g})),r.d(t,"analyzeTokenTypes",(function(){return f})),r.d(t,"validatePatterns",(function(){return E})),r.d(t,"findMissingPatterns",(function(){return T})),r.d(t,"findInvalidPatterns",(function(){return y})),r.d(t,"findEndOfInputAnchor",(function(){return R})),r.d(t,"findEmptyMatchRegExps",(function(){return N})),r.d(t,"findStartOfInputAnchor",(function(){return C})),r.d(t,"findUnsupportedFlags",(function(){return v})),r.d(t,"findDuplicatePatterns",(function(){return O})),r.d(t,"findInvalidGroupType",(function(){return M})),r.d(t,"findModesThatDoNotExist",(function(){return I})),r.d(t,"findUnreachablePatterns",(function(){return w})),r.d(t,"addStartOfInput",(function(){return F})),r.d(t,"addStickyFlag",(function(){return b})),r.d(t,"performRuntimeChecks",(function(){return _})),r.d(t,"performWarningRuntimeChecks",(function(){return L})),r.d(t,"cloneEmptyGroups",(function(){return x})),r.d(t,"isCustomPattern",(function(){return P})),r.d(t,"isShortPattern",(function(){return U})),r.d(t,"LineTerminatorOptimizedTester",(function(){return V})),r.d(t,"buildLineBreakIssueMessage",(function(){return B})),r.d(t,"minOptimizationVal",(function(){return j})),r.d(t,"charCodeToOptimizedIndex",(function(){return W}));var n,o=r(95),s=r(93),i=r(88),a=r(96),u=r(97),c=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}),l="PATTERN",h="defaultMode",d="modes",p="boolean"==typeof new RegExp("(?:)").sticky;function m(){p=!1}function g(){p=!0}function f(e,t){var r,n=(t=Object(i.defaults)(t,{useSticky:p,debug:!1,safeMode:!1,positionTracking:"full",lineTerminatorCharacters:["\r","\n"],tracer:function(e,t){return t()}})).tracer;n("initCharCodeToOptimizedIndexMap",(function(){!function(){if(Object(i.isEmpty)(H)){H=new Array(65536);for(var e=0;e<65536;e++)H[e]=e>255?255+~~(e/255):e}}()})),n("Reject Lexer.NA",(function(){r=Object(i.reject)(e,(function(e){return e[l]===s.Lexer.NA}))}));var o,u,c,h,d,m,g,f,E,T,y,A=!1;n("Transform Patterns",(function(){A=!1,o=Object(i.map)(r,(function(e){var r=e[l];if(Object(i.isRegExp)(r)){var n=r.source;return 1===n.length&&"^"!==n&&"$"!==n&&"."!==n?n:2!==n.length||"\\"!==n[0]||Object(i.contains)(["d","D","s","S","t","r","n","t","0","c","b","B","f","v","w","W"],n[1])?t.useSticky?b(r):F(r):n[1]}if(Object(i.isFunction)(r))return A=!0,{exec:r};if(Object(i.has)(r,"exec"))return A=!0,r;if("string"==typeof r){if(1===r.length)return r;var o=r.replace(/[\\^$.*+?()[\]{}|]/g,"\\$&"),s=new RegExp(o);return t.useSticky?b(s):F(s)}throw Error("non exhaustive match")}))})),n("misc mapping",(function(){u=Object(i.map)(r,(function(e){return e.tokenTypeIdx})),c=Object(i.map)(r,(function(e){var t=e.GROUP;if(t!==s.Lexer.SKIPPED){if(Object(i.isString)(t))return t;if(Object(i.isUndefined)(t))return!1;throw Error("non exhaustive match")}})),h=Object(i.map)(r,(function(e){var t=e.LONGER_ALT;if(t)return Object(i.indexOf)(r,t)})),d=Object(i.map)(r,(function(e){return e.PUSH_MODE})),m=Object(i.map)(r,(function(e){return Object(i.has)(e,"POP_MODE")}))})),n("Line Terminator Handling",(function(){var e=k(t.lineTerminatorCharacters);g=Object(i.map)(r,(function(e){return!1})),"onlyOffset"!==t.positionTracking&&(g=Object(i.map)(r,(function(t){return Object(i.has)(t,"LINE_BREAKS")?t.LINE_BREAKS:!1===D(t,e)?Object(a.canMatchCharCode)(e,t.PATTERN):void 0})))})),n("Misc Mapping #2",(function(){f=Object(i.map)(r,P),E=Object(i.map)(o,U),T=Object(i.reduce)(r,(function(e,t){var r=t.GROUP;return Object(i.isString)(r)&&r!==s.Lexer.SKIPPED&&(e[r]=[]),e}),{}),y=Object(i.map)(o,(function(e,t){return{pattern:o[t],longerAlt:h[t],canLineTerminator:g[t],isCustom:f[t],short:E[t],group:c[t],push:d[t],pop:m[t],tokenTypeIdx:u[t],tokenType:r[t]}}))}));var R=!0,N=[];return t.safeMode||n("First Char Optimization",(function(){N=Object(i.reduce)(r,(function(e,r,n){if("string"==typeof r.PATTERN){var o=W(r.PATTERN.charCodeAt(0));G(e,o,y[n])}else if(Object(i.isArray)(r.START_CHARS_HINT)){var s;Object(i.forEach)(r.START_CHARS_HINT,(function(t){var r=W("string"==typeof t?t.charCodeAt(0):t);s!==r&&(s=r,G(e,r,y[n]))}))}else if(Object(i.isRegExp)(r.PATTERN))if(r.PATTERN.unicode)R=!1,t.ensureOptimizations&&Object(i.PRINT_ERROR)(a.failedOptimizationPrefixMsg+"\tUnable to analyze < "+r.PATTERN.toString()+" > pattern.\n\tThe regexp unicode flag is not currently supported by the regexp-to-ast library.\n\tThis will disable the lexer's first char optimizations.\n\tFor details See: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#UNICODE_OPTIMIZE");else{var u=Object(a.getOptimizedStartCodesIndices)(r.PATTERN,t.ensureOptimizations);Object(i.isEmpty)(u)&&(R=!1),Object(i.forEach)(u,(function(t){G(e,t,y[n])}))}else t.ensureOptimizations&&Object(i.PRINT_ERROR)(a.failedOptimizationPrefixMsg+"\tTokenType: <"+r.name+"> is using a custom token pattern without providing <start_chars_hint> parameter.\n\tThis will disable the lexer's first char optimizations.\n\tFor details See: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#CUSTOM_OPTIMIZE"),R=!1;return e}),[])})),n("ArrayPacking",(function(){N=Object(i.packArray)(N)})),{emptyGroups:T,patternIdxToConfig:y,charCodeToPatternIdxToConfig:N,hasCustom:A,canBeOptimized:R}}function E(e,t){var r=[],n=T(e);r=r.concat(n.errors);var o=y(n.valid),s=o.valid;return r=(r=(r=(r=(r=r.concat(o.errors)).concat(function(e){var t=[],r=Object(i.filter)(e,(function(e){return Object(i.isRegExp)(e[l])}));return t=(t=(t=(t=(t=t.concat(R(r))).concat(C(r))).concat(v(r))).concat(O(r))).concat(N(r))}(s))).concat(M(s))).concat(I(s,t))).concat(w(s))}function T(e){var t=Object(i.filter)(e,(function(e){return!Object(i.has)(e,l)}));return{errors:Object(i.map)(t,(function(e){return{message:"Token Type: ->"+e.name+"<- missing static 'PATTERN' property",type:s.LexerDefinitionErrorType.MISSING_PATTERN,tokenTypes:[e]}})),valid:Object(i.difference)(e,t)}}function y(e){var t=Object(i.filter)(e,(function(e){var t=e[l];return!(Object(i.isRegExp)(t)||Object(i.isFunction)(t)||Object(i.has)(t,"exec")||Object(i.isString)(t))}));return{errors:Object(i.map)(t,(function(e){return{message:"Token Type: ->"+e.name+"<- static 'PATTERN' can only be a RegExp, a Function matching the {CustomPatternMatcherFunc} type or an Object matching the {ICustomPattern} interface.",type:s.LexerDefinitionErrorType.INVALID_PATTERN,tokenTypes:[e]}})),valid:Object(i.difference)(e,t)}}var A=/[^\\][\$]/;function R(e){var t=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.found=!1,t}return c(t,e),t.prototype.visitEndAnchor=function(e){this.found=!0},t}(o.BaseRegExpVisitor),r=Object(i.filter)(e,(function(e){var r=e[l];try{var n=Object(u.getRegExpAst)(r),o=new t;return o.visit(n),o.found}catch(e){return A.test(r.source)}}));return Object(i.map)(r,(function(e){return{message:"Unexpected RegExp Anchor Error:\n\tToken Type: ->"+e.name+"<- static 'PATTERN' cannot contain end of input anchor '$'\n\tSee sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#ANCHORS\tfor details.",type:s.LexerDefinitionErrorType.EOI_ANCHOR_FOUND,tokenTypes:[e]}}))}function N(e){var t=Object(i.filter)(e,(function(e){return e[l].test("")}));return Object(i.map)(t,(function(e){return{message:"Token Type: ->"+e.name+"<- static 'PATTERN' must not match an empty string",type:s.LexerDefinitionErrorType.EMPTY_MATCH_PATTERN,tokenTypes:[e]}}))}var S=/[^\\[][\^]|^\^/;function C(e){var t=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.found=!1,t}return c(t,e),t.prototype.visitStartAnchor=function(e){this.found=!0},t}(o.BaseRegExpVisitor),r=Object(i.filter)(e,(function(e){var r=e[l];try{var n=Object(u.getRegExpAst)(r),o=new t;return o.visit(n),o.found}catch(e){return S.test(r.source)}}));return Object(i.map)(r,(function(e){return{message:"Unexpected RegExp Anchor Error:\n\tToken Type: ->"+e.name+"<- static 'PATTERN' cannot contain start of input anchor '^'\n\tSee https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#ANCHORS\tfor details.",type:s.LexerDefinitionErrorType.SOI_ANCHOR_FOUND,tokenTypes:[e]}}))}function v(e){var t=Object(i.filter)(e,(function(e){var t=e[l];return t instanceof RegExp&&(t.multiline||t.global)}));return Object(i.map)(t,(function(e){return{message:"Token Type: ->"+e.name+"<- static 'PATTERN' may NOT contain global('g') or multiline('m')",type:s.LexerDefinitionErrorType.UNSUPPORTED_FLAGS_FOUND,tokenTypes:[e]}}))}function O(e){var t=[],r=Object(i.map)(e,(function(r){return Object(i.reduce)(e,(function(e,n){return r.PATTERN.source!==n.PATTERN.source||Object(i.contains)(t,n)||n.PATTERN===s.Lexer.NA||(t.push(n),e.push(n)),e}),[])}));r=Object(i.compact)(r);var n=Object(i.filter)(r,(function(e){return e.length>1}));return Object(i.map)(n,(function(e){var t=Object(i.map)(e,(function(e){return e.name}));return{message:"The same RegExp pattern ->"+Object(i.first)(e).PATTERN+"<-has been used in all of the following Token Types: "+t.join(", ")+" <-",type:s.LexerDefinitionErrorType.DUPLICATE_PATTERNS_FOUND,tokenTypes:e}}))}function M(e){var t=Object(i.filter)(e,(function(e){if(!Object(i.has)(e,"GROUP"))return!1;var t=e.GROUP;return t!==s.Lexer.SKIPPED&&t!==s.Lexer.NA&&!Object(i.isString)(t)}));return Object(i.map)(t,(function(e){return{message:"Token Type: ->"+e.name+"<- static 'GROUP' can only be Lexer.SKIPPED/Lexer.NA/A String",type:s.LexerDefinitionErrorType.INVALID_GROUP_TYPE_FOUND,tokenTypes:[e]}}))}function I(e,t){var r=Object(i.filter)(e,(function(e){return void 0!==e.PUSH_MODE&&!Object(i.contains)(t,e.PUSH_MODE)}));return Object(i.map)(r,(function(e){return{message:"Token Type: ->"+e.name+"<- static 'PUSH_MODE' value cannot refer to a Lexer Mode ->"+e.PUSH_MODE+"<-which does not exist",type:s.LexerDefinitionErrorType.PUSH_MODE_DOES_NOT_EXIST,tokenTypes:[e]}}))}function w(e){var t=[],r=Object(i.reduce)(e,(function(e,t,r){var n,o=t.PATTERN;return o===s.Lexer.NA||(Object(i.isString)(o)?e.push({str:o,idx:r,tokenType:t}):Object(i.isRegExp)(o)&&(n=o,void 0===Object(i.find)([".","\\","[","]","|","^","$","(",")","?","*","+","{"],(function(e){return-1!==n.source.indexOf(e)})))&&e.push({str:o.source,idx:r,tokenType:t})),e}),[]);return Object(i.forEach)(e,(function(e,n){Object(i.forEach)(r,(function(r){var o=r.str,a=r.idx,u=r.tokenType;if(n<a&&function(e,t){if(Object(i.isRegExp)(t)){var r=t.exec(e);return null!==r&&0===r.index}if(Object(i.isFunction)(t))return t(e,0,[],{});if(Object(i.has)(t,"exec"))return t.exec(e,0,[],{});if("string"==typeof t)return t===e;throw Error("non exhaustive match")}(o,e.PATTERN)){var c="Token: ->"+u.name+"<- can never be matched.\nBecause it appears AFTER the Token Type ->"+e.name+"<-in the lexer's definition.\nSee https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#UNREACHABLE";t.push({message:c,type:s.LexerDefinitionErrorType.UNREACHABLE_PATTERN,tokenTypes:[e,u]})}}))})),t}function F(e){var t=e.ignoreCase?"i":"";return new RegExp("^(?:"+e.source+")",t)}function b(e){var t=e.ignoreCase?"iy":"y";return new RegExp(""+e.source,t)}function _(e,t,r){var n=[];return Object(i.has)(e,h)||n.push({message:"A MultiMode Lexer cannot be initialized without a <"+h+"> property in its definition\n",type:s.LexerDefinitionErrorType.MULTI_MODE_LEXER_WITHOUT_DEFAULT_MODE}),Object(i.has)(e,d)||n.push({message:"A MultiMode Lexer cannot be initialized without a <"+d+"> property in its definition\n",type:s.LexerDefinitionErrorType.MULTI_MODE_LEXER_WITHOUT_MODES_PROPERTY}),Object(i.has)(e,d)&&Object(i.has)(e,h)&&!Object(i.has)(e.modes,e.defaultMode)&&n.push({message:"A MultiMode Lexer cannot be initialized with a "+h+": <"+e.defaultMode+">which does not exist\n",type:s.LexerDefinitionErrorType.MULTI_MODE_LEXER_DEFAULT_MODE_VALUE_DOES_NOT_EXIST}),Object(i.has)(e,d)&&Object(i.forEach)(e.modes,(function(e,t){Object(i.forEach)(e,(function(e,r){Object(i.isUndefined)(e)&&n.push({message:"A Lexer cannot be initialized using an undefined Token Type. Mode:<"+t+"> at index: <"+r+">\n",type:s.LexerDefinitionErrorType.LEXER_DEFINITION_CANNOT_CONTAIN_UNDEFINED})}))})),n}function L(e,t,r){var n=[],o=!1,u=Object(i.compact)(Object(i.flatten)(Object(i.mapValues)(e.modes,(function(e){return e})))),c=Object(i.reject)(u,(function(e){return e[l]===s.Lexer.NA})),h=k(r);return t&&Object(i.forEach)(c,(function(e){var t=D(e,h);if(!1!==t){var r={message:B(e,t),type:t.issue,tokenType:e};n.push(r)}else Object(i.has)(e,"LINE_BREAKS")?!0===e.LINE_BREAKS&&(o=!0):Object(a.canMatchCharCode)(h,e.PATTERN)&&(o=!0)})),t&&!o&&n.push({message:"Warning: No LINE_BREAKS Found.\n\tThis Lexer has been defined to track line and column information,\n\tBut none of the Token Types can be identified as matching a line terminator.\n\tSee https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#LINE_BREAKS \n\tfor details.",type:s.LexerDefinitionErrorType.NO_LINE_BREAKS_FLAGS}),n}function x(e){var t={},r=Object(i.keys)(e);return Object(i.forEach)(r,(function(r){var n=e[r];if(!Object(i.isArray)(n))throw Error("non exhaustive match");t[r]=[]})),t}function P(e){var t=e.PATTERN;if(Object(i.isRegExp)(t))return!1;if(Object(i.isFunction)(t))return!0;if(Object(i.has)(t,"exec"))return!0;if(Object(i.isString)(t))return!1;throw Error("non exhaustive match")}function U(e){return!(!Object(i.isString)(e)||1!==e.length)&&e.charCodeAt(0)}var V={test:function(e){for(var t=e.length,r=this.lastIndex;r<t;r++){var n=e.charCodeAt(r);if(10===n)return this.lastIndex=r+1,!0;if(13===n)return 10===e.charCodeAt(r+1)?this.lastIndex=r+2:this.lastIndex=r+1,!0}return!1},lastIndex:0};function D(e,t){if(Object(i.has)(e,"LINE_BREAKS"))return!1;if(Object(i.isRegExp)(e.PATTERN)){try{Object(a.canMatchCharCode)(t,e.PATTERN)}catch(e){return{issue:s.LexerDefinitionErrorType.IDENTIFY_TERMINATOR,errMsg:e.message}}return!1}if(Object(i.isString)(e.PATTERN))return!1;if(P(e))return{issue:s.LexerDefinitionErrorType.CUSTOM_LINE_BREAK};throw Error("non exhaustive match")}function B(e,t){if(t.issue===s.LexerDefinitionErrorType.IDENTIFY_TERMINATOR)return"Warning: unable to identify line terminator usage in pattern.\n\tThe problem is in the <"+e.name+"> Token Type\n\t Root cause: "+t.errMsg+".\n\tFor details See: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#IDENTIFY_TERMINATOR";if(t.issue===s.LexerDefinitionErrorType.CUSTOM_LINE_BREAK)return"Warning: A Custom Token Pattern should specify the <line_breaks> option.\n\tThe problem is in the <"+e.name+"> Token Type\n\tFor details See: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#CUSTOM_LINE_BREAK";throw Error("non exhaustive match")}function k(e){return Object(i.map)(e,(function(e){return Object(i.isString)(e)&&e.length>0?e.charCodeAt(0):e}))}function G(e,t,r){void 0===e[t]?e[t]=[r]:e[t].push(r)}var j=256;function W(e){return e<j?e:H[e]}var H=[]},function(e,t,r){var n,o,s;"undefined"!=typeof self&&self,o=[],void 0===(s="function"==typeof(n=function(){function e(){}e.prototype.saveState=function(){return{idx:this.idx,input:this.input,groupIdx:this.groupIdx}},e.prototype.restoreState=function(e){this.idx=e.idx,this.input=e.input,this.groupIdx=e.groupIdx},e.prototype.pattern=function(e){this.idx=0,this.input=e,this.groupIdx=0,this.consumeChar("/");var t=this.disjunction();this.consumeChar("/");for(var r={type:"Flags",global:!1,ignoreCase:!1,multiLine:!1,unicode:!1,sticky:!1};this.isRegExpFlag();)switch(this.popChar()){case"g":a(r,"global");break;case"i":a(r,"ignoreCase");break;case"m":a(r,"multiLine");break;case"u":a(r,"unicode");break;case"y":a(r,"sticky")}if(this.idx!==this.input.length)throw Error("Redundant input: "+this.input.substring(this.idx));return{type:"Pattern",flags:r,value:t}},e.prototype.disjunction=function(){var e=[];for(e.push(this.alternative());"|"===this.peekChar();)this.consumeChar("|"),e.push(this.alternative());return{type:"Disjunction",value:e}},e.prototype.alternative=function(){for(var e=[];this.isTerm();)e.push(this.term());return{type:"Alternative",value:e}},e.prototype.term=function(){return this.isAssertion()?this.assertion():this.atom()},e.prototype.assertion=function(){switch(this.popChar()){case"^":return{type:"StartAnchor"};case"$":return{type:"EndAnchor"};case"\\":switch(this.popChar()){case"b":return{type:"WordBoundary"};case"B":return{type:"NonWordBoundary"}}throw Error("Invalid Assertion Escape");case"(":var e;switch(this.consumeChar("?"),this.popChar()){case"=":e="Lookahead";break;case"!":e="NegativeLookahead"}u(e);var t=this.disjunction();return this.consumeChar(")"),{type:e,value:t}}!function(){throw Error("Internal Error - Should never get here!")}()},e.prototype.quantifier=function(e){var t;switch(this.popChar()){case"*":t={atLeast:0,atMost:1/0};break;case"+":t={atLeast:1,atMost:1/0};break;case"?":t={atLeast:0,atMost:1};break;case"{":var r=this.integerIncludingZero();switch(this.popChar()){case"}":t={atLeast:r,atMost:r};break;case",":t=this.isDigit()?{atLeast:r,atMost:this.integerIncludingZero()}:{atLeast:r,atMost:1/0},this.consumeChar("}")}if(!0===e&&void 0===t)return;u(t)}if(!0!==e||void 0!==t)return u(t),"?"===this.peekChar(0)?(this.consumeChar("?"),t.greedy=!1):t.greedy=!0,t.type="Quantifier",t},e.prototype.atom=function(){var e;switch(this.peekChar()){case".":e=this.dotAll();break;case"\\":e=this.atomEscape();break;case"[":e=this.characterClass();break;case"(":e=this.group()}return void 0===e&&this.isPatternCharacter()&&(e=this.patternCharacter()),u(e),this.isQuantifier()&&(e.quantifier=this.quantifier()),e},e.prototype.dotAll=function(){return this.consumeChar("."),{type:"Set",complement:!0,value:[s("\n"),s("\r"),s("\u2028"),s("\u2029")]}},e.prototype.atomEscape=function(){switch(this.consumeChar("\\"),this.peekChar()){case"1":case"2":case"3":case"4":case"5":case"6":case"7":case"8":case"9":return this.decimalEscapeAtom();case"d":case"D":case"s":case"S":case"w":case"W":return this.characterClassEscape();case"f":case"n":case"r":case"t":case"v":return this.controlEscapeAtom();case"c":return this.controlLetterEscapeAtom();case"0":return this.nulCharacterAtom();case"x":return this.hexEscapeSequenceAtom();case"u":return this.regExpUnicodeEscapeSequenceAtom();default:return this.identityEscapeAtom()}},e.prototype.decimalEscapeAtom=function(){return{type:"GroupBackReference",value:this.positiveInteger()}},e.prototype.characterClassEscape=function(){var e,t=!1;switch(this.popChar()){case"d":e=c;break;case"D":e=c,t=!0;break;case"s":e=h;break;case"S":e=h,t=!0;break;case"w":e=l;break;case"W":e=l,t=!0}return u(e),{type:"Set",value:e,complement:t}},e.prototype.controlEscapeAtom=function(){var e;switch(this.popChar()){case"f":e=s("\f");break;case"n":e=s("\n");break;case"r":e=s("\r");break;case"t":e=s("\t");break;case"v":e=s("\v")}return u(e),{type:"Character",value:e}},e.prototype.controlLetterEscapeAtom=function(){this.consumeChar("c");var e=this.popChar();if(!1===/[a-zA-Z]/.test(e))throw Error("Invalid ");return{type:"Character",value:e.toUpperCase().charCodeAt(0)-64}},e.prototype.nulCharacterAtom=function(){return this.consumeChar("0"),{type:"Character",value:s("\0")}},e.prototype.hexEscapeSequenceAtom=function(){return this.consumeChar("x"),this.parseHexDigits(2)},e.prototype.regExpUnicodeEscapeSequenceAtom=function(){return this.consumeChar("u"),this.parseHexDigits(4)},e.prototype.identityEscapeAtom=function(){return{type:"Character",value:s(this.popChar())}},e.prototype.classPatternCharacterAtom=function(){switch(this.peekChar()){case"\n":case"\r":case"\u2028":case"\u2029":case"\\":case"]":throw Error("TBD");default:return{type:"Character",value:s(this.popChar())}}},e.prototype.characterClass=function(){var e=[],t=!1;for(this.consumeChar("["),"^"===this.peekChar(0)&&(this.consumeChar("^"),t=!0);this.isClassAtom();){var r=this.classAtom();if("Character"===r.type&&this.isRangeDash()){this.consumeChar("-");var n=this.classAtom();if("Character"===n.type){if(n.value<r.value)throw Error("Range out of order in character class");e.push({from:r.value,to:n.value})}else i(r.value,e),e.push(s("-")),i(n.value,e)}else i(r.value,e)}return this.consumeChar("]"),{type:"Set",complement:t,value:e}},e.prototype.classAtom=function(){switch(this.peekChar()){case"]":case"\n":case"\r":case"\u2028":case"\u2029":throw Error("TBD");case"\\":return this.classEscape();default:return this.classPatternCharacterAtom()}},e.prototype.classEscape=function(){switch(this.consumeChar("\\"),this.peekChar()){case"b":return this.consumeChar("b"),{type:"Character",value:s("\b")};case"d":case"D":case"s":case"S":case"w":case"W":return this.characterClassEscape();case"f":case"n":case"r":case"t":case"v":return this.controlEscapeAtom();case"c":return this.controlLetterEscapeAtom();case"0":return this.nulCharacterAtom();case"x":return this.hexEscapeSequenceAtom();case"u":return this.regExpUnicodeEscapeSequenceAtom();default:return this.identityEscapeAtom()}},e.prototype.group=function(){var e=!0;switch(this.consumeChar("("),this.peekChar(0)){case"?":this.consumeChar("?"),this.consumeChar(":"),e=!1;break;default:this.groupIdx++}var t=this.disjunction();this.consumeChar(")");var r={type:"Group",capturing:e,value:t};return e&&(r.idx=this.groupIdx),r},e.prototype.positiveInteger=function(){var e=this.popChar();if(!1===o.test(e))throw Error("Expecting a positive integer");for(;n.test(this.peekChar(0));)e+=this.popChar();return parseInt(e,10)},e.prototype.integerIncludingZero=function(){var e=this.popChar();if(!1===n.test(e))throw Error("Expecting an integer");for(;n.test(this.peekChar(0));)e+=this.popChar();return parseInt(e,10)},e.prototype.patternCharacter=function(){var e=this.popChar();switch(e){case"\n":case"\r":case"\u2028":case"\u2029":case"^":case"$":case"\\":case".":case"*":case"+":case"?":case"(":case")":case"[":case"|":throw Error("TBD");default:return{type:"Character",value:s(e)}}},e.prototype.isRegExpFlag=function(){switch(this.peekChar(0)){case"g":case"i":case"m":case"u":case"y":return!0;default:return!1}},e.prototype.isRangeDash=function(){return"-"===this.peekChar()&&this.isClassAtom(1)},e.prototype.isDigit=function(){return n.test(this.peekChar(0))},e.prototype.isClassAtom=function(e){switch(void 0===e&&(e=0),this.peekChar(e)){case"]":case"\n":case"\r":case"\u2028":case"\u2029":return!1;default:return!0}},e.prototype.isTerm=function(){return this.isAtom()||this.isAssertion()},e.prototype.isAtom=function(){if(this.isPatternCharacter())return!0;switch(this.peekChar(0)){case".":case"\\":case"[":case"(":return!0;default:return!1}},e.prototype.isAssertion=function(){switch(this.peekChar(0)){case"^":case"$":return!0;case"\\":switch(this.peekChar(1)){case"b":case"B":return!0;default:return!1}case"(":return"?"===this.peekChar(1)&&("="===this.peekChar(2)||"!"===this.peekChar(2));default:return!1}},e.prototype.isQuantifier=function(){var e=this.saveState();try{return void 0!==this.quantifier(!0)}catch(e){return!1}finally{this.restoreState(e)}},e.prototype.isPatternCharacter=function(){switch(this.peekChar()){case"^":case"$":case"\\":case".":case"*":case"+":case"?":case"(":case")":case"[":case"|":case"/":case"\n":case"\r":case"\u2028":case"\u2029":return!1;default:return!0}},e.prototype.parseHexDigits=function(e){for(var t="",n=0;n<e;n++){var o=this.popChar();if(!1===r.test(o))throw Error("Expecting a HexDecimal digits");t+=o}return{type:"Character",value:parseInt(t,16)}},e.prototype.peekChar=function(e){return void 0===e&&(e=0),this.input[this.idx+e]},e.prototype.popChar=function(){var e=this.peekChar(0);return this.consumeChar(),e},e.prototype.consumeChar=function(e){if(void 0!==e&&this.input[this.idx]!==e)throw Error("Expected: '"+e+"' but found: '"+this.input[this.idx]+"' at offset: "+this.idx);if(this.idx>=this.input.length)throw Error("Unexpected end of input");this.idx++};var t,r=/[0-9a-fA-F]/,n=/[0-9]/,o=/[1-9]/;function s(e){return e.charCodeAt(0)}function i(e,t){void 0!==e.length?e.forEach((function(e){t.push(e)})):t.push(e)}function a(e,t){if(!0===e[t])throw"duplicate flag "+t;e[t]=!0}function u(e){if(void 0===e)throw Error("Internal Error - Should never get here!")}var c=[];for(t=s("0");t<=s("9");t++)c.push(t);var l=[s("_")].concat(c);for(t=s("a");t<=s("z");t++)l.push(t);for(t=s("A");t<=s("Z");t++)l.push(t);var h=[s(" "),s("\f"),s("\n"),s("\r"),s("\t"),s("\v"),s("\t"),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s(" "),s("\u2028"),s("\u2029"),s(" "),s(" "),s("　"),s("\ufeff")];function d(){}return d.prototype.visitChildren=function(e){for(var t in e){var r=e[t];e.hasOwnProperty(t)&&(void 0!==r.type?this.visit(r):Array.isArray(r)&&r.forEach((function(e){this.visit(e)}),this))}},d.prototype.visit=function(e){switch(e.type){case"Pattern":this.visitPattern(e);break;case"Flags":this.visitFlags(e);break;case"Disjunction":this.visitDisjunction(e);break;case"Alternative":this.visitAlternative(e);break;case"StartAnchor":this.visitStartAnchor(e);break;case"EndAnchor":this.visitEndAnchor(e);break;case"WordBoundary":this.visitWordBoundary(e);break;case"NonWordBoundary":this.visitNonWordBoundary(e);break;case"Lookahead":this.visitLookahead(e);break;case"NegativeLookahead":this.visitNegativeLookahead(e);break;case"Character":this.visitCharacter(e);break;case"Set":this.visitSet(e);break;case"Group":this.visitGroup(e);break;case"GroupBackReference":this.visitGroupBackReference(e);break;case"Quantifier":this.visitQuantifier(e)}this.visitChildren(e)},d.prototype.visitPattern=function(e){},d.prototype.visitFlags=function(e){},d.prototype.visitDisjunction=function(e){},d.prototype.visitAlternative=function(e){},d.prototype.visitStartAnchor=function(e){},d.prototype.visitEndAnchor=function(e){},d.prototype.visitWordBoundary=function(e){},d.prototype.visitNonWordBoundary=function(e){},d.prototype.visitLookahead=function(e){},d.prototype.visitNegativeLookahead=function(e){},d.prototype.visitCharacter=function(e){},d.prototype.visitSet=function(e){},d.prototype.visitGroup=function(e){},d.prototype.visitGroupBackReference=function(e){},d.prototype.visitQuantifier=function(e){},{RegExpParser:e,BaseRegExpVisitor:d,VERSION:"0.4.0"}})?n.apply(t,o):n)||(e.exports=s)},function(e,t,r){"use strict";r.r(t),r.d(t,"failedOptimizationPrefixMsg",(function(){return c})),r.d(t,"getOptimizedStartCodesIndices",(function(){return l})),r.d(t,"firstCharOptimizedIndices",(function(){return h})),r.d(t,"canMatchCharCode",(function(){return f}));var n,o=r(95),s=r(88),i=r(97),a=r(94),u=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}),c='Unable to use "first char" lexer optimizations:\n';function l(e,t){void 0===t&&(t=!1);try{var r=Object(i.getRegExpAst)(e);return h(r.value,{},r.flags.ignoreCase)}catch(r){if("Complement Sets are not supported for first char optimization"===r.message)t&&Object(s.PRINT_WARNING)(c+"\tUnable to optimize: < "+e.toString()+" >\n\tComplement Sets cannot be automatically optimized.\n\tThis will disable the lexer's first char optimizations.\n\tSee: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#COMPLEMENT for details.");else{var n="";t&&(n="\n\tThis will disable the lexer's first char optimizations.\n\tSee: https://sap.github.io/chevrotain/docs/guide/resolving_lexer_errors.html#REGEXP_PARSING for details."),Object(s.PRINT_ERROR)(c+"\n\tFailed parsing: < "+e.toString()+" >\n\tUsing the regexp-to-ast library version: "+o.VERSION+"\n\tPlease open an issue at: https://github.com/bd82/regexp-to-ast/issues"+n)}}return[]}function h(e,t,r){switch(e.type){case"Disjunction":for(var n=0;n<e.value.length;n++)h(e.value[n],t,r);break;case"Alternative":var o=e.value;for(n=0;n<o.length;n++){var i=o[n];switch(i.type){case"EndAnchor":case"GroupBackReference":case"Lookahead":case"NegativeLookahead":case"StartAnchor":case"WordBoundary":case"NonWordBoundary":continue}var u=i;switch(u.type){case"Character":d(u.value,t,r);break;case"Set":if(!0===u.complement)throw Error("Complement Sets are not supported for first char optimization");Object(s.forEach)(u.value,(function(e){if("number"==typeof e)d(e,t,r);else{var n=e;if(!0===r)for(var o=n.from;o<=n.to;o++)d(o,t,r);else{for(o=n.from;o<=n.to&&o<a.minOptimizationVal;o++)d(o,t,r);if(n.to>=a.minOptimizationVal)for(var s=n.from>=a.minOptimizationVal?n.from:a.minOptimizationVal,i=n.to,u=Object(a.charCodeToOptimizedIndex)(s),c=Object(a.charCodeToOptimizedIndex)(i),l=u;l<=c;l++)t[l]=l}}}));break;case"Group":h(u.value,t,r);break;default:throw Error("Non Exhaustive Match")}var c=void 0!==u.quantifier&&0===u.quantifier.atLeast;if("Group"===u.type&&!1===m(u)||"Group"!==u.type&&!1===c)break}break;default:throw Error("non exhaustive match!")}return Object(s.values)(t)}function d(e,t,r){var n=Object(a.charCodeToOptimizedIndex)(e);t[n]=n,!0===r&&function(e,t){var r=String.fromCharCode(e),n=r.toUpperCase();if(n!==r){var o=Object(a.charCodeToOptimizedIndex)(n.charCodeAt(0));t[o]=o}else{var s=r.toLowerCase();if(s!==r){o=Object(a.charCodeToOptimizedIndex)(s.charCodeAt(0));t[o]=o}}}(e,t)}function p(e,t){return Object(s.find)(e.value,(function(e){if("number"==typeof e)return Object(s.contains)(t,e);var r=e;return void 0!==Object(s.find)(t,(function(e){return r.from<=e&&e<=r.to}))}))}function m(e){return!(!e.quantifier||0!==e.quantifier.atLeast)||!!e.value&&(Object(s.isArray)(e.value)?Object(s.every)(e.value,m):m(e.value))}var g=function(e){function t(t){var r=e.call(this)||this;return r.targetCharCodes=t,r.found=!1,r}return u(t,e),t.prototype.visitChildren=function(t){if(!0!==this.found){switch(t.type){case"Lookahead":return void this.visitLookahead(t);case"NegativeLookahead":return void this.visitNegativeLookahead(t)}e.prototype.visitChildren.call(this,t)}},t.prototype.visitCharacter=function(e){Object(s.contains)(this.targetCharCodes,e.value)&&(this.found=!0)},t.prototype.visitSet=function(e){e.complement?void 0===p(e,this.targetCharCodes)&&(this.found=!0):void 0!==p(e,this.targetCharCodes)&&(this.found=!0)},t}(o.BaseRegExpVisitor);function f(e,t){if(t instanceof RegExp){var r=Object(i.getRegExpAst)(t),n=new g(e);return n.visit(r),n.found}return void 0!==Object(s.find)(t,(function(t){return Object(s.contains)(e,t.charCodeAt(0))}))}},function(e,t,r){"use strict";r.r(t),r.d(t,"getRegExpAst",(function(){return i})),r.d(t,"clearRegExpParserCache",(function(){return a}));var n=r(95),o={},s=new n.RegExpParser;function i(e){var t=e.toString();if(o.hasOwnProperty(t))return o[t];var r=s.pattern(t);return o[t]=r,r}function a(){o={}}},function(e,t,r){"use strict";r.r(t),r.d(t,"tokenStructuredMatcher",(function(){return o})),r.d(t,"tokenStructuredMatcherNoCategories",(function(){return s})),r.d(t,"tokenShortNameIdx",(function(){return i})),r.d(t,"tokenIdxToClass",(function(){return a})),r.d(t,"augmentTokenTypes",(function(){return u})),r.d(t,"expandCategories",(function(){return c})),r.d(t,"assignTokenDefaultProps",(function(){return l})),r.d(t,"assignCategoriesTokensProp",(function(){return h})),r.d(t,"assignCategoriesMapProp",(function(){return d})),r.d(t,"singleAssignCategoriesToksMap",(function(){return p})),r.d(t,"hasShortKeyProperty",(function(){return m})),r.d(t,"hasCategoriesProperty",(function(){return g})),r.d(t,"hasExtendingTokensTypesProperty",(function(){return f})),r.d(t,"hasExtendingTokensTypesMapProperty",(function(){return E})),r.d(t,"isTokenType",(function(){return T}));var n=r(88);function o(e,t){var r=e.tokenTypeIdx;return r===t.tokenTypeIdx||!0===t.isParent&&!0===t.categoryMatchesMap[r]}function s(e,t){return e.tokenTypeIdx===t.tokenTypeIdx}var i=1,a={};function u(e){var t=c(e);l(t),d(t),h(t),Object(n.forEach)(t,(function(e){e.isParent=e.categoryMatches.length>0}))}function c(e){for(var t=Object(n.cloneArr)(e),r=e,o=!0;o;){r=Object(n.compact)(Object(n.flatten)(Object(n.map)(r,(function(e){return e.CATEGORIES}))));var s=Object(n.difference)(r,t);t=t.concat(s),Object(n.isEmpty)(s)?o=!1:r=s}return t}function l(e){Object(n.forEach)(e,(function(e){m(e)||(a[i]=e,e.tokenTypeIdx=i++),g(e)&&!Object(n.isArray)(e.CATEGORIES)&&(e.CATEGORIES=[e.CATEGORIES]),g(e)||(e.CATEGORIES=[]),f(e)||(e.categoryMatches=[]),E(e)||(e.categoryMatchesMap={})}))}function h(e){Object(n.forEach)(e,(function(e){e.categoryMatches=[],Object(n.forEach)(e.categoryMatchesMap,(function(t,r){e.categoryMatches.push(a[r].tokenTypeIdx)}))}))}function d(e){Object(n.forEach)(e,(function(e){p([],e)}))}function p(e,t){Object(n.forEach)(e,(function(e){t.categoryMatchesMap[e.tokenTypeIdx]=!0})),Object(n.forEach)(t.CATEGORIES,(function(r){var o=e.concat(t);Object(n.contains)(o,r)||p(o,r)}))}function m(e){return Object(n.has)(e,"tokenTypeIdx")}function g(e){return Object(n.has)(e,"CATEGORIES")}function f(e){return Object(n.has)(e,"categoryMatches")}function E(e){return Object(n.has)(e,"categoryMatchesMap")}function T(e){return Object(n.has)(e,"tokenTypeIdx")}},function(e,t,r){"use strict";r.r(t),r.d(t,"defaultLexerErrorProvider",(function(){return n}));var n={buildUnableToPopLexerModeMessage:function(e){return"Unable to pop Lexer Mode after encountering Token ->"+e.image+"<- The Mode Stack is empty"},buildUnexpectedCharactersMessage:function(e,t,r,n,o){return"unexpected character: ->"+e.charAt(t)+"<- at offset: "+t+", skipped "+r+" characters."}}},function(e,t,r){"use strict";r.r(t),r.d(t,"first",(function(){return i})),r.d(t,"firstForSequence",(function(){return a})),r.d(t,"firstForBranching",(function(){return u})),r.d(t,"firstForTerminal",(function(){return c}));var n=r(88),o=r(91),s=r(101);function i(e){if(e instanceof o.NonTerminal)return i(e.referencedRule);if(e instanceof o.Terminal)return c(e);if(Object(s.isSequenceProd)(e))return a(e);if(Object(s.isBranchingProd)(e))return u(e);throw Error("non exhaustive match")}function a(e){for(var t,r=[],o=e.definition,a=0,u=o.length>a,c=!0;u&&c;)t=o[a],c=Object(s.isOptionalProd)(t),r=r.concat(i(t)),a+=1,u=o.length>a;return Object(n.uniq)(r)}function u(e){var t=Object(n.map)(e.definition,(function(e){return i(e)}));return Object(n.uniq)(Object(n.flatten)(t))}function c(e){return[e.terminalType]}},function(e,t,r){"use strict";r.r(t),r.d(t,"isSequenceProd",(function(){return u})),r.d(t,"isOptionalProd",(function(){return c})),r.d(t,"isBranchingProd",(function(){return l})),r.d(t,"getProductionDslName",(function(){return h})),r.d(t,"DslMethodsCollectorVisitor",(function(){return d})),r.d(t,"collectMethods",(function(){return m}));var n,o=r(88),s=r(91),i=r(102),a=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)});function u(e){return e instanceof s.Flat||e instanceof s.Option||e instanceof s.Repetition||e instanceof s.RepetitionMandatory||e instanceof s.RepetitionMandatoryWithSeparator||e instanceof s.RepetitionWithSeparator||e instanceof s.Terminal||e instanceof s.Rule}function c(e,t){return void 0===t&&(t=[]),!!(e instanceof s.Option||e instanceof s.Repetition||e instanceof s.RepetitionWithSeparator)||(e instanceof s.Alternation?Object(o.some)(e.definition,(function(e){return c(e,t)})):!(e instanceof s.NonTerminal&&Object(o.contains)(t,e))&&(e instanceof s.AbstractProduction&&(e instanceof s.NonTerminal&&t.push(e),Object(o.every)(e.definition,(function(e){return c(e,t)})))))}function l(e){return e instanceof s.Alternation}function h(e){if(e instanceof s.NonTerminal)return"SUBRULE";if(e instanceof s.Option)return"OPTION";if(e instanceof s.Alternation)return"OR";if(e instanceof s.RepetitionMandatory)return"AT_LEAST_ONE";if(e instanceof s.RepetitionMandatoryWithSeparator)return"AT_LEAST_ONE_SEP";if(e instanceof s.RepetitionWithSeparator)return"MANY_SEP";if(e instanceof s.Repetition)return"MANY";if(e instanceof s.Terminal)return"CONSUME";throw Error("non exhaustive match")}var d=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.separator="-",t.dslMethods={option:[],alternation:[],repetition:[],repetitionWithSeparator:[],repetitionMandatory:[],repetitionMandatoryWithSeparator:[]},t}return a(t,e),t.prototype.reset=function(){this.dslMethods={option:[],alternation:[],repetition:[],repetitionWithSeparator:[],repetitionMandatory:[],repetitionMandatoryWithSeparator:[]}},t.prototype.visitTerminal=function(e){var t=e.terminalType.name+this.separator+"Terminal";Object(o.has)(this.dslMethods,t)||(this.dslMethods[t]=[]),this.dslMethods[t].push(e)},t.prototype.visitNonTerminal=function(e){var t=e.nonTerminalName+this.separator+"Terminal";Object(o.has)(this.dslMethods,t)||(this.dslMethods[t]=[]),this.dslMethods[t].push(e)},t.prototype.visitOption=function(e){this.dslMethods.option.push(e)},t.prototype.visitRepetitionWithSeparator=function(e){this.dslMethods.repetitionWithSeparator.push(e)},t.prototype.visitRepetitionMandatory=function(e){this.dslMethods.repetitionMandatory.push(e)},t.prototype.visitRepetitionMandatoryWithSeparator=function(e){this.dslMethods.repetitionMandatoryWithSeparator.push(e)},t.prototype.visitRepetition=function(e){this.dslMethods.repetition.push(e)},t.prototype.visitAlternation=function(e){this.dslMethods.alternation.push(e)},t}(i.GAstVisitor),p=new d;function m(e){p.reset(),e.accept(p);var t=p.dslMethods;return p.reset(),t}},function(e,t,r){"use strict";r.r(t),r.d(t,"GAstVisitor",(function(){return o}));var n=r(91),o=function(){function e(){}return e.prototype.visit=function(e){var t=e;switch(t.constructor){case n.NonTerminal:return this.visitNonTerminal(t);case n.Flat:return this.visitFlat(t);case n.Option:return this.visitOption(t);case n.RepetitionMandatory:return this.visitRepetitionMandatory(t);case n.RepetitionMandatoryWithSeparator:return this.visitRepetitionMandatoryWithSeparator(t);case n.RepetitionWithSeparator:return this.visitRepetitionWithSeparator(t);case n.Repetition:return this.visitRepetition(t);case n.Alternation:return this.visitAlternation(t);case n.Terminal:return this.visitTerminal(t);case n.Rule:return this.visitRule(t);default:throw Error("non exhaustive match")}},e.prototype.visitNonTerminal=function(e){},e.prototype.visitFlat=function(e){},e.prototype.visitOption=function(e){},e.prototype.visitRepetition=function(e){},e.prototype.visitRepetitionMandatory=function(e){},e.prototype.visitRepetitionMandatoryWithSeparator=function(e){},e.prototype.visitRepetitionWithSeparator=function(e){},e.prototype.visitAlternation=function(e){},e.prototype.visitTerminal=function(e){},e.prototype.visitRule=function(e){},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"IN",(function(){return n}));var n="_~IN~_"},function(e,t,r){"use strict";r.r(t),r.d(t,"setNodeLocationOnlyOffset",(function(){return c})),r.d(t,"setNodeLocationFull",(function(){return l})),r.d(t,"addTerminalToCst",(function(){return h})),r.d(t,"addNoneTerminalToCst",(function(){return d})),r.d(t,"NamedDSLMethodsCollectorVisitor",(function(){return p})),r.d(t,"expandAllNestedRuleNames",(function(){return m}));var n,o=r(88),s=r(105),i=r(91),a=r(102),u=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)});function c(e,t){!0===isNaN(e.startOffset)?(e.startOffset=t.startOffset,e.endOffset=t.endOffset):e.endOffset<t.endOffset==!0&&(e.endOffset=t.endOffset)}function l(e,t){!0===isNaN(e.startOffset)?(e.startOffset=t.startOffset,e.startColumn=t.startColumn,e.startLine=t.startLine,e.endOffset=t.endOffset,e.endColumn=t.endColumn,e.endLine=t.endLine):e.endOffset<t.endOffset==!0&&(e.endOffset=t.endOffset,e.endColumn=t.endColumn,e.endLine=t.endLine)}function h(e,t,r){void 0===e.children[r]?e.children[r]=[t]:e.children[r].push(t)}function d(e,t,r){void 0===e.children[t]?e.children[t]=[r]:e.children[t].push(r)}var p=function(e){function t(t){var r=e.call(this)||this;return r.result=[],r.ruleIdx=t,r}return u(t,e),t.prototype.collectNamedDSLMethod=function(e,t,r){if(!Object(o.isUndefined)(e.name)){var n=void 0;if(e instanceof i.Option||e instanceof i.Repetition||e instanceof i.RepetitionMandatory||e instanceof i.Alternation)n=new t({definition:e.definition,idx:e.idx});else{if(!(e instanceof i.RepetitionMandatoryWithSeparator||e instanceof i.RepetitionWithSeparator))throw Error("non exhaustive match");n=new t({definition:e.definition,idx:e.idx,separator:e.separator})}var a=[n],u=Object(s.getKeyForAutomaticLookahead)(this.ruleIdx,r,e.idx);this.result.push({def:a,key:u,name:e.name,orgProd:e})}},t.prototype.visitOption=function(e){this.collectNamedDSLMethod(e,i.Option,s.OPTION_IDX)},t.prototype.visitRepetition=function(e){this.collectNamedDSLMethod(e,i.Repetition,s.MANY_IDX)},t.prototype.visitRepetitionMandatory=function(e){this.collectNamedDSLMethod(e,i.RepetitionMandatory,s.AT_LEAST_ONE_IDX)},t.prototype.visitRepetitionMandatoryWithSeparator=function(e){this.collectNamedDSLMethod(e,i.RepetitionMandatoryWithSeparator,s.AT_LEAST_ONE_SEP_IDX)},t.prototype.visitRepetitionWithSeparator=function(e){this.collectNamedDSLMethod(e,i.RepetitionWithSeparator,s.MANY_SEP_IDX)},t.prototype.visitAlternation=function(e){var t=this;this.collectNamedDSLMethod(e,i.Alternation,s.OR_IDX);var r=e.definition.length>1;Object(o.forEach)(e.definition,(function(n,a){if(!Object(o.isUndefined)(n.name)){var u=n.definition;u=r?[new i.Option({definition:n.definition})]:n.definition;var c=Object(s.getKeyForAltIndex)(t.ruleIdx,s.OR_IDX,e.idx,a);t.result.push({def:u,key:c,name:n.name,orgProd:n})}}))},t}(a.GAstVisitor);function m(e,t){var r={allRuleNames:[]};return Object(o.forEach)(e,(function(e){var n=t[e.name];r.allRuleNames.push(e.name);var s=new p(n);e.accept(s),Object(o.forEach)(s.result,(function(t){t.def,t.key;var n=t.name;r.allRuleNames.push(e.name+n)}))})),r}},function(e,t,r){"use strict";r.r(t),r.d(t,"BITS_FOR_METHOD_TYPE",(function(){return n})),r.d(t,"BITS_FOR_OCCURRENCE_IDX",(function(){return o})),r.d(t,"BITS_FOR_RULE_IDX",(function(){return s})),r.d(t,"BITS_FOR_ALT_IDX",(function(){return i})),r.d(t,"OR_IDX",(function(){return a})),r.d(t,"OPTION_IDX",(function(){return u})),r.d(t,"MANY_IDX",(function(){return c})),r.d(t,"AT_LEAST_ONE_IDX",(function(){return l})),r.d(t,"MANY_SEP_IDX",(function(){return h})),r.d(t,"AT_LEAST_ONE_SEP_IDX",(function(){return d})),r.d(t,"getKeyForAutomaticLookahead",(function(){return p})),r.d(t,"getKeyForAltIndex",(function(){return g}));var n=4,o=8,s=12,i=8,a=1<<o,u=2<<o,c=3<<o,l=4<<o,h=5<<o,d=6<<o;function p(e,t,r){return r|t|e}var m=32-i;function g(e,t,r,n){var o=n+1<<m;return p(e,t,r)|o}},function(e,t,r){"use strict";r.r(t),r.d(t,"defaultParserErrorProvider",(function(){return u})),r.d(t,"defaultGrammarResolverErrorProvider",(function(){return c})),r.d(t,"defaultGrammarValidatorErrorProvider",(function(){return l}));var n=r(92),o=r(88),s=r(91),i=r(101),a=r(107),u={buildMismatchTokenMessage:function(e){var t=e.expected,r=e.actual;e.previous,e.ruleName;return"Expecting "+(Object(n.hasTokenLabel)(t)?"--\x3e "+Object(n.tokenLabel)(t)+" <--":"token of type --\x3e "+t.name+" <--")+" but found --\x3e '"+r.image+"' <--"},buildNotAllInputParsedMessage:function(e){var t=e.firstRedundant;e.ruleName;return"Redundant input, expecting EOF but found: "+t.image},buildNoViableAltMessage:function(e){var t=e.expectedPathsPerAlt,r=e.actual,s=(e.previous,e.customUserDescription),i=(e.ruleName,"\nbut found: '"+Object(o.first)(r).image+"'");if(s)return"Expecting: "+s+i;var a=Object(o.reduce)(t,(function(e,t){return e.concat(t)}),[]),u=Object(o.map)(a,(function(e){return"["+Object(o.map)(e,(function(e){return Object(n.tokenLabel)(e)})).join(", ")+"]"}));return"Expecting: "+("one of these possible Token sequences:\n"+Object(o.map)(u,(function(e,t){return"  "+(t+1)+". "+e})).join("\n"))+i},buildEarlyExitMessage:function(e){var t=e.expectedIterationPaths,r=e.actual,s=e.customUserDescription,i=(e.ruleName,"\nbut found: '"+Object(o.first)(r).image+"'");return s?"Expecting: "+s+i:"Expecting: "+("expecting at least one iteration which starts with one of these possible Token sequences::\n  <"+Object(o.map)(t,(function(e){return"["+Object(o.map)(e,(function(e){return Object(n.tokenLabel)(e)})).join(",")+"]"})).join(" ,")+">")+i}};Object.freeze(u);var c={buildRuleNotFoundError:function(e,t){return"Invalid grammar, reference to a rule which is not defined: ->"+t.nonTerminalName+"<-\ninside top level rule: ->"+e.name+"<-"}},l={buildDuplicateFoundError:function(e,t){var r,n=e.name,a=Object(o.first)(t),u=a.idx,c=Object(i.getProductionDslName)(a),l=(r=a)instanceof s.Terminal?r.terminalType.name:r instanceof s.NonTerminal?r.nonTerminalName:"",h="->"+c+(u>0?u:"")+"<- "+(l?"with argument: ->"+l+"<-":"")+"\n                  appears more than once ("+t.length+" times) in the top level rule: ->"+n+"<-.                  \n                  For further details see: https://sap.github.io/chevrotain/docs/FAQ.html#NUMERICAL_SUFFIXES \n                  ";return h=(h=h.replace(/[ \t]+/g," ")).replace(/\s\s+/g,"\n")},buildInvalidNestedRuleNameError:function(e,t){return"Invalid nested rule name: ->"+t.name+"<- inside rule: ->"+e.name+"<-\nit must match the pattern: ->"+a.validNestedRuleName.toString()+"<-.\nNote that this means a nested rule name must start with the '$'(dollar) sign."},buildDuplicateNestedRuleNameError:function(e,t){return"Duplicate nested rule name: ->"+Object(o.first)(t).name+"<- inside rule: ->"+e.name+"<-\nA nested name must be unique in the scope of a top level grammar rule."},buildNamespaceConflictError:function(e){return"Namespace conflict found in grammar.\nThe grammar has both a Terminal(Token) and a Non-Terminal(Rule) named: <"+e.name+">.\nTo resolve this make sure each Terminal and Non-Terminal names are unique\nThis is easy to accomplish by using the convention that Terminal names start with an uppercase letter\nand Non-Terminal names start with a lower case letter."},buildAlternationPrefixAmbiguityError:function(e){var t=Object(o.map)(e.prefixPath,(function(e){return Object(n.tokenLabel)(e)})).join(", "),r=0===e.alternation.idx?"":e.alternation.idx;return"Ambiguous alternatives: <"+e.ambiguityIndices.join(" ,")+"> due to common lookahead prefix\nin <OR"+r+"> inside <"+e.topLevelRule.name+"> Rule,\n<"+t+"> may appears as a prefix path in all these alternatives.\nSee: https://sap.github.io/chevrotain/docs/guide/resolving_grammar_errors.html#COMMON_PREFIX\nFor Further details."},buildAlternationAmbiguityError:function(e){var t=Object(o.map)(e.prefixPath,(function(e){return Object(n.tokenLabel)(e)})).join(", "),r=0===e.alternation.idx?"":e.alternation.idx,s="Ambiguous Alternatives Detected: <"+e.ambiguityIndices.join(" ,")+"> in <OR"+r+"> inside <"+e.topLevelRule.name+"> Rule,\n<"+t+"> may appears as a prefix path in all these alternatives.\n";return s+="See: https://sap.github.io/chevrotain/docs/guide/resolving_grammar_errors.html#AMBIGUOUS_ALTERNATIVES\nFor Further details."},buildEmptyRepetitionError:function(e){var t=Object(i.getProductionDslName)(e.repetition);return 0!==e.repetition.idx&&(t+=e.repetition.idx),"The repetition <"+t+"> within Rule <"+e.topLevelRule.name+"> can never consume any tokens.\nThis could lead to an infinite loop."},buildTokenNameError:function(e){return"Invalid Grammar Token name: ->"+e.tokenType.name+"<- it must match the pattern: ->"+e.expectedPattern.toString()+"<-"},buildEmptyAlternationError:function(e){return"Ambiguous empty alternative: <"+(e.emptyChoiceIdx+1)+"> in <OR"+e.alternation.idx+"> inside <"+e.topLevelRule.name+"> Rule.\nOnly the last alternative may be an empty alternative."},buildTooManyAlternativesError:function(e){return"An Alternation cannot have more than 256 alternatives:\n<OR"+e.alternation.idx+"> inside <"+e.topLevelRule.name+"> Rule.\n has "+(e.alternation.definition.length+1)+" alternatives."},buildLeftRecursionError:function(e){var t=e.topLevelRule.name;return"Left Recursion found in grammar.\nrule: <"+t+"> can be invoked from itself (directly or indirectly)\nwithout consuming any Tokens. The grammar path that causes this is: \n "+(t+" --\x3e "+o.map(e.leftRecursionPath,(function(e){return e.name})).concat([t]).join(" --\x3e "))+"\n To fix this refactor your grammar to remove the left recursion.\nsee: https://en.wikipedia.org/wiki/LL_parser#Left_Factoring."},buildInvalidRuleNameError:function(e){return"Invalid grammar rule name: ->"+e.topLevelRule.name+"<- it must match the pattern: ->"+e.expectedPattern.toString()+"<-"},buildDuplicateRuleNameError:function(e){return"Duplicate definition, rule: ->"+(e.topLevelRule instanceof s.Rule?e.topLevelRule.name:e.topLevelRule)+"<- is already defined in the grammar: ->"+e.grammarName+"<-"}}},function(e,t,r){"use strict";r.r(t),r.d(t,"validateGrammar",(function(){return p})),r.d(t,"identifyProductionForDuplicates",(function(){return m})),r.d(t,"OccurrenceValidationCollector",(function(){return f})),r.d(t,"validTermsPattern",(function(){return E})),r.d(t,"validNestedRuleName",(function(){return T})),r.d(t,"validateRuleName",(function(){return y})),r.d(t,"validateNestedRuleName",(function(){return A})),r.d(t,"validateTokenName",(function(){return R})),r.d(t,"validateRuleDoesNotAlreadyExist",(function(){return N})),r.d(t,"validateRuleIsOverridden",(function(){return S})),r.d(t,"validateNoLeftRecursion",(function(){return C})),r.d(t,"getFirstNoneTerminal",(function(){return v})),r.d(t,"validateEmptyOrAlternative",(function(){return M})),r.d(t,"validateAmbiguousAlternationAlternatives",(function(){return I})),r.d(t,"RepetionCollector",(function(){return w})),r.d(t,"validateTooManyAlts",(function(){return F})),r.d(t,"validateSomeNonEmptyLookaheadPath",(function(){return b})),r.d(t,"checkPrefixAlternativesAmbiguities",(function(){return _}));var n,o=r(88),s=r(87),i=r(101),a=r(108),u=r(104),c=r(109),l=r(91),h=r(102),d=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)});function p(e,t,r,n,a,c){var l=o.map(e,(function(e){return function(e,t){var r=new f;e.accept(r);var n=r.allProductions,a=o.groupBy(n,m),u=o.pick(a,(function(e){return e.length>1}));return o.map(o.values(u),(function(r){var n=o.first(r),a=t.buildDuplicateFoundError(e,r),u=Object(i.getProductionDslName)(n),c={message:a,type:s.ParserDefinitionErrorType.DUPLICATE_PRODUCTIONS,ruleName:e.name,dslName:u,occurrence:n.idx},l=g(n);return l&&(c.parameter=l),c}))}(e,a)})),h=o.map(e,(function(e){return C(e,e,a)})),d=[],p=[],E=[];Object(o.every)(h,o.isEmpty)&&(d=Object(o.map)(e,(function(e){return M(e,a)})),p=Object(o.map)(e,(function(e){return I(e,t,n,a)})),E=b(e,t,a));var T=function(e,t,r){var n=[],i=Object(o.map)(t,(function(e){return e.name}));return Object(o.forEach)(e,(function(e){var t=e.name;if(Object(o.contains)(i,t)){var a=r.buildNamespaceConflictError(e);n.push({message:a,type:s.ParserDefinitionErrorType.CONFLICT_TOKENS_RULES_NAMESPACE,ruleName:t})}})),n}(e,r,a),S=o.map(r,(function(e){return R(e,a)})),v=function(e,t){var r=[];return Object(o.forEach)(e,(function(e){var n=new u.NamedDSLMethodsCollectorVisitor("");e.accept(n);var s=Object(o.map)(n.result,(function(e){return e.orgProd}));r.push(Object(o.map)(s,(function(r){return A(e,r,t)})))})),Object(o.flatten)(r)}(e,a),O=function(e,t){var r=[];return Object(o.forEach)(e,(function(e){var n=new u.NamedDSLMethodsCollectorVisitor("");e.accept(n);var i=Object(o.groupBy)(n.result,(function(e){return e.name})),a=Object(o.pick)(i,(function(e){return e.length>1}));Object(o.forEach)(Object(o.values)(a),(function(n){var i=Object(o.map)(n,(function(e){return e.orgProd})),a=t.buildDuplicateNestedRuleNameError(e,i);r.push({message:a,type:s.ParserDefinitionErrorType.DUPLICATE_NESTED_NAME,ruleName:e.name})}))})),r}(e,a),w=Object(o.map)(e,(function(e){return F(e,a)})),_=Object(o.map)(e,(function(e){return y(e,a)})),L=Object(o.map)(e,(function(t){return N(t,e,c,a)}));return o.flatten(l.concat(S,v,O,E,h,d,p,T,w,_,L))}function m(e){return Object(i.getProductionDslName)(e)+"_#_"+e.idx+"_#_"+g(e)}function g(e){return e instanceof l.Terminal?e.terminalType.name:e instanceof l.NonTerminal?e.nonTerminalName:""}var f=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.allProductions=[],t}return d(t,e),t.prototype.visitNonTerminal=function(e){this.allProductions.push(e)},t.prototype.visitOption=function(e){this.allProductions.push(e)},t.prototype.visitRepetitionWithSeparator=function(e){this.allProductions.push(e)},t.prototype.visitRepetitionMandatory=function(e){this.allProductions.push(e)},t.prototype.visitRepetitionMandatoryWithSeparator=function(e){this.allProductions.push(e)},t.prototype.visitRepetition=function(e){this.allProductions.push(e)},t.prototype.visitAlternation=function(e){this.allProductions.push(e)},t.prototype.visitTerminal=function(e){this.allProductions.push(e)},t}(h.GAstVisitor),E=/^[a-zA-Z_]\w*$/,T=new RegExp(E.source.replace("^","^\\$"));function y(e,t){var r=[],n=e.name;return n.match(E)||r.push({message:t.buildInvalidRuleNameError({topLevelRule:e,expectedPattern:E}),type:s.ParserDefinitionErrorType.INVALID_RULE_NAME,ruleName:n}),r}function A(e,t,r){var n,o=[];return t.name.match(T)||(n=r.buildInvalidNestedRuleNameError(e,t),o.push({message:n,type:s.ParserDefinitionErrorType.INVALID_NESTED_RULE_NAME,ruleName:e.name})),o}function R(e,t){var r=[];return e.name.match(E)||r.push({message:t.buildTokenNameError({tokenType:e,expectedPattern:E}),type:s.ParserDefinitionErrorType.INVALID_TOKEN_NAME}),r}function N(e,t,r,n){var i=[];if(Object(o.reduce)(t,(function(t,r){return r.name===e.name?t+1:t}),0)>1){var a=n.buildDuplicateRuleNameError({topLevelRule:e,grammarName:r});i.push({message:a,type:s.ParserDefinitionErrorType.DUPLICATE_RULE_NAME,ruleName:e.name})}return i}function S(e,t,r){var n,i=[];return o.contains(t,e)||(n="Invalid rule override, rule: ->"+e+"<- cannot be overridden in the grammar: ->"+r+"<-as it is not defined in any of the super grammars ",i.push({message:n,type:s.ParserDefinitionErrorType.INVALID_RULE_OVERRIDE,ruleName:e})),i}function C(e,t,r,n){void 0===n&&(n=[]);var i=[],a=v(t.definition);if(o.isEmpty(a))return[];var u=e.name;o.contains(a,e)&&i.push({message:r.buildLeftRecursionError({topLevelRule:e,leftRecursionPath:n}),type:s.ParserDefinitionErrorType.LEFT_RECURSION,ruleName:u});var c=o.difference(a,n.concat([e])),l=o.map(c,(function(t){var s=o.cloneArr(n);return s.push(t),C(e,t,r,s)}));return i.concat(o.flatten(l))}function v(e){var t=[];if(o.isEmpty(e))return t;var r=o.first(e);if(r instanceof l.NonTerminal)t.push(r.referencedRule);else if(r instanceof l.Flat||r instanceof l.Option||r instanceof l.RepetitionMandatory||r instanceof l.RepetitionMandatoryWithSeparator||r instanceof l.RepetitionWithSeparator||r instanceof l.Repetition)t=t.concat(v(r.definition));else if(r instanceof l.Alternation)t=o.flatten(o.map(r.definition,(function(e){return v(e.definition)})));else if(!(r instanceof l.Terminal))throw Error("non exhaustive match");var n=Object(i.isOptionalProd)(r),s=e.length>1;if(n&&s){var a=o.drop(e);return t.concat(v(a))}return t}var O=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.alternations=[],t}return d(t,e),t.prototype.visitAlternation=function(e){this.alternations.push(e)},t}(h.GAstVisitor);function M(e,t){var r=new O;e.accept(r);var n=r.alternations;return o.reduce(n,(function(r,n){var i=o.dropRight(n.definition),a=o.map(i,(function(r,i){var a=Object(c.nextPossibleTokensAfter)([r],[],null,1);return o.isEmpty(a)?{message:t.buildEmptyAlternationError({topLevelRule:e,alternation:n,emptyChoiceIdx:i}),type:s.ParserDefinitionErrorType.NONE_LAST_EMPTY_ALT,ruleName:e.name,occurrence:n.idx,alternative:i+1}:null}));return r.concat(o.compact(a))}),[])}function I(e,t,r,n){var u=new O;e.accept(u);var c=u.alternations,l=r[e.name];return l&&(c=Object(o.reject)(c,(function(e){return l[Object(i.getProductionDslName)(e)+(0===e.idx?"":e.idx)]}))),c=Object(o.reject)(c,(function(e){return!0===e.ignoreAmbiguities})),o.reduce(c,(function(r,i){var u=i.idx,c=i.maxLookahead||t,l=Object(a.getLookaheadPathsForOr)(u,e,c,i),h=function(e,t,r,n){var i=[],u=Object(o.reduce)(e,(function(r,n,s){return!0===t.definition[s].ignoreAmbiguities||Object(o.forEach)(n,(function(n){var u=[s];Object(o.forEach)(e,(function(e,r){s!==r&&Object(a.containsPath)(e,n)&&!0!==t.definition[r].ignoreAmbiguities&&u.push(r)})),u.length>1&&!Object(a.containsPath)(i,n)&&(i.push(n),r.push({alts:u,path:n}))})),r}),[]);return o.map(u,(function(e){var i=Object(o.map)(e.alts,(function(e){return e+1}));return{message:n.buildAlternationAmbiguityError({topLevelRule:r,alternation:t,ambiguityIndices:i,prefixPath:e.path}),type:s.ParserDefinitionErrorType.AMBIGUOUS_ALTS,ruleName:r.name,occurrence:t.idx,alternatives:[e.alts]}}))}(l,i,e,n),d=_(l,i,e,n);return r.concat(h,d)}),[])}var w=function(e){function t(){var t=null!==e&&e.apply(this,arguments)||this;return t.allProductions=[],t}return d(t,e),t.prototype.visitRepetitionWithSeparator=function(e){this.allProductions.push(e)},t.prototype.visitRepetitionMandatory=function(e){this.allProductions.push(e)},t.prototype.visitRepetitionMandatoryWithSeparator=function(e){this.allProductions.push(e)},t.prototype.visitRepetition=function(e){this.allProductions.push(e)},t}(h.GAstVisitor);function F(e,t){var r=new O;e.accept(r);var n=r.alternations;return o.reduce(n,(function(r,n){return n.definition.length>255&&r.push({message:t.buildTooManyAlternativesError({topLevelRule:e,alternation:n}),type:s.ParserDefinitionErrorType.TOO_MANY_ALTS,ruleName:e.name,occurrence:n.idx}),r}),[])}function b(e,t,r){var n=[];return Object(o.forEach)(e,(function(e){var i=new w;e.accept(i);var u=i.allProductions;Object(o.forEach)(u,(function(i){var u=Object(a.getProdType)(i),c=i.maxLookahead||t,l=i.idx,h=Object(a.getLookaheadPathsForOptionalProd)(l,e,u,c)[0];if(Object(o.isEmpty)(Object(o.flatten)(h))){var d=r.buildEmptyRepetitionError({topLevelRule:e,repetition:i});n.push({message:d,type:s.ParserDefinitionErrorType.NO_NON_EMPTY_LOOKAHEAD,ruleName:e.name})}}))})),n}function _(e,t,r,n){var i=[],u=Object(o.reduce)(e,(function(e,t,r){var n=Object(o.map)(t,(function(e){return{idx:r,path:e}}));return e.concat(n)}),[]);return Object(o.forEach)(u,(function(e){if(!0!==t.definition[e.idx].ignoreAmbiguities){var c=e.idx,l=e.path,h=Object(o.findAll)(u,(function(e){return!0!==t.definition[e.idx].ignoreAmbiguities&&e.idx<c&&Object(a.isStrictPrefixOfPath)(e.path,l)})),d=Object(o.map)(h,(function(e){var o=[e.idx+1,c+1],i=0===t.idx?"":t.idx;return{message:n.buildAlternationPrefixAmbiguityError({topLevelRule:r,alternation:t,ambiguityIndices:o,prefixPath:e.path}),type:s.ParserDefinitionErrorType.AMBIGUOUS_PREFIX_ALTS,ruleName:r.name,occurrence:i,alternatives:o}}));i=i.concat(d)}})),i}},function(e,t,r){"use strict";r.r(t),r.d(t,"PROD_TYPE",(function(){return o})),r.d(t,"getProdType",(function(){return d})),r.d(t,"buildLookaheadFuncForOr",(function(){return p})),r.d(t,"buildLookaheadFuncForOptionalProd",(function(){return m})),r.d(t,"buildAlternativesLookAheadFunc",(function(){return g})),r.d(t,"buildSingleAlternativeLookaheadFunction",(function(){return f})),r.d(t,"lookAheadSequenceFromAlternatives",(function(){return N})),r.d(t,"getLookaheadPathsForOr",(function(){return S})),r.d(t,"getLookaheadPathsForOptionalProd",(function(){return C})),r.d(t,"containsPath",(function(){return v})),r.d(t,"isStrictPrefixOfPath",(function(){return O})),r.d(t,"areTokenCategoriesNotUsed",(function(){return M}));var n,o,s=r(88),i=r(109),a=r(90),u=r(98),c=r(91),l=r(102),h=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)});function d(e){if(e instanceof c.Option)return o.OPTION;if(e instanceof c.Repetition)return o.REPETITION;if(e instanceof c.RepetitionMandatory)return o.REPETITION_MANDATORY;if(e instanceof c.RepetitionMandatoryWithSeparator)return o.REPETITION_MANDATORY_WITH_SEPARATOR;if(e instanceof c.RepetitionWithSeparator)return o.REPETITION_WITH_SEPARATOR;if(e instanceof c.Alternation)return o.ALTERNATION;throw Error("non exhaustive match")}function p(e,t,r,n,o,s){var i=S(e,t,r);return s(i,n,M(i)?u.tokenStructuredMatcherNoCategories:u.tokenStructuredMatcher,o)}function m(e,t,r,n,o,s){var i=C(e,t,o,r),a=M(i)?u.tokenStructuredMatcherNoCategories:u.tokenStructuredMatcher;return s(i[0],a,n)}function g(e,t,r,n){var o=e.length,i=Object(s.every)(e,(function(e){return Object(s.every)(e,(function(e){return 1===e.length}))}));if(t)return function(t){for(var n=Object(s.map)(t,(function(e){return e.GATE})),i=0;i<o;i++){var a=e[i],u=a.length,c=n[i];if(void 0===c||!1!==c.call(this))e:for(var l=0;l<u;l++){for(var h=a[l],d=h.length,p=0;p<d;p++){var m=this.LA(p+1);if(!1===r(m,h[p]))continue e}return i}}};if(i&&!n){var a=Object(s.map)(e,(function(e){return Object(s.flatten)(e)})),u=Object(s.reduce)(a,(function(e,t,r){return Object(s.forEach)(t,(function(t){Object(s.has)(e,t.tokenTypeIdx)||(e[t.tokenTypeIdx]=r),Object(s.forEach)(t.categoryMatches,(function(t){Object(s.has)(e,t)||(e[t]=r)}))})),e}),[]);return function(){var e=this.LA(1);return u[e.tokenTypeIdx]}}return function(){for(var t=0;t<o;t++){var n=e[t],s=n.length;e:for(var i=0;i<s;i++){for(var a=n[i],u=a.length,c=0;c<u;c++){var l=this.LA(c+1);if(!1===r(l,a[c]))continue e}return t}}}}function f(e,t,r){var n=Object(s.every)(e,(function(e){return 1===e.length})),o=e.length;if(n&&!r){var i=Object(s.flatten)(e);if(1===i.length&&Object(s.isEmpty)(i[0].categoryMatches)){var a=i[0].tokenTypeIdx;return function(){return this.LA(1).tokenTypeIdx===a}}var u=Object(s.reduce)(i,(function(e,t,r){return e[t.tokenTypeIdx]=!0,Object(s.forEach)(t.categoryMatches,(function(t){e[t]=!0})),e}),[]);return function(){var e=this.LA(1);return!0===u[e.tokenTypeIdx]}}return function(){e:for(var r=0;r<o;r++){for(var n=e[r],s=n.length,i=0;i<s;i++){var a=this.LA(i+1);if(!1===t(a,n[i]))continue e}return!0}return!1}}!function(e){e[e.OPTION=0]="OPTION",e[e.REPETITION=1]="REPETITION",e[e.REPETITION_MANDATORY=2]="REPETITION_MANDATORY",e[e.REPETITION_MANDATORY_WITH_SEPARATOR=3]="REPETITION_MANDATORY_WITH_SEPARATOR",e[e.REPETITION_WITH_SEPARATOR=4]="REPETITION_WITH_SEPARATOR",e[e.ALTERNATION=5]="ALTERNATION"}(o||(o={}));var E=function(e){function t(t,r,n){var o=e.call(this)||this;return o.topProd=t,o.targetOccurrence=r,o.targetProdType=n,o}return h(t,e),t.prototype.startWalking=function(){return this.walk(this.topProd),this.restDef},t.prototype.checkIsTarget=function(e,t,r,n){return e.idx===this.targetOccurrence&&this.targetProdType===t&&(this.restDef=r.concat(n),!0)},t.prototype.walkOption=function(t,r,n){this.checkIsTarget(t,o.OPTION,r,n)||e.prototype.walkOption.call(this,t,r,n)},t.prototype.walkAtLeastOne=function(t,r,n){this.checkIsTarget(t,o.REPETITION_MANDATORY,r,n)||e.prototype.walkOption.call(this,t,r,n)},t.prototype.walkAtLeastOneSep=function(t,r,n){this.checkIsTarget(t,o.REPETITION_MANDATORY_WITH_SEPARATOR,r,n)||e.prototype.walkOption.call(this,t,r,n)},t.prototype.walkMany=function(t,r,n){this.checkIsTarget(t,o.REPETITION,r,n)||e.prototype.walkOption.call(this,t,r,n)},t.prototype.walkManySep=function(t,r,n){this.checkIsTarget(t,o.REPETITION_WITH_SEPARATOR,r,n)||e.prototype.walkOption.call(this,t,r,n)},t}(a.RestWalker),T=function(e){function t(t,r,n){var o=e.call(this)||this;return o.targetOccurrence=t,o.targetProdType=r,o.targetRef=n,o.result=[],o}return h(t,e),t.prototype.checkIsTarget=function(e,t){e.idx!==this.targetOccurrence||this.targetProdType!==t||void 0!==this.targetRef&&e!==this.targetRef||(this.result=e.definition)},t.prototype.visitOption=function(e){this.checkIsTarget(e,o.OPTION)},t.prototype.visitRepetition=function(e){this.checkIsTarget(e,o.REPETITION)},t.prototype.visitRepetitionMandatory=function(e){this.checkIsTarget(e,o.REPETITION_MANDATORY)},t.prototype.visitRepetitionMandatoryWithSeparator=function(e){this.checkIsTarget(e,o.REPETITION_MANDATORY_WITH_SEPARATOR)},t.prototype.visitRepetitionWithSeparator=function(e){this.checkIsTarget(e,o.REPETITION_WITH_SEPARATOR)},t.prototype.visitAlternation=function(e){this.checkIsTarget(e,o.ALTERNATION)},t}(l.GAstVisitor);function y(e){for(var t=new Array(e),r=0;r<e;r++)t[r]=[];return t}function A(e){for(var t=[""],r=0;r<e.length;r++){for(var n=e[r],o=[],s=0;s<t.length;s++){var i=t[s];o.push(i+"_"+n.tokenTypeIdx);for(var a=0;a<n.categoryMatches.length;a++){var u="_"+n.categoryMatches[a];o.push(i+u)}}t=o}return t}function R(e,t,r){for(var n=0;n<e.length;n++)if(n!==r)for(var o=e[n],s=0;s<t.length;s++){if(!0===o[t[s]])return!1}return!0}function N(e,t){for(var r=Object(s.map)(e,(function(e){return Object(i.possiblePathsFrom)([e],1)})),n=y(r.length),o=Object(s.map)(r,(function(e){var t={};return Object(s.forEach)(e,(function(e){var r=A(e.partialPath);Object(s.forEach)(r,(function(e){t[e]=!0}))})),t})),a=r,u=1;u<=t;u++){var c=a;a=y(c.length);for(var l=function(e){for(var r=c[e],l=0;l<r.length;l++){var h=r[l].partialPath,d=r[l].suffixDef,p=A(h);if(R(o,p,e)||Object(s.isEmpty)(d)||h.length===t){var m=n[e];if(!1===v(m,h)){m.push(h);for(var g=0;g<p.length;g++){var f=p[g];o[e][f]=!0}}}else{var E=Object(i.possiblePathsFrom)(d,u+1,h);a[e]=a[e].concat(E),Object(s.forEach)(E,(function(t){var r=A(t.partialPath);Object(s.forEach)(r,(function(t){o[e][t]=!0}))}))}}},h=0;h<c.length;h++)l(h)}return n}function S(e,t,r,n){var s=new T(e,o.ALTERNATION,n);return t.accept(s),N(s.result,r)}function C(e,t,r,n){var o=new T(e,r);t.accept(o);var s=o.result,i=new E(t,e,r).startWalking();return N([new c.Flat({definition:s}),new c.Flat({definition:i})],n)}function v(e,t){e:for(var r=0;r<e.length;r++){var n=e[r];if(n.length===t.length){for(var o=0;o<n.length;o++){var s=t[o],i=n[o];if(!1===(s===i||void 0!==i.categoryMatchesMap[s.tokenTypeIdx]))continue e}return!0}}return!1}function O(e,t){return e.length<t.length&&Object(s.every)(e,(function(e,r){var n=t[r];return e===n||n.categoryMatchesMap[e.tokenTypeIdx]}))}function M(e){return Object(s.every)(e,(function(e){return Object(s.every)(e,(function(e){return Object(s.every)(e,(function(e){return Object(s.isEmpty)(e.categoryMatches)}))}))}))}},function(e,t,r){"use strict";r.r(t),r.d(t,"AbstractNextPossibleTokensWalker",(function(){return c})),r.d(t,"NextAfterTokenWalker",(function(){return l})),r.d(t,"AbstractNextTerminalAfterProductionWalker",(function(){return h})),r.d(t,"NextTerminalAfterManyWalker",(function(){return d})),r.d(t,"NextTerminalAfterManySepWalker",(function(){return p})),r.d(t,"NextTerminalAfterAtLeastOneWalker",(function(){return m})),r.d(t,"NextTerminalAfterAtLeastOneSepWalker",(function(){return g})),r.d(t,"possiblePathsFrom",(function(){return f})),r.d(t,"nextPossibleTokensAfter",(function(){return E}));var n,o=r(90),s=r(88),i=r(100),a=r(91),u=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}),c=function(e){function t(t,r){var n=e.call(this)||this;return n.topProd=t,n.path=r,n.possibleTokTypes=[],n.nextProductionName="",n.nextProductionOccurrence=0,n.found=!1,n.isAtEndOfPath=!1,n}return u(t,e),t.prototype.startWalking=function(){if(this.found=!1,this.path.ruleStack[0]!==this.topProd.name)throw Error("The path does not start with the walker's top Rule!");return this.ruleStack=Object(s.cloneArr)(this.path.ruleStack).reverse(),this.occurrenceStack=Object(s.cloneArr)(this.path.occurrenceStack).reverse(),this.ruleStack.pop(),this.occurrenceStack.pop(),this.updateExpectedNext(),this.walk(this.topProd),this.possibleTokTypes},t.prototype.walk=function(t,r){void 0===r&&(r=[]),this.found||e.prototype.walk.call(this,t,r)},t.prototype.walkProdRef=function(e,t,r){if(e.referencedRule.name===this.nextProductionName&&e.idx===this.nextProductionOccurrence){var n=t.concat(r);this.updateExpectedNext(),this.walk(e.referencedRule,n)}},t.prototype.updateExpectedNext=function(){Object(s.isEmpty)(this.ruleStack)?(this.nextProductionName="",this.nextProductionOccurrence=0,this.isAtEndOfPath=!0):(this.nextProductionName=this.ruleStack.pop(),this.nextProductionOccurrence=this.occurrenceStack.pop())},t}(o.RestWalker),l=function(e){function t(t,r){var n=e.call(this,t,r)||this;return n.path=r,n.nextTerminalName="",n.nextTerminalOccurrence=0,n.nextTerminalName=n.path.lastTok.name,n.nextTerminalOccurrence=n.path.lastTokOccurrence,n}return u(t,e),t.prototype.walkTerminal=function(e,t,r){if(this.isAtEndOfPath&&e.terminalType.name===this.nextTerminalName&&e.idx===this.nextTerminalOccurrence&&!this.found){var n=t.concat(r),o=new a.Flat({definition:n});this.possibleTokTypes=Object(i.first)(o),this.found=!0}},t}(c),h=function(e){function t(t,r){var n=e.call(this)||this;return n.topRule=t,n.occurrence=r,n.result={token:void 0,occurrence:void 0,isEndOfRule:void 0},n}return u(t,e),t.prototype.startWalking=function(){return this.walk(this.topRule),this.result},t}(o.RestWalker),d=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return u(t,e),t.prototype.walkMany=function(t,r,n){if(t.idx===this.occurrence){var o=Object(s.first)(r.concat(n));this.result.isEndOfRule=void 0===o,o instanceof a.Terminal&&(this.result.token=o.terminalType,this.result.occurrence=o.idx)}else e.prototype.walkMany.call(this,t,r,n)},t}(h),p=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return u(t,e),t.prototype.walkManySep=function(t,r,n){if(t.idx===this.occurrence){var o=Object(s.first)(r.concat(n));this.result.isEndOfRule=void 0===o,o instanceof a.Terminal&&(this.result.token=o.terminalType,this.result.occurrence=o.idx)}else e.prototype.walkManySep.call(this,t,r,n)},t}(h),m=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return u(t,e),t.prototype.walkAtLeastOne=function(t,r,n){if(t.idx===this.occurrence){var o=Object(s.first)(r.concat(n));this.result.isEndOfRule=void 0===o,o instanceof a.Terminal&&(this.result.token=o.terminalType,this.result.occurrence=o.idx)}else e.prototype.walkAtLeastOne.call(this,t,r,n)},t}(h),g=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return u(t,e),t.prototype.walkAtLeastOneSep=function(t,r,n){if(t.idx===this.occurrence){var o=Object(s.first)(r.concat(n));this.result.isEndOfRule=void 0===o,o instanceof a.Terminal&&(this.result.token=o.terminalType,this.result.occurrence=o.idx)}else e.prototype.walkAtLeastOneSep.call(this,t,r,n)},t}(h);function f(e,t,r){void 0===r&&(r=[]),r=Object(s.cloneArr)(r);var n=[],o=0;function i(i){var a=f(i.concat(Object(s.drop)(e,o+1)),t,r);return n.concat(a)}for(;r.length<t&&o<e.length;){var u=e[o];if(u instanceof a.Flat)return i(u.definition);if(u instanceof a.NonTerminal)return i(u.definition);if(u instanceof a.Option)n=i(u.definition);else{if(u instanceof a.RepetitionMandatory)return i(c=u.definition.concat([new a.Repetition({definition:u.definition})]));if(u instanceof a.RepetitionMandatoryWithSeparator)return i(c=[new a.Flat({definition:u.definition}),new a.Repetition({definition:[new a.Terminal({terminalType:u.separator})].concat(u.definition)})]);if(u instanceof a.RepetitionWithSeparator){var c=u.definition.concat([new a.Repetition({definition:[new a.Terminal({terminalType:u.separator})].concat(u.definition)})]);n=i(c)}else if(u instanceof a.Repetition){c=u.definition.concat([new a.Repetition({definition:u.definition})]);n=i(c)}else{if(u instanceof a.Alternation)return Object(s.forEach)(u.definition,(function(e){n=i(e.definition)})),n;if(!(u instanceof a.Terminal))throw Error("non exhaustive match");r.push(u.terminalType)}}o++}return n.push({partialPath:r,suffixDef:Object(s.drop)(e,o)}),n}function E(e,t,r,n){var o=["EXIT_NONE_TERMINAL"],i=!1,u=t.length,c=u-n-1,l=[],h=[];for(h.push({idx:-1,def:e,ruleStack:[],occurrenceStack:[]});!Object(s.isEmpty)(h);){var d=h.pop();if("EXIT_ALTERNATIVE"!==d){var p=d.def,m=d.idx,g=d.ruleStack,f=d.occurrenceStack;if(!Object(s.isEmpty)(p)){var E=p[0];if("EXIT_NONE_TERMINAL"===E){var y={idx:m,def:Object(s.drop)(p),ruleStack:Object(s.dropRight)(g),occurrenceStack:Object(s.dropRight)(f)};h.push(y)}else if(E instanceof a.Terminal)if(m<u-1){var A=m+1;if(r(t[A],E.terminalType)){y={idx:A,def:Object(s.drop)(p),ruleStack:g,occurrenceStack:f};h.push(y)}}else{if(m!==u-1)throw Error("non exhaustive match");l.push({nextTokenType:E.terminalType,nextTokenOccurrence:E.idx,ruleStack:g,occurrenceStack:f}),i=!0}else if(E instanceof a.NonTerminal){var R=Object(s.cloneArr)(g);R.push(E.nonTerminalName);var N=Object(s.cloneArr)(f);N.push(E.idx);y={idx:m,def:E.definition.concat(o,Object(s.drop)(p)),ruleStack:R,occurrenceStack:N};h.push(y)}else if(E instanceof a.Option){var S={idx:m,def:Object(s.drop)(p),ruleStack:g,occurrenceStack:f};h.push(S),h.push("EXIT_ALTERNATIVE");var C={idx:m,def:E.definition.concat(Object(s.drop)(p)),ruleStack:g,occurrenceStack:f};h.push(C)}else if(E instanceof a.RepetitionMandatory){var v=new a.Repetition({definition:E.definition,idx:E.idx});y={idx:m,def:E.definition.concat([v],Object(s.drop)(p)),ruleStack:g,occurrenceStack:f};h.push(y)}else if(E instanceof a.RepetitionMandatoryWithSeparator){var O=new a.Terminal({terminalType:E.separator});v=new a.Repetition({definition:[O].concat(E.definition),idx:E.idx}),y={idx:m,def:E.definition.concat([v],Object(s.drop)(p)),ruleStack:g,occurrenceStack:f};h.push(y)}else if(E instanceof a.RepetitionWithSeparator){S={idx:m,def:Object(s.drop)(p),ruleStack:g,occurrenceStack:f};h.push(S),h.push("EXIT_ALTERNATIVE");O=new a.Terminal({terminalType:E.separator});var M=new a.Repetition({definition:[O].concat(E.definition),idx:E.idx});C={idx:m,def:E.definition.concat([M],Object(s.drop)(p)),ruleStack:g,occurrenceStack:f};h.push(C)}else if(E instanceof a.Repetition){S={idx:m,def:Object(s.drop)(p),ruleStack:g,occurrenceStack:f};h.push(S),h.push("EXIT_ALTERNATIVE");M=new a.Repetition({definition:E.definition,idx:E.idx}),C={idx:m,def:E.definition.concat([M],Object(s.drop)(p)),ruleStack:g,occurrenceStack:f};h.push(C)}else if(E instanceof a.Alternation)for(var I=E.definition.length-1;I>=0;I--){var w={idx:m,def:E.definition[I].definition.concat(Object(s.drop)(p)),ruleStack:g,occurrenceStack:f};h.push(w),h.push("EXIT_ALTERNATIVE")}else if(E instanceof a.Flat)h.push({idx:m,def:E.definition.concat(Object(s.drop)(p)),ruleStack:g,occurrenceStack:f});else{if(!(E instanceof a.Rule))throw Error("non exhaustive match");h.push(T(E,m,g,f))}}}else i&&Object(s.last)(h).idx<=c&&h.pop()}return l}function T(e,t,r,n){var o=Object(s.cloneArr)(r);o.push(e.name);var i=Object(s.cloneArr)(n);return i.push(1),{idx:t,def:e.definition,ruleStack:o,occurrenceStack:i}}},function(e,t,r){"use strict";r.r(t),r.d(t,"resolveGrammar",(function(){return u})),r.d(t,"validateGrammar",(function(){return c})),r.d(t,"assignOccurrenceIndices",(function(){return l}));var n=r(88),o=r(111),s=r(107),i=r(106),a=r(101);function u(e){e=Object(n.defaults)(e,{errMsgProvider:i.defaultGrammarResolverErrorProvider});var t={};return Object(n.forEach)(e.rules,(function(e){t[e.name]=e})),Object(o.resolveGrammar)(t,e.errMsgProvider)}function c(e){return e=Object(n.defaults)(e,{errMsgProvider:i.defaultGrammarValidatorErrorProvider,ignoredIssues:{}}),Object(s.validateGrammar)(e.rules,e.maxLookahead,e.tokenTypes,e.ignoredIssues,e.errMsgProvider,e.grammarName)}function l(e){Object(n.forEach)(e.rules,(function(e){var t=new a.DslMethodsCollectorVisitor;e.accept(t),Object(n.forEach)(t.dslMethods,(function(e){Object(n.forEach)(e,(function(e,t){e.idx=t+1}))}))}))}},function(e,t,r){"use strict";r.r(t),r.d(t,"resolveGrammar",(function(){return u})),r.d(t,"GastRefResolverVisitor",(function(){return c}));var n,o=r(87),s=r(88),i=r(102),a=(n=function(e,t){return(n=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)},function(e,t){function r(){this.constructor=e}n(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)});function u(e,t){var r=new c(e,t);return r.resolveRefs(),r.errors}var c=function(e){function t(t,r){var n=e.call(this)||this;return n.nameToTopRule=t,n.errMsgProvider=r,n.errors=[],n}return a(t,e),t.prototype.resolveRefs=function(){var e=this;Object(s.forEach)(Object(s.values)(this.nameToTopRule),(function(t){e.currTopLevel=t,t.accept(e)}))},t.prototype.visitNonTerminal=function(e){var t=this.nameToTopRule[e.nonTerminalName];if(t)e.referencedRule=t;else{var r=this.errMsgProvider.buildRuleNotFoundError(this.currTopLevel,e);this.errors.push({message:r,type:o.ParserDefinitionErrorType.UNRESOLVED_SUBRULE_REF,ruleName:this.currTopLevel.name,unresolvedRefName:e.nonTerminalName})}},t}(i.GAstVisitor)},function(e,t,r){"use strict";r.r(t),r.d(t,"EOF_FOLLOW_KEY",(function(){return u})),r.d(t,"IN_RULE_RECOVERY_EXCEPTION",(function(){return c})),r.d(t,"InRuleRecoveryException",(function(){return l})),r.d(t,"Recoverable",(function(){return h})),r.d(t,"attemptInRepetitionRecovery",(function(){return d}));var n=r(92),o=r(88),s=r(113),i=r(103),a=r(87),u={},c="InRuleRecoveryException";function l(e){this.name=c,this.message=e}l.prototype=Error.prototype;var h=function(){function e(){}return e.prototype.initRecoverable=function(e){this.firstAfterRepMap={},this.resyncFollows={},this.recoveryEnabled=Object(o.has)(e,"recoveryEnabled")?e.recoveryEnabled:a.DEFAULT_PARSER_CONFIG.recoveryEnabled,this.recoveryEnabled&&(this.attemptInRepetitionRecovery=d)},e.prototype.getTokenToInsert=function(e){var t=Object(n.createTokenInstance)(e,"",NaN,NaN,NaN,NaN,NaN,NaN);return t.isInsertedInRecovery=!0,t},e.prototype.canTokenTypeBeInsertedInRecovery=function(e){return!0},e.prototype.tryInRepetitionRecovery=function(e,t,r,n){for(var i=this,a=this.findReSyncTokenType(),u=this.exportLexerState(),c=[],l=!1,h=this.LA(1),d=this.LA(1),p=function(){var e=i.LA(0),t=i.errorMessageProvider.buildMismatchTokenMessage({expected:n,actual:h,previous:e,ruleName:i.getCurrRuleFullName()}),r=new s.MismatchedTokenException(t,h,i.LA(0));r.resyncedTokens=Object(o.dropRight)(c),i.SAVE_ERROR(r)};!l;){if(this.tokenMatcher(d,n))return void p();if(r.call(this))return p(),void e.apply(this,t);this.tokenMatcher(d,a)?l=!0:(d=this.SKIP_TOKEN(),this.addToResyncTokens(d,c))}this.importLexerState(u)},e.prototype.shouldInRepetitionRecoveryBeTried=function(e,t,r){return!1!==r&&(void 0!==e&&void 0!==t&&(!this.tokenMatcher(this.LA(1),e)&&(!this.isBackTracking()&&!this.canPerformInRuleRecovery(e,this.getFollowsForInRuleRecovery(e,t)))))},e.prototype.getFollowsForInRuleRecovery=function(e,t){var r=this.getCurrentGrammarPath(e,t);return this.getNextPossibleTokenTypes(r)},e.prototype.tryInRuleRecovery=function(e,t){if(this.canRecoverWithSingleTokenInsertion(e,t))return this.getTokenToInsert(e);if(this.canRecoverWithSingleTokenDeletion(e)){var r=this.SKIP_TOKEN();return this.consumeToken(),r}throw new l("sad sad panda")},e.prototype.canPerformInRuleRecovery=function(e,t){return this.canRecoverWithSingleTokenInsertion(e,t)||this.canRecoverWithSingleTokenDeletion(e)},e.prototype.canRecoverWithSingleTokenInsertion=function(e,t){var r=this;if(!this.canTokenTypeBeInsertedInRecovery(e))return!1;if(Object(o.isEmpty)(t))return!1;var n=this.LA(1);return void 0!==Object(o.find)(t,(function(e){return r.tokenMatcher(n,e)}))},e.prototype.canRecoverWithSingleTokenDeletion=function(e){return this.tokenMatcher(this.LA(2),e)},e.prototype.isInCurrentRuleReSyncSet=function(e){var t=this.getCurrFollowKey(),r=this.getFollowSetFromFollowKey(t);return Object(o.contains)(r,e)},e.prototype.findReSyncTokenType=function(){for(var e=this.flattenFollowSet(),t=this.LA(1),r=2;;){var n=t.tokenType;if(Object(o.contains)(e,n))return n;t=this.LA(r),r++}},e.prototype.getCurrFollowKey=function(){if(1===this.RULE_STACK.length)return u;var e=this.getLastExplicitRuleShortName(),t=this.getLastExplicitRuleOccurrenceIndex(),r=this.getPreviousExplicitRuleShortName();return{ruleName:this.shortRuleNameToFullName(e),idxInCallingRule:t,inRule:this.shortRuleNameToFullName(r)}},e.prototype.buildFullFollowKeyStack=function(){var e=this,t=this.RULE_STACK,r=this.RULE_OCCURRENCE_STACK;return Object(o.isEmpty)(this.LAST_EXPLICIT_RULE_STACK)||(t=Object(o.map)(this.LAST_EXPLICIT_RULE_STACK,(function(t){return e.RULE_STACK[t]})),r=Object(o.map)(this.LAST_EXPLICIT_RULE_STACK,(function(t){return e.RULE_OCCURRENCE_STACK[t]}))),Object(o.map)(t,(function(n,o){return 0===o?u:{ruleName:e.shortRuleNameToFullName(n),idxInCallingRule:r[o],inRule:e.shortRuleNameToFullName(t[o-1])}}))},e.prototype.flattenFollowSet=function(){var e=this,t=Object(o.map)(this.buildFullFollowKeyStack(),(function(t){return e.getFollowSetFromFollowKey(t)}));return Object(o.flatten)(t)},e.prototype.getFollowSetFromFollowKey=function(e){if(e===u)return[n.EOF];var t=e.ruleName+e.idxInCallingRule+i.IN+e.inRule;return this.resyncFollows[t]},e.prototype.addToResyncTokens=function(e,t){return this.tokenMatcher(e,n.EOF)||t.push(e),t},e.prototype.reSyncTo=function(e){for(var t=[],r=this.LA(1);!1===this.tokenMatcher(r,e);)r=this.SKIP_TOKEN(),this.addToResyncTokens(r,t);return Object(o.dropRight)(t)},e.prototype.attemptInRepetitionRecovery=function(e,t,r,n,o,s,i){},e.prototype.getCurrentGrammarPath=function(e,t){return{ruleStack:this.getHumanReadableRuleStack(),occurrenceStack:Object(o.cloneArr)(this.RULE_OCCURRENCE_STACK),lastTok:e,lastTokOccurrence:t}},e.prototype.getHumanReadableRuleStack=function(){var e=this;return Object(o.isEmpty)(this.LAST_EXPLICIT_RULE_STACK)?Object(o.map)(this.RULE_STACK,(function(t){return e.shortRuleNameToFullName(t)})):Object(o.map)(this.LAST_EXPLICIT_RULE_STACK,(function(t){return e.shortRuleNameToFullName(e.RULE_STACK[t])}))},e}();function d(e,t,r,o,s,i,a){var u=this.getKeyForAutomaticLookahead(o,s),c=this.firstAfterRepMap[u];if(void 0===c){var l=this.getCurrRuleFullName();c=new i(this.getGAstProductions()[l],s).startWalking(),this.firstAfterRepMap[u]=c}var h=c.token,d=c.occurrence,p=c.isEndOfRule;1===this.RULE_STACK.length&&p&&void 0===h&&(h=n.EOF,d=1),this.shouldInRepetitionRecoveryBeTried(h,d,a)&&this.tryInRepetitionRecovery(e,t,r,h)}},function(e,t,r){"use strict";r.r(t),r.d(t,"isRecognitionException",(function(){return s})),r.d(t,"MismatchedTokenException",(function(){return i})),r.d(t,"NoViableAltException",(function(){return a})),r.d(t,"NotAllInputParsedException",(function(){return u})),r.d(t,"EarlyExitException",(function(){return c}));var n=r(88),o=["MismatchedTokenException","NoViableAltException","EarlyExitException","NotAllInputParsedException"];function s(e){return Object(n.contains)(o,e.name)}function i(e,t,r){this.name="MismatchedTokenException",this.message=e,this.token=t,this.previousToken=r,this.resyncedTokens=[]}function a(e,t,r){this.name="NoViableAltException",this.message=e,this.token=t,this.previousToken=r,this.resyncedTokens=[]}function u(e,t){this.name="NotAllInputParsedException",this.message=e,this.token=t,this.resyncedTokens=[]}function c(e,t,r){this.name="EarlyExitException",this.message=e,this.token=t,this.previousToken=r,this.resyncedTokens=[]}Object.freeze(o),i.prototype=Error.prototype,a.prototype=Error.prototype,u.prototype=Error.prototype,c.prototype=Error.prototype},function(e,t,r){"use strict";r.r(t),r.d(t,"LooksAhead",(function(){return u}));var n=r(108),o=r(88),s=r(87),i=r(105),a=r(101),u=function(){function e(){}return e.prototype.initLooksAhead=function(e){this.dynamicTokensEnabled=Object(o.has)(e,"dynamicTokensEnabled")?e.dynamicTokensEnabled:s.DEFAULT_PARSER_CONFIG.dynamicTokensEnabled,this.maxLookahead=Object(o.has)(e,"maxLookahead")?e.maxLookahead:s.DEFAULT_PARSER_CONFIG.maxLookahead,this.lookAheadFuncsCache=Object(o.isES2015MapSupported)()?new Map:[],Object(o.isES2015MapSupported)()?(this.getLaFuncFromCache=this.getLaFuncFromMap,this.setLaFuncCache=this.setLaFuncCacheUsingMap):(this.getLaFuncFromCache=this.getLaFuncFromObj,this.setLaFuncCache=this.setLaFuncUsingObj)},e.prototype.preComputeLookaheadFunctions=function(e){var t=this;Object(o.forEach)(e,(function(e){t.TRACE_INIT(e.name+" Rule Lookahead",(function(){var r=Object(a.collectMethods)(e),s=r.alternation,u=r.repetition,c=r.option,l=r.repetitionMandatory,h=r.repetitionMandatoryWithSeparator,d=r.repetitionWithSeparator;Object(o.forEach)(s,(function(r){var o=0===r.idx?"":r.idx;t.TRACE_INIT(""+Object(a.getProductionDslName)(r)+o,(function(){var o=Object(n.buildLookaheadFuncForOr)(r.idx,e,r.maxLookahead||t.maxLookahead,r.hasPredicates,t.dynamicTokensEnabled,t.lookAheadBuilderForAlternatives),s=Object(i.getKeyForAutomaticLookahead)(t.fullRuleNameToShort[e.name],i.OR_IDX,r.idx);t.setLaFuncCache(s,o)}))})),Object(o.forEach)(u,(function(r){t.computeLookaheadFunc(e,r.idx,i.MANY_IDX,n.PROD_TYPE.REPETITION,r.maxLookahead,Object(a.getProductionDslName)(r))})),Object(o.forEach)(c,(function(r){t.computeLookaheadFunc(e,r.idx,i.OPTION_IDX,n.PROD_TYPE.OPTION,r.maxLookahead,Object(a.getProductionDslName)(r))})),Object(o.forEach)(l,(function(r){t.computeLookaheadFunc(e,r.idx,i.AT_LEAST_ONE_IDX,n.PROD_TYPE.REPETITION_MANDATORY,r.maxLookahead,Object(a.getProductionDslName)(r))})),Object(o.forEach)(h,(function(r){t.computeLookaheadFunc(e,r.idx,i.AT_LEAST_ONE_SEP_IDX,n.PROD_TYPE.REPETITION_MANDATORY_WITH_SEPARATOR,r.maxLookahead,Object(a.getProductionDslName)(r))})),Object(o.forEach)(d,(function(r){t.computeLookaheadFunc(e,r.idx,i.MANY_SEP_IDX,n.PROD_TYPE.REPETITION_WITH_SEPARATOR,r.maxLookahead,Object(a.getProductionDslName)(r))}))}))}))},e.prototype.computeLookaheadFunc=function(e,t,r,o,s,a){var u=this;this.TRACE_INIT(""+a+(0===t?"":t),(function(){var a=Object(n.buildLookaheadFuncForOptionalProd)(t,e,s||u.maxLookahead,u.dynamicTokensEnabled,o,u.lookAheadBuilderForOptional),c=Object(i.getKeyForAutomaticLookahead)(u.fullRuleNameToShort[e.name],r,t);u.setLaFuncCache(c,a)}))},e.prototype.lookAheadBuilderForOptional=function(e,t,r){return Object(n.buildSingleAlternativeLookaheadFunction)(e,t,r)},e.prototype.lookAheadBuilderForAlternatives=function(e,t,r,o){return Object(n.buildAlternativesLookAheadFunc)(e,t,r,o)},e.prototype.getKeyForAutomaticLookahead=function(e,t){var r=this.getLastExplicitRuleShortName();return Object(i.getKeyForAutomaticLookahead)(r,e,t)},e.prototype.getLaFuncFromCache=function(e){},e.prototype.getLaFuncFromMap=function(e){return this.lookAheadFuncsCache.get(e)},e.prototype.getLaFuncFromObj=function(e){return this.lookAheadFuncsCache[e]},e.prototype.setLaFuncCache=function(e,t){},e.prototype.setLaFuncCacheUsingMap=function(e,t){this.lookAheadFuncsCache.set(e,t)},e.prototype.setLaFuncUsingObj=function(e,t){this.lookAheadFuncsCache[e]=t},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"TreeBuilder",(function(){return u}));var n=r(104),o=r(88),s=r(116),i=r(105),a=r(87),u=function(){function e(){}return e.prototype.initTreeBuilder=function(e){if(this.LAST_EXPLICIT_RULE_STACK=[],this.CST_STACK=[],this.outputCst=Object(o.has)(e,"outputCst")?e.outputCst:a.DEFAULT_PARSER_CONFIG.outputCst,this.nodeLocationTracking=Object(o.has)(e,"nodeLocationTracking")?e.nodeLocationTracking:a.DEFAULT_PARSER_CONFIG.nodeLocationTracking,this.outputCst)if(/full/i.test(this.nodeLocationTracking))this.recoveryEnabled?(this.setNodeLocationFromToken=n.setNodeLocationFull,this.setNodeLocationFromNode=n.setNodeLocationFull,this.cstPostRule=o.NOOP,this.setInitialNodeLocation=this.setInitialNodeLocationFullRecovery):(this.setNodeLocationFromToken=o.NOOP,this.setNodeLocationFromNode=o.NOOP,this.cstPostRule=this.cstPostRuleFull,this.setInitialNodeLocation=this.setInitialNodeLocationFullRegular);else if(/onlyOffset/i.test(this.nodeLocationTracking))this.recoveryEnabled?(this.setNodeLocationFromToken=n.setNodeLocationOnlyOffset,this.setNodeLocationFromNode=n.setNodeLocationOnlyOffset,this.cstPostRule=o.NOOP,this.setInitialNodeLocation=this.setInitialNodeLocationOnlyOffsetRecovery):(this.setNodeLocationFromToken=o.NOOP,this.setNodeLocationFromNode=o.NOOP,this.cstPostRule=this.cstPostRuleOnlyOffset,this.setInitialNodeLocation=this.setInitialNodeLocationOnlyOffsetRegular);else{if(!/none/i.test(this.nodeLocationTracking))throw Error('Invalid <nodeLocationTracking> config option: "'+e.nodeLocationTracking+'"');this.setNodeLocationFromToken=o.NOOP,this.setNodeLocationFromNode=o.NOOP,this.cstPostRule=o.NOOP,this.setInitialNodeLocation=o.NOOP}else this.cstInvocationStateUpdate=o.NOOP,this.cstFinallyStateUpdate=o.NOOP,this.cstPostTerminal=o.NOOP,this.cstPostNonTerminal=o.NOOP,this.cstPostRule=o.NOOP,this.getLastExplicitRuleShortName=this.getLastExplicitRuleShortNameNoCst,this.getPreviousExplicitRuleShortName=this.getPreviousExplicitRuleShortNameNoCst,this.getLastExplicitRuleOccurrenceIndex=this.getLastExplicitRuleOccurrenceIndexNoCst,this.manyInternal=this.manyInternalNoCst,this.orInternal=this.orInternalNoCst,this.optionInternal=this.optionInternalNoCst,this.atLeastOneInternal=this.atLeastOneInternalNoCst,this.manySepFirstInternal=this.manySepFirstInternalNoCst,this.atLeastOneSepFirstInternal=this.atLeastOneSepFirstInternalNoCst},e.prototype.setInitialNodeLocationOnlyOffsetRecovery=function(e){e.location={startOffset:NaN,endOffset:NaN}},e.prototype.setInitialNodeLocationOnlyOffsetRegular=function(e){e.location={startOffset:this.LA(1).startOffset,endOffset:NaN}},e.prototype.setInitialNodeLocationFullRecovery=function(e){e.location={startOffset:NaN,startLine:NaN,startColumn:NaN,endOffset:NaN,endLine:NaN,endColumn:NaN}},e.prototype.setInitialNodeLocationFullRegular=function(e){var t=this.LA(1);e.location={startOffset:t.startOffset,startLine:t.startLine,startColumn:t.startColumn,endOffset:NaN,endLine:NaN,endColumn:NaN}},e.prototype.cstNestedInvocationStateUpdate=function(e,t){var r={name:e,fullName:this.shortRuleNameToFull[this.getLastExplicitRuleShortName()]+e,children:{}};this.setInitialNodeLocation(r),this.CST_STACK.push(r)},e.prototype.cstInvocationStateUpdate=function(e,t){this.LAST_EXPLICIT_RULE_STACK.push(this.RULE_STACK.length-1);var r={name:e,children:{}};this.setInitialNodeLocation(r),this.CST_STACK.push(r)},e.prototype.cstFinallyStateUpdate=function(){this.LAST_EXPLICIT_RULE_STACK.pop(),this.CST_STACK.pop()},e.prototype.cstNestedFinallyStateUpdate=function(){var e=this.CST_STACK.pop();this.cstPostRule(e)},e.prototype.cstPostRuleFull=function(e){var t=this.LA(0),r=e.location;r.startOffset<=t.startOffset==!0?(r.endOffset=t.endOffset,r.endLine=t.endLine,r.endColumn=t.endColumn):(r.startOffset=NaN,r.startLine=NaN,r.startColumn=NaN)},e.prototype.cstPostRuleOnlyOffset=function(e){var t=this.LA(0),r=e.location;r.startOffset<=t.startOffset==!0?r.endOffset=t.endOffset:r.startOffset=NaN},e.prototype.cstPostTerminal=function(e,t){var r=this.CST_STACK[this.CST_STACK.length-1];Object(n.addTerminalToCst)(r,t,e),this.setNodeLocationFromToken(r.location,t)},e.prototype.cstPostNonTerminal=function(e,t){if(!0!==this.isBackTracking()){var r=this.CST_STACK[this.CST_STACK.length-1];Object(n.addNoneTerminalToCst)(r,t,e),this.setNodeLocationFromNode(r.location,e.location)}},e.prototype.getBaseCstVisitorConstructor=function(){if(Object(o.isUndefined)(this.baseCstVisitorConstructor)){var e=Object(s.createBaseSemanticVisitorConstructor)(this.className,this.allRuleNames);return this.baseCstVisitorConstructor=e,e}return this.baseCstVisitorConstructor},e.prototype.getBaseCstVisitorConstructorWithDefaults=function(){if(Object(o.isUndefined)(this.baseCstVisitorWithDefaultsConstructor)){var e=Object(s.createBaseVisitorConstructorWithDefaults)(this.className,this.allRuleNames,this.getBaseCstVisitorConstructor());return this.baseCstVisitorWithDefaultsConstructor=e,e}return this.baseCstVisitorWithDefaultsConstructor},e.prototype.nestedRuleBeforeClause=function(e,t){var r;return void 0!==e.NAME?(r=e.NAME,this.nestedRuleInvocationStateUpdate(r,t),r):void 0},e.prototype.nestedAltBeforeClause=function(e,t,r,n){var o,s=this.getLastExplicitRuleShortName(),a=Object(i.getKeyForAltIndex)(s,r,t,n);return void 0!==e.NAME?(o=e.NAME,this.nestedRuleInvocationStateUpdate(o,a),{shortName:a,nestedName:o}):void 0},e.prototype.nestedRuleFinallyClause=function(e,t){var r=this.CST_STACK,o=r[r.length-1];this.nestedRuleFinallyStateUpdate();var s=r[r.length-1];Object(n.addNoneTerminalToCst)(s,t,o),this.setNodeLocationFromNode(s.location,o.location)},e.prototype.getLastExplicitRuleShortName=function(){var e=this.LAST_EXPLICIT_RULE_STACK[this.LAST_EXPLICIT_RULE_STACK.length-1];return this.RULE_STACK[e]},e.prototype.getLastExplicitRuleShortNameNoCst=function(){var e=this.RULE_STACK;return e[e.length-1]},e.prototype.getPreviousExplicitRuleShortName=function(){var e=this.LAST_EXPLICIT_RULE_STACK[this.LAST_EXPLICIT_RULE_STACK.length-2];return this.RULE_STACK[e]},e.prototype.getPreviousExplicitRuleShortNameNoCst=function(){var e=this.RULE_STACK;return e[e.length-2]},e.prototype.getLastExplicitRuleOccurrenceIndex=function(){var e=this.LAST_EXPLICIT_RULE_STACK[this.LAST_EXPLICIT_RULE_STACK.length-1];return this.RULE_OCCURRENCE_STACK[e]},e.prototype.getLastExplicitRuleOccurrenceIndexNoCst=function(){var e=this.RULE_OCCURRENCE_STACK;return e[e.length-1]},e.prototype.nestedRuleInvocationStateUpdate=function(e,t){this.RULE_OCCURRENCE_STACK.push(1),this.RULE_STACK.push(t),this.cstNestedInvocationStateUpdate(e,t)},e.prototype.nestedRuleFinallyStateUpdate=function(){this.RULE_STACK.pop(),this.RULE_OCCURRENCE_STACK.pop(),this.cstNestedFinallyStateUpdate()},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"defaultVisit",(function(){return a})),r.d(t,"createBaseSemanticVisitorConstructor",(function(){return u})),r.d(t,"createBaseVisitorConstructorWithDefaults",(function(){return c})),r.d(t,"CstVisitorDefinitionError",(function(){return n})),r.d(t,"validateVisitor",(function(){return l})),r.d(t,"validateMissingCstMethods",(function(){return h})),r.d(t,"validateRedundantMethods",(function(){return p}));var n,o=r(88),s=r(117),i=r(107);function a(e,t){for(var r=Object(o.keys)(e),n=r.length,s=0;s<n;s++)for(var i=e[r[s]],a=i.length,u=0;u<a;u++){var c=i[u];void 0===c.tokenTypeIdx&&(void 0!==c.fullName?this[c.fullName](c.children,t):this[c.name](c.children,t))}}function u(e,t){var r=function(){};return Object(s.defineNameProp)(r,e+"BaseSemantics"),(r.prototype={visit:function(e,t){if(Object(o.isArray)(e)&&(e=e[0]),!Object(o.isUndefined)(e))return void 0!==e.fullName?this[e.fullName](e.children,t):this[e.name](e.children,t)},validateVisitor:function(){var e=l(this,t);if(!Object(o.isEmpty)(e)){var r=Object(o.map)(e,(function(e){return e.msg}));throw Error("Errors Detected in CST Visitor <"+Object(s.functionName)(this.constructor)+">:\n\t"+r.join("\n\n").replace(/\n/g,"\n\t"))}}}).constructor=r,r._RULE_NAMES=t,r}function c(e,t,r){var n=function(){};Object(s.defineNameProp)(n,e+"BaseSemanticsWithDefaults");var i=Object.create(r.prototype);return Object(o.forEach)(t,(function(e){i[e]=a})),(n.prototype=i).constructor=n,n}function l(e,t){var r=h(e,t),n=p(e,t);return r.concat(n)}function h(e,t){var r=Object(o.map)(t,(function(t){if(!Object(o.isFunction)(e[t]))return{msg:"Missing visitor method: <"+t+"> on "+Object(s.functionName)(e.constructor)+" CST Visitor.",type:n.MISSING_METHOD,methodName:t}}));return Object(o.compact)(r)}!function(e){e[e.REDUNDANT_METHOD=0]="REDUNDANT_METHOD",e[e.MISSING_METHOD=1]="MISSING_METHOD"}(n||(n={}));var d=["constructor","visit","validateVisitor"];function p(e,t){var r=[];for(var a in e)i.validTermsPattern.test(a)&&Object(o.isFunction)(e[a])&&!Object(o.contains)(d,a)&&!Object(o.contains)(t,a)&&r.push({msg:"Redundant visitor method: <"+a+"> on "+Object(s.functionName)(e.constructor)+" CST Visitor\nThere is no Grammar Rule corresponding to this method's name.\nFor utility methods on visitor classes use methods names that do not match /"+i.validTermsPattern.source+"/.",type:n.REDUNDANT_METHOD,methodName:a});return r}},function(e,t,r){"use strict";r.r(t),r.d(t,"classNameFromInstance",(function(){return o})),r.d(t,"functionName",(function(){return i})),r.d(t,"defineNameProp",(function(){return a}));var n=r(88);function o(e){return i(e.constructor)}var s=/^\s*function\s*(\S*)\s*\(/;function i(e){var t=e.name;return t||e.toString().match(s)[1]}function a(e,t){var r=Object.getOwnPropertyDescriptor(e,"name");return!(!Object(n.isUndefined)(r)&&!r.configurable)&&(Object.defineProperty(e,"name",{enumerable:!1,configurable:!0,writable:!1,value:t}),!0)}},function(e,t,r){"use strict";r.r(t),r.d(t,"LexerAdapter",(function(){return o}));var n=r(87),o=function(){function e(){}return e.prototype.initLexerAdapter=function(){this.tokVector=[],this.tokVectorLength=0,this.currIdx=-1},Object.defineProperty(e.prototype,"input",{get:function(){return this.tokVector},set:function(e){if(!0!==this.selfAnalysisDone)throw Error("Missing <performSelfAnalysis> invocation at the end of the Parser's constructor.");this.reset(),this.tokVector=e,this.tokVectorLength=e.length},enumerable:!0,configurable:!0}),e.prototype.SKIP_TOKEN=function(){return this.currIdx<=this.tokVector.length-2?(this.consumeToken(),this.LA(1)):n.END_OF_FILE},e.prototype.LA=function(e){var t=this.currIdx+e;return t<0||this.tokVectorLength<=t?n.END_OF_FILE:this.tokVector[t]},e.prototype.consumeToken=function(){this.currIdx++},e.prototype.exportLexerState=function(){return this.currIdx},e.prototype.importLexerState=function(e){this.currIdx=e},e.prototype.resetLexerState=function(){this.currIdx=-1},e.prototype.moveToTerminatedState=function(){this.currIdx=this.tokVector.length-1},e.prototype.getLexerPosition=function(){return this.exportLexerState()},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"RecognizerApi",(function(){return c}));var n=r(88),o=r(113),s=r(87),i=r(106),a=r(107),u=r(91),c=function(){function e(){}return e.prototype.ACTION=function(e){return e.call(this)},e.prototype.consume=function(e,t,r){return this.consumeInternal(t,e,r)},e.prototype.subrule=function(e,t,r){return this.subruleInternal(t,e,r)},e.prototype.option=function(e,t){return this.optionInternal(t,e)},e.prototype.or=function(e,t){return this.orInternal(t,e)},e.prototype.many=function(e,t){return this.manyInternal(e,t)},e.prototype.atLeastOne=function(e,t){return this.atLeastOneInternal(e,t)},e.prototype.CONSUME=function(e,t){return this.consumeInternal(e,0,t)},e.prototype.CONSUME1=function(e,t){return this.consumeInternal(e,1,t)},e.prototype.CONSUME2=function(e,t){return this.consumeInternal(e,2,t)},e.prototype.CONSUME3=function(e,t){return this.consumeInternal(e,3,t)},e.prototype.CONSUME4=function(e,t){return this.consumeInternal(e,4,t)},e.prototype.CONSUME5=function(e,t){return this.consumeInternal(e,5,t)},e.prototype.CONSUME6=function(e,t){return this.consumeInternal(e,6,t)},e.prototype.CONSUME7=function(e,t){return this.consumeInternal(e,7,t)},e.prototype.CONSUME8=function(e,t){return this.consumeInternal(e,8,t)},e.prototype.CONSUME9=function(e,t){return this.consumeInternal(e,9,t)},e.prototype.SUBRULE=function(e,t){return this.subruleInternal(e,0,t)},e.prototype.SUBRULE1=function(e,t){return this.subruleInternal(e,1,t)},e.prototype.SUBRULE2=function(e,t){return this.subruleInternal(e,2,t)},e.prototype.SUBRULE3=function(e,t){return this.subruleInternal(e,3,t)},e.prototype.SUBRULE4=function(e,t){return this.subruleInternal(e,4,t)},e.prototype.SUBRULE5=function(e,t){return this.subruleInternal(e,5,t)},e.prototype.SUBRULE6=function(e,t){return this.subruleInternal(e,6,t)},e.prototype.SUBRULE7=function(e,t){return this.subruleInternal(e,7,t)},e.prototype.SUBRULE8=function(e,t){return this.subruleInternal(e,8,t)},e.prototype.SUBRULE9=function(e,t){return this.subruleInternal(e,9,t)},e.prototype.OPTION=function(e){return this.optionInternal(e,0)},e.prototype.OPTION1=function(e){return this.optionInternal(e,1)},e.prototype.OPTION2=function(e){return this.optionInternal(e,2)},e.prototype.OPTION3=function(e){return this.optionInternal(e,3)},e.prototype.OPTION4=function(e){return this.optionInternal(e,4)},e.prototype.OPTION5=function(e){return this.optionInternal(e,5)},e.prototype.OPTION6=function(e){return this.optionInternal(e,6)},e.prototype.OPTION7=function(e){return this.optionInternal(e,7)},e.prototype.OPTION8=function(e){return this.optionInternal(e,8)},e.prototype.OPTION9=function(e){return this.optionInternal(e,9)},e.prototype.OR=function(e){return this.orInternal(e,0)},e.prototype.OR1=function(e){return this.orInternal(e,1)},e.prototype.OR2=function(e){return this.orInternal(e,2)},e.prototype.OR3=function(e){return this.orInternal(e,3)},e.prototype.OR4=function(e){return this.orInternal(e,4)},e.prototype.OR5=function(e){return this.orInternal(e,5)},e.prototype.OR6=function(e){return this.orInternal(e,6)},e.prototype.OR7=function(e){return this.orInternal(e,7)},e.prototype.OR8=function(e){return this.orInternal(e,8)},e.prototype.OR9=function(e){return this.orInternal(e,9)},e.prototype.MANY=function(e){this.manyInternal(0,e)},e.prototype.MANY1=function(e){this.manyInternal(1,e)},e.prototype.MANY2=function(e){this.manyInternal(2,e)},e.prototype.MANY3=function(e){this.manyInternal(3,e)},e.prototype.MANY4=function(e){this.manyInternal(4,e)},e.prototype.MANY5=function(e){this.manyInternal(5,e)},e.prototype.MANY6=function(e){this.manyInternal(6,e)},e.prototype.MANY7=function(e){this.manyInternal(7,e)},e.prototype.MANY8=function(e){this.manyInternal(8,e)},e.prototype.MANY9=function(e){this.manyInternal(9,e)},e.prototype.MANY_SEP=function(e){this.manySepFirstInternal(0,e)},e.prototype.MANY_SEP1=function(e){this.manySepFirstInternal(1,e)},e.prototype.MANY_SEP2=function(e){this.manySepFirstInternal(2,e)},e.prototype.MANY_SEP3=function(e){this.manySepFirstInternal(3,e)},e.prototype.MANY_SEP4=function(e){this.manySepFirstInternal(4,e)},e.prototype.MANY_SEP5=function(e){this.manySepFirstInternal(5,e)},e.prototype.MANY_SEP6=function(e){this.manySepFirstInternal(6,e)},e.prototype.MANY_SEP7=function(e){this.manySepFirstInternal(7,e)},e.prototype.MANY_SEP8=function(e){this.manySepFirstInternal(8,e)},e.prototype.MANY_SEP9=function(e){this.manySepFirstInternal(9,e)},e.prototype.AT_LEAST_ONE=function(e){this.atLeastOneInternal(0,e)},e.prototype.AT_LEAST_ONE1=function(e){return this.atLeastOneInternal(1,e)},e.prototype.AT_LEAST_ONE2=function(e){this.atLeastOneInternal(2,e)},e.prototype.AT_LEAST_ONE3=function(e){this.atLeastOneInternal(3,e)},e.prototype.AT_LEAST_ONE4=function(e){this.atLeastOneInternal(4,e)},e.prototype.AT_LEAST_ONE5=function(e){this.atLeastOneInternal(5,e)},e.prototype.AT_LEAST_ONE6=function(e){this.atLeastOneInternal(6,e)},e.prototype.AT_LEAST_ONE7=function(e){this.atLeastOneInternal(7,e)},e.prototype.AT_LEAST_ONE8=function(e){this.atLeastOneInternal(8,e)},e.prototype.AT_LEAST_ONE9=function(e){this.atLeastOneInternal(9,e)},e.prototype.AT_LEAST_ONE_SEP=function(e){this.atLeastOneSepFirstInternal(0,e)},e.prototype.AT_LEAST_ONE_SEP1=function(e){this.atLeastOneSepFirstInternal(1,e)},e.prototype.AT_LEAST_ONE_SEP2=function(e){this.atLeastOneSepFirstInternal(2,e)},e.prototype.AT_LEAST_ONE_SEP3=function(e){this.atLeastOneSepFirstInternal(3,e)},e.prototype.AT_LEAST_ONE_SEP4=function(e){this.atLeastOneSepFirstInternal(4,e)},e.prototype.AT_LEAST_ONE_SEP5=function(e){this.atLeastOneSepFirstInternal(5,e)},e.prototype.AT_LEAST_ONE_SEP6=function(e){this.atLeastOneSepFirstInternal(6,e)},e.prototype.AT_LEAST_ONE_SEP7=function(e){this.atLeastOneSepFirstInternal(7,e)},e.prototype.AT_LEAST_ONE_SEP8=function(e){this.atLeastOneSepFirstInternal(8,e)},e.prototype.AT_LEAST_ONE_SEP9=function(e){this.atLeastOneSepFirstInternal(9,e)},e.prototype.RULE=function(e,t,r){if(void 0===r&&(r=s.DEFAULT_RULE_CONFIG),Object(n.contains)(this.definedRulesNames,e)){var o={message:i.defaultGrammarValidatorErrorProvider.buildDuplicateRuleNameError({topLevelRule:e,grammarName:this.className}),type:s.ParserDefinitionErrorType.DUPLICATE_RULE_NAME,ruleName:e};this.definitionErrors.push(o)}this.definedRulesNames.push(e);var a=this.defineRule(e,t,r);return this[e]=a,a},e.prototype.OVERRIDE_RULE=function(e,t,r){void 0===r&&(r=s.DEFAULT_RULE_CONFIG);var n=[];n=n.concat(Object(a.validateRuleIsOverridden)(e,this.definedRulesNames,this.className)),this.definitionErrors.push.apply(this.definitionErrors,n);var o=this.defineRule(e,t,r);return this[e]=o,o},e.prototype.BACKTRACK=function(e,t){return function(){this.isBackTrackingStack.push(1);var r=this.saveRecogState();try{return e.apply(this,t),!0}catch(e){if(Object(o.isRecognitionException)(e))return!1;throw e}finally{this.reloadRecogState(r),this.isBackTrackingStack.pop()}}},e.prototype.getGAstProductions=function(){return this.gastProductionsCache},e.prototype.getSerializedGastProductions=function(){return Object(u.serializeGrammar)(Object(n.values)(this.gastProductionsCache))},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"RecognizerEngine",(function(){return p}));var n=r(88),o=r(105),s=r(113),i=r(108),a=r(109),u=r(87),c=r(112),l=r(92),h=r(98),d=r(117),p=function(){function e(){}return e.prototype.initRecognizerEngine=function(e,t){if(this.className=Object(d.classNameFromInstance)(this),this.shortRuleNameToFull={},this.fullRuleNameToShort={},this.ruleShortNameIdx=256,this.tokenMatcher=h.tokenStructuredMatcherNoCategories,this.definedRulesNames=[],this.tokensMap={},this.allRuleNames=[],this.isBackTrackingStack=[],this.RULE_STACK=[],this.RULE_OCCURRENCE_STACK=[],this.gastProductionsCache={},Object(n.has)(t,"serializedGrammar"))throw Error("The Parser's configuration can no longer contain a <serializedGrammar> property.\n\tSee: https://sap.github.io/chevrotain/docs/changes/BREAKING_CHANGES.html#_6-0-0\n\tFor Further details.");if(Object(n.isArray)(e)){if(Object(n.isEmpty)(e))throw Error("A Token Vocabulary cannot be empty.\n\tNote that the first argument for the parser constructor\n\tis no longer a Token vector (since v4.0).");if("number"==typeof e[0].startOffset)throw Error("The Parser constructor no longer accepts a token vector as the first argument.\n\tSee: https://sap.github.io/chevrotain/docs/changes/BREAKING_CHANGES.html#_4-0-0\n\tFor Further details.")}if(Object(n.isArray)(e))this.tokensMap=Object(n.reduce)(e,(function(e,t){return e[t.name]=t,e}),{});else if(Object(n.has)(e,"modes")&&Object(n.every)(Object(n.flatten)(Object(n.values)(e.modes)),h.isTokenType)){var r=Object(n.flatten)(Object(n.values)(e.modes)),o=Object(n.uniq)(r);this.tokensMap=Object(n.reduce)(o,(function(e,t){return e[t.name]=t,e}),{})}else{if(!Object(n.isObject)(e))throw new Error("<tokensDictionary> argument must be An Array of Token constructors, A dictionary of Token constructors or an IMultiModeLexerDefinition");this.tokensMap=Object(n.cloneObj)(e)}this.tokensMap.EOF=l.EOF;var s=Object(n.every)(Object(n.values)(e),(function(e){return Object(n.isEmpty)(e.categoryMatches)}));this.tokenMatcher=s?h.tokenStructuredMatcherNoCategories:h.tokenStructuredMatcher,Object(h.augmentTokenTypes)(Object(n.values)(this.tokensMap))},e.prototype.defineRule=function(e,t,r){if(this.selfAnalysisDone)throw Error("Grammar rule <"+e+"> may not be defined after the 'performSelfAnalysis' method has been called'\nMake sure that all grammar rule definitions are done before 'performSelfAnalysis' is called.");var s,i=Object(n.has)(r,"resyncEnabled")?r.resyncEnabled:u.DEFAULT_RULE_CONFIG.resyncEnabled,a=Object(n.has)(r,"recoveryValueFunc")?r.recoveryValueFunc:u.DEFAULT_RULE_CONFIG.recoveryValueFunc,c=this.ruleShortNameIdx<<o.BITS_FOR_METHOD_TYPE+o.BITS_FOR_OCCURRENCE_IDX;function l(e){try{if(!0===this.outputCst){t.apply(this,e);var r=this.CST_STACK[this.CST_STACK.length-1];return this.cstPostRule(r),r}return t.apply(this,e)}catch(e){return this.invokeRuleCatch(e,i,a)}finally{this.ruleFinallyStateUpdate()}}this.ruleShortNameIdx++,this.shortRuleNameToFull[c]=e,this.fullRuleNameToShort[e]=c;return(s=function(t,r){return void 0===t&&(t=0),this.ruleInvocationStateUpdate(c,e,t),l.call(this,r)}).ruleName=e,s.originalGrammarAction=t,s},e.prototype.invokeRuleCatch=function(e,t,r){var n=1===this.RULE_STACK.length,o=t&&!this.isBackTracking()&&this.recoveryEnabled;if(Object(s.isRecognitionException)(e)){var i=e;if(o){var a,u=this.findReSyncTokenType();if(this.isInCurrentRuleReSyncSet(u))return i.resyncedTokens=this.reSyncTo(u),this.outputCst?((a=this.CST_STACK[this.CST_STACK.length-1]).recoveredNode=!0,a):r();throw this.outputCst&&((a=this.CST_STACK[this.CST_STACK.length-1]).recoveredNode=!0,i.partialCstResult=a),i}if(n)return this.moveToTerminatedState(),r();throw i}throw e},e.prototype.optionInternal=function(e,t){var r=this.getKeyForAutomaticLookahead(o.OPTION_IDX,t),n=this.nestedRuleBeforeClause(e,r);try{return this.optionInternalLogic(e,t,r)}finally{void 0!==n&&this.nestedRuleFinallyClause(r,n)}},e.prototype.optionInternalNoCst=function(e,t){var r=this.getKeyForAutomaticLookahead(o.OPTION_IDX,t);return this.optionInternalLogic(e,t,r)},e.prototype.optionInternalLogic=function(e,t,r){var n,o,s=this,i=this.getLaFuncFromCache(r);if(void 0!==e.DEF){if(n=e.DEF,void 0!==(o=e.GATE)){var a=i;i=function(){return o.call(s)&&a.call(s)}}}else n=e;if(!0===i.call(this))return n.call(this)},e.prototype.atLeastOneInternal=function(e,t){var r=this.getKeyForAutomaticLookahead(o.AT_LEAST_ONE_IDX,e),n=this.nestedRuleBeforeClause(t,r);try{return this.atLeastOneInternalLogic(e,t,r)}finally{void 0!==n&&this.nestedRuleFinallyClause(r,n)}},e.prototype.atLeastOneInternalNoCst=function(e,t){var r=this.getKeyForAutomaticLookahead(o.AT_LEAST_ONE_IDX,e);this.atLeastOneInternalLogic(e,t,r)},e.prototype.atLeastOneInternalLogic=function(e,t,r){var n,s,u=this,c=this.getLaFuncFromCache(r);if(void 0!==t.DEF){if(n=t.DEF,void 0!==(s=t.GATE)){var l=c;c=function(){return s.call(u)&&l.call(u)}}}else n=t;if(!0!==c.call(this))throw this.raiseEarlyExitException(e,i.PROD_TYPE.REPETITION_MANDATORY,t.ERR_MSG);for(var h=this.doSingleRepetition(n);!0===c.call(this)&&!0===h;)h=this.doSingleRepetition(n);this.attemptInRepetitionRecovery(this.atLeastOneInternal,[e,t],c,o.AT_LEAST_ONE_IDX,e,a.NextTerminalAfterAtLeastOneWalker)},e.prototype.atLeastOneSepFirstInternal=function(e,t){var r=this.getKeyForAutomaticLookahead(o.AT_LEAST_ONE_SEP_IDX,e),n=this.nestedRuleBeforeClause(t,r);try{this.atLeastOneSepFirstInternalLogic(e,t,r)}finally{void 0!==n&&this.nestedRuleFinallyClause(r,n)}},e.prototype.atLeastOneSepFirstInternalNoCst=function(e,t){var r=this.getKeyForAutomaticLookahead(o.AT_LEAST_ONE_SEP_IDX,e);this.atLeastOneSepFirstInternalLogic(e,t,r)},e.prototype.atLeastOneSepFirstInternalLogic=function(e,t,r){var n=this,s=t.DEF,u=t.SEP;if(!0!==this.getLaFuncFromCache(r).call(this))throw this.raiseEarlyExitException(e,i.PROD_TYPE.REPETITION_MANDATORY_WITH_SEPARATOR,t.ERR_MSG);s.call(this);for(var c=function(){return n.tokenMatcher(n.LA(1),u)};!0===this.tokenMatcher(this.LA(1),u);)this.CONSUME(u),s.call(this);this.attemptInRepetitionRecovery(this.repetitionSepSecondInternal,[e,u,c,s,a.NextTerminalAfterAtLeastOneSepWalker],c,o.AT_LEAST_ONE_SEP_IDX,e,a.NextTerminalAfterAtLeastOneSepWalker)},e.prototype.manyInternal=function(e,t){var r=this.getKeyForAutomaticLookahead(o.MANY_IDX,e),n=this.nestedRuleBeforeClause(t,r);try{return this.manyInternalLogic(e,t,r)}finally{void 0!==n&&this.nestedRuleFinallyClause(r,n)}},e.prototype.manyInternalNoCst=function(e,t){var r=this.getKeyForAutomaticLookahead(o.MANY_IDX,e);return this.manyInternalLogic(e,t,r)},e.prototype.manyInternalLogic=function(e,t,r){var n,s,i=this,u=this.getLaFuncFromCache(r);if(void 0!==t.DEF){if(n=t.DEF,void 0!==(s=t.GATE)){var c=u;u=function(){return s.call(i)&&c.call(i)}}}else n=t;for(var l=!0;!0===u.call(this)&&!0===l;)l=this.doSingleRepetition(n);this.attemptInRepetitionRecovery(this.manyInternal,[e,t],u,o.MANY_IDX,e,a.NextTerminalAfterManyWalker,l)},e.prototype.manySepFirstInternal=function(e,t){var r=this.getKeyForAutomaticLookahead(o.MANY_SEP_IDX,e),n=this.nestedRuleBeforeClause(t,r);try{this.manySepFirstInternalLogic(e,t,r)}finally{void 0!==n&&this.nestedRuleFinallyClause(r,n)}},e.prototype.manySepFirstInternalNoCst=function(e,t){var r=this.getKeyForAutomaticLookahead(o.MANY_SEP_IDX,e);this.manySepFirstInternalLogic(e,t,r)},e.prototype.manySepFirstInternalLogic=function(e,t,r){var n=this,s=t.DEF,i=t.SEP;if(!0===this.getLaFuncFromCache(r).call(this)){s.call(this);for(var u=function(){return n.tokenMatcher(n.LA(1),i)};!0===this.tokenMatcher(this.LA(1),i);)this.CONSUME(i),s.call(this);this.attemptInRepetitionRecovery(this.repetitionSepSecondInternal,[e,i,u,s,a.NextTerminalAfterManySepWalker],u,o.MANY_SEP_IDX,e,a.NextTerminalAfterManySepWalker)}},e.prototype.repetitionSepSecondInternal=function(e,t,r,n,s){for(;r();)this.CONSUME(t),n.call(this);this.attemptInRepetitionRecovery(this.repetitionSepSecondInternal,[e,t,r,n,s],r,o.AT_LEAST_ONE_SEP_IDX,e,s)},e.prototype.doSingleRepetition=function(e){var t=this.getLexerPosition();return e.call(this),this.getLexerPosition()>t},e.prototype.orInternalNoCst=function(e,t){var r=Object(n.isArray)(e)?e:e.DEF,s=this.getKeyForAutomaticLookahead(o.OR_IDX,t),i=this.getLaFuncFromCache(s).call(this,r);if(void 0!==i)return r[i].ALT.call(this);this.raiseNoAltException(t,e.ERR_MSG)},e.prototype.orInternal=function(e,t){var r=this.getKeyForAutomaticLookahead(o.OR_IDX,t),s=this.nestedRuleBeforeClause(e,r);try{var i=Object(n.isArray)(e)?e:e.DEF,a=this.getLaFuncFromCache(r).call(this,i);if(void 0!==a){var u=i[a],c=this.nestedAltBeforeClause(u,t,o.OR_IDX,a);try{return u.ALT.call(this)}finally{void 0!==c&&this.nestedRuleFinallyClause(c.shortName,c.nestedName)}}this.raiseNoAltException(t,e.ERR_MSG)}finally{void 0!==s&&this.nestedRuleFinallyClause(r,s)}},e.prototype.ruleFinallyStateUpdate=function(){if(this.RULE_STACK.pop(),this.RULE_OCCURRENCE_STACK.pop(),this.cstFinallyStateUpdate(),0===this.RULE_STACK.length&&!1===this.isAtEndOfInput()){var e=this.LA(1),t=this.errorMessageProvider.buildNotAllInputParsedMessage({firstRedundant:e,ruleName:this.getCurrRuleFullName()});this.SAVE_ERROR(new s.NotAllInputParsedException(t,e))}},e.prototype.subruleInternal=function(e,t,r){var n;try{var o=void 0!==r?r.ARGS:void 0;return n=e.call(this,t,o),this.cstPostNonTerminal(n,void 0!==r&&void 0!==r.LABEL?r.LABEL:e.ruleName),n}catch(t){this.subruleInternalError(t,r,e.ruleName)}},e.prototype.subruleInternalError=function(e,t,r){throw Object(s.isRecognitionException)(e)&&void 0!==e.partialCstResult&&(this.cstPostNonTerminal(e.partialCstResult,void 0!==t&&void 0!==t.LABEL?t.LABEL:r),delete e.partialCstResult),e},e.prototype.consumeInternal=function(e,t,r){var n;try{var o=this.LA(1);!0===this.tokenMatcher(o,e)?(this.consumeToken(),n=o):this.consumeInternalError(e,o,r)}catch(r){n=this.consumeInternalRecovery(e,t,r)}return this.cstPostTerminal(void 0!==r&&void 0!==r.LABEL?r.LABEL:e.name,n),n},e.prototype.consumeInternalError=function(e,t,r){var n,o=this.LA(0);throw n=void 0!==r&&r.ERR_MSG?r.ERR_MSG:this.errorMessageProvider.buildMismatchTokenMessage({expected:e,actual:t,previous:o,ruleName:this.getCurrRuleFullName()}),this.SAVE_ERROR(new s.MismatchedTokenException(n,t,o))},e.prototype.consumeInternalRecovery=function(e,t,r){if(!this.recoveryEnabled||"MismatchedTokenException"!==r.name||this.isBackTracking())throw r;var n=this.getFollowsForInRuleRecovery(e,t);try{return this.tryInRuleRecovery(e,n)}catch(e){throw e.name===c.IN_RULE_RECOVERY_EXCEPTION?r:e}},e.prototype.saveRecogState=function(){var e=this.errors,t=Object(n.cloneArr)(this.RULE_STACK);return{errors:e,lexerState:this.exportLexerState(),RULE_STACK:t,CST_STACK:this.CST_STACK,LAST_EXPLICIT_RULE_STACK:this.LAST_EXPLICIT_RULE_STACK}},e.prototype.reloadRecogState=function(e){this.errors=e.errors,this.importLexerState(e.lexerState),this.RULE_STACK=e.RULE_STACK},e.prototype.ruleInvocationStateUpdate=function(e,t,r){this.RULE_OCCURRENCE_STACK.push(r),this.RULE_STACK.push(e),this.cstInvocationStateUpdate(t,e)},e.prototype.isBackTracking=function(){return 0!==this.isBackTrackingStack.length},e.prototype.getCurrRuleFullName=function(){var e=this.getLastExplicitRuleShortName();return this.shortRuleNameToFull[e]},e.prototype.shortRuleNameToFullName=function(e){return this.shortRuleNameToFull[e]},e.prototype.isAtEndOfInput=function(){return this.tokenMatcher(this.LA(1),l.EOF)},e.prototype.reset=function(){this.resetLexerState(),this.isBackTrackingStack=[],this.errors=[],this.RULE_STACK=[],this.LAST_EXPLICIT_RULE_STACK=[],this.CST_STACK=[],this.RULE_OCCURRENCE_STACK=[]},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"ErrorHandler",(function(){return a}));var n=r(113),o=r(88),s=r(108),i=r(87),a=function(){function e(){}return e.prototype.initErrorHandler=function(e){this._errors=[],this.errorMessageProvider=Object(o.defaults)(e.errorMessageProvider,i.DEFAULT_PARSER_CONFIG.errorMessageProvider)},e.prototype.SAVE_ERROR=function(e){if(Object(n.isRecognitionException)(e))return e.context={ruleStack:this.getHumanReadableRuleStack(),ruleOccurrenceStack:Object(o.cloneArr)(this.RULE_OCCURRENCE_STACK)},this._errors.push(e),e;throw Error("Trying to save an Error which is not a RecognitionException")},Object.defineProperty(e.prototype,"errors",{get:function(){return Object(o.cloneArr)(this._errors)},set:function(e){this._errors=e},enumerable:!0,configurable:!0}),e.prototype.raiseEarlyExitException=function(e,t,r){for(var o=this.getCurrRuleFullName(),i=this.getGAstProductions()[o],a=Object(s.getLookaheadPathsForOptionalProd)(e,i,t,this.maxLookahead)[0],u=[],c=1;c<=this.maxLookahead;c++)u.push(this.LA(c));var l=this.errorMessageProvider.buildEarlyExitMessage({expectedIterationPaths:a,actual:u,previous:this.LA(0),customUserDescription:r,ruleName:o});throw this.SAVE_ERROR(new n.EarlyExitException(l,this.LA(1),this.LA(0)))},e.prototype.raiseNoAltException=function(e,t){for(var r=this.getCurrRuleFullName(),o=this.getGAstProductions()[r],i=Object(s.getLookaheadPathsForOr)(e,o,this.maxLookahead),a=[],u=1;u<=this.maxLookahead;u++)a.push(this.LA(u));var c=this.LA(0),l=this.errorMessageProvider.buildNoViableAltMessage({expectedPathsPerAlt:i,actual:a,previous:c,customUserDescription:t,ruleName:this.getCurrRuleFullName()});throw this.SAVE_ERROR(new n.NoViableAltException(l,this.LA(1),c))},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"ContentAssist",(function(){return s}));var n=r(109),o=r(88),s=function(){function e(){}return e.prototype.initContentAssist=function(){},e.prototype.computeContentAssist=function(e,t){var r=this.gastProductionsCache[e];if(Object(o.isUndefined)(r))throw Error("Rule ->"+e+"<- does not exist in this grammar.");return Object(n.nextPossibleTokensAfter)([r],t,this.tokenMatcher,this.maxLookahead)},e.prototype.getNextPossibleTokenTypes=function(e){var t=Object(o.first)(e.ruleStack),r=this.getGAstProductions()[t];return new n.NextAfterTokenWalker(r,e).startWalking()},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"GastRecorder",(function(){return g}));var n=r(88),o=r(91),s=r(93),i=r(98),a=r(92),u=r(87),c=r(105),l={description:"This Object indicates the Parser is during Recording Phase"};Object.freeze(l);var h=Math.pow(2,c.BITS_FOR_OCCURRENCE_IDX)-1,d=Object(a.createToken)({name:"RECORDING_PHASE_TOKEN",pattern:s.Lexer.NA});Object(i.augmentTokenTypes)([d]);var p=Object(a.createTokenInstance)(d,"This IToken indicates the Parser is in Recording Phase\n\tSee: https://sap.github.io/chevrotain/docs/guide/internals.html#grammar-recording for details",-1,-1,-1,-1,-1,-1);Object.freeze(p);var m={name:"This CSTNode indicates the Parser is in Recording Phase\n\tSee: https://sap.github.io/chevrotain/docs/guide/internals.html#grammar-recording for details",children:{}},g=function(){function e(){}return e.prototype.initGastRecorder=function(e){this.recordingProdStack=[],this.RECORDING_PHASE=!1},e.prototype.enableRecording=function(){var e=this;this.RECORDING_PHASE=!0,this.TRACE_INIT("Enable Recording",(function(){for(var t=function(t){var r=t>0?t:"";e["CONSUME"+r]=function(e,r){return this.consumeInternalRecord(e,t,r)},e["SUBRULE"+r]=function(e,r){return this.subruleInternalRecord(e,t,r)},e["OPTION"+r]=function(e){return this.optionInternalRecord(e,t)},e["OR"+r]=function(e){return this.orInternalRecord(e,t)},e["MANY"+r]=function(e){this.manyInternalRecord(t,e)},e["MANY_SEP"+r]=function(e){this.manySepFirstInternalRecord(t,e)},e["AT_LEAST_ONE"+r]=function(e){this.atLeastOneInternalRecord(t,e)},e["AT_LEAST_ONE_SEP"+r]=function(e){this.atLeastOneSepFirstInternalRecord(t,e)}},r=0;r<10;r++)t(r);e.consume=function(e,t,r){return this.consumeInternalRecord(t,e,r)},e.subrule=function(e,t,r){return this.subruleInternalRecord(t,e,r)},e.option=function(e,t){return this.optionInternalRecord(t,e)},e.or=function(e,t){return this.orInternalRecord(t,e)},e.many=function(e,t){this.manyInternalRecord(e,t)},e.atLeastOne=function(e,t){this.atLeastOneInternalRecord(e,t)},e.ACTION=e.ACTION_RECORD,e.BACKTRACK=e.BACKTRACK_RECORD,e.LA=e.LA_RECORD}))},e.prototype.disableRecording=function(){var e=this;this.RECORDING_PHASE=!1,this.TRACE_INIT("Deleting Recording methods",(function(){for(var t=0;t<10;t++){var r=t>0?t:"";delete e["CONSUME"+r],delete e["SUBRULE"+r],delete e["OPTION"+r],delete e["OR"+r],delete e["MANY"+r],delete e["MANY_SEP"+r],delete e["AT_LEAST_ONE"+r],delete e["AT_LEAST_ONE_SEP"+r]}delete e.consume,delete e.subrule,delete e.option,delete e.or,delete e.many,delete e.atLeastOne,delete e.ACTION,delete e.BACKTRACK,delete e.LA}))},e.prototype.ACTION_RECORD=function(e){},e.prototype.BACKTRACK_RECORD=function(e,t){return function(){return!0}},e.prototype.LA_RECORD=function(e){return u.END_OF_FILE},e.prototype.topLevelRuleRecord=function(e,t){try{var r=new o.Rule({definition:[],name:e});return r.name=e,this.recordingProdStack.push(r),t.call(this),this.recordingProdStack.pop(),r}catch(e){if(!0!==e.KNOWN_RECORDER_ERROR)try{e.message=e.message+'\n\t This error was thrown during the "grammar recording phase" For more info see:\n\thttps://sap.github.io/chevrotain/docs/guide/internals.html#grammar-recording'}catch(t){throw e}throw e}},e.prototype.optionInternalRecord=function(e,t){return f.call(this,o.Option,e,t)},e.prototype.atLeastOneInternalRecord=function(e,t){f.call(this,o.RepetitionMandatory,t,e)},e.prototype.atLeastOneSepFirstInternalRecord=function(e,t){f.call(this,o.RepetitionMandatoryWithSeparator,t,e,!0)},e.prototype.manyInternalRecord=function(e,t){f.call(this,o.Repetition,t,e)},e.prototype.manySepFirstInternalRecord=function(e,t){f.call(this,o.RepetitionWithSeparator,t,e,!0)},e.prototype.orInternalRecord=function(e,t){return E.call(this,e,t)},e.prototype.subruleInternalRecord=function(e,t,r){if(y(t),!e||!1===Object(n.has)(e,"ruleName")){var s=new Error("<SUBRULE"+T(t)+"> argument is invalid expecting a Parser method reference but got: <"+JSON.stringify(e)+">\n inside top level rule: <"+this.recordingProdStack[0].name+">");throw s.KNOWN_RECORDER_ERROR=!0,s}var i=Object(n.peek)(this.recordingProdStack),a=e.ruleName,u=new o.NonTerminal({idx:t,nonTerminalName:a,referencedRule:void 0});return i.definition.push(u),this.outputCst?m:l},e.prototype.consumeInternalRecord=function(e,t,r){if(y(t),!Object(i.hasShortKeyProperty)(e)){var s=new Error("<CONSUME"+T(t)+"> argument is invalid expecting a TokenType reference but got: <"+JSON.stringify(e)+">\n inside top level rule: <"+this.recordingProdStack[0].name+">");throw s.KNOWN_RECORDER_ERROR=!0,s}var a=Object(n.peek)(this.recordingProdStack),u=new o.Terminal({idx:t,terminalType:e});return a.definition.push(u),p},e}();function f(e,t,r,o){void 0===o&&(o=!1),y(r);var s=Object(n.peek)(this.recordingProdStack),i=Object(n.isFunction)(t)?t:t.DEF,a=new e({definition:[],idx:r});return Object(n.has)(t,"NAME")&&(a.name=t.NAME),o&&(a.separator=t.SEP),Object(n.has)(t,"MAX_LOOKAHEAD")&&(a.maxLookahead=t.MAX_LOOKAHEAD),this.recordingProdStack.push(a),i.call(this),s.definition.push(a),this.recordingProdStack.pop(),l}function E(e,t){var r=this;y(t);var s=Object(n.peek)(this.recordingProdStack),i=!1===Object(n.isArray)(e),a=!1===i?e:e.DEF,u=new o.Alternation({definition:[],idx:t,ignoreAmbiguities:i&&!0===e.IGNORE_AMBIGUITIES});Object(n.has)(e,"NAME")&&(u.name=e.NAME),Object(n.has)(e,"MAX_LOOKAHEAD")&&(u.maxLookahead=e.MAX_LOOKAHEAD);var c=Object(n.some)(a,(function(e){return Object(n.isFunction)(e.GATE)}));return u.hasPredicates=c,s.definition.push(u),Object(n.forEach)(a,(function(e){var t=new o.Flat({definition:[]});u.definition.push(t),Object(n.has)(e,"NAME")&&(t.name=e.NAME),Object(n.has)(e,"IGNORE_AMBIGUITIES")?t.ignoreAmbiguities=e.IGNORE_AMBIGUITIES:Object(n.has)(e,"GATE")&&(t.ignoreAmbiguities=!0),r.recordingProdStack.push(t),e.ALT.call(r),r.recordingProdStack.pop()})),l}function T(e){return 0===e?"":""+e}function y(e){if(e<0||e>h){var t=new Error("Invalid DSL Method idx value: <"+e+">\n\tIdx value must be a none negative value smaller than "+(h+1));throw t.KNOWN_RECORDER_ERROR=!0,t}}},function(e,t,r){"use strict";r.r(t),r.d(t,"PerformanceTracer",(function(){return s}));var n=r(88),o=r(87),s=function(){function e(){}return e.prototype.initPerformanceTracer=function(e){if(Object(n.has)(e,"traceInitPerf")){var t=e.traceInitPerf,r="number"==typeof t;this.traceInitMaxIdent=r?t:1/0,this.traceInitPerf=r?t>0:t}else this.traceInitMaxIdent=0,this.traceInitPerf=o.DEFAULT_PARSER_CONFIG.traceInitPerf;this.traceInitIndent=-1},e.prototype.TRACE_INIT=function(e,t){if(!0===this.traceInitPerf){this.traceInitIndent++;var r=new Array(this.traceInitIndent+1).join("\t");this.traceInitIndent<this.traceInitMaxIdent&&console.log(r+"--\x3e <"+e+">");var o=Object(n.timer)(t),s=o.time,i=o.value,a=s>10?console.warn:console.log;return this.traceInitIndent<this.traceInitMaxIdent&&a(r+"<-- <"+e+"> time: "+s+"ms"),this.traceInitIndent--,i}return t()},e}()},function(e,t,r){"use strict";r.r(t),r.d(t,"createSyntaxDiagramsCode",(function(){return o}));var n=r(86);function o(e,t){var r=void 0===t?{}:t,o=r.resourceBase,s=void 0===o?"https://unpkg.com/chevrotain@"+n.VERSION+"/diagrams/":o,i=r.css;return'\n\x3c!-- This is a generated file --\x3e\n<!DOCTYPE html>\n<meta charset="utf-8">\n<style>\n  body {\n    background-color: hsl(30, 20%, 95%)\n  }\n</style>\n\n'+("\n<link rel='stylesheet' href='"+(void 0===i?"https://unpkg.com/chevrotain@"+n.VERSION+"/diagrams/diagrams.css":i)+"'>\n")+("\n<script src='"+s+"vendor/railroad-diagrams.js'><\/script>\n<script src='"+s+"src/diagrams_builder.js'><\/script>\n<script src='"+s+"src/diagrams_behavior.js'><\/script>\n<script src='"+s+"src/main.js'><\/script>\n")+'\n<div id="diagrams" align="center"></div>    \n'+("\n<script>\n    window.serializedGrammar = "+JSON.stringify(e,null,"  ")+";\n<\/script>\n")+'\n<script>\n    var diagramsDiv = document.getElementById("diagrams");\n    main.drawDiagramsFromSerializedGrammar(serializedGrammar, diagramsDiv);\n<\/script>\n'}},function(e,t,r){"use strict";r.r(t),r.d(t,"generateParserFactory",(function(){return o})),r.d(t,"generateParserModule",(function(){return s}));var n=r(127);function o(e){var t=Object(n.genWrapperFunction)({name:e.name,rules:e.rules}),o=new Function("tokenVocabulary","config","chevrotain",t);return function(t){return o(e.tokenVocabulary,t,r(85))}}function s(e){return Object(n.genUmdModule)({name:e.name,rules:e.rules})}},function(e,t,r){"use strict";r.r(t),r.d(t,"genUmdModule",(function(){return i})),r.d(t,"genWrapperFunction",(function(){return a})),r.d(t,"genClass",(function(){return u})),r.d(t,"genAllRules",(function(){return c})),r.d(t,"genRule",(function(){return l})),r.d(t,"genTerminal",(function(){return h})),r.d(t,"genNonTerminal",(function(){return d})),r.d(t,"genAlternation",(function(){return p})),r.d(t,"genSingleAlt",(function(){return m}));var n=r(88),o=r(91),s="\n";function i(e){return"\n(function (root, factory) {\n    if (typeof define === 'function' && define.amd) {\n        // AMD. Register as an anonymous module.\n        define(['chevrotain'], factory);\n    } else if (typeof module === 'object' && module.exports) {\n        // Node. Does not work with strict CommonJS, but\n        // only CommonJS-like environments that support module.exports,\n        // like Node.\n        module.exports = factory(require('chevrotain'));\n    } else {\n        // Browser globals (root is window)\n        root.returnExports = factory(root.b);\n    }\n}(typeof self !== 'undefined' ? self : this, function (chevrotain) {\n\n"+u(e)+"\n    \nreturn {\n    "+e.name+": "+e.name+" \n}\n}));\n"}function a(e){return"    \n"+u(e)+"\nreturn new "+e.name+"(tokenVocabulary, config)    \n"}function u(e){return"\nfunction "+e.name+"(tokenVocabulary, config) {\n    // invoke super constructor\n    // No support for embedded actions currently, so we can 'hardcode'\n    // The use of CstParser.\n    chevrotain.CstParser.call(this, tokenVocabulary, config)\n\n    const $ = this\n\n    "+c(e.rules)+"\n\n    // very important to call this after all the rules have been defined.\n    // otherwise the parser may not work correctly as it will lack information\n    // derived during the self analysis phase.\n    this.performSelfAnalysis(this)\n}\n\n// inheritance as implemented in javascript in the previous decade... :(\n"+e.name+".prototype = Object.create(chevrotain.CstParser.prototype)\n"+e.name+".prototype.constructor = "+e.name+"    \n    "}function c(e){return Object(n.map)(e,(function(e){return l(e,1)})).join("\n")}function l(e,t){var r=T(t,'$.RULE("'+e.name+'", function() {')+s;return r+=E(e.definition,t+1),r+=T(t+1,"})")+s}function h(e,t){var r=e.terminalType.name;return T(t,"$.CONSUME"+e.idx+"(this.tokensMap."+r+")"+s)}function d(e,t){return T(t,"$.SUBRULE"+e.idx+"($."+e.nonTerminalName+")"+s)}function p(e,t){var r=T(t,"$.OR"+e.idx+"([")+s;return r+=Object(n.map)(e.definition,(function(e){return m(e,t+1)})).join(",\n"),r+=s+T(t,"])\n")}function m(e,t){var r=T(t,"{")+s;return e.name&&(r+=T(t+1,'NAME: "'+e.name+'",')+s),r+=T(t+1,"ALT: function() {")+s,r+=E(e.definition,t+1),r+=T(t+1,"}")+s,r+=T(t,"}")}function g(e,t,r){var n=T(r,"$."+(e+t.idx)+"(");return t.name||t.separator?(n+="{\n",t.name&&(n+=T(r+1,'NAME: "'+t.name+'"')+","+s),t.separator&&(n+=T(r+1,"SEP: this.tokensMap."+t.separator.name)+","+s),n+="DEF: "+f(t.definition,r+2)+s,n+=T(r,"}")+s):n+=f(t.definition,r+1),n+=T(r,")")+s}function f(e,t){var r="function() {\n";return r+=E(e,t),r+=T(t,"}")+s}function E(e,t){var r="";return Object(n.forEach)(e,(function(e){r+=function(e,t){if(e instanceof o.NonTerminal)return d(e,t);if(e instanceof o.Option)return g("OPTION",e,t);if(e instanceof o.RepetitionMandatory)return g("AT_LEAST_ONE",e,t);if(e instanceof o.RepetitionMandatoryWithSeparator)return g("AT_LEAST_ONE_SEP",e,t);if(e instanceof o.RepetitionWithSeparator)return g("MANY_SEP",e,t);if(e instanceof o.Repetition)return g("MANY",e,t);if(e instanceof o.Alternation)return p(e,t);if(e instanceof o.Terminal)return h(e,t);if(e instanceof o.Flat)return E(e.definition,t);throw Error("non exhaustive match")}(e,t+1)})),r}function T(e,t){return Array(4*e+1).join(" ")+t}},function(e,t,r){"use strict";t.__esModule=!0,t.buildTimesOpAst=t.buildStringAst=t.buildRowRangeAst=t.buildProcedureAst=t.buildPowerOpAst=t.buildPlusUnaryOpAst=t.buildPlusOpAst=t.buildPercentOpAst=t.buildParsingErrorAst=t.buildParenthesisAst=t.buildNumberAst=t.buildNotEqualOpAst=t.buildNamedExpressionAst=t.buildMinusUnaryOpAst=t.buildMinusOpAst=t.buildLessThanOrEqualOpAst=t.buildLessThanOpAst=t.buildGreaterThanOrEqualOpAst=t.buildGreaterThanOpAst=t.buildErrorWithRawInputAst=t.buildEqualsOpAst=t.buildEmptyArgAst=t.buildDivOpAst=t.buildConcatenateOpAst=t.buildColumnRangeAst=t.buildCellReferenceAst=t.buildCellRangeAst=t.buildCellErrorAst=t.buildArrayAst=t.RangeSheetReferenceType=t.ParsingErrorType=t.AstNodeType=void 0,t.imageWithWhitespace=function(e,t){return(null!=t?t:"")+e},t.parsingError=void 0;var n=r(74);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */var o,s,i;t.parsingError=(e,t)=>({type:e,message:t}),t.ParsingErrorType=o,function(e){e.LexingError="LexingError",e.ParserError="ParsingError",e.StaticOffsetError="StaticOffsetError",e.StaticOffsetOutOfRangeError="StaticOffsetOutOfRangeError",e.RangeOffsetNotAllowed="RangeOffsetNotAllowed"}(o||(t.ParsingErrorType=o={})),t.AstNodeType=s,function(e){e.EMPTY="EMPTY",e.NUMBER="NUMBER",e.STRING="STRING",e.MINUS_UNARY_OP="MINUS_UNARY_OP",e.PLUS_UNARY_OP="PLUS_UNARY_OP",e.PERCENT_OP="PERCENT_OP",e.CONCATENATE_OP="CONCATENATE_OP",e.EQUALS_OP="EQUALS_OP",e.NOT_EQUAL_OP="NOT_EQUAL_OP",e.GREATER_THAN_OP="GREATER_THAN_OP",e.LESS_THAN_OP="LESS_THAN_OP",e.GREATER_THAN_OR_EQUAL_OP="GREATER_THAN_OR_EQUAL_OP",e.LESS_THAN_OR_EQUAL_OP="LESS_THAN_OR_EQUAL_OP",e.PLUS_OP="PLUS_OP",e.MINUS_OP="MINUS_OP",e.TIMES_OP="TIMES_OP",e.DIV_OP="DIV_OP",e.POWER_OP="POWER_OP",e.FUNCTION_CALL="FUNCTION_CALL",e.NAMED_EXPRESSION="NAMED_EXPRESSION",e.PARENTHESIS="PARENTHESES",e.CELL_REFERENCE="CELL_REFERENCE",e.CELL_RANGE="CELL_RANGE",e.COLUMN_RANGE="COLUMN_RANGE",e.ROW_RANGE="ROW_RANGE",e.ERROR="ERROR",e.ERROR_WITH_RAW_INPUT="ERROR_WITH_RAW_INPUT",e.ARRAY="ARRAY"}(s||(t.AstNodeType=s={})),t.RangeSheetReferenceType=i,function(e){e[e.RELATIVE=0]="RELATIVE",e[e.START_ABSOLUTE=1]="START_ABSOLUTE",e[e.BOTH_ABSOLUTE=2]="BOTH_ABSOLUTE"}(i||(t.RangeSheetReferenceType=i={}));t.buildEmptyArgAst=e=>({type:s.EMPTY,leadingWhitespace:null==e?void 0:e.image});t.buildNumberAst=(e,t)=>({type:s.NUMBER,value:e,leadingWhitespace:null==t?void 0:t.image});t.buildStringAst=e=>{var t;return{type:s.STRING,value:e.image.slice(1,-1),leadingWhitespace:null===(t=e.leadingWhitespace)||void 0===t?void 0:t.image}};t.buildCellReferenceAst=(e,t)=>({type:s.CELL_REFERENCE,reference:e,leadingWhitespace:null==t?void 0:t.image});t.buildCellRangeAst=(e,t,r,n)=>(a(e,t,r),{type:s.CELL_RANGE,start:e,end:t,sheetReferenceType:r,leadingWhitespace:n});t.buildColumnRangeAst=(e,t,r,n)=>(a(e,t,r),{type:s.COLUMN_RANGE,start:e,end:t,sheetReferenceType:r,leadingWhitespace:null==n?void 0:n.image});t.buildRowRangeAst=(e,t,r,n)=>(a(e,t,r),{type:s.ROW_RANGE,start:e,end:t,sheetReferenceType:r,leadingWhitespace:null==n?void 0:n.image});t.buildConcatenateOpAst=(e,t,r)=>({type:s.CONCATENATE_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildEqualsOpAst=(e,t,r)=>({type:s.EQUALS_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildNotEqualOpAst=(e,t,r)=>({type:s.NOT_EQUAL_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildGreaterThanOpAst=(e,t,r)=>({type:s.GREATER_THAN_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildLessThanOpAst=(e,t,r)=>({type:s.LESS_THAN_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildGreaterThanOrEqualOpAst=(e,t,r)=>({type:s.GREATER_THAN_OR_EQUAL_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildLessThanOrEqualOpAst=(e,t,r)=>({type:s.LESS_THAN_OR_EQUAL_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildPlusOpAst=(e,t,r)=>({type:s.PLUS_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildMinusOpAst=(e,t,r)=>({type:s.MINUS_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildTimesOpAst=(e,t,r)=>({type:s.TIMES_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildDivOpAst=(e,t,r)=>({type:s.DIV_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildPowerOpAst=(e,t,r)=>({type:s.POWER_OP,left:e,right:t,leadingWhitespace:null==r?void 0:r.image});t.buildMinusUnaryOpAst=(e,t)=>({type:s.MINUS_UNARY_OP,value:e,leadingWhitespace:null==t?void 0:t.image});t.buildPlusUnaryOpAst=(e,t)=>({type:s.PLUS_UNARY_OP,value:e,leadingWhitespace:null==t?void 0:t.image});t.buildPercentOpAst=(e,t)=>({type:s.PERCENT_OP,value:e,leadingWhitespace:null==t?void 0:t.image});t.buildProcedureAst=(e,t,r,n,o)=>({type:s.FUNCTION_CALL,procedureName:e,args:t,leadingWhitespace:null==r?void 0:r.image,internalWhitespace:null==n?void 0:n.image,hyperlink:o});t.buildArrayAst=(e,t,r)=>({type:s.ARRAY,args:e,leadingWhitespace:null==t?void 0:t.image,internalWhitespace:null==r?void 0:r.image});t.buildNamedExpressionAst=(e,t)=>({type:s.NAMED_EXPRESSION,expressionName:e,leadingWhitespace:null==t?void 0:t.image});t.buildParenthesisAst=(e,t,r)=>({type:s.PARENTHESIS,expression:e,leadingWhitespace:null==t?void 0:t.image,internalWhitespace:null==r?void 0:r.image});t.buildCellErrorAst=(e,t)=>({type:s.ERROR,error:e,leadingWhitespace:null==t?void 0:t.image});t.buildErrorWithRawInputAst=(e,t,r)=>({type:s.ERROR_WITH_RAW_INPUT,error:t,rawInput:e,leadingWhitespace:null==r?void 0:r.image});function a(e,t,r){if(void 0!==e.sheet&&void 0===t.sheet||void 0===e.sheet&&void 0!==t.sheet)throw new Error("Start address inconsistent with end address");if(void 0===e.sheet&&r!==i.RELATIVE||void 0!==e.sheet&&r===i.RELATIVE)throw new Error("Sheet address inconsistent with sheet reference type")}t.buildParsingErrorAst=()=>({type:s.ERROR,error:n.CellError.parsingError()})},function(e,t,r){"use strict";t.__esModule=!0,t.binaryOpTokenMap=void 0;var n=r(128);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */const o={[n.AstNodeType.PLUS_OP]:"+",[n.AstNodeType.MINUS_OP]:"-",[n.AstNodeType.TIMES_OP]:"*",[n.AstNodeType.DIV_OP]:"/",[n.AstNodeType.CONCATENATE_OP]:"&",[n.AstNodeType.POWER_OP]:"^",[n.AstNodeType.EQUALS_OP]:"=",[n.AstNodeType.NOT_EQUAL_OP]:"<>",[n.AstNodeType.GREATER_THAN_OP]:">",[n.AstNodeType.GREATER_THAN_OR_EQUAL_OP]:">=",[n.AstNodeType.LESS_THAN_OP]:"<",[n.AstNodeType.LESS_THAN_OR_EQUAL_OP]:"<="};t.binaryOpTokenMap=o},function(e,t,r){"use strict";t.__esModule=!0,t.doesContainFunctions=t.Cache=void 0;var n=r(78);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.Cache=class{constructor(e){this.functionRegistry=e,this.cache=new Map}set(e,t){const r=((e,t,r,n)=>({ast:e,relativeDependencies:t,hasVolatileFunction:r,hasStructuralChangeFunction:n}))(t,(0,n.collectDependencies)(t,this.functionRegistry),o(t,this.functionRegistry.isFunctionVolatile),o(t,this.functionRegistry.isFunctionDependentOnSheetStructureChange));return this.cache.set(e,r),r}get(e){return this.cache.get(e)}maybeSetAndThenGet(e,t){const r=this.cache.get(e);return void 0!==r?r.ast:(this.set(e,t),t)}};const o=(e,t)=>{switch(e.type){case n.AstNodeType.EMPTY:case n.AstNodeType.NUMBER:case n.AstNodeType.STRING:case n.AstNodeType.ERROR:case n.AstNodeType.ERROR_WITH_RAW_INPUT:case n.AstNodeType.CELL_REFERENCE:case n.AstNodeType.CELL_RANGE:case n.AstNodeType.COLUMN_RANGE:case n.AstNodeType.ROW_RANGE:case n.AstNodeType.NAMED_EXPRESSION:return!1;case n.AstNodeType.PERCENT_OP:case n.AstNodeType.PLUS_UNARY_OP:case n.AstNodeType.MINUS_UNARY_OP:return o(e.value,t);case n.AstNodeType.CONCATENATE_OP:case n.AstNodeType.EQUALS_OP:case n.AstNodeType.NOT_EQUAL_OP:case n.AstNodeType.LESS_THAN_OP:case n.AstNodeType.GREATER_THAN_OP:case n.AstNodeType.LESS_THAN_OR_EQUAL_OP:case n.AstNodeType.GREATER_THAN_OR_EQUAL_OP:case n.AstNodeType.MINUS_OP:case n.AstNodeType.PLUS_OP:case n.AstNodeType.TIMES_OP:case n.AstNodeType.DIV_OP:case n.AstNodeType.POWER_OP:return o(e.left,t)||o(e.right,t);case n.AstNodeType.PARENTHESIS:return o(e.expression,t);case n.AstNodeType.FUNCTION_CALL:return!!t(e.procedureName)||e.args.some(e=>o(e,t));case n.AstNodeType.ARRAY:return e.args.some(e=>e.some(e=>o(e,t)))}};t.doesContainFunctions=o},function(e,t,r){"use strict";t.__esModule=!0,t.FormulaParser=t.FormulaLexer=void 0,r(2);var n=r(85),o=r(74),s=r(132),i=r(79),a=r(128),u=r(80),c=r(133);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class l extends n.EmbeddedActionsParser{constructor(e,t){super(e.allTokens,{outputCst:!1,maxLookahead:7}),this.booleanExpressionOrEmpty=this.RULE("booleanExpressionOrEmpty",()=>this.OR([{ALT:()=>this.SUBRULE(this.booleanExpression)},{ALT:(0,n.EMPTY_ALT)((0,a.buildEmptyArgAst)())}])),this.procedureExpression=this.RULE("procedureExpression",()=>{var e;const t=this.CONSUME(c.ProcedureName),r=t.image.toUpperCase().slice(0,-1),n=null!==(e=this.lexerConfig.functionMapping[r])&&void 0!==e?e:r,o=[];let s=this.SUBRULE(this.booleanExpressionOrEmpty);this.MANY(()=>{var e;const t=this.CONSUME(this.lexerConfig.ArgSeparator);s.type===a.AstNodeType.EMPTY&&(s.leadingWhitespace=null===(e=t.leadingWhitespace)||void 0===e?void 0:e.image),o.push(s),s=this.SUBRULE2(this.booleanExpressionOrEmpty)}),o.push(s),1===o.length&&o[0].type===a.AstNodeType.EMPTY&&(o.length=0);const i=this.CONSUME(c.RParen);return(0,a.buildProcedureAst)(n,o,t.leadingWhitespace,i.leadingWhitespace)}),this.namedExpressionExpression=this.RULE("namedExpressionExpression",()=>{const e=this.CONSUME(c.NamedExpression);return(0,a.buildNamedExpressionAst)(e.image,e.leadingWhitespace)}),this.offsetProcedureExpression=this.RULE("offsetProcedureExpression",()=>{const e=[];return this.CONSUME(this.lexerConfig.OffsetProcedureName),this.CONSUME(c.LParen),this.MANY_SEP({SEP:this.lexerConfig.ArgSeparator,DEF:()=>{e.push(this.SUBRULE(this.booleanExpression))}}),this.CONSUME(c.RParen),this.handleOffsetHeuristic(e)}),this.columnRangeExpression=this.RULE("columnRangeExpression",()=>{const e=this.CONSUME(c.ColumnRange),[t,r]=e.image.split(":"),n=this.ACTION(()=>(0,i.columnAddressFromString)(this.sheetMapping,t,this.formulaAddress)),s=this.ACTION(()=>(0,i.columnAddressFromString)(this.sheetMapping,r,this.formulaAddress));if(void 0===n||void 0===s)return(0,a.buildCellErrorAst)(new o.CellError(o.ErrorType.REF));if(n.exceedsSheetSizeLimits(this.lexerConfig.maxColumns)||s.exceedsSheetSizeLimits(this.lexerConfig.maxColumns))return(0,a.buildErrorWithRawInputAst)(e.image,new o.CellError(o.ErrorType.NAME),e.leadingWhitespace);if(void 0===n.sheet&&void 0!==s.sheet)return this.parsingError(a.ParsingErrorType.ParserError,"Malformed range expression");const{firstEnd:u,secondEnd:h,sheetRefType:d}=l.fixSheetIdsForRangeEnds(n,s);return(0,a.buildColumnRangeAst)(u,h,d,e.leadingWhitespace)}),this.rowRangeExpression=this.RULE("rowRangeExpression",()=>{const e=this.CONSUME(c.RowRange),[t,r]=e.image.split(":"),n=this.ACTION(()=>(0,i.rowAddressFromString)(this.sheetMapping,t,this.formulaAddress)),s=this.ACTION(()=>(0,i.rowAddressFromString)(this.sheetMapping,r,this.formulaAddress));if(void 0===n||void 0===s)return(0,a.buildCellErrorAst)(new o.CellError(o.ErrorType.REF));if(n.exceedsSheetSizeLimits(this.lexerConfig.maxRows)||s.exceedsSheetSizeLimits(this.lexerConfig.maxRows))return(0,a.buildErrorWithRawInputAst)(e.image,new o.CellError(o.ErrorType.NAME),e.leadingWhitespace);if(void 0===n.sheet&&void 0!==s.sheet)return this.parsingError(a.ParsingErrorType.ParserError,"Malformed range expression");const{firstEnd:u,secondEnd:h,sheetRefType:d}=l.fixSheetIdsForRangeEnds(n,s);return(0,a.buildRowRangeAst)(u,h,d,e.leadingWhitespace)}),this.cellReference=this.RULE("cellReference",()=>{const e=this.CONSUME(c.CellReference),t=this.ACTION(()=>(0,i.cellAddressFromString)(this.sheetMapping,e.image,this.formulaAddress));return void 0===t?(0,a.buildErrorWithRawInputAst)(e.image,new o.CellError(o.ErrorType.REF),e.leadingWhitespace):t.exceedsSheetSizeLimits(this.lexerConfig.maxColumns,this.lexerConfig.maxRows)?(0,a.buildErrorWithRawInputAst)(e.image,new o.CellError(o.ErrorType.NAME),e.leadingWhitespace):(0,a.buildCellReferenceAst)(t,e.leadingWhitespace)}),this.endRangeReference=this.RULE("endRangeReference",e=>{var t;const r=this.CONSUME(c.CellReference),n=this.ACTION(()=>(0,i.cellAddressFromString)(this.sheetMapping,e.image,this.formulaAddress)),s=this.ACTION(()=>(0,i.cellAddressFromString)(this.sheetMapping,r.image,this.formulaAddress));return void 0===n||void 0===s?this.ACTION(()=>(0,a.buildErrorWithRawInputAst)(`${e.image}:${r.image}`,new o.CellError(o.ErrorType.REF),e.leadingWhitespace)):n.exceedsSheetSizeLimits(this.lexerConfig.maxColumns,this.lexerConfig.maxRows)||s.exceedsSheetSizeLimits(this.lexerConfig.maxColumns,this.lexerConfig.maxRows)?this.ACTION(()=>(0,a.buildErrorWithRawInputAst)(`${e.image}:${r.image}`,new o.CellError(o.ErrorType.NAME),e.leadingWhitespace)):this.buildCellRange(n,s,null===(t=e.leadingWhitespace)||void 0===t?void 0:t.image)}),this.endOfRangeExpression=this.RULE("endOfRangeExpression",e=>this.OR([{ALT:()=>this.SUBRULE(this.endRangeReference,{ARGS:[e]})},{ALT:()=>{var t;const r=this.SUBRULE(this.offsetProcedureExpression),n=this.ACTION(()=>(0,i.cellAddressFromString)(this.sheetMapping,e.image,this.formulaAddress));return void 0===n?(0,a.buildCellErrorAst)(new o.CellError(o.ErrorType.REF)):r.type===a.AstNodeType.CELL_REFERENCE?this.buildCellRange(n,r.reference,null===(t=e.leadingWhitespace)||void 0===t?void 0:t.image):this.parsingError(a.ParsingErrorType.RangeOffsetNotAllowed,"Range offset not allowed here")}}])),this.cellRangeExpression=this.RULE("cellRangeExpression",()=>{const e=this.CONSUME(c.CellReference);return this.CONSUME2(c.RangeSeparator),this.SUBRULE(this.endOfRangeExpression,{ARGS:[e]})}),this.endRangeWithOffsetStartReference=this.RULE("endRangeWithOffsetStartReference",e=>{const t=this.CONSUME(c.CellReference),r=this.ACTION(()=>(0,i.cellAddressFromString)(this.sheetMapping,t.image,this.formulaAddress));return void 0===r?this.ACTION(()=>(0,a.buildCellErrorAst)(new o.CellError(o.ErrorType.REF))):this.buildCellRange(e.reference,r,e.leadingWhitespace)}),this.endOfRangeWithOffsetStartExpression=this.RULE("endOfRangeWithOffsetStartExpression",e=>this.OR([{ALT:()=>this.SUBRULE(this.endRangeWithOffsetStartReference,{ARGS:[e]})},{ALT:()=>{const t=this.SUBRULE(this.offsetProcedureExpression);return t.type===a.AstNodeType.CELL_REFERENCE?this.buildCellRange(e.reference,t.reference,e.leadingWhitespace):this.parsingError(a.ParsingErrorType.RangeOffsetNotAllowed,"Range offset not allowed here")}}])),this.offsetExpression=this.RULE("offsetExpression",()=>{const e=this.SUBRULE(this.offsetProcedureExpression);let t;return this.OPTION(()=>{this.CONSUME(c.RangeSeparator),t=e.type===a.AstNodeType.CELL_RANGE?this.parsingError(a.ParsingErrorType.RangeOffsetNotAllowed,"Range offset not allowed here"):this.SUBRULE(this.endOfRangeWithOffsetStartExpression,{ARGS:[e]})}),void 0!==t?t:e}),this.insideArrayExpression=this.RULE("insideArrayExpression",()=>{const e=[[]];return e[e.length-1].push(this.SUBRULE(this.booleanExpression)),this.MANY(()=>{this.OR([{ALT:()=>{this.CONSUME(this.lexerConfig.ArrayColSeparator),e[e.length-1].push(this.SUBRULE2(this.booleanExpression))}},{ALT:()=>{this.CONSUME(this.lexerConfig.ArrayRowSeparator),e.push([]),e[e.length-1].push(this.SUBRULE3(this.booleanExpression))}}])}),(0,a.buildArrayAst)(e)}),this.parenthesisExpression=this.RULE("parenthesisExpression",()=>{const e=this.CONSUME(c.LParen),t=this.SUBRULE(this.booleanExpression),r=this.CONSUME(c.RParen);return(0,a.buildParenthesisAst)(t,e.leadingWhitespace,r.leadingWhitespace)}),this.arrayExpression=this.RULE("arrayExpression",()=>this.OR([{ALT:()=>{const e=this.CONSUME(c.ArrayLParen),t=this.SUBRULE(this.insideArrayExpression),r=this.CONSUME(c.ArrayRParen);return(0,a.buildArrayAst)(t.args,e.leadingWhitespace,r.leadingWhitespace)}},{ALT:()=>this.SUBRULE(this.parenthesisExpression)}])),this.numericStringToNumber=e=>{const t=e.replace(this.lexerConfig.decimalSeparator,".");return Number(t)},this.positiveAtomicExpression=this.RULE("positiveAtomicExpression",()=>{var e;return this.OR(null!==(e=this.atomicExpCache)&&void 0!==e?e:this.atomicExpCache=[{ALT:()=>this.SUBRULE(this.arrayExpression)},{ALT:()=>this.SUBRULE(this.cellRangeExpression)},{ALT:()=>this.SUBRULE(this.columnRangeExpression)},{ALT:()=>this.SUBRULE(this.rowRangeExpression)},{ALT:()=>this.SUBRULE(this.offsetExpression)},{ALT:()=>this.SUBRULE(this.cellReference)},{ALT:()=>this.SUBRULE(this.procedureExpression)},{ALT:()=>this.SUBRULE(this.namedExpressionExpression)},{ALT:()=>{const e=this.CONSUME(this.lexerConfig.NumberLiteral);return(0,a.buildNumberAst)(this.numericStringToNumber(e.image),e.leadingWhitespace)}},{ALT:()=>{const e=this.CONSUME(c.StringLiteral);return(0,a.buildStringAst)(e)}},{ALT:()=>{const e=this.CONSUME(c.ErrorLiteral),t=e.image.toUpperCase(),r=this.lexerConfig.errorMapping[t];return r?(0,a.buildCellErrorAst)(new o.CellError(r),e.leadingWhitespace):this.parsingError(a.ParsingErrorType.ParserError,"Unknown error literal")}}])}),this.rightUnaryOpAtomicExpression=this.RULE("rightUnaryOpAtomicExpression",()=>{const e=this.SUBRULE(this.positiveAtomicExpression),t=this.OPTION(()=>this.CONSUME(c.PercentOp));return t?(0,a.buildPercentOpAst)(e,t.leadingWhitespace):e}),this.atomicExpression=this.RULE("atomicExpression",()=>this.OR([{ALT:()=>{const e=this.CONSUME(c.AdditionOp),t=this.SUBRULE(this.atomicExpression);return(0,n.tokenMatcher)(e,c.PlusOp)?(0,a.buildPlusUnaryOpAst)(t,e.leadingWhitespace):(0,n.tokenMatcher)(e,c.MinusOp)?(0,a.buildMinusUnaryOpAst)(t,e.leadingWhitespace):(this.customParsingError=(0,a.parsingError)(a.ParsingErrorType.ParserError,"Mismatched token type"),this.customParsingError)}},{ALT:()=>this.SUBRULE2(this.rightUnaryOpAtomicExpression)}])),this.powerExpression=this.RULE("powerExpression",()=>{let e=this.SUBRULE(this.atomicExpression);return this.MANY(()=>{const t=this.CONSUME(c.PowerOp),r=this.SUBRULE2(this.atomicExpression);(0,n.tokenMatcher)(t,c.PowerOp)?e=(0,a.buildPowerOpAst)(e,r,t.leadingWhitespace):this.ACTION(()=>{throw Error("Operator not supported")})}),e}),this.multiplicationExpression=this.RULE("multiplicationExpression",()=>{let e=this.SUBRULE(this.powerExpression);return this.MANY(()=>{const t=this.CONSUME(c.MultiplicationOp),r=this.SUBRULE2(this.powerExpression);(0,n.tokenMatcher)(t,c.TimesOp)?e=(0,a.buildTimesOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.DivOp)?e=(0,a.buildDivOpAst)(e,r,t.leadingWhitespace):this.ACTION(()=>{throw Error("Operator not supported")})}),e}),this.additionExpression=this.RULE("additionExpression",()=>{let e=this.SUBRULE(this.multiplicationExpression);return this.MANY(()=>{const t=this.CONSUME(c.AdditionOp),r=this.SUBRULE2(this.multiplicationExpression);(0,n.tokenMatcher)(t,c.PlusOp)?e=(0,a.buildPlusOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.MinusOp)?e=(0,a.buildMinusOpAst)(e,r,t.leadingWhitespace):this.ACTION(()=>{throw Error("Operator not supported")})}),e}),this.concatenateExpression=this.RULE("concatenateExpression",()=>{let e=this.SUBRULE(this.additionExpression);return this.MANY(()=>{const t=this.CONSUME(c.ConcatenateOp),r=this.SUBRULE2(this.additionExpression);e=(0,a.buildConcatenateOpAst)(e,r,t.leadingWhitespace)}),e}),this.booleanExpression=this.RULE("booleanExpression",()=>{let e=this.SUBRULE(this.concatenateExpression);return this.MANY(()=>{const t=this.CONSUME(c.BooleanOp),r=this.SUBRULE2(this.concatenateExpression);(0,n.tokenMatcher)(t,c.EqualsOp)?e=(0,a.buildEqualsOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.NotEqualOp)?e=(0,a.buildNotEqualOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.GreaterThanOp)?e=(0,a.buildGreaterThanOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.LessThanOp)?e=(0,a.buildLessThanOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.GreaterThanOrEqualOp)?e=(0,a.buildGreaterThanOrEqualOpAst)(e,r,t.leadingWhitespace):(0,n.tokenMatcher)(t,c.LessThanOrEqualOp)?e=(0,a.buildLessThanOrEqualOpAst)(e,r,t.leadingWhitespace):this.ACTION(()=>{throw Error("Operator not supported")})}),e}),this.formula=this.RULE("formula",()=>(this.CONSUME(c.EqualsOp),this.SUBRULE(this.booleanExpression))),this.lexerConfig=e,this.sheetMapping=t,this.formulaAddress=(0,o.simpleCellAddress)(0,0,0),this.performSelfAnalysis()}parseFromTokens(e,t){this.input=e;let r=this.formulaWithContext(t),n=[];return this.customParsingError&&n.push(this.customParsingError),n=n.concat(this.errors.map(e=>({type:a.ParsingErrorType.ParserError,message:e.message}))),n.length>0&&(r=(0,a.buildParsingErrorAst)()),{ast:r,errors:n}}reset(){super.reset(),this.customParsingError=void 0}formulaWithContext(e){return this.formulaAddress=e,this.formula()}buildCellRange(e,t,r){if(void 0===e.sheet&&void 0!==t.sheet)return this.parsingError(a.ParsingErrorType.ParserError,"Malformed range expression");const{firstEnd:n,secondEnd:o,sheetRefType:s}=l.fixSheetIdsForRangeEnds(e,t);return(0,a.buildCellRangeAst)(n,o,s,r)}static fixSheetIdsForRangeEnds(e,t){const r=l.rangeSheetReferenceType(e.sheet,t.sheet);return{firstEnd:e,secondEnd:void 0!==e.sheet&&void 0===t.sheet?t.withSheet(e.sheet):t,sheetRefType:r}}handleOffsetHeuristic(e){const t=e[0];if(t.type!==a.AstNodeType.CELL_REFERENCE)return this.parsingError(a.ParsingErrorType.StaticOffsetError,"First argument to OFFSET is not a reference");const r=e[1];let n;if(r.type===a.AstNodeType.NUMBER&&Number.isInteger(r.value))n=r.value;else if(r.type===a.AstNodeType.PLUS_UNARY_OP&&r.value.type===a.AstNodeType.NUMBER&&Number.isInteger(r.value.value))n=r.value.value;else{if(r.type!==a.AstNodeType.MINUS_UNARY_OP||r.value.type!==a.AstNodeType.NUMBER||!Number.isInteger(r.value.value))return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Second argument to OFFSET is not a static number");n=-r.value.value}const i=e[2];let c;if(i.type===a.AstNodeType.NUMBER&&Number.isInteger(i.value))c=i.value;else if(i.type===a.AstNodeType.PLUS_UNARY_OP&&i.value.type===a.AstNodeType.NUMBER&&Number.isInteger(i.value.value))c=i.value.value;else{if(i.type!==a.AstNodeType.MINUS_UNARY_OP||i.value.type!==a.AstNodeType.NUMBER||!Number.isInteger(i.value.value))return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Third argument to OFFSET is not a static number");c=-i.value.value}const l=e[3];let h;if(void 0===l)h=1;else{if(l.type!==a.AstNodeType.NUMBER)return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Fourth argument to OFFSET is not a static number");if(h=l.value,h<1)return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Fourth argument to OFFSET is too small number");if(!Number.isInteger(h))return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Fourth argument to OFFSET is not integer")}const d=e[4];let p;if(void 0===d)p=1;else{if(d.type!==a.AstNodeType.NUMBER)return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Fifth argument to OFFSET is not a static number");if(p=d.value,p<1)return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Fifth argument to OFFSET is too small number");if(!Number.isInteger(p))return this.parsingError(a.ParsingErrorType.StaticOffsetError,"Fifth argument to OFFSET is not integer")}const m=new u.CellAddress(t.reference.col+c,t.reference.row+n,t.reference.type);let g=m.col,f=m.row;if(t.reference.type!==u.CellReferenceType.CELL_REFERENCE_RELATIVE&&t.reference.type!==u.CellReferenceType.CELL_REFERENCE_ABSOLUTE_COL||(f+=this.formulaAddress.row),t.reference.type!==u.CellReferenceType.CELL_REFERENCE_RELATIVE&&t.reference.type!==u.CellReferenceType.CELL_REFERENCE_ABSOLUTE_ROW||(g+=this.formulaAddress.col),g<0||f<0)return(0,a.buildCellErrorAst)(new o.CellError(o.ErrorType.REF,s.ErrorMessage.OutOfSheet));if(1===p&&1===h)return(0,a.buildCellReferenceAst)(m);{const e=new u.CellAddress(m.col+p-1,m.row+h-1,m.type);return(0,a.buildCellRangeAst)(m,e,a.RangeSheetReferenceType.RELATIVE)}}parsingError(e,t){return this.customParsingError=(0,a.parsingError)(e,t),(0,a.buildParsingErrorAst)()}static rangeSheetReferenceType(e,t){return void 0===e?a.RangeSheetReferenceType.RELATIVE:void 0===t?a.RangeSheetReferenceType.START_ABSOLUTE:a.RangeSheetReferenceType.BOTH_ABSOLUTE}}t.FormulaParser=l;class h{constructor(e){this.lexerConfig=e,this.lexer=new n.Lexer(e.allTokens,{ensureOptimizations:!0})}tokenizeFormula(e){const t=this.lexer.tokenize(e);let r=t.tokens;return r=this.trimTrailingWhitespaces(r),r=this.skipWhitespacesInsideRanges(r),r=this.skipWhitespacesBeforeArgSeparators(r),t.tokens=r,t}skipWhitespacesInsideRanges(e){return h.filterTokensByNeighbors(e,(e,t,r)=>((0,n.tokenMatcher)(e,c.CellReference)||(0,n.tokenMatcher)(e,c.RangeSeparator))&&(0,n.tokenMatcher)(t,this.lexerConfig.WhiteSpace)&&((0,n.tokenMatcher)(r,c.CellReference)||(0,n.tokenMatcher)(r,c.RangeSeparator)))}skipWhitespacesBeforeArgSeparators(e){return h.filterTokensByNeighbors(e,(e,t,r)=>!(0,n.tokenMatcher)(e,this.lexerConfig.ArgSeparator)&&(0,n.tokenMatcher)(t,this.lexerConfig.WhiteSpace)&&(0,n.tokenMatcher)(r,this.lexerConfig.ArgSeparator))}static filterTokensByNeighbors(e,t){if(e.length<3)return e;let r=0;const n=[e[r++]];for(;r<e.length-1;)t(e[r-1],e[r],e[r+1])||n.push(e[r]),++r;return n.push(e[r]),n}trimTrailingWhitespaces(e){return e.length>0&&(0,n.tokenMatcher)(e[e.length-1],this.lexerConfig.WhiteSpace)&&e.pop(),e}}t.FormulaLexer=h},function(e,t,r){"use strict";t.__esModule=!0,t.ErrorMessage=void 0;
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class n{}t.ErrorMessage=n,n.DistinctSigns="Distinct signs.",n.WrongArgNumber="Wrong number of arguments.",n.EmptyArg="Empty function argument.",n.EmptyArray="Empty array not allowed.",n.ArrayDimensions="Array dimensions are not compatible.",n.NoSpaceForArrayResult="No space for array result.",n.ValueSmall="Value too small.",n.ValueLarge="Value too large.",n.BadCriterion="Incorrect criterion.",n.RangeManySheets="Range spans more than one sheet.",n.CellRangeExpected="Cell range expected.",n.WrongDimension="Wrong range dimension.",n.ScalarExpected="Cell range not allowed.",n.NumberCoercion="Value cannot be coerced to number.",n.NumberExpected="Number argument expected.",n.IntegerExpected="Value needs to be an integer.",n.BadMode="Mode not recognized.",n.DateBounds="Date outside of bounds.",n.OutOfSheet="Resulting reference is out of the sheet.",n.WrongType="Wrong type of argument.",n.NaN="NaN or infinite value encountered.",n.EqualLength="Ranges need to be of equal length.",n.Negative="Value cannot be negative.",n.NotBinary="String does not represent a binary number.",n.NotOctal="String does not represent an octal number.",n.NotHex="String does not represent a hexadecimal number.",n.EndStartPeriod="End period needs to be at least start period.",n.CellRefExpected="Cell reference expected.",n.EmptyRange="Empty range not allowed.",n.BadRef="Address is not correct.",n.NumberRange="Number-only range expected.",n.ValueNotFound="Value not found.",n.ValueBaseLarge="Value in base too large.",n.ValueBaseSmall="Value in base too small.",n.ValueBaseLong="Value in base too long.",n.NegativeLength="Length cannot be negative.",n.PatternNotFound="Pattern not found.",n.OneValue="Needs at least one value.",n.TwoValues="Range needs to contain at least two elements.",n.ThreeValues="Range needs to contain at least three elements.",n.IndexBounds="Index out of bounds.",n.IndexLarge="Index too large.",n.Formula="Expected formula.",n.NegativeCount="Count cannot be negative.",n.ParseError="Parsing error.",n.SheetRef="Sheet does not exist.",n.PeriodLong="Period number cannot exceed life length.",n.InvalidDate="Invalid date.",n.BitshiftLong="Result of bitshift is too long.",n.EmptyString="Empty-string argument not allowed.",n.LengthBounds="Length out of bounds.",n.NegativeTime="Time cannot be negative.",n.NoDefault="No default option.",n.NoConditionMet="None of the conditions were met.",n.Selector="Selector cannot exceed the number of arguments.",n.StartEndDate="Start date needs to be earlier than end date.",n.IncorrectDateTime="String does not represent correct DateTime.",n.CharacterCodeBounds="Character code out of bounds.",n.NonZero="Argument cannot be 0.",n.LessThanOne="Argument cannot be less than 1.",n.WeekendString="Incorrect weekend bitmask string.",n.InvalidRoman="Invalid roman numeral.",n.WrongOrder="Wrong order of values.",n.ComplexNumberExpected="Complex number expected.",n.ShouldBeIorJ="Should be 'i' or 'j'.",n.SizeMismatch="Array dimensions mismatched.",n.FunctionName=e=>`Function name ${e} not recognized.`,n.NamedExpressionName=e=>`Named expression ${e} not recognized.`,n.LicenseKey=e=>`License key is ${e}.`},function(e,t,r){"use strict";t.__esModule=!0,t.buildLexerConfig=t.TimesOp=t.StringLiteral=t.RowRange=t.RangeSeparator=t.RParen=t.ProcedureName=t.PowerOp=t.PlusOp=t.PercentOp=t.NotEqualOp=t.NamedExpression=t.MultiplicationOp=t.MinusOp=t.LessThanOrEqualOp=t.LessThanOp=t.LParen=t.GreaterThanOrEqualOp=t.GreaterThanOp=t.ErrorLiteral=t.EqualsOp=t.DivOp=t.ConcatenateOp=t.ColumnRange=t.CellReference=t.BooleanOp=t.ArrayRParen=t.ArrayLParen=t.AdditionOp=void 0;var n=r(85),o=r(83),s=r(134),i=r(135);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const a=(0,n.createToken)({name:"AdditionOp",pattern:n.Lexer.NA});t.AdditionOp=a;const u=(0,n.createToken)({name:"PlusOp",pattern:/\+/,categories:a});t.PlusOp=u;const c=(0,n.createToken)({name:"MinusOp",pattern:/-/,categories:a});t.MinusOp=c;const l=(0,n.createToken)({name:"MultiplicationOp",pattern:n.Lexer.NA});t.MultiplicationOp=l;const h=(0,n.createToken)({name:"TimesOp",pattern:/\*/,categories:l});t.TimesOp=h;const d=(0,n.createToken)({name:"DivOp",pattern:/\//,categories:l});t.DivOp=d;const p=(0,n.createToken)({name:"PowerOp",pattern:/\^/});t.PowerOp=p;const m=(0,n.createToken)({name:"PercentOp",pattern:/%/});t.PercentOp=m;const g=(0,n.createToken)({name:"BooleanOp",pattern:n.Lexer.NA});t.BooleanOp=g;const f=(0,n.createToken)({name:"EqualsOp",pattern:/=/,categories:g});t.EqualsOp=f;const E=(0,n.createToken)({name:"NotEqualOp",pattern:/<>/,categories:g});t.NotEqualOp=E;const T=(0,n.createToken)({name:"GreaterThanOp",pattern:/>/,categories:g});t.GreaterThanOp=T;const y=(0,n.createToken)({name:"LessThanOp",pattern:/</,categories:g});t.LessThanOp=y;const A=(0,n.createToken)({name:"GreaterThanOrEqualOp",pattern:/>=/,categories:g});t.GreaterThanOrEqualOp=A;const R=(0,n.createToken)({name:"LessThanOrEqualOp",pattern:/<=/,categories:g});t.LessThanOrEqualOp=R;const N=(0,n.createToken)({name:"ConcatenateOp",pattern:/&/});t.ConcatenateOp=N;const S=(0,n.createToken)({name:"LParen",pattern:/\(/});t.LParen=S;const C=(0,n.createToken)({name:"RParen",pattern:/\)/});t.RParen=C;const v=(0,n.createToken)({name:"ArrayLParen",pattern:/{/});t.ArrayLParen=v;const O=(0,n.createToken)({name:"ArrayRParen",pattern:/}/});t.ArrayRParen=O;const M=(0,n.createToken)({name:"StringLiteral",pattern:/"([^"\\]*(\\.[^"\\]*)*)"/});t.StringLiteral=M;const I=(0,n.createToken)({name:"ErrorLiteral",pattern:/#[A-Za-z0-9\/]+[?!]?/});t.ErrorLiteral=I;const w=(0,n.createToken)({name:"RangeSeparator",pattern:new RegExp(o.RANGE_OPERATOR)});t.RangeSeparator=w;const F=(0,n.createToken)({name:"ColumnRange",pattern:new RegExp(`${o.COLUMN_REFERENCE_PATTERN}${o.RANGE_OPERATOR}${o.COLUMN_REFERENCE_PATTERN}`)});t.ColumnRange=F;const b=(0,n.createToken)({name:"RowRange",pattern:new RegExp(`${o.ROW_REFERENCE_PATTERN}${o.RANGE_OPERATOR}${o.ROW_REFERENCE_PATTERN}`)});t.RowRange=b;const _=(0,n.createToken)({name:"ProcedureName",pattern:new RegExp(`([${o.UNICODE_LETTER_PATTERN}][${o.NON_RESERVED_CHARACTER_PATTERN}]*)\\(`)});t.ProcedureName=_;const L=new s.CellReferenceMatcher,x=(0,n.createToken)({name:"CellReference",pattern:L.match.bind(L),start_chars_hint:L.POSSIBLE_START_CHARACTERS,line_breaks:!1});t.CellReference=x;const P=new i.NamedExpressionMatcher,U=(0,n.createToken)({name:"NamedExpression",pattern:P.match.bind(P),start_chars_hint:P.POSSIBLE_START_CHARACTERS,line_breaks:!1});t.NamedExpression=U;t.buildLexerConfig=e=>{const t=e.translationPackage.getFunctionTranslation("OFFSET"),r=e.errorMapping,s=e.translationPackage.buildFunctionMapping(),i=new RegExp("standard"===e.ignoreWhiteSpace?o.ODFF_WHITESPACE_PATTERN:o.ALL_WHITESPACE_PATTERN),L=(0,n.createToken)({name:"WhiteSpace",pattern:i}),P=(0,n.createToken)({name:"ArrayRowSep",pattern:e.arrayRowSeparator}),V=(0,n.createToken)({name:"ArrayColSep",pattern:e.arrayColumnSeparator}),D=(0,n.createToken)({name:"NumberLiteral",pattern:new RegExp(`(([${e.decimalSeparator}]\\d+)|(\\d+([${e.decimalSeparator}]\\d*)?))(e[+-]?\\d+)?`)}),B=(0,n.createToken)({name:"OffsetProcedureName",pattern:new RegExp(t,"i")});let k,G;e.functionArgSeparator===e.arrayColumnSeparator?(k=V,G=[]):e.functionArgSeparator===e.arrayRowSeparator?(k=P,G=[]):(k=(0,n.createToken)({name:"ArgSeparator",pattern:e.functionArgSeparator}),G=[k]);return{ArgSeparator:k,NumberLiteral:D,OffsetProcedureName:B,ArrayRowSeparator:P,ArrayColSeparator:V,WhiteSpace:L,allTokens:[L,u,c,h,d,p,f,E,m,A,R,T,y,S,C,v,O,B,_,w,...G,F,b,D,M,I,N,g,a,l,x,U,P,V],errorMapping:r,functionMapping:s,decimalSeparator:e.decimalSeparator,maxColumns:e.maxColumns,maxRows:e.maxRows}}},function(e,t,r){"use strict";t.__esModule=!0,t.CellReferenceMatcher=void 0;var n=r(83);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.CellReferenceMatcher=class{constructor(){this.POSSIBLE_START_CHARACTERS=[...n.ALL_UNICODE_LETTERS_ARRAY,...n.ALL_DIGITS_ARRAY,n.ABSOLUTE_OPERATOR,"'","_"],this.cellReferenceRegexp=new RegExp(n.CELL_REFERENCE_WITH_NEXT_CHARACTER_PATTERN,"y")}match(e,t){this.cellReferenceRegexp.lastIndex=t;const r=this.cellReferenceRegexp.exec(e+"@");return null==r||null==r[1]?null:(r[0]=r[1],r)}}},function(e,t,r){"use strict";t.__esModule=!0,t.NamedExpressionMatcher=void 0;var n=r(83);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.NamedExpressionMatcher=class{constructor(){this.POSSIBLE_START_CHARACTERS=[...n.ALL_UNICODE_LETTERS_ARRAY,"_"],this.namedExpressionRegexp=new RegExp(n.NAMED_EXPRESSION_PATTERN,"y"),this.r1c1CellRefRegexp=new RegExp(`^${n.R1C1_CELL_REFERENCE_PATTERN}$`)}match(e,t){this.namedExpressionRegexp.lastIndex=t;const r=this.namedExpressionRegexp.exec(e);return null==r||null==r[0]||this.r1c1CellRefRegexp.test(r[0])?null:r}}},function(e,t,r){"use strict";t.__esModule=!0,t.Unparser=void 0,t.formatNumber=u;var n=r(74),o=r(0),s=r(79),i=r(128),a=r(129);function u(e,t){return e.toString().replace(".",t)}t.Unparser=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r,n){this.config=e,this.lexerConfig=t,this.sheetMappingFn=r,this.namedExpressions=n}unparse(e,t){return"="+this.unparseAst(e,t)}unparseAst(e,t){var r,o;switch(e.type){case i.AstNodeType.EMPTY:return(0,i.imageWithWhitespace)("",e.leadingWhitespace);case i.AstNodeType.NUMBER:return(0,i.imageWithWhitespace)(u(e.value,this.config.decimalSeparator),e.leadingWhitespace);case i.AstNodeType.STRING:return(0,i.imageWithWhitespace)('"'+e.value+'"',e.leadingWhitespace);case i.AstNodeType.FUNCTION_CALL:{const r=e.args.map(e=>void 0!==e?this.unparseAst(e,t):"").join(this.config.functionArgSeparator),n=(this.config.translationPackage.isFunctionTranslated(e.procedureName)?this.config.translationPackage.getFunctionTranslation(e.procedureName):e.procedureName)+"("+r+(0,i.imageWithWhitespace)(")",e.internalWhitespace);return(0,i.imageWithWhitespace)(n,e.leadingWhitespace)}case i.AstNodeType.NAMED_EXPRESSION:{const n=null===(r=this.namedExpressions.nearestNamedExpression(e.expressionName,t.sheet))||void 0===r?void 0:r.displayName;return(0,i.imageWithWhitespace)(n||e.expressionName,e.leadingWhitespace)}case i.AstNodeType.CELL_REFERENCE:{let r;return r=void 0!==e.reference.sheet?this.unparseSheetName(e.reference.sheet)+"!":"",r+=null!==(o=e.reference.unparse(t))&&void 0!==o?o:this.config.translationPackage.getErrorTranslation(n.ErrorType.REF),(0,i.imageWithWhitespace)(r,e.leadingWhitespace)}case i.AstNodeType.COLUMN_RANGE:case i.AstNodeType.ROW_RANGE:case i.AstNodeType.CELL_RANGE:return(0,i.imageWithWhitespace)(this.formatRange(e,t),e.leadingWhitespace);case i.AstNodeType.PLUS_UNARY_OP:{const r=this.unparseAst(e.value,t);return(0,i.imageWithWhitespace)("+",e.leadingWhitespace)+r}case i.AstNodeType.MINUS_UNARY_OP:{const r=this.unparseAst(e.value,t);return(0,i.imageWithWhitespace)("-",e.leadingWhitespace)+r}case i.AstNodeType.PERCENT_OP:return this.unparseAst(e.value,t)+(0,i.imageWithWhitespace)("%",e.leadingWhitespace);case i.AstNodeType.ERROR:{const t=this.config.translationPackage.getErrorTranslation(e.error?e.error.type:n.ErrorType.ERROR);return(0,i.imageWithWhitespace)(t,e.leadingWhitespace)}case i.AstNodeType.ERROR_WITH_RAW_INPUT:return(0,i.imageWithWhitespace)(e.rawInput,e.leadingWhitespace);case i.AstNodeType.PARENTHESIS:{const r="("+this.unparseAst(e.expression,t)+(0,i.imageWithWhitespace)(")",e.internalWhitespace);return(0,i.imageWithWhitespace)(r,e.leadingWhitespace)}case i.AstNodeType.ARRAY:{const r="{"+e.args.map(e=>e.map(e=>this.unparseAst(e,t)).join(this.config.arrayColumnSeparator)).join(this.config.arrayRowSeparator)+(0,i.imageWithWhitespace)("}",e.internalWhitespace);return(0,i.imageWithWhitespace)(r,e.leadingWhitespace)}default:{const r=this.unparseAst(e.left,t),n=this.unparseAst(e.right,t);return r+(0,i.imageWithWhitespace)(a.binaryOpTokenMap[e.type],e.leadingWhitespace)+n}}}unparseSheetName(e){const t=(0,s.sheetIndexToString)(e,this.sheetMappingFn);if(void 0===t)throw new o.NoSheetWithIdError(e);return t}formatRange(e,t){let r="",o="";void 0!==e.start.sheet&&e.sheetReferenceType!==i.RangeSheetReferenceType.RELATIVE&&(r=this.unparseSheetName(e.start.sheet)+"!"),void 0!==e.end.sheet&&e.sheetReferenceType===i.RangeSheetReferenceType.BOTH_ABSOLUTE&&(o=this.unparseSheetName(e.end.sheet)+"!");const s=e.start.unparse(t),a=e.end.unparse(t);return void 0===s||void 0===a?this.config.translationPackage.getErrorTranslation(n.ErrorType.REF):`${r}${s}:${o}${a}`}}},function(e,t,r){"use strict";t.__esModule=!0,t.collectDependencies=void 0,r(2);var n=r(78);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */const o=(e,t,r,s)=>{switch(e.type){case n.AstNodeType.EMPTY:case n.AstNodeType.NUMBER:case n.AstNodeType.STRING:case n.AstNodeType.ERROR:return;case n.AstNodeType.NAMED_EXPRESSION:return void(s&&r.push(new n.NamedExpressionDependency(e.expressionName)));case n.AstNodeType.CELL_REFERENCE:return void(s&&r.push(new n.AddressDependency(e.reference)));case n.AstNodeType.CELL_RANGE:return void(s&&e.start.sheet===e.end.sheet&&r.push(new n.CellRangeDependency(e.start,e.end)));case n.AstNodeType.COLUMN_RANGE:return void(s&&e.start.sheet===e.end.sheet&&r.push(new n.ColumnRangeDependency(e.start,e.end)));case n.AstNodeType.ROW_RANGE:return void(s&&e.start.sheet===e.end.sheet&&r.push(new n.RowRangeDependency(e.start,e.end)));case n.AstNodeType.PERCENT_OP:case n.AstNodeType.PLUS_UNARY_OP:case n.AstNodeType.MINUS_UNARY_OP:return void o(e.value,t,r,!0);case n.AstNodeType.CONCATENATE_OP:case n.AstNodeType.EQUALS_OP:case n.AstNodeType.NOT_EQUAL_OP:case n.AstNodeType.LESS_THAN_OP:case n.AstNodeType.GREATER_THAN_OP:case n.AstNodeType.LESS_THAN_OR_EQUAL_OP:case n.AstNodeType.GREATER_THAN_OR_EQUAL_OP:case n.AstNodeType.MINUS_OP:case n.AstNodeType.PLUS_OP:case n.AstNodeType.TIMES_OP:case n.AstNodeType.DIV_OP:case n.AstNodeType.POWER_OP:return o(e.left,t,r,!0),void o(e.right,t,r,!0);case n.AstNodeType.PARENTHESIS:return void o(e.expression,t,r,s);case n.AstNodeType.FUNCTION_CALL:{const n=!t.doesFunctionNeedArgumentToBeComputed(e.procedureName);return void e.args.forEach(e=>o(e,t,r,n))}}};t.collectDependencies=(e,t)=>{const r=new Array;return o(e,t,r,!0),r}},function(e,t,r){"use strict";t.__esModule=!0,t.RowRangeDependency=t.NamedExpressionDependency=t.ColumnRangeDependency=t.CellRangeDependency=t.AddressDependency=void 0;var n=r(73);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.AddressDependency=class{constructor(e){this.dependency=e}absolutize(e){return this.dependency.toSimpleCellAddress(e)}};t.CellRangeDependency=class{constructor(e,t){this.start=e,this.end=t}absolutize(e){return new n.AbsoluteCellRange(this.start.toSimpleCellAddress(e),this.end.toSimpleCellAddress(e))}};t.ColumnRangeDependency=class{constructor(e,t){this.start=e,this.end=t}absolutize(e){const t=this.start.toSimpleColumnAddress(e),r=this.end.toSimpleColumnAddress(e);return new n.AbsoluteColumnRange(t.sheet,t.col,r.col)}};t.RowRangeDependency=class{constructor(e,t){this.start=e,this.end=t}absolutize(e){const t=this.start.toSimpleRowAddress(e),r=this.end.toSimpleRowAddress(e);return new n.AbsoluteRowRange(t.sheet,t.row,r.row)}};t.NamedExpressionDependency=class{constructor(e){this.name=e}absolutize(e){return this}}},function(e,t,r){"use strict";t.__esModule=!0,t.ContentChanges=void 0,r(2);var n=r(74),o=r(140);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s{constructor(){this.changes=new Map}static empty(){return new s}addAll(e){for(const t of e.changes.values())this.add(t.address,t);return this}addChange(e,t,r){this.addInterpreterValue(e,t,r)}exportChanges(e){let t=[];return this.changes.forEach(r=>{const n=e.exportChange(r);Array.isArray(n)?t=t.concat(n):t.push(n)}),t}getChanges(){return Array.from(this.changes.values())}isEmpty(){return 0===this.changes.size}add(e,t){const r=t.value;if(r instanceof o.SimpleRangeValue)for(const t of r.effectiveAddressesFromData(e))this.changes.delete(`${t.sheet},${t.col},${t.row}`);this.changes.set((0,n.addressKey)(e),t)}addInterpreterValue(e,t,r){this.add(t,{address:t,value:e,oldValue:r})}}t.ContentChanges=s},function(e,t,r){"use strict";t.__esModule=!0,t.SimpleRangeValue=void 0,r(2);var n=r(1),o=r(74),s=r(132),i=r(141);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a{constructor(e,t,r,o){this._data=e,this.range=t,this.dependencyGraph=r,this._hasOnlyNumbers=o,this.size=void 0===e?new n.ArraySize(t.effectiveWidth(r),t.effectiveHeight(r)):new n.ArraySize(e[0].length,e.length)}get data(){return this.ensureThatComputed(),this._data}static fromRange(e,t,r){return new a(e,t,r,!0)}static onlyNumbers(e){return new a(e,void 0,void 0,!0)}static onlyValues(e){return new a(e,void 0,void 0,void 0)}static onlyRange(e,t){return new a(void 0,e,t,void 0)}static fromScalar(e){return new a([[e]],void 0,void 0,void 0)}isAdHoc(){return void 0===this.range}width(){return this.size.width}height(){return this.size.height}valuesFromTopLeftCorner(){this.ensureThatComputed();const e=[];for(let t=0;t<this._data.length;t++)for(let r=0;r<this._data[0].length;r++)e.push(this._data[t][r]);return e}*effectiveAddressesFromData(e){for(let t=0;t<this.data.length;++t){const r=this.data[t];for(let n=0;n<r.length;++n)yield(0,o.simpleCellAddress)(e.sheet,e.col+n,e.row+t)}}*entriesFromTopLeftCorner(e){this.ensureThatComputed();for(let t=0;t<this.size.height;++t)for(let r=0;r<this.size.width;++r)yield[this._data[t][r],(0,o.simpleCellAddress)(e.sheet,e.col+r,e.row+t)]}*iterateValuesFromTopLeftCorner(){yield*this.valuesFromTopLeftCorner()}numberOfElements(){return this.size.width*this.size.height}hasOnlyNumbers(){if(void 0===this._hasOnlyNumbers){this._hasOnlyNumbers=!0;for(const e of this.data)for(const t of e)if("number"!=typeof t)return this._hasOnlyNumbers=!1,!1}return this._hasOnlyNumbers}rawNumbers(){return this._data}rawData(){var e;return this.ensureThatComputed(),null!==(e=this._data)&&void 0!==e?e:[]}sameDimensionsAs(e){return this.width()===e.width()&&this.height()===e.height()}ensureThatComputed(){void 0===this._data&&(this._hasOnlyNumbers=!0,this._data=this.range.addressesArrayMap(this.dependencyGraph,e=>{const t=this.dependencyGraph.getCellValue(e);return t instanceof a?(this._hasOnlyNumbers=!1,new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.ScalarExpected)):((0,i.isExtendedNumber)(t)||(this._hasOnlyNumbers=!1),t)}))}}t.SimpleRangeValue=a},function(e,t,r){"use strict";t.__esModule=!0,t.TimeNumber=t.RichNumber=t.PercentNumber=t.NumberType=t.EmptyValue=t.DateTimeNumber=t.DateNumber=t.CurrencyNumber=void 0,t.cloneNumber=function(e,t){if("number"==typeof e)return t;{const r=e.fromNumber(t);return r.format=e.format,r}},t.getFormatOfExtendedNumber=function(e){return e instanceof o?e.format:void 0},t.getRawValue=function(e){return e instanceof o?e.val:e},t.getTypeFormatOfExtendedNumber=function(e){return e instanceof o?{type:e.getDetailedType(),format:e.format}:{type:s.NUMBER_RAW}},t.getTypeOfExtendedNumber=function(e){return e instanceof o?e.getDetailedType():s.NUMBER_RAW},t.isExtendedNumber=function(e){return"number"==typeof e||e instanceof o};
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const n=Symbol("Empty value");t.EmptyValue=n;class o{constructor(e,t){this.val=e,this.format=t}fromNumber(e){return new this.constructor(e)}}t.RichNumber=o;t.DateNumber=class extends o{getDetailedType(){return s.NUMBER_DATE}};t.CurrencyNumber=class extends o{getDetailedType(){return s.NUMBER_CURRENCY}};t.TimeNumber=class extends o{getDetailedType(){return s.NUMBER_TIME}};t.DateTimeNumber=class extends o{getDetailedType(){return s.NUMBER_DATETIME}};var s;t.PercentNumber=class extends o{getDetailedType(){return s.NUMBER_PERCENT}},t.NumberType=s,function(e){e.NUMBER_RAW="NUMBER_RAW",e.NUMBER_DATE="NUMBER_DATE",e.NUMBER_TIME="NUMBER_TIME",e.NUMBER_DATETIME="NUMBER_DATETIME",e.NUMBER_CURRENCY="NUMBER_CURRENCY",e.NUMBER_PERCENT="NUMBER_PERCENT"}(s||(t.NumberType=s={}))},function(e,t,r){"use strict";t.__esModule=!0,t.RowsSpan=t.ColumnsSpan=void 0;
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class n{constructor(e,t,r){if(this.sheet=e,this.rowStart=t,this.rowEnd=r,t<0)throw Error("Starting row cant be less than 0");if(r<t)throw Error("Row span cant end before start")}get numberOfRows(){return this.rowEnd-this.rowStart+1}get start(){return this.rowStart}get end(){return this.rowEnd}static fromNumberOfRows(e,t,r){return new n(e,t,t+r-1)}static fromRowStartAndEnd(e,t,r){return new n(e,t,r)}*rows(){for(let e=this.rowStart;e<=this.rowEnd;++e)yield e}intersect(e){if(this.sheet!==e.sheet)throw Error("Can't intersect spans from different sheets");const t=Math.max(this.rowStart,e.rowStart),r=Math.min(this.rowEnd,e.rowEnd);return t>r?null:new n(this.sheet,t,r)}firstRow(){return new n(this.sheet,this.rowStart,this.rowStart)}}t.RowsSpan=n;class o{constructor(e,t,r){if(this.sheet=e,this.columnStart=t,this.columnEnd=r,t<0)throw Error("Starting column cant be less than 0");if(r<t)throw Error("Column span cant end before start")}get numberOfColumns(){return this.columnEnd-this.columnStart+1}get start(){return this.columnStart}get end(){return this.columnEnd}static fromNumberOfColumns(e,t,r){return new o(e,t,t+r-1)}static fromColumnStartAndEnd(e,t,r){return new o(e,t,r)}*columns(){for(let e=this.columnStart;e<=this.columnEnd;++e)yield e}intersect(e){if(this.sheet!==e.sheet)throw Error("Can't intersect spans from different sheets");const t=Math.max(this.columnStart,e.columnStart),r=Math.min(this.columnEnd,e.columnEnd);return t>r?null:new o(this.sheet,t,r)}firstColumn(){return new o(this.sheet,this.columnStart,this.columnStart)}}t.ColumnsSpan=o},function(e,t,r){"use strict";t.__esModule=!0,t.Statistics=t.StatType=t.EmptyStatistics=void 0;var n=r(144);t.EmptyStatistics=n.EmptyStatistics;var o=r(145);t.Statistics=o.Statistics;var s=r(146);t.StatType=s.StatType},function(e,t,r){"use strict";t.__esModule=!0,t.EmptyStatistics=void 0;var n=r(145);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.Statistics{incrementCriterionFunctionFullCacheUsed(){}incrementCriterionFunctionPartialCacheUsed(){}start(e){}end(e){}}t.EmptyStatistics=o},function(e,t,r){"use strict";t.__esModule=!0,t.Statistics=void 0;var n=r(146);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.Statistics=class{constructor(){this.stats=new Map([[n.StatType.CRITERION_FUNCTION_FULL_CACHE_USED,0],[n.StatType.CRITERION_FUNCTION_PARTIAL_CACHE_USED,0]]),this.startTimes=new Map}incrementCriterionFunctionFullCacheUsed(){var e;const t=(null!==(e=this.stats.get(n.StatType.CRITERION_FUNCTION_FULL_CACHE_USED))&&void 0!==e?e:0)+1;this.stats.set(n.StatType.CRITERION_FUNCTION_FULL_CACHE_USED,t)}incrementCriterionFunctionPartialCacheUsed(){var e;const t=(null!==(e=this.stats.get(n.StatType.CRITERION_FUNCTION_PARTIAL_CACHE_USED))&&void 0!==e?e:0)+1;this.stats.set(n.StatType.CRITERION_FUNCTION_PARTIAL_CACHE_USED,t)}reset(){this.stats.clear(),this.startTimes.clear(),this.stats.set(n.StatType.CRITERION_FUNCTION_FULL_CACHE_USED,0),this.stats.set(n.StatType.CRITERION_FUNCTION_PARTIAL_CACHE_USED,0)}start(e){if(this.startTimes.get(e))throw Error(`Statistics ${e} already started`);this.startTimes.set(e,Date.now())}end(e){var t;const r=Date.now(),n=this.startTimes.get(e);if(!n)throw Error(`Statistics ${e} not started`);{let o=null!==(t=this.stats.get(e))&&void 0!==t?t:0;o+=r-n,this.stats.set(e,o),this.startTimes.delete(e)}}measure(e,t){this.start(e);const r=t();return this.end(e),r}snapshot(){return new Map(this.stats)}}},function(e,t,r){"use strict";
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
var n;t.__esModule=!0,t.StatType=void 0,t.StatType=n,function(e){e.BUILD_ENGINE_TOTAL="BUILD_ENGINE_TOTAL",e.PARSER="PARSER",e.GRAPH_BUILD="GRAPH_BUILD",e.COLLECT_DEPENDENCIES="COLLECT_DEPENDENCIES",e.PROCESS_DEPENDENCIES="PROCESS_DEPENDENCIES",e.TOP_SORT="TOP_SORT",e.BUILD_COLUMN_INDEX="BUILD_COLUMN_INDEX",e.EVALUATION="EVALUATION",e.VLOOKUP="VLOOKUP",e.TRANSFORM_ASTS="TRANSFORM_ASTS",e.TRANSFORM_ASTS_POSTPONED="TRANSFORM_ASTS_POSTPONED",e.ADJUSTING_ADDRESS_MAPPING="ADJUSTING_ADDRESS_MAPPING",e.ADJUSTING_ARRAY_MAPPING="ADJUSTING_ARRAY_MAPPING",e.ADJUSTING_RANGES="ADJUSTING_RANGES",e.ADJUSTING_GRAPH="ADJUSTING_GRAPH",e.CRITERION_FUNCTION_FULL_CACHE_USED="CRITERION_FUNCTION_FULL_CACHE_USED",e.CRITERION_FUNCTION_PARTIAL_CACHE_USED="CRITERION_FUNCTION_PARTIAL_CACHE_USED"}(n||(t.StatType=n={}))},function(e,t,r){"use strict";t.__esModule=!0,t.AddressMapping=void 0;var n=r(148),o=r(141),s=r(75);t.AddressMapping=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.policy=e,this.mapping=new Map}getCell(e){const t=this.mapping.get(e.sheet);if(void 0===t)throw new n.NoSheetWithIdError(e.sheet);return t.getCell(e)}fetchCell(e){const t=this.mapping.get(e.sheet);if(void 0===t)throw new n.NoSheetWithIdError(e.sheet);const r=t.getCell(e);if(!r)throw Error("Vertex for address missing in AddressMapping");return r}strategyFor(e){const t=this.mapping.get(e);if(void 0===t)throw new n.NoSheetWithIdError(e);return t}addSheet(e,t){if(this.mapping.has(e))throw Error("Sheet already added");this.mapping.set(e,t)}autoAddSheet(e,t,r){const{height:n,width:o,fill:s}=r,i=this.policy.call(s);this.addSheet(e,new i(o,n))}getCellValue(e){const t=this.getCell(e);return void 0===t?o.EmptyValue:t instanceof s.ArrayVertex?t.getArrayCellValue(e):t.getCellValue()}getRawValue(e){const t=this.getCell(e);return t instanceof s.ValueCellVertex?t.getValues().rawValue:t instanceof s.ArrayVertex?t.getArrayCellRawValue(e):null}setCell(e,t){const r=this.mapping.get(e.sheet);if(!r)throw Error("Sheet not initialized");r.setCell(e,t)}moveCell(e,t){const r=this.mapping.get(e.sheet);if(!r)throw Error("Sheet not initialized.");if(e.sheet!==t.sheet)throw Error("Cannot move cells between sheets.");if(r.has(t))throw new Error("Cannot move cell. Destination already occupied.");const n=r.getCell(e);if(void 0===n)throw new Error("Cannot move cell. No cell with such address.");this.setCell(t,n),this.removeCell(e)}removeCell(e){const t=this.mapping.get(e.sheet);if(!t)throw Error("Sheet not initialized");t.removeCell(e)}has(e){const t=this.mapping.get(e.sheet);return void 0!==t&&t.has(e)}getHeight(e){const t=this.mapping.get(e);if(void 0===t)throw new n.NoSheetWithIdError(e);return t.getHeight()}getWidth(e){const t=this.mapping.get(e);if(!t)throw new n.NoSheetWithIdError(e);return t.getWidth()}addRows(e,t,r){const o=this.mapping.get(e);if(void 0===o)throw new n.NoSheetWithIdError(e);o.addRows(t,r)}removeRows(e){const t=this.mapping.get(e.sheet);if(void 0===t)throw new n.NoSheetWithIdError(e.sheet);t.removeRows(e)}removeSheet(e){this.mapping.delete(e)}addColumns(e,t,r){const o=this.mapping.get(e);if(void 0===o)throw new n.NoSheetWithIdError(e);o.addColumns(t,r)}removeColumns(e){const t=this.mapping.get(e.sheet);if(void 0===t)throw new n.NoSheetWithIdError(e.sheet);t.removeColumns(e)}*verticesFromRowsSpan(e){yield*this.mapping.get(e.sheet).verticesFromRowsSpan(e)}*verticesFromColumnsSpan(e){yield*this.mapping.get(e.sheet).verticesFromColumnsSpan(e)}*entriesFromRowsSpan(e){yield*this.mapping.get(e.sheet).entriesFromRowsSpan(e)}*entriesFromColumnsSpan(e){yield*this.mapping.get(e.sheet).entriesFromColumnsSpan(e)}*entries(){for(const[e,t]of this.mapping.entries())yield*t.getEntries(e)}*sheetEntries(e){const t=this.mapping.get(e);if(void 0===t)throw new n.NoSheetWithIdError(e);yield*t.getEntries(e)}}},function(e,t,r){"use strict";t.__esModule=!0,t.UnableToParseError=t.TargetLocationHasArrayError=t.SourceLocationHasArrayError=t.SheetsNotEqual=t.SheetSizeLimitExceededError=t.SheetNameAlreadyTakenError=t.ProtectedFunctionTranslationError=t.ProtectedFunctionError=t.NothingToPasteError=t.NotAFormulaError=t.NoSheetWithNameError=t.NoSheetWithIdError=t.NoRelativeAddressesAllowedError=t.NoOperationToUndoError=t.NoOperationToRedoError=t.NamedExpressionNameIsInvalidError=t.NamedExpressionNameIsAlreadyTakenError=t.NamedExpressionDoesNotExistError=t.MissingTranslationError=t.LanguageNotRegisteredError=t.LanguageAlreadyRegisteredError=t.InvalidArgumentsError=t.InvalidAddressError=t.FunctionPluginValidationError=t.ExpectedValueOfTypeError=t.ExpectedOneOfValuesError=t.EvaluationSuspendedError=t.ConfigValueTooSmallError=t.ConfigValueTooBigError=t.ConfigValueEmpty=t.AliasAlreadyExisting=void 0;
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class n extends Error{constructor(e){super("There's no sheet with id = "+e)}}t.NoSheetWithIdError=n;class o extends Error{constructor(e){super(`There's no sheet with name '${e}'`)}}t.NoSheetWithNameError=o;class s extends Error{constructor(e){super(`Sheet with name ${e} already exists`)}}t.SheetNameAlreadyTakenError=s;class i extends Error{constructor(){super("Sheet size limit exceeded")}}t.SheetSizeLimitExceededError=i;class a extends Error{constructor(){super("This is not a formula")}}t.NotAFormulaError=a;class u extends Error{constructor(e){super(`Address (row = ${e.row}, col = ${e.col}) is invalid`)}}t.InvalidAddressError=u;class c extends Error{constructor(e){super("Invalid arguments, expected "+e)}}t.InvalidArgumentsError=c;class l extends Error{constructor(e,t){super(`Sheets ${e} and ${t} are not equal.`)}}t.SheetsNotEqual=l;class h extends Error{constructor(e){super(`Name of Named Expression '${e}' is already present`)}}t.NamedExpressionNameIsAlreadyTakenError=h;class d extends Error{constructor(e){super(`Name of Named Expression '${e}' is invalid`)}}t.NamedExpressionNameIsInvalidError=d;class p extends Error{constructor(e){super(`Named Expression '${e}' does not exist`)}}t.NamedExpressionDoesNotExistError=p;class m extends Error{constructor(){super("There is no operation to undo")}}t.NoOperationToUndoError=m;class g extends Error{constructor(){super("There is no operation to redo")}}t.NoOperationToRedoError=g;class f extends Error{constructor(){super("There is nothing to paste")}}function E(e,t){switch(typeof t){case"function":case"symbol":return t.toString();case"bigint":return"BigInt("+t.toString()+")";default:return t instanceof RegExp?"RegExp("+t.toString()+")":t}}t.NothingToPasteError=f;class T extends Error{constructor(e){super("Unable to parse value: "+JSON.stringify(e,E,4))}}t.UnableToParseError=T;class y extends Error{constructor(e,t){super(`Expected value of type: ${e} for config parameter: ${t}`)}}t.ExpectedValueOfTypeError=y;class A extends Error{constructor(e){super(`Config parameter ${e} cannot be empty.`)}}t.ConfigValueEmpty=A;class R extends Error{constructor(e,t){super(`Config parameter ${e} should be at least ${t}`)}}t.ConfigValueTooSmallError=R;class N extends Error{constructor(e,t){super(`Config parameter ${e} should be at most ${t}`)}}t.ConfigValueTooBigError=N;class S extends Error{constructor(e,t){super(`Expected one of ${e} for config parameter: ${t}`)}}t.ExpectedOneOfValuesError=S;class C extends Error{constructor(){super("Computations are suspended")}}t.EvaluationSuspendedError=C;class v extends Error{constructor(e){super(`Translation for ${e} is missing in the translation package you're using.`)}}t.MissingTranslationError=v;class O extends Error{constructor(e){super("Cannot register translation for function with id: "+e)}}t.ProtectedFunctionTranslationError=O;class M extends Error{constructor(){super("Language not registered.")}}t.LanguageNotRegisteredError=M;class I extends Error{constructor(){super("Language already registered.")}}t.LanguageAlreadyRegisteredError=I;class w extends Error{static functionNotDeclaredInPlugin(e,t){return new w(`Function with id ${e} not declared in plugin ${t}`)}static functionMethodNotFound(e,t){return new w(`Function method ${e} not found in plugin ${t}`)}}t.FunctionPluginValidationError=w;class F extends Error{static cannotRegisterFunctionWithId(e){return new F("Cannot register function with id "+e)}static cannotUnregisterFunctionWithId(e){return new F("Cannot unregister function with id "+e)}static cannotUnregisterProtectedPlugin(){return new F("Cannot unregister protected plugin")}}t.ProtectedFunctionError=F;class b extends Error{constructor(){super("Cannot perform this operation, source location has an array inside.")}}t.SourceLocationHasArrayError=b;class _ extends Error{constructor(){super("Cannot perform this operation, target location has an array inside.")}}t.TargetLocationHasArrayError=_;class L extends Error{constructor(){super("Relative addresses not allowed in named expressions.")}}t.NoRelativeAddressesAllowedError=L;class x extends Error{constructor(e,t){super(`Alias id ${e} in plugin ${t} already defined as a function or alias.`)}}t.AliasAlreadyExisting=x},function(e,t,r){"use strict";t.__esModule=!0,t.ArrayMapping=void 0,r(2);var n=r(74);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.ArrayMapping=class{constructor(){this.arrayMapping=new Map}getArray(e){const t=this.getArrayByCorner(e.start);if(null==t?void 0:t.getRange().sameAs(e))return t}getArrayByCorner(e){return this.arrayMapping.get((0,n.addressKey)(e))}setArray(e,t){this.arrayMapping.set((0,n.addressKey)(e.start),t)}removeArray(e){"string"==typeof e?this.arrayMapping.delete(e):this.arrayMapping.delete((0,n.addressKey)(e.start))}count(){return this.arrayMapping.size}*arraysInRows(e){for(const[t,r]of this.arrayMapping.entries())r.spansThroughSheetRows(e.sheet,e.rowStart,e.rowEnd)&&(yield[t,r])}*arraysInCols(e){for(const[t,r]of this.arrayMapping.entries())r.spansThroughSheetColumn(e.sheet,e.columnStart,e.columnEnd)&&(yield[t,r])}isFormulaArrayInRow(e,t){for(const r of this.arrayMapping.values())if(r.spansThroughSheetRows(e,t))return!0;return!1}isFormulaArrayInAllRows(e){let t=!0;for(const r of e.rows())this.isFormulaArrayInRow(e.sheet,r)||(t=!1);return t}isFormulaArrayInColumn(e,t){for(const r of this.arrayMapping.values())if(r.spansThroughSheetColumn(e,t))return!0;return!1}isFormulaArrayInAllColumns(e){let t=!0;for(const r of e.columns())this.isFormulaArrayInColumn(e.sheet,r)||(t=!1);return t}isFormulaArrayInRange(e){for(const t of this.arrayMapping.values())if(t.getRange().doesOverlap(e))return!0;return!1}isFormulaArrayAtAddress(e){for(const t of this.arrayMapping.values())if(t.getRange().addressInRange(e))return!0;return!1}moveArrayVerticesAfterRowByRows(e,t,r){this.updateArrayVerticesInSheet(e,(e,n)=>{const o=n.getRange();return t<=o.start.row?[o.shifted(0,r),n]:void 0})}moveArrayVerticesAfterColumnByColumns(e,t,r){this.updateArrayVerticesInSheet(e,(e,n)=>{const o=n.getRange();return t<=o.start.col?[o.shifted(r,0),n]:void 0})}updateArrayVerticesInSheet(e,t){const r=Array();for(const[n,o]of this.arrayMapping.entries()){if(o.sheet!==e)continue;const s=t(n,o);void 0!==s&&(this.removeArray(n),r.push(s))}r.forEach(e=>{let[t,r]=e;this.setArray(t,r)})}}},function(e,t,r){"use strict";t.__esModule=!0,t.collectAddressesDependentToRange=void 0;var n=r(78),o=r(151),s=r(153);t.collectAddressesDependentToRange=(e,t,r,i,a)=>{if(t instanceof s.RangeVertex){const e=t.range.intersectionWith(r);return void 0!==e?Array.from(e.addresses(a)):[]}let u,c;return t instanceof o.FormulaVertex?(u=t.getFormula(i),c=t.getAddress(i),(0,n.collectDependencies)(u,e).filter(e=>e instanceof n.AddressDependency).map(e=>e.dependency.toSimpleCellAddress(c)).filter(e=>r.addressInRange(e))):[]}},function(e,t,r){"use strict";t.__esModule=!0,t.FormulaVertex=t.FormulaCellVertex=t.ArrayVertex=void 0;var n=r(73),o=r(1),s=r(152),i=r(74),a=r(132),u=r(141),c=r(142);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class l{constructor(e,t,r){this.formula=e,this.cellAddress=t,this.version=r}get width(){return 1}get height(){return 1}static fromAst(e,t,r,n){return r.isScalar()?new d(e,t,n):new h(e,t,r,n)}getFormula(e){return this.ensureRecentData(e),this.formula}ensureRecentData(e){if(this.version!=e.version()){const[t,r,n]=e.applyTransformations(this.formula,this.cellAddress,this.version);this.formula=t,this.cellAddress=r,this.version=n}}getAddress(e){return this.ensureRecentData(e),this.cellAddress}}t.FormulaVertex=l;class h extends l{constructor(e,t,r){super(e,t,arguments.length>3&&void 0!==arguments[3]?arguments[3]:0),r.isRef?this.array=new s.ErroredArray(new i.CellError(i.ErrorType.REF,a.ErrorMessage.NoSpaceForArrayResult),o.ArraySize.error()):this.array=new s.NotComputedArray(r)}get width(){return this.array.width()}get height(){return this.array.height()}get sheet(){return this.cellAddress.sheet}get leftCorner(){return this.cellAddress}setCellValue(e){if(e instanceof i.CellError)return this.setErrorValue(e),e;const t=s.ArrayValue.fromInterpreterValue(e);return t.resize(this.array.size),this.array=t,e}getCellValue(){if(this.array instanceof s.NotComputedArray)throw Error("Array not computed yet.");return this.array.simpleRangeValue()}valueOrUndef(){if(!(this.array instanceof s.NotComputedArray))return this.array.simpleRangeValue()}getArrayCellValue(e){const t=e.col-this.cellAddress.col,r=e.row-this.cellAddress.row;try{return this.array.get(t,r)}catch(e){return new i.CellError(i.ErrorType.REF)}}getArrayCellRawValue(e){const t=this.getArrayCellValue(e);return t instanceof i.CellError||t===u.EmptyValue?void 0:(0,u.getRawValue)(t)}setArrayCellValue(e,t){const r=e.col-this.cellAddress.col,n=e.row-this.cellAddress.row;this.array instanceof s.ArrayValue&&this.array.set(r,n,t)}setNoSpace(){return this.array=new s.ErroredArray(new i.CellError(i.ErrorType.SPILL,a.ErrorMessage.NoSpaceForArrayResult),o.ArraySize.error()),this.getCellValue()}getRange(){return n.AbsoluteCellRange.spanFrom(this.cellAddress,this.width,this.height)}getRangeOrUndef(){return n.AbsoluteCellRange.spanFromOrUndef(this.cellAddress,this.width,this.height)}setAddress(e){this.cellAddress=e}setFormula(e){this.formula=e}spansThroughSheetRows(e,t){let r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:t;return this.cellAddress.sheet===e&&this.cellAddress.row<=r&&t<this.cellAddress.row+this.height}spansThroughSheetColumn(e,t){let r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:t;return this.cellAddress.sheet===e&&this.cellAddress.col<=r&&t<this.cellAddress.col+this.width}isComputed(){return!(this.array instanceof s.NotComputedArray)}columnsFromArray(){return c.ColumnsSpan.fromNumberOfColumns(this.cellAddress.sheet,this.cellAddress.col,this.width)}rowsFromArray(){return c.RowsSpan.fromNumberOfRows(this.cellAddress.sheet,this.cellAddress.row,this.height)}ensureRecentData(e){}isLeftCorner(e){return(0,i.equalSimpleCellAddress)(this.cellAddress,e)}setErrorValue(e){this.array=new s.ErroredArray(e,this.array.size)}}t.ArrayVertex=h;class d extends l{constructor(e,t,r){super(e,t,r)}valueOrUndef(){return this.cachedCellValue}setCellValue(e){return this.cachedCellValue=e,this.cachedCellValue}getCellValue(){if(void 0!==this.cachedCellValue)return this.cachedCellValue;throw Error("Value of the formula cell is not computed.")}isComputed(){return void 0!==this.cachedCellValue}}t.FormulaCellVertex=d},function(e,t,r){"use strict";t.__esModule=!0,t.NotComputedArray=t.ErroredArray=t.ArrayValue=void 0,r(2);var n=r(1),o=r(141),s=r(140);t.NotComputedArray=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.size=e}width(){return this.size.width}height(){return this.size.height}get(e,t){throw Error("Array not computed yet.")}simpleRangeValue(){throw Error("Array not computed yet.")}};class i{constructor(e){this.size=new n.ArraySize(e.length>0?e[0].length:0,e.length),this.array=e}static fromInterpreterValue(e){return e instanceof s.SimpleRangeValue?new i(e.data):new i([[e]])}simpleRangeValue(){return s.SimpleRangeValue.onlyValues(this.array)}addRows(e,t){this.array.splice(e,0,...this.nullArrays(t,this.width())),this.size.height+=t}addColumns(e,t){for(let r=0;r<this.height();r++)this.array[r].splice(e,0,...new Array(t).fill(o.EmptyValue));this.size.width+=t}removeRows(e,t){if(this.outOfBound(0,e)||this.outOfBound(0,t))throw Error("Array index out of bound");const r=t-e+1;this.array.splice(e,r),this.size.height-=r}removeColumns(e,t){if(this.outOfBound(e,0)||this.outOfBound(t,0))throw Error("Array index out of bound");const r=t-e+1;for(const t of this.array)t.splice(e,r);this.size.width-=r}nullArrays(e,t){const r=[];for(let n=0;n<e;++n)r.push(new Array(t).fill(o.EmptyValue));return r}get(e,t){if(this.outOfBound(e,t))throw Error("Array index out of bound");return this.array[t][e]}set(e,t,r){if(this.outOfBound(e,t))throw Error("Array index out of bound");this.array[t][e]=r}width(){return this.size.width}height(){return this.size.height}raw(){return this.array}resize(e){if(this.height()<e.height&&isFinite(e.height)&&this.addRows(this.height(),e.height-this.height()),this.height()>e.height)throw Error("Resizing to smaller array");if(this.width()<e.width&&isFinite(e.width)&&this.addColumns(this.width(),e.width-this.width()),this.width()>e.width)throw Error("Resizing to smaller array")}outOfBound(e,t){return e<0||t<0||t>this.size.height-1||e>this.size.width-1}}t.ArrayValue=i;t.ErroredArray=class{constructor(e,t){this.error=e,this.size=t}get(e,t){return this.error}width(){return this.size.width}height(){return this.size.height}simpleRangeValue(){return this.error}}},function(e,t,r){"use strict";t.__esModule=!0,t.RangeVertex=void 0;t.RangeVertex=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.range=e,this.functionCache=new Map,this.criterionFunctionCache=new Map,this.dependentCacheRanges=new Set,this.bruteForce=!1}get start(){return this.range.start}get end(){return this.range.end}get sheet(){return this.range.start.sheet}getFunctionValue(e){return this.functionCache.get(e)}setFunctionValue(e,t){this.functionCache.set(e,t)}getCriterionFunctionValue(e,t){var r;return null===(r=this.getCriterionFunctionValues(e).get(t))||void 0===r?void 0:r[0]}getCriterionFunctionValues(e){var t;return null!==(t=this.criterionFunctionCache.get(e))&&void 0!==t?t:new Map}setCriterionFunctionValues(e,t){this.criterionFunctionCache.set(e,t)}addDependentCacheRange(e){e!==this&&this.dependentCacheRanges.add(e)}clearCache(){this.functionCache.clear(),this.criterionFunctionCache.clear(),this.dependentCacheRanges.forEach(e=>e.criterionFunctionCache.clear()),this.dependentCacheRanges.clear()}getStart(){return this.start}getEnd(){return this.end}}},function(e,t,r){"use strict";
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
var n;t.__esModule=!0,t.Graph=void 0,r(2),function(e){e[e.ON_STACK=0]="ON_STACK",e[e.PROCESSED=1]="PROCESSED",e[e.POPPED=2]="POPPED"}(n||(n={}));t.Graph=class{constructor(e){this.dependencyQuery=e,this.nodes=new Set,this.specialNodes=new Set,this.specialNodesStructuralChanges=new Set,this.specialNodesRecentlyChanged=new Set,this.infiniteRanges=new Set,this.edges=new Map}addNode(e){this.nodes.add(e),this.edges.has(e)||this.edges.set(e,new Set)}addEdge(e,t){if(!this.nodes.has(e))throw new Error("Unknown node "+e);if(!this.nodes.has(t))throw new Error("Unknown node "+t);this.edges.get(e).add(t)}removeEdge(e,t){if(!this.existsEdge(e,t))throw new Error("Edge does not exist");this.edges.get(e).delete(t)}softRemoveEdge(e,t){var r;null===(r=this.edges.get(e))||void 0===r||r.delete(t)}removeIncomingEdges(e){this.edges.forEach(t=>{t.delete(e)})}adjacentNodes(e){return this.edges.get(e)}adjacentNodesCount(e){return this.adjacentNodes(e).size}hasNode(e){return this.nodes.has(e)}nodesCount(){return this.nodes.size}edgesCount(){let e=0;return this.edges.forEach(t=>e+=t.size),e}removeNode(e){for(const t of this.adjacentNodes(e).values())this.markNodeAsSpecialRecentlyChanged(t);return this.edges.delete(e),this.nodes.delete(e),this.specialNodes.delete(e),this.specialNodesRecentlyChanged.delete(e),this.specialNodesStructuralChanges.delete(e),this.infiniteRanges.delete(e),this.removeDependencies(e)}markNodeAsSpecial(e){this.specialNodes.add(e)}markNodeAsSpecialRecentlyChanged(e){this.nodes.has(e)&&this.specialNodesRecentlyChanged.add(e)}markNodeAsChangingWithStructure(e){this.specialNodesStructuralChanges.add(e)}clearSpecialNodesRecentlyChanged(){this.specialNodesRecentlyChanged.clear()}markNodeAsInfiniteRange(e){this.infiniteRanges.add(e)}existsEdge(e,t){var r,n;return null!==(n=null===(r=this.edges.get(e))||void 0===r?void 0:r.has(t))&&void 0!==n&&n}topSortWithScc(){return this.getTopSortedWithSccSubgraphFrom(Array.from(this.nodes),()=>!0,()=>{})}getTopSortedWithSccSubgraphFrom(e,t,r){const o=new Map,s=new Map,i=new Map,a=new Set,u=new Map,c=[];let l=0;const h=new Set;e.reverse(),e.forEach(e=>{if(void 0!==u.get(e))return;const t=[e],r=[];for(u.set(e,n.ON_STACK);t.length>0;){const e=t[t.length-1];switch(u.get(e)){case n.ON_STACK:o.set(e,l),s.set(e,l),r.push(e),l++,this.adjacentNodes(e).forEach(r=>{void 0===o.get(r)&&(t.push(r),i.set(r,e),u.set(r,n.ON_STACK))}),u.set(e,n.PROCESSED);break;case n.PROCESSED:{let l;if(l=o.get(e),this.adjacentNodes(e).forEach(t=>{a.has(t)||(l=i.get(t)===e?Math.min(l,s.get(t)):Math.min(l,o.get(t)))}),s.set(e,l),l===o.get(e)){const t=[];do{t.push(r[r.length-1]),r.pop()}while(t[t.length-1]!==e);t.forEach(e=>{a.add(e)}),c.push(...t),t.length>1&&t.forEach(e=>{h.add(e)})}t.pop(),u.set(e,n.POPPED);break}case n.POPPED:t.pop()}}});const d=new Set(e),p=[],m=[];return c.reverse(),c.forEach(e=>{h.has(e)||this.adjacentNodes(e).has(e)?(m.push(e),r(e),this.adjacentNodes(e).forEach(e=>d.add(e))):(p.push(e),d.has(e)&&t(e)&&this.adjacentNodes(e).forEach(e=>d.add(e)))}),{sorted:p,cycled:m}}getDependencies(e){const t=[];return this.edges.forEach((r,n)=>{r.has(e)&&t.push(n)}),t}removeDependencies(e){const t=this.dependencyQuery(e);for(const[r,n]of t)this.softRemoveEdge(n,e);return t}}},function(e,t,r){"use strict";t.__esModule=!0,t.RangeMapping=void 0,r(2);var n=r(73),o=r(74);function s(e,t){return`${e.col},${e.row},${t.col},${t.row}`}t.RangeMapping=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(){this.rangeMapping=new Map}getMappingSize(e){var t,r;return null!==(r=null===(t=this.rangeMapping.get(e))||void 0===t?void 0:t.size)&&void 0!==r?r:0}setRange(e){let t=this.rangeMapping.get(e.getStart().sheet);void 0===t&&(t=new Map,this.rangeMapping.set(e.getStart().sheet,t));const r=s(e.getStart(),e.getEnd());t.set(r,e)}removeRange(e){const t=e.getStart().sheet,r=this.rangeMapping.get(t);if(void 0===r)return;const n=s(e.getStart(),e.getEnd());r.delete(n),0===r.size&&this.rangeMapping.delete(t)}getRange(e,t){const r=this.rangeMapping.get(e.sheet),n=s(e,t);return null==r?void 0:r.get(n)}fetchRange(e,t){const r=this.getRange(e,t);if(!r)throw Error("Range does not exist");return r}truncateRanges(e,t){const r=Array(),n=Array(),o=Array(),a=e.sheet;for(const[s,i]of this.entriesFromSheet(e.sheet)){const u=i.range;e.start<=t(i.range.end)&&(u.removeSpan(e),u.shouldBeRemoved()?(this.removeByKey(a,s),r.push(i)):n.push([s,i]),o.push(i))}const u=[];n.sort((e,r)=>i(e[1],r[1],t));for(const[e,t]of n){const r=s((c=t.range).start,c.end);if(r===e)continue;const n=this.getByKey(a,r);this.removeByKey(a,e),void 0!==n&&t!=n?u.push([n,t]):this.setRange(t)}var c;return{verticesToRemove:r,verticesToMerge:u,verticesWithChangedSize:o}}moveAllRangesInSheetAfterRowByRows(e,t,r){return this.updateVerticesFromSheet(e,(e,n)=>t<=n.start.row?(n.range.shiftByRows(r),{changedSize:!1,vertex:n}):t>n.start.row&&t<=n.end.row?(n.range.expandByRows(r),{changedSize:!0,vertex:n}):void 0)}moveAllRangesInSheetAfterColumnByColumns(e,t,r){return this.updateVerticesFromSheet(e,(e,n)=>t<=n.start.col?(n.range.shiftByColumns(r),{changedSize:!1,vertex:n}):t>n.start.col&&t<=n.end.col?(n.range.expandByColumns(r),{changedSize:!0,vertex:n}):void 0)}moveRangesInsideSourceRange(e,t,r,n){this.updateVerticesFromSheet(e.sheet,(o,s)=>e.containsRange(s.range)?(s.range.shiftByColumns(t),s.range.shiftByRows(r),s.range.moveToSheet(n),{changedSize:!1,vertex:s}):void 0)}removeRangesInSheet(e){if(this.rangeMapping.has(e)){const t=this.rangeMapping.get(e).values();return this.rangeMapping.delete(e),t}return[][Symbol.iterator]()}*rangesInSheet(e){const t=this.rangeMapping.get(e);t&&(yield*t.values())}*rangeVerticesContainedInRange(e){for(const t of this.rangesInSheet(e.sheet))e.containsRange(t.range)&&(yield t)}findSmallerRange(e){if(e.height()>1&&Number.isFinite(e.height())){const t=(0,o.simpleCellAddress)(e.end.sheet,e.end.col,e.end.row-1),r=this.getRange(e.start,t);if(void 0!==r){return{smallerRangeVertex:r,restRange:n.AbsoluteCellRange.fromSimpleCellAddresses((0,o.simpleCellAddress)(e.start.sheet,e.start.col,e.end.row),e.end)}}}return{restRange:e}}*entriesFromSheet(e){const t=this.rangeMapping.get(e);t&&(yield*t.entries())}removeByKey(e,t){this.rangeMapping.get(e).delete(t)}getByKey(e,t){var r;return null===(r=this.rangeMapping.get(e))||void 0===r?void 0:r.get(t)}updateVerticesFromSheet(e,t){const r=Array();for(const[n,o]of this.entriesFromSheet(e)){const s=t(n,o);void 0!==s&&(this.removeByKey(e,n),r.push(s))}return r.forEach(e=>{this.setRange(e.vertex)}),{verticesWithChangedSize:r.filter(e=>e.changedSize).map(e=>e.vertex)}}};const i=(e,t,r)=>{const n=r(e.range.start),o=r(e.range.start);if(n===o){return r(e.range.end)-r(t.range.end)}return n-o}},function(e,t,r){"use strict";t.__esModule=!0,t.SheetMapping=void 0;var n=r(148),o=r(157);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function s(e){return e.toLowerCase()}class i{constructor(e,t){this.id=e,this.displayName=t}get canonicalName(){return s(this.displayName)}}t.SheetMapping=class{constructor(e){this.languages=e,this.mappingFromCanonicalName=new Map,this.mappingFromId=new Map,this.lastSheetId=-1,this.fetch=e=>{const t=this.mappingFromCanonicalName.get(s(e));if(void 0===t)throw new n.NoSheetWithNameError(e);return t.id},this.get=e=>{var t;return null===(t=this.mappingFromCanonicalName.get(s(e)))||void 0===t?void 0:t.id},this.fetchDisplayName=e=>this.fetchSheetById(e).displayName,this.sheetNamePrefix=e.getUITranslation(o.UIElement.NEW_SHEET_PREFIX)}addSheet(){let e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:`${this.sheetNamePrefix}${this.lastSheetId+2}`;const t=s(e);if(this.mappingFromCanonicalName.has(t))throw new n.SheetNameAlreadyTakenError(e);this.lastSheetId++;const r=new i(this.lastSheetId,e);return this.store(r),r.id}removeSheet(e){const t=this.fetchSheetById(e);e==this.lastSheetId&&--this.lastSheetId,this.mappingFromCanonicalName.delete(t.canonicalName),this.mappingFromId.delete(t.id)}getDisplayName(e){var t;return null===(t=this.mappingFromId.get(e))||void 0===t?void 0:t.displayName}*displayNames(){for(const e of this.mappingFromCanonicalName.values())yield e.displayName}numberOfSheets(){return this.mappingFromCanonicalName.size}hasSheetWithId(e){return this.mappingFromId.has(e)}hasSheetWithName(e){return this.mappingFromCanonicalName.has(s(e))}renameSheet(e,t){const r=this.fetchSheetById(e),o=r.displayName;if(o===t)return;const i=this.mappingFromCanonicalName.get(s(t));if(void 0!==i&&i.id!==r.id)throw new n.SheetNameAlreadyTakenError(t);const a=r.canonicalName;return this.mappingFromCanonicalName.delete(a),r.displayName=t,this.store(r),o}sheetNames(){return Array.from(this.mappingFromId.values()).map(e=>e.displayName)}store(e){this.mappingFromId.set(e.id,e),this.mappingFromCanonicalName.set(e.canonicalName,e)}fetchSheetById(e){const t=this.mappingFromId.get(e);if(void 0===t)throw new n.NoSheetWithIdError(e);return t}}},function(e,t,r){"use strict";t.__esModule=!0,t.UIElement=void 0;var n,o=r(158);t.buildTranslationPackage=o.buildTranslationPackage,t.TranslationPackage=o.TranslationPackage,t.UIElement=n,function(e){e.NEW_SHEET_PREFIX="NEW_SHEET_PREFIX"}(n||(t.UIElement=n={}))},function(e,t,r){"use strict";t.__esModule=!0,t.TranslationPackage=void 0,t.buildTranslationPackage=function(e){return new i(Object.assign({},e.functions),Object.assign({},e.errors),Object.assign({},e.ui))},r(2);var n=r(74),o=r(148),s=r(157);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i{constructor(e,t,r){this.functions=e,this.errors=t,this.ui=r,this._protectedTranslations={VERSION:"VERSION"},this.checkUI(),this.checkErrors(),this.checkFunctionTranslations(this.functions),Object.assign(this.functions,this._protectedTranslations)}extendFunctions(e){this.checkFunctionTranslations(e),Object.assign(this.functions,e)}buildFunctionMapping(){return Object.keys(this.functions).reduce((e,t)=>(e[this.functions[t]]=t,e),{})}buildErrorMapping(){return Object.keys(this.errors).reduce((e,t)=>(e[this.errors[t]]=t,e),{})}isFunctionTranslated(e){return void 0!==this.functions[e]}getFunctionTranslations(e){const t=[];for(const r of e)this.isFunctionTranslated(r)&&t.push(this.functions[r]);return t}getFunctionTranslation(e){const t=this.functions[e];if(void 0===t)throw new o.MissingTranslationError("functions."+e);return t}getMaybeFunctionTranslation(e){return this.functions[e]}getErrorTranslation(e){if(e===n.ErrorType.LIC)return`#${n.ErrorType.LIC}!`;const t=this.errors[e];if(void 0===t)throw new o.MissingTranslationError("errors."+e);return t}getUITranslation(e){const t=this.ui[e];if(void 0===t)throw new o.MissingTranslationError("ui."+e);return t}checkUI(){for(const e of Object.values(s.UIElement))if(!(e in this.ui))throw new o.MissingTranslationError("ui."+e)}checkErrors(){for(const e of Object.values(n.ErrorType))if(!(e in this.errors)&&e!==n.ErrorType.LIC)throw new o.MissingTranslationError("errors."+e)}checkFunctionTranslations(e){const t=new Set(Object.getOwnPropertyNames(e));for(const e of Object.getOwnPropertyNames(this._protectedTranslations))if(t.has(e))throw new o.ProtectedFunctionTranslationError(e)}}t.TranslationPackage=i},function(e,t,r){"use strict";t.__esModule=!0,t.EmptyCellVertex=void 0;var n=r(141);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.EmptyCellVertex=class{constructor(){}getCellValue(){return n.EmptyValue}}},function(e,t,r){"use strict";t.__esModule=!0,t.ValueCellVertex=void 0;t.ValueCellVertex=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t){this.parsedValue=e,this.rawValue=t}getValues(){return{parsedValue:this.parsedValue,rawValue:this.rawValue}}setValues(e){this.parsedValue=e.parsedValue,this.rawValue=e.rawValue}getCellValue(){return this.parsedValue}setCellValue(e){throw Error("SetCellValue is deprecated for ValueCellVertex")}}},function(e,t,r){"use strict";t.__esModule=!0,t.ParsingErrorVertex=void 0;var n=r(74);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.ParsingErrorVertex=class{constructor(e,t){this.errors=e,this.rawInput=t}getCellValue(){return n.CellError.parsingError()}getFormula(){return this.rawInput}}},function(e,t,r){"use strict";t.__esModule=!0,t.SparseStrategy=void 0;var n=r(74);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.SparseStrategy=class{constructor(e,t){this.width=e,this.height=t,this.mapping=new Map}getCell(e){var t;return null===(t=this.mapping.get(e.col))||void 0===t?void 0:t.get(e.row)}setCell(e,t){this.width=Math.max(this.width,e.col+1),this.height=Math.max(this.height,e.row+1);let r=this.mapping.get(e.col);r||(r=new Map,this.mapping.set(e.col,r)),r.set(e.row,t)}has(e){var t;return!!(null===(t=this.mapping.get(e.col))||void 0===t?void 0:t.get(e.row))}getHeight(){return this.height}getWidth(){return this.width}removeCell(e){var t;null===(t=this.mapping.get(e.col))||void 0===t||t.delete(e.row)}addRows(e,t){this.mapping.forEach(r=>{const n=new Map;r.forEach((o,s)=>{s>=e&&(n.set(s+t,o),r.delete(s))}),n.forEach((e,t)=>{r.set(t,e)})}),this.height+=t}addColumns(e,t){const r=new Map;this.mapping.forEach((n,o)=>{o>=e&&(r.set(o+t,n),this.mapping.delete(o))}),r.forEach((e,t)=>{this.mapping.set(t,e)}),this.width+=t}removeRows(e){this.mapping.forEach(t=>{const r=new Map;t.forEach((n,o)=>{o>=e.rowStart&&(t.delete(o),o>e.rowEnd&&r.set(o-e.numberOfRows,n))}),r.forEach((e,r)=>{t.set(r,e)})});const t=Math.min(this.height-1,e.rowEnd),r=Math.max(0,t-e.rowStart+1);this.height=Math.max(0,this.height-r)}removeColumns(e){const t=new Map;this.mapping.forEach((r,n)=>{n>=e.columnStart&&(this.mapping.delete(n),n>e.columnEnd&&t.set(n-e.numberOfColumns,r))}),t.forEach((e,t)=>{this.mapping.set(t,e)});const r=Math.min(this.width-1,e.columnEnd),n=Math.max(0,r-e.columnStart+1);this.width=Math.max(0,this.width-n)}*getEntries(e){for(const[t,r]of this.mapping)for(const[o,s]of r)yield[(0,n.simpleCellAddress)(e,t,o),s]}*verticesFromColumn(e){const t=this.mapping.get(e);if(void 0!==t)for(const[e,r]of t)yield r}*verticesFromRow(e){for(const t of this.mapping.values()){const r=t.get(e);void 0!==r&&(yield r)}}*verticesFromColumnsSpan(e){for(const t of e.columns()){const e=this.mapping.get(t);if(void 0!==e)for(const[t,r]of e)yield r}}*verticesFromRowsSpan(e){for(const t of this.mapping.values())for(const r of e.rows()){const e=t.get(r);void 0!==e&&(yield e)}}*entriesFromRowsSpan(e){for(const[t,r]of this.mapping.entries())for(const o of e.rows()){const s=r.get(o);void 0!==s&&(yield[(0,n.simpleCellAddress)(e.sheet,t,o),s])}}*entriesFromColumnsSpan(e){for(const t of e.columns()){const r=this.mapping.get(t);if(void 0!==r)for(const[o,s]of r.entries())yield[(0,n.simpleCellAddress)(e.sheet,t,o),s]}}*vertices(){for(const[e,t]of this.mapping)for(const[e,r]of t)void 0!==r&&(yield r)}}},function(e,t,r){"use strict";t.__esModule=!0,t.DenseStrategy=void 0,r(2);var n=r(74);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.DenseStrategy=class{constructor(e,t){this.width=e,this.height=t,this.mapping=new Array(t);for(let r=0;r<t;r++)this.mapping[r]=new Array(e)}getCell(e){return this.getCellVertex(e.col,e.row)}setCell(e,t){this.width=Math.max(this.width,e.col+1),this.height=Math.max(this.height,e.row+1);this.mapping[e.row]||(this.mapping[e.row]=new Array(this.width)),this.mapping[e.row][e.col]=t}has(e){const t=this.mapping[e.row];return!!t&&!!t[e.col]}getHeight(){return this.height}getWidth(){return this.width}removeCell(e){void 0!==this.mapping[e.row]&&delete this.mapping[e.row][e.col]}addRows(e,t){const r=[];for(let e=0;e<t;e++)r.push(new Array(this.width));this.mapping.splice(e,0,...r),this.height+=t}addColumns(e,t){for(let r=0;r<this.height;r++)this.mapping[r].splice(e,0,...new Array(t));this.width+=t}removeRows(e){this.mapping.splice(e.rowStart,e.numberOfRows);const t=Math.min(this.height-1,e.rowEnd),r=Math.max(0,t-e.rowStart+1);this.height=Math.max(0,this.height-r)}removeColumns(e){for(let t=0;t<this.height;t++)this.mapping[t].splice(e.columnStart,e.numberOfColumns);const t=Math.min(this.width-1,e.columnEnd),r=Math.max(0,t-e.columnStart+1);this.width=Math.max(0,this.width-r)}*getEntries(e){for(let t=0;t<this.height;++t)for(let r=0;r<this.width;++r){const o=this.getCellVertex(r,t);o&&(yield[(0,n.simpleCellAddress)(e,r,t),o])}}*verticesFromColumn(e){for(let t=0;t<this.height;++t){const r=this.getCellVertex(e,t);r&&(yield r)}}*verticesFromRow(e){for(let t=0;t<this.width;++t){const r=this.getCellVertex(t,e);r&&(yield r)}}*verticesFromColumnsSpan(e){for(let t=e.columnStart;t<=e.columnEnd;++t)for(let e=0;e<this.height;++e){const r=this.getCellVertex(t,e);r&&(yield r)}}*verticesFromRowsSpan(e){for(let t=0;t<this.width;++t)for(let r=e.rowStart;r<=e.rowEnd;++r){const e=this.getCellVertex(t,r);e&&(yield e)}}*entriesFromRowsSpan(e){for(let t=0;t<this.width;++t)for(let r=e.rowStart;r<=e.rowEnd;++r){const o=this.getCellVertex(t,r);o&&(yield[(0,n.simpleCellAddress)(e.sheet,t,r),o])}}*entriesFromColumnsSpan(e){for(let t=e.columnStart;t<=e.columnEnd;++t)for(let r=0;r<this.height;++r){const o=this.getCellVertex(t,r);o&&(yield[(0,n.simpleCellAddress)(e.sheet,t,r),o])}}*vertices(){for(let e=0;e<this.height;++e)for(let t=0;t<this.width;++t){const r=this.getCellVertex(t,e);r&&(yield r)}}getCellVertex(e,t){var r;return null===(r=this.mapping[t])||void 0===r?void 0:r[e]}}},function(e,t,r){"use strict";t.__esModule=!0,t.InterpreterState=void 0;t.InterpreterState=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r){this.formulaAddress=e,this.arraysFlag=t,this.formulaVertex=r}}},function(e,t,r){"use strict";t.__esModule=!0,t.FunctionPlugin=t.FunctionArgumentType=void 0,r(2);var n,o=r(73),s=r(74),i=r(132),a=r(78),u=r(166),c=r(141),l=r(140);t.FunctionArgumentType=n,function(e){e.STRING="STRING",e.NUMBER="NUMBER",e.BOOLEAN="BOOLEAN",e.SCALAR="SCALAR",e.NOERROR="NOERROR",e.RANGE="RANGE",e.INTEGER="INTEGER",e.COMPLEX="COMPLEX",e.ANY="ANY"}(n||(t.FunctionArgumentType=n={}));t.FunctionPlugin=class{constructor(e){var t=this;this.coerceScalarToNumberOrError=e=>this.arithmeticHelper.coerceScalarToNumberOrError(e),this.runFunction=(e,t,r,n)=>{const o=this.evaluateArguments(e,t,r),a=o.map(e=>{let[t,r]=e;return t}),u=o.map(e=>{let[t,r]=e;return r}),c=this.buildMetadataForEachArgumentValue(a.length,r),h=t.arraysFlag&&!r.vectorizationForbidden;if(!this.isNumberOfArgumentValuesValid(c,a.length))return new s.CellError(s.ErrorType.NA,i.ErrorMessage.WrongArgNumber);const[d,p]=h?this.calculateSizeOfVectorizedResultArray(a,c):[1,1];if(1===d&&1===p){const e=this.vectorizeAndBroadcastArgumentsIfNecessary(h,a,c,0,0);return this.calculateSingleCellOfResultArray(t,e,c,u,n,r.returnNumberType)}const m=[...Array(d).keys()].map(e=>[...Array(p).keys()].map(o=>{const s=this.vectorizeAndBroadcastArgumentsIfNecessary(h,a,c,e,o),i=this.calculateSingleCellOfResultArray(t,s,c,u,n,r.returnNumberType);if(i instanceof l.SimpleRangeValue)throw new Error("Function returning array cannot be vectorized.");return i}));return l.SimpleRangeValue.onlyValues(m)},this.runFunctionWithReferenceArgument=function(e,r,n,u,c){let l=arguments.length>5&&void 0!==arguments[5]?arguments[5]:()=>new s.CellError(s.ErrorType.NA,i.ErrorMessage.CellRefExpected);if(0===e.length)return t.returnNumberWrapper(u(),n.returnNumberType);if(e.length>1)return new s.CellError(s.ErrorType.NA,i.ErrorMessage.WrongArgNumber);let h,d=e[0];for(;d.type===a.AstNodeType.PARENTHESIS;)d=d.expression;if(d.type===a.AstNodeType.CELL_REFERENCE)h=d.reference.toSimpleCellAddress(r.formulaAddress);else if(d.type===a.AstNodeType.CELL_RANGE||d.type===a.AstNodeType.COLUMN_RANGE||d.type===a.AstNodeType.ROW_RANGE)try{h=o.AbsoluteCellRange.fromAst(d,r.formulaAddress).start}catch(e){return new s.CellError(s.ErrorType.REF,i.ErrorMessage.CellRefExpected)}return void 0!==h?t.returnNumberWrapper(c(h),n.returnNumberType):t.runFunction(e,r,n,l)},this.interpreter=e,this.dependencyGraph=e.dependencyGraph,this.columnSearch=e.columnSearch,this.config=e.config,this.serialization=e.serialization,this.arraySizePredictor=e.arraySizePredictor,this.dateTimeHelper=e.dateTimeHelper,this.arithmeticHelper=e.arithmeticHelper}evaluateAst(e,t){return this.interpreter.evaluateAst(e,t)}arraySizeForAst(e,t){return this.arraySizePredictor.checkArraySizeForAst(e,t)}listOfScalarValues(e,t){const r=[];for(const n of e){const e=this.evaluateAst(n,t);if(e instanceof l.SimpleRangeValue)for(const t of e.valuesFromTopLeftCorner())r.push([t,!0]);else r.push([e,!1])}return r}coerceToType(e,t,r){let o;if(e instanceof l.SimpleRangeValue)switch(t.argumentType){case n.RANGE:case n.ANY:o=e;break;default:{const t=(0,u.coerceRangeToScalar)(e,r);if(void 0===t)return;e=t}}if(!(e instanceof l.SimpleRangeValue))switch(t.argumentType){case n.INTEGER:case n.NUMBER:const r=this.coerceScalarToNumberOrError(e);if(!(0,c.isExtendedNumber)(r)){o=r;break}const a=(0,c.getRawValue)(r);if(void 0!==t.maxValue&&a>t.maxValue)return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ValueLarge);if(void 0!==t.minValue&&a<t.minValue)return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ValueSmall);if(void 0!==t.lessThan&&a>=t.lessThan)return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ValueLarge);if(void 0!==t.greaterThan&&a<=t.greaterThan)return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ValueSmall);if(t.argumentType===n.INTEGER&&!Number.isInteger(a))return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.IntegerExpected);o=r;break;case n.STRING:o=(0,u.coerceScalarToString)(e);break;case n.BOOLEAN:o=(0,u.coerceScalarToBoolean)(e);break;case n.SCALAR:case n.NOERROR:case n.ANY:o=e;break;case n.RANGE:if(e instanceof s.CellError)return e;o=(0,u.coerceToRange)(e);break;case n.COMPLEX:return this.arithmeticHelper.coerceScalarToComplex((0,c.getRawValue)(e))}return t.passSubtype||void 0===o?o:(0,c.getRawValue)(o)}calculateSingleCellOfResultArray(e,t,r,n,o,i){const a=this.coerceArgumentsToRequiredTypes(e,t,r,n);if(a instanceof s.CellError)return a;const u=o(...a);return this.returnNumberWrapper(u,i)}coerceArgumentsToRequiredTypes(e,t,r,o){const a=[];for(let u=0;u<r.length;u++){const c=r[u],l=void 0!==t[u]?t[u]:null==c?void 0:c.defaultValue;if(void 0===l){a.push(void 0);continue}const h=this.coerceToType(l,c,e);if(void 0===h&&!o[u])return new s.CellError(s.ErrorType.VALUE,i.ErrorMessage.WrongType);if(h instanceof s.CellError&&c.argumentType!==n.SCALAR)return h;a.push(h)}return a}vectorizeAndBroadcastArgumentsIfNecessary(e,t,r,n,o){return t.map((t,s)=>e&&this.isRangePassedAsAScalarArgument(t,r[s])?this.vectorizeAndBroadcastRangeArgument(t,n,o):t)}vectorizeAndBroadcastRangeArgument(e,t,r){var n;const o=1===e.height()?0:t,s=1===e.width()?0:r;return null===(n=e.data[o])||void 0===n?void 0:n[s]}evaluateArguments(e,t,r){return r.expandRanges?this.listOfScalarValues(e,t):e.map(e=>[this.evaluateAst(e,t),!1])}buildMetadataForEachArgumentValue(e,t){const r=t.parameters?[...t.parameters]:[];if(void 0!==t.repeatLastArgs&&Number.isInteger(t.repeatLastArgs)&&t.repeatLastArgs>0)for(;e>r.length;)r.push(...r.slice(r.length-t.repeatLastArgs));return r}isNumberOfArgumentValuesValid(e,t){if(t>e.length)return!1;if(t<e.length){return e.slice(t).every(e=>(null==e?void 0:e.optionalArg)||void 0!==(null==e?void 0:e.defaultValue))}return!0}calculateSizeOfVectorizedResultArray(e,t){const r=e.filter((e,r)=>this.isRangePassedAsAScalarArgument(e,t[r]));return[Math.max(1,...r.map(e=>e.height())),Math.max(1,...r.map(e=>e.width()))]}isRangePassedAsAScalarArgument(e,t){return null!=e&&null!=t&&(e instanceof l.SimpleRangeValue&&![n.RANGE,n.ANY].includes(t.argumentType))}metadata(e){const t=this.constructor.implementedFunctions[e];if(void 0!==t)return t;throw new Error(`No metadata for function ${e}.`)}returnNumberWrapper(e,t,r){return void 0!==t&&(0,c.isExtendedNumber)(e)?this.arithmeticHelper.ExtendedNumberFactory((0,c.getRawValue)(e),{type:t,format:r}):e}}},function(e,t,r){"use strict";t.__esModule=!0,t.ArithmeticHelper=void 0,t.coerceBooleanToNumber=h,t.coerceComplexToString=function(e,t){let[r,n]=e;if(!isFinite(r)||!isFinite(n))return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.NaN);if(t=null!=t?t:"i",0===n)return""+r;const o=`${-1===n||1===n?"":Math.abs(n)}${t}`;if(0===r)return`${n<0?"-":""}${o}`;return`${r}${n<0?"-":"+"}${o}`},t.coerceEmptyToValue=d,t.coerceRangeToScalar=function(e,t){var r;if(e.isAdHoc())return null===(r=e.data[0])||void 0===r?void 0:r[0];const n=e.range;if(t.formulaAddress.sheet===n.sheet)if(1===n.width()){const r=t.formulaAddress.row-n.start.row;if(r>=0&&r<n.height())return e.data[r][0]}else if(1===n.height()){const r=t.formulaAddress.col-n.start.col;if(r>=0&&r<n.width())return e.data[0][r]}return},t.coerceScalarToBoolean=function(e){if(e instanceof s.CellError||"boolean"==typeof e)return e;if(e===u.EmptyValue)return!1;if((0,u.isExtendedNumber)(e))return 0!==(0,u.getRawValue)(e);{const t=e.toUpperCase();return"TRUE"===t||"FALSE"!==t&&(""!==t&&void 0)}},t.coerceScalarToString=function(e){return e instanceof s.CellError||"string"==typeof e?e:e===u.EmptyValue?"":(0,u.isExtendedNumber)(e)?(0,u.getRawValue)(e).toString():e?"TRUE":"FALSE"},t.coerceToRange=function(e){return e instanceof c.SimpleRangeValue?e:c.SimpleRangeValue.fromScalar(e)},t.coerceToRangeNumbersOrError=function(e){return e instanceof c.SimpleRangeValue&&e.hasOnlyNumbers()||e instanceof s.CellError?e:(0,u.isExtendedNumber)(e)?c.SimpleRangeValue.fromScalar(e):null},t.fixNegativeZero=function(e){return 0===e?0:e},t.forceNormalizeString=function(e){return y(e.toLowerCase(),"nfd").replace(/[\u0300-\u036f]/g,"")},t.isNumberOverflow=function(e){return isNaN(e)||e===1/0||e===-1/0},t.normalizeString=y,t.numberCmp=p,t.zeroIfEmpty=function(e){return e===u.EmptyValue?0:e},r(2);var n,o=(n=r(167))&&n.__esModule?n:{default:n},s=r(74),i=r(132),a=r(168),u=r(141),c=r(140);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const l=/^\s*([+-]?)\s*(([\d\.,]+(e[+-]?\d+)?)\s*([ij]?)|([ij]))\s*(([+-])\s*([+-]?)\s*(([\d\.,]+(e[+-]?\d+)?)\s*([ij]?)|([ij])))?$/;function h(e){return Number(e)}function d(e){return"string"==typeof e?"":(0,u.isExtendedNumber)(e)?0:"boolean"!=typeof e&&u.EmptyValue}function p(e,t){const r=(0,u.getRawValue)(e),n=(0,u.getRawValue)(t);return r>n?1:r<n?-1:0}function m(e){return["*","?"].includes(e)}t.ArithmeticHelper=class{constructor(e,t,r){this.config=e,this.dateTimeHelper=t,this.numberLiteralsHelper=r,this.lt=(e,t)=>this.compare(e,t)<0,this.leq=(e,t)=>this.compare(e,t)<=0,this.gt=(e,t)=>this.compare(e,t)>0,this.geq=(e,t)=>this.compare(e,t)>=0,this.eq=(e,t)=>0===this.compare(e,t),this.neq=(e,t)=>0!==this.compare(e,t),this.pow=(e,t)=>Math.pow((0,u.getRawValue)(e),(0,u.getRawValue)(t)),this.addWithEpsilonRaw=(e,t)=>{const r=e+t;return Math.abs(r)<this.actualEps*Math.abs(e)?0:r},this.addWithEpsilon=(e,t)=>{const r=E(e,t);return this.ExtendedNumberFactory(this.addWithEpsilonRaw((0,u.getRawValue)(e),(0,u.getRawValue)(t)),r)},this.unaryMinus=e=>(0,u.cloneNumber)(e,-(0,u.getRawValue)(e)),this.unaryPlus=e=>e,this.unaryPercent=e=>new u.PercentNumber((0,u.getRawValue)(e)/100),this.concat=(e,t)=>e.concat(t),this.nonstrictadd=(e,t)=>e instanceof s.CellError?e:t instanceof s.CellError?t:"number"==typeof e?"number"==typeof t?this.addWithEpsilonRaw(e,t):e:"number"==typeof t?t:0,this.subtract=(e,t)=>{const r=E(e,t),n=(0,u.getRawValue)(e);let o=n-(0,u.getRawValue)(t);return Math.abs(o)<this.actualEps*Math.abs(n)&&(o=0),this.ExtendedNumberFactory(o,r)},this.divide=(e,t)=>{const r=(0,u.getRawValue)(e),n=(0,u.getRawValue)(t);if(0===n)return new s.CellError(s.ErrorType.DIV_BY_ZERO);{const o=T(e,t);return this.ExtendedNumberFactory(r/n,o)}},this.multiply=(e,t)=>{const r=T(e,t);return this.ExtendedNumberFactory((0,u.getRawValue)(e)*(0,u.getRawValue)(t),r)},this.manyToExactComplex=e=>{const t=[];for(const r of e){if(r instanceof s.CellError)return r;if((0,u.isExtendedNumber)(r)||"string"==typeof r){const e=this.coerceScalarToComplex(r);e instanceof s.CellError||t.push(e)}}return t},this.coerceNumbersExactRanges=e=>this.manyToNumbers(e,this.manyToExactNumbers),this.coerceNumbersCoerceRangesDropNulls=e=>this.manyToNumbers(e,this.manyToCoercedNumbersDropNulls),this.manyToExactNumbers=e=>{const t=[];for(const r of e){if(r instanceof s.CellError)return r;(0,u.isExtendedNumber)(r)&&t.push((0,u.getRawValue)(r))}return t},this.manyToOnlyNumbersDropNulls=e=>{const t=[];for(const r of e){if(r instanceof s.CellError)return r;if((0,u.isExtendedNumber)(r))t.push((0,u.getRawValue)(r));else if(r!==u.EmptyValue)return new s.CellError(s.ErrorType.VALUE,i.ErrorMessage.NumberExpected)}return t},this.manyToCoercedNumbersDropNulls=e=>{const t=[];for(const r of e){if(r instanceof s.CellError)return r;if(r===u.EmptyValue)continue;const e=this.coerceScalarToNumberOrError(r);(0,u.isExtendedNumber)(e)&&t.push((0,u.getRawValue)(e))}return t},this.collator=(0,a.collatorFromConfig)(e),this.actualEps=e.smartRounding?e.precisionEpsilon:0}eqMatcherFunction(e){const t=this.buildRegex(e);return e=>"string"==typeof e&&t.test(this.normalizeString(e))}neqMatcherFunction(e){const t=this.buildRegex(e);return e=>!("string"==typeof e&&t.test(this.normalizeString(e)))}searchString(e,t){var r;const n=this.buildRegex(e,!1).exec(t);return null!==(r=null==n?void 0:n.index)&&void 0!==r?r:-1}requiresRegex(e){if(!this.config.useRegularExpressions&&!this.config.useWildcards)return!this.config.matchWholeCell;for(let t=0;t<e.length;t++){const r=e.charAt(t);if(m(r)||this.config.useRegularExpressions&&f(r))return!0}return!1}floatCmp(e,t){const r=(0,u.getRawValue)(e),n=(0,u.getRawValue)(t),o=1+this.actualEps;return n>=0&&r*o>=n&&r<=n*o||n<=0&&r*o<=n&&r>=n*o?0:r>n?1:-1}coerceScalarToNumberOrError(e){var t;return e instanceof s.CellError?e:null!==(t=this.coerceToMaybeNumber(e))&&void 0!==t?t:new s.CellError(s.ErrorType.VALUE,i.ErrorMessage.NumberCoercion)}coerceToMaybeNumber(e){var t;return null!==(t=this.coerceNonDateScalarToMaybeNumber(e))&&void 0!==t?t:"string"==typeof e?this.dateTimeHelper.dateStringToDateNumber(e):void 0}coerceNonDateScalarToMaybeNumber(e){if(e===u.EmptyValue)return 0;if("string"==typeof e){if(""===e)return 0;const t=this.coerceStringToMaybePercentNumber(e);if(void 0!==t)return t;const r=this.coerceStringToMaybeCurrencyNumber(e);return void 0!==r?r:this.numberLiteralsHelper.numericStringToMaybeNumber(e.trim())}return(0,u.isExtendedNumber)(e)?e:"boolean"==typeof e?Number(e):void 0}coerceStringToMaybePercentNumber(e){const t=e.trim();if(t.endsWith("%")){const e=t.slice(0,t.length-1).trim(),r=this.numberLiteralsHelper.numericStringToMaybeNumber(e);if(void 0!==r)return new u.PercentNumber(r/100)}}coerceStringToMaybeCurrencyNumber(e){const t=this.currencyMatcher(e.trim());if(void 0!==t){const[e,r]=t,n=this.numberLiteralsHelper.numericStringToMaybeNumber(r);if(void 0!==n)return new u.CurrencyNumber(n,e)}}currencyMatcher(e){for(const t of this.config.currencySymbol){if(e.startsWith(t))return[t,e.slice(t.length).trim()];if(e.endsWith(t))return[t,e.slice(0,e.length-t.length).trim()]}}coerceComplexExactRanges(e){const t=[];for(const r of e)if(r instanceof c.SimpleRangeValue)t.push(r);else if(r!==u.EmptyValue){const e=this.coerceScalarToComplex(r);if(e instanceof s.CellError)return e;t.push(e)}const r=[];for(const e of t)if(e instanceof c.SimpleRangeValue){const t=this.manyToExactComplex(e.valuesFromTopLeftCorner());if(t instanceof s.CellError)return t;r.push(...t)}else r.push(e);return r}coerceScalarToComplex(e){return e instanceof s.CellError?e:e===u.EmptyValue?[0,0]:(0,u.isExtendedNumber)(e)?[(0,u.getRawValue)(e),0]:"string"==typeof e?this.coerceStringToComplex(e):new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ComplexNumberExpected)}ExtendedNumberFactory(e,t){const{type:r,format:n}=t;switch(r){case u.NumberType.NUMBER_RAW:return e;case u.NumberType.NUMBER_CURRENCY:return new u.CurrencyNumber(e,null!=n?n:this.config.currencySymbol[0]);case u.NumberType.NUMBER_DATE:return new u.DateNumber(e,n);case u.NumberType.NUMBER_DATETIME:return new u.DateTimeNumber(e,n);case u.NumberType.NUMBER_TIME:return new u.TimeNumber(e,n);case u.NumberType.NUMBER_PERCENT:return new u.PercentNumber(e,n)}}buildRegex(e){let t,r=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];e=this.normalizeString(e);let n=this.config.useWildcards,o=this.config.useRegularExpressions;if(o)try{RegExp(e)}catch(e){o=!1,n=!1}return t=o?function(e,t){let r="";for(let n=0;n<e.length;n++){const o=e.charAt(n);m(o)||f(o)?r+=o:r+=t?o:o.toLowerCase()}return r}(e,this.config.caseSensitive):n?function(e,t){let r="";for(let n=0;n<e.length;n++){const o=e.charAt(n);if("~"===o){if(n==e.length-1){r+="~";continue}const t=e.charAt(n+1);m(t)||f(t)?(r+="\\"+t,n++):(r+=t,n++)}else m(o)?r+="."+o:f(o)?r+="\\"+o:r+=t?o:o.toLowerCase()}return r}(e,this.config.caseSensitive):function(e,t){let r="";for(let n=0;n<e.length;n++){const o=e.charAt(n);m(o)||f(o)?r+="\\"+o:r+=t?o:o.toLowerCase()}return r}(e,this.config.caseSensitive),this.config.matchWholeCell&&r?RegExp("^("+t+")$"):RegExp(t)}normalizeString(e){return this.config.caseSensitive||(e=e.toLowerCase()),this.config.accentSensitive||(e=y(e,"nfd").replace(/[\u0300-\u036f]/g,"")),e}compare(e,t){if("string"==typeof e||"string"==typeof t){const r="string"==typeof e?this.dateTimeHelper.dateStringToDateNumber(e):e,n="string"==typeof t?this.dateTimeHelper.dateStringToDateNumber(t):t;if((0,u.isExtendedNumber)(r)&&(0,u.isExtendedNumber)(n))return this.floatCmp(r,n)}return e===u.EmptyValue?e=d(t):t===u.EmptyValue&&(t=d(e)),"string"==typeof e&&"string"==typeof t?this.stringCmp(e,t):"boolean"==typeof e&&"boolean"==typeof t?p(h(e),h(t)):(0,u.isExtendedNumber)(e)&&(0,u.isExtendedNumber)(t)?this.floatCmp(e,t):e===u.EmptyValue&&t===u.EmptyValue?0:p((0,s.CellValueTypeOrd)((0,s.getCellValueType)(e)),(0,s.CellValueTypeOrd)((0,s.getCellValueType)(t)))}stringCmp(e,t){return this.collator.compare(e,t)}manyToNumbers(e,t){const r=[];for(const t of e)if(t instanceof c.SimpleRangeValue)r.push(t);else{const e=(0,u.getRawValue)(this.coerceScalarToNumberOrError(t));if(e instanceof s.CellError)return e;r.push(e)}const n=[];for(const e of r)if(e instanceof c.SimpleRangeValue){const r=t(e.valuesFromTopLeftCorner());if(r instanceof s.CellError)return r;n.push(...r)}else n.push(e);return n}coerceStringToComplex(e){const t=l.exec(e);if(null===t)return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ComplexNumberExpected);let r,n;return r=void 0!==t[6]?"-"===t[1]?[0,-1]:[0,1]:this.parseComplexToken(t[1]+t[3],t[5]),r instanceof s.CellError||void 0===t[8]?r:(n=void 0!==t[14]?"-"===t[9]?[0,-1]:[0,1]:this.parseComplexToken(t[9]+t[11],t[13]),n instanceof s.CellError?n:""!==t[5]||""===t[13]?new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ComplexNumberExpected):"+"===t[8]?[r[0]+n[0],r[1]+n[1]]:[r[0]-n[0],r[1]-n[1]])}parseComplexToken(e,t){const r=(0,u.getRawValue)(this.coerceNonDateScalarToMaybeNumber(e));return void 0===r?new s.CellError(s.ErrorType.NUM,i.ErrorMessage.ComplexNumberExpected):""===t?[r,0]:[0,r]}};const g=["{","}","[","]","(",")","<",">","=",".","+","-",",","\\","$","^","!"];function f(e){return g.includes(e)}function E(e,t){const{type:r,format:n}=(0,u.getTypeFormatOfExtendedNumber)(e),{type:o,format:s}=(0,u.getTypeFormatOfExtendedNumber)(t);if(r===u.NumberType.NUMBER_RAW)return{type:o,format:s};if(o===u.NumberType.NUMBER_RAW)return{type:r,format:n};if(!(r!==u.NumberType.NUMBER_DATETIME&&r!==u.NumberType.NUMBER_DATE||o!==u.NumberType.NUMBER_DATETIME&&o!==u.NumberType.NUMBER_DATE))return{type:u.NumberType.NUMBER_RAW};if(r===u.NumberType.NUMBER_TIME){if(o===u.NumberType.NUMBER_DATE)return{type:u.NumberType.NUMBER_DATETIME,format:`${s} ${n}`};if(o===u.NumberType.NUMBER_DATETIME)return{type:u.NumberType.NUMBER_DATETIME,format:s}}if(o===u.NumberType.NUMBER_TIME){if(r===u.NumberType.NUMBER_DATE)return{type:u.NumberType.NUMBER_DATETIME,format:`${n} ${s}`};if(r===u.NumberType.NUMBER_DATETIME)return{type:u.NumberType.NUMBER_DATETIME,format:n}}return{type:r,format:n}}function T(e,t){let{type:r,format:n}=(0,u.getTypeFormatOfExtendedNumber)(e),{type:o,format:s}=(0,u.getTypeFormatOfExtendedNumber)(t);return r===u.NumberType.NUMBER_PERCENT&&(r=u.NumberType.NUMBER_RAW,n=void 0),o===u.NumberType.NUMBER_PERCENT&&(o=u.NumberType.NUMBER_RAW,s=void 0),r===u.NumberType.NUMBER_RAW?{type:o,format:s}:o===u.NumberType.NUMBER_RAW?{type:r,format:n}:{type:u.NumberType.NUMBER_RAW}}function y(e,t){return"function"==typeof e.normalize?e.normalize(t.toUpperCase()):o.default[t](e)}},function(e,t,r){!function(t){"use strict";for(var r=[null,0,{}],n=function(e,t){this.codepoint=e,this.feature=t},o={},s=[],i=0;i<=255;++i)s[i]=0;var a=[function(e,t,o){return t<60||13311<t&&t<42607?new n(t,r):e(t,o)},function(e,t,r){var n=o[t];return n||(n=e(t,r)).feature&&++s[t>>8&255]>10&&(o[t]=n),n},function(e,t,r){return r?e(t,r):new n(t,null)},function(e,t,r){var o;if(t<4352||4371<=t&&t<44032||55204<t)return e(t,r);if(4352<=t&&t<4371){var s={},i=21*(t-4352);for(o=0;o<21;++o)s[4449+o]=44032+28*(o+i);return new n(t,[,,s])}var a=t-44032,u=a%28,c=[];if(0!==u)c[0]=[44032+a-u,4519+u];else for(c[0]=[4352+Math.floor(a/588),4449+Math.floor(a%588/28)],c[2]={},o=1;o<28;++o)c[2][4519+o]=t+o;return new n(t,c)},function(e,t,o){var s=(n.udata[65280&t]||{})[t];return new n(t,s||r)}];n.fromCharCode=a.reduceRight((function(e,t){return function(r,n){return t(e,r,n)}}),null),n.isHighSurrogate=function(e){return e>=55296&&e<=56319},n.isLowSurrogate=function(e){return e>=56320&&e<=57343},n.prototype.prepFeature=function(){this.feature||(this.feature=n.fromCharCode(this.codepoint,!0).feature)},n.prototype.toString=function(){if(this.codepoint<65536)return String.fromCharCode(this.codepoint);var e=this.codepoint-65536;return String.fromCharCode(Math.floor(e/1024)+55296,e%1024+56320)},n.prototype.getDecomp=function(){return this.prepFeature(),this.feature[0]||null},n.prototype.isCompatibility=function(){return this.prepFeature(),!!this.feature[1]&&256&this.feature[1]},n.prototype.isExclude=function(){return this.prepFeature(),!!this.feature[1]&&512&this.feature[1]},n.prototype.getCanonicalClass=function(){return this.prepFeature(),this.feature[1]?255&this.feature[1]:0},n.prototype.getComposite=function(e){if(this.prepFeature(),!this.feature[2])return null;var t=this.feature[2][e.codepoint];return t?n.fromCharCode(t):null};var u=function(e){this.str=e,this.cursor=0};u.prototype.next=function(){if(this.str&&this.cursor<this.str.length){var e,t=this.str.charCodeAt(this.cursor++);return n.isHighSurrogate(t)&&this.cursor<this.str.length&&n.isLowSurrogate(e=this.str.charCodeAt(this.cursor))&&(t=1024*(t-55296)+(e-56320)+65536,++this.cursor),n.fromCharCode(t)}return this.str=null,null};var c=function(e,t){this.it=e,this.canonical=t,this.resBuf=[]};c.prototype.next=function(){if(0===this.resBuf.length){var e=this.it.next();if(!e)return null;this.resBuf=function e(t,r){var o=r.getDecomp();if(!o||t&&r.isCompatibility())return[r];for(var s=[],i=0;i<o.length;++i){var a=e(t,n.fromCharCode(o[i]));s=s.concat(a)}return s}(this.canonical,e)}return this.resBuf.shift()};var l=function(e){this.it=e,this.resBuf=[]};l.prototype.next=function(){var e;if(0===this.resBuf.length)do{var t=this.it.next();if(!t)break;e=t.getCanonicalClass();var r=this.resBuf.length;if(0!==e)for(;r>0;--r){if(this.resBuf[r-1].getCanonicalClass()<=e)break}this.resBuf.splice(r,0,t)}while(0!==e);return this.resBuf.shift()};var h=function(e){this.it=e,this.procBuf=[],this.resBuf=[],this.lastClass=null};h.prototype.next=function(){for(;0===this.resBuf.length;){var e=this.it.next();if(!e){this.resBuf=this.procBuf,this.procBuf=[];break}if(0===this.procBuf.length)this.lastClass=e.getCanonicalClass(),this.procBuf.push(e);else{var t=this.procBuf[0].getComposite(e),r=e.getCanonicalClass();t&&(this.lastClass<r||0===this.lastClass)?this.procBuf[0]=t:(0===r&&(this.resBuf=this.procBuf,this.procBuf=[]),this.lastClass=r,this.procBuf.push(e))}}return this.resBuf.shift()};var d=function(e,t){for(var r,n=function(e,t){switch(e){case"NFD":return new l(new c(new u(t),!0));case"NFKD":return new l(new c(new u(t),!1));case"NFC":return new h(new l(new c(new u(t),!0)));case"NFKC":return new h(new l(new c(new u(t),!1)))}throw e+" is invalid"}(e,t),o="";r=n.next();)o+=r.toString();return o};n.udata={0:{60:[,,{824:8814}],61:[,,{824:8800}],62:[,,{824:8815}],65:[,,{768:192,769:193,770:194,771:195,772:256,774:258,775:550,776:196,777:7842,778:197,780:461,783:512,785:514,803:7840,805:7680,808:260}],66:[,,{775:7682,803:7684,817:7686}],67:[,,{769:262,770:264,775:266,780:268,807:199}],68:[,,{775:7690,780:270,803:7692,807:7696,813:7698,817:7694}],69:[,,{768:200,769:201,770:202,771:7868,772:274,774:276,775:278,776:203,777:7866,780:282,783:516,785:518,803:7864,807:552,808:280,813:7704,816:7706}],70:[,,{775:7710}],71:[,,{769:500,770:284,772:7712,774:286,775:288,780:486,807:290}],72:[,,{770:292,775:7714,776:7718,780:542,803:7716,807:7720,814:7722}],73:[,,{768:204,769:205,770:206,771:296,772:298,774:300,775:304,776:207,777:7880,780:463,783:520,785:522,803:7882,808:302,816:7724}],74:[,,{770:308}],75:[,,{769:7728,780:488,803:7730,807:310,817:7732}],76:[,,{769:313,780:317,803:7734,807:315,813:7740,817:7738}],77:[,,{769:7742,775:7744,803:7746}],78:[,,{768:504,769:323,771:209,775:7748,780:327,803:7750,807:325,813:7754,817:7752}],79:[,,{768:210,769:211,770:212,771:213,772:332,774:334,775:558,776:214,777:7886,779:336,780:465,783:524,785:526,795:416,803:7884,808:490}],80:[,,{769:7764,775:7766}],82:[,,{769:340,775:7768,780:344,783:528,785:530,803:7770,807:342,817:7774}],83:[,,{769:346,770:348,775:7776,780:352,803:7778,806:536,807:350}],84:[,,{775:7786,780:356,803:7788,806:538,807:354,813:7792,817:7790}],85:[,,{768:217,769:218,770:219,771:360,772:362,774:364,776:220,777:7910,778:366,779:368,780:467,783:532,785:534,795:431,803:7908,804:7794,808:370,813:7798,816:7796}],86:[,,{771:7804,803:7806}],87:[,,{768:7808,769:7810,770:372,775:7814,776:7812,803:7816}],88:[,,{775:7818,776:7820}],89:[,,{768:7922,769:221,770:374,771:7928,772:562,775:7822,776:376,777:7926,803:7924}],90:[,,{769:377,770:7824,775:379,780:381,803:7826,817:7828}],97:[,,{768:224,769:225,770:226,771:227,772:257,774:259,775:551,776:228,777:7843,778:229,780:462,783:513,785:515,803:7841,805:7681,808:261}],98:[,,{775:7683,803:7685,817:7687}],99:[,,{769:263,770:265,775:267,780:269,807:231}],100:[,,{775:7691,780:271,803:7693,807:7697,813:7699,817:7695}],101:[,,{768:232,769:233,770:234,771:7869,772:275,774:277,775:279,776:235,777:7867,780:283,783:517,785:519,803:7865,807:553,808:281,813:7705,816:7707}],102:[,,{775:7711}],103:[,,{769:501,770:285,772:7713,774:287,775:289,780:487,807:291}],104:[,,{770:293,775:7715,776:7719,780:543,803:7717,807:7721,814:7723,817:7830}],105:[,,{768:236,769:237,770:238,771:297,772:299,774:301,776:239,777:7881,780:464,783:521,785:523,803:7883,808:303,816:7725}],106:[,,{770:309,780:496}],107:[,,{769:7729,780:489,803:7731,807:311,817:7733}],108:[,,{769:314,780:318,803:7735,807:316,813:7741,817:7739}],109:[,,{769:7743,775:7745,803:7747}],110:[,,{768:505,769:324,771:241,775:7749,780:328,803:7751,807:326,813:7755,817:7753}],111:[,,{768:242,769:243,770:244,771:245,772:333,774:335,775:559,776:246,777:7887,779:337,780:466,783:525,785:527,795:417,803:7885,808:491}],112:[,,{769:7765,775:7767}],114:[,,{769:341,775:7769,780:345,783:529,785:531,803:7771,807:343,817:7775}],115:[,,{769:347,770:349,775:7777,780:353,803:7779,806:537,807:351}],116:[,,{775:7787,776:7831,780:357,803:7789,806:539,807:355,813:7793,817:7791}],117:[,,{768:249,769:250,770:251,771:361,772:363,774:365,776:252,777:7911,778:367,779:369,780:468,783:533,785:535,795:432,803:7909,804:7795,808:371,813:7799,816:7797}],118:[,,{771:7805,803:7807}],119:[,,{768:7809,769:7811,770:373,775:7815,776:7813,778:7832,803:7817}],120:[,,{775:7819,776:7821}],121:[,,{768:7923,769:253,770:375,771:7929,772:563,775:7823,776:255,777:7927,778:7833,803:7925}],122:[,,{769:378,770:7825,775:380,780:382,803:7827,817:7829}],160:[[32],256],168:[[32,776],256,{768:8173,769:901,834:8129}],170:[[97],256],175:[[32,772],256],178:[[50],256],179:[[51],256],180:[[32,769],256],181:[[956],256],184:[[32,807],256],185:[[49],256],186:[[111],256],188:[[49,8260,52],256],189:[[49,8260,50],256],190:[[51,8260,52],256],192:[[65,768]],193:[[65,769]],194:[[65,770],,{768:7846,769:7844,771:7850,777:7848}],195:[[65,771]],196:[[65,776],,{772:478}],197:[[65,778],,{769:506}],198:[,,{769:508,772:482}],199:[[67,807],,{769:7688}],200:[[69,768]],201:[[69,769]],202:[[69,770],,{768:7872,769:7870,771:7876,777:7874}],203:[[69,776]],204:[[73,768]],205:[[73,769]],206:[[73,770]],207:[[73,776],,{769:7726}],209:[[78,771]],210:[[79,768]],211:[[79,769]],212:[[79,770],,{768:7890,769:7888,771:7894,777:7892}],213:[[79,771],,{769:7756,772:556,776:7758}],214:[[79,776],,{772:554}],216:[,,{769:510}],217:[[85,768]],218:[[85,769]],219:[[85,770]],220:[[85,776],,{768:475,769:471,772:469,780:473}],221:[[89,769]],224:[[97,768]],225:[[97,769]],226:[[97,770],,{768:7847,769:7845,771:7851,777:7849}],227:[[97,771]],228:[[97,776],,{772:479}],229:[[97,778],,{769:507}],230:[,,{769:509,772:483}],231:[[99,807],,{769:7689}],232:[[101,768]],233:[[101,769]],234:[[101,770],,{768:7873,769:7871,771:7877,777:7875}],235:[[101,776]],236:[[105,768]],237:[[105,769]],238:[[105,770]],239:[[105,776],,{769:7727}],241:[[110,771]],242:[[111,768]],243:[[111,769]],244:[[111,770],,{768:7891,769:7889,771:7895,777:7893}],245:[[111,771],,{769:7757,772:557,776:7759}],246:[[111,776],,{772:555}],248:[,,{769:511}],249:[[117,768]],250:[[117,769]],251:[[117,770]],252:[[117,776],,{768:476,769:472,772:470,780:474}],253:[[121,769]],255:[[121,776]]},256:{256:[[65,772]],257:[[97,772]],258:[[65,774],,{768:7856,769:7854,771:7860,777:7858}],259:[[97,774],,{768:7857,769:7855,771:7861,777:7859}],260:[[65,808]],261:[[97,808]],262:[[67,769]],263:[[99,769]],264:[[67,770]],265:[[99,770]],266:[[67,775]],267:[[99,775]],268:[[67,780]],269:[[99,780]],270:[[68,780]],271:[[100,780]],274:[[69,772],,{768:7700,769:7702}],275:[[101,772],,{768:7701,769:7703}],276:[[69,774]],277:[[101,774]],278:[[69,775]],279:[[101,775]],280:[[69,808]],281:[[101,808]],282:[[69,780]],283:[[101,780]],284:[[71,770]],285:[[103,770]],286:[[71,774]],287:[[103,774]],288:[[71,775]],289:[[103,775]],290:[[71,807]],291:[[103,807]],292:[[72,770]],293:[[104,770]],296:[[73,771]],297:[[105,771]],298:[[73,772]],299:[[105,772]],300:[[73,774]],301:[[105,774]],302:[[73,808]],303:[[105,808]],304:[[73,775]],306:[[73,74],256],307:[[105,106],256],308:[[74,770]],309:[[106,770]],310:[[75,807]],311:[[107,807]],313:[[76,769]],314:[[108,769]],315:[[76,807]],316:[[108,807]],317:[[76,780]],318:[[108,780]],319:[[76,183],256],320:[[108,183],256],323:[[78,769]],324:[[110,769]],325:[[78,807]],326:[[110,807]],327:[[78,780]],328:[[110,780]],329:[[700,110],256],332:[[79,772],,{768:7760,769:7762}],333:[[111,772],,{768:7761,769:7763}],334:[[79,774]],335:[[111,774]],336:[[79,779]],337:[[111,779]],340:[[82,769]],341:[[114,769]],342:[[82,807]],343:[[114,807]],344:[[82,780]],345:[[114,780]],346:[[83,769],,{775:7780}],347:[[115,769],,{775:7781}],348:[[83,770]],349:[[115,770]],350:[[83,807]],351:[[115,807]],352:[[83,780],,{775:7782}],353:[[115,780],,{775:7783}],354:[[84,807]],355:[[116,807]],356:[[84,780]],357:[[116,780]],360:[[85,771],,{769:7800}],361:[[117,771],,{769:7801}],362:[[85,772],,{776:7802}],363:[[117,772],,{776:7803}],364:[[85,774]],365:[[117,774]],366:[[85,778]],367:[[117,778]],368:[[85,779]],369:[[117,779]],370:[[85,808]],371:[[117,808]],372:[[87,770]],373:[[119,770]],374:[[89,770]],375:[[121,770]],376:[[89,776]],377:[[90,769]],378:[[122,769]],379:[[90,775]],380:[[122,775]],381:[[90,780]],382:[[122,780]],383:[[115],256,{775:7835}],416:[[79,795],,{768:7900,769:7898,771:7904,777:7902,803:7906}],417:[[111,795],,{768:7901,769:7899,771:7905,777:7903,803:7907}],431:[[85,795],,{768:7914,769:7912,771:7918,777:7916,803:7920}],432:[[117,795],,{768:7915,769:7913,771:7919,777:7917,803:7921}],439:[,,{780:494}],452:[[68,381],256],453:[[68,382],256],454:[[100,382],256],455:[[76,74],256],456:[[76,106],256],457:[[108,106],256],458:[[78,74],256],459:[[78,106],256],460:[[110,106],256],461:[[65,780]],462:[[97,780]],463:[[73,780]],464:[[105,780]],465:[[79,780]],466:[[111,780]],467:[[85,780]],468:[[117,780]],469:[[220,772]],470:[[252,772]],471:[[220,769]],472:[[252,769]],473:[[220,780]],474:[[252,780]],475:[[220,768]],476:[[252,768]],478:[[196,772]],479:[[228,772]],480:[[550,772]],481:[[551,772]],482:[[198,772]],483:[[230,772]],486:[[71,780]],487:[[103,780]],488:[[75,780]],489:[[107,780]],490:[[79,808],,{772:492}],491:[[111,808],,{772:493}],492:[[490,772]],493:[[491,772]],494:[[439,780]],495:[[658,780]],496:[[106,780]],497:[[68,90],256],498:[[68,122],256],499:[[100,122],256],500:[[71,769]],501:[[103,769]],504:[[78,768]],505:[[110,768]],506:[[197,769]],507:[[229,769]],508:[[198,769]],509:[[230,769]],510:[[216,769]],511:[[248,769]],66045:[,220]},512:{512:[[65,783]],513:[[97,783]],514:[[65,785]],515:[[97,785]],516:[[69,783]],517:[[101,783]],518:[[69,785]],519:[[101,785]],520:[[73,783]],521:[[105,783]],522:[[73,785]],523:[[105,785]],524:[[79,783]],525:[[111,783]],526:[[79,785]],527:[[111,785]],528:[[82,783]],529:[[114,783]],530:[[82,785]],531:[[114,785]],532:[[85,783]],533:[[117,783]],534:[[85,785]],535:[[117,785]],536:[[83,806]],537:[[115,806]],538:[[84,806]],539:[[116,806]],542:[[72,780]],543:[[104,780]],550:[[65,775],,{772:480}],551:[[97,775],,{772:481}],552:[[69,807],,{774:7708}],553:[[101,807],,{774:7709}],554:[[214,772]],555:[[246,772]],556:[[213,772]],557:[[245,772]],558:[[79,775],,{772:560}],559:[[111,775],,{772:561}],560:[[558,772]],561:[[559,772]],562:[[89,772]],563:[[121,772]],658:[,,{780:495}],688:[[104],256],689:[[614],256],690:[[106],256],691:[[114],256],692:[[633],256],693:[[635],256],694:[[641],256],695:[[119],256],696:[[121],256],728:[[32,774],256],729:[[32,775],256],730:[[32,778],256],731:[[32,808],256],732:[[32,771],256],733:[[32,779],256],736:[[611],256],737:[[108],256],738:[[115],256],739:[[120],256],740:[[661],256],66272:[,220]},768:{768:[,230],769:[,230],770:[,230],771:[,230],772:[,230],773:[,230],774:[,230],775:[,230],776:[,230,{769:836}],777:[,230],778:[,230],779:[,230],780:[,230],781:[,230],782:[,230],783:[,230],784:[,230],785:[,230],786:[,230],787:[,230],788:[,230],789:[,232],790:[,220],791:[,220],792:[,220],793:[,220],794:[,232],795:[,216],796:[,220],797:[,220],798:[,220],799:[,220],800:[,220],801:[,202],802:[,202],803:[,220],804:[,220],805:[,220],806:[,220],807:[,202],808:[,202],809:[,220],810:[,220],811:[,220],812:[,220],813:[,220],814:[,220],815:[,220],816:[,220],817:[,220],818:[,220],819:[,220],820:[,1],821:[,1],822:[,1],823:[,1],824:[,1],825:[,220],826:[,220],827:[,220],828:[,220],829:[,230],830:[,230],831:[,230],832:[[768],230],833:[[769],230],834:[,230],835:[[787],230],836:[[776,769],230],837:[,240],838:[,230],839:[,220],840:[,220],841:[,220],842:[,230],843:[,230],844:[,230],845:[,220],846:[,220],848:[,230],849:[,230],850:[,230],851:[,220],852:[,220],853:[,220],854:[,220],855:[,230],856:[,232],857:[,220],858:[,220],859:[,230],860:[,233],861:[,234],862:[,234],863:[,233],864:[,234],865:[,234],866:[,233],867:[,230],868:[,230],869:[,230],870:[,230],871:[,230],872:[,230],873:[,230],874:[,230],875:[,230],876:[,230],877:[,230],878:[,230],879:[,230],884:[[697]],890:[[32,837],256],894:[[59]],900:[[32,769],256],901:[[168,769]],902:[[913,769]],903:[[183]],904:[[917,769]],905:[[919,769]],906:[[921,769]],908:[[927,769]],910:[[933,769]],911:[[937,769]],912:[[970,769]],913:[,,{768:8122,769:902,772:8121,774:8120,787:7944,788:7945,837:8124}],917:[,,{768:8136,769:904,787:7960,788:7961}],919:[,,{768:8138,769:905,787:7976,788:7977,837:8140}],921:[,,{768:8154,769:906,772:8153,774:8152,776:938,787:7992,788:7993}],927:[,,{768:8184,769:908,787:8008,788:8009}],929:[,,{788:8172}],933:[,,{768:8170,769:910,772:8169,774:8168,776:939,788:8025}],937:[,,{768:8186,769:911,787:8040,788:8041,837:8188}],938:[[921,776]],939:[[933,776]],940:[[945,769],,{837:8116}],941:[[949,769]],942:[[951,769],,{837:8132}],943:[[953,769]],944:[[971,769]],945:[,,{768:8048,769:940,772:8113,774:8112,787:7936,788:7937,834:8118,837:8115}],949:[,,{768:8050,769:941,787:7952,788:7953}],951:[,,{768:8052,769:942,787:7968,788:7969,834:8134,837:8131}],953:[,,{768:8054,769:943,772:8145,774:8144,776:970,787:7984,788:7985,834:8150}],959:[,,{768:8056,769:972,787:8e3,788:8001}],961:[,,{787:8164,788:8165}],965:[,,{768:8058,769:973,772:8161,774:8160,776:971,787:8016,788:8017,834:8166}],969:[,,{768:8060,769:974,787:8032,788:8033,834:8182,837:8179}],970:[[953,776],,{768:8146,769:912,834:8151}],971:[[965,776],,{768:8162,769:944,834:8167}],972:[[959,769]],973:[[965,769]],974:[[969,769],,{837:8180}],976:[[946],256],977:[[952],256],978:[[933],256,{769:979,776:980}],979:[[978,769]],980:[[978,776]],981:[[966],256],982:[[960],256],1008:[[954],256],1009:[[961],256],1010:[[962],256],1012:[[920],256],1013:[[949],256],1017:[[931],256],66422:[,230],66423:[,230],66424:[,230],66425:[,230],66426:[,230]},1024:{1024:[[1045,768]],1025:[[1045,776]],1027:[[1043,769]],1030:[,,{776:1031}],1031:[[1030,776]],1036:[[1050,769]],1037:[[1048,768]],1038:[[1059,774]],1040:[,,{774:1232,776:1234}],1043:[,,{769:1027}],1045:[,,{768:1024,774:1238,776:1025}],1046:[,,{774:1217,776:1244}],1047:[,,{776:1246}],1048:[,,{768:1037,772:1250,774:1049,776:1252}],1049:[[1048,774]],1050:[,,{769:1036}],1054:[,,{776:1254}],1059:[,,{772:1262,774:1038,776:1264,779:1266}],1063:[,,{776:1268}],1067:[,,{776:1272}],1069:[,,{776:1260}],1072:[,,{774:1233,776:1235}],1075:[,,{769:1107}],1077:[,,{768:1104,774:1239,776:1105}],1078:[,,{774:1218,776:1245}],1079:[,,{776:1247}],1080:[,,{768:1117,772:1251,774:1081,776:1253}],1081:[[1080,774]],1082:[,,{769:1116}],1086:[,,{776:1255}],1091:[,,{772:1263,774:1118,776:1265,779:1267}],1095:[,,{776:1269}],1099:[,,{776:1273}],1101:[,,{776:1261}],1104:[[1077,768]],1105:[[1077,776]],1107:[[1075,769]],1110:[,,{776:1111}],1111:[[1110,776]],1116:[[1082,769]],1117:[[1080,768]],1118:[[1091,774]],1140:[,,{783:1142}],1141:[,,{783:1143}],1142:[[1140,783]],1143:[[1141,783]],1155:[,230],1156:[,230],1157:[,230],1158:[,230],1159:[,230],1217:[[1046,774]],1218:[[1078,774]],1232:[[1040,774]],1233:[[1072,774]],1234:[[1040,776]],1235:[[1072,776]],1238:[[1045,774]],1239:[[1077,774]],1240:[,,{776:1242}],1241:[,,{776:1243}],1242:[[1240,776]],1243:[[1241,776]],1244:[[1046,776]],1245:[[1078,776]],1246:[[1047,776]],1247:[[1079,776]],1250:[[1048,772]],1251:[[1080,772]],1252:[[1048,776]],1253:[[1080,776]],1254:[[1054,776]],1255:[[1086,776]],1256:[,,{776:1258}],1257:[,,{776:1259}],1258:[[1256,776]],1259:[[1257,776]],1260:[[1069,776]],1261:[[1101,776]],1262:[[1059,772]],1263:[[1091,772]],1264:[[1059,776]],1265:[[1091,776]],1266:[[1059,779]],1267:[[1091,779]],1268:[[1063,776]],1269:[[1095,776]],1272:[[1067,776]],1273:[[1099,776]]},1280:{1415:[[1381,1410],256],1425:[,220],1426:[,230],1427:[,230],1428:[,230],1429:[,230],1430:[,220],1431:[,230],1432:[,230],1433:[,230],1434:[,222],1435:[,220],1436:[,230],1437:[,230],1438:[,230],1439:[,230],1440:[,230],1441:[,230],1442:[,220],1443:[,220],1444:[,220],1445:[,220],1446:[,220],1447:[,220],1448:[,230],1449:[,230],1450:[,220],1451:[,230],1452:[,230],1453:[,222],1454:[,228],1455:[,230],1456:[,10],1457:[,11],1458:[,12],1459:[,13],1460:[,14],1461:[,15],1462:[,16],1463:[,17],1464:[,18],1465:[,19],1466:[,19],1467:[,20],1468:[,21],1469:[,22],1471:[,23],1473:[,24],1474:[,25],1476:[,230],1477:[,220],1479:[,18]},1536:{1552:[,230],1553:[,230],1554:[,230],1555:[,230],1556:[,230],1557:[,230],1558:[,230],1559:[,230],1560:[,30],1561:[,31],1562:[,32],1570:[[1575,1619]],1571:[[1575,1620]],1572:[[1608,1620]],1573:[[1575,1621]],1574:[[1610,1620]],1575:[,,{1619:1570,1620:1571,1621:1573}],1608:[,,{1620:1572}],1610:[,,{1620:1574}],1611:[,27],1612:[,28],1613:[,29],1614:[,30],1615:[,31],1616:[,32],1617:[,33],1618:[,34],1619:[,230],1620:[,230],1621:[,220],1622:[,220],1623:[,230],1624:[,230],1625:[,230],1626:[,230],1627:[,230],1628:[,220],1629:[,230],1630:[,230],1631:[,220],1648:[,35],1653:[[1575,1652],256],1654:[[1608,1652],256],1655:[[1735,1652],256],1656:[[1610,1652],256],1728:[[1749,1620]],1729:[,,{1620:1730}],1730:[[1729,1620]],1746:[,,{1620:1747}],1747:[[1746,1620]],1749:[,,{1620:1728}],1750:[,230],1751:[,230],1752:[,230],1753:[,230],1754:[,230],1755:[,230],1756:[,230],1759:[,230],1760:[,230],1761:[,230],1762:[,230],1763:[,220],1764:[,230],1767:[,230],1768:[,230],1770:[,220],1771:[,230],1772:[,230],1773:[,220]},1792:{1809:[,36],1840:[,230],1841:[,220],1842:[,230],1843:[,230],1844:[,220],1845:[,230],1846:[,230],1847:[,220],1848:[,220],1849:[,220],1850:[,230],1851:[,220],1852:[,220],1853:[,230],1854:[,220],1855:[,230],1856:[,230],1857:[,230],1858:[,220],1859:[,230],1860:[,220],1861:[,230],1862:[,220],1863:[,230],1864:[,220],1865:[,230],1866:[,230],2027:[,230],2028:[,230],2029:[,230],2030:[,230],2031:[,230],2032:[,230],2033:[,230],2034:[,220],2035:[,230]},2048:{2070:[,230],2071:[,230],2072:[,230],2073:[,230],2075:[,230],2076:[,230],2077:[,230],2078:[,230],2079:[,230],2080:[,230],2081:[,230],2082:[,230],2083:[,230],2085:[,230],2086:[,230],2087:[,230],2089:[,230],2090:[,230],2091:[,230],2092:[,230],2093:[,230],2137:[,220],2138:[,220],2139:[,220],2276:[,230],2277:[,230],2278:[,220],2279:[,230],2280:[,230],2281:[,220],2282:[,230],2283:[,230],2284:[,230],2285:[,220],2286:[,220],2287:[,220],2288:[,27],2289:[,28],2290:[,29],2291:[,230],2292:[,230],2293:[,230],2294:[,220],2295:[,230],2296:[,230],2297:[,220],2298:[,220],2299:[,230],2300:[,230],2301:[,230],2302:[,230],2303:[,230]},2304:{2344:[,,{2364:2345}],2345:[[2344,2364]],2352:[,,{2364:2353}],2353:[[2352,2364]],2355:[,,{2364:2356}],2356:[[2355,2364]],2364:[,7],2381:[,9],2385:[,230],2386:[,220],2387:[,230],2388:[,230],2392:[[2325,2364],512],2393:[[2326,2364],512],2394:[[2327,2364],512],2395:[[2332,2364],512],2396:[[2337,2364],512],2397:[[2338,2364],512],2398:[[2347,2364],512],2399:[[2351,2364],512],2492:[,7],2503:[,,{2494:2507,2519:2508}],2507:[[2503,2494]],2508:[[2503,2519]],2509:[,9],2524:[[2465,2492],512],2525:[[2466,2492],512],2527:[[2479,2492],512]},2560:{2611:[[2610,2620],512],2614:[[2616,2620],512],2620:[,7],2637:[,9],2649:[[2582,2620],512],2650:[[2583,2620],512],2651:[[2588,2620],512],2654:[[2603,2620],512],2748:[,7],2765:[,9],68109:[,220],68111:[,230],68152:[,230],68153:[,1],68154:[,220],68159:[,9],68325:[,230],68326:[,220]},2816:{2876:[,7],2887:[,,{2878:2891,2902:2888,2903:2892}],2888:[[2887,2902]],2891:[[2887,2878]],2892:[[2887,2903]],2893:[,9],2908:[[2849,2876],512],2909:[[2850,2876],512],2962:[,,{3031:2964}],2964:[[2962,3031]],3014:[,,{3006:3018,3031:3020}],3015:[,,{3006:3019}],3018:[[3014,3006]],3019:[[3015,3006]],3020:[[3014,3031]],3021:[,9]},3072:{3142:[,,{3158:3144}],3144:[[3142,3158]],3149:[,9],3157:[,84],3158:[,91],3260:[,7],3263:[,,{3285:3264}],3264:[[3263,3285]],3270:[,,{3266:3274,3285:3271,3286:3272}],3271:[[3270,3285]],3272:[[3270,3286]],3274:[[3270,3266],,{3285:3275}],3275:[[3274,3285]],3277:[,9]},3328:{3398:[,,{3390:3402,3415:3404}],3399:[,,{3390:3403}],3402:[[3398,3390]],3403:[[3399,3390]],3404:[[3398,3415]],3405:[,9],3530:[,9],3545:[,,{3530:3546,3535:3548,3551:3550}],3546:[[3545,3530]],3548:[[3545,3535],,{3530:3549}],3549:[[3548,3530]],3550:[[3545,3551]]},3584:{3635:[[3661,3634],256],3640:[,103],3641:[,103],3642:[,9],3656:[,107],3657:[,107],3658:[,107],3659:[,107],3763:[[3789,3762],256],3768:[,118],3769:[,118],3784:[,122],3785:[,122],3786:[,122],3787:[,122],3804:[[3755,3737],256],3805:[[3755,3745],256]},3840:{3852:[[3851],256],3864:[,220],3865:[,220],3893:[,220],3895:[,220],3897:[,216],3907:[[3906,4023],512],3917:[[3916,4023],512],3922:[[3921,4023],512],3927:[[3926,4023],512],3932:[[3931,4023],512],3945:[[3904,4021],512],3953:[,129],3954:[,130],3955:[[3953,3954],512],3956:[,132],3957:[[3953,3956],512],3958:[[4018,3968],512],3959:[[4018,3969],256],3960:[[4019,3968],512],3961:[[4019,3969],256],3962:[,130],3963:[,130],3964:[,130],3965:[,130],3968:[,130],3969:[[3953,3968],512],3970:[,230],3971:[,230],3972:[,9],3974:[,230],3975:[,230],3987:[[3986,4023],512],3997:[[3996,4023],512],4002:[[4001,4023],512],4007:[[4006,4023],512],4012:[[4011,4023],512],4025:[[3984,4021],512],4038:[,220]},4096:{4133:[,,{4142:4134}],4134:[[4133,4142]],4151:[,7],4153:[,9],4154:[,9],4237:[,220],4348:[[4316],256],69702:[,9],69759:[,9],69785:[,,{69818:69786}],69786:[[69785,69818]],69787:[,,{69818:69788}],69788:[[69787,69818]],69797:[,,{69818:69803}],69803:[[69797,69818]],69817:[,9],69818:[,7]},4352:{69888:[,230],69889:[,230],69890:[,230],69934:[[69937,69927]],69935:[[69938,69927]],69937:[,,{69927:69934}],69938:[,,{69927:69935}],69939:[,9],69940:[,9],70003:[,7],70080:[,9]},4608:{70197:[,9],70198:[,7],70377:[,7],70378:[,9]},4864:{4957:[,230],4958:[,230],4959:[,230],70460:[,7],70471:[,,{70462:70475,70487:70476}],70475:[[70471,70462]],70476:[[70471,70487]],70477:[,9],70502:[,230],70503:[,230],70504:[,230],70505:[,230],70506:[,230],70507:[,230],70508:[,230],70512:[,230],70513:[,230],70514:[,230],70515:[,230],70516:[,230]},5120:{70841:[,,{70832:70844,70842:70843,70845:70846}],70843:[[70841,70842]],70844:[[70841,70832]],70846:[[70841,70845]],70850:[,9],70851:[,7]},5376:{71096:[,,{71087:71098}],71097:[,,{71087:71099}],71098:[[71096,71087]],71099:[[71097,71087]],71103:[,9],71104:[,7]},5632:{71231:[,9],71350:[,9],71351:[,7]},5888:{5908:[,9],5940:[,9],6098:[,9],6109:[,230]},6144:{6313:[,228]},6400:{6457:[,222],6458:[,230],6459:[,220]},6656:{6679:[,230],6680:[,220],6752:[,9],6773:[,230],6774:[,230],6775:[,230],6776:[,230],6777:[,230],6778:[,230],6779:[,230],6780:[,230],6783:[,220],6832:[,230],6833:[,230],6834:[,230],6835:[,230],6836:[,230],6837:[,220],6838:[,220],6839:[,220],6840:[,220],6841:[,220],6842:[,220],6843:[,230],6844:[,230],6845:[,220]},6912:{6917:[,,{6965:6918}],6918:[[6917,6965]],6919:[,,{6965:6920}],6920:[[6919,6965]],6921:[,,{6965:6922}],6922:[[6921,6965]],6923:[,,{6965:6924}],6924:[[6923,6965]],6925:[,,{6965:6926}],6926:[[6925,6965]],6929:[,,{6965:6930}],6930:[[6929,6965]],6964:[,7],6970:[,,{6965:6971}],6971:[[6970,6965]],6972:[,,{6965:6973}],6973:[[6972,6965]],6974:[,,{6965:6976}],6975:[,,{6965:6977}],6976:[[6974,6965]],6977:[[6975,6965]],6978:[,,{6965:6979}],6979:[[6978,6965]],6980:[,9],7019:[,230],7020:[,220],7021:[,230],7022:[,230],7023:[,230],7024:[,230],7025:[,230],7026:[,230],7027:[,230],7082:[,9],7083:[,9],7142:[,7],7154:[,9],7155:[,9]},7168:{7223:[,7],7376:[,230],7377:[,230],7378:[,230],7380:[,1],7381:[,220],7382:[,220],7383:[,220],7384:[,220],7385:[,220],7386:[,230],7387:[,230],7388:[,220],7389:[,220],7390:[,220],7391:[,220],7392:[,230],7394:[,1],7395:[,1],7396:[,1],7397:[,1],7398:[,1],7399:[,1],7400:[,1],7405:[,220],7412:[,230],7416:[,230],7417:[,230]},7424:{7468:[[65],256],7469:[[198],256],7470:[[66],256],7472:[[68],256],7473:[[69],256],7474:[[398],256],7475:[[71],256],7476:[[72],256],7477:[[73],256],7478:[[74],256],7479:[[75],256],7480:[[76],256],7481:[[77],256],7482:[[78],256],7484:[[79],256],7485:[[546],256],7486:[[80],256],7487:[[82],256],7488:[[84],256],7489:[[85],256],7490:[[87],256],7491:[[97],256],7492:[[592],256],7493:[[593],256],7494:[[7426],256],7495:[[98],256],7496:[[100],256],7497:[[101],256],7498:[[601],256],7499:[[603],256],7500:[[604],256],7501:[[103],256],7503:[[107],256],7504:[[109],256],7505:[[331],256],7506:[[111],256],7507:[[596],256],7508:[[7446],256],7509:[[7447],256],7510:[[112],256],7511:[[116],256],7512:[[117],256],7513:[[7453],256],7514:[[623],256],7515:[[118],256],7516:[[7461],256],7517:[[946],256],7518:[[947],256],7519:[[948],256],7520:[[966],256],7521:[[967],256],7522:[[105],256],7523:[[114],256],7524:[[117],256],7525:[[118],256],7526:[[946],256],7527:[[947],256],7528:[[961],256],7529:[[966],256],7530:[[967],256],7544:[[1085],256],7579:[[594],256],7580:[[99],256],7581:[[597],256],7582:[[240],256],7583:[[604],256],7584:[[102],256],7585:[[607],256],7586:[[609],256],7587:[[613],256],7588:[[616],256],7589:[[617],256],7590:[[618],256],7591:[[7547],256],7592:[[669],256],7593:[[621],256],7594:[[7557],256],7595:[[671],256],7596:[[625],256],7597:[[624],256],7598:[[626],256],7599:[[627],256],7600:[[628],256],7601:[[629],256],7602:[[632],256],7603:[[642],256],7604:[[643],256],7605:[[427],256],7606:[[649],256],7607:[[650],256],7608:[[7452],256],7609:[[651],256],7610:[[652],256],7611:[[122],256],7612:[[656],256],7613:[[657],256],7614:[[658],256],7615:[[952],256],7616:[,230],7617:[,230],7618:[,220],7619:[,230],7620:[,230],7621:[,230],7622:[,230],7623:[,230],7624:[,230],7625:[,230],7626:[,220],7627:[,230],7628:[,230],7629:[,234],7630:[,214],7631:[,220],7632:[,202],7633:[,230],7634:[,230],7635:[,230],7636:[,230],7637:[,230],7638:[,230],7639:[,230],7640:[,230],7641:[,230],7642:[,230],7643:[,230],7644:[,230],7645:[,230],7646:[,230],7647:[,230],7648:[,230],7649:[,230],7650:[,230],7651:[,230],7652:[,230],7653:[,230],7654:[,230],7655:[,230],7656:[,230],7657:[,230],7658:[,230],7659:[,230],7660:[,230],7661:[,230],7662:[,230],7663:[,230],7664:[,230],7665:[,230],7666:[,230],7667:[,230],7668:[,230],7669:[,230],7676:[,233],7677:[,220],7678:[,230],7679:[,220]},7680:{7680:[[65,805]],7681:[[97,805]],7682:[[66,775]],7683:[[98,775]],7684:[[66,803]],7685:[[98,803]],7686:[[66,817]],7687:[[98,817]],7688:[[199,769]],7689:[[231,769]],7690:[[68,775]],7691:[[100,775]],7692:[[68,803]],7693:[[100,803]],7694:[[68,817]],7695:[[100,817]],7696:[[68,807]],7697:[[100,807]],7698:[[68,813]],7699:[[100,813]],7700:[[274,768]],7701:[[275,768]],7702:[[274,769]],7703:[[275,769]],7704:[[69,813]],7705:[[101,813]],7706:[[69,816]],7707:[[101,816]],7708:[[552,774]],7709:[[553,774]],7710:[[70,775]],7711:[[102,775]],7712:[[71,772]],7713:[[103,772]],7714:[[72,775]],7715:[[104,775]],7716:[[72,803]],7717:[[104,803]],7718:[[72,776]],7719:[[104,776]],7720:[[72,807]],7721:[[104,807]],7722:[[72,814]],7723:[[104,814]],7724:[[73,816]],7725:[[105,816]],7726:[[207,769]],7727:[[239,769]],7728:[[75,769]],7729:[[107,769]],7730:[[75,803]],7731:[[107,803]],7732:[[75,817]],7733:[[107,817]],7734:[[76,803],,{772:7736}],7735:[[108,803],,{772:7737}],7736:[[7734,772]],7737:[[7735,772]],7738:[[76,817]],7739:[[108,817]],7740:[[76,813]],7741:[[108,813]],7742:[[77,769]],7743:[[109,769]],7744:[[77,775]],7745:[[109,775]],7746:[[77,803]],7747:[[109,803]],7748:[[78,775]],7749:[[110,775]],7750:[[78,803]],7751:[[110,803]],7752:[[78,817]],7753:[[110,817]],7754:[[78,813]],7755:[[110,813]],7756:[[213,769]],7757:[[245,769]],7758:[[213,776]],7759:[[245,776]],7760:[[332,768]],7761:[[333,768]],7762:[[332,769]],7763:[[333,769]],7764:[[80,769]],7765:[[112,769]],7766:[[80,775]],7767:[[112,775]],7768:[[82,775]],7769:[[114,775]],7770:[[82,803],,{772:7772}],7771:[[114,803],,{772:7773}],7772:[[7770,772]],7773:[[7771,772]],7774:[[82,817]],7775:[[114,817]],7776:[[83,775]],7777:[[115,775]],7778:[[83,803],,{775:7784}],7779:[[115,803],,{775:7785}],7780:[[346,775]],7781:[[347,775]],7782:[[352,775]],7783:[[353,775]],7784:[[7778,775]],7785:[[7779,775]],7786:[[84,775]],7787:[[116,775]],7788:[[84,803]],7789:[[116,803]],7790:[[84,817]],7791:[[116,817]],7792:[[84,813]],7793:[[116,813]],7794:[[85,804]],7795:[[117,804]],7796:[[85,816]],7797:[[117,816]],7798:[[85,813]],7799:[[117,813]],7800:[[360,769]],7801:[[361,769]],7802:[[362,776]],7803:[[363,776]],7804:[[86,771]],7805:[[118,771]],7806:[[86,803]],7807:[[118,803]],7808:[[87,768]],7809:[[119,768]],7810:[[87,769]],7811:[[119,769]],7812:[[87,776]],7813:[[119,776]],7814:[[87,775]],7815:[[119,775]],7816:[[87,803]],7817:[[119,803]],7818:[[88,775]],7819:[[120,775]],7820:[[88,776]],7821:[[120,776]],7822:[[89,775]],7823:[[121,775]],7824:[[90,770]],7825:[[122,770]],7826:[[90,803]],7827:[[122,803]],7828:[[90,817]],7829:[[122,817]],7830:[[104,817]],7831:[[116,776]],7832:[[119,778]],7833:[[121,778]],7834:[[97,702],256],7835:[[383,775]],7840:[[65,803],,{770:7852,774:7862}],7841:[[97,803],,{770:7853,774:7863}],7842:[[65,777]],7843:[[97,777]],7844:[[194,769]],7845:[[226,769]],7846:[[194,768]],7847:[[226,768]],7848:[[194,777]],7849:[[226,777]],7850:[[194,771]],7851:[[226,771]],7852:[[7840,770]],7853:[[7841,770]],7854:[[258,769]],7855:[[259,769]],7856:[[258,768]],7857:[[259,768]],7858:[[258,777]],7859:[[259,777]],7860:[[258,771]],7861:[[259,771]],7862:[[7840,774]],7863:[[7841,774]],7864:[[69,803],,{770:7878}],7865:[[101,803],,{770:7879}],7866:[[69,777]],7867:[[101,777]],7868:[[69,771]],7869:[[101,771]],7870:[[202,769]],7871:[[234,769]],7872:[[202,768]],7873:[[234,768]],7874:[[202,777]],7875:[[234,777]],7876:[[202,771]],7877:[[234,771]],7878:[[7864,770]],7879:[[7865,770]],7880:[[73,777]],7881:[[105,777]],7882:[[73,803]],7883:[[105,803]],7884:[[79,803],,{770:7896}],7885:[[111,803],,{770:7897}],7886:[[79,777]],7887:[[111,777]],7888:[[212,769]],7889:[[244,769]],7890:[[212,768]],7891:[[244,768]],7892:[[212,777]],7893:[[244,777]],7894:[[212,771]],7895:[[244,771]],7896:[[7884,770]],7897:[[7885,770]],7898:[[416,769]],7899:[[417,769]],7900:[[416,768]],7901:[[417,768]],7902:[[416,777]],7903:[[417,777]],7904:[[416,771]],7905:[[417,771]],7906:[[416,803]],7907:[[417,803]],7908:[[85,803]],7909:[[117,803]],7910:[[85,777]],7911:[[117,777]],7912:[[431,769]],7913:[[432,769]],7914:[[431,768]],7915:[[432,768]],7916:[[431,777]],7917:[[432,777]],7918:[[431,771]],7919:[[432,771]],7920:[[431,803]],7921:[[432,803]],7922:[[89,768]],7923:[[121,768]],7924:[[89,803]],7925:[[121,803]],7926:[[89,777]],7927:[[121,777]],7928:[[89,771]],7929:[[121,771]]},7936:{7936:[[945,787],,{768:7938,769:7940,834:7942,837:8064}],7937:[[945,788],,{768:7939,769:7941,834:7943,837:8065}],7938:[[7936,768],,{837:8066}],7939:[[7937,768],,{837:8067}],7940:[[7936,769],,{837:8068}],7941:[[7937,769],,{837:8069}],7942:[[7936,834],,{837:8070}],7943:[[7937,834],,{837:8071}],7944:[[913,787],,{768:7946,769:7948,834:7950,837:8072}],7945:[[913,788],,{768:7947,769:7949,834:7951,837:8073}],7946:[[7944,768],,{837:8074}],7947:[[7945,768],,{837:8075}],7948:[[7944,769],,{837:8076}],7949:[[7945,769],,{837:8077}],7950:[[7944,834],,{837:8078}],7951:[[7945,834],,{837:8079}],7952:[[949,787],,{768:7954,769:7956}],7953:[[949,788],,{768:7955,769:7957}],7954:[[7952,768]],7955:[[7953,768]],7956:[[7952,769]],7957:[[7953,769]],7960:[[917,787],,{768:7962,769:7964}],7961:[[917,788],,{768:7963,769:7965}],7962:[[7960,768]],7963:[[7961,768]],7964:[[7960,769]],7965:[[7961,769]],7968:[[951,787],,{768:7970,769:7972,834:7974,837:8080}],7969:[[951,788],,{768:7971,769:7973,834:7975,837:8081}],7970:[[7968,768],,{837:8082}],7971:[[7969,768],,{837:8083}],7972:[[7968,769],,{837:8084}],7973:[[7969,769],,{837:8085}],7974:[[7968,834],,{837:8086}],7975:[[7969,834],,{837:8087}],7976:[[919,787],,{768:7978,769:7980,834:7982,837:8088}],7977:[[919,788],,{768:7979,769:7981,834:7983,837:8089}],7978:[[7976,768],,{837:8090}],7979:[[7977,768],,{837:8091}],7980:[[7976,769],,{837:8092}],7981:[[7977,769],,{837:8093}],7982:[[7976,834],,{837:8094}],7983:[[7977,834],,{837:8095}],7984:[[953,787],,{768:7986,769:7988,834:7990}],7985:[[953,788],,{768:7987,769:7989,834:7991}],7986:[[7984,768]],7987:[[7985,768]],7988:[[7984,769]],7989:[[7985,769]],7990:[[7984,834]],7991:[[7985,834]],7992:[[921,787],,{768:7994,769:7996,834:7998}],7993:[[921,788],,{768:7995,769:7997,834:7999}],7994:[[7992,768]],7995:[[7993,768]],7996:[[7992,769]],7997:[[7993,769]],7998:[[7992,834]],7999:[[7993,834]],8e3:[[959,787],,{768:8002,769:8004}],8001:[[959,788],,{768:8003,769:8005}],8002:[[8e3,768]],8003:[[8001,768]],8004:[[8e3,769]],8005:[[8001,769]],8008:[[927,787],,{768:8010,769:8012}],8009:[[927,788],,{768:8011,769:8013}],8010:[[8008,768]],8011:[[8009,768]],8012:[[8008,769]],8013:[[8009,769]],8016:[[965,787],,{768:8018,769:8020,834:8022}],8017:[[965,788],,{768:8019,769:8021,834:8023}],8018:[[8016,768]],8019:[[8017,768]],8020:[[8016,769]],8021:[[8017,769]],8022:[[8016,834]],8023:[[8017,834]],8025:[[933,788],,{768:8027,769:8029,834:8031}],8027:[[8025,768]],8029:[[8025,769]],8031:[[8025,834]],8032:[[969,787],,{768:8034,769:8036,834:8038,837:8096}],8033:[[969,788],,{768:8035,769:8037,834:8039,837:8097}],8034:[[8032,768],,{837:8098}],8035:[[8033,768],,{837:8099}],8036:[[8032,769],,{837:8100}],8037:[[8033,769],,{837:8101}],8038:[[8032,834],,{837:8102}],8039:[[8033,834],,{837:8103}],8040:[[937,787],,{768:8042,769:8044,834:8046,837:8104}],8041:[[937,788],,{768:8043,769:8045,834:8047,837:8105}],8042:[[8040,768],,{837:8106}],8043:[[8041,768],,{837:8107}],8044:[[8040,769],,{837:8108}],8045:[[8041,769],,{837:8109}],8046:[[8040,834],,{837:8110}],8047:[[8041,834],,{837:8111}],8048:[[945,768],,{837:8114}],8049:[[940]],8050:[[949,768]],8051:[[941]],8052:[[951,768],,{837:8130}],8053:[[942]],8054:[[953,768]],8055:[[943]],8056:[[959,768]],8057:[[972]],8058:[[965,768]],8059:[[973]],8060:[[969,768],,{837:8178}],8061:[[974]],8064:[[7936,837]],8065:[[7937,837]],8066:[[7938,837]],8067:[[7939,837]],8068:[[7940,837]],8069:[[7941,837]],8070:[[7942,837]],8071:[[7943,837]],8072:[[7944,837]],8073:[[7945,837]],8074:[[7946,837]],8075:[[7947,837]],8076:[[7948,837]],8077:[[7949,837]],8078:[[7950,837]],8079:[[7951,837]],8080:[[7968,837]],8081:[[7969,837]],8082:[[7970,837]],8083:[[7971,837]],8084:[[7972,837]],8085:[[7973,837]],8086:[[7974,837]],8087:[[7975,837]],8088:[[7976,837]],8089:[[7977,837]],8090:[[7978,837]],8091:[[7979,837]],8092:[[7980,837]],8093:[[7981,837]],8094:[[7982,837]],8095:[[7983,837]],8096:[[8032,837]],8097:[[8033,837]],8098:[[8034,837]],8099:[[8035,837]],8100:[[8036,837]],8101:[[8037,837]],8102:[[8038,837]],8103:[[8039,837]],8104:[[8040,837]],8105:[[8041,837]],8106:[[8042,837]],8107:[[8043,837]],8108:[[8044,837]],8109:[[8045,837]],8110:[[8046,837]],8111:[[8047,837]],8112:[[945,774]],8113:[[945,772]],8114:[[8048,837]],8115:[[945,837]],8116:[[940,837]],8118:[[945,834],,{837:8119}],8119:[[8118,837]],8120:[[913,774]],8121:[[913,772]],8122:[[913,768]],8123:[[902]],8124:[[913,837]],8125:[[32,787],256],8126:[[953]],8127:[[32,787],256,{768:8141,769:8142,834:8143}],8128:[[32,834],256],8129:[[168,834]],8130:[[8052,837]],8131:[[951,837]],8132:[[942,837]],8134:[[951,834],,{837:8135}],8135:[[8134,837]],8136:[[917,768]],8137:[[904]],8138:[[919,768]],8139:[[905]],8140:[[919,837]],8141:[[8127,768]],8142:[[8127,769]],8143:[[8127,834]],8144:[[953,774]],8145:[[953,772]],8146:[[970,768]],8147:[[912]],8150:[[953,834]],8151:[[970,834]],8152:[[921,774]],8153:[[921,772]],8154:[[921,768]],8155:[[906]],8157:[[8190,768]],8158:[[8190,769]],8159:[[8190,834]],8160:[[965,774]],8161:[[965,772]],8162:[[971,768]],8163:[[944]],8164:[[961,787]],8165:[[961,788]],8166:[[965,834]],8167:[[971,834]],8168:[[933,774]],8169:[[933,772]],8170:[[933,768]],8171:[[910]],8172:[[929,788]],8173:[[168,768]],8174:[[901]],8175:[[96]],8178:[[8060,837]],8179:[[969,837]],8180:[[974,837]],8182:[[969,834],,{837:8183}],8183:[[8182,837]],8184:[[927,768]],8185:[[908]],8186:[[937,768]],8187:[[911]],8188:[[937,837]],8189:[[180]],8190:[[32,788],256,{768:8157,769:8158,834:8159}]},8192:{8192:[[8194]],8193:[[8195]],8194:[[32],256],8195:[[32],256],8196:[[32],256],8197:[[32],256],8198:[[32],256],8199:[[32],256],8200:[[32],256],8201:[[32],256],8202:[[32],256],8209:[[8208],256],8215:[[32,819],256],8228:[[46],256],8229:[[46,46],256],8230:[[46,46,46],256],8239:[[32],256],8243:[[8242,8242],256],8244:[[8242,8242,8242],256],8246:[[8245,8245],256],8247:[[8245,8245,8245],256],8252:[[33,33],256],8254:[[32,773],256],8263:[[63,63],256],8264:[[63,33],256],8265:[[33,63],256],8279:[[8242,8242,8242,8242],256],8287:[[32],256],8304:[[48],256],8305:[[105],256],8308:[[52],256],8309:[[53],256],8310:[[54],256],8311:[[55],256],8312:[[56],256],8313:[[57],256],8314:[[43],256],8315:[[8722],256],8316:[[61],256],8317:[[40],256],8318:[[41],256],8319:[[110],256],8320:[[48],256],8321:[[49],256],8322:[[50],256],8323:[[51],256],8324:[[52],256],8325:[[53],256],8326:[[54],256],8327:[[55],256],8328:[[56],256],8329:[[57],256],8330:[[43],256],8331:[[8722],256],8332:[[61],256],8333:[[40],256],8334:[[41],256],8336:[[97],256],8337:[[101],256],8338:[[111],256],8339:[[120],256],8340:[[601],256],8341:[[104],256],8342:[[107],256],8343:[[108],256],8344:[[109],256],8345:[[110],256],8346:[[112],256],8347:[[115],256],8348:[[116],256],8360:[[82,115],256],8400:[,230],8401:[,230],8402:[,1],8403:[,1],8404:[,230],8405:[,230],8406:[,230],8407:[,230],8408:[,1],8409:[,1],8410:[,1],8411:[,230],8412:[,230],8417:[,230],8421:[,1],8422:[,1],8423:[,230],8424:[,220],8425:[,230],8426:[,1],8427:[,1],8428:[,220],8429:[,220],8430:[,220],8431:[,220],8432:[,230]},8448:{8448:[[97,47,99],256],8449:[[97,47,115],256],8450:[[67],256],8451:[[176,67],256],8453:[[99,47,111],256],8454:[[99,47,117],256],8455:[[400],256],8457:[[176,70],256],8458:[[103],256],8459:[[72],256],8460:[[72],256],8461:[[72],256],8462:[[104],256],8463:[[295],256],8464:[[73],256],8465:[[73],256],8466:[[76],256],8467:[[108],256],8469:[[78],256],8470:[[78,111],256],8473:[[80],256],8474:[[81],256],8475:[[82],256],8476:[[82],256],8477:[[82],256],8480:[[83,77],256],8481:[[84,69,76],256],8482:[[84,77],256],8484:[[90],256],8486:[[937]],8488:[[90],256],8490:[[75]],8491:[[197]],8492:[[66],256],8493:[[67],256],8495:[[101],256],8496:[[69],256],8497:[[70],256],8499:[[77],256],8500:[[111],256],8501:[[1488],256],8502:[[1489],256],8503:[[1490],256],8504:[[1491],256],8505:[[105],256],8507:[[70,65,88],256],8508:[[960],256],8509:[[947],256],8510:[[915],256],8511:[[928],256],8512:[[8721],256],8517:[[68],256],8518:[[100],256],8519:[[101],256],8520:[[105],256],8521:[[106],256],8528:[[49,8260,55],256],8529:[[49,8260,57],256],8530:[[49,8260,49,48],256],8531:[[49,8260,51],256],8532:[[50,8260,51],256],8533:[[49,8260,53],256],8534:[[50,8260,53],256],8535:[[51,8260,53],256],8536:[[52,8260,53],256],8537:[[49,8260,54],256],8538:[[53,8260,54],256],8539:[[49,8260,56],256],8540:[[51,8260,56],256],8541:[[53,8260,56],256],8542:[[55,8260,56],256],8543:[[49,8260],256],8544:[[73],256],8545:[[73,73],256],8546:[[73,73,73],256],8547:[[73,86],256],8548:[[86],256],8549:[[86,73],256],8550:[[86,73,73],256],8551:[[86,73,73,73],256],8552:[[73,88],256],8553:[[88],256],8554:[[88,73],256],8555:[[88,73,73],256],8556:[[76],256],8557:[[67],256],8558:[[68],256],8559:[[77],256],8560:[[105],256],8561:[[105,105],256],8562:[[105,105,105],256],8563:[[105,118],256],8564:[[118],256],8565:[[118,105],256],8566:[[118,105,105],256],8567:[[118,105,105,105],256],8568:[[105,120],256],8569:[[120],256],8570:[[120,105],256],8571:[[120,105,105],256],8572:[[108],256],8573:[[99],256],8574:[[100],256],8575:[[109],256],8585:[[48,8260,51],256],8592:[,,{824:8602}],8594:[,,{824:8603}],8596:[,,{824:8622}],8602:[[8592,824]],8603:[[8594,824]],8622:[[8596,824]],8653:[[8656,824]],8654:[[8660,824]],8655:[[8658,824]],8656:[,,{824:8653}],8658:[,,{824:8655}],8660:[,,{824:8654}]},8704:{8707:[,,{824:8708}],8708:[[8707,824]],8712:[,,{824:8713}],8713:[[8712,824]],8715:[,,{824:8716}],8716:[[8715,824]],8739:[,,{824:8740}],8740:[[8739,824]],8741:[,,{824:8742}],8742:[[8741,824]],8748:[[8747,8747],256],8749:[[8747,8747,8747],256],8751:[[8750,8750],256],8752:[[8750,8750,8750],256],8764:[,,{824:8769}],8769:[[8764,824]],8771:[,,{824:8772}],8772:[[8771,824]],8773:[,,{824:8775}],8775:[[8773,824]],8776:[,,{824:8777}],8777:[[8776,824]],8781:[,,{824:8813}],8800:[[61,824]],8801:[,,{824:8802}],8802:[[8801,824]],8804:[,,{824:8816}],8805:[,,{824:8817}],8813:[[8781,824]],8814:[[60,824]],8815:[[62,824]],8816:[[8804,824]],8817:[[8805,824]],8818:[,,{824:8820}],8819:[,,{824:8821}],8820:[[8818,824]],8821:[[8819,824]],8822:[,,{824:8824}],8823:[,,{824:8825}],8824:[[8822,824]],8825:[[8823,824]],8826:[,,{824:8832}],8827:[,,{824:8833}],8828:[,,{824:8928}],8829:[,,{824:8929}],8832:[[8826,824]],8833:[[8827,824]],8834:[,,{824:8836}],8835:[,,{824:8837}],8836:[[8834,824]],8837:[[8835,824]],8838:[,,{824:8840}],8839:[,,{824:8841}],8840:[[8838,824]],8841:[[8839,824]],8849:[,,{824:8930}],8850:[,,{824:8931}],8866:[,,{824:8876}],8872:[,,{824:8877}],8873:[,,{824:8878}],8875:[,,{824:8879}],8876:[[8866,824]],8877:[[8872,824]],8878:[[8873,824]],8879:[[8875,824]],8882:[,,{824:8938}],8883:[,,{824:8939}],8884:[,,{824:8940}],8885:[,,{824:8941}],8928:[[8828,824]],8929:[[8829,824]],8930:[[8849,824]],8931:[[8850,824]],8938:[[8882,824]],8939:[[8883,824]],8940:[[8884,824]],8941:[[8885,824]]},8960:{9001:[[12296]],9002:[[12297]]},9216:{9312:[[49],256],9313:[[50],256],9314:[[51],256],9315:[[52],256],9316:[[53],256],9317:[[54],256],9318:[[55],256],9319:[[56],256],9320:[[57],256],9321:[[49,48],256],9322:[[49,49],256],9323:[[49,50],256],9324:[[49,51],256],9325:[[49,52],256],9326:[[49,53],256],9327:[[49,54],256],9328:[[49,55],256],9329:[[49,56],256],9330:[[49,57],256],9331:[[50,48],256],9332:[[40,49,41],256],9333:[[40,50,41],256],9334:[[40,51,41],256],9335:[[40,52,41],256],9336:[[40,53,41],256],9337:[[40,54,41],256],9338:[[40,55,41],256],9339:[[40,56,41],256],9340:[[40,57,41],256],9341:[[40,49,48,41],256],9342:[[40,49,49,41],256],9343:[[40,49,50,41],256],9344:[[40,49,51,41],256],9345:[[40,49,52,41],256],9346:[[40,49,53,41],256],9347:[[40,49,54,41],256],9348:[[40,49,55,41],256],9349:[[40,49,56,41],256],9350:[[40,49,57,41],256],9351:[[40,50,48,41],256],9352:[[49,46],256],9353:[[50,46],256],9354:[[51,46],256],9355:[[52,46],256],9356:[[53,46],256],9357:[[54,46],256],9358:[[55,46],256],9359:[[56,46],256],9360:[[57,46],256],9361:[[49,48,46],256],9362:[[49,49,46],256],9363:[[49,50,46],256],9364:[[49,51,46],256],9365:[[49,52,46],256],9366:[[49,53,46],256],9367:[[49,54,46],256],9368:[[49,55,46],256],9369:[[49,56,46],256],9370:[[49,57,46],256],9371:[[50,48,46],256],9372:[[40,97,41],256],9373:[[40,98,41],256],9374:[[40,99,41],256],9375:[[40,100,41],256],9376:[[40,101,41],256],9377:[[40,102,41],256],9378:[[40,103,41],256],9379:[[40,104,41],256],9380:[[40,105,41],256],9381:[[40,106,41],256],9382:[[40,107,41],256],9383:[[40,108,41],256],9384:[[40,109,41],256],9385:[[40,110,41],256],9386:[[40,111,41],256],9387:[[40,112,41],256],9388:[[40,113,41],256],9389:[[40,114,41],256],9390:[[40,115,41],256],9391:[[40,116,41],256],9392:[[40,117,41],256],9393:[[40,118,41],256],9394:[[40,119,41],256],9395:[[40,120,41],256],9396:[[40,121,41],256],9397:[[40,122,41],256],9398:[[65],256],9399:[[66],256],9400:[[67],256],9401:[[68],256],9402:[[69],256],9403:[[70],256],9404:[[71],256],9405:[[72],256],9406:[[73],256],9407:[[74],256],9408:[[75],256],9409:[[76],256],9410:[[77],256],9411:[[78],256],9412:[[79],256],9413:[[80],256],9414:[[81],256],9415:[[82],256],9416:[[83],256],9417:[[84],256],9418:[[85],256],9419:[[86],256],9420:[[87],256],9421:[[88],256],9422:[[89],256],9423:[[90],256],9424:[[97],256],9425:[[98],256],9426:[[99],256],9427:[[100],256],9428:[[101],256],9429:[[102],256],9430:[[103],256],9431:[[104],256],9432:[[105],256],9433:[[106],256],9434:[[107],256],9435:[[108],256],9436:[[109],256],9437:[[110],256],9438:[[111],256],9439:[[112],256],9440:[[113],256],9441:[[114],256],9442:[[115],256],9443:[[116],256],9444:[[117],256],9445:[[118],256],9446:[[119],256],9447:[[120],256],9448:[[121],256],9449:[[122],256],9450:[[48],256]},10752:{10764:[[8747,8747,8747,8747],256],10868:[[58,58,61],256],10869:[[61,61],256],10870:[[61,61,61],256],10972:[[10973,824],512]},11264:{11388:[[106],256],11389:[[86],256],11503:[,230],11504:[,230],11505:[,230]},11520:{11631:[[11617],256],11647:[,9],11744:[,230],11745:[,230],11746:[,230],11747:[,230],11748:[,230],11749:[,230],11750:[,230],11751:[,230],11752:[,230],11753:[,230],11754:[,230],11755:[,230],11756:[,230],11757:[,230],11758:[,230],11759:[,230],11760:[,230],11761:[,230],11762:[,230],11763:[,230],11764:[,230],11765:[,230],11766:[,230],11767:[,230],11768:[,230],11769:[,230],11770:[,230],11771:[,230],11772:[,230],11773:[,230],11774:[,230],11775:[,230]},11776:{11935:[[27597],256],12019:[[40863],256]},12032:{12032:[[19968],256],12033:[[20008],256],12034:[[20022],256],12035:[[20031],256],12036:[[20057],256],12037:[[20101],256],12038:[[20108],256],12039:[[20128],256],12040:[[20154],256],12041:[[20799],256],12042:[[20837],256],12043:[[20843],256],12044:[[20866],256],12045:[[20886],256],12046:[[20907],256],12047:[[20960],256],12048:[[20981],256],12049:[[20992],256],12050:[[21147],256],12051:[[21241],256],12052:[[21269],256],12053:[[21274],256],12054:[[21304],256],12055:[[21313],256],12056:[[21340],256],12057:[[21353],256],12058:[[21378],256],12059:[[21430],256],12060:[[21448],256],12061:[[21475],256],12062:[[22231],256],12063:[[22303],256],12064:[[22763],256],12065:[[22786],256],12066:[[22794],256],12067:[[22805],256],12068:[[22823],256],12069:[[22899],256],12070:[[23376],256],12071:[[23424],256],12072:[[23544],256],12073:[[23567],256],12074:[[23586],256],12075:[[23608],256],12076:[[23662],256],12077:[[23665],256],12078:[[24027],256],12079:[[24037],256],12080:[[24049],256],12081:[[24062],256],12082:[[24178],256],12083:[[24186],256],12084:[[24191],256],12085:[[24308],256],12086:[[24318],256],12087:[[24331],256],12088:[[24339],256],12089:[[24400],256],12090:[[24417],256],12091:[[24435],256],12092:[[24515],256],12093:[[25096],256],12094:[[25142],256],12095:[[25163],256],12096:[[25903],256],12097:[[25908],256],12098:[[25991],256],12099:[[26007],256],12100:[[26020],256],12101:[[26041],256],12102:[[26080],256],12103:[[26085],256],12104:[[26352],256],12105:[[26376],256],12106:[[26408],256],12107:[[27424],256],12108:[[27490],256],12109:[[27513],256],12110:[[27571],256],12111:[[27595],256],12112:[[27604],256],12113:[[27611],256],12114:[[27663],256],12115:[[27668],256],12116:[[27700],256],12117:[[28779],256],12118:[[29226],256],12119:[[29238],256],12120:[[29243],256],12121:[[29247],256],12122:[[29255],256],12123:[[29273],256],12124:[[29275],256],12125:[[29356],256],12126:[[29572],256],12127:[[29577],256],12128:[[29916],256],12129:[[29926],256],12130:[[29976],256],12131:[[29983],256],12132:[[29992],256],12133:[[3e4],256],12134:[[30091],256],12135:[[30098],256],12136:[[30326],256],12137:[[30333],256],12138:[[30382],256],12139:[[30399],256],12140:[[30446],256],12141:[[30683],256],12142:[[30690],256],12143:[[30707],256],12144:[[31034],256],12145:[[31160],256],12146:[[31166],256],12147:[[31348],256],12148:[[31435],256],12149:[[31481],256],12150:[[31859],256],12151:[[31992],256],12152:[[32566],256],12153:[[32593],256],12154:[[32650],256],12155:[[32701],256],12156:[[32769],256],12157:[[32780],256],12158:[[32786],256],12159:[[32819],256],12160:[[32895],256],12161:[[32905],256],12162:[[33251],256],12163:[[33258],256],12164:[[33267],256],12165:[[33276],256],12166:[[33292],256],12167:[[33307],256],12168:[[33311],256],12169:[[33390],256],12170:[[33394],256],12171:[[33400],256],12172:[[34381],256],12173:[[34411],256],12174:[[34880],256],12175:[[34892],256],12176:[[34915],256],12177:[[35198],256],12178:[[35211],256],12179:[[35282],256],12180:[[35328],256],12181:[[35895],256],12182:[[35910],256],12183:[[35925],256],12184:[[35960],256],12185:[[35997],256],12186:[[36196],256],12187:[[36208],256],12188:[[36275],256],12189:[[36523],256],12190:[[36554],256],12191:[[36763],256],12192:[[36784],256],12193:[[36789],256],12194:[[37009],256],12195:[[37193],256],12196:[[37318],256],12197:[[37324],256],12198:[[37329],256],12199:[[38263],256],12200:[[38272],256],12201:[[38428],256],12202:[[38582],256],12203:[[38585],256],12204:[[38632],256],12205:[[38737],256],12206:[[38750],256],12207:[[38754],256],12208:[[38761],256],12209:[[38859],256],12210:[[38893],256],12211:[[38899],256],12212:[[38913],256],12213:[[39080],256],12214:[[39131],256],12215:[[39135],256],12216:[[39318],256],12217:[[39321],256],12218:[[39340],256],12219:[[39592],256],12220:[[39640],256],12221:[[39647],256],12222:[[39717],256],12223:[[39727],256],12224:[[39730],256],12225:[[39740],256],12226:[[39770],256],12227:[[40165],256],12228:[[40565],256],12229:[[40575],256],12230:[[40613],256],12231:[[40635],256],12232:[[40643],256],12233:[[40653],256],12234:[[40657],256],12235:[[40697],256],12236:[[40701],256],12237:[[40718],256],12238:[[40723],256],12239:[[40736],256],12240:[[40763],256],12241:[[40778],256],12242:[[40786],256],12243:[[40845],256],12244:[[40860],256],12245:[[40864],256]},12288:{12288:[[32],256],12330:[,218],12331:[,228],12332:[,232],12333:[,222],12334:[,224],12335:[,224],12342:[[12306],256],12344:[[21313],256],12345:[[21316],256],12346:[[21317],256],12358:[,,{12441:12436}],12363:[,,{12441:12364}],12364:[[12363,12441]],12365:[,,{12441:12366}],12366:[[12365,12441]],12367:[,,{12441:12368}],12368:[[12367,12441]],12369:[,,{12441:12370}],12370:[[12369,12441]],12371:[,,{12441:12372}],12372:[[12371,12441]],12373:[,,{12441:12374}],12374:[[12373,12441]],12375:[,,{12441:12376}],12376:[[12375,12441]],12377:[,,{12441:12378}],12378:[[12377,12441]],12379:[,,{12441:12380}],12380:[[12379,12441]],12381:[,,{12441:12382}],12382:[[12381,12441]],12383:[,,{12441:12384}],12384:[[12383,12441]],12385:[,,{12441:12386}],12386:[[12385,12441]],12388:[,,{12441:12389}],12389:[[12388,12441]],12390:[,,{12441:12391}],12391:[[12390,12441]],12392:[,,{12441:12393}],12393:[[12392,12441]],12399:[,,{12441:12400,12442:12401}],12400:[[12399,12441]],12401:[[12399,12442]],12402:[,,{12441:12403,12442:12404}],12403:[[12402,12441]],12404:[[12402,12442]],12405:[,,{12441:12406,12442:12407}],12406:[[12405,12441]],12407:[[12405,12442]],12408:[,,{12441:12409,12442:12410}],12409:[[12408,12441]],12410:[[12408,12442]],12411:[,,{12441:12412,12442:12413}],12412:[[12411,12441]],12413:[[12411,12442]],12436:[[12358,12441]],12441:[,8],12442:[,8],12443:[[32,12441],256],12444:[[32,12442],256],12445:[,,{12441:12446}],12446:[[12445,12441]],12447:[[12424,12426],256],12454:[,,{12441:12532}],12459:[,,{12441:12460}],12460:[[12459,12441]],12461:[,,{12441:12462}],12462:[[12461,12441]],12463:[,,{12441:12464}],12464:[[12463,12441]],12465:[,,{12441:12466}],12466:[[12465,12441]],12467:[,,{12441:12468}],12468:[[12467,12441]],12469:[,,{12441:12470}],12470:[[12469,12441]],12471:[,,{12441:12472}],12472:[[12471,12441]],12473:[,,{12441:12474}],12474:[[12473,12441]],12475:[,,{12441:12476}],12476:[[12475,12441]],12477:[,,{12441:12478}],12478:[[12477,12441]],12479:[,,{12441:12480}],12480:[[12479,12441]],12481:[,,{12441:12482}],12482:[[12481,12441]],12484:[,,{12441:12485}],12485:[[12484,12441]],12486:[,,{12441:12487}],12487:[[12486,12441]],12488:[,,{12441:12489}],12489:[[12488,12441]],12495:[,,{12441:12496,12442:12497}],12496:[[12495,12441]],12497:[[12495,12442]],12498:[,,{12441:12499,12442:12500}],12499:[[12498,12441]],12500:[[12498,12442]],12501:[,,{12441:12502,12442:12503}],12502:[[12501,12441]],12503:[[12501,12442]],12504:[,,{12441:12505,12442:12506}],12505:[[12504,12441]],12506:[[12504,12442]],12507:[,,{12441:12508,12442:12509}],12508:[[12507,12441]],12509:[[12507,12442]],12527:[,,{12441:12535}],12528:[,,{12441:12536}],12529:[,,{12441:12537}],12530:[,,{12441:12538}],12532:[[12454,12441]],12535:[[12527,12441]],12536:[[12528,12441]],12537:[[12529,12441]],12538:[[12530,12441]],12541:[,,{12441:12542}],12542:[[12541,12441]],12543:[[12467,12488],256]},12544:{12593:[[4352],256],12594:[[4353],256],12595:[[4522],256],12596:[[4354],256],12597:[[4524],256],12598:[[4525],256],12599:[[4355],256],12600:[[4356],256],12601:[[4357],256],12602:[[4528],256],12603:[[4529],256],12604:[[4530],256],12605:[[4531],256],12606:[[4532],256],12607:[[4533],256],12608:[[4378],256],12609:[[4358],256],12610:[[4359],256],12611:[[4360],256],12612:[[4385],256],12613:[[4361],256],12614:[[4362],256],12615:[[4363],256],12616:[[4364],256],12617:[[4365],256],12618:[[4366],256],12619:[[4367],256],12620:[[4368],256],12621:[[4369],256],12622:[[4370],256],12623:[[4449],256],12624:[[4450],256],12625:[[4451],256],12626:[[4452],256],12627:[[4453],256],12628:[[4454],256],12629:[[4455],256],12630:[[4456],256],12631:[[4457],256],12632:[[4458],256],12633:[[4459],256],12634:[[4460],256],12635:[[4461],256],12636:[[4462],256],12637:[[4463],256],12638:[[4464],256],12639:[[4465],256],12640:[[4466],256],12641:[[4467],256],12642:[[4468],256],12643:[[4469],256],12644:[[4448],256],12645:[[4372],256],12646:[[4373],256],12647:[[4551],256],12648:[[4552],256],12649:[[4556],256],12650:[[4558],256],12651:[[4563],256],12652:[[4567],256],12653:[[4569],256],12654:[[4380],256],12655:[[4573],256],12656:[[4575],256],12657:[[4381],256],12658:[[4382],256],12659:[[4384],256],12660:[[4386],256],12661:[[4387],256],12662:[[4391],256],12663:[[4393],256],12664:[[4395],256],12665:[[4396],256],12666:[[4397],256],12667:[[4398],256],12668:[[4399],256],12669:[[4402],256],12670:[[4406],256],12671:[[4416],256],12672:[[4423],256],12673:[[4428],256],12674:[[4593],256],12675:[[4594],256],12676:[[4439],256],12677:[[4440],256],12678:[[4441],256],12679:[[4484],256],12680:[[4485],256],12681:[[4488],256],12682:[[4497],256],12683:[[4498],256],12684:[[4500],256],12685:[[4510],256],12686:[[4513],256],12690:[[19968],256],12691:[[20108],256],12692:[[19977],256],12693:[[22235],256],12694:[[19978],256],12695:[[20013],256],12696:[[19979],256],12697:[[30002],256],12698:[[20057],256],12699:[[19993],256],12700:[[19969],256],12701:[[22825],256],12702:[[22320],256],12703:[[20154],256]},12800:{12800:[[40,4352,41],256],12801:[[40,4354,41],256],12802:[[40,4355,41],256],12803:[[40,4357,41],256],12804:[[40,4358,41],256],12805:[[40,4359,41],256],12806:[[40,4361,41],256],12807:[[40,4363,41],256],12808:[[40,4364,41],256],12809:[[40,4366,41],256],12810:[[40,4367,41],256],12811:[[40,4368,41],256],12812:[[40,4369,41],256],12813:[[40,4370,41],256],12814:[[40,4352,4449,41],256],12815:[[40,4354,4449,41],256],12816:[[40,4355,4449,41],256],12817:[[40,4357,4449,41],256],12818:[[40,4358,4449,41],256],12819:[[40,4359,4449,41],256],12820:[[40,4361,4449,41],256],12821:[[40,4363,4449,41],256],12822:[[40,4364,4449,41],256],12823:[[40,4366,4449,41],256],12824:[[40,4367,4449,41],256],12825:[[40,4368,4449,41],256],12826:[[40,4369,4449,41],256],12827:[[40,4370,4449,41],256],12828:[[40,4364,4462,41],256],12829:[[40,4363,4457,4364,4453,4523,41],256],12830:[[40,4363,4457,4370,4462,41],256],12832:[[40,19968,41],256],12833:[[40,20108,41],256],12834:[[40,19977,41],256],12835:[[40,22235,41],256],12836:[[40,20116,41],256],12837:[[40,20845,41],256],12838:[[40,19971,41],256],12839:[[40,20843,41],256],12840:[[40,20061,41],256],12841:[[40,21313,41],256],12842:[[40,26376,41],256],12843:[[40,28779,41],256],12844:[[40,27700,41],256],12845:[[40,26408,41],256],12846:[[40,37329,41],256],12847:[[40,22303,41],256],12848:[[40,26085,41],256],12849:[[40,26666,41],256],12850:[[40,26377,41],256],12851:[[40,31038,41],256],12852:[[40,21517,41],256],12853:[[40,29305,41],256],12854:[[40,36001,41],256],12855:[[40,31069,41],256],12856:[[40,21172,41],256],12857:[[40,20195,41],256],12858:[[40,21628,41],256],12859:[[40,23398,41],256],12860:[[40,30435,41],256],12861:[[40,20225,41],256],12862:[[40,36039,41],256],12863:[[40,21332,41],256],12864:[[40,31085,41],256],12865:[[40,20241,41],256],12866:[[40,33258,41],256],12867:[[40,33267,41],256],12868:[[21839],256],12869:[[24188],256],12870:[[25991],256],12871:[[31631],256],12880:[[80,84,69],256],12881:[[50,49],256],12882:[[50,50],256],12883:[[50,51],256],12884:[[50,52],256],12885:[[50,53],256],12886:[[50,54],256],12887:[[50,55],256],12888:[[50,56],256],12889:[[50,57],256],12890:[[51,48],256],12891:[[51,49],256],12892:[[51,50],256],12893:[[51,51],256],12894:[[51,52],256],12895:[[51,53],256],12896:[[4352],256],12897:[[4354],256],12898:[[4355],256],12899:[[4357],256],12900:[[4358],256],12901:[[4359],256],12902:[[4361],256],12903:[[4363],256],12904:[[4364],256],12905:[[4366],256],12906:[[4367],256],12907:[[4368],256],12908:[[4369],256],12909:[[4370],256],12910:[[4352,4449],256],12911:[[4354,4449],256],12912:[[4355,4449],256],12913:[[4357,4449],256],12914:[[4358,4449],256],12915:[[4359,4449],256],12916:[[4361,4449],256],12917:[[4363,4449],256],12918:[[4364,4449],256],12919:[[4366,4449],256],12920:[[4367,4449],256],12921:[[4368,4449],256],12922:[[4369,4449],256],12923:[[4370,4449],256],12924:[[4366,4449,4535,4352,4457],256],12925:[[4364,4462,4363,4468],256],12926:[[4363,4462],256],12928:[[19968],256],12929:[[20108],256],12930:[[19977],256],12931:[[22235],256],12932:[[20116],256],12933:[[20845],256],12934:[[19971],256],12935:[[20843],256],12936:[[20061],256],12937:[[21313],256],12938:[[26376],256],12939:[[28779],256],12940:[[27700],256],12941:[[26408],256],12942:[[37329],256],12943:[[22303],256],12944:[[26085],256],12945:[[26666],256],12946:[[26377],256],12947:[[31038],256],12948:[[21517],256],12949:[[29305],256],12950:[[36001],256],12951:[[31069],256],12952:[[21172],256],12953:[[31192],256],12954:[[30007],256],12955:[[22899],256],12956:[[36969],256],12957:[[20778],256],12958:[[21360],256],12959:[[27880],256],12960:[[38917],256],12961:[[20241],256],12962:[[20889],256],12963:[[27491],256],12964:[[19978],256],12965:[[20013],256],12966:[[19979],256],12967:[[24038],256],12968:[[21491],256],12969:[[21307],256],12970:[[23447],256],12971:[[23398],256],12972:[[30435],256],12973:[[20225],256],12974:[[36039],256],12975:[[21332],256],12976:[[22812],256],12977:[[51,54],256],12978:[[51,55],256],12979:[[51,56],256],12980:[[51,57],256],12981:[[52,48],256],12982:[[52,49],256],12983:[[52,50],256],12984:[[52,51],256],12985:[[52,52],256],12986:[[52,53],256],12987:[[52,54],256],12988:[[52,55],256],12989:[[52,56],256],12990:[[52,57],256],12991:[[53,48],256],12992:[[49,26376],256],12993:[[50,26376],256],12994:[[51,26376],256],12995:[[52,26376],256],12996:[[53,26376],256],12997:[[54,26376],256],12998:[[55,26376],256],12999:[[56,26376],256],13e3:[[57,26376],256],13001:[[49,48,26376],256],13002:[[49,49,26376],256],13003:[[49,50,26376],256],13004:[[72,103],256],13005:[[101,114,103],256],13006:[[101,86],256],13007:[[76,84,68],256],13008:[[12450],256],13009:[[12452],256],13010:[[12454],256],13011:[[12456],256],13012:[[12458],256],13013:[[12459],256],13014:[[12461],256],13015:[[12463],256],13016:[[12465],256],13017:[[12467],256],13018:[[12469],256],13019:[[12471],256],13020:[[12473],256],13021:[[12475],256],13022:[[12477],256],13023:[[12479],256],13024:[[12481],256],13025:[[12484],256],13026:[[12486],256],13027:[[12488],256],13028:[[12490],256],13029:[[12491],256],13030:[[12492],256],13031:[[12493],256],13032:[[12494],256],13033:[[12495],256],13034:[[12498],256],13035:[[12501],256],13036:[[12504],256],13037:[[12507],256],13038:[[12510],256],13039:[[12511],256],13040:[[12512],256],13041:[[12513],256],13042:[[12514],256],13043:[[12516],256],13044:[[12518],256],13045:[[12520],256],13046:[[12521],256],13047:[[12522],256],13048:[[12523],256],13049:[[12524],256],13050:[[12525],256],13051:[[12527],256],13052:[[12528],256],13053:[[12529],256],13054:[[12530],256]},13056:{13056:[[12450,12497,12540,12488],256],13057:[[12450,12523,12501,12449],256],13058:[[12450,12531,12506,12450],256],13059:[[12450,12540,12523],256],13060:[[12452,12491,12531,12464],256],13061:[[12452,12531,12481],256],13062:[[12454,12457,12531],256],13063:[[12456,12473,12463,12540,12489],256],13064:[[12456,12540,12459,12540],256],13065:[[12458,12531,12473],256],13066:[[12458,12540,12512],256],13067:[[12459,12452,12522],256],13068:[[12459,12521,12483,12488],256],13069:[[12459,12525,12522,12540],256],13070:[[12460,12525,12531],256],13071:[[12460,12531,12510],256],13072:[[12462,12460],256],13073:[[12462,12491,12540],256],13074:[[12461,12517,12522,12540],256],13075:[[12462,12523,12480,12540],256],13076:[[12461,12525],256],13077:[[12461,12525,12464,12521,12512],256],13078:[[12461,12525,12513,12540,12488,12523],256],13079:[[12461,12525,12527,12483,12488],256],13080:[[12464,12521,12512],256],13081:[[12464,12521,12512,12488,12531],256],13082:[[12463,12523,12476,12452,12525],256],13083:[[12463,12525,12540,12493],256],13084:[[12465,12540,12473],256],13085:[[12467,12523,12490],256],13086:[[12467,12540,12509],256],13087:[[12469,12452,12463,12523],256],13088:[[12469,12531,12481,12540,12512],256],13089:[[12471,12522,12531,12464],256],13090:[[12475,12531,12481],256],13091:[[12475,12531,12488],256],13092:[[12480,12540,12473],256],13093:[[12487,12471],256],13094:[[12489,12523],256],13095:[[12488,12531],256],13096:[[12490,12494],256],13097:[[12494,12483,12488],256],13098:[[12495,12452,12484],256],13099:[[12497,12540,12475,12531,12488],256],13100:[[12497,12540,12484],256],13101:[[12496,12540,12524,12523],256],13102:[[12500,12450,12473,12488,12523],256],13103:[[12500,12463,12523],256],13104:[[12500,12467],256],13105:[[12499,12523],256],13106:[[12501,12449,12521,12483,12489],256],13107:[[12501,12451,12540,12488],256],13108:[[12502,12483,12471,12455,12523],256],13109:[[12501,12521,12531],256],13110:[[12504,12463,12479,12540,12523],256],13111:[[12506,12477],256],13112:[[12506,12491,12498],256],13113:[[12504,12523,12484],256],13114:[[12506,12531,12473],256],13115:[[12506,12540,12472],256],13116:[[12505,12540,12479],256],13117:[[12509,12452,12531,12488],256],13118:[[12508,12523,12488],256],13119:[[12507,12531],256],13120:[[12509,12531,12489],256],13121:[[12507,12540,12523],256],13122:[[12507,12540,12531],256],13123:[[12510,12452,12463,12525],256],13124:[[12510,12452,12523],256],13125:[[12510,12483,12495],256],13126:[[12510,12523,12463],256],13127:[[12510,12531,12471,12519,12531],256],13128:[[12511,12463,12525,12531],256],13129:[[12511,12522],256],13130:[[12511,12522,12496,12540,12523],256],13131:[[12513,12460],256],13132:[[12513,12460,12488,12531],256],13133:[[12513,12540,12488,12523],256],13134:[[12516,12540,12489],256],13135:[[12516,12540,12523],256],13136:[[12518,12450,12531],256],13137:[[12522,12483,12488,12523],256],13138:[[12522,12521],256],13139:[[12523,12500,12540],256],13140:[[12523,12540,12502,12523],256],13141:[[12524,12512],256],13142:[[12524,12531,12488,12466,12531],256],13143:[[12527,12483,12488],256],13144:[[48,28857],256],13145:[[49,28857],256],13146:[[50,28857],256],13147:[[51,28857],256],13148:[[52,28857],256],13149:[[53,28857],256],13150:[[54,28857],256],13151:[[55,28857],256],13152:[[56,28857],256],13153:[[57,28857],256],13154:[[49,48,28857],256],13155:[[49,49,28857],256],13156:[[49,50,28857],256],13157:[[49,51,28857],256],13158:[[49,52,28857],256],13159:[[49,53,28857],256],13160:[[49,54,28857],256],13161:[[49,55,28857],256],13162:[[49,56,28857],256],13163:[[49,57,28857],256],13164:[[50,48,28857],256],13165:[[50,49,28857],256],13166:[[50,50,28857],256],13167:[[50,51,28857],256],13168:[[50,52,28857],256],13169:[[104,80,97],256],13170:[[100,97],256],13171:[[65,85],256],13172:[[98,97,114],256],13173:[[111,86],256],13174:[[112,99],256],13175:[[100,109],256],13176:[[100,109,178],256],13177:[[100,109,179],256],13178:[[73,85],256],13179:[[24179,25104],256],13180:[[26157,21644],256],13181:[[22823,27491],256],13182:[[26126,27835],256],13183:[[26666,24335,20250,31038],256],13184:[[112,65],256],13185:[[110,65],256],13186:[[956,65],256],13187:[[109,65],256],13188:[[107,65],256],13189:[[75,66],256],13190:[[77,66],256],13191:[[71,66],256],13192:[[99,97,108],256],13193:[[107,99,97,108],256],13194:[[112,70],256],13195:[[110,70],256],13196:[[956,70],256],13197:[[956,103],256],13198:[[109,103],256],13199:[[107,103],256],13200:[[72,122],256],13201:[[107,72,122],256],13202:[[77,72,122],256],13203:[[71,72,122],256],13204:[[84,72,122],256],13205:[[956,8467],256],13206:[[109,8467],256],13207:[[100,8467],256],13208:[[107,8467],256],13209:[[102,109],256],13210:[[110,109],256],13211:[[956,109],256],13212:[[109,109],256],13213:[[99,109],256],13214:[[107,109],256],13215:[[109,109,178],256],13216:[[99,109,178],256],13217:[[109,178],256],13218:[[107,109,178],256],13219:[[109,109,179],256],13220:[[99,109,179],256],13221:[[109,179],256],13222:[[107,109,179],256],13223:[[109,8725,115],256],13224:[[109,8725,115,178],256],13225:[[80,97],256],13226:[[107,80,97],256],13227:[[77,80,97],256],13228:[[71,80,97],256],13229:[[114,97,100],256],13230:[[114,97,100,8725,115],256],13231:[[114,97,100,8725,115,178],256],13232:[[112,115],256],13233:[[110,115],256],13234:[[956,115],256],13235:[[109,115],256],13236:[[112,86],256],13237:[[110,86],256],13238:[[956,86],256],13239:[[109,86],256],13240:[[107,86],256],13241:[[77,86],256],13242:[[112,87],256],13243:[[110,87],256],13244:[[956,87],256],13245:[[109,87],256],13246:[[107,87],256],13247:[[77,87],256],13248:[[107,937],256],13249:[[77,937],256],13250:[[97,46,109,46],256],13251:[[66,113],256],13252:[[99,99],256],13253:[[99,100],256],13254:[[67,8725,107,103],256],13255:[[67,111,46],256],13256:[[100,66],256],13257:[[71,121],256],13258:[[104,97],256],13259:[[72,80],256],13260:[[105,110],256],13261:[[75,75],256],13262:[[75,77],256],13263:[[107,116],256],13264:[[108,109],256],13265:[[108,110],256],13266:[[108,111,103],256],13267:[[108,120],256],13268:[[109,98],256],13269:[[109,105,108],256],13270:[[109,111,108],256],13271:[[80,72],256],13272:[[112,46,109,46],256],13273:[[80,80,77],256],13274:[[80,82],256],13275:[[115,114],256],13276:[[83,118],256],13277:[[87,98],256],13278:[[86,8725,109],256],13279:[[65,8725,109],256],13280:[[49,26085],256],13281:[[50,26085],256],13282:[[51,26085],256],13283:[[52,26085],256],13284:[[53,26085],256],13285:[[54,26085],256],13286:[[55,26085],256],13287:[[56,26085],256],13288:[[57,26085],256],13289:[[49,48,26085],256],13290:[[49,49,26085],256],13291:[[49,50,26085],256],13292:[[49,51,26085],256],13293:[[49,52,26085],256],13294:[[49,53,26085],256],13295:[[49,54,26085],256],13296:[[49,55,26085],256],13297:[[49,56,26085],256],13298:[[49,57,26085],256],13299:[[50,48,26085],256],13300:[[50,49,26085],256],13301:[[50,50,26085],256],13302:[[50,51,26085],256],13303:[[50,52,26085],256],13304:[[50,53,26085],256],13305:[[50,54,26085],256],13306:[[50,55,26085],256],13307:[[50,56,26085],256],13308:[[50,57,26085],256],13309:[[51,48,26085],256],13310:[[51,49,26085],256],13311:[[103,97,108],256]},27136:{92912:[,1],92913:[,1],92914:[,1],92915:[,1],92916:[,1]},27392:{92976:[,230],92977:[,230],92978:[,230],92979:[,230],92980:[,230],92981:[,230],92982:[,230]},42496:{42607:[,230],42612:[,230],42613:[,230],42614:[,230],42615:[,230],42616:[,230],42617:[,230],42618:[,230],42619:[,230],42620:[,230],42621:[,230],42652:[[1098],256],42653:[[1100],256],42655:[,230],42736:[,230],42737:[,230]},42752:{42864:[[42863],256],43e3:[[294],256],43001:[[339],256]},43008:{43014:[,9],43204:[,9],43232:[,230],43233:[,230],43234:[,230],43235:[,230],43236:[,230],43237:[,230],43238:[,230],43239:[,230],43240:[,230],43241:[,230],43242:[,230],43243:[,230],43244:[,230],43245:[,230],43246:[,230],43247:[,230],43248:[,230],43249:[,230]},43264:{43307:[,220],43308:[,220],43309:[,220],43347:[,9],43443:[,7],43456:[,9]},43520:{43696:[,230],43698:[,230],43699:[,230],43700:[,220],43703:[,230],43704:[,230],43710:[,230],43711:[,230],43713:[,230],43766:[,9]},43776:{43868:[[42791],256],43869:[[43831],256],43870:[[619],256],43871:[[43858],256],44013:[,9]},48128:{113822:[,1]},53504:{119134:[[119127,119141],512],119135:[[119128,119141],512],119136:[[119135,119150],512],119137:[[119135,119151],512],119138:[[119135,119152],512],119139:[[119135,119153],512],119140:[[119135,119154],512],119141:[,216],119142:[,216],119143:[,1],119144:[,1],119145:[,1],119149:[,226],119150:[,216],119151:[,216],119152:[,216],119153:[,216],119154:[,216],119163:[,220],119164:[,220],119165:[,220],119166:[,220],119167:[,220],119168:[,220],119169:[,220],119170:[,220],119173:[,230],119174:[,230],119175:[,230],119176:[,230],119177:[,230],119178:[,220],119179:[,220],119210:[,230],119211:[,230],119212:[,230],119213:[,230],119227:[[119225,119141],512],119228:[[119226,119141],512],119229:[[119227,119150],512],119230:[[119228,119150],512],119231:[[119227,119151],512],119232:[[119228,119151],512]},53760:{119362:[,230],119363:[,230],119364:[,230]},54272:{119808:[[65],256],119809:[[66],256],119810:[[67],256],119811:[[68],256],119812:[[69],256],119813:[[70],256],119814:[[71],256],119815:[[72],256],119816:[[73],256],119817:[[74],256],119818:[[75],256],119819:[[76],256],119820:[[77],256],119821:[[78],256],119822:[[79],256],119823:[[80],256],119824:[[81],256],119825:[[82],256],119826:[[83],256],119827:[[84],256],119828:[[85],256],119829:[[86],256],119830:[[87],256],119831:[[88],256],119832:[[89],256],119833:[[90],256],119834:[[97],256],119835:[[98],256],119836:[[99],256],119837:[[100],256],119838:[[101],256],119839:[[102],256],119840:[[103],256],119841:[[104],256],119842:[[105],256],119843:[[106],256],119844:[[107],256],119845:[[108],256],119846:[[109],256],119847:[[110],256],119848:[[111],256],119849:[[112],256],119850:[[113],256],119851:[[114],256],119852:[[115],256],119853:[[116],256],119854:[[117],256],119855:[[118],256],119856:[[119],256],119857:[[120],256],119858:[[121],256],119859:[[122],256],119860:[[65],256],119861:[[66],256],119862:[[67],256],119863:[[68],256],119864:[[69],256],119865:[[70],256],119866:[[71],256],119867:[[72],256],119868:[[73],256],119869:[[74],256],119870:[[75],256],119871:[[76],256],119872:[[77],256],119873:[[78],256],119874:[[79],256],119875:[[80],256],119876:[[81],256],119877:[[82],256],119878:[[83],256],119879:[[84],256],119880:[[85],256],119881:[[86],256],119882:[[87],256],119883:[[88],256],119884:[[89],256],119885:[[90],256],119886:[[97],256],119887:[[98],256],119888:[[99],256],119889:[[100],256],119890:[[101],256],119891:[[102],256],119892:[[103],256],119894:[[105],256],119895:[[106],256],119896:[[107],256],119897:[[108],256],119898:[[109],256],119899:[[110],256],119900:[[111],256],119901:[[112],256],119902:[[113],256],119903:[[114],256],119904:[[115],256],119905:[[116],256],119906:[[117],256],119907:[[118],256],119908:[[119],256],119909:[[120],256],119910:[[121],256],119911:[[122],256],119912:[[65],256],119913:[[66],256],119914:[[67],256],119915:[[68],256],119916:[[69],256],119917:[[70],256],119918:[[71],256],119919:[[72],256],119920:[[73],256],119921:[[74],256],119922:[[75],256],119923:[[76],256],119924:[[77],256],119925:[[78],256],119926:[[79],256],119927:[[80],256],119928:[[81],256],119929:[[82],256],119930:[[83],256],119931:[[84],256],119932:[[85],256],119933:[[86],256],119934:[[87],256],119935:[[88],256],119936:[[89],256],119937:[[90],256],119938:[[97],256],119939:[[98],256],119940:[[99],256],119941:[[100],256],119942:[[101],256],119943:[[102],256],119944:[[103],256],119945:[[104],256],119946:[[105],256],119947:[[106],256],119948:[[107],256],119949:[[108],256],119950:[[109],256],119951:[[110],256],119952:[[111],256],119953:[[112],256],119954:[[113],256],119955:[[114],256],119956:[[115],256],119957:[[116],256],119958:[[117],256],119959:[[118],256],119960:[[119],256],119961:[[120],256],119962:[[121],256],119963:[[122],256],119964:[[65],256],119966:[[67],256],119967:[[68],256],119970:[[71],256],119973:[[74],256],119974:[[75],256],119977:[[78],256],119978:[[79],256],119979:[[80],256],119980:[[81],256],119982:[[83],256],119983:[[84],256],119984:[[85],256],119985:[[86],256],119986:[[87],256],119987:[[88],256],119988:[[89],256],119989:[[90],256],119990:[[97],256],119991:[[98],256],119992:[[99],256],119993:[[100],256],119995:[[102],256],119997:[[104],256],119998:[[105],256],119999:[[106],256],12e4:[[107],256],120001:[[108],256],120002:[[109],256],120003:[[110],256],120005:[[112],256],120006:[[113],256],120007:[[114],256],120008:[[115],256],120009:[[116],256],120010:[[117],256],120011:[[118],256],120012:[[119],256],120013:[[120],256],120014:[[121],256],120015:[[122],256],120016:[[65],256],120017:[[66],256],120018:[[67],256],120019:[[68],256],120020:[[69],256],120021:[[70],256],120022:[[71],256],120023:[[72],256],120024:[[73],256],120025:[[74],256],120026:[[75],256],120027:[[76],256],120028:[[77],256],120029:[[78],256],120030:[[79],256],120031:[[80],256],120032:[[81],256],120033:[[82],256],120034:[[83],256],120035:[[84],256],120036:[[85],256],120037:[[86],256],120038:[[87],256],120039:[[88],256],120040:[[89],256],120041:[[90],256],120042:[[97],256],120043:[[98],256],120044:[[99],256],120045:[[100],256],120046:[[101],256],120047:[[102],256],120048:[[103],256],120049:[[104],256],120050:[[105],256],120051:[[106],256],120052:[[107],256],120053:[[108],256],120054:[[109],256],120055:[[110],256],120056:[[111],256],120057:[[112],256],120058:[[113],256],120059:[[114],256],120060:[[115],256],120061:[[116],256],120062:[[117],256],120063:[[118],256]},54528:{120064:[[119],256],120065:[[120],256],120066:[[121],256],120067:[[122],256],120068:[[65],256],120069:[[66],256],120071:[[68],256],120072:[[69],256],120073:[[70],256],120074:[[71],256],120077:[[74],256],120078:[[75],256],120079:[[76],256],120080:[[77],256],120081:[[78],256],120082:[[79],256],120083:[[80],256],120084:[[81],256],120086:[[83],256],120087:[[84],256],120088:[[85],256],120089:[[86],256],120090:[[87],256],120091:[[88],256],120092:[[89],256],120094:[[97],256],120095:[[98],256],120096:[[99],256],120097:[[100],256],120098:[[101],256],120099:[[102],256],120100:[[103],256],120101:[[104],256],120102:[[105],256],120103:[[106],256],120104:[[107],256],120105:[[108],256],120106:[[109],256],120107:[[110],256],120108:[[111],256],120109:[[112],256],120110:[[113],256],120111:[[114],256],120112:[[115],256],120113:[[116],256],120114:[[117],256],120115:[[118],256],120116:[[119],256],120117:[[120],256],120118:[[121],256],120119:[[122],256],120120:[[65],256],120121:[[66],256],120123:[[68],256],120124:[[69],256],120125:[[70],256],120126:[[71],256],120128:[[73],256],120129:[[74],256],120130:[[75],256],120131:[[76],256],120132:[[77],256],120134:[[79],256],120138:[[83],256],120139:[[84],256],120140:[[85],256],120141:[[86],256],120142:[[87],256],120143:[[88],256],120144:[[89],256],120146:[[97],256],120147:[[98],256],120148:[[99],256],120149:[[100],256],120150:[[101],256],120151:[[102],256],120152:[[103],256],120153:[[104],256],120154:[[105],256],120155:[[106],256],120156:[[107],256],120157:[[108],256],120158:[[109],256],120159:[[110],256],120160:[[111],256],120161:[[112],256],120162:[[113],256],120163:[[114],256],120164:[[115],256],120165:[[116],256],120166:[[117],256],120167:[[118],256],120168:[[119],256],120169:[[120],256],120170:[[121],256],120171:[[122],256],120172:[[65],256],120173:[[66],256],120174:[[67],256],120175:[[68],256],120176:[[69],256],120177:[[70],256],120178:[[71],256],120179:[[72],256],120180:[[73],256],120181:[[74],256],120182:[[75],256],120183:[[76],256],120184:[[77],256],120185:[[78],256],120186:[[79],256],120187:[[80],256],120188:[[81],256],120189:[[82],256],120190:[[83],256],120191:[[84],256],120192:[[85],256],120193:[[86],256],120194:[[87],256],120195:[[88],256],120196:[[89],256],120197:[[90],256],120198:[[97],256],120199:[[98],256],120200:[[99],256],120201:[[100],256],120202:[[101],256],120203:[[102],256],120204:[[103],256],120205:[[104],256],120206:[[105],256],120207:[[106],256],120208:[[107],256],120209:[[108],256],120210:[[109],256],120211:[[110],256],120212:[[111],256],120213:[[112],256],120214:[[113],256],120215:[[114],256],120216:[[115],256],120217:[[116],256],120218:[[117],256],120219:[[118],256],120220:[[119],256],120221:[[120],256],120222:[[121],256],120223:[[122],256],120224:[[65],256],120225:[[66],256],120226:[[67],256],120227:[[68],256],120228:[[69],256],120229:[[70],256],120230:[[71],256],120231:[[72],256],120232:[[73],256],120233:[[74],256],120234:[[75],256],120235:[[76],256],120236:[[77],256],120237:[[78],256],120238:[[79],256],120239:[[80],256],120240:[[81],256],120241:[[82],256],120242:[[83],256],120243:[[84],256],120244:[[85],256],120245:[[86],256],120246:[[87],256],120247:[[88],256],120248:[[89],256],120249:[[90],256],120250:[[97],256],120251:[[98],256],120252:[[99],256],120253:[[100],256],120254:[[101],256],120255:[[102],256],120256:[[103],256],120257:[[104],256],120258:[[105],256],120259:[[106],256],120260:[[107],256],120261:[[108],256],120262:[[109],256],120263:[[110],256],120264:[[111],256],120265:[[112],256],120266:[[113],256],120267:[[114],256],120268:[[115],256],120269:[[116],256],120270:[[117],256],120271:[[118],256],120272:[[119],256],120273:[[120],256],120274:[[121],256],120275:[[122],256],120276:[[65],256],120277:[[66],256],120278:[[67],256],120279:[[68],256],120280:[[69],256],120281:[[70],256],120282:[[71],256],120283:[[72],256],120284:[[73],256],120285:[[74],256],120286:[[75],256],120287:[[76],256],120288:[[77],256],120289:[[78],256],120290:[[79],256],120291:[[80],256],120292:[[81],256],120293:[[82],256],120294:[[83],256],120295:[[84],256],120296:[[85],256],120297:[[86],256],120298:[[87],256],120299:[[88],256],120300:[[89],256],120301:[[90],256],120302:[[97],256],120303:[[98],256],120304:[[99],256],120305:[[100],256],120306:[[101],256],120307:[[102],256],120308:[[103],256],120309:[[104],256],120310:[[105],256],120311:[[106],256],120312:[[107],256],120313:[[108],256],120314:[[109],256],120315:[[110],256],120316:[[111],256],120317:[[112],256],120318:[[113],256],120319:[[114],256]},54784:{120320:[[115],256],120321:[[116],256],120322:[[117],256],120323:[[118],256],120324:[[119],256],120325:[[120],256],120326:[[121],256],120327:[[122],256],120328:[[65],256],120329:[[66],256],120330:[[67],256],120331:[[68],256],120332:[[69],256],120333:[[70],256],120334:[[71],256],120335:[[72],256],120336:[[73],256],120337:[[74],256],120338:[[75],256],120339:[[76],256],120340:[[77],256],120341:[[78],256],120342:[[79],256],120343:[[80],256],120344:[[81],256],120345:[[82],256],120346:[[83],256],120347:[[84],256],120348:[[85],256],120349:[[86],256],120350:[[87],256],120351:[[88],256],120352:[[89],256],120353:[[90],256],120354:[[97],256],120355:[[98],256],120356:[[99],256],120357:[[100],256],120358:[[101],256],120359:[[102],256],120360:[[103],256],120361:[[104],256],120362:[[105],256],120363:[[106],256],120364:[[107],256],120365:[[108],256],120366:[[109],256],120367:[[110],256],120368:[[111],256],120369:[[112],256],120370:[[113],256],120371:[[114],256],120372:[[115],256],120373:[[116],256],120374:[[117],256],120375:[[118],256],120376:[[119],256],120377:[[120],256],120378:[[121],256],120379:[[122],256],120380:[[65],256],120381:[[66],256],120382:[[67],256],120383:[[68],256],120384:[[69],256],120385:[[70],256],120386:[[71],256],120387:[[72],256],120388:[[73],256],120389:[[74],256],120390:[[75],256],120391:[[76],256],120392:[[77],256],120393:[[78],256],120394:[[79],256],120395:[[80],256],120396:[[81],256],120397:[[82],256],120398:[[83],256],120399:[[84],256],120400:[[85],256],120401:[[86],256],120402:[[87],256],120403:[[88],256],120404:[[89],256],120405:[[90],256],120406:[[97],256],120407:[[98],256],120408:[[99],256],120409:[[100],256],120410:[[101],256],120411:[[102],256],120412:[[103],256],120413:[[104],256],120414:[[105],256],120415:[[106],256],120416:[[107],256],120417:[[108],256],120418:[[109],256],120419:[[110],256],120420:[[111],256],120421:[[112],256],120422:[[113],256],120423:[[114],256],120424:[[115],256],120425:[[116],256],120426:[[117],256],120427:[[118],256],120428:[[119],256],120429:[[120],256],120430:[[121],256],120431:[[122],256],120432:[[65],256],120433:[[66],256],120434:[[67],256],120435:[[68],256],120436:[[69],256],120437:[[70],256],120438:[[71],256],120439:[[72],256],120440:[[73],256],120441:[[74],256],120442:[[75],256],120443:[[76],256],120444:[[77],256],120445:[[78],256],120446:[[79],256],120447:[[80],256],120448:[[81],256],120449:[[82],256],120450:[[83],256],120451:[[84],256],120452:[[85],256],120453:[[86],256],120454:[[87],256],120455:[[88],256],120456:[[89],256],120457:[[90],256],120458:[[97],256],120459:[[98],256],120460:[[99],256],120461:[[100],256],120462:[[101],256],120463:[[102],256],120464:[[103],256],120465:[[104],256],120466:[[105],256],120467:[[106],256],120468:[[107],256],120469:[[108],256],120470:[[109],256],120471:[[110],256],120472:[[111],256],120473:[[112],256],120474:[[113],256],120475:[[114],256],120476:[[115],256],120477:[[116],256],120478:[[117],256],120479:[[118],256],120480:[[119],256],120481:[[120],256],120482:[[121],256],120483:[[122],256],120484:[[305],256],120485:[[567],256],120488:[[913],256],120489:[[914],256],120490:[[915],256],120491:[[916],256],120492:[[917],256],120493:[[918],256],120494:[[919],256],120495:[[920],256],120496:[[921],256],120497:[[922],256],120498:[[923],256],120499:[[924],256],120500:[[925],256],120501:[[926],256],120502:[[927],256],120503:[[928],256],120504:[[929],256],120505:[[1012],256],120506:[[931],256],120507:[[932],256],120508:[[933],256],120509:[[934],256],120510:[[935],256],120511:[[936],256],120512:[[937],256],120513:[[8711],256],120514:[[945],256],120515:[[946],256],120516:[[947],256],120517:[[948],256],120518:[[949],256],120519:[[950],256],120520:[[951],256],120521:[[952],256],120522:[[953],256],120523:[[954],256],120524:[[955],256],120525:[[956],256],120526:[[957],256],120527:[[958],256],120528:[[959],256],120529:[[960],256],120530:[[961],256],120531:[[962],256],120532:[[963],256],120533:[[964],256],120534:[[965],256],120535:[[966],256],120536:[[967],256],120537:[[968],256],120538:[[969],256],120539:[[8706],256],120540:[[1013],256],120541:[[977],256],120542:[[1008],256],120543:[[981],256],120544:[[1009],256],120545:[[982],256],120546:[[913],256],120547:[[914],256],120548:[[915],256],120549:[[916],256],120550:[[917],256],120551:[[918],256],120552:[[919],256],120553:[[920],256],120554:[[921],256],120555:[[922],256],120556:[[923],256],120557:[[924],256],120558:[[925],256],120559:[[926],256],120560:[[927],256],120561:[[928],256],120562:[[929],256],120563:[[1012],256],120564:[[931],256],120565:[[932],256],120566:[[933],256],120567:[[934],256],120568:[[935],256],120569:[[936],256],120570:[[937],256],120571:[[8711],256],120572:[[945],256],120573:[[946],256],120574:[[947],256],120575:[[948],256]},55040:{120576:[[949],256],120577:[[950],256],120578:[[951],256],120579:[[952],256],120580:[[953],256],120581:[[954],256],120582:[[955],256],120583:[[956],256],120584:[[957],256],120585:[[958],256],120586:[[959],256],120587:[[960],256],120588:[[961],256],120589:[[962],256],120590:[[963],256],120591:[[964],256],120592:[[965],256],120593:[[966],256],120594:[[967],256],120595:[[968],256],120596:[[969],256],120597:[[8706],256],120598:[[1013],256],120599:[[977],256],120600:[[1008],256],120601:[[981],256],120602:[[1009],256],120603:[[982],256],120604:[[913],256],120605:[[914],256],120606:[[915],256],120607:[[916],256],120608:[[917],256],120609:[[918],256],120610:[[919],256],120611:[[920],256],120612:[[921],256],120613:[[922],256],120614:[[923],256],120615:[[924],256],120616:[[925],256],120617:[[926],256],120618:[[927],256],120619:[[928],256],120620:[[929],256],120621:[[1012],256],120622:[[931],256],120623:[[932],256],120624:[[933],256],120625:[[934],256],120626:[[935],256],120627:[[936],256],120628:[[937],256],120629:[[8711],256],120630:[[945],256],120631:[[946],256],120632:[[947],256],120633:[[948],256],120634:[[949],256],120635:[[950],256],120636:[[951],256],120637:[[952],256],120638:[[953],256],120639:[[954],256],120640:[[955],256],120641:[[956],256],120642:[[957],256],120643:[[958],256],120644:[[959],256],120645:[[960],256],120646:[[961],256],120647:[[962],256],120648:[[963],256],120649:[[964],256],120650:[[965],256],120651:[[966],256],120652:[[967],256],120653:[[968],256],120654:[[969],256],120655:[[8706],256],120656:[[1013],256],120657:[[977],256],120658:[[1008],256],120659:[[981],256],120660:[[1009],256],120661:[[982],256],120662:[[913],256],120663:[[914],256],120664:[[915],256],120665:[[916],256],120666:[[917],256],120667:[[918],256],120668:[[919],256],120669:[[920],256],120670:[[921],256],120671:[[922],256],120672:[[923],256],120673:[[924],256],120674:[[925],256],120675:[[926],256],120676:[[927],256],120677:[[928],256],120678:[[929],256],120679:[[1012],256],120680:[[931],256],120681:[[932],256],120682:[[933],256],120683:[[934],256],120684:[[935],256],120685:[[936],256],120686:[[937],256],120687:[[8711],256],120688:[[945],256],120689:[[946],256],120690:[[947],256],120691:[[948],256],120692:[[949],256],120693:[[950],256],120694:[[951],256],120695:[[952],256],120696:[[953],256],120697:[[954],256],120698:[[955],256],120699:[[956],256],120700:[[957],256],120701:[[958],256],120702:[[959],256],120703:[[960],256],120704:[[961],256],120705:[[962],256],120706:[[963],256],120707:[[964],256],120708:[[965],256],120709:[[966],256],120710:[[967],256],120711:[[968],256],120712:[[969],256],120713:[[8706],256],120714:[[1013],256],120715:[[977],256],120716:[[1008],256],120717:[[981],256],120718:[[1009],256],120719:[[982],256],120720:[[913],256],120721:[[914],256],120722:[[915],256],120723:[[916],256],120724:[[917],256],120725:[[918],256],120726:[[919],256],120727:[[920],256],120728:[[921],256],120729:[[922],256],120730:[[923],256],120731:[[924],256],120732:[[925],256],120733:[[926],256],120734:[[927],256],120735:[[928],256],120736:[[929],256],120737:[[1012],256],120738:[[931],256],120739:[[932],256],120740:[[933],256],120741:[[934],256],120742:[[935],256],120743:[[936],256],120744:[[937],256],120745:[[8711],256],120746:[[945],256],120747:[[946],256],120748:[[947],256],120749:[[948],256],120750:[[949],256],120751:[[950],256],120752:[[951],256],120753:[[952],256],120754:[[953],256],120755:[[954],256],120756:[[955],256],120757:[[956],256],120758:[[957],256],120759:[[958],256],120760:[[959],256],120761:[[960],256],120762:[[961],256],120763:[[962],256],120764:[[963],256],120765:[[964],256],120766:[[965],256],120767:[[966],256],120768:[[967],256],120769:[[968],256],120770:[[969],256],120771:[[8706],256],120772:[[1013],256],120773:[[977],256],120774:[[1008],256],120775:[[981],256],120776:[[1009],256],120777:[[982],256],120778:[[988],256],120779:[[989],256],120782:[[48],256],120783:[[49],256],120784:[[50],256],120785:[[51],256],120786:[[52],256],120787:[[53],256],120788:[[54],256],120789:[[55],256],120790:[[56],256],120791:[[57],256],120792:[[48],256],120793:[[49],256],120794:[[50],256],120795:[[51],256],120796:[[52],256],120797:[[53],256],120798:[[54],256],120799:[[55],256],120800:[[56],256],120801:[[57],256],120802:[[48],256],120803:[[49],256],120804:[[50],256],120805:[[51],256],120806:[[52],256],120807:[[53],256],120808:[[54],256],120809:[[55],256],120810:[[56],256],120811:[[57],256],120812:[[48],256],120813:[[49],256],120814:[[50],256],120815:[[51],256],120816:[[52],256],120817:[[53],256],120818:[[54],256],120819:[[55],256],120820:[[56],256],120821:[[57],256],120822:[[48],256],120823:[[49],256],120824:[[50],256],120825:[[51],256],120826:[[52],256],120827:[[53],256],120828:[[54],256],120829:[[55],256],120830:[[56],256],120831:[[57],256]},59392:{125136:[,220],125137:[,220],125138:[,220],125139:[,220],125140:[,220],125141:[,220],125142:[,220]},60928:{126464:[[1575],256],126465:[[1576],256],126466:[[1580],256],126467:[[1583],256],126469:[[1608],256],126470:[[1586],256],126471:[[1581],256],126472:[[1591],256],126473:[[1610],256],126474:[[1603],256],126475:[[1604],256],126476:[[1605],256],126477:[[1606],256],126478:[[1587],256],126479:[[1593],256],126480:[[1601],256],126481:[[1589],256],126482:[[1602],256],126483:[[1585],256],126484:[[1588],256],126485:[[1578],256],126486:[[1579],256],126487:[[1582],256],126488:[[1584],256],126489:[[1590],256],126490:[[1592],256],126491:[[1594],256],126492:[[1646],256],126493:[[1722],256],126494:[[1697],256],126495:[[1647],256],126497:[[1576],256],126498:[[1580],256],126500:[[1607],256],126503:[[1581],256],126505:[[1610],256],126506:[[1603],256],126507:[[1604],256],126508:[[1605],256],126509:[[1606],256],126510:[[1587],256],126511:[[1593],256],126512:[[1601],256],126513:[[1589],256],126514:[[1602],256],126516:[[1588],256],126517:[[1578],256],126518:[[1579],256],126519:[[1582],256],126521:[[1590],256],126523:[[1594],256],126530:[[1580],256],126535:[[1581],256],126537:[[1610],256],126539:[[1604],256],126541:[[1606],256],126542:[[1587],256],126543:[[1593],256],126545:[[1589],256],126546:[[1602],256],126548:[[1588],256],126551:[[1582],256],126553:[[1590],256],126555:[[1594],256],126557:[[1722],256],126559:[[1647],256],126561:[[1576],256],126562:[[1580],256],126564:[[1607],256],126567:[[1581],256],126568:[[1591],256],126569:[[1610],256],126570:[[1603],256],126572:[[1605],256],126573:[[1606],256],126574:[[1587],256],126575:[[1593],256],126576:[[1601],256],126577:[[1589],256],126578:[[1602],256],126580:[[1588],256],126581:[[1578],256],126582:[[1579],256],126583:[[1582],256],126585:[[1590],256],126586:[[1592],256],126587:[[1594],256],126588:[[1646],256],126590:[[1697],256],126592:[[1575],256],126593:[[1576],256],126594:[[1580],256],126595:[[1583],256],126596:[[1607],256],126597:[[1608],256],126598:[[1586],256],126599:[[1581],256],126600:[[1591],256],126601:[[1610],256],126603:[[1604],256],126604:[[1605],256],126605:[[1606],256],126606:[[1587],256],126607:[[1593],256],126608:[[1601],256],126609:[[1589],256],126610:[[1602],256],126611:[[1585],256],126612:[[1588],256],126613:[[1578],256],126614:[[1579],256],126615:[[1582],256],126616:[[1584],256],126617:[[1590],256],126618:[[1592],256],126619:[[1594],256],126625:[[1576],256],126626:[[1580],256],126627:[[1583],256],126629:[[1608],256],126630:[[1586],256],126631:[[1581],256],126632:[[1591],256],126633:[[1610],256],126635:[[1604],256],126636:[[1605],256],126637:[[1606],256],126638:[[1587],256],126639:[[1593],256],126640:[[1601],256],126641:[[1589],256],126642:[[1602],256],126643:[[1585],256],126644:[[1588],256],126645:[[1578],256],126646:[[1579],256],126647:[[1582],256],126648:[[1584],256],126649:[[1590],256],126650:[[1592],256],126651:[[1594],256]},61696:{127232:[[48,46],256],127233:[[48,44],256],127234:[[49,44],256],127235:[[50,44],256],127236:[[51,44],256],127237:[[52,44],256],127238:[[53,44],256],127239:[[54,44],256],127240:[[55,44],256],127241:[[56,44],256],127242:[[57,44],256],127248:[[40,65,41],256],127249:[[40,66,41],256],127250:[[40,67,41],256],127251:[[40,68,41],256],127252:[[40,69,41],256],127253:[[40,70,41],256],127254:[[40,71,41],256],127255:[[40,72,41],256],127256:[[40,73,41],256],127257:[[40,74,41],256],127258:[[40,75,41],256],127259:[[40,76,41],256],127260:[[40,77,41],256],127261:[[40,78,41],256],127262:[[40,79,41],256],127263:[[40,80,41],256],127264:[[40,81,41],256],127265:[[40,82,41],256],127266:[[40,83,41],256],127267:[[40,84,41],256],127268:[[40,85,41],256],127269:[[40,86,41],256],127270:[[40,87,41],256],127271:[[40,88,41],256],127272:[[40,89,41],256],127273:[[40,90,41],256],127274:[[12308,83,12309],256],127275:[[67],256],127276:[[82],256],127277:[[67,68],256],127278:[[87,90],256],127280:[[65],256],127281:[[66],256],127282:[[67],256],127283:[[68],256],127284:[[69],256],127285:[[70],256],127286:[[71],256],127287:[[72],256],127288:[[73],256],127289:[[74],256],127290:[[75],256],127291:[[76],256],127292:[[77],256],127293:[[78],256],127294:[[79],256],127295:[[80],256],127296:[[81],256],127297:[[82],256],127298:[[83],256],127299:[[84],256],127300:[[85],256],127301:[[86],256],127302:[[87],256],127303:[[88],256],127304:[[89],256],127305:[[90],256],127306:[[72,86],256],127307:[[77,86],256],127308:[[83,68],256],127309:[[83,83],256],127310:[[80,80,86],256],127311:[[87,67],256],127338:[[77,67],256],127339:[[77,68],256],127376:[[68,74],256]},61952:{127488:[[12411,12363],256],127489:[[12467,12467],256],127490:[[12469],256],127504:[[25163],256],127505:[[23383],256],127506:[[21452],256],127507:[[12487],256],127508:[[20108],256],127509:[[22810],256],127510:[[35299],256],127511:[[22825],256],127512:[[20132],256],127513:[[26144],256],127514:[[28961],256],127515:[[26009],256],127516:[[21069],256],127517:[[24460],256],127518:[[20877],256],127519:[[26032],256],127520:[[21021],256],127521:[[32066],256],127522:[[29983],256],127523:[[36009],256],127524:[[22768],256],127525:[[21561],256],127526:[[28436],256],127527:[[25237],256],127528:[[25429],256],127529:[[19968],256],127530:[[19977],256],127531:[[36938],256],127532:[[24038],256],127533:[[20013],256],127534:[[21491],256],127535:[[25351],256],127536:[[36208],256],127537:[[25171],256],127538:[[31105],256],127539:[[31354],256],127540:[[21512],256],127541:[[28288],256],127542:[[26377],256],127543:[[26376],256],127544:[[30003],256],127545:[[21106],256],127546:[[21942],256],127552:[[12308,26412,12309],256],127553:[[12308,19977,12309],256],127554:[[12308,20108,12309],256],127555:[[12308,23433,12309],256],127556:[[12308,28857,12309],256],127557:[[12308,25171,12309],256],127558:[[12308,30423,12309],256],127559:[[12308,21213,12309],256],127560:[[12308,25943,12309],256],127568:[[24471],256],127569:[[21487],256]},63488:{194560:[[20029]],194561:[[20024]],194562:[[20033]],194563:[[131362]],194564:[[20320]],194565:[[20398]],194566:[[20411]],194567:[[20482]],194568:[[20602]],194569:[[20633]],194570:[[20711]],194571:[[20687]],194572:[[13470]],194573:[[132666]],194574:[[20813]],194575:[[20820]],194576:[[20836]],194577:[[20855]],194578:[[132380]],194579:[[13497]],194580:[[20839]],194581:[[20877]],194582:[[132427]],194583:[[20887]],194584:[[20900]],194585:[[20172]],194586:[[20908]],194587:[[20917]],194588:[[168415]],194589:[[20981]],194590:[[20995]],194591:[[13535]],194592:[[21051]],194593:[[21062]],194594:[[21106]],194595:[[21111]],194596:[[13589]],194597:[[21191]],194598:[[21193]],194599:[[21220]],194600:[[21242]],194601:[[21253]],194602:[[21254]],194603:[[21271]],194604:[[21321]],194605:[[21329]],194606:[[21338]],194607:[[21363]],194608:[[21373]],194609:[[21375]],194610:[[21375]],194611:[[21375]],194612:[[133676]],194613:[[28784]],194614:[[21450]],194615:[[21471]],194616:[[133987]],194617:[[21483]],194618:[[21489]],194619:[[21510]],194620:[[21662]],194621:[[21560]],194622:[[21576]],194623:[[21608]],194624:[[21666]],194625:[[21750]],194626:[[21776]],194627:[[21843]],194628:[[21859]],194629:[[21892]],194630:[[21892]],194631:[[21913]],194632:[[21931]],194633:[[21939]],194634:[[21954]],194635:[[22294]],194636:[[22022]],194637:[[22295]],194638:[[22097]],194639:[[22132]],194640:[[20999]],194641:[[22766]],194642:[[22478]],194643:[[22516]],194644:[[22541]],194645:[[22411]],194646:[[22578]],194647:[[22577]],194648:[[22700]],194649:[[136420]],194650:[[22770]],194651:[[22775]],194652:[[22790]],194653:[[22810]],194654:[[22818]],194655:[[22882]],194656:[[136872]],194657:[[136938]],194658:[[23020]],194659:[[23067]],194660:[[23079]],194661:[[23e3]],194662:[[23142]],194663:[[14062]],194664:[[14076]],194665:[[23304]],194666:[[23358]],194667:[[23358]],194668:[[137672]],194669:[[23491]],194670:[[23512]],194671:[[23527]],194672:[[23539]],194673:[[138008]],194674:[[23551]],194675:[[23558]],194676:[[24403]],194677:[[23586]],194678:[[14209]],194679:[[23648]],194680:[[23662]],194681:[[23744]],194682:[[23693]],194683:[[138724]],194684:[[23875]],194685:[[138726]],194686:[[23918]],194687:[[23915]],194688:[[23932]],194689:[[24033]],194690:[[24034]],194691:[[14383]],194692:[[24061]],194693:[[24104]],194694:[[24125]],194695:[[24169]],194696:[[14434]],194697:[[139651]],194698:[[14460]],194699:[[24240]],194700:[[24243]],194701:[[24246]],194702:[[24266]],194703:[[172946]],194704:[[24318]],194705:[[140081]],194706:[[140081]],194707:[[33281]],194708:[[24354]],194709:[[24354]],194710:[[14535]],194711:[[144056]],194712:[[156122]],194713:[[24418]],194714:[[24427]],194715:[[14563]],194716:[[24474]],194717:[[24525]],194718:[[24535]],194719:[[24569]],194720:[[24705]],194721:[[14650]],194722:[[14620]],194723:[[24724]],194724:[[141012]],194725:[[24775]],194726:[[24904]],194727:[[24908]],194728:[[24910]],194729:[[24908]],194730:[[24954]],194731:[[24974]],194732:[[25010]],194733:[[24996]],194734:[[25007]],194735:[[25054]],194736:[[25074]],194737:[[25078]],194738:[[25104]],194739:[[25115]],194740:[[25181]],194741:[[25265]],194742:[[25300]],194743:[[25424]],194744:[[142092]],194745:[[25405]],194746:[[25340]],194747:[[25448]],194748:[[25475]],194749:[[25572]],194750:[[142321]],194751:[[25634]],194752:[[25541]],194753:[[25513]],194754:[[14894]],194755:[[25705]],194756:[[25726]],194757:[[25757]],194758:[[25719]],194759:[[14956]],194760:[[25935]],194761:[[25964]],194762:[[143370]],194763:[[26083]],194764:[[26360]],194765:[[26185]],194766:[[15129]],194767:[[26257]],194768:[[15112]],194769:[[15076]],194770:[[20882]],194771:[[20885]],194772:[[26368]],194773:[[26268]],194774:[[32941]],194775:[[17369]],194776:[[26391]],194777:[[26395]],194778:[[26401]],194779:[[26462]],194780:[[26451]],194781:[[144323]],194782:[[15177]],194783:[[26618]],194784:[[26501]],194785:[[26706]],194786:[[26757]],194787:[[144493]],194788:[[26766]],194789:[[26655]],194790:[[26900]],194791:[[15261]],194792:[[26946]],194793:[[27043]],194794:[[27114]],194795:[[27304]],194796:[[145059]],194797:[[27355]],194798:[[15384]],194799:[[27425]],194800:[[145575]],194801:[[27476]],194802:[[15438]],194803:[[27506]],194804:[[27551]],194805:[[27578]],194806:[[27579]],194807:[[146061]],194808:[[138507]],194809:[[146170]],194810:[[27726]],194811:[[146620]],194812:[[27839]],194813:[[27853]],194814:[[27751]],194815:[[27926]]},63744:{63744:[[35912]],63745:[[26356]],63746:[[36554]],63747:[[36040]],63748:[[28369]],63749:[[20018]],63750:[[21477]],63751:[[40860]],63752:[[40860]],63753:[[22865]],63754:[[37329]],63755:[[21895]],63756:[[22856]],63757:[[25078]],63758:[[30313]],63759:[[32645]],63760:[[34367]],63761:[[34746]],63762:[[35064]],63763:[[37007]],63764:[[27138]],63765:[[27931]],63766:[[28889]],63767:[[29662]],63768:[[33853]],63769:[[37226]],63770:[[39409]],63771:[[20098]],63772:[[21365]],63773:[[27396]],63774:[[29211]],63775:[[34349]],63776:[[40478]],63777:[[23888]],63778:[[28651]],63779:[[34253]],63780:[[35172]],63781:[[25289]],63782:[[33240]],63783:[[34847]],63784:[[24266]],63785:[[26391]],63786:[[28010]],63787:[[29436]],63788:[[37070]],63789:[[20358]],63790:[[20919]],63791:[[21214]],63792:[[25796]],63793:[[27347]],63794:[[29200]],63795:[[30439]],63796:[[32769]],63797:[[34310]],63798:[[34396]],63799:[[36335]],63800:[[38706]],63801:[[39791]],63802:[[40442]],63803:[[30860]],63804:[[31103]],63805:[[32160]],63806:[[33737]],63807:[[37636]],63808:[[40575]],63809:[[35542]],63810:[[22751]],63811:[[24324]],63812:[[31840]],63813:[[32894]],63814:[[29282]],63815:[[30922]],63816:[[36034]],63817:[[38647]],63818:[[22744]],63819:[[23650]],63820:[[27155]],63821:[[28122]],63822:[[28431]],63823:[[32047]],63824:[[32311]],63825:[[38475]],63826:[[21202]],63827:[[32907]],63828:[[20956]],63829:[[20940]],63830:[[31260]],63831:[[32190]],63832:[[33777]],63833:[[38517]],63834:[[35712]],63835:[[25295]],63836:[[27138]],63837:[[35582]],63838:[[20025]],63839:[[23527]],63840:[[24594]],63841:[[29575]],63842:[[30064]],63843:[[21271]],63844:[[30971]],63845:[[20415]],63846:[[24489]],63847:[[19981]],63848:[[27852]],63849:[[25976]],63850:[[32034]],63851:[[21443]],63852:[[22622]],63853:[[30465]],63854:[[33865]],63855:[[35498]],63856:[[27578]],63857:[[36784]],63858:[[27784]],63859:[[25342]],63860:[[33509]],63861:[[25504]],63862:[[30053]],63863:[[20142]],63864:[[20841]],63865:[[20937]],63866:[[26753]],63867:[[31975]],63868:[[33391]],63869:[[35538]],63870:[[37327]],63871:[[21237]],63872:[[21570]],63873:[[22899]],63874:[[24300]],63875:[[26053]],63876:[[28670]],63877:[[31018]],63878:[[38317]],63879:[[39530]],63880:[[40599]],63881:[[40654]],63882:[[21147]],63883:[[26310]],63884:[[27511]],63885:[[36706]],63886:[[24180]],63887:[[24976]],63888:[[25088]],63889:[[25754]],63890:[[28451]],63891:[[29001]],63892:[[29833]],63893:[[31178]],63894:[[32244]],63895:[[32879]],63896:[[36646]],63897:[[34030]],63898:[[36899]],63899:[[37706]],63900:[[21015]],63901:[[21155]],63902:[[21693]],63903:[[28872]],63904:[[35010]],63905:[[35498]],63906:[[24265]],63907:[[24565]],63908:[[25467]],63909:[[27566]],63910:[[31806]],63911:[[29557]],63912:[[20196]],63913:[[22265]],63914:[[23527]],63915:[[23994]],63916:[[24604]],63917:[[29618]],63918:[[29801]],63919:[[32666]],63920:[[32838]],63921:[[37428]],63922:[[38646]],63923:[[38728]],63924:[[38936]],63925:[[20363]],63926:[[31150]],63927:[[37300]],63928:[[38584]],63929:[[24801]],63930:[[20102]],63931:[[20698]],63932:[[23534]],63933:[[23615]],63934:[[26009]],63935:[[27138]],63936:[[29134]],63937:[[30274]],63938:[[34044]],63939:[[36988]],63940:[[40845]],63941:[[26248]],63942:[[38446]],63943:[[21129]],63944:[[26491]],63945:[[26611]],63946:[[27969]],63947:[[28316]],63948:[[29705]],63949:[[30041]],63950:[[30827]],63951:[[32016]],63952:[[39006]],63953:[[20845]],63954:[[25134]],63955:[[38520]],63956:[[20523]],63957:[[23833]],63958:[[28138]],63959:[[36650]],63960:[[24459]],63961:[[24900]],63962:[[26647]],63963:[[29575]],63964:[[38534]],63965:[[21033]],63966:[[21519]],63967:[[23653]],63968:[[26131]],63969:[[26446]],63970:[[26792]],63971:[[27877]],63972:[[29702]],63973:[[30178]],63974:[[32633]],63975:[[35023]],63976:[[35041]],63977:[[37324]],63978:[[38626]],63979:[[21311]],63980:[[28346]],63981:[[21533]],63982:[[29136]],63983:[[29848]],63984:[[34298]],63985:[[38563]],63986:[[40023]],63987:[[40607]],63988:[[26519]],63989:[[28107]],63990:[[33256]],63991:[[31435]],63992:[[31520]],63993:[[31890]],63994:[[29376]],63995:[[28825]],63996:[[35672]],63997:[[20160]],63998:[[33590]],63999:[[21050]],194816:[[27966]],194817:[[28023]],194818:[[27969]],194819:[[28009]],194820:[[28024]],194821:[[28037]],194822:[[146718]],194823:[[27956]],194824:[[28207]],194825:[[28270]],194826:[[15667]],194827:[[28363]],194828:[[28359]],194829:[[147153]],194830:[[28153]],194831:[[28526]],194832:[[147294]],194833:[[147342]],194834:[[28614]],194835:[[28729]],194836:[[28702]],194837:[[28699]],194838:[[15766]],194839:[[28746]],194840:[[28797]],194841:[[28791]],194842:[[28845]],194843:[[132389]],194844:[[28997]],194845:[[148067]],194846:[[29084]],194847:[[148395]],194848:[[29224]],194849:[[29237]],194850:[[29264]],194851:[[149e3]],194852:[[29312]],194853:[[29333]],194854:[[149301]],194855:[[149524]],194856:[[29562]],194857:[[29579]],194858:[[16044]],194859:[[29605]],194860:[[16056]],194861:[[16056]],194862:[[29767]],194863:[[29788]],194864:[[29809]],194865:[[29829]],194866:[[29898]],194867:[[16155]],194868:[[29988]],194869:[[150582]],194870:[[30014]],194871:[[150674]],194872:[[30064]],194873:[[139679]],194874:[[30224]],194875:[[151457]],194876:[[151480]],194877:[[151620]],194878:[[16380]],194879:[[16392]],194880:[[30452]],194881:[[151795]],194882:[[151794]],194883:[[151833]],194884:[[151859]],194885:[[30494]],194886:[[30495]],194887:[[30495]],194888:[[30538]],194889:[[16441]],194890:[[30603]],194891:[[16454]],194892:[[16534]],194893:[[152605]],194894:[[30798]],194895:[[30860]],194896:[[30924]],194897:[[16611]],194898:[[153126]],194899:[[31062]],194900:[[153242]],194901:[[153285]],194902:[[31119]],194903:[[31211]],194904:[[16687]],194905:[[31296]],194906:[[31306]],194907:[[31311]],194908:[[153980]],194909:[[154279]],194910:[[154279]],194911:[[31470]],194912:[[16898]],194913:[[154539]],194914:[[31686]],194915:[[31689]],194916:[[16935]],194917:[[154752]],194918:[[31954]],194919:[[17056]],194920:[[31976]],194921:[[31971]],194922:[[32e3]],194923:[[155526]],194924:[[32099]],194925:[[17153]],194926:[[32199]],194927:[[32258]],194928:[[32325]],194929:[[17204]],194930:[[156200]],194931:[[156231]],194932:[[17241]],194933:[[156377]],194934:[[32634]],194935:[[156478]],194936:[[32661]],194937:[[32762]],194938:[[32773]],194939:[[156890]],194940:[[156963]],194941:[[32864]],194942:[[157096]],194943:[[32880]],194944:[[144223]],194945:[[17365]],194946:[[32946]],194947:[[33027]],194948:[[17419]],194949:[[33086]],194950:[[23221]],194951:[[157607]],194952:[[157621]],194953:[[144275]],194954:[[144284]],194955:[[33281]],194956:[[33284]],194957:[[36766]],194958:[[17515]],194959:[[33425]],194960:[[33419]],194961:[[33437]],194962:[[21171]],194963:[[33457]],194964:[[33459]],194965:[[33469]],194966:[[33510]],194967:[[158524]],194968:[[33509]],194969:[[33565]],194970:[[33635]],194971:[[33709]],194972:[[33571]],194973:[[33725]],194974:[[33767]],194975:[[33879]],194976:[[33619]],194977:[[33738]],194978:[[33740]],194979:[[33756]],194980:[[158774]],194981:[[159083]],194982:[[158933]],194983:[[17707]],194984:[[34033]],194985:[[34035]],194986:[[34070]],194987:[[160714]],194988:[[34148]],194989:[[159532]],194990:[[17757]],194991:[[17761]],194992:[[159665]],194993:[[159954]],194994:[[17771]],194995:[[34384]],194996:[[34396]],194997:[[34407]],194998:[[34409]],194999:[[34473]],195e3:[[34440]],195001:[[34574]],195002:[[34530]],195003:[[34681]],195004:[[34600]],195005:[[34667]],195006:[[34694]],195007:[[17879]],195008:[[34785]],195009:[[34817]],195010:[[17913]],195011:[[34912]],195012:[[34915]],195013:[[161383]],195014:[[35031]],195015:[[35038]],195016:[[17973]],195017:[[35066]],195018:[[13499]],195019:[[161966]],195020:[[162150]],195021:[[18110]],195022:[[18119]],195023:[[35488]],195024:[[35565]],195025:[[35722]],195026:[[35925]],195027:[[162984]],195028:[[36011]],195029:[[36033]],195030:[[36123]],195031:[[36215]],195032:[[163631]],195033:[[133124]],195034:[[36299]],195035:[[36284]],195036:[[36336]],195037:[[133342]],195038:[[36564]],195039:[[36664]],195040:[[165330]],195041:[[165357]],195042:[[37012]],195043:[[37105]],195044:[[37137]],195045:[[165678]],195046:[[37147]],195047:[[37432]],195048:[[37591]],195049:[[37592]],195050:[[37500]],195051:[[37881]],195052:[[37909]],195053:[[166906]],195054:[[38283]],195055:[[18837]],195056:[[38327]],195057:[[167287]],195058:[[18918]],195059:[[38595]],195060:[[23986]],195061:[[38691]],195062:[[168261]],195063:[[168474]],195064:[[19054]],195065:[[19062]],195066:[[38880]],195067:[[168970]],195068:[[19122]],195069:[[169110]],195070:[[38923]],195071:[[38923]]},64e3:{64e3:[[20999]],64001:[[24230]],64002:[[25299]],64003:[[31958]],64004:[[23429]],64005:[[27934]],64006:[[26292]],64007:[[36667]],64008:[[34892]],64009:[[38477]],64010:[[35211]],64011:[[24275]],64012:[[20800]],64013:[[21952]],64016:[[22618]],64018:[[26228]],64021:[[20958]],64022:[[29482]],64023:[[30410]],64024:[[31036]],64025:[[31070]],64026:[[31077]],64027:[[31119]],64028:[[38742]],64029:[[31934]],64030:[[32701]],64032:[[34322]],64034:[[35576]],64037:[[36920]],64038:[[37117]],64042:[[39151]],64043:[[39164]],64044:[[39208]],64045:[[40372]],64046:[[37086]],64047:[[38583]],64048:[[20398]],64049:[[20711]],64050:[[20813]],64051:[[21193]],64052:[[21220]],64053:[[21329]],64054:[[21917]],64055:[[22022]],64056:[[22120]],64057:[[22592]],64058:[[22696]],64059:[[23652]],64060:[[23662]],64061:[[24724]],64062:[[24936]],64063:[[24974]],64064:[[25074]],64065:[[25935]],64066:[[26082]],64067:[[26257]],64068:[[26757]],64069:[[28023]],64070:[[28186]],64071:[[28450]],64072:[[29038]],64073:[[29227]],64074:[[29730]],64075:[[30865]],64076:[[31038]],64077:[[31049]],64078:[[31048]],64079:[[31056]],64080:[[31062]],64081:[[31069]],64082:[[31117]],64083:[[31118]],64084:[[31296]],64085:[[31361]],64086:[[31680]],64087:[[32244]],64088:[[32265]],64089:[[32321]],64090:[[32626]],64091:[[32773]],64092:[[33261]],64093:[[33401]],64094:[[33401]],64095:[[33879]],64096:[[35088]],64097:[[35222]],64098:[[35585]],64099:[[35641]],64100:[[36051]],64101:[[36104]],64102:[[36790]],64103:[[36920]],64104:[[38627]],64105:[[38911]],64106:[[38971]],64107:[[24693]],64108:[[148206]],64109:[[33304]],64112:[[20006]],64113:[[20917]],64114:[[20840]],64115:[[20352]],64116:[[20805]],64117:[[20864]],64118:[[21191]],64119:[[21242]],64120:[[21917]],64121:[[21845]],64122:[[21913]],64123:[[21986]],64124:[[22618]],64125:[[22707]],64126:[[22852]],64127:[[22868]],64128:[[23138]],64129:[[23336]],64130:[[24274]],64131:[[24281]],64132:[[24425]],64133:[[24493]],64134:[[24792]],64135:[[24910]],64136:[[24840]],64137:[[24974]],64138:[[24928]],64139:[[25074]],64140:[[25140]],64141:[[25540]],64142:[[25628]],64143:[[25682]],64144:[[25942]],64145:[[26228]],64146:[[26391]],64147:[[26395]],64148:[[26454]],64149:[[27513]],64150:[[27578]],64151:[[27969]],64152:[[28379]],64153:[[28363]],64154:[[28450]],64155:[[28702]],64156:[[29038]],64157:[[30631]],64158:[[29237]],64159:[[29359]],64160:[[29482]],64161:[[29809]],64162:[[29958]],64163:[[30011]],64164:[[30237]],64165:[[30239]],64166:[[30410]],64167:[[30427]],64168:[[30452]],64169:[[30538]],64170:[[30528]],64171:[[30924]],64172:[[31409]],64173:[[31680]],64174:[[31867]],64175:[[32091]],64176:[[32244]],64177:[[32574]],64178:[[32773]],64179:[[33618]],64180:[[33775]],64181:[[34681]],64182:[[35137]],64183:[[35206]],64184:[[35222]],64185:[[35519]],64186:[[35576]],64187:[[35531]],64188:[[35585]],64189:[[35582]],64190:[[35565]],64191:[[35641]],64192:[[35722]],64193:[[36104]],64194:[[36664]],64195:[[36978]],64196:[[37273]],64197:[[37494]],64198:[[38524]],64199:[[38627]],64200:[[38742]],64201:[[38875]],64202:[[38911]],64203:[[38923]],64204:[[38971]],64205:[[39698]],64206:[[40860]],64207:[[141386]],64208:[[141380]],64209:[[144341]],64210:[[15261]],64211:[[16408]],64212:[[16441]],64213:[[152137]],64214:[[154832]],64215:[[163539]],64216:[[40771]],64217:[[40846]],195072:[[38953]],195073:[[169398]],195074:[[39138]],195075:[[19251]],195076:[[39209]],195077:[[39335]],195078:[[39362]],195079:[[39422]],195080:[[19406]],195081:[[170800]],195082:[[39698]],195083:[[4e4]],195084:[[40189]],195085:[[19662]],195086:[[19693]],195087:[[40295]],195088:[[172238]],195089:[[19704]],195090:[[172293]],195091:[[172558]],195092:[[172689]],195093:[[40635]],195094:[[19798]],195095:[[40697]],195096:[[40702]],195097:[[40709]],195098:[[40719]],195099:[[40726]],195100:[[40763]],195101:[[173568]]},64256:{64256:[[102,102],256],64257:[[102,105],256],64258:[[102,108],256],64259:[[102,102,105],256],64260:[[102,102,108],256],64261:[[383,116],256],64262:[[115,116],256],64275:[[1396,1398],256],64276:[[1396,1381],256],64277:[[1396,1387],256],64278:[[1406,1398],256],64279:[[1396,1389],256],64285:[[1497,1460],512],64286:[,26],64287:[[1522,1463],512],64288:[[1506],256],64289:[[1488],256],64290:[[1491],256],64291:[[1492],256],64292:[[1499],256],64293:[[1500],256],64294:[[1501],256],64295:[[1512],256],64296:[[1514],256],64297:[[43],256],64298:[[1513,1473],512],64299:[[1513,1474],512],64300:[[64329,1473],512],64301:[[64329,1474],512],64302:[[1488,1463],512],64303:[[1488,1464],512],64304:[[1488,1468],512],64305:[[1489,1468],512],64306:[[1490,1468],512],64307:[[1491,1468],512],64308:[[1492,1468],512],64309:[[1493,1468],512],64310:[[1494,1468],512],64312:[[1496,1468],512],64313:[[1497,1468],512],64314:[[1498,1468],512],64315:[[1499,1468],512],64316:[[1500,1468],512],64318:[[1502,1468],512],64320:[[1504,1468],512],64321:[[1505,1468],512],64323:[[1507,1468],512],64324:[[1508,1468],512],64326:[[1510,1468],512],64327:[[1511,1468],512],64328:[[1512,1468],512],64329:[[1513,1468],512],64330:[[1514,1468],512],64331:[[1493,1465],512],64332:[[1489,1471],512],64333:[[1499,1471],512],64334:[[1508,1471],512],64335:[[1488,1500],256],64336:[[1649],256],64337:[[1649],256],64338:[[1659],256],64339:[[1659],256],64340:[[1659],256],64341:[[1659],256],64342:[[1662],256],64343:[[1662],256],64344:[[1662],256],64345:[[1662],256],64346:[[1664],256],64347:[[1664],256],64348:[[1664],256],64349:[[1664],256],64350:[[1658],256],64351:[[1658],256],64352:[[1658],256],64353:[[1658],256],64354:[[1663],256],64355:[[1663],256],64356:[[1663],256],64357:[[1663],256],64358:[[1657],256],64359:[[1657],256],64360:[[1657],256],64361:[[1657],256],64362:[[1700],256],64363:[[1700],256],64364:[[1700],256],64365:[[1700],256],64366:[[1702],256],64367:[[1702],256],64368:[[1702],256],64369:[[1702],256],64370:[[1668],256],64371:[[1668],256],64372:[[1668],256],64373:[[1668],256],64374:[[1667],256],64375:[[1667],256],64376:[[1667],256],64377:[[1667],256],64378:[[1670],256],64379:[[1670],256],64380:[[1670],256],64381:[[1670],256],64382:[[1671],256],64383:[[1671],256],64384:[[1671],256],64385:[[1671],256],64386:[[1677],256],64387:[[1677],256],64388:[[1676],256],64389:[[1676],256],64390:[[1678],256],64391:[[1678],256],64392:[[1672],256],64393:[[1672],256],64394:[[1688],256],64395:[[1688],256],64396:[[1681],256],64397:[[1681],256],64398:[[1705],256],64399:[[1705],256],64400:[[1705],256],64401:[[1705],256],64402:[[1711],256],64403:[[1711],256],64404:[[1711],256],64405:[[1711],256],64406:[[1715],256],64407:[[1715],256],64408:[[1715],256],64409:[[1715],256],64410:[[1713],256],64411:[[1713],256],64412:[[1713],256],64413:[[1713],256],64414:[[1722],256],64415:[[1722],256],64416:[[1723],256],64417:[[1723],256],64418:[[1723],256],64419:[[1723],256],64420:[[1728],256],64421:[[1728],256],64422:[[1729],256],64423:[[1729],256],64424:[[1729],256],64425:[[1729],256],64426:[[1726],256],64427:[[1726],256],64428:[[1726],256],64429:[[1726],256],64430:[[1746],256],64431:[[1746],256],64432:[[1747],256],64433:[[1747],256],64467:[[1709],256],64468:[[1709],256],64469:[[1709],256],64470:[[1709],256],64471:[[1735],256],64472:[[1735],256],64473:[[1734],256],64474:[[1734],256],64475:[[1736],256],64476:[[1736],256],64477:[[1655],256],64478:[[1739],256],64479:[[1739],256],64480:[[1733],256],64481:[[1733],256],64482:[[1737],256],64483:[[1737],256],64484:[[1744],256],64485:[[1744],256],64486:[[1744],256],64487:[[1744],256],64488:[[1609],256],64489:[[1609],256],64490:[[1574,1575],256],64491:[[1574,1575],256],64492:[[1574,1749],256],64493:[[1574,1749],256],64494:[[1574,1608],256],64495:[[1574,1608],256],64496:[[1574,1735],256],64497:[[1574,1735],256],64498:[[1574,1734],256],64499:[[1574,1734],256],64500:[[1574,1736],256],64501:[[1574,1736],256],64502:[[1574,1744],256],64503:[[1574,1744],256],64504:[[1574,1744],256],64505:[[1574,1609],256],64506:[[1574,1609],256],64507:[[1574,1609],256],64508:[[1740],256],64509:[[1740],256],64510:[[1740],256],64511:[[1740],256]},64512:{64512:[[1574,1580],256],64513:[[1574,1581],256],64514:[[1574,1605],256],64515:[[1574,1609],256],64516:[[1574,1610],256],64517:[[1576,1580],256],64518:[[1576,1581],256],64519:[[1576,1582],256],64520:[[1576,1605],256],64521:[[1576,1609],256],64522:[[1576,1610],256],64523:[[1578,1580],256],64524:[[1578,1581],256],64525:[[1578,1582],256],64526:[[1578,1605],256],64527:[[1578,1609],256],64528:[[1578,1610],256],64529:[[1579,1580],256],64530:[[1579,1605],256],64531:[[1579,1609],256],64532:[[1579,1610],256],64533:[[1580,1581],256],64534:[[1580,1605],256],64535:[[1581,1580],256],64536:[[1581,1605],256],64537:[[1582,1580],256],64538:[[1582,1581],256],64539:[[1582,1605],256],64540:[[1587,1580],256],64541:[[1587,1581],256],64542:[[1587,1582],256],64543:[[1587,1605],256],64544:[[1589,1581],256],64545:[[1589,1605],256],64546:[[1590,1580],256],64547:[[1590,1581],256],64548:[[1590,1582],256],64549:[[1590,1605],256],64550:[[1591,1581],256],64551:[[1591,1605],256],64552:[[1592,1605],256],64553:[[1593,1580],256],64554:[[1593,1605],256],64555:[[1594,1580],256],64556:[[1594,1605],256],64557:[[1601,1580],256],64558:[[1601,1581],256],64559:[[1601,1582],256],64560:[[1601,1605],256],64561:[[1601,1609],256],64562:[[1601,1610],256],64563:[[1602,1581],256],64564:[[1602,1605],256],64565:[[1602,1609],256],64566:[[1602,1610],256],64567:[[1603,1575],256],64568:[[1603,1580],256],64569:[[1603,1581],256],64570:[[1603,1582],256],64571:[[1603,1604],256],64572:[[1603,1605],256],64573:[[1603,1609],256],64574:[[1603,1610],256],64575:[[1604,1580],256],64576:[[1604,1581],256],64577:[[1604,1582],256],64578:[[1604,1605],256],64579:[[1604,1609],256],64580:[[1604,1610],256],64581:[[1605,1580],256],64582:[[1605,1581],256],64583:[[1605,1582],256],64584:[[1605,1605],256],64585:[[1605,1609],256],64586:[[1605,1610],256],64587:[[1606,1580],256],64588:[[1606,1581],256],64589:[[1606,1582],256],64590:[[1606,1605],256],64591:[[1606,1609],256],64592:[[1606,1610],256],64593:[[1607,1580],256],64594:[[1607,1605],256],64595:[[1607,1609],256],64596:[[1607,1610],256],64597:[[1610,1580],256],64598:[[1610,1581],256],64599:[[1610,1582],256],64600:[[1610,1605],256],64601:[[1610,1609],256],64602:[[1610,1610],256],64603:[[1584,1648],256],64604:[[1585,1648],256],64605:[[1609,1648],256],64606:[[32,1612,1617],256],64607:[[32,1613,1617],256],64608:[[32,1614,1617],256],64609:[[32,1615,1617],256],64610:[[32,1616,1617],256],64611:[[32,1617,1648],256],64612:[[1574,1585],256],64613:[[1574,1586],256],64614:[[1574,1605],256],64615:[[1574,1606],256],64616:[[1574,1609],256],64617:[[1574,1610],256],64618:[[1576,1585],256],64619:[[1576,1586],256],64620:[[1576,1605],256],64621:[[1576,1606],256],64622:[[1576,1609],256],64623:[[1576,1610],256],64624:[[1578,1585],256],64625:[[1578,1586],256],64626:[[1578,1605],256],64627:[[1578,1606],256],64628:[[1578,1609],256],64629:[[1578,1610],256],64630:[[1579,1585],256],64631:[[1579,1586],256],64632:[[1579,1605],256],64633:[[1579,1606],256],64634:[[1579,1609],256],64635:[[1579,1610],256],64636:[[1601,1609],256],64637:[[1601,1610],256],64638:[[1602,1609],256],64639:[[1602,1610],256],64640:[[1603,1575],256],64641:[[1603,1604],256],64642:[[1603,1605],256],64643:[[1603,1609],256],64644:[[1603,1610],256],64645:[[1604,1605],256],64646:[[1604,1609],256],64647:[[1604,1610],256],64648:[[1605,1575],256],64649:[[1605,1605],256],64650:[[1606,1585],256],64651:[[1606,1586],256],64652:[[1606,1605],256],64653:[[1606,1606],256],64654:[[1606,1609],256],64655:[[1606,1610],256],64656:[[1609,1648],256],64657:[[1610,1585],256],64658:[[1610,1586],256],64659:[[1610,1605],256],64660:[[1610,1606],256],64661:[[1610,1609],256],64662:[[1610,1610],256],64663:[[1574,1580],256],64664:[[1574,1581],256],64665:[[1574,1582],256],64666:[[1574,1605],256],64667:[[1574,1607],256],64668:[[1576,1580],256],64669:[[1576,1581],256],64670:[[1576,1582],256],64671:[[1576,1605],256],64672:[[1576,1607],256],64673:[[1578,1580],256],64674:[[1578,1581],256],64675:[[1578,1582],256],64676:[[1578,1605],256],64677:[[1578,1607],256],64678:[[1579,1605],256],64679:[[1580,1581],256],64680:[[1580,1605],256],64681:[[1581,1580],256],64682:[[1581,1605],256],64683:[[1582,1580],256],64684:[[1582,1605],256],64685:[[1587,1580],256],64686:[[1587,1581],256],64687:[[1587,1582],256],64688:[[1587,1605],256],64689:[[1589,1581],256],64690:[[1589,1582],256],64691:[[1589,1605],256],64692:[[1590,1580],256],64693:[[1590,1581],256],64694:[[1590,1582],256],64695:[[1590,1605],256],64696:[[1591,1581],256],64697:[[1592,1605],256],64698:[[1593,1580],256],64699:[[1593,1605],256],64700:[[1594,1580],256],64701:[[1594,1605],256],64702:[[1601,1580],256],64703:[[1601,1581],256],64704:[[1601,1582],256],64705:[[1601,1605],256],64706:[[1602,1581],256],64707:[[1602,1605],256],64708:[[1603,1580],256],64709:[[1603,1581],256],64710:[[1603,1582],256],64711:[[1603,1604],256],64712:[[1603,1605],256],64713:[[1604,1580],256],64714:[[1604,1581],256],64715:[[1604,1582],256],64716:[[1604,1605],256],64717:[[1604,1607],256],64718:[[1605,1580],256],64719:[[1605,1581],256],64720:[[1605,1582],256],64721:[[1605,1605],256],64722:[[1606,1580],256],64723:[[1606,1581],256],64724:[[1606,1582],256],64725:[[1606,1605],256],64726:[[1606,1607],256],64727:[[1607,1580],256],64728:[[1607,1605],256],64729:[[1607,1648],256],64730:[[1610,1580],256],64731:[[1610,1581],256],64732:[[1610,1582],256],64733:[[1610,1605],256],64734:[[1610,1607],256],64735:[[1574,1605],256],64736:[[1574,1607],256],64737:[[1576,1605],256],64738:[[1576,1607],256],64739:[[1578,1605],256],64740:[[1578,1607],256],64741:[[1579,1605],256],64742:[[1579,1607],256],64743:[[1587,1605],256],64744:[[1587,1607],256],64745:[[1588,1605],256],64746:[[1588,1607],256],64747:[[1603,1604],256],64748:[[1603,1605],256],64749:[[1604,1605],256],64750:[[1606,1605],256],64751:[[1606,1607],256],64752:[[1610,1605],256],64753:[[1610,1607],256],64754:[[1600,1614,1617],256],64755:[[1600,1615,1617],256],64756:[[1600,1616,1617],256],64757:[[1591,1609],256],64758:[[1591,1610],256],64759:[[1593,1609],256],64760:[[1593,1610],256],64761:[[1594,1609],256],64762:[[1594,1610],256],64763:[[1587,1609],256],64764:[[1587,1610],256],64765:[[1588,1609],256],64766:[[1588,1610],256],64767:[[1581,1609],256]},64768:{64768:[[1581,1610],256],64769:[[1580,1609],256],64770:[[1580,1610],256],64771:[[1582,1609],256],64772:[[1582,1610],256],64773:[[1589,1609],256],64774:[[1589,1610],256],64775:[[1590,1609],256],64776:[[1590,1610],256],64777:[[1588,1580],256],64778:[[1588,1581],256],64779:[[1588,1582],256],64780:[[1588,1605],256],64781:[[1588,1585],256],64782:[[1587,1585],256],64783:[[1589,1585],256],64784:[[1590,1585],256],64785:[[1591,1609],256],64786:[[1591,1610],256],64787:[[1593,1609],256],64788:[[1593,1610],256],64789:[[1594,1609],256],64790:[[1594,1610],256],64791:[[1587,1609],256],64792:[[1587,1610],256],64793:[[1588,1609],256],64794:[[1588,1610],256],64795:[[1581,1609],256],64796:[[1581,1610],256],64797:[[1580,1609],256],64798:[[1580,1610],256],64799:[[1582,1609],256],64800:[[1582,1610],256],64801:[[1589,1609],256],64802:[[1589,1610],256],64803:[[1590,1609],256],64804:[[1590,1610],256],64805:[[1588,1580],256],64806:[[1588,1581],256],64807:[[1588,1582],256],64808:[[1588,1605],256],64809:[[1588,1585],256],64810:[[1587,1585],256],64811:[[1589,1585],256],64812:[[1590,1585],256],64813:[[1588,1580],256],64814:[[1588,1581],256],64815:[[1588,1582],256],64816:[[1588,1605],256],64817:[[1587,1607],256],64818:[[1588,1607],256],64819:[[1591,1605],256],64820:[[1587,1580],256],64821:[[1587,1581],256],64822:[[1587,1582],256],64823:[[1588,1580],256],64824:[[1588,1581],256],64825:[[1588,1582],256],64826:[[1591,1605],256],64827:[[1592,1605],256],64828:[[1575,1611],256],64829:[[1575,1611],256],64848:[[1578,1580,1605],256],64849:[[1578,1581,1580],256],64850:[[1578,1581,1580],256],64851:[[1578,1581,1605],256],64852:[[1578,1582,1605],256],64853:[[1578,1605,1580],256],64854:[[1578,1605,1581],256],64855:[[1578,1605,1582],256],64856:[[1580,1605,1581],256],64857:[[1580,1605,1581],256],64858:[[1581,1605,1610],256],64859:[[1581,1605,1609],256],64860:[[1587,1581,1580],256],64861:[[1587,1580,1581],256],64862:[[1587,1580,1609],256],64863:[[1587,1605,1581],256],64864:[[1587,1605,1581],256],64865:[[1587,1605,1580],256],64866:[[1587,1605,1605],256],64867:[[1587,1605,1605],256],64868:[[1589,1581,1581],256],64869:[[1589,1581,1581],256],64870:[[1589,1605,1605],256],64871:[[1588,1581,1605],256],64872:[[1588,1581,1605],256],64873:[[1588,1580,1610],256],64874:[[1588,1605,1582],256],64875:[[1588,1605,1582],256],64876:[[1588,1605,1605],256],64877:[[1588,1605,1605],256],64878:[[1590,1581,1609],256],64879:[[1590,1582,1605],256],64880:[[1590,1582,1605],256],64881:[[1591,1605,1581],256],64882:[[1591,1605,1581],256],64883:[[1591,1605,1605],256],64884:[[1591,1605,1610],256],64885:[[1593,1580,1605],256],64886:[[1593,1605,1605],256],64887:[[1593,1605,1605],256],64888:[[1593,1605,1609],256],64889:[[1594,1605,1605],256],64890:[[1594,1605,1610],256],64891:[[1594,1605,1609],256],64892:[[1601,1582,1605],256],64893:[[1601,1582,1605],256],64894:[[1602,1605,1581],256],64895:[[1602,1605,1605],256],64896:[[1604,1581,1605],256],64897:[[1604,1581,1610],256],64898:[[1604,1581,1609],256],64899:[[1604,1580,1580],256],64900:[[1604,1580,1580],256],64901:[[1604,1582,1605],256],64902:[[1604,1582,1605],256],64903:[[1604,1605,1581],256],64904:[[1604,1605,1581],256],64905:[[1605,1581,1580],256],64906:[[1605,1581,1605],256],64907:[[1605,1581,1610],256],64908:[[1605,1580,1581],256],64909:[[1605,1580,1605],256],64910:[[1605,1582,1580],256],64911:[[1605,1582,1605],256],64914:[[1605,1580,1582],256],64915:[[1607,1605,1580],256],64916:[[1607,1605,1605],256],64917:[[1606,1581,1605],256],64918:[[1606,1581,1609],256],64919:[[1606,1580,1605],256],64920:[[1606,1580,1605],256],64921:[[1606,1580,1609],256],64922:[[1606,1605,1610],256],64923:[[1606,1605,1609],256],64924:[[1610,1605,1605],256],64925:[[1610,1605,1605],256],64926:[[1576,1582,1610],256],64927:[[1578,1580,1610],256],64928:[[1578,1580,1609],256],64929:[[1578,1582,1610],256],64930:[[1578,1582,1609],256],64931:[[1578,1605,1610],256],64932:[[1578,1605,1609],256],64933:[[1580,1605,1610],256],64934:[[1580,1581,1609],256],64935:[[1580,1605,1609],256],64936:[[1587,1582,1609],256],64937:[[1589,1581,1610],256],64938:[[1588,1581,1610],256],64939:[[1590,1581,1610],256],64940:[[1604,1580,1610],256],64941:[[1604,1605,1610],256],64942:[[1610,1581,1610],256],64943:[[1610,1580,1610],256],64944:[[1610,1605,1610],256],64945:[[1605,1605,1610],256],64946:[[1602,1605,1610],256],64947:[[1606,1581,1610],256],64948:[[1602,1605,1581],256],64949:[[1604,1581,1605],256],64950:[[1593,1605,1610],256],64951:[[1603,1605,1610],256],64952:[[1606,1580,1581],256],64953:[[1605,1582,1610],256],64954:[[1604,1580,1605],256],64955:[[1603,1605,1605],256],64956:[[1604,1580,1605],256],64957:[[1606,1580,1581],256],64958:[[1580,1581,1610],256],64959:[[1581,1580,1610],256],64960:[[1605,1580,1610],256],64961:[[1601,1605,1610],256],64962:[[1576,1581,1610],256],64963:[[1603,1605,1605],256],64964:[[1593,1580,1605],256],64965:[[1589,1605,1605],256],64966:[[1587,1582,1610],256],64967:[[1606,1580,1610],256],65008:[[1589,1604,1746],256],65009:[[1602,1604,1746],256],65010:[[1575,1604,1604,1607],256],65011:[[1575,1603,1576,1585],256],65012:[[1605,1581,1605,1583],256],65013:[[1589,1604,1593,1605],256],65014:[[1585,1587,1608,1604],256],65015:[[1593,1604,1610,1607],256],65016:[[1608,1587,1604,1605],256],65017:[[1589,1604,1609],256],65018:[[1589,1604,1609,32,1575,1604,1604,1607,32,1593,1604,1610,1607,32,1608,1587,1604,1605],256],65019:[[1580,1604,32,1580,1604,1575,1604,1607],256],65020:[[1585,1740,1575,1604],256]},65024:{65040:[[44],256],65041:[[12289],256],65042:[[12290],256],65043:[[58],256],65044:[[59],256],65045:[[33],256],65046:[[63],256],65047:[[12310],256],65048:[[12311],256],65049:[[8230],256],65056:[,230],65057:[,230],65058:[,230],65059:[,230],65060:[,230],65061:[,230],65062:[,230],65063:[,220],65064:[,220],65065:[,220],65066:[,220],65067:[,220],65068:[,220],65069:[,220],65072:[[8229],256],65073:[[8212],256],65074:[[8211],256],65075:[[95],256],65076:[[95],256],65077:[[40],256],65078:[[41],256],65079:[[123],256],65080:[[125],256],65081:[[12308],256],65082:[[12309],256],65083:[[12304],256],65084:[[12305],256],65085:[[12298],256],65086:[[12299],256],65087:[[12296],256],65088:[[12297],256],65089:[[12300],256],65090:[[12301],256],65091:[[12302],256],65092:[[12303],256],65095:[[91],256],65096:[[93],256],65097:[[8254],256],65098:[[8254],256],65099:[[8254],256],65100:[[8254],256],65101:[[95],256],65102:[[95],256],65103:[[95],256],65104:[[44],256],65105:[[12289],256],65106:[[46],256],65108:[[59],256],65109:[[58],256],65110:[[63],256],65111:[[33],256],65112:[[8212],256],65113:[[40],256],65114:[[41],256],65115:[[123],256],65116:[[125],256],65117:[[12308],256],65118:[[12309],256],65119:[[35],256],65120:[[38],256],65121:[[42],256],65122:[[43],256],65123:[[45],256],65124:[[60],256],65125:[[62],256],65126:[[61],256],65128:[[92],256],65129:[[36],256],65130:[[37],256],65131:[[64],256],65136:[[32,1611],256],65137:[[1600,1611],256],65138:[[32,1612],256],65140:[[32,1613],256],65142:[[32,1614],256],65143:[[1600,1614],256],65144:[[32,1615],256],65145:[[1600,1615],256],65146:[[32,1616],256],65147:[[1600,1616],256],65148:[[32,1617],256],65149:[[1600,1617],256],65150:[[32,1618],256],65151:[[1600,1618],256],65152:[[1569],256],65153:[[1570],256],65154:[[1570],256],65155:[[1571],256],65156:[[1571],256],65157:[[1572],256],65158:[[1572],256],65159:[[1573],256],65160:[[1573],256],65161:[[1574],256],65162:[[1574],256],65163:[[1574],256],65164:[[1574],256],65165:[[1575],256],65166:[[1575],256],65167:[[1576],256],65168:[[1576],256],65169:[[1576],256],65170:[[1576],256],65171:[[1577],256],65172:[[1577],256],65173:[[1578],256],65174:[[1578],256],65175:[[1578],256],65176:[[1578],256],65177:[[1579],256],65178:[[1579],256],65179:[[1579],256],65180:[[1579],256],65181:[[1580],256],65182:[[1580],256],65183:[[1580],256],65184:[[1580],256],65185:[[1581],256],65186:[[1581],256],65187:[[1581],256],65188:[[1581],256],65189:[[1582],256],65190:[[1582],256],65191:[[1582],256],65192:[[1582],256],65193:[[1583],256],65194:[[1583],256],65195:[[1584],256],65196:[[1584],256],65197:[[1585],256],65198:[[1585],256],65199:[[1586],256],65200:[[1586],256],65201:[[1587],256],65202:[[1587],256],65203:[[1587],256],65204:[[1587],256],65205:[[1588],256],65206:[[1588],256],65207:[[1588],256],65208:[[1588],256],65209:[[1589],256],65210:[[1589],256],65211:[[1589],256],65212:[[1589],256],65213:[[1590],256],65214:[[1590],256],65215:[[1590],256],65216:[[1590],256],65217:[[1591],256],65218:[[1591],256],65219:[[1591],256],65220:[[1591],256],65221:[[1592],256],65222:[[1592],256],65223:[[1592],256],65224:[[1592],256],65225:[[1593],256],65226:[[1593],256],65227:[[1593],256],65228:[[1593],256],65229:[[1594],256],65230:[[1594],256],65231:[[1594],256],65232:[[1594],256],65233:[[1601],256],65234:[[1601],256],65235:[[1601],256],65236:[[1601],256],65237:[[1602],256],65238:[[1602],256],65239:[[1602],256],65240:[[1602],256],65241:[[1603],256],65242:[[1603],256],65243:[[1603],256],65244:[[1603],256],65245:[[1604],256],65246:[[1604],256],65247:[[1604],256],65248:[[1604],256],65249:[[1605],256],65250:[[1605],256],65251:[[1605],256],65252:[[1605],256],65253:[[1606],256],65254:[[1606],256],65255:[[1606],256],65256:[[1606],256],65257:[[1607],256],65258:[[1607],256],65259:[[1607],256],65260:[[1607],256],65261:[[1608],256],65262:[[1608],256],65263:[[1609],256],65264:[[1609],256],65265:[[1610],256],65266:[[1610],256],65267:[[1610],256],65268:[[1610],256],65269:[[1604,1570],256],65270:[[1604,1570],256],65271:[[1604,1571],256],65272:[[1604,1571],256],65273:[[1604,1573],256],65274:[[1604,1573],256],65275:[[1604,1575],256],65276:[[1604,1575],256]},65280:{65281:[[33],256],65282:[[34],256],65283:[[35],256],65284:[[36],256],65285:[[37],256],65286:[[38],256],65287:[[39],256],65288:[[40],256],65289:[[41],256],65290:[[42],256],65291:[[43],256],65292:[[44],256],65293:[[45],256],65294:[[46],256],65295:[[47],256],65296:[[48],256],65297:[[49],256],65298:[[50],256],65299:[[51],256],65300:[[52],256],65301:[[53],256],65302:[[54],256],65303:[[55],256],65304:[[56],256],65305:[[57],256],65306:[[58],256],65307:[[59],256],65308:[[60],256],65309:[[61],256],65310:[[62],256],65311:[[63],256],65312:[[64],256],65313:[[65],256],65314:[[66],256],65315:[[67],256],65316:[[68],256],65317:[[69],256],65318:[[70],256],65319:[[71],256],65320:[[72],256],65321:[[73],256],65322:[[74],256],65323:[[75],256],65324:[[76],256],65325:[[77],256],65326:[[78],256],65327:[[79],256],65328:[[80],256],65329:[[81],256],65330:[[82],256],65331:[[83],256],65332:[[84],256],65333:[[85],256],65334:[[86],256],65335:[[87],256],65336:[[88],256],65337:[[89],256],65338:[[90],256],65339:[[91],256],65340:[[92],256],65341:[[93],256],65342:[[94],256],65343:[[95],256],65344:[[96],256],65345:[[97],256],65346:[[98],256],65347:[[99],256],65348:[[100],256],65349:[[101],256],65350:[[102],256],65351:[[103],256],65352:[[104],256],65353:[[105],256],65354:[[106],256],65355:[[107],256],65356:[[108],256],65357:[[109],256],65358:[[110],256],65359:[[111],256],65360:[[112],256],65361:[[113],256],65362:[[114],256],65363:[[115],256],65364:[[116],256],65365:[[117],256],65366:[[118],256],65367:[[119],256],65368:[[120],256],65369:[[121],256],65370:[[122],256],65371:[[123],256],65372:[[124],256],65373:[[125],256],65374:[[126],256],65375:[[10629],256],65376:[[10630],256],65377:[[12290],256],65378:[[12300],256],65379:[[12301],256],65380:[[12289],256],65381:[[12539],256],65382:[[12530],256],65383:[[12449],256],65384:[[12451],256],65385:[[12453],256],65386:[[12455],256],65387:[[12457],256],65388:[[12515],256],65389:[[12517],256],65390:[[12519],256],65391:[[12483],256],65392:[[12540],256],65393:[[12450],256],65394:[[12452],256],65395:[[12454],256],65396:[[12456],256],65397:[[12458],256],65398:[[12459],256],65399:[[12461],256],65400:[[12463],256],65401:[[12465],256],65402:[[12467],256],65403:[[12469],256],65404:[[12471],256],65405:[[12473],256],65406:[[12475],256],65407:[[12477],256],65408:[[12479],256],65409:[[12481],256],65410:[[12484],256],65411:[[12486],256],65412:[[12488],256],65413:[[12490],256],65414:[[12491],256],65415:[[12492],256],65416:[[12493],256],65417:[[12494],256],65418:[[12495],256],65419:[[12498],256],65420:[[12501],256],65421:[[12504],256],65422:[[12507],256],65423:[[12510],256],65424:[[12511],256],65425:[[12512],256],65426:[[12513],256],65427:[[12514],256],65428:[[12516],256],65429:[[12518],256],65430:[[12520],256],65431:[[12521],256],65432:[[12522],256],65433:[[12523],256],65434:[[12524],256],65435:[[12525],256],65436:[[12527],256],65437:[[12531],256],65438:[[12441],256],65439:[[12442],256],65440:[[12644],256],65441:[[12593],256],65442:[[12594],256],65443:[[12595],256],65444:[[12596],256],65445:[[12597],256],65446:[[12598],256],65447:[[12599],256],65448:[[12600],256],65449:[[12601],256],65450:[[12602],256],65451:[[12603],256],65452:[[12604],256],65453:[[12605],256],65454:[[12606],256],65455:[[12607],256],65456:[[12608],256],65457:[[12609],256],65458:[[12610],256],65459:[[12611],256],65460:[[12612],256],65461:[[12613],256],65462:[[12614],256],65463:[[12615],256],65464:[[12616],256],65465:[[12617],256],65466:[[12618],256],65467:[[12619],256],65468:[[12620],256],65469:[[12621],256],65470:[[12622],256],65474:[[12623],256],65475:[[12624],256],65476:[[12625],256],65477:[[12626],256],65478:[[12627],256],65479:[[12628],256],65482:[[12629],256],65483:[[12630],256],65484:[[12631],256],65485:[[12632],256],65486:[[12633],256],65487:[[12634],256],65490:[[12635],256],65491:[[12636],256],65492:[[12637],256],65493:[[12638],256],65494:[[12639],256],65495:[[12640],256],65498:[[12641],256],65499:[[12642],256],65500:[[12643],256],65504:[[162],256],65505:[[163],256],65506:[[172],256],65507:[[175],256],65508:[[166],256],65509:[[165],256],65510:[[8361],256],65512:[[9474],256],65513:[[8592],256],65514:[[8593],256],65515:[[8594],256],65516:[[8595],256],65517:[[9632],256],65518:[[9675],256]}};var p={nfc:function(e){return d("NFC",e)},nfd:function(e){return d("NFD",e)},nfkc:function(e){return d("NFKC",e)},nfkd:function(e){return d("NFKD",e)}};e.exports=p,p.shimApplied=!1,String.prototype.normalize||(Object.defineProperty(String.prototype,"normalize",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=""+this,t=void 0===arguments[0]?"NFC":arguments[0];if(null==this)throw new TypeError("Cannot call method on "+Object.prototype.toString.call(this));if("NFC"===t)return p.nfc(e);if("NFD"===t)return p.nfd(e);if("NFKC"===t)return p.nfkc(e);if("NFKD"===t)return p.nfkd(e);throw new RangeError("Invalid normalization form: "+t)}}),p.shimApplied=!0)}()},function(e,t,r){"use strict";t.__esModule=!0,t.collatorFromConfig=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e){const t=e.caseSensitive?e.accentSensitive?"variant":"case":e.accentSensitive?"accent":"base",r=e.caseFirst,n=e.ignorePunctuation;return new Intl.Collator(e.localeLang,{sensitivity:t,caseFirst:r,ignorePunctuation:n})}},function(e,t,r){"use strict";t.__esModule=!0,t.DetailedCellError=void 0;t.DetailedCellError=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r){var n;this.value=t,this.address=r,this.type=e.type,this.message=null!==(n=e.message)&&void 0!==n?n:""}toString(){return this.value}valueOf(){return this.value}}},function(e,t,r){"use strict";t.__esModule=!0,t.Config=void 0,t.getDefaultConfig=function(){return p({})};var n=r(171),o=r(172),s=r(173),i=r(174),a=r(148),u=r(175),c=r(177),l=r(179);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const h=new WeakMap;class d{constructor(){let e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];const{accentSensitive:r,binarySearchThreshold:o,caseSensitive:i,caseFirst:a,chooseAddressMappingPolicy:u,currencySymbol:p,dateFormats:m,decimalSeparator:g,evaluateNullToZero:f,functionArgSeparator:E,functionPlugins:T,ignorePunctuation:y,leapYear1900:A,localeLang:R,language:N,ignoreWhiteSpace:S,licenseKey:C,matchWholeCell:v,arrayColumnSeparator:O,arrayRowSeparator:M,maxRows:I,maxColumns:w,nullYear:F,nullDate:b,parseDateTime:_,precisionEpsilon:L,precisionRounding:x,stringifyDateTime:P,stringifyDuration:U,smartRounding:V,timeFormats:D,thousandSeparator:B,useArrayArithmetic:k,useStats:G,undoLimit:j,useColumnIndex:W,useRegularExpressions:H,useWildcards:z}=e;t&&d.warnDeprecatedOptions(e),this.useArrayArithmetic=(0,n.configValueFromParam)(k,"boolean","useArrayArithmetic"),this.accentSensitive=(0,n.configValueFromParam)(r,"boolean","accentSensitive"),this.caseSensitive=(0,n.configValueFromParam)(i,"boolean","caseSensitive"),this.caseFirst=(0,n.configValueFromParam)(a,["upper","lower","false"],"caseFirst"),this.ignorePunctuation=(0,n.configValueFromParam)(y,"boolean","ignorePunctuation"),this.chooseAddressMappingPolicy=null!=u?u:d.defaultConfig.chooseAddressMappingPolicy,this.dateFormats=[...(0,n.configValueFromParamCheck)(m,Array.isArray,"array","dateFormats")],this.timeFormats=[...(0,n.configValueFromParamCheck)(D,Array.isArray,"array","timeFormats")],this.functionArgSeparator=(0,n.configValueFromParam)(E,"string","functionArgSeparator"),this.decimalSeparator=(0,n.configValueFromParam)(g,[".",","],"decimalSeparator"),this.language=(0,n.configValueFromParam)(N,"string","language"),this.ignoreWhiteSpace=(0,n.configValueFromParam)(S,["standard","any"],"ignoreWhiteSpace"),this.licenseKey=(0,n.configValueFromParam)(C,"string","licenseKey"),this.thousandSeparator=(0,n.configValueFromParam)(B,["",","," ","."],"thousandSeparator"),this.arrayColumnSeparator=(0,n.configValueFromParam)(O,[",",";"],"arrayColumnSeparator"),this.arrayRowSeparator=(0,n.configValueFromParam)(M,[";","|"],"arrayRowSeparator"),this.localeLang=(0,n.configValueFromParam)(R,"string","localeLang"),this.functionPlugins=[...null!=T?T:d.defaultConfig.functionPlugins],this.smartRounding=(0,n.configValueFromParam)(V,"boolean","smartRounding"),this.evaluateNullToZero=(0,n.configValueFromParam)(f,"boolean","evaluateNullToZero"),this.nullYear=(0,n.configValueFromParam)(F,"number","nullYear"),(0,n.validateNumberToBeAtLeast)(this.nullYear,"nullYear",0),(0,n.validateNumberToBeAtMost)(this.nullYear,"nullYear",100),this.precisionRounding=(0,n.configValueFromParam)(x,"number","precisionRounding"),(0,n.validateNumberToBeAtLeast)(this.precisionRounding,"precisionRounding",0),this.precisionEpsilon=(0,n.configValueFromParam)(L,"number","precisionEpsilon"),(0,n.validateNumberToBeAtLeast)(this.precisionEpsilon,"precisionEpsilon",0),this.useColumnIndex=(0,n.configValueFromParam)(W,"boolean","useColumnIndex"),this.useStats=(0,n.configValueFromParam)(G,"boolean","useStats"),this.binarySearchThreshold=null!=o?o:d.defaultConfig.binarySearchThreshold,this.parseDateTime=(0,n.configValueFromParam)(_,"function","parseDateTime"),this.stringifyDateTime=(0,n.configValueFromParam)(P,"function","stringifyDateTime"),this.stringifyDuration=(0,n.configValueFromParam)(U,"function","stringifyDuration"),this.translationPackage=l.HyperFormula.getLanguage(this.language),this.errorMapping=this.translationPackage.buildErrorMapping(),this.nullDate=(0,n.configValueFromParamCheck)(b,s.instanceOfSimpleDate,"IDate","nullDate"),this.leapYear1900=(0,n.configValueFromParam)(A,"boolean","leapYear1900"),this.undoLimit=(0,n.configValueFromParam)(j,"number","undoLimit"),this.useRegularExpressions=(0,n.configValueFromParam)(H,"boolean","useRegularExpressions"),this.useWildcards=(0,n.configValueFromParam)(z,"boolean","useWildcards"),this.matchWholeCell=(0,n.configValueFromParam)(v,"boolean","matchWholeCell"),(0,n.validateNumberToBeAtLeast)(this.undoLimit,"undoLimit",0),this.maxRows=(0,n.configValueFromParam)(I,"number","maxRows"),(0,n.validateNumberToBeAtLeast)(this.maxRows,"maxRows",1),this.maxColumns=(0,n.configValueFromParam)(w,"number","maxColumns"),this.currencySymbol=this.setupCurrencySymbol(p),(0,n.validateNumberToBeAtLeast)(this.maxColumns,"maxColumns",1),h.set(this,{licenseKeyValidityState:(0,c.checkLicenseKeyValidity)(this.licenseKey)}),(0,n.configCheckIfParametersNotInConflict)({value:this.decimalSeparator,name:"decimalSeparator"},{value:this.functionArgSeparator,name:"functionArgSeparator"},{value:this.thousandSeparator,name:"thousandSeparator"}),(0,n.configCheckIfParametersNotInConflict)({value:this.arrayRowSeparator,name:"arrayRowSeparator"},{value:this.arrayColumnSeparator,name:"arrayColumnSeparator"})}setupCurrencySymbol(e){const t=[...(0,n.configValueFromParamCheck)(e,Array.isArray,"array","currencySymbol")];return t.forEach(e=>{if("string"!=typeof e)throw new a.ExpectedValueOfTypeError("string[]","currencySymbol");if(""===e)throw new a.ConfigValueEmpty("currencySymbol")}),t}get licenseKeyValidityState(){return h.get(this).licenseKeyValidityState}getConfig(){return p(this)}mergeConfig(e){const t=Object.assign({},this.getConfig(),e);return d.warnDeprecatedOptions(e),new d(t,!1)}static warnDeprecatedOptions(e){d.warnDeprecatedIfUsed(e.binarySearchThreshold,"binarySearchThreshold","1.1")}static warnDeprecatedIfUsed(e,t,r,n){void 0!==e&&(void 0===n?console.warn(`${t} option is deprecated since ${r}`):console.warn(`${t} option is deprecated since ${r}, please use ${n}`))}}function p(e){var t;const r={};for(const n in d.defaultConfig){const o=null!==(t=e[n])&&void 0!==t?t:d.defaultConfig[n];Array.isArray(o)?r[n]=[...o]:r[n]=o}return r}t.Config=d,d.defaultConfig={accentSensitive:!1,binarySearchThreshold:20,currencySymbol:["$"],caseSensitive:!1,caseFirst:"lower",chooseAddressMappingPolicy:new i.AlwaysDense,dateFormats:["DD/MM/YYYY","DD/MM/YY"],decimalSeparator:".",evaluateNullToZero:!1,functionArgSeparator:",",functionPlugins:[],ignorePunctuation:!1,language:"enGB",ignoreWhiteSpace:"standard",licenseKey:"",leapYear1900:!1,localeLang:"en",matchWholeCell:!0,arrayColumnSeparator:",",arrayRowSeparator:";",maxRows:4e4,maxColumns:18278,nullYear:30,nullDate:{year:1899,month:12,day:30},parseDateTime:o.defaultParseToDateTime,precisionEpsilon:1e-13,precisionRounding:14,smartRounding:!0,stringifyDateTime:u.defaultStringifyDateTime,stringifyDuration:u.defaultStringifyDuration,timeFormats:["hh:mm","hh:mm:ss.sss"],thousandSeparator:"",undoLimit:20,useRegularExpressions:!1,useWildcards:!0,useColumnIndex:!1,useStats:!1,useArrayArithmetic:!1}},function(e,t,r){"use strict";t.__esModule=!0,t.configCheckIfParametersNotInConflict=function(){const e=new Map;for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];r.forEach(t=>{const r=e.get(t.value)||[];r.push(t.name),e.set(t.value,r)});const o=[];for(const t of e.values())t.length>1&&o.push(t);if(o.length>0){o.forEach(e=>e.sort());const e=o.map(e=>`[${e}]`).join("; ");throw new Error("Config initialization failed. Parameters in conflict: "+e)}},t.configValueFromParam=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e,t,r){if(void 0===e)return n.Config.defaultConfig[r];if("string"==typeof t){if(typeof e===t)return e;throw new o.ExpectedValueOfTypeError(t,r)}if(t.includes(e))return e;throw new o.ExpectedOneOfValuesError(t.map(e=>`'${e}'`).join(" "),r)},t.configValueFromParamCheck=function(e,t,r,s){if(t(e))return e;if(void 0===e)return n.Config.defaultConfig[s];throw new o.ExpectedValueOfTypeError(r,s)},t.validateArgToType=function(e,t,r){if(typeof e!==t)throw new o.ExpectedValueOfTypeError(t,r)},t.validateNumberToBeAtLeast=function(e,t,r){if(e<r)throw new o.ConfigValueTooSmallError(t,r)},t.validateNumberToBeAtMost=function(e,t,r){if(e>r)throw new o.ConfigValueTooBigError(t,r)},r(2);var n=r(170),o=r(148)},function(e,t,r){"use strict";t.__esModule=!0,t.defaultParseToDateTime=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e,t,r){let n=(e=e.replace(/\s\s+/g," ").trim().toLowerCase()).substring(e.length-2);"am"===n||"pm"===n?e=e.substring(0,e.length-2).trim():(n=e.substring(e.length-1),"a"===n||"p"===n?e=e.substring(0,e.length-1).trim():n=void 0);const i=e.split(/[ /.-]/g);i.length>=2&&i[i.length-2].includes(":")&&(i[i.length-2]=i[i.length-2]+"."+i[i.length-1],i.pop());const a=i[i.length-1].split(":");void 0!==n&&a.push(n);if(1===i.length)return o(a,r);if(1===a.length)return s(i,t);const u=s(i.slice(0,i.length-1),t),c=o(a,r);return void 0===u||void 0===c?void 0:Object.assign(Object.assign({},u),c)},t.secondsExtendedRegexp=void 0,r(2);const n=/^ss(\.(s+|0+))?$/;function o(e,t){if(void 0===t)return;(t=t.toLowerCase()).endsWith("am/pm")?t=t.substring(0,t.length-5).trim():t.endsWith("a/p")&&(t=t.substring(0,t.length-3).trim());const r=t.split(":");let o=void 0;if("am"===e[e.length-1]||"a"===e[e.length-1]?(o=!1,e.pop()):"pm"!==e[e.length-1]&&"p"!==e[e.length-1]||(o=!0,e.pop()),e.length!==r.length)return;const s=r.indexOf("hh"),i=r.indexOf("mm"),a=r.findIndex(e=>n.test(e)),u=-1!==s?e[s]:"0";if(!/^\d+$/.test(u))return;let c=Number(u);if(void 0!==o){if(c<0||c>12)return;c%=12,o&&(c+=12)}const l=-1!==i?e[i]:"0";if(!/^\d+$/.test(l))return;const h=Number(l),d=-1!==a?e[a]:"0";if(!/^\d+(\.\d+)?$/.test(d))return;return{hours:c,minutes:h,seconds:Math.round(1e3*Number(d))/1e3}}function s(e,t){if(void 0===t)return;const r=t.toLowerCase().split(/[ /.-]/g);if(e.length!==r.length)return;const n=r.indexOf("mm"),o=r.indexOf("dd"),s=r.indexOf("yyyy"),i=r.indexOf("yy");if(!(n in e)||!(o in e)||!(s in e)&&!(i in e))return;if(s in e&&i in e)return;let a;if(s in e){const t=e[s];if(!/^\d+$/.test(t))return;if(a=Number(t),a<1e3||a>9999)return}else{const t=e[i];if(!/^\d+$/.test(t))return;if(a=Number(t),a<0||a>99)return}const u=e[n];if(!/^\d+$/.test(u))return;const c=Number(u),l=e[o];if(!/^\d+$/.test(l))return;return{year:a,month:c,day:Number(l)}}t.secondsExtendedRegexp=n},function(e,t,r){"use strict";t.__esModule=!0,t.DateTimeHelper=void 0,t.instanceOfSimpleDate=i,t.instanceOfSimpleTime=a,t.maxDate=void 0,t.numberToSimpleTime=c,t.offsetMonth=function(e,t){const r=12*e.year+e.month-1+t;return{year:Math.floor(r/12),month:r%12+1,day:e.day}},t.roundToNearestSecond=function(e){return Math.round(3600*e*24)/86400},t.timeToNumber=l,t.toBasisEU=function(e){return{year:e.year,month:e.month,day:Math.min(30,e.day)}},t.truncateDayInMonth=function(e){return{year:e.year,month:e.month,day:Math.min(e.day,o[e.month-1])}};var n=r(141);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */const o=[31,28,31,30,31,30,31,31,30,31,30,31],s=[0,31,59,90,120,151,181,212,243,273,304,334];function i(e){return!(!e||"object"!=typeof e&&"function"!=typeof e)&&("year"in e&&"number"==typeof e.year&&"month"in e&&"number"==typeof e.month&&"day"in e&&"number"==typeof e.day)}function a(e){return!(!e||"object"!=typeof e&&"function"!=typeof e)&&("hours"in e&&"number"==typeof e.hours&&"minutes"in e&&"number"==typeof e.minutes&&"seconds"in e&&"number"==typeof e.seconds)}const u={year:9999,month:12,day:31};t.maxDate=u;function c(e){const t=24*e*60*60,r=function(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1;return Math.round(e*t)/t}(t%60,1e5)%60,n=(t-r)/60,o=Math.round(n%60)%60,s=(n-o)/60;return{hours:Math.round(s),minutes:o,seconds:r}}function l(e){return((e.seconds/60+e.minutes)/60+e.hours)/24}t.DateTimeHelper=class{constructor(e){this.config=e,this.minDateAbsoluteValue=this.dateToNumberFromZero(e.nullDate),this.maxDateValue=this.dateToNumber(u),this.leapYear1900=e.leapYear1900,!this.leapYear1900&&0<=this.dateToNumber({year:1900,month:2,day:28})?this.epochYearZero=this.numberToSimpleDate(2).year:this.epochYearZero=this.numberToSimpleDate(1).year,this.parseDateTime=e.parseDateTime}getWithinBounds(e){return e<=this.maxDateValue&&e>=0?e:void 0}dateStringToDateNumber(e){const{dateTime:t,dateFormat:r="",timeFormat:o=""}=this.parseDateTimeFromConfigFormats(e);if(void 0!==t)return a(t)?i(t)?new n.DateTimeNumber(l(t)+this.dateToNumber(t),r+" "+o):new n.TimeNumber(l(t),o):i(t)?new n.DateNumber(this.dateToNumber(t),r):0}parseDateTimeFromConfigFormats(e){return this.parseDateTimeFromFormats(e,this.config.dateFormats,this.config.timeFormats)}getNullYear(){return this.config.nullYear}getEpochYearZero(){return this.epochYearZero}isValidDate(e){return!(isNaN(e.year)||isNaN(e.month)||isNaN(e.day))&&(e.day===Math.round(e.day)&&e.month===Math.round(e.month)&&e.year===Math.round(e.year)&&(!(e.year<1582)&&(!(e.month<1||e.month>12)&&(!(e.day<1)&&(this.isLeapYear(e.year)&&2===e.month?e.day<=29:e.day<=o[e.month-1])))))}dateToNumber(e){return this.dateToNumberFromZero(e)-this.minDateAbsoluteValue}relativeNumberToAbsoluteNumber(e){return e+this.minDateAbsoluteValue-(this.leapYear1900?1:0)}numberToSimpleDate(e){const t=Math.floor(e)+this.minDateAbsoluteValue;let r=Math.floor(t/365.2425);this.dateToNumberFromZero({year:r+1,month:1,day:1})<=t?r++:this.dateToNumberFromZero({year:r-1,month:1,day:1})>t&&r--;const n=t-this.dateToNumberFromZero({year:r,month:1,day:1}),o=function(e){let t=0;s[t+6]<=e&&(t+=6);s[t+3]<=e&&(t+=3);s[t+2]<=e?t+=2:s[t+1]<=e&&(t+=1);return t}(n-(this.isLeapYear(r)&&n>=59?1:0));return{year:r,month:o+1,day:n-s[o]-(this.isLeapYear(r)&&o>1?1:0)+1}}numberToSimpleDateTime(e){const t=c(e%1),r=Math.floor(t.hours/24);t.hours=t.hours%24;const n=this.numberToSimpleDate(Math.floor(e)+r);return Object.assign(Object.assign({},n),t)}leapYearsCount(e){return Math.floor(e/4)-Math.floor(e/100)+Math.floor(e/400)+(this.config.leapYear1900&&e>=1900?1:0)}daysInMonth(e,t){return this.isLeapYear(e)&&2===t?29:o[t-1]}endOfMonth(e){return{year:e.year,month:e.month,day:this.daysInMonth(e.year,e.month)}}toBasisUS(e,t){return 31===e.day&&(e.day=30),30===e.day&&31===t.day&&(t.day=30),2===e.month&&e.day===this.daysInMonth(e.year,e.month)&&(e.day=30,2===t.month&&t.day===this.daysInMonth(t.year,t.month)&&(t.day=30)),[e,t]}yearLengthForBasis(e,t){return e.year!==t.year?e.year+1!==t.year||e.month<t.month||e.month===t.month&&e.day<t.day?(this.leapYearsCount(t.year)-this.leapYearsCount(e.year-1))/(t.year-e.year+1)+365:this.countLeapDays(t)!==this.countLeapDays({year:e.year,month:e.month,day:e.day-1})?366:365:this.isLeapYear(e.year)?366:365}parseSingleFormat(e,t,r){const n=this.parseDateTime(e,t,r);if(!i(n)||(n.year>=0&&n.year<100&&(n.year<this.getNullYear()?n.year+=2e3:n.year+=1900),this.isValidDate(n)))return n}parseDateTimeFromFormats(e,t,r){const n=0===t.length?[void 0]:t,o=0===r.length?[void 0]:r;for(const t of n)for(const r of o){const n=this.parseSingleFormat(e,t,r);if(void 0!==n)return{dateTime:n,timeFormat:r,dateFormat:t}}return{}}countLeapDays(e){return e.month>2||2===e.month&&e.day>=29?this.leapYearsCount(e.year):this.leapYearsCount(e.year-1)}dateToNumberFromZero(e){return 365*e.year+s[e.month-1]+e.day-1+(e.month<=2?this.leapYearsCount(e.year-1):this.leapYearsCount(e.year))}isLeapYear(e){return!(e%4)&&(!!(e%100)||(!(e%400)||1900===e&&this.config.leapYear1900))}}},function(e,t,r){"use strict";t.__esModule=!0,t.DenseSparseChooseBasedOnThreshold=t.AlwaysSparse=t.AlwaysDense=void 0;var n=r(163),o=r(162);t.DenseSparseChooseBasedOnThreshold=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.threshold=e}call(e){return e>this.threshold?n.DenseStrategy:o.SparseStrategy}};t.AlwaysSparse=class{call(){return o.SparseStrategy}};t.AlwaysDense=class{call(){return n.DenseStrategy}}},function(e,t,r){"use strict";t.__esModule=!0,t.defaultStringifyDateTime=function(e,t){const r=(0,s.parseForDateTimeFormat)(t);if(void 0===r)return;const o=r.tokens;let a="",u=!1;const c=o.some(e=>e.type===s.TokenType.FORMAT&&("a/p"===e.value||"A/P"===e.value||"am/pm"===e.value||"AM/PM"===e.value));for(let t=0;t<o.length;t++){const r=o[t];if(r.type!==s.TokenType.FREE_TEXT)switch(r.value.toLowerCase()){case"h":case"hh":u=!0,a+=i(c?(e.hours+11)%12+1:e.hours,r.value.length);break;case"d":case"dd":a+=i(e.day,r.value.length);break;case"s":case"ss":a+=i(Math.floor(e.seconds),r.value.length);break;case"m":case"mm":t+1<o.length&&o[t+1].value.startsWith(":")&&(u=!0),a+=i(u?e.minutes:e.month,r.value.length),u=!0;break;case"yy":a+=i(e.year%100,r.value.length);break;case"yyyy":a+=e.year;break;case"am/pm":case"a/p":{const[t,n]=r.value.split("/");a+=e.hours<12?t:n;break}default:if(n.secondsExtendedRegexp.test(r.value)){const t=r.value.length-3;a+=`${e.seconds<10?"0":""}${Math.floor(e.seconds*Math.pow(10,t))/Math.pow(10,t)}`;continue}return}else a+=r.value}return a},t.defaultStringifyDuration=function(e,t){const r=(0,s.parseForDateTimeFormat)(t);if(void 0===r)return;const o=r.tokens;let a="";for(const t of o)if(t.type!==s.TokenType.FREE_TEXT)switch(t.value.toLowerCase()){case"h":case"hh":a+=i(e.hours,t.value.length),e.hours=0;break;case"[hh]":a+=i(e.hours,t.value.length-2),e.hours=0;break;case"m":case"mm":a+=i(e.minutes,t.value.length),e.minutes=0;break;case"[mm]":a+=i(e.minutes+60*e.hours,t.value.length-2),e.minutes=0,e.hours=0;break;case"s":case"ss":a+=i(Math.floor(e.seconds),t.value.length);break;default:if(n.secondsExtendedRegexp.test(t.value)){const r=Math.max(t.value.length-3,0);a+=`${e.seconds<10?"0":""}${Math.floor(e.seconds*Math.pow(10,r))/Math.pow(10,r)}`;continue}return}else a+=t.value;return a},t.format=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e,t,r,n){const c=r.stringifyDateTime(n.numberToSimpleDateTime(e),t);if(void 0!==c)return c;const l=r.stringifyDuration((0,o.numberToSimpleTime)(e),t);if(void 0!==l)return l;const h=(0,s.parseForNumberFormat)(t);if(void 0!==h)return function(e,t){let r="";for(let n=0;n<e.length;++n){const o=e[n];if(o.type===s.TokenType.FREE_TEXT){r+=o.value;continue}const c=o.value.split("."),l=c[0],h=c[1]||"",d=c[1]?".":"",p=Number(t.toFixed(h.length)).toString().split(".");let m=p[0]||"",g=p[1]||"";if(l.length>m.length){const e=u(l.substr(0,l.length-m.length),"0");m=i(m,e+m.length)}const f=u(h.substr(g.length,h.length-g.length),"0");g=a(g,f+g.length),r+=m+d+g}return r}(h.tokens,e);return t},t.padLeft=i,t.padRight=a;var n=r(172),o=r(173),s=r(176);function i(e,t){let r=""+e;for(;r.length<t;)r="0"+r;return r}function a(e,t){let r=""+e;for(;r.length<t;)r+="0";return r}function u(e,t){return e.split(t).length-1}},function(e,t,r){"use strict";t.__esModule=!0,t.TokenType=t.FormatExpressionType=void 0,t.formatToken=a,t.isEscapeToken=h,t.parse=function(e){var t,r;return null!==(r=null!==(t=c(e))&&void 0!==t?t:l(e))&&void 0!==r?r:{type:i.STRING,tokens:[{type:s.FREE_TEXT,value:e}]}},t.parseForDateTimeFormat=c,t.parseForNumberFormat=l,r(2);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const n=/(\\.|dd|DD|d|D|mm|MM|m|M|YYYY|YY|yyyy|yy|HH|hh|H|h|ss(\.(0+|s+))?|s|AM\/PM|am\/pm|A\/P|a\/p|\[mm]|\[MM]|\[hh]|\[HH])/g,o=/(\\.|[#0]+(\.[#0]*)?)/g;var s,i;function a(e,t){return{type:e,value:t}}function u(e,t){const r=[];let n=0;for(let o=0;o<e.length;++o){const i=e[o];if(i.index!==n){const e=t.substr(n,i.index-n);r.push(a(s.FREE_TEXT,e))}i[0].startsWith("\\")?r.push(a(s.FREE_TEXT,i[0])):r.push(a(s.FORMAT,i[0])),n=i.index+i[0].length}const o=e[e.length-1];if(o.index+o[0].length<t.length){const e=t.substr(o.index+o[0].length,t.length);r.push(a(s.FREE_TEXT,e))}return r}function c(e){const t=function(e){n.lastIndex=0;const t=[];let r;do{r=n.exec(e),null!==r&&t.push(r)}while(r);return t}(e);return t.every(e=>h(e))?void 0:{type:i.DATE,tokens:u(t,e)}}function l(e){const t=function(e){o.lastIndex=0;const t=o.exec(e);return null!==t?[t]:[]}(e);return t.every(e=>h(e))?void 0:{type:i.NUMBER,tokens:u(t,e)}}function h(e){return e[0].startsWith("\\")}t.TokenType=s,function(e){e.FORMAT="FORMAT",e.FREE_TEXT="FREE_TEXT"}(s||(t.TokenType=s={})),t.FormatExpressionType=i,function(e){e.DATE="DATE",e.NUMBER="NUMBER",e.STRING="STRING"}(i||(t.FormatExpressionType=i={}))},function(e,t,r){"use strict";t.__esModule=!0,t.checkLicenseKeyValidity=function(e){const t={template:"missing",vars:{}};if("gpl-v3"===e||"internal-use-in-handsontable"===e)t.template="valid";else if("string"==typeof e&&(0,n.checkKeySchema)(e)){const[r,o,s]="29/05/2023".split("/"),i=Math.floor(new Date(`${o}/${r}/${s}`).getTime()/864e5),a=(0,n.extractTime)(e);t.vars.keyValidityDate=function(e){const t=["January","February","March","April","May","June","July","August","September","October","November","December"][e.getMonth()],r=e.getDate(),n=e.getFullYear();return`${t} ${r}, ${n}`}(new Date(864e5*(a+1))),t.template=i>a?"expired":"valid"}else""!==e&&(t.template="invalid");s||"valid"===t.template||(console.warn(o[t.template](t.vars)),s=!0);return t.template};var n=r(178);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */const o={invalid:()=>"The license key for HyperFormula is invalid.",expired:e=>{let{keyValidityDate:t}=e;return`The license key for HyperFormula expired on ${t}, and is not valid for the installed version.`},missing:()=>"The license key for HyperFormula is missing."};let s=!1},function(e,t,r){"use strict";t.__esModule=!0,t.checkKeySchema=function(e){if((e=(e+"").replace(/\-/g,""))[n]!==u("Z"))return!1;let t=0;return[[0,u("G")+1],[u("H"),u("H")-1],[u("G")+u("H"),u("G")]].reduce((r,n,i)=>{let[u,c]=n;return r|=(s(""+s(o(a(e,t+u-(0===i?0:2),c+(i?2:0)))+(o(a(e,t+u+c,2))+[]).padStart(2,"0")))%97||2)>>1,t+=2,r},u("A"))===[]+1>>1},t.extractTime=function(e){return 25===i(e)[n]?o(a(i(e),o("12"),u("F")))/(o(a(i(e),u("C"),u("Y")>>4))||1+(~~![][n]<<3)):0};
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const n="length",o=e=>parseInt(e,16),s=e=>parseInt(e,10),i=e=>(e+"").replace(/\-/g,""),a=(e,t,r)=>e.substr(t,r),u=e=>e.codePointAt(0)-65},function(e,t,r){"use strict";t.__esModule=!0,t.HyperFormula=void 0;var n=r(73),o=r(171),s=r(180),i=r(74),a=r(181),u=r(170),c=r(173),l=r(212),h=r(213),d=r(148),p=r(157),m=r(199),g=r(185),f=r(78);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class E{constructor(e,t,r,n,o,s,i,a,u,c,l,d,p,m){this._config=e,this._stats=t,this._dependencyGraph=r,this._columnSearch=n,this._parser=o,this._unparser=s,this._cellContentParser=i,this._evaluator=a,this._lazilyTransformingAstService=u,this._crudOperations=c,this._exporter=l,this._namedExpressions=d,this._serialization=p,this._functionRegistry=m,this._emitter=new h.Emitter,this._evaluationSuspended=!1}static get defaultConfig(){return(0,u.getDefaultConfig)()}get graph(){return this.dependencyGraph.graph}get rangeMapping(){return this.dependencyGraph.rangeMapping}get arrayMapping(){return this.dependencyGraph.arrayMapping}get sheetMapping(){return this.dependencyGraph.sheetMapping}get addressMapping(){return this.dependencyGraph.addressMapping}get dependencyGraph(){return this._dependencyGraph}get evaluator(){return this._evaluator}get columnSearch(){return this._columnSearch}get lazilyTransformingAstService(){return this._lazilyTransformingAstService}get licenseKeyValidityState(){return this._config.licenseKeyValidityState}static buildFromArray(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[];return this.buildFromEngineState(s.BuildEngineFactory.buildFromSheet(e,t,r))}static buildFromSheets(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[];return this.buildFromEngineState(s.BuildEngineFactory.buildFromSheets(e,t,r))}static buildEmpty(){let e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[];return this.buildFromEngineState(s.BuildEngineFactory.buildEmpty(e,t))}static getLanguage(e){(0,o.validateArgToType)(e,"string","languageCode");const t=this.registeredLanguages.get(e);if(void 0===t)throw new d.LanguageNotRegisteredError;return t}static registerLanguage(e,t){if((0,o.validateArgToType)(e,"string","languageCode"),this.registeredLanguages.has(e))throw new d.LanguageAlreadyRegisteredError;this.registeredLanguages.set(e,(0,p.buildTranslationPackage)(t))}static unregisterLanguage(e){if((0,o.validateArgToType)(e,"string","languageCode"),!this.registeredLanguages.has(e))throw new d.LanguageNotRegisteredError;this.registeredLanguages.delete(e)}static getRegisteredLanguagesCodes(){return Array.from(this.registeredLanguages.keys())}static registerFunctionPlugin(e,t){m.FunctionRegistry.registerFunctionPlugin(e,t)}static unregisterFunctionPlugin(e){m.FunctionRegistry.unregisterFunctionPlugin(e)}static registerFunction(e,t,r){(0,o.validateArgToType)(e,"string","functionId"),m.FunctionRegistry.registerFunction(e,t,r)}static unregisterFunction(e){(0,o.validateArgToType)(e,"string","functionId"),m.FunctionRegistry.unregisterFunction(e)}static unregisterAllFunctions(){m.FunctionRegistry.unregisterAll()}static getRegisteredFunctionNames(e){(0,o.validateArgToType)(e,"string","code");const t=m.FunctionRegistry.getRegisteredFunctionIds();return this.getLanguage(e).getFunctionTranslations(t)}static getFunctionPlugin(e){return(0,o.validateArgToType)(e,"string","functionId"),m.FunctionRegistry.getFunctionPlugin(e)}static getAllFunctionPlugins(){return m.FunctionRegistry.getPlugins()}static buildFromEngineState(e){return new E(e.config,e.stats,e.dependencyGraph,e.columnSearch,e.parser,e.unparser,e.cellContentParser,e.evaluator,e.lazilyTransformingAstService,e.crudOperations,e.exporter,e.namedExpressions,e.serialization,e.functionRegistry)}getCellValue(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return this.ensureEvaluationIsNotSuspended(),this._serialization.getCellValue(e)}getCellFormula(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return this._serialization.getCellFormula(e)}getCellHyperlink(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return this.ensureEvaluationIsNotSuspended(),this._serialization.getCellHyperlink(e)}getCellSerialized(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return this.ensureEvaluationIsNotSuspended(),this._serialization.getCellSerialized(e)}getSheetValues(e){return(0,o.validateArgToType)(e,"number","sheetId"),this.ensureEvaluationIsNotSuspended(),this._serialization.getSheetValues(e)}getSheetFormulas(e){return(0,o.validateArgToType)(e,"number","sheetId"),this._serialization.getSheetFormulas(e)}getSheetSerialized(e){return(0,o.validateArgToType)(e,"number","sheetId"),this.ensureEvaluationIsNotSuspended(),this._serialization.getSheetSerialized(e)}getAllSheetsDimensions(){return this._serialization.genericAllSheetsGetter(e=>this.getSheetDimensions(e))}getSheetDimensions(e){return(0,o.validateArgToType)(e,"number","sheetId"),{width:this.dependencyGraph.getSheetWidth(e),height:this.dependencyGraph.getSheetHeight(e)}}getAllSheetsValues(){return this.ensureEvaluationIsNotSuspended(),this._serialization.getAllSheetsValues()}getAllSheetsFormulas(){return this._serialization.getAllSheetsFormulas()}getAllSheetsSerialized(){return this.ensureEvaluationIsNotSuspended(),this._serialization.getAllSheetsSerialized()}updateConfig(e){Object.entries(e).every(e=>{let[t,r]=e;return this._config[t]===r})||this.rebuildWithConfig(e)}getConfig(){return this._config.getConfig()}rebuildAndRecalculate(){this.rebuildWithConfig({})}getStats(){return this._stats.snapshot()}undo(){return this._crudOperations.undo(),this.recomputeIfDependencyGraphNeedsIt()}redo(){return this._crudOperations.redo(),this.recomputeIfDependencyGraphNeedsIt()}isThereSomethingToUndo(){return this._crudOperations.isThereSomethingToUndo()}isThereSomethingToRedo(){return this._crudOperations.isThereSomethingToRedo()}isItPossibleToSetCellContents(e){let t;if((0,i.isSimpleCellAddress)(e))t=new n.AbsoluteCellRange(e,e);else{if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress | SimpleCellRange","address");t=new n.AbsoluteCellRange(e.start,e.end)}try{this._crudOperations.ensureRangeInSizeLimits(t);for(const e of t.addresses(this._dependencyGraph))this._crudOperations.ensureItIsPossibleToChangeContent(e)}catch(e){return!1}return!0}setCellContents(e,t){return this._crudOperations.setCellContents(e,t),this.recomputeIfDependencyGraphNeedsIt()}swapRowIndexes(e,t){return(0,o.validateArgToType)(e,"number","sheetId"),this._crudOperations.setRowOrder(e,t),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToSwapRowIndexes(e,t){(0,o.validateArgToType)(e,"number","sheetId");try{return this._crudOperations.validateSwapRowIndexes(e,t),this._crudOperations.testRowOrderForArrays(e,t),!0}catch(e){return!1}}setRowOrder(e,t){(0,o.validateArgToType)(e,"number","sheetId");const r=this._crudOperations.mappingFromOrder(e,t,"row");return this.swapRowIndexes(e,r)}isItPossibleToSetRowOrder(e,t){(0,o.validateArgToType)(e,"number","sheetId");try{const r=this._crudOperations.mappingFromOrder(e,t,"row");return this._crudOperations.validateSwapRowIndexes(e,r),this._crudOperations.testRowOrderForArrays(e,r),!0}catch(e){return!1}}swapColumnIndexes(e,t){return(0,o.validateArgToType)(e,"number","sheetId"),this._crudOperations.setColumnOrder(e,t),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToSwapColumnIndexes(e,t){(0,o.validateArgToType)(e,"number","sheetId");try{return this._crudOperations.validateSwapColumnIndexes(e,t),this._crudOperations.testColumnOrderForArrays(e,t),!0}catch(e){return!1}}setColumnOrder(e,t){(0,o.validateArgToType)(e,"number","sheetId");const r=this._crudOperations.mappingFromOrder(e,t,"column");return this.swapColumnIndexes(e,r)}isItPossibleToSetColumnOrder(e,t){(0,o.validateArgToType)(e,"number","sheetId");try{const r=this._crudOperations.mappingFromOrder(e,t,"column");return this._crudOperations.validateSwapColumnIndexes(e,r),this._crudOperations.testColumnOrderForArrays(e,r),!0}catch(e){return!1}}isItPossibleToAddRows(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const s=(0,g.normalizeAddedIndexes)(r);try{return this._crudOperations.ensureItIsPossibleToAddRows(e,...s),!0}catch(e){return!1}}addRows(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];return this._crudOperations.addRows(e,...r),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToRemoveRows(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const s=(0,g.normalizeRemovedIndexes)(r);try{return this._crudOperations.ensureItIsPossibleToRemoveRows(e,...s),!0}catch(e){return!1}}removeRows(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];return this._crudOperations.removeRows(e,...r),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToAddColumns(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const s=(0,g.normalizeAddedIndexes)(r);try{return this._crudOperations.ensureItIsPossibleToAddColumns(e,...s),!0}catch(e){return!1}}addColumns(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];return this._crudOperations.addColumns(e,...r),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToRemoveColumns(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const s=(0,g.normalizeRemovedIndexes)(r);try{return this._crudOperations.ensureItIsPossibleToRemoveColumns(e,...s),!0}catch(e){return!1}}removeColumns(e){(0,o.validateArgToType)(e,"number","sheetId");for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];return this._crudOperations.removeColumns(e,...r),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToMoveCells(e,t){if(!(0,i.isSimpleCellAddress)(t))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","destinationLeftCorner");if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");try{const r=new n.AbsoluteCellRange(e.start,e.end);return this._crudOperations.operations.ensureItIsPossibleToMoveCells(r.start,r.width(),r.height(),t),!0}catch(e){return!1}}moveCells(e,t){if(!(0,i.isSimpleCellAddress)(t))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","destinationLeftCorner");if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");const r=new n.AbsoluteCellRange(e.start,e.end);return this._crudOperations.moveCells(r.start,r.width(),r.height(),t),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToMoveRows(e,t,r,n){(0,o.validateArgToType)(e,"number","sheetId"),(0,o.validateArgToType)(t,"number","startRow"),(0,o.validateArgToType)(r,"number","numberOfRows"),(0,o.validateArgToType)(n,"number","targetRow");try{return this._crudOperations.ensureItIsPossibleToMoveRows(e,t,r,n),!0}catch(e){return!1}}moveRows(e,t,r,n){return(0,o.validateArgToType)(e,"number","sheetId"),(0,o.validateArgToType)(t,"number","startRow"),(0,o.validateArgToType)(r,"number","numberOfRows"),(0,o.validateArgToType)(n,"number","targetRow"),this._crudOperations.moveRows(e,t,r,n),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToMoveColumns(e,t,r,n){(0,o.validateArgToType)(e,"number","sheetId"),(0,o.validateArgToType)(t,"number","startColumn"),(0,o.validateArgToType)(r,"number","numberOfColumns"),(0,o.validateArgToType)(n,"number","targetColumn");try{return this._crudOperations.ensureItIsPossibleToMoveColumns(e,t,r,n),!0}catch(e){return!1}}moveColumns(e,t,r,n){return(0,o.validateArgToType)(e,"number","sheetId"),(0,o.validateArgToType)(t,"number","startColumn"),(0,o.validateArgToType)(r,"number","numberOfColumns"),(0,o.validateArgToType)(n,"number","targetColumn"),this._crudOperations.moveColumns(e,t,r,n),this.recomputeIfDependencyGraphNeedsIt()}copy(e){if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");const t=new n.AbsoluteCellRange(e.start,e.end);return this._crudOperations.copy(t.start,t.width(),t.height()),this.getRangeValues(e)}cut(e){if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");const t=new n.AbsoluteCellRange(e.start,e.end);return this._crudOperations.cut(t.start,t.width(),t.height()),this.getRangeValues(e)}paste(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","targetLeftCorner");return this.ensureEvaluationIsNotSuspended(),this._crudOperations.paste(e),this.recomputeIfDependencyGraphNeedsIt()}isClipboardEmpty(){return this._crudOperations.isClipboardEmpty()}clearClipboard(){this._crudOperations.clearClipboard()}clearRedoStack(){this._crudOperations.undoRedo.clearRedoStack()}clearUndoStack(){this._crudOperations.undoRedo.clearUndoStack()}getRangeValues(e){if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");return new n.AbsoluteCellRange(e.start,e.end).arrayOfAddressesInRange().map(e=>e.map(e=>this.getCellValue(e)))}getRangeFormulas(e){if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");return new n.AbsoluteCellRange(e.start,e.end).arrayOfAddressesInRange().map(e=>e.map(e=>this.getCellFormula(e)))}getRangeSerialized(e){if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");return new n.AbsoluteCellRange(e.start,e.end).arrayOfAddressesInRange().map(e=>e.map(e=>this.getCellSerialized(e)))}getFillRangeData(e,t){let r=arguments.length>2&&void 0!==arguments[2]&&arguments[2];if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","source");if(!(0,n.isSimpleCellRange)(t))throw new d.ExpectedValueOfTypeError("SimpleCellRange","target");const o=new n.AbsoluteCellRange(e.start,e.end),s=new n.AbsoluteCellRange(t.start,t.end);return this.ensureEvaluationIsNotSuspended(),s.arrayOfAddressesInRange().map(n=>n.map(n=>{const s=((n.row-(r?t:e).start.row)%o.height()+o.height())%o.height()+e.start.row,i=((n.col-(r?t:e).start.col)%o.width()+o.width())%o.width()+e.start.col;return this._serialization.getCellSerialized({row:s,col:i,sheet:o.sheet},n)}))}isItPossibleToAddSheet(e){(0,o.validateArgToType)(e,"string","sheetName");try{return this._crudOperations.ensureItIsPossibleToAddSheet(e),!0}catch(e){return!1}}addSheet(e){void 0!==e&&(0,o.validateArgToType)(e,"string","sheetName");const t=this._crudOperations.addSheet(e);return this._emitter.emit(h.Events.SheetAdded,t),t}isItPossibleToRemoveSheet(e){(0,o.validateArgToType)(e,"number","sheetId");try{return this._crudOperations.ensureScopeIdIsValid(e),!0}catch(e){return!1}}removeSheet(e){(0,o.validateArgToType)(e,"number","sheetId");const t=this.sheetMapping.getDisplayName(e);this._crudOperations.removeSheet(e);const r=this.recomputeIfDependencyGraphNeedsIt();return this._emitter.emit(h.Events.SheetRemoved,t,r),r}isItPossibleToClearSheet(e){(0,o.validateArgToType)(e,"number","sheetId");try{return this._crudOperations.ensureScopeIdIsValid(e),!0}catch(e){return!1}}clearSheet(e){return(0,o.validateArgToType)(e,"number","sheetId"),this._crudOperations.clearSheet(e),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToReplaceSheetContent(e,t){(0,o.validateArgToType)(e,"number","sheetId");try{return this._crudOperations.ensureScopeIdIsValid(e),this._crudOperations.ensureItIsPossibleToChangeSheetContents(e,t),!0}catch(e){return!1}}setSheetContent(e,t){return(0,o.validateArgToType)(e,"number","sheetId"),this._crudOperations.setSheetContent(e,t),this.recomputeIfDependencyGraphNeedsIt()}simpleCellAddressFromString(e,t){return(0,o.validateArgToType)(e,"string","cellAddress"),(0,o.validateArgToType)(t,"number","sheetId"),(0,f.simpleCellAddressFromString)(this.sheetMapping.get,e,t)}simpleCellRangeFromString(e,t){return(0,o.validateArgToType)(e,"string","cellRange"),(0,o.validateArgToType)(t,"number","sheetId"),(0,f.simpleCellRangeFromString)(this.sheetMapping.get,e,t)}simpleCellAddressToString(e,t){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return(0,o.validateArgToType)(t,"number","sheetId"),(0,f.simpleCellAddressToString)(this.sheetMapping.fetchDisplayName,e,t)}simpleCellRangeToString(e,t){if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellRange","cellRange");return(0,o.validateArgToType)(t,"number","sheetId"),(0,f.simpleCellRangeToString)(this.sheetMapping.fetchDisplayName,e,t)}getCellDependents(e){let t;if((0,i.isSimpleCellAddress)(e))t=this._dependencyGraph.addressMapping.getCell(e);else{if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress | SimpleCellRange",e);t=this._dependencyGraph.rangeMapping.getRange(e.start,e.end)}return void 0===t?[]:this._dependencyGraph.getAdjacentNodesAddresses(t)}getCellPrecedents(e){let t;if((0,i.isSimpleCellAddress)(e))t=this._dependencyGraph.addressMapping.getCell(e);else{if(!(0,n.isSimpleCellRange)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress | SimpleCellRange",e);t=this._dependencyGraph.rangeMapping.getRange(e.start,e.end)}return void 0===t?[]:this._dependencyGraph.dependencyQueryAddresses(t)}getSheetName(e){return(0,o.validateArgToType)(e,"number","sheetId"),this.sheetMapping.getDisplayName(e)}getSheetNames(){return this.sheetMapping.sheetNames()}getSheetId(e){return(0,o.validateArgToType)(e,"string","sheetName"),this.sheetMapping.get(e)}doesSheetExist(e){return(0,o.validateArgToType)(e,"string","sheetName"),this.sheetMapping.hasSheetWithName(e)}getCellType(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");const t=this.dependencyGraph.getCell(e);return(0,i.getCellType)(t,e)}doesCellHaveSimpleValue(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return this.getCellType(e)===i.CellType.VALUE}doesCellHaveFormula(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");const t=this.getCellType(e);return t===i.CellType.FORMULA||t===i.CellType.ARRAYFORMULA}isCellEmpty(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");return this.getCellType(e)===i.CellType.EMPTY}isCellPartOfArray(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");const t=this.getCellType(e);return t===i.CellType.ARRAY||t===i.CellType.ARRAYFORMULA}getCellValueType(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");this.ensureEvaluationIsNotSuspended();const t=this.dependencyGraph.getCellValue(e);return(0,i.getCellValueType)(t)}getCellValueDetailedType(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");this.ensureEvaluationIsNotSuspended();const t=this.dependencyGraph.getCellValue(e);return(0,i.getCellValueDetailedType)(t)}getCellValueFormat(e){if(!(0,i.isSimpleCellAddress)(e))throw new d.ExpectedValueOfTypeError("SimpleCellAddress","cellAddress");this.ensureEvaluationIsNotSuspended();const t=this.dependencyGraph.getCellValue(e);return(0,i.getCellValueFormat)(t)}countSheets(){return this.sheetMapping.numberOfSheets()}isItPossibleToRenameSheet(e,t){(0,o.validateArgToType)(e,"number","sheetId"),(0,o.validateArgToType)(t,"string","newName");try{return this._crudOperations.ensureItIsPossibleToRenameSheet(e,t),!0}catch(e){return!1}}renameSheet(e,t){(0,o.validateArgToType)(e,"number","sheetId"),(0,o.validateArgToType)(t,"string","newName");const r=this._crudOperations.renameSheet(e,t);void 0!==r&&this._emitter.emit(h.Events.SheetRenamed,r,t)}batch(e){this.suspendEvaluation(),this._crudOperations.beginUndoRedoBatchMode();try{e()}catch(e){throw this._crudOperations.commitUndoRedoBatchMode(),this.resumeEvaluation(),e}return this._crudOperations.commitUndoRedoBatchMode(),this.resumeEvaluation()}suspendEvaluation(){this._evaluationSuspended=!0,this._emitter.emit(h.Events.EvaluationSuspended)}resumeEvaluation(){this._evaluationSuspended=!1;const e=this.recomputeIfDependencyGraphNeedsIt();return this._emitter.emit(h.Events.EvaluationResumed,e),e}isEvaluationSuspended(){return this._evaluationSuspended}isItPossibleToAddNamedExpression(e,t,r){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==r&&(0,o.validateArgToType)(r,"number","scope");try{return this._crudOperations.ensureItIsPossibleToAddNamedExpression(e,t,r),!0}catch(e){return!1}}addNamedExpression(e,t,r,n){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==r&&(0,o.validateArgToType)(r,"number","scope"),this._crudOperations.addNamedExpression(e,t,r,n);const s=this.recomputeIfDependencyGraphNeedsIt();return this._emitter.emit(h.Events.NamedExpressionAdded,e,s),s}getNamedExpressionValue(e,t){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==t&&(0,o.validateArgToType)(t,"number","scope"),this.ensureEvaluationIsNotSuspended(),this._crudOperations.ensureScopeIdIsValid(t);const r=this._namedExpressions.namedExpressionForScope(e,t);return r?this._serialization.getCellValue(r.address):void 0}getNamedExpressionFormula(e,t){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==t&&(0,o.validateArgToType)(t,"number","scope"),this._crudOperations.ensureScopeIdIsValid(t);const r=this._namedExpressions.namedExpressionForScope(e,t);return void 0===r?void 0:this._serialization.getCellFormula(r.address)}getNamedExpression(e,t){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==t&&(0,o.validateArgToType)(t,"number","scope");const r=this._namedExpressions.namedExpressionForScope(e,t);if(void 0===r)return;return{name:e,scope:t,expression:this._serialization.getCellFormula(r.address),options:r.options}}isItPossibleToChangeNamedExpression(e,t,r){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==r&&(0,o.validateArgToType)(r,"number","scope");try{return this._crudOperations.ensureItIsPossibleToChangeNamedExpression(e,t,r),!0}catch(e){return!1}}changeNamedExpression(e,t,r,n){return(0,o.validateArgToType)(e,"string","expressionName"),void 0!==r&&(0,o.validateArgToType)(r,"number","scope"),this._crudOperations.changeNamedExpressionExpression(e,r,t,n),this.recomputeIfDependencyGraphNeedsIt()}isItPossibleToRemoveNamedExpression(e,t){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==t&&(0,o.validateArgToType)(t,"number","scope");try{return this._crudOperations.isItPossibleToRemoveNamedExpression(e,t),!0}catch(e){return!1}}removeNamedExpression(e,t){(0,o.validateArgToType)(e,"string","expressionName"),void 0!==t&&(0,o.validateArgToType)(t,"number","scope");const r=this._crudOperations.removeNamedExpression(e,t);if(r){const e=this.recomputeIfDependencyGraphNeedsIt();return this._emitter.emit(h.Events.NamedExpressionRemoved,r.displayName,e),e}return[]}listNamedExpressions(e){return void 0!==e&&(0,o.validateArgToType)(e,"number","scope"),this._crudOperations.ensureScopeIdIsValid(e),this._namedExpressions.getAllNamedExpressionsNamesInScope(e)}getAllNamedExpressionsSerialized(){return this._serialization.getAllNamedExpressionsSerialized()}normalizeFormula(e){(0,o.validateArgToType)(e,"string","formulaString");const{ast:t,address:r}=this.extractTemporaryFormula(e);if(void 0===t)throw new d.NotAFormulaError;return this._unparser.unparse(t,r)}calculateFormula(e,t){(0,o.validateArgToType)(e,"string","formulaString"),(0,o.validateArgToType)(t,"number","sheetId"),this._crudOperations.ensureScopeIdIsValid(t);const{ast:r,address:n,dependencies:s}=this.extractTemporaryFormula(e,t);if(void 0===r)throw new d.NotAFormulaError;const i=this.evaluator.runAndForget(r,n,s);return this._exporter.exportScalarOrRange(i)}validateFormula(e){(0,o.validateArgToType)(e,"string","formulaString");const{ast:t}=this.extractTemporaryFormula(e);return void 0!==t&&!(t.type===f.AstNodeType.ERROR&&!t.error)}getRegisteredFunctionNames(){return E.getLanguage(this._config.language).getFunctionTranslations(this._functionRegistry.getRegisteredFunctionIds())}getFunctionPlugin(e){return(0,o.validateArgToType)(e,"string","functionId"),this._functionRegistry.getFunctionPlugin(e)}getAllFunctionPlugins(){return this._functionRegistry.getPlugins()}numberToDateTime(e){return(0,o.validateArgToType)(e,"number","val"),this._evaluator.interpreter.dateTimeHelper.numberToSimpleDateTime(e)}numberToDate(e){return(0,o.validateArgToType)(e,"number","val"),this._evaluator.interpreter.dateTimeHelper.numberToSimpleDate(e)}numberToTime(e){return(0,o.validateArgToType)(e,"number","val"),(0,c.numberToSimpleTime)(e)}on(e,t){this._emitter.on(e,t)}once(e,t){this._emitter.once(e,t)}off(e,t){this._emitter.off(e,t)}destroy(){(0,l.objectDestroy)(this)}ensureEvaluationIsNotSuspended(){if(this._evaluationSuspended)throw new d.EvaluationSuspendedError}extractTemporaryFormula(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1;const r=this._cellContentParser.parse(e),n={sheet:t,col:0,row:0};if(!(r instanceof a.CellContent.Formula))return{address:n,dependencies:[]};const{ast:o,errors:s,dependencies:i}=this._parser.parse(r.formula,n);return s.length>0?{address:n,dependencies:[]}:{ast:o,address:n,dependencies:i}}rebuildWithConfig(e){const t=this._config.mergeConfig(e),r=this._config.mergeConfig({language:e.language}),n=this._serialization.withNewConfig(r,this._namedExpressions).getAllSheetsSerialized(),o=this._serialization.getAllNamedExpressionsSerialized(),i=s.BuildEngineFactory.rebuildWithConfig(t,n,o,this._stats);this._config=i.config,this._stats=i.stats,this._dependencyGraph=i.dependencyGraph,this._columnSearch=i.columnSearch,this._parser=i.parser,this._unparser=i.unparser,this._cellContentParser=i.cellContentParser,this._evaluator=i.evaluator,this._lazilyTransformingAstService=i.lazilyTransformingAstService,this._crudOperations=i.crudOperations,this._exporter=i.exporter,this._namedExpressions=i.namedExpressions,this._serialization=i.serialization,this._functionRegistry=i.functionRegistry}recomputeIfDependencyGraphNeedsIt(){if(this._evaluationSuspended)return[];{const e=this._crudOperations.getAndClearContentChanges(),t=Array.from(this.dependencyGraph.verticesToRecompute());this.dependencyGraph.clearRecentlyChangedVertices(),t.length>0&&e.addAll(this.evaluator.partialRun(t));const r=e.exportChanges(this._exporter);return e.isEmpty()||this._emitter.emit(h.Events.ValuesUpdated,r),r}}}t.HyperFormula=E,E.version="2.5.0",E.buildDate="29/05/2023 09:32:45",E.releaseDate="29/05/2023",E.languages={},E.registeredLanguages=new Map},function(e,t,r){"use strict";t.__esModule=!0,t.BuildEngineFactory=void 0;var n=r(1),o=r(181),s=r(182),i=r(170),a=r(183),u=r(173),c=r(75),l=r(148),h=r(196),d=r(197),p=r(198),m=r(157),g=r(166),f=r(199),E=r(201),T=r(203),y=r(205),A=r(184),R=r(210),N=r(185),S=r(78),C=r(211),v=r(194),O=r(143),M=r(195);t.BuildEngineFactory=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{static buildFromSheets(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[];const n=new i.Config(t);return this.buildEngine(n,e,r)}static buildFromSheet(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[];const n=new i.Config(t),o=n.translationPackage.getUITranslation(m.UIElement.NEW_SHEET_PREFIX)+"1";return this.buildEngine(n,{[o]:e},r)}static buildEmpty(){let e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[];return this.buildEngine(new i.Config(e),{},t)}static rebuildWithConfig(e,t,r,n){return this.buildEngine(e,t,r,n)}static buildEngine(e){let t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:[],i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:e.useStats?new O.Statistics:new O.EmptyStatistics;i.start(O.StatType.BUILD_ENGINE_TOTAL);const m=new A.NamedExpressions,I=new f.FunctionRegistry(e),w=new T.LazilyTransformingAstService(i),F=c.DependencyGraph.buildEmpty(w,e,I,m,i),b=(0,y.buildColumnSearchStrategy)(F,e,i),_=F.sheetMapping,L=F.addressMapping;for(const r in t)if(Object.prototype.hasOwnProperty.call(t,r)){const n=t[r];(0,v.validateAsSheet)(n);const o=(0,v.findBoundaries)(n);if(o.height>e.maxRows||o.width>e.maxColumns)throw new l.SheetSizeLimitExceededError;const s=_.addSheet(r);L.autoAddSheet(s,n,o)}const x=new S.ParserWithCaching(e,I,_.get);w.parser=x;const P=new S.Unparser(e,(0,S.buildLexerConfig)(e),_.fetchDisplayName,m),U=new u.DateTimeHelper(e),V=new R.NumberLiteralHelper(e),D=new g.ArithmeticHelper(e,U,V),B=new o.CellContentParser(e,U,V),k=new n.ArraySizePredictor(e,I),G=new N.Operations(e,F,b,B,x,i,w,m,k),j=new M.UndoRedo(e,G);w.undoRedo=j;const W=new s.ClipboardOperations(e,F,G),H=new a.CrudOperations(e,G,j,W,F,b,x,B,w,m);r.forEach(e=>{H.ensureItIsPossibleToAddNamedExpression(e.name,e.expression,e.scope),H.operations.addNamedExpression(e.name,e.expression,e.scope,e.options)});const z=new d.Exporter(e,m,_.fetchDisplayName,w),Y=new C.Serialization(F,P,z),X=new E.Interpreter(e,F,b,i,D,I,m,Y,k,U);i.measure(O.StatType.GRAPH_BUILD,()=>{new p.GraphBuilder(F,b,x,B,i,k).buildGraph(t,i)});const K=new h.Evaluator(e,i,X,w,F,b);return K.run(),i.end(O.StatType.BUILD_ENGINE_TOTAL),{config:e,stats:i,dependencyGraph:F,columnSearch:b,parser:x,unparser:P,cellContentParser:B,evaluator:K,lazilyTransformingAstService:w,crudOperations:H,exporter:z,namedExpressions:m,serialization:Y,functionRegistry:I}}}},function(e,t,r){"use strict";t.__esModule=!0,t.CellContentParser=t.CellContent=void 0,t.isBoolean=h,t.isError=d,t.isFormula=l;var n,o=r(74),s=r(173),i=r(132),a=r(148),u=r(166),c=r(141);function l(e){return e.startsWith("=")}function h(e){const t=e.toLowerCase();return"true"===t||"false"===t}function d(e,t){const r=e.toUpperCase();return/#[A-Za-z0-9\/]+[?!]?/.test(r)&&Object.prototype.hasOwnProperty.call(t,r)}t.CellContent=n,function(e){e.Number=class{constructor(e){this.value=e,this.value=(0,c.cloneNumber)(this.value,(0,u.fixNegativeZero)((0,c.getRawValue)(this.value)))}};e.String=class{constructor(e){this.value=e}};e.Boolean=class{constructor(e){this.value=e}};class t{static getSingletonInstance(){return t.instance||(t.instance=new t),t.instance}}e.Empty=t;e.Formula=class{constructor(e){this.formula=e}};e.Error=class{constructor(e,t){this.value=new o.CellError(e,t)}}}(n||(t.CellContent=n={}));t.CellContentParser=class{constructor(e,t,r){this.config=e,this.dateHelper=t,this.numberLiteralsHelper=r}parse(e){if(null==e)return n.Empty.getSingletonInstance();if("number"==typeof e)return(0,u.isNumberOverflow)(e)?new n.Error(o.ErrorType.NUM,i.ErrorMessage.ValueLarge):new n.Number(e);if("boolean"==typeof e)return new n.Boolean(e);if(e instanceof Date){const t=this.dateHelper.dateToNumber({day:e.getDate(),month:e.getMonth()+1,year:e.getFullYear()})+(0,s.timeToNumber)({hours:e.getHours(),minutes:e.getMinutes(),seconds:e.getSeconds()+e.getMilliseconds()/1e3});return t<0?new n.Error(o.ErrorType.NUM,i.ErrorMessage.DateBounds):t%1==0?new n.Number(new c.DateNumber(t,"Date()")):t<1?new n.Number(new c.TimeNumber(t,"Date()")):new n.Number(new c.DateTimeNumber(t,"Date()"))}if("string"==typeof e){if(h(e))return new n.Boolean("true"===e.toLowerCase());if(l(e))return new n.Formula(e);if(d(e,this.config.errorMapping))return new n.Error(this.config.errorMapping[e.toUpperCase()]);{let t,r=e.trim(),o=0;if(r.endsWith("%"))o=1,r=r.slice(0,r.length-1);else{const e=this.currencyMatcher(r);void 0!==e&&(o=2,[t,r]=e)}const s=this.numberLiteralsHelper.numericStringToMaybeNumber(r);if(void 0!==s){let e;return e=1===o?new c.PercentNumber(s/100):2===o?new c.CurrencyNumber(s,t):s,new n.Number(e)}const i=this.dateHelper.dateStringToDateNumber(r);return void 0!==i?new n.Number(i):new n.String(e.startsWith("'")?e.slice(1):e)}}throw new a.UnableToParseError(e)}currencyMatcher(e){for(const t of this.config.currencySymbol){if(e.startsWith(t))return[t,e.slice(t.length)];if(e.endsWith(t))return[t,e.slice(0,e.length-t.length)]}}}},function(e,t,r){"use strict";t.__esModule=!0,t.ClipboardOperations=t.ClipboardCellType=void 0,r(2);var n,o,s=r(73),i=r(74),a=r(148);!function(e){e[e.COPY=0]="COPY",e[e.CUT=1]="CUT"}(n||(n={})),t.ClipboardCellType=o,function(e){e[e.VALUE=0]="VALUE",e[e.EMPTY=1]="EMPTY",e[e.FORMULA=2]="FORMULA",e[e.PARSING_ERROR=3]="PARSING_ERROR"}(o||(t.ClipboardCellType=o={}));class u{constructor(e,t,r,n,o){this.sourceLeftCorner=e,this.width=t,this.height=r,this.type=n,this.content=o}*getContent(e){if(void 0!==this.content)for(let t=0;t<this.height;++t)for(let r=0;r<this.width;++r)yield[(0,i.simpleCellAddress)(e.sheet,e.col+r,e.row+t),this.content[t][r]]}}t.ClipboardOperations=class{constructor(e,t,r){this.dependencyGraph=t,this.operations=r,this.maxRows=e.maxRows,this.maxColumns=e.maxColumns}cut(e,t,r){this.clipboard=new u(e,t,r,n.CUT)}copy(e,t,r){const o=[];for(let n=0;n<r;++n){o[n]=[];for(let r=0;r<t;++r){const t=this.operations.getClipboardCell((0,i.simpleCellAddress)(e.sheet,e.col+r,e.row+n));o[n].push(t)}}this.clipboard=new u(e,t,r,n.COPY,o)}abortCut(){this.clipboard&&this.clipboard.type===n.CUT&&this.clear()}clear(){this.clipboard=void 0}ensureItIsPossibleToCopyPaste(e){if(void 0===this.clipboard)return;if((0,i.invalidSimpleCellAddress)(e)||!this.dependencyGraph.sheetMapping.hasSheetWithId(e.sheet))throw new a.InvalidArgumentsError("a valid target address.");const t=s.AbsoluteCellRange.spanFrom(e,this.clipboard.width,this.clipboard.height);if(t.exceedsSheetSizeLimits(this.maxColumns,this.maxRows))throw new a.SheetSizeLimitExceededError;if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(t))throw new Error("It is not possible to paste onto an array")}isCutClipboard(){return void 0!==this.clipboard&&this.clipboard.type===n.CUT}isCopyClipboard(){return void 0!==this.clipboard&&this.clipboard.type===n.COPY}}},function(e,t,r){"use strict";t.__esModule=!0,t.CrudOperations=void 0,r(2);var n=r(73),o=r(74),s=r(181),i=r(148),a=r(184),u=r(185),c=r(194),l=r(142),h=r(195);function d(e){return Number.isInteger(e)&&e>0}function p(e){return Number.isInteger(e)&&e>=0}t.CrudOperations=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r,n,o,s,i,a,u,c){this.operations=t,this.undoRedo=r,this.clipboardOperations=n,this.dependencyGraph=o,this.columnSearch=s,this.parser=i,this.cellContentParser=a,this.lazilyTransformingAstService=u,this.namedExpressions=c,this.maxRows=e.maxRows,this.maxColumns=e.maxColumns}get sheetMapping(){return this.dependencyGraph.sheetMapping}addRows(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const o=new u.AddRowsCommand(e,r);this.ensureItIsPossibleToAddRows(e,...r),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut(),this.operations.addRows(o),this.undoRedo.saveOperation(new h.AddRowsUndoEntry(o))}removeRows(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const o=new u.RemoveRowsCommand(e,r);this.ensureItIsPossibleToRemoveRows(e,...r),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const s=this.operations.removeRows(o);this.undoRedo.saveOperation(new h.RemoveRowsUndoEntry(o,s))}addColumns(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const o=new u.AddColumnsCommand(e,r);this.ensureItIsPossibleToAddColumns(e,...r),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut(),this.operations.addColumns(o),this.undoRedo.saveOperation(new h.AddColumnsUndoEntry(o))}removeColumns(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];const o=new u.RemoveColumnsCommand(e,r);this.ensureItIsPossibleToRemoveColumns(e,...r),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const s=this.operations.removeColumns(o);this.undoRedo.saveOperation(new h.RemoveColumnsUndoEntry(o,s))}moveCells(e,t,r,n){this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const{version:o,overwrittenCellsData:s,addedGlobalNamedExpressions:i}=this.operations.moveCells(e,t,r,n);this.undoRedo.saveOperation(new h.MoveCellsUndoEntry(e,t,r,n,s,i,o))}moveRows(e,t,r,n){this.ensureItIsPossibleToMoveRows(e,t,r,n),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const o=this.operations.moveRows(e,t,r,n);this.undoRedo.saveOperation(new h.MoveRowsUndoEntry(e,t,r,n,o))}moveColumns(e,t,r,n){this.ensureItIsPossibleToMoveColumns(e,t,r,n),this.undoRedo.clearRedoStack();const o=this.operations.moveColumns(e,t,r,n);this.undoRedo.saveOperation(new h.MoveColumnsUndoEntry(e,t,r,n,o))}cut(e,t,r){this.clipboardOperations.cut(e,t,r)}ensureItIsPossibleToCopy(e,t,r){if(!d(t))throw new i.InvalidArgumentsError("width to be positive integer.");if(!d(r))throw new i.InvalidArgumentsError("height to be positive integer.")}copy(e,t,r){this.ensureItIsPossibleToCopy(e,t,r),this.clipboardOperations.copy(e,t,r)}paste(e){const t=this.clipboardOperations.clipboard;if(void 0===t)throw new i.NothingToPasteError;if(this.clipboardOperations.isCutClipboard())this.moveCells(t.sourceLeftCorner,t.width,t.height,e);else if(this.clipboardOperations.isCopyClipboard()){this.clipboardOperations.ensureItIsPossibleToCopyPaste(e);const r=n.AbsoluteCellRange.spanFrom(e,t.width,t.height),o=this.operations.getRangeClipboardCells(r);this.undoRedo.clearRedoStack();const s=this.operations.restoreClipboardCells(t.sourceLeftCorner.sheet,t.getContent(e));this.undoRedo.saveOperation(new h.PasteUndoEntry(e,o,t.content,s))}}beginUndoRedoBatchMode(){this.undoRedo.beginBatchMode()}commitUndoRedoBatchMode(){this.undoRedo.commitBatchMode()}isClipboardEmpty(){return void 0===this.clipboardOperations.clipboard}clearClipboard(){this.clipboardOperations.clear()}addSheet(e){void 0!==e&&this.ensureItIsPossibleToAddSheet(e),this.undoRedo.clearRedoStack();const t=this.operations.addSheet(e);return this.undoRedo.saveOperation(new h.AddSheetUndoEntry(t)),t}removeSheet(e){this.ensureScopeIdIsValid(e),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const t=this.sheetMapping.fetchDisplayName(e),r=this.operations.getSheetClipboardCells(e),{version:n,scopedNamedExpressions:o}=this.operations.removeSheet(e);this.undoRedo.saveOperation(new h.RemoveSheetUndoEntry(t,e,r,o,n))}renameSheet(e,t){this.ensureItIsPossibleToRenameSheet(e,t);const r=this.operations.renameSheet(e,t);return void 0!==r&&(this.undoRedo.clearRedoStack(),this.undoRedo.saveOperation(new h.RenameSheetUndoEntry(e,r,t))),r}clearSheet(e){this.ensureScopeIdIsValid(e),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const t=this.operations.getSheetClipboardCells(e);this.operations.clearSheet(e),this.undoRedo.saveOperation(new h.ClearSheetUndoEntry(e,t))}setCellContents(e,t){if(t instanceof Array){for(let e=0;e<t.length;e++)if(!(t[e]instanceof Array))throw new i.InvalidArgumentsError("an array of arrays or a raw cell value.")}else t=[[t]];this.ensureItIsPossibleToChangeCellContents(e,t),this.undoRedo.clearRedoStack();const r=[];for(let n=0;n<t.length;n++)for(let o=0;o<t[n].length;o++){const s={sheet:e.sheet,row:e.row+n,col:e.col+o},i=t[n][o];this.clipboardOperations.abortCut();const a=this.operations.setCellContent(s,i);r.push({address:s,newContent:i,oldContent:a})}this.undoRedo.saveOperation(new h.SetCellContentsUndoEntry(r))}setSheetContent(e,t){this.ensureScopeIdIsValid(e),this.ensureItIsPossibleToChangeSheetContents(e,t),(0,c.validateAsSheet)(t),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const r=this.operations.getSheetClipboardCells(e);this.operations.setSheetContent(e,t),this.undoRedo.saveOperation(new h.SetSheetContentUndoEntry(e,r,t))}setRowOrder(e,t){this.validateSwapRowIndexes(e,t),this.testRowOrderForArrays(e,t),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const r=this.operations.setRowOrder(e,t);this.undoRedo.saveOperation(new h.SetRowOrderUndoEntry(e,t,r))}validateSwapRowIndexes(e,t){if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e);this.validateRowOrColumnMapping(e,t,"row")}testColumnOrderForArrays(e,t){for(const[r,o]of t)if(r!==o){const t=n.AbsoluteCellRange.spanFrom({sheet:e,col:r,row:0},1,1/0);if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(t))throw new i.SourceLocationHasArrayError}}setColumnOrder(e,t){this.validateSwapColumnIndexes(e,t),this.testColumnOrderForArrays(e,t),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut();const r=this.operations.setColumnOrder(e,t);this.undoRedo.saveOperation(new h.SetColumnOrderUndoEntry(e,t,r))}validateSwapColumnIndexes(e,t){if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e);this.validateRowOrColumnMapping(e,t,"column")}testRowOrderForArrays(e,t){for(const[r,o]of t)if(r!==o){const t=n.AbsoluteCellRange.spanFrom({sheet:e,col:0,row:r},1/0,1);if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(t))throw new i.SourceLocationHasArrayError}}mappingFromOrder(e,t,r){if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e);const n="row"===r?this.dependencyGraph.getSheetHeight(e):this.dependencyGraph.getSheetWidth(e);if(t.length!==n)throw new i.InvalidArgumentsError(`number of ${r}s provided to be sheet ${"row"===r?"height":"width"}.`);const o=[];for(let e=0;e<n;e++)t[e]!==e&&o.push([e,t[e]]);return o}undo(){if(this.undoRedo.isUndoStackEmpty())throw new i.NoOperationToUndoError;this.clipboardOperations.abortCut(),this.undoRedo.undo()}redo(){if(this.undoRedo.isRedoStackEmpty())throw new i.NoOperationToRedoError;this.clipboardOperations.abortCut(),this.undoRedo.redo()}addNamedExpression(e,t,r,n){this.ensureItIsPossibleToAddNamedExpression(e,t,r),this.operations.addNamedExpression(e,t,r,n),this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut(),this.undoRedo.saveOperation(new h.AddNamedExpressionUndoEntry(e,t,r,n))}changeNamedExpressionExpression(e,t,r,n){this.ensureItIsPossibleToChangeNamedExpression(e,r,t);const[o,s]=this.operations.changeNamedExpressionExpression(e,r,t,n);this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut(),this.undoRedo.saveOperation(new h.ChangeNamedExpressionUndoEntry(o,r,s,t,n))}removeNamedExpression(e,t){this.ensureScopeIdIsValid(t);const[r,n]=this.operations.removeNamedExpression(e,t);return this.undoRedo.clearRedoStack(),this.clipboardOperations.abortCut(),this.undoRedo.saveOperation(new h.RemoveNamedExpressionUndoEntry(r,n,t)),r}ensureItIsPossibleToAddNamedExpression(e,t,r){this.ensureScopeIdIsValid(r),this.ensureNamedExpressionNameIsValid(e,r),this.ensureNamedExpressionIsValid(t)}ensureItIsPossibleToChangeNamedExpression(e,t,r){if(this.ensureScopeIdIsValid(r),void 0===this.namedExpressions.namedExpressionForScope(e,r))throw new i.NamedExpressionDoesNotExistError(e);this.ensureNamedExpressionIsValid(t)}isItPossibleToRemoveNamedExpression(e,t){if(this.ensureScopeIdIsValid(t),void 0===this.namedExpressions.namedExpressionForScope(e,t))throw new i.NamedExpressionDoesNotExistError(e)}ensureItIsPossibleToAddRows(e){if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e);const t=this.dependencyGraph.getSheetHeight(e);for(var r=arguments.length,n=new Array(r>1?r-1:0),o=1;o<r;o++)n[o-1]=arguments[o];if(t+n.map(e=>e[1]).reduce((e,t)=>e+t,0)>this.maxRows)throw new i.SheetSizeLimitExceededError;for(const[e,t]of n)if(!p(e)||!d(t))throw new i.InvalidArgumentsError("row number to be nonnegative and number of rows to add to be positive.")}ensureItIsPossibleToRemoveRows(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];for(const[t,n]of r){const r=t+n-1;if(!p(t)||!p(r))throw new i.InvalidArgumentsError("starting and ending row to be nonnegative.");if(r<t)throw new i.InvalidArgumentsError("starting row to be smaller than the ending row.");if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e)}}ensureItIsPossibleToAddColumns(e){if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e);const t=this.dependencyGraph.getSheetWidth(e);for(var r=arguments.length,n=new Array(r>1?r-1:0),o=1;o<r;o++)n[o-1]=arguments[o];if(t+n.map(e=>e[1]).reduce((e,t)=>e+t,0)>this.maxColumns)throw new i.SheetSizeLimitExceededError;for(const[e,t]of n)if(!p(e)||!d(t))throw new i.InvalidArgumentsError("column number to be nonnegative and number of columns to add to be positive.")}ensureItIsPossibleToRemoveColumns(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];for(const[t,n]of r){const r=t+n-1;if(!p(t)||!p(r))throw new i.InvalidArgumentsError("starting and ending column to be nonnegative.");if(r<t)throw new i.InvalidArgumentsError("starting column to be smaller than the ending column.");if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e)}}ensureItIsPossibleToMoveRows(e,t,r,s){this.ensureItIsPossibleToAddRows(e,[s,r]);const a=(0,o.simpleCellAddress)(e,0,t),u=(0,o.simpleCellAddress)(e,0,s);if(!this.sheetMapping.hasSheetWithId(e)||(0,o.invalidSimpleCellAddress)(a)||(0,o.invalidSimpleCellAddress)(u)||!d(r)||s<=t+r&&s>=t)throw new i.InvalidArgumentsError("a valid range of rows to move.");const c=this.dependencyGraph.getSheetWidth(e),h=n.AbsoluteCellRange.spanFrom(a,c,r);if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(h))throw new i.SourceLocationHasArrayError;if(s>0&&this.dependencyGraph.arrayMapping.isFormulaArrayInAllRows(l.RowsSpan.fromNumberOfRows(e,s-1,2)))throw new i.TargetLocationHasArrayError}ensureItIsPossibleToMoveColumns(e,t,r,s){this.ensureItIsPossibleToAddColumns(e,[s,r]);const a=(0,o.simpleCellAddress)(e,t,0),u=(0,o.simpleCellAddress)(e,s,0);if(!this.sheetMapping.hasSheetWithId(e)||(0,o.invalidSimpleCellAddress)(a)||(0,o.invalidSimpleCellAddress)(u)||!d(r)||s<=t+r&&s>=t)throw new i.InvalidArgumentsError("a valid range of columns to move.");const c=this.dependencyGraph.getSheetHeight(e),h=n.AbsoluteCellRange.spanFrom(a,r,c);if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(h))throw new i.SourceLocationHasArrayError;if(s>0&&this.dependencyGraph.arrayMapping.isFormulaArrayInAllColumns(l.ColumnsSpan.fromNumberOfColumns(e,s-1,2)))throw new i.TargetLocationHasArrayError}ensureItIsPossibleToAddSheet(e){if(this.sheetMapping.hasSheetWithName(e))throw new i.SheetNameAlreadyTakenError(e)}ensureItIsPossibleToRenameSheet(e,t){if(!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e);const r=this.sheetMapping.get(t);if(void 0!==r&&r!==e)throw new i.SheetNameAlreadyTakenError(t)}ensureItIsPossibleToChangeContent(e){if((0,o.invalidSimpleCellAddress)(e))throw new i.InvalidAddressError(e);if(!this.sheetMapping.hasSheetWithId(e.sheet))throw new i.NoSheetWithIdError(e.sheet)}ensureItIsPossibleToChangeCellContents(e,t){const r=(0,c.findBoundaries)(t),o=n.AbsoluteCellRange.spanFrom(e,r.width,r.height);this.ensureRangeInSizeLimits(o);for(const e of o.addresses(this.dependencyGraph))this.ensureItIsPossibleToChangeContent(e)}ensureItIsPossibleToChangeSheetContents(e,t){const r=(0,c.findBoundaries)(t),s=n.AbsoluteCellRange.spanFrom((0,o.simpleCellAddress)(e,0,0),r.width,r.height);this.ensureRangeInSizeLimits(s)}ensureRangeInSizeLimits(e){if(e.exceedsSheetSizeLimits(this.maxColumns,this.maxRows))throw new i.SheetSizeLimitExceededError}isThereSomethingToUndo(){return!this.undoRedo.isUndoStackEmpty()}isThereSomethingToRedo(){return!this.undoRedo.isRedoStackEmpty()}getAndClearContentChanges(){return this.operations.getAndClearContentChanges()}ensureScopeIdIsValid(e){if(void 0!==e&&!this.sheetMapping.hasSheetWithId(e))throw new i.NoSheetWithIdError(e)}validateRowOrColumnMapping(e,t,r){const n="row"===r?this.dependencyGraph.getSheetHeight(e):this.dependencyGraph.getSheetWidth(e),o=t.map(e=>{let[t,r]=e;return t}).sort((e,t)=>e-t),s=t.map(e=>{let[t,r]=e;return r}).sort((e,t)=>e-t);for(let e=0;e<o.length;e++){if(!p(o[e])||o[e]>=n)throw new i.InvalidArgumentsError(`${r} numbers to be nonnegative integers and less than sheet ${"row"===r?"height":"width"}.`);if(o[e]===o[e+1])throw new i.InvalidArgumentsError(`source ${r} numbers to be unique.`);if(o[e]!==s[e])throw new i.InvalidArgumentsError(`target ${r} numbers to be permutation of source ${r} numbers.`)}}ensureNamedExpressionNameIsValid(e,t){if(!this.namedExpressions.isNameValid(e))throw new i.NamedExpressionNameIsInvalidError(e);if(!this.namedExpressions.isNameAvailable(e,t))throw new i.NamedExpressionNameIsAlreadyTakenError(e)}ensureNamedExpressionIsValid(e){const t=this.cellContentParser.parse(e);if(t instanceof s.CellContent.Formula){const e=this.parser.parse(t.formula,(0,o.simpleCellAddress)(-1,0,0));if((0,a.doesContainRelativeReferences)(e.ast))throw new i.NoRelativeAddressesAllowedError}}}},function(e,t,r){"use strict";t.__esModule=!0,t.doesContainRelativeReferences=t.NamedExpressions=t.InternalNamedExpression=void 0,r(2);var n=r(74),o=r(78),s=r(83);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i{constructor(e,t,r,n){this.displayName=e,this.address=t,this.added=r,this.options=n}normalizeExpressionName(){return this.displayName.toLowerCase()}copy(){return new i(this.displayName,this.address,this.added,this.options)}}t.InternalNamedExpression=i;class a{constructor(){this.mapping=new Map}has(e){return this.mapping.has(this.normalizeExpressionName(e))}isNameAvailable(e){const t=this.normalizeExpressionName(e),r=this.mapping.get(t);return!(r&&r.added)}add(e){this.mapping.set(e.normalizeExpressionName(),e)}get(e){return this.mapping.get(this.normalizeExpressionName(e))}getExisting(e){const t=this.mapping.get(this.normalizeExpressionName(e));return t&&t.added?t:void 0}remove(e){const t=this.normalizeExpressionName(e),r=this.mapping.get(t);r&&(r.added=!1)}getAllNamedExpressions(){return Array.from(this.mapping.values()).filter(e=>e.added)}normalizeExpressionName(e){return e.toLowerCase()}}class u{constructor(){this.mapping=new Map}add(e){this.mapping.set(this.normalizeExpressionName(e.displayName),e)}get(e){return this.mapping.get(this.normalizeExpressionName(e))}has(e){return this.mapping.has(this.normalizeExpressionName(e))}getAllNamedExpressions(){return Array.from(this.mapping.values()).filter(e=>e.added)}isNameAvailable(e){const t=this.normalizeExpressionName(e);return!this.mapping.has(t)}remove(e){const t=this.normalizeExpressionName(e);this.mapping.get(t)&&this.mapping.delete(t)}normalizeExpressionName(e){return e.toLowerCase()}}class c{constructor(){this.nextNamedExpressionRow=0,this.workbookStore=new a,this.worksheetStores=new Map,this.addressCache=new Map}isNameAvailable(e,t){var r,n;return void 0===t?this.workbookStore.isNameAvailable(e):null===(n=null===(r=this.worksheetStore(t))||void 0===r?void 0:r.isNameAvailable(e))||void 0===n||n}namedExpressionInAddress(e){const t=this.addressCache.get(e);return t&&t.added?t:void 0}namedExpressionForScope(e,t){var r;return void 0===t?this.workbookStore.getExisting(e):null===(r=this.worksheetStore(t))||void 0===r?void 0:r.get(e)}nearestNamedExpression(e,t){var r,n;return null!==(n=null===(r=this.worksheetStore(t))||void 0===r?void 0:r.get(e))&&void 0!==n?n:this.workbookStore.getExisting(e)}isExpressionInScope(e,t){var r,n;return null!==(n=null===(r=this.worksheetStore(t))||void 0===r?void 0:r.has(e))&&void 0!==n&&n}isNameValid(e){const t=new RegExp(`^${s.CELL_REFERENCE_PATTERN}$`),r=new RegExp(`^${s.R1C1_CELL_REFERENCE_PATTERN}$`),n=new RegExp(`^${s.NAMED_EXPRESSION_PATTERN}$`);return!t.test(e)&&!r.test(e)&&n.test(e)}addNamedExpression(e,t,r){const n=void 0===t?this.workbookStore:this.worksheetStoreOrCreate(t);let o=n.get(e);return void 0!==o?(o.added=!0,o.displayName=e,o.options=r):(o=new i(e,this.nextAddress(),!0,r),n.add(o)),this.addressCache.set(o.address.row,o),o}restoreNamedExpression(e,t){const r=void 0===t?this.workbookStore:this.worksheetStoreOrCreate(t);return e.added=!0,r.add(e),this.addressCache.set(e.address.row,e),e}namedExpressionOrPlaceholder(e,t){var r;return null!==(r=this.worksheetStoreOrCreate(t).get(e))&&void 0!==r?r:this.workbookNamedExpressionOrPlaceholder(e)}workbookNamedExpressionOrPlaceholder(e){let t=this.workbookStore.get(e);return void 0===t&&(t=new i(e,this.nextAddress(),!1),this.workbookStore.add(t)),t}remove(e,t){let r;r=void 0===t?this.workbookStore:this.worksheetStore(t);const n=null==r?void 0:r.get(e);if(void 0===r||void 0===n||!n.added)throw Error("Named expression does not exist");r.remove(e),r instanceof u&&0===r.mapping.size&&this.worksheetStores.delete(t),this.addressCache.delete(n.address.row)}getAllNamedExpressionsNamesInScope(e){return this.getAllNamedExpressions().filter(t=>{let{scope:r}=t;return r===e}).map(e=>e.expression.displayName)}getAllNamedExpressionsNames(){return this.getAllNamedExpressions().map(e=>e.expression.displayName)}getAllNamedExpressions(){const e=[];return this.workbookStore.getAllNamedExpressions().forEach(t=>{e.push({expression:t,scope:void 0})}),this.worksheetStores.forEach((t,r)=>{t.getAllNamedExpressions().forEach(t=>{e.push({expression:t,scope:r})})}),e}getAllNamedExpressionsForScope(e){var t,r;return void 0===e?this.workbookStore.getAllNamedExpressions():null!==(r=null===(t=this.worksheetStores.get(e))||void 0===t?void 0:t.getAllNamedExpressions())&&void 0!==r?r:[]}worksheetStoreOrCreate(e){let t=this.worksheetStores.get(e);return t||(t=new u,this.worksheetStores.set(e,t)),t}worksheetStore(e){return this.worksheetStores.get(e)}nextAddress(){return(0,n.simpleCellAddress)(c.SHEET_FOR_WORKBOOK_EXPRESSIONS,0,this.nextNamedExpressionRow++)}}t.NamedExpressions=c,c.SHEET_FOR_WORKBOOK_EXPRESSIONS=-1;const l=e=>{switch(e.type){case o.AstNodeType.EMPTY:case o.AstNodeType.NUMBER:case o.AstNodeType.STRING:case o.AstNodeType.ERROR:case o.AstNodeType.ERROR_WITH_RAW_INPUT:return!1;case o.AstNodeType.CELL_REFERENCE:return!e.reference.isAbsolute();case o.AstNodeType.CELL_RANGE:case o.AstNodeType.COLUMN_RANGE:case o.AstNodeType.ROW_RANGE:return!e.start.isAbsolute();case o.AstNodeType.NAMED_EXPRESSION:return!1;case o.AstNodeType.PERCENT_OP:case o.AstNodeType.PLUS_UNARY_OP:case o.AstNodeType.MINUS_UNARY_OP:return l(e.value);case o.AstNodeType.CONCATENATE_OP:case o.AstNodeType.EQUALS_OP:case o.AstNodeType.NOT_EQUAL_OP:case o.AstNodeType.LESS_THAN_OP:case o.AstNodeType.GREATER_THAN_OP:case o.AstNodeType.LESS_THAN_OR_EQUAL_OP:case o.AstNodeType.GREATER_THAN_OR_EQUAL_OP:case o.AstNodeType.MINUS_OP:case o.AstNodeType.PLUS_OP:case o.AstNodeType.TIMES_OP:case o.AstNodeType.DIV_OP:case o.AstNodeType.POWER_OP:return l(e.left)||l(e.right);case o.AstNodeType.PARENTHESIS:return l(e.expression);case o.AstNodeType.FUNCTION_CALL:return e.args.some(e=>l(e));case o.AstNodeType.ARRAY:return e.args.some(e=>e.some(e=>l(e)))}};t.doesContainRelativeReferences=l},function(e,t,r){"use strict";t.__esModule=!0,t.RemoveRowsCommand=t.RemoveColumnsCommand=t.Operations=t.AddRowsCommand=t.AddColumnsCommand=void 0,t.normalizeAddedIndexes=M,t.normalizeRemovedIndexes=O,r(2);var n=r(73),o=r(77),s=r(1),i=r(74),a=r(181),u=r(182),c=r(139),l=r(75),h=r(151),d=r(186),p=r(188),m=r(189),g=r(190),f=r(191),E=r(192),T=r(193),y=r(148),A=r(141),R=r(184),N=r(78),S=r(194),C=r(142),v=r(143);t.RemoveRowsCommand=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t){this.sheet=e,this.indexes=t}normalizedIndexes(){return O(this.indexes)}rowsSpans(){return this.normalizedIndexes().map(e=>C.RowsSpan.fromNumberOfRows(this.sheet,e[0],e[1]))}};t.AddRowsCommand=class{constructor(e,t){this.sheet=e,this.indexes=t}normalizedIndexes(){return M(this.indexes)}rowsSpans(){return this.normalizedIndexes().map(e=>C.RowsSpan.fromNumberOfRows(this.sheet,e[0],e[1]))}};t.AddColumnsCommand=class{constructor(e,t){this.sheet=e,this.indexes=t}normalizedIndexes(){return M(this.indexes)}columnsSpans(){return this.normalizedIndexes().map(e=>C.ColumnsSpan.fromNumberOfColumns(this.sheet,e[0],e[1]))}};t.RemoveColumnsCommand=class{constructor(e,t){this.sheet=e,this.indexes=t}normalizedIndexes(){return O(this.indexes)}columnsSpans(){return this.normalizedIndexes().map(e=>C.ColumnsSpan.fromNumberOfColumns(this.sheet,e[0],e[1]))}};function O(e){if(e.length<=1)return e;const t=[...e].sort((e,t)=>{let[r]=e,[n]=t;return r-n}),r=t.reduce((e,t)=>{let[r,n]=t;const o=e[e.length-1],s=o[0]+o[1];return r<=s?o[1]+=Math.max(0,n-(s-r)):e.push([r,n]),e},[t[0]]);let n=0;for(let e=0;e<r.length;++e)r[e][0]-=n,n+=r[e][1];return r}function M(e){if(e.length<=1)return e;const t=[...e].sort((e,t)=>{let[r]=e,[n]=t;return r-n}),r=t.reduce((e,t)=>{let[r,n]=t;const o=e[e.length-1];return r===o[0]?o[1]=Math.max(o[1],n):e.push([r,n]),e},[t[0]]);let n=0;for(let e=0;e<r.length;++e)r[e][0]+=n,n+=r[e][1];return r}function I(e){return Number.isInteger(e)&&e>0}t.Operations=class{constructor(e,t,r,n,o,s,i,a,u){this.dependencyGraph=t,this.columnSearch=r,this.cellContentParser=n,this.parser=o,this.stats=s,this.lazilyTransformingAstService=i,this.namedExpressions=a,this.arraySizePredictor=u,this.changes=c.ContentChanges.empty(),this.allocateNamedExpressionAddressSpace(),this.maxColumns=e.maxColumns,this.maxRows=e.maxRows}get sheetMapping(){return this.dependencyGraph.sheetMapping}get addressMapping(){return this.dependencyGraph.addressMapping}removeRows(e){const t=[];for(const r of e.rowsSpans()){const e=this.doRemoveRows(r);e&&t.push(e)}return t}addRows(e){for(const t of e.rowsSpans())this.doAddRows(t)}addColumns(e){for(const t of e.columnsSpans())this.doAddColumns(t)}removeColumns(e){const t=[];for(const r of e.columnsSpans()){const e=this.doRemoveColumns(r);e&&t.push(e)}return t}removeSheet(e){let t;this.dependencyGraph.removeSheet(e),this.stats.measure(v.StatType.TRANSFORM_ASTS,()=>{const r=new T.RemoveSheetTransformer(e);r.performEagerTransformations(this.dependencyGraph,this.parser),t=this.lazilyTransformingAstService.addTransformation(r)}),this.sheetMapping.removeSheet(e),this.columnSearch.removeSheet(e);const r=this.namedExpressions.getAllNamedExpressionsForScope(e).map(t=>this.removeNamedExpression(t.normalizeExpressionName(),e));return{version:t,scopedNamedExpressions:r}}removeSheetByName(e){const t=this.sheetMapping.fetch(e);return this.removeSheet(t)}clearSheet(e){this.dependencyGraph.clearSheet(e),this.columnSearch.removeSheet(e)}addSheet(e){const t=this.sheetMapping.addSheet(e),r=[];return this.dependencyGraph.addressMapping.autoAddSheet(t,r,(0,S.findBoundaries)(r)),this.sheetMapping.fetchDisplayName(t)}renameSheet(e,t){return this.sheetMapping.renameSheet(e,t)}moveRows(e,t,r,n){const o=C.RowsSpan.fromNumberOfRows(e,n,r);this.lazilyTransformingAstService.beginCombinedMode(e),this.doAddRows(o),n<t&&(t+=r);const s=(0,i.simpleCellAddress)(e,0,t),a=(0,i.simpleCellAddress)(e,0,n);this.moveCells(s,Number.POSITIVE_INFINITY,r,a);const u=C.RowsSpan.fromNumberOfRows(e,t,r);return this.doRemoveRows(u),this.lazilyTransformingAstService.commitCombinedMode()}moveColumns(e,t,r,n){const o=C.ColumnsSpan.fromNumberOfColumns(e,n,r);this.lazilyTransformingAstService.beginCombinedMode(e),this.doAddColumns(o),n<t&&(t+=r);const s=(0,i.simpleCellAddress)(e,t,0),a=(0,i.simpleCellAddress)(e,n,0);this.moveCells(s,r,Number.POSITIVE_INFINITY,a);const u=C.ColumnsSpan.fromNumberOfColumns(e,t,r);return this.doRemoveColumns(u),this.lazilyTransformingAstService.commitCombinedMode()}moveCells(e,t,r,o){this.ensureItIsPossibleToMoveCells(e,t,r,o);const s=n.AbsoluteCellRange.spanFrom(e,t,r),i=n.AbsoluteCellRange.spanFrom(o,t,r),a=o.col-e.col,u=o.row-e.row,c=o.sheet,l=this.getRangeClipboardCells(i),h=this.dependencyGraph.rawValuesFromRange(i);this.columnSearch.removeValues(h);const d=this.dependencyGraph.rawValuesFromRange(s);let p;this.columnSearch.moveValues(d,a,u,c),this.stats.measure(v.StatType.TRANSFORM_ASTS,()=>{const e=new g.MoveCellsTransformer(s,a,u,c);e.performEagerTransformations(this.dependencyGraph,this.parser),p=this.lazilyTransformingAstService.addTransformation(e)}),this.dependencyGraph.moveCells(s,a,u,c);const m=this.updateNamedExpressionsForMovedCells(e,t,r,o);return{version:p,overwrittenCellsData:l,addedGlobalNamedExpressions:m}}setRowOrder(e,t){const r=[];let o=[];for(const[s,i]of t)if(s!==i){const t=n.AbsoluteCellRange.spanFrom({sheet:e,col:0,row:s},1/0,1),a=this.getRangeClipboardCells(t);o=o.concat(a),r.push(a.map(e=>{let[{sheet:t,col:r},n]=e;return[{sheet:t,col:r,row:i},n]}))}return r.forEach(t=>this.restoreClipboardCells(e,t.values())),o}setColumnOrder(e,t){const r=[];let o=[];for(const[s,i]of t)if(s!==i){const t=n.AbsoluteCellRange.spanFrom({sheet:e,col:s,row:0},1,1/0),a=this.getRangeClipboardCells(t);o=o.concat(a),r.push(a.map(e=>{let[{sheet:t,col:r,row:n},o]=e;return[{sheet:t,col:i,row:n},o]}))}return r.forEach(t=>this.restoreClipboardCells(e,t.values())),o}addNamedExpression(e,t,r,n){const o=this.namedExpressions.addNamedExpression(e,r,n);this.storeNamedExpressionInCell(o.address,t),this.adjustNamedExpressionEdges(o,e,r)}restoreNamedExpression(e,t,r){const n=e.displayName;this.restoreCell(e.address,t);const o=this.namedExpressions.restoreNamedExpression(e,r);this.adjustNamedExpressionEdges(o,n,r)}changeNamedExpressionExpression(e,t,r,n){const o=this.namedExpressions.namedExpressionForScope(e,r);if(!o)throw new y.NamedExpressionDoesNotExistError(e);const s=o.copy();o.options=n;const i=this.getClipboardCell(o.address);return this.storeNamedExpressionInCell(o.address,t),[s,i]}removeNamedExpression(e,t){const r=this.namedExpressions.namedExpressionForScope(e,t);if(!r)throw new y.NamedExpressionDoesNotExistError(e);this.namedExpressions.remove(r.displayName,t);const n=this.getClipboardCell(r.address);if(void 0!==t){const t=this.namedExpressions.workbookNamedExpressionOrPlaceholder(e);this.dependencyGraph.exchangeNode(r.address,t.address)}else this.dependencyGraph.setCellEmpty(r.address);return[r,n]}ensureItIsPossibleToMoveCells(e,t,r,o){if((0,i.invalidSimpleCellAddress)(e)||!(I(t)&&I(r)||function(e,t,r){return 0===e.row&&I(t)&&r===Number.POSITIVE_INFINITY||0===e.col&&I(r)&&t===Number.POSITIVE_INFINITY}(e,t,r))||(0,i.invalidSimpleCellAddress)(o)||!this.sheetMapping.hasSheetWithId(e.sheet)||!this.sheetMapping.hasSheetWithId(o.sheet))throw new y.InvalidArgumentsError("a valid range of cells to move.");const s=n.AbsoluteCellRange.spanFrom(e,t,r),a=n.AbsoluteCellRange.spanFrom(o,t,r);if(a.exceedsSheetSizeLimits(this.maxColumns,this.maxRows))throw new y.SheetSizeLimitExceededError;if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(s))throw new y.SourceLocationHasArrayError;if(this.dependencyGraph.arrayMapping.isFormulaArrayInRange(a))throw new y.TargetLocationHasArrayError}restoreClipboardCells(e,t){const r=[];for(const[n,o]of t)if(this.restoreCell(n,o),o.type===u.ClipboardCellType.FORMULA){const{dependencies:t}=this.parser.fetchCachedResult(o.hash);r.push(...this.updateNamedExpressionsForTargetAddress(e,n,t))}return r}restoreCell(e,t){switch(t.type){case u.ClipboardCellType.VALUE:this.setValueToCell(t,e);break;case u.ClipboardCellType.FORMULA:this.setFormulaToCellFromCache(t.hash,e);break;case u.ClipboardCellType.EMPTY:this.setCellEmpty(e);break;case u.ClipboardCellType.PARSING_ERROR:this.setParsingErrorToCell(t.rawInput,t.errors,e)}}getOldContent(e){const t=this.dependencyGraph.getCell(e);if(void 0===t||t instanceof l.EmptyCellVertex)return[e,{type:u.ClipboardCellType.EMPTY}];if(t instanceof l.ValueCellVertex)return[e,Object.assign({type:u.ClipboardCellType.VALUE},t.getValues())];if(t instanceof h.FormulaVertex)return[t.getAddress(this.lazilyTransformingAstService),{type:u.ClipboardCellType.FORMULA,hash:this.parser.computeHashFromAst(t.getFormula(this.lazilyTransformingAstService))}];if(t instanceof l.ParsingErrorVertex)return[e,{type:u.ClipboardCellType.PARSING_ERROR,rawInput:t.rawInput,errors:t.errors}];throw Error("Trying to copy unsupported type")}getClipboardCell(e){const t=this.dependencyGraph.getCell(e);if(void 0===t||t instanceof l.EmptyCellVertex)return{type:u.ClipboardCellType.EMPTY};if(t instanceof l.ValueCellVertex)return Object.assign({type:u.ClipboardCellType.VALUE},t.getValues());if(t instanceof l.ArrayVertex){const r=t.getArrayCellValue(e);return r===A.EmptyValue?{type:u.ClipboardCellType.EMPTY}:{type:u.ClipboardCellType.VALUE,parsedValue:r,rawValue:t.getArrayCellRawValue(e)}}if(t instanceof l.FormulaCellVertex)return{type:u.ClipboardCellType.FORMULA,hash:this.parser.computeHashFromAst(t.getFormula(this.lazilyTransformingAstService))};if(t instanceof l.ParsingErrorVertex)return{type:u.ClipboardCellType.PARSING_ERROR,rawInput:t.rawInput,errors:t.errors};throw Error("Trying to copy unsupported type")}getSheetClipboardCells(e){const t=this.dependencyGraph.getSheetHeight(e),r=this.dependencyGraph.getSheetWidth(e),n=new Array(t);for(let o=0;o<t;o++){n[o]=new Array(r);for(let t=0;t<r;t++){const r=(0,i.simpleCellAddress)(e,t,o);n[o][t]=this.getClipboardCell(r)}}return n}getRangeClipboardCells(e){const t=[];for(const r of e.addresses(this.dependencyGraph))t.push([r,this.getClipboardCell(r)]);return t}setCellContent(e,t){const r=this.cellContentParser.parse(t),n=this.getOldContent(e);if(r instanceof a.CellContent.Formula){const t=this.parser.parse(r.formula,e),{ast:n,errors:o}=t;if(o.length>0)this.setParsingErrorToCell(r.formula,o,e);else{const r=this.arraySizePredictor.checkArraySize(n,e);this.setFormulaToCell(e,r,t)}}else r instanceof a.CellContent.Empty?this.setCellEmpty(e):this.setValueToCell({parsedValue:r.value,rawValue:t},e);return n}setSheetContent(e,t){this.clearSheet(e);for(let r=0;r<t.length;r++)for(let n=0;n<t[r].length;n++){const o=(0,i.simpleCellAddress)(e,n,r);this.setCellContent(o,t[r][n])}}setParsingErrorToCell(e,t,r){const n=this.dependencyGraph.getCellValue(r),o=new l.ParsingErrorVertex(t,e),s=this.dependencyGraph.setParsingErrorToCell(r,o);this.columnSearch.remove((0,A.getRawValue)(n),r),this.columnSearch.applyChanges(s.getChanges()),this.changes.addAll(s),this.changes.addChange(o.getCellValue(),r)}setFormulaToCell(e,t,r){let{ast:n,hasVolatileFunction:s,hasStructuralChangeFunction:i,dependencies:a}=r;const u=this.dependencyGraph.getCellValue(e),c=this.dependencyGraph.setFormulaToCell(e,n,(0,o.absolutizeDependencies)(a,e),t,s,i);this.columnSearch.remove((0,A.getRawValue)(u),e),this.columnSearch.applyChanges(c.getChanges()),this.changes.addAll(c)}setValueToCell(e,t){const r=this.dependencyGraph.getCellValue(t),n=this.dependencyGraph.setValueToCell(t,e);this.columnSearch.change((0,A.getRawValue)(r),(0,A.getRawValue)(e.parsedValue),t),this.columnSearch.applyChanges(n.getChanges().filter(e=>!(0,i.equalSimpleCellAddress)(e.address,t))),this.changes.addAll(n),this.changes.addChange(e.parsedValue,t)}setCellEmpty(e){if(this.dependencyGraph.isArrayInternalCell(e))return;const t=this.dependencyGraph.getCellValue(e),r=this.dependencyGraph.setCellEmpty(e);this.columnSearch.remove((0,A.getRawValue)(t),e),this.columnSearch.applyChanges(r.getChanges()),this.changes.addAll(r),this.changes.addChange(A.EmptyValue,e)}setFormulaToCellFromCache(e,t){const{ast:r,hasVolatileFunction:n,hasStructuralChangeFunction:s,dependencies:i}=this.parser.fetchCachedResult(e),a=(0,o.absolutizeDependencies)(i,t),[u]=new m.CleanOutOfScopeDependenciesTransformer(t.sheet).transformSingleAst(r,t);this.parser.rememberNewAst(u);const c=(0,o.filterDependenciesOutOfScope)(a),l=this.arraySizePredictor.checkArraySize(r,t);this.dependencyGraph.setFormulaToCell(t,u,c,l,n,s)}rowEffectivelyNotInSheet(e,t){return e>=this.dependencyGraph.addressMapping.getHeight(t)}getAndClearContentChanges(){const e=this.changes;return this.changes=c.ContentChanges.empty(),e}forceApplyPostponedTransformations(){this.dependencyGraph.forceApplyPostponedTransformations()}doRemoveRows(e){if(this.rowEffectivelyNotInSheet(e.rowStart,e.sheet))return;const t=[];for(const[r]of this.dependencyGraph.entriesFromRowsSpan(e))t.push({address:r,cellType:this.getClipboardCell(r)});const{affectedArrays:r,contentChanges:n}=this.dependencyGraph.removeRows(e);let o;return this.columnSearch.applyChanges(n.getChanges()),this.stats.measure(v.StatType.TRANSFORM_ASTS,()=>{const t=new E.RemoveRowsTransformer(e);t.performEagerTransformations(this.dependencyGraph,this.parser),o=this.lazilyTransformingAstService.addTransformation(t)}),this.rewriteAffectedArrays(r),{version:o,removedCells:t,rowFrom:e.rowStart,rowCount:e.numberOfRows}}doRemoveColumns(e){if(this.columnEffectivelyNotInSheet(e.columnStart,e.sheet))return;const t=[];for(const[r]of this.dependencyGraph.entriesFromColumnsSpan(e))t.push({address:r,cellType:this.getClipboardCell(r)});const{affectedArrays:r,contentChanges:n}=this.dependencyGraph.removeColumns(e);let o;return this.columnSearch.applyChanges(n.getChanges()),this.columnSearch.removeColumns(e),this.stats.measure(v.StatType.TRANSFORM_ASTS,()=>{const t=new f.RemoveColumnsTransformer(e);t.performEagerTransformations(this.dependencyGraph,this.parser),o=this.lazilyTransformingAstService.addTransformation(t)}),this.rewriteAffectedArrays(r),{version:o,removedCells:t,columnFrom:e.columnStart,columnCount:e.numberOfColumns}}doAddRows(e){if(this.rowEffectivelyNotInSheet(e.rowStart,e.sheet))return;const{affectedArrays:t}=this.dependencyGraph.addRows(e);this.stats.measure(v.StatType.TRANSFORM_ASTS,()=>{const t=new p.AddRowsTransformer(e);t.performEagerTransformations(this.dependencyGraph,this.parser),this.lazilyTransformingAstService.addTransformation(t)}),this.rewriteAffectedArrays(t)}rewriteAffectedArrays(e){for(const t of e.values()){if(t.array.size.isRef)continue;const e=t.getFormula(this.lazilyTransformingAstService),r=t.getAddress(this.lazilyTransformingAstService),n=this.parser.computeHashFromAst(e);this.setFormulaToCellFromCache(n,r)}}doAddColumns(e){if(this.columnEffectivelyNotInSheet(e.columnStart,e.sheet))return;const{affectedArrays:t,contentChanges:r}=this.dependencyGraph.addColumns(e);this.columnSearch.addColumns(e),this.columnSearch.applyChanges(r.getChanges()),this.stats.measure(v.StatType.TRANSFORM_ASTS,()=>{const t=new d.AddColumnsTransformer(e);t.performEagerTransformations(this.dependencyGraph,this.parser),this.lazilyTransformingAstService.addTransformation(t)}),this.rewriteAffectedArrays(t)}columnEffectivelyNotInSheet(e,t){return e>=this.dependencyGraph.addressMapping.getWidth(t)}adjustNamedExpressionEdges(e,t,r){if(void 0===r)return;const n=this.dependencyGraph.fetchCellOrCreateEmpty(e.address),s=this.namedExpressions.workbookNamedExpressionOrPlaceholder(t),i=this.dependencyGraph.fetchCellOrCreateEmpty(s.address);for(const t of this.dependencyGraph.graph.adjacentNodes(i))if(t instanceof l.FormulaCellVertex&&t.getAddress(this.lazilyTransformingAstService).sheet===r){const r=t.getFormula(this.lazilyTransformingAstService),s=t.getAddress(this.lazilyTransformingAstService),{dependencies:a}=this.parser.fetchCachedResultForAst(r);for(const r of(0,o.absolutizeDependencies)(a,s))r instanceof N.NamedExpressionDependency&&r.name.toLowerCase()===e.displayName.toLowerCase()&&(this.dependencyGraph.graph.removeEdge(i,t),this.dependencyGraph.graph.addEdge(n,t))}}storeNamedExpressionInCell(e,t){const r=this.cellContentParser.parse(t);if(r instanceof a.CellContent.Formula){const t=this.parser.parse(r.formula,(0,i.simpleCellAddress)(-1,0,0));if((0,R.doesContainRelativeReferences)(t.ast))throw new y.NoRelativeAddressesAllowedError;const{ast:n,hasVolatileFunction:a,hasStructuralChangeFunction:u,dependencies:c}=t;this.dependencyGraph.setFormulaToCell(e,n,(0,o.absolutizeDependencies)(c,e),s.ArraySize.scalar(),a,u)}else r instanceof a.CellContent.Empty?this.setCellEmpty(e):this.setValueToCell({parsedValue:r.value,rawValue:t},e)}updateNamedExpressionsForMovedCells(e,t,r,o){if(e.sheet===o.sheet)return[];const s=[],i=n.AbsoluteCellRange.spanFrom(o,t,r);for(const t of i.addresses(this.dependencyGraph)){const r=this.addressMapping.fetchCell(t);if(r instanceof l.FormulaCellVertex&&t.sheet!==e.sheet){const n=r.getFormula(this.lazilyTransformingAstService),{dependencies:o}=this.parser.fetchCachedResultForAst(n);s.push(...this.updateNamedExpressionsForTargetAddress(e.sheet,t,o))}}return s}updateNamedExpressionsForTargetAddress(e,t,r){if(e===t.sheet)return[];const n=[],s=this.addressMapping.fetchCell(t);for(const i of(0,o.absolutizeDependencies)(r,t)){if(!(i instanceof N.NamedExpressionDependency))continue;const r=i.name,o=this.dependencyGraph.fetchNamedExpressionVertex(r,e),a=this.namedExpressions.isExpressionInScope(r,t.sheet)?this.dependencyGraph.fetchNamedExpressionVertex(r,t.sheet):this.copyOrFetchGlobalNamedExpressionVertex(r,o,n);a!==o&&(this.dependencyGraph.graph.softRemoveEdge(o,s),this.dependencyGraph.graph.addEdge(a,s))}return n}allocateNamedExpressionAddressSpace(){this.dependencyGraph.addressMapping.addSheet(R.NamedExpressions.SHEET_FOR_WORKBOOK_EXPRESSIONS,new l.SparseStrategy(0,0))}copyOrFetchGlobalNamedExpressionVertex(e,t,r){let n=this.namedExpressions.namedExpressionForScope(e);if(void 0===n)if(n=this.namedExpressions.addNamedExpression(e),r.push(n.normalizeExpressionName()),t instanceof l.FormulaCellVertex){const e=this.parser.fetchCachedResultForAst(t.getFormula(this.lazilyTransformingAstService)),{ast:r,hasVolatileFunction:i,hasStructuralChangeFunction:a,dependencies:u}=e;this.dependencyGraph.setFormulaToCell(n.address,r,(0,o.absolutizeDependencies)(u,n.address),s.ArraySize.scalar(),i,a)}else t instanceof l.EmptyCellVertex?this.setCellEmpty(n.address):t instanceof l.ValueCellVertex&&this.setValueToCell(t.getValues(),n.address);return this.dependencyGraph.fetchCellOrCreateEmpty(n.address)}}},function(e,t,r){"use strict";t.__esModule=!0,t.AddColumnsTransformer=void 0;var n=r(74),o=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.Transformer{constructor(e){super(),this.columnsSpan=e}get sheet(){return this.columnsSpan.sheet}isIrreversible(){return!1}transformRowRangeAst(e,t){return e}transformCellRange(e,t,r){return this.transformRange(e,t,r)}transformRowRange(e,t,r){throw Error("Not implemented")}transformColumnRange(e,t,r){return this.transformRange(e,t,r)}transformCellAddress(e,t){const r=(0,n.absoluteSheetReference)(e,t);if(r!==this.columnsSpan.sheet&&t.sheet!==this.columnsSpan.sheet)return!1;const o=e.toSimpleColumnAddress(t);if(r===this.columnsSpan.sheet&&t.sheet!==this.columnsSpan.sheet)return this.columnsSpan.columnStart<=o.col&&e.shiftedByColumns(this.columnsSpan.numberOfColumns);if(t.sheet===this.columnsSpan.sheet&&r!==this.columnsSpan.sheet)return!e.isColumnAbsolute()&&(!(t.col<this.columnsSpan.columnStart)&&e.shiftedByColumns(-this.columnsSpan.numberOfColumns));if(e.isColumnAbsolute())return!(e.col<this.columnsSpan.columnStart)&&e.shiftedByColumns(this.columnsSpan.numberOfColumns);return e.toSimpleColumnAddress(t).col<this.columnsSpan.columnStart?!(t.col<this.columnsSpan.columnStart)&&e.shiftedByColumns(-this.columnsSpan.numberOfColumns):t.col<this.columnsSpan.columnStart&&e.shiftedByColumns(this.columnsSpan.numberOfColumns)}fixNodeAddress(e){return this.columnsSpan.sheet===e.sheet&&this.columnsSpan.columnStart<=e.col?Object.assign(Object.assign({},e),{col:e.col+this.columnsSpan.numberOfColumns}):e}transformRange(e,t,r){const o=this.transformCellAddress(e,r),s=this.transformCellAddress(t,r);return o===n.ErrorType.REF||s===n.ErrorType.REF?n.ErrorType.REF:!(!o&&!s)&&[o||e,s||t]}}t.AddColumnsTransformer=s},function(e,t,r){"use strict";t.__esModule=!0,t.Transformer=void 0;var n=r(74),o=r(78);t.Transformer=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{performEagerTransformations(e,t){for(const r of e.arrayFormulaNodes()){const[n,o]=this.transformSingleAst(r.getFormula(e.lazilyTransformingAstService),r.getAddress(e.lazilyTransformingAstService)),s=t.rememberNewAst(n);r.setFormula(s),r.setAddress(o)}}transformSingleAst(e,t){return[this.transformAst(e,t),this.fixNodeAddress(t)]}transformAst(e,t){switch(e.type){case o.AstNodeType.CELL_REFERENCE:return this.transformCellReferenceAst(e,t);case o.AstNodeType.CELL_RANGE:return this.transformCellRangeAst(e,t);case o.AstNodeType.COLUMN_RANGE:return this.transformColumnRangeAst(e,t);case o.AstNodeType.ROW_RANGE:return this.transformRowRangeAst(e,t);case o.AstNodeType.EMPTY:case o.AstNodeType.ERROR:case o.AstNodeType.NUMBER:case o.AstNodeType.NAMED_EXPRESSION:case o.AstNodeType.ERROR_WITH_RAW_INPUT:case o.AstNodeType.STRING:return e;case o.AstNodeType.PERCENT_OP:case o.AstNodeType.MINUS_UNARY_OP:case o.AstNodeType.PLUS_UNARY_OP:return Object.assign(Object.assign({},e),{value:this.transformAst(e.value,t)});case o.AstNodeType.FUNCTION_CALL:return Object.assign(Object.assign({},e),{procedureName:e.procedureName,args:e.args.map(e=>this.transformAst(e,t))});case o.AstNodeType.PARENTHESIS:return Object.assign(Object.assign({},e),{expression:this.transformAst(e.expression,t)});case o.AstNodeType.ARRAY:return Object.assign(Object.assign({},e),{args:e.args.map(e=>e.map(e=>this.transformAst(e,t)))});default:return Object.assign(Object.assign({},e),{left:this.transformAst(e.left,t),right:this.transformAst(e.right,t)})}}transformCellReferenceAst(e,t){const r=this.transformCellAddress(e.reference,t);return r instanceof o.CellAddress?Object.assign(Object.assign({},e),{reference:r}):r===n.ErrorType.REF?(0,o.buildCellErrorAst)(new n.CellError(n.ErrorType.REF)):e}transformCellRangeAst(e,t){const r=this.transformCellRange(e.start,e.end,t);return Array.isArray(r)?Object.assign(Object.assign({},e),{start:r[0],end:r[1]}):r===n.ErrorType.REF?(0,o.buildCellErrorAst)(new n.CellError(n.ErrorType.REF)):e}transformColumnRangeAst(e,t){const r=this.transformColumnRange(e.start,e.end,t);return Array.isArray(r)?Object.assign(Object.assign({},e),{start:r[0],end:r[1]}):r===n.ErrorType.REF?(0,o.buildCellErrorAst)(new n.CellError(n.ErrorType.REF)):e}transformRowRangeAst(e,t){const r=this.transformRowRange(e.start,e.end,t);return Array.isArray(r)?Object.assign(Object.assign({},e),{start:r[0],end:r[1]}):r===n.ErrorType.REF?(0,o.buildCellErrorAst)(new n.CellError(n.ErrorType.REF)):e}}},function(e,t,r){"use strict";t.__esModule=!0,t.AddRowsTransformer=void 0;var n=r(74),o=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.Transformer{constructor(e){super(),this.rowsSpan=e}get sheet(){return this.rowsSpan.sheet}isIrreversible(){return!1}transformColumnRangeAst(e,t){return e}transformCellRange(e,t,r){return this.transformRange(e,t,r)}transformRowRange(e,t,r){return this.transformRange(e,t,r)}transformColumnRange(e,t,r){throw Error("Not implemented")}transformCellAddress(e,t){const r=(0,n.absoluteSheetReference)(e,t);if(r!==this.rowsSpan.sheet&&t.sheet!==this.rowsSpan.sheet)return!1;const o=e.toSimpleRowAddress(t);return r===this.rowsSpan.sheet&&t.sheet!==this.rowsSpan.sheet?this.rowsSpan.rowStart<=o.row&&e.shiftedByRows(this.rowsSpan.numberOfRows):t.sheet===this.rowsSpan.sheet&&r!==this.rowsSpan.sheet?!e.isRowAbsolute()&&(!(t.row<this.rowsSpan.rowStart)&&e.shiftedByRows(-this.rowsSpan.numberOfRows)):e.isRowAbsolute()?!(e.row<this.rowsSpan.rowStart)&&e.shiftedByRows(this.rowsSpan.numberOfRows):o.row<this.rowsSpan.rowStart?!(t.row<this.rowsSpan.rowStart)&&e.shiftedByRows(-this.rowsSpan.numberOfRows):t.row<this.rowsSpan.rowStart&&e.shiftedByRows(this.rowsSpan.numberOfRows)}fixNodeAddress(e){return this.rowsSpan.sheet===e.sheet&&this.rowsSpan.rowStart<=e.row?Object.assign(Object.assign({},e),{row:e.row+this.rowsSpan.numberOfRows}):e}transformRange(e,t,r){const o=this.transformCellAddress(e,r),s=this.transformCellAddress(t,r);return o===n.ErrorType.REF||s===n.ErrorType.REF?n.ErrorType.REF:!(!o&&!s)&&[o||e,s||t]}}t.AddRowsTransformer=s},function(e,t,r){"use strict";t.__esModule=!0,t.CleanOutOfScopeDependenciesTransformer=void 0;var n=r(74),o=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.Transformer{constructor(e){super(),this.sheet=e}isIrreversible(){return!0}fixNodeAddress(e){return e}transformCellAddress(e,t){return!!e.isInvalid(t)&&n.ErrorType.REF}transformCellRange(e,t,r){return!(!e.isInvalid(r)&&!t.isInvalid(r))&&n.ErrorType.REF}transformColumnRange(e,t,r){return!(!e.isInvalid(r)&&!t.isInvalid(r))&&n.ErrorType.REF}transformRowRange(e,t,r){return!(!e.isInvalid(r)&&!t.isInvalid(r))&&n.ErrorType.REF}}t.CleanOutOfScopeDependenciesTransformer=s},function(e,t,r){"use strict";t.__esModule=!0,t.MoveCellsTransformer=t.DependentFormulaTransformer=void 0;var n=r(74),o=r(78),s=r(82),i=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends i.Transformer{constructor(e,t,r,n){super(),this.sourceRange=e,this.toRight=t,this.toBottom=r,this.toSheet=n,this.dependentFormulaTransformer=new u(e,t,r,n)}get sheet(){return this.sourceRange.sheet}isIrreversible(){return!0}transformSingleAst(e,t){if(this.sourceRange.addressInRange(t)){return[this.transformAst(e,t),this.fixNodeAddress(t)]}return this.dependentFormulaTransformer.transformSingleAst(e,t)}fixNodeAddress(e){return(0,n.simpleCellAddress)(this.toSheet,e.col+this.toRight,e.row+this.toBottom)}transformCellAddress(e,t){return this.transformAddress(e,t)}transformCellRange(e,t,r){return this.transformRange(e,t,r)}transformColumnRange(e,t,r){return this.transformRange(e,t,r)}transformRowRange(e,t,r){return this.transformRange(e,t,r)}transformAddress(e,t){const r=this.sourceRange;if(e instanceof o.CellAddress){const n=e.toSimpleCellAddress(t);if(r.addressInRange(n))return e.shiftAbsoluteDimensions(this.toRight,this.toBottom)}return e.shiftRelativeDimensions(-this.toRight,-this.toBottom)}transformRange(e,t,r){const n=this.sourceRange;if(e instanceof o.CellAddress&&t instanceof o.CellAddress){const o=e.toSimpleCellAddress(r),s=t.toSimpleCellAddress(r);if(n.addressInRange(o)&&n.addressInRange(s))return[e.shiftAbsoluteDimensions(this.toRight,this.toBottom),t.shiftAbsoluteDimensions(this.toRight,this.toBottom)]}return[e.shiftRelativeDimensions(-this.toRight,-this.toBottom),t.shiftRelativeDimensions(-this.toRight,-this.toBottom)]}}t.MoveCellsTransformer=a;class u extends i.Transformer{constructor(e,t,r,n){super(),this.sourceRange=e,this.toRight=t,this.toBottom=r,this.toSheet=n}get sheet(){return this.sourceRange.sheet}isIrreversible(){return!0}fixNodeAddress(e){return e}transformCellAddress(e,t){return!!this.shouldMove(e,t)&&e.moved(this.toSheet,this.toRight,this.toBottom)}transformCellRange(e,t,r){return this.transformRange(e,t,r)}transformColumnRange(e,t,r){return this.transformRange(e,t,r)}transformRowRange(e,t,r){return this.transformRange(e,t,r)}shouldMove(e,t){return e instanceof o.CellAddress?this.sourceRange.addressInRange(e.toSimpleCellAddress(t)):e instanceof s.RowAddress?this.sourceRange.rowInRange(e.toSimpleRowAddress(t))&&!this.sourceRange.isFinite():this.sourceRange.columnInRange(e.toSimpleColumnAddress(t))&&!this.sourceRange.isFinite()}transformRange(e,t,r){const n=this.transformCellAddress(e,r),o=this.transformCellAddress(t,r);return!(!n||!o)&&[n,o]}}t.DependentFormulaTransformer=u},function(e,t,r){"use strict";t.__esModule=!0,t.RemoveColumnsTransformer=void 0;var n=r(74),o=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.Transformer{constructor(e){super(),this.columnsSpan=e}get sheet(){return this.columnsSpan.sheet}isIrreversible(){return!0}transformRowRangeAst(e,t){return e}transformCellRange(e,t,r){return this.transformRange(e,t,r)}transformRowRange(e,t,r){throw Error("Not implemented")}transformColumnRange(e,t,r){return this.transformRange(e,t,r)}transformCellAddress(e,t){const r=(0,n.absoluteSheetReference)(e,t);if(this.columnsSpan.sheet!==t.sheet&&this.columnsSpan.sheet!==r)return!1;if(this.columnsSpan.sheet!==t.sheet&&this.columnsSpan.sheet===r){const r=e.toSimpleColumnAddress(t);if(r.col<this.columnsSpan.columnStart)return!1;if(r.col>this.columnsSpan.columnEnd)return e.shiftedByColumns(-this.columnsSpan.numberOfColumns)}if(this.columnsSpan.sheet===t.sheet&&this.columnsSpan.sheet!==r){if(e.isColumnAbsolute())return!1;if(t.col<this.columnsSpan.columnStart)return!1;if(t.col>this.columnsSpan.columnEnd)return e.shiftedByColumns(this.columnsSpan.numberOfColumns)}if(this.columnsSpan.sheet===t.sheet&&this.columnsSpan.sheet===r)if(e.isColumnAbsolute()){if(e.col<this.columnsSpan.columnStart)return!1;if(e.col>this.columnsSpan.columnEnd)return e.shiftedByColumns(-this.columnsSpan.numberOfColumns)}else{const r=e.toSimpleColumnAddress(t);if(r.col<this.columnsSpan.columnStart){if(t.col<this.columnsSpan.columnStart)return!1;if(t.col>this.columnsSpan.columnEnd)return e.shiftedByColumns(this.columnsSpan.numberOfColumns)}else if(r.col>this.columnsSpan.columnEnd){if(t.col<this.columnsSpan.columnStart)return e.shiftedByColumns(-this.columnsSpan.numberOfColumns);if(t.col>this.columnsSpan.columnEnd)return!1}}return n.ErrorType.REF}fixNodeAddress(e){return this.columnsSpan.sheet===e.sheet&&this.columnsSpan.columnStart<=e.col?Object.assign(Object.assign({},e),{col:e.col-this.columnsSpan.numberOfColumns}):e}transformRange(e,t,r){const o=(0,n.absoluteSheetReference)(e,r);let s=e,i=t;if(this.columnsSpan.sheet===o){const o=e.toSimpleColumnAddress(r),a=t.toSimpleColumnAddress(r);if(this.columnsSpan.columnStart<=o.col&&this.columnsSpan.columnEnd>=a.col)return n.ErrorType.REF;o.col>=this.columnsSpan.columnStart&&o.col<=this.columnsSpan.columnEnd&&(s=e.shiftedByColumns(this.columnsSpan.columnEnd-o.col+1)),a.col>=this.columnsSpan.columnStart&&a.col<=this.columnsSpan.columnEnd&&(i=t.shiftedByColumns(-(a.col-this.columnsSpan.columnStart+1)))}const a=this.transformCellAddress(s,r),u=this.transformCellAddress(i,r);if(!1===a&&!1===u)return[s,i];if(a===n.ErrorType.REF||u===n.ErrorType.REF)throw Error("Cannot happen");return[a||s,u||i]}}t.RemoveColumnsTransformer=s},function(e,t,r){"use strict";t.__esModule=!0,t.RemoveRowsTransformer=void 0;var n=r(74),o=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.Transformer{constructor(e){super(),this.rowsSpan=e}get sheet(){return this.rowsSpan.sheet}isIrreversible(){return!0}transformColumnRangeAst(e,t){return e}transformCellAddress(e,t){const r=(0,n.absoluteSheetReference)(e,t);if(this.rowsSpan.sheet!==t.sheet&&this.rowsSpan.sheet!==r)return!1;if(this.rowsSpan.sheet!==t.sheet&&this.rowsSpan.sheet===r){const r=e.toSimpleRowAddress(t);if(r.row<this.rowsSpan.rowStart)return!1;if(r.row>this.rowsSpan.rowEnd)return e.shiftedByRows(-this.rowsSpan.numberOfRows)}if(this.rowsSpan.sheet===t.sheet&&this.rowsSpan.sheet!==r){if(e.isRowAbsolute())return!1;if(t.row<this.rowsSpan.rowStart)return!1;if(t.row>this.rowsSpan.rowEnd)return e.shiftedByRows(this.rowsSpan.numberOfRows)}if(this.rowsSpan.sheet===t.sheet&&this.rowsSpan.sheet===r)if(e.isRowAbsolute()){if(e.row<this.rowsSpan.rowStart)return!1;if(e.row>this.rowsSpan.rowEnd)return e.shiftedByRows(-this.rowsSpan.numberOfRows)}else{const r=e.toSimpleRowAddress(t);if(r.row<this.rowsSpan.rowStart){if(t.row<this.rowsSpan.rowStart)return!1;if(t.row>this.rowsSpan.rowEnd)return e.shiftedByRows(this.rowsSpan.numberOfRows)}else if(r.row>this.rowsSpan.rowEnd){if(t.row<this.rowsSpan.rowStart)return e.shiftedByRows(-this.rowsSpan.numberOfRows);if(t.row>this.rowsSpan.rowEnd)return!1}}return n.ErrorType.REF}transformCellRange(e,t,r){return this.transformRange(e,t,r)}transformRowRange(e,t,r){return this.transformRange(e,t,r)}transformColumnRange(e,t,r){throw Error("Not implemented")}fixNodeAddress(e){return this.rowsSpan.sheet===e.sheet&&this.rowsSpan.rowStart<=e.row?Object.assign(Object.assign({},e),{row:e.row-this.rowsSpan.numberOfRows}):e}transformRange(e,t,r){const o=(0,n.absoluteSheetReference)(e,r);let s=e,i=t;if(this.rowsSpan.sheet===o){const o=e.toSimpleRowAddress(r),a=t.toSimpleRowAddress(r);if(this.rowsSpan.rowStart<=o.row&&this.rowsSpan.rowEnd>=a.row)return n.ErrorType.REF;o.row>=this.rowsSpan.rowStart&&o.row<=this.rowsSpan.rowEnd&&(s=e.shiftedByRows(this.rowsSpan.rowEnd-o.row+1)),a.row>=this.rowsSpan.rowStart&&a.row<=this.rowsSpan.rowEnd&&(i=t.shiftedByRows(-(a.row-this.rowsSpan.rowStart+1)))}const a=this.transformCellAddress(s,r),u=this.transformCellAddress(i,r);return!1===a&&!1===u?[s,i]:a===n.ErrorType.REF||u===n.ErrorType.REF?n.ErrorType.REF:[a||s,u||i]}}t.RemoveRowsTransformer=s},function(e,t,r){"use strict";t.__esModule=!0,t.RemoveSheetTransformer=void 0;var n=r(74),o=r(187);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.Transformer{constructor(e){super(),this.sheet=e}isIrreversible(){return!0}performEagerTransformations(e,t){for(const t of e.arrayFormulaNodes()){const[r]=this.transformSingleAst(t.getFormula(e.lazilyTransformingAstService),t.getAddress(e.lazilyTransformingAstService));t.setFormula(r)}}fixNodeAddress(e){return e}transformCellAddress(e,t){return this.transformAddress(e)}transformCellRange(e,t,r){return this.transformAddress(e)}transformColumnRange(e,t,r){return this.transformAddress(e)}transformRowRange(e,t,r){return this.transformAddress(e)}transformAddress(e){return e.sheet===this.sheet&&n.ErrorType.REF}}t.RemoveSheetTransformer=s},function(e,t,r){"use strict";t.__esModule=!0,t.findBoundaries=function(e){let t=0,r=0,n=0;for(let o=0;o<e.length;o++){let s=0;for(let t=0;t<e[o].length;t++){const r=e[o][t];null!=r&&(s=t+1,++n)}t=Math.max(t,s),s>0&&(r=o+1)}const o=t*r;return{height:r,width:t,fill:0===o?0:n/o}},t.validateAsSheet=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e){if(!Array.isArray(e))throw new n.InvalidArgumentsError("an array of arrays.");for(let t=0;t<e.length;t++)if(!Array.isArray(e[t]))throw new n.InvalidArgumentsError("an array of arrays.")};var n=r(148)},function(e,t,r){"use strict";t.__esModule=!0,t.UndoRedo=t.SetSheetContentUndoEntry=t.SetRowOrderUndoEntry=t.SetColumnOrderUndoEntry=t.SetCellContentsUndoEntry=t.RenameSheetUndoEntry=t.RemoveSheetUndoEntry=t.RemoveRowsUndoEntry=t.RemoveNamedExpressionUndoEntry=t.RemoveColumnsUndoEntry=t.PasteUndoEntry=t.MoveRowsUndoEntry=t.MoveColumnsUndoEntry=t.MoveCellsUndoEntry=t.ClearSheetUndoEntry=t.ChangeNamedExpressionUndoEntry=t.BatchUndoEntry=t.BaseUndoEntry=t.AddSheetUndoEntry=t.AddRowsUndoEntry=t.AddNamedExpressionUndoEntry=t.AddColumnsUndoEntry=void 0,r(2);var n=r(74),o=r(185);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s{}t.BaseUndoEntry=s;t.RemoveRowsUndoEntry=class extends s{constructor(e,t){super(),this.command=e,this.rowsRemovals=t}doUndo(e){e.undoRemoveRows(this)}doRedo(e){e.redoRemoveRows(this)}};t.MoveCellsUndoEntry=class extends s{constructor(e,t,r,n,o,s,i){super(),this.sourceLeftCorner=e,this.width=t,this.height=r,this.destinationLeftCorner=n,this.overwrittenCellsData=o,this.addedGlobalNamedExpressions=s,this.version=i}doUndo(e){e.undoMoveCells(this)}doRedo(e){e.redoMoveCells(this)}};t.AddRowsUndoEntry=class extends s{constructor(e){super(),this.command=e}doUndo(e){e.undoAddRows(this)}doRedo(e){e.redoAddRows(this)}};t.SetRowOrderUndoEntry=class extends s{constructor(e,t,r){super(),this.sheetId=e,this.rowMapping=t,this.oldContent=r}doUndo(e){e.undoSetRowOrder(this)}doRedo(e){e.redoSetRowOrder(this)}};t.SetColumnOrderUndoEntry=class extends s{constructor(e,t,r){super(),this.sheetId=e,this.columnMapping=t,this.oldContent=r}doUndo(e){e.undoSetColumnOrder(this)}doRedo(e){e.redoSetColumnOrder(this)}};t.SetSheetContentUndoEntry=class extends s{constructor(e,t,r){super(),this.sheetId=e,this.oldSheetContent=t,this.newSheetContent=r}doUndo(e){e.undoSetSheetContent(this)}doRedo(e){e.redoSetSheetContent(this)}};t.MoveRowsUndoEntry=class extends s{constructor(e,t,r,n,o){super(),this.sheet=e,this.startRow=t,this.numberOfRows=r,this.targetRow=n,this.version=o,this.undoStart=this.startRow<this.targetRow?this.targetRow-this.numberOfRows:this.targetRow,this.undoEnd=this.startRow>this.targetRow?this.startRow+this.numberOfRows:this.startRow}doUndo(e){e.undoMoveRows(this)}doRedo(e){e.redoMoveRows(this)}};t.MoveColumnsUndoEntry=class extends s{constructor(e,t,r,n,o){super(),this.sheet=e,this.startColumn=t,this.numberOfColumns=r,this.targetColumn=n,this.version=o,this.undoStart=this.startColumn<this.targetColumn?this.targetColumn-this.numberOfColumns:this.targetColumn,this.undoEnd=this.startColumn>this.targetColumn?this.startColumn+this.numberOfColumns:this.startColumn}doUndo(e){e.undoMoveColumns(this)}doRedo(e){e.redoMoveColumns(this)}};t.AddColumnsUndoEntry=class extends s{constructor(e){super(),this.command=e}doUndo(e){e.undoAddColumns(this)}doRedo(e){e.redoAddColumns(this)}};t.RemoveColumnsUndoEntry=class extends s{constructor(e,t){super(),this.command=e,this.columnsRemovals=t}doUndo(e){e.undoRemoveColumns(this)}doRedo(e){e.redoRemoveColumns(this)}};t.AddSheetUndoEntry=class extends s{constructor(e){super(),this.sheetName=e}doUndo(e){e.undoAddSheet(this)}doRedo(e){e.redoAddSheet(this)}};t.RemoveSheetUndoEntry=class extends s{constructor(e,t,r,n,o){super(),this.sheetName=e,this.sheetId=t,this.oldSheetContent=r,this.scopedNamedExpressions=n,this.version=o}doUndo(e){e.undoRemoveSheet(this)}doRedo(e){e.redoRemoveSheet(this)}};t.RenameSheetUndoEntry=class extends s{constructor(e,t,r){super(),this.sheetId=e,this.oldName=t,this.newName=r}doUndo(e){e.undoRenameSheet(this)}doRedo(e){e.redoRenameSheet(this)}};t.ClearSheetUndoEntry=class extends s{constructor(e,t){super(),this.sheetId=e,this.oldSheetContent=t}doUndo(e){e.undoClearSheet(this)}doRedo(e){e.redoClearSheet(this)}};t.SetCellContentsUndoEntry=class extends s{constructor(e){super(),this.cellContents=e}doUndo(e){e.undoSetCellContents(this)}doRedo(e){e.redoSetCellContents(this)}};t.PasteUndoEntry=class extends s{constructor(e,t,r,n){super(),this.targetLeftCorner=e,this.oldContent=t,this.newContent=r,this.addedGlobalNamedExpressions=n}doUndo(e){e.undoPaste(this)}doRedo(e){e.redoPaste(this)}};t.AddNamedExpressionUndoEntry=class extends s{constructor(e,t,r,n){super(),this.name=e,this.newContent=t,this.scope=r,this.options=n}doUndo(e){e.undoAddNamedExpression(this)}doRedo(e){e.redoAddNamedExpression(this)}};t.RemoveNamedExpressionUndoEntry=class extends s{constructor(e,t,r){super(),this.namedExpression=e,this.content=t,this.scope=r}doUndo(e){e.undoRemoveNamedExpression(this)}doRedo(e){e.redoRemoveNamedExpression(this)}};t.ChangeNamedExpressionUndoEntry=class extends s{constructor(e,t,r,n,o){super(),this.namedExpression=e,this.newContent=t,this.oldContent=r,this.scope=n,this.options=o}doUndo(e){e.undoChangeNamedExpression(this)}doRedo(e){e.redoChangeNamedExpression(this)}};class i extends s{constructor(){super(...arguments),this.operations=[]}add(e){this.operations.push(e)}*reversedOperations(){for(let e=this.operations.length-1;e>=0;e--)yield this.operations[e]}doUndo(e){e.undoBatch(this)}doRedo(e){e.redoBatch(this)}}t.BatchUndoEntry=i;t.UndoRedo=class{constructor(e,t){this.operations=t,this.oldData=new Map,this.undoStack=[],this.redoStack=[],this.undoLimit=e.undoLimit}saveOperation(e){void 0!==this.batchUndoEntry?this.batchUndoEntry.add(e):this.addUndoEntry(e)}beginBatchMode(){this.batchUndoEntry=new i}commitBatchMode(){if(void 0===this.batchUndoEntry)throw Error("Batch mode wasn't started");this.addUndoEntry(this.batchUndoEntry),this.batchUndoEntry=void 0}storeDataForVersion(e,t,r){this.oldData.has(e)||this.oldData.set(e,[]);this.oldData.get(e).push([t,r])}clearRedoStack(){this.redoStack=[]}clearUndoStack(){this.undoStack=[]}isUndoStackEmpty(){return 0===this.undoStack.length}isRedoStackEmpty(){return 0===this.redoStack.length}undo(){const e=this.undoStack.pop();if(!e)throw Error("Attempted to undo without operation on stack");this.undoEntry(e),this.redoStack.push(e)}undoBatch(e){for(const t of e.reversedOperations())this.undoEntry(t)}undoRemoveRows(e){this.operations.forceApplyPostponedTransformations();const{command:{sheet:t},rowsRemovals:r}=e;for(let e=r.length-1;e>=0;--e){const n=r[e];this.operations.addRows(new o.AddRowsCommand(t,[[n.rowFrom,n.rowCount]]));for(const{address:e,cellType:t}of n.removedCells)this.operations.restoreCell(e,t);this.restoreOldDataFromVersion(n.version-1)}}undoRemoveColumns(e){this.operations.forceApplyPostponedTransformations();const{command:{sheet:t},columnsRemovals:r}=e;for(let e=r.length-1;e>=0;--e){const n=r[e];this.operations.addColumns(new o.AddColumnsCommand(t,[[n.columnFrom,n.columnCount]]));for(const{address:e,cellType:t}of n.removedCells)this.operations.restoreCell(e,t);this.restoreOldDataFromVersion(n.version-1)}}undoAddRows(e){const t=e.command.rowsSpans();for(let r=t.length-1;r>=0;--r){const n=t[r];this.operations.removeRows(new o.RemoveRowsCommand(e.command.sheet,[[n.rowStart,n.numberOfRows]]))}}undoAddColumns(e){const t=e.command.columnsSpans();for(let r=t.length-1;r>=0;--r){const n=t[r];this.operations.removeColumns(new o.RemoveColumnsCommand(e.command.sheet,[[n.columnStart,n.numberOfColumns]]))}}undoSetCellContents(e){for(const t of e.cellContents){const e=t.address,[r,o]=t.oldContent;(0,n.equalSimpleCellAddress)(e,r)||this.operations.setCellEmpty(e),this.operations.restoreCell(r,o)}}undoPaste(e){this.restoreOperationOldContent(e.oldContent);for(const t of e.addedGlobalNamedExpressions)this.operations.removeNamedExpression(t)}undoMoveRows(e){const{sheet:t}=e;this.operations.moveRows(t,e.undoStart,e.numberOfRows,e.undoEnd),this.restoreOldDataFromVersion(e.version-1)}undoMoveColumns(e){const{sheet:t}=e;this.operations.moveColumns(t,e.undoStart,e.numberOfColumns,e.undoEnd),this.restoreOldDataFromVersion(e.version-1)}undoMoveCells(e){this.operations.forceApplyPostponedTransformations(),this.operations.moveCells(e.destinationLeftCorner,e.width,e.height,e.sourceLeftCorner),this.restoreOperationOldContent(e.overwrittenCellsData),this.restoreOldDataFromVersion(e.version-1);for(const t of e.addedGlobalNamedExpressions)this.operations.removeNamedExpression(t)}undoAddSheet(e){const{sheetName:t}=e;this.operations.removeSheetByName(t)}undoRemoveSheet(e){this.operations.forceApplyPostponedTransformations();const{oldSheetContent:t,sheetId:r}=e;this.operations.addSheet(e.sheetName);for(let e=0;e<t.length;e++){const o=t[e];for(let t=0;t<o.length;t++){const s=o[t],i=(0,n.simpleCellAddress)(r,t,e);this.operations.restoreCell(i,s)}}for(const[t,n]of e.scopedNamedExpressions)this.operations.restoreNamedExpression(t,n,r);this.restoreOldDataFromVersion(e.version-1)}undoRenameSheet(e){this.operations.renameSheet(e.sheetId,e.oldName)}undoClearSheet(e){const{oldSheetContent:t,sheetId:r}=e;for(let e=0;e<t.length;e++){const o=t[e];for(let t=0;t<o.length;t++){const s=o[t],i=(0,n.simpleCellAddress)(r,t,e);this.operations.restoreCell(i,s)}}}undoSetSheetContent(e){const{oldSheetContent:t,sheetId:r}=e;this.operations.clearSheet(r);for(let e=0;e<t.length;e++){const o=t[e];for(let t=0;t<o.length;t++){const s=o[t],i=(0,n.simpleCellAddress)(r,t,e);this.operations.restoreCell(i,s)}}}undoAddNamedExpression(e){this.operations.removeNamedExpression(e.name,e.scope)}undoRemoveNamedExpression(e){this.operations.restoreNamedExpression(e.namedExpression,e.content,e.scope)}undoChangeNamedExpression(e){this.operations.restoreNamedExpression(e.namedExpression,e.oldContent,e.scope)}undoSetRowOrder(e){this.restoreOperationOldContent(e.oldContent)}undoSetColumnOrder(e){this.restoreOperationOldContent(e.oldContent)}redo(){const e=this.redoStack.pop();if(!e)throw Error("Attempted to redo without operation on stack");this.redoEntry(e),this.undoStack.push(e)}redoBatch(e){for(const t of e.operations)this.redoEntry(t)}redoRemoveRows(e){this.operations.removeRows(e.command)}redoMoveCells(e){this.operations.moveCells(e.sourceLeftCorner,e.width,e.height,e.destinationLeftCorner)}redoRemoveColumns(e){this.operations.removeColumns(e.command)}redoPaste(e){const{targetLeftCorner:t,newContent:r}=e,o=r.length,s=r[0].length;for(let e=0;e<o;++e)for(let o=0;o<s;++o){const s=(0,n.simpleCellAddress)(t.sheet,t.col+o,t.row+e);this.operations.restoreCell(s,r[e][o])}}redoSetCellContents(e){for(const t of e.cellContents)this.operations.setCellContent(t.address,t.newContent)}redoAddRows(e){this.operations.addRows(e.command)}redoAddColumns(e){this.operations.addColumns(e.command)}redoRemoveSheet(e){this.operations.removeSheetByName(e.sheetName)}redoAddSheet(e){this.operations.addSheet(e.sheetName)}redoRenameSheet(e){this.operations.renameSheet(e.sheetId,e.newName)}redoMoveRows(e){this.operations.moveRows(e.sheet,e.startRow,e.numberOfRows,e.targetRow)}redoMoveColumns(e){this.operations.moveColumns(e.sheet,e.startColumn,e.numberOfColumns,e.targetColumn)}redoClearSheet(e){this.operations.clearSheet(e.sheetId)}redoSetSheetContent(e){const{sheetId:t,newSheetContent:r}=e;this.operations.setSheetContent(t,r)}redoAddNamedExpression(e){this.operations.addNamedExpression(e.name,e.newContent,e.scope,e.options)}redoRemoveNamedExpression(e){this.operations.removeNamedExpression(e.namedExpression.displayName,e.scope)}redoChangeNamedExpression(e){this.operations.changeNamedExpressionExpression(e.namedExpression.displayName,e.newContent,e.scope,e.options)}redoSetRowOrder(e){this.operations.setRowOrder(e.sheetId,e.rowMapping)}redoSetColumnOrder(e){this.operations.setColumnOrder(e.sheetId,e.columnMapping)}addUndoEntry(e){this.undoStack.push(e),this.undoStack.splice(0,Math.max(0,this.undoStack.length-this.undoLimit))}undoEntry(e){e.doUndo(this)}restoreOperationOldContent(e){for(const[t,r]of e)this.operations.restoreCell(t,r)}redoEntry(e){e.doRedo(this)}restoreOldDataFromVersion(e){const t=this.oldData.get(e)||[];for(const e of t){const[t,r]=e;this.operations.setFormulaToCellFromCache(r,t)}}}},function(e,t,r){"use strict";t.__esModule=!0,t.Evaluator=void 0,r(2);var n=r(73),o=r(77),s=r(74),i=r(139),a=r(75),u=r(151),c=r(164),l=r(141),h=r(140),d=r(143);t.Evaluator=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r,n,o,s){this.config=e,this.stats=t,this.interpreter=r,this.lazilyTransformingAstService=n,this.dependencyGraph=o,this.columnSearch=s}run(){this.stats.start(d.StatType.TOP_SORT);const{sorted:e,cycled:t}=this.dependencyGraph.topSortWithScc();this.stats.end(d.StatType.TOP_SORT),this.stats.measure(d.StatType.EVALUATION,()=>{this.recomputeFormulas(t,e)})}partialRun(e){const t=i.ContentChanges.empty();return this.stats.measure(d.StatType.EVALUATION,()=>{this.dependencyGraph.graph.getTopSortedWithSccSubgraphFrom(e,e=>{if(e instanceof u.FormulaVertex){const r=e.isComputed()?e.getCellValue():void 0,n=this.recomputeFormulaVertexValue(e);if(n!==r){const o=e.getAddress(this.lazilyTransformingAstService);return t.addChange(n,o),this.columnSearch.change((0,l.getRawValue)(r),(0,l.getRawValue)(n),o),!0}return!1}return!(e instanceof a.RangeVertex)||(e.clearCache(),!0)},e=>{if(e instanceof a.RangeVertex)e.clearCache();else if(e instanceof u.FormulaVertex){const r=e.getAddress(this.lazilyTransformingAstService);this.columnSearch.remove((0,l.getRawValue)(e.valueOrUndef()),r);const n=new s.CellError(s.ErrorType.CYCLE,void 0,e);e.setCellValue(n),t.addChange(n,r)}})}),t}runAndForget(e,t,r){const s=[];for(const e of(0,o.absolutizeDependencies)(r,t))if(e instanceof n.AbsoluteCellRange){const t=e;if(void 0===this.dependencyGraph.getRange(t.start,t.end)){const e=new a.RangeVertex(t);this.dependencyGraph.rangeMapping.setRange(e),s.push(e)}}const i=this.evaluateAstToCellValue(e,new c.InterpreterState(t,this.config.useArrayArithmetic));return s.forEach(e=>{this.dependencyGraph.rangeMapping.removeRange(e)}),i}recomputeFormulas(e,t){e.forEach(e=>{e instanceof u.FormulaVertex&&e.setCellValue(new s.CellError(s.ErrorType.CYCLE,void 0,e))}),t.forEach(e=>{if(e instanceof u.FormulaVertex){const t=this.recomputeFormulaVertexValue(e),r=e.getAddress(this.lazilyTransformingAstService);this.columnSearch.add((0,l.getRawValue)(t),r)}else e instanceof a.RangeVertex&&e.clearCache()})}recomputeFormulaVertexValue(e){const t=e.getAddress(this.lazilyTransformingAstService);if(e instanceof a.ArrayVertex&&(e.array.size.isRef||!this.dependencyGraph.isThereSpaceForArray(e)))return e.setNoSpace();{const r=e.getFormula(this.lazilyTransformingAstService),n=this.evaluateAstToCellValue(r,new c.InterpreterState(t,this.config.useArrayArithmetic,e));return e.setCellValue(n)}}evaluateAstToCellValue(e,t){const r=this.interpreter.evaluateAst(e,t);return r instanceof h.SimpleRangeValue?r:r===l.EmptyValue&&this.config.evaluateNullToZero?0:r}}},function(e,t,r){"use strict";t.__esModule=!0,t.Exporter=t.ExportedNamedExpressionChange=t.ExportedCellChange=void 0,r(2);var n=r(74),o=r(169),s=r(132),i=r(141),a=r(140),u=r(184),c=r(79);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class l{constructor(e,t){this.address=e,this.newValue=t}get col(){return this.address.col}get row(){return this.address.row}get sheet(){return this.address.sheet}get value(){return this.newValue}}t.ExportedCellChange=l;class h{constructor(e,t){this.name=e,this.newValue=t}}t.ExportedNamedExpressionChange=h;t.Exporter=class{constructor(e,t,r,n){this.config=e,this.namedExpressions=t,this.sheetIndexMapping=r,this.lazilyTransformingService=n}exportChange(e){const t=e.value,r=e.address;if(r.sheet===u.NamedExpressions.SHEET_FOR_WORKBOOK_EXPRESSIONS){const e=this.namedExpressions.namedExpressionInAddress(r.row);if(!e)throw new Error("Missing named expression");return new h(e.displayName,this.exportScalarOrRange(t))}if(t instanceof a.SimpleRangeValue){const e=[];for(const[n,o]of t.entriesFromTopLeftCorner(r))e.push(new l(o,this.exportValue(n)));return e}return new l(r,this.exportValue(t))}exportValue(e){return e instanceof a.SimpleRangeValue?this.detailedError(new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.ScalarExpected)):this.config.smartRounding&&(0,i.isExtendedNumber)(e)?this.cellValueRounding((0,i.getRawValue)(e)):e instanceof n.CellError?this.detailedError(e):e===i.EmptyValue?null:(0,i.getRawValue)(e)}exportScalarOrRange(e){return e instanceof a.SimpleRangeValue?e.rawData().map(e=>e.map(e=>this.exportValue(e))):this.exportValue(e)}detailedError(e){var t,r;let n=void 0;const s=null===(t=e.root)||void 0===t?void 0:t.getAddress(this.lazilyTransformingService);return void 0!==s&&(n=s.sheet===u.NamedExpressions.SHEET_FOR_WORKBOOK_EXPRESSIONS?null===(r=this.namedExpressions.namedExpressionInAddress(s.row))||void 0===r?void 0:r.displayName:(0,c.simpleCellAddressToString)(this.sheetIndexMapping,s,-1)),new o.DetailedCellError(e,this.config.translationPackage.getErrorTranslation(e.type),n)}cellValueRounding(e){if(0===e)return e;const t=Math.floor(Math.log10(Math.abs(e))),r=Math.pow(10,this.config.precisionRounding-t);return e<0?-Math.round(-e*r)/r:Math.round(e*r)/r}}},function(e,t,r){"use strict";t.__esModule=!0,t.SimpleStrategy=t.GraphBuilder=void 0;var n=r(77),o=r(1),s=r(74),i=r(181),a=r(75),u=r(141),c=r(143);t.GraphBuilder=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r,n,o,s){this.dependencyGraph=e,this.columnSearch=t,this.parser=r,this.cellContentParser=n,this.stats=o,this.arraySizePredictor=s,this.buildStrategy=new l(e,t,r,o,n,s)}buildGraph(e,t){const r=t.measure(c.StatType.COLLECT_DEPENDENCIES,()=>this.buildStrategy.run(e));this.dependencyGraph.getAndClearContentChanges(),t.measure(c.StatType.PROCESS_DEPENDENCIES,()=>this.processDependencies(r))}processDependencies(e){e.forEach((e,t)=>{this.dependencyGraph.processCellDependencies(e,t)})}};class l{constructor(e,t,r,n,o,s){this.dependencyGraph=e,this.columnIndex=t,this.parser=r,this.stats=n,this.cellContentParser=o,this.arraySizePredictor=s}run(e){const t=new Map;for(const r in e){const l=this.dependencyGraph.getSheetId(r),h=e[r];for(let e=0;e<h.length;++e){const r=h[e];for(let h=0;h<r.length;++h){const d=r[h],p=(0,s.simpleCellAddress)(l,h,e),m=this.cellContentParser.parse(d);if(m instanceof i.CellContent.Formula){const e=this.stats.measure(c.StatType.PARSER,()=>this.parser.parse(m.formula,p));if(e.errors.length>0){this.shrinkArrayIfNeeded(p);const t=new a.ParsingErrorVertex(e.errors,m.formula);this.dependencyGraph.addVertex(p,t)}else{this.shrinkArrayIfNeeded(p);const r=this.arraySizePredictor.checkArraySize(e.ast,p);if(r.isScalar()){const r=new a.FormulaCellVertex(e.ast,p,0);t.set(r,(0,n.absolutizeDependencies)(e.dependencies,p)),this.dependencyGraph.addVertex(p,r),e.hasVolatileFunction&&this.dependencyGraph.markAsVolatile(r),e.hasStructuralChangeFunction&&this.dependencyGraph.markAsDependentOnStructureChange(r)}else{const s=new a.ArrayVertex(e.ast,p,new o.ArraySize(r.width,r.height));t.set(s,(0,n.absolutizeDependencies)(e.dependencies,p)),this.dependencyGraph.addArrayVertex(p,s)}}}else if(m instanceof i.CellContent.Empty);else{this.shrinkArrayIfNeeded(p);const e=new a.ValueCellVertex(m.value,d);this.columnIndex.add((0,u.getRawValue)(m.value),p),this.dependencyGraph.addVertex(p,e)}}}}return t}shrinkArrayIfNeeded(e){const t=this.dependencyGraph.getCell(e);t instanceof a.ArrayVertex&&this.dependencyGraph.shrinkArrayToCorner(t)}}t.SimpleStrategy=l},function(e,t,r){"use strict";t.__esModule=!0,t.FunctionRegistry=void 0,r(2);var n=r(148),o=r(179),s=r(200);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function i(e,t){var r;let o=t.implementedFunctions[e];const s=null===(r=t.aliases)||void 0===r?void 0:r[e];if(void 0!==s){if(void 0!==o)throw new n.AliasAlreadyExisting(e,t.name);o=t.implementedFunctions[s]}if(void 0===o)throw n.FunctionPluginValidationError.functionNotDeclaredInPlugin(e,t.name);return o}class a{constructor(e){if(this.config=e,this.functions=new Map,this.arraySizeFunctions=new Map,this.volatileFunctions=new Set,this.arrayFunctions=new Set,this.structuralChangeFunctions=new Set,this.functionsWhichDoesNotNeedArgumentsToBeComputed=new Set,this.functionsMetadata=new Map,this.doesFunctionNeedArgumentToBeComputed=e=>this.functionsWhichDoesNotNeedArgumentsToBeComputed.has(e),this.isFunctionVolatile=e=>this.volatileFunctions.has(e),this.isArrayFunction=e=>this.arrayFunctions.has(e),this.isFunctionDependentOnSheetStructureChange=e=>this.structuralChangeFunctions.has(e),e.functionPlugins.length>0){this.instancePlugins=new Map;for(const t of e.functionPlugins)a.loadPluginFunctions(t,this.instancePlugins)}else this.instancePlugins=new Map(a.plugins);for(const[e,t]of a.protectedFunctions())a.loadFunctionUnprotected(t,e,this.instancePlugins);for(const[e,t]of this.instancePlugins.entries())this.categorizeFunction(e,i(e,t))}static registerFunctionPlugin(e,t){this.loadPluginFunctions(e,this.plugins),void 0!==t&&this.loadTranslations(t)}static registerFunction(e,t,r){this.loadPluginFunction(t,e,this.plugins),void 0!==r&&this.loadTranslations(r)}static unregisterFunction(e){if(this.functionIsProtected(e))throw n.ProtectedFunctionError.cannotUnregisterFunctionWithId(e);this.plugins.delete(e)}static unregisterFunctionPlugin(e){for(const t of this.protectedPlugins())if(t===e)throw n.ProtectedFunctionError.cannotUnregisterProtectedPlugin();for(const[t,r]of this.plugins.entries())r===e&&this.plugins.delete(t)}static unregisterAll(){this.plugins.clear()}static getRegisteredFunctionIds(){return[...Array.from(this.plugins.keys()),...Array.from(this._protectedPlugins.keys())]}static getPlugins(){return Array.from(new Set(this.plugins.values()).values())}static getFunctionPlugin(e){return this.functionIsProtected(e)?void 0:this.plugins.get(e)}static functionIsProtected(e){return this._protectedPlugins.has(e)}static loadTranslations(e){const t=new Set(o.HyperFormula.getRegisteredLanguagesCodes());Object.keys(e).forEach(r=>{t.has(r)&&o.HyperFormula.getLanguage(r).extendFunctions(e[r])})}static loadPluginFunctions(e,t){Object.keys(e.implementedFunctions).forEach(r=>{this.loadPluginFunction(e,r,t)}),void 0!==e.aliases&&Object.keys(e.aliases).forEach(r=>{this.loadPluginFunction(e,r,t)})}static loadPluginFunction(e,t,r){if(this.functionIsProtected(t))throw n.ProtectedFunctionError.cannotRegisterFunctionWithId(t);this.loadFunctionUnprotected(e,t,r)}static loadFunctionUnprotected(e,t,r){const o=i(t,e).method;if(!Object.prototype.hasOwnProperty.call(e.prototype,o))throw n.FunctionPluginValidationError.functionMethodNotFound(o,e.name);r.set(t,e)}static*protectedFunctions(){for(const[e,t]of this._protectedPlugins)void 0!==t&&(yield[e,t])}static*protectedPlugins(){for(const[,e]of this._protectedPlugins)void 0!==e&&(yield e)}initializePlugins(e){const t=[];for(const[r,n]of this.instancePlugins.entries()){let o=t.find(e=>e instanceof n);void 0===o&&(o=new n(e),t.push(o));const s=i(r,n),a=s.method;this.functions.set(r,[a,o]);const u=s.arraySizeMethod;void 0!==u&&this.arraySizeFunctions.set(r,[u,o])}}getFunctionPlugin(e){if(!a.functionIsProtected(e))return this.instancePlugins.get(e)}getFunction(e){const t=this.functions.get(e);if(void 0!==t&&this.config.translationPackage.isFunctionTranslated(e)){const[e,r]=t;return(t,n)=>r[e](t,n)}}getArraySizeFunction(e){const t=this.arraySizeFunctions.get(e);if(void 0!==t&&this.config.translationPackage.isFunctionTranslated(e)){const[e,r]=t;return(t,n)=>r[e](t,n)}}getMetadata(e){return this.functionsMetadata.get(e)}getPlugins(){const e=new Set;for(const[t,r]of this.instancePlugins)a.functionIsProtected(t)||e.add(r);return Array.from(e)}getRegisteredFunctionIds(){return Array.from(this.functions.keys())}categorizeFunction(e,t){t.isVolatile&&this.volatileFunctions.add(e),t.arrayFunction&&this.arrayFunctions.add(e),t.doesNotNeedArgumentsToBeComputed&&this.functionsWhichDoesNotNeedArgumentsToBeComputed.add(e),t.isDependentOnSheetStructureChange&&this.structuralChangeFunctions.add(e),this.functionsMetadata.set(e,t)}}t.FunctionRegistry=a,a.plugins=new Map,a._protectedPlugins=new Map([["VERSION",s.VersionPlugin],["OFFSET",void 0]])},function(e,t,r){"use strict";t.__esModule=!0,t.VersionPlugin=void 0;var n=r(179),o=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
const s=new Map([["gpl-v3",1],["missing",2],["invalid",3],["expired",4]]);class i extends o.FunctionPlugin{version(e,t){return this.runFunction(e.args,t,this.metadata("VERSION"),()=>{const{licenseKeyValidityState:e,licenseKey:t}=this.config;let r;return s.has(t)?r=s.get(t):s.has(e)?r=s.get(e):"valid"===e&&(r=t.slice(-5)),`HyperFormula v${n.HyperFormula.version}, ${r}`})}}t.VersionPlugin=i,i.implementedFunctions={VERSION:{method:"version",parameters:[]}}},function(e,t,r){"use strict";t.__esModule=!0,t.Interpreter=void 0,r(2);var n=r(73),o=r(152),s=r(74),i=r(132),a=r(128),u=r(166),c=r(202),l=r(199),h=r(164),d=r(141),p=r(140);function m(e,t){return t instanceof s.CellError?t:e(t)}function g(e,t,r){return t instanceof s.CellError?t:r instanceof s.CellError?r:e(t,r)}t.Interpreter=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e,t,r,n,o,s,i,a,l,h){this.config=e,this.dependencyGraph=t,this.columnSearch=r,this.stats=n,this.arithmeticHelper=o,this.functionRegistry=s,this.namedExpressions=i,this.serialization=a,this.arraySizePredictor=l,this.dateTimeHelper=h,this.equalOp=(e,t)=>g(this.arithmeticHelper.eq,e,t),this.notEqualOp=(e,t)=>g(this.arithmeticHelper.neq,e,t),this.greaterThanOp=(e,t)=>g(this.arithmeticHelper.gt,e,t),this.lessThanOp=(e,t)=>g(this.arithmeticHelper.lt,e,t),this.greaterThanOrEqualOp=(e,t)=>g(this.arithmeticHelper.geq,e,t),this.lessThanOrEqualOp=(e,t)=>g(this.arithmeticHelper.leq,e,t),this.concatOp=(e,t)=>g(this.arithmeticHelper.concat,(0,u.coerceScalarToString)(e),(0,u.coerceScalarToString)(t)),this.plusOp=(e,t)=>g(this.arithmeticHelper.addWithEpsilon,this.arithmeticHelper.coerceScalarToNumberOrError(e),this.arithmeticHelper.coerceScalarToNumberOrError(t)),this.minusOp=(e,t)=>g(this.arithmeticHelper.subtract,this.arithmeticHelper.coerceScalarToNumberOrError(e),this.arithmeticHelper.coerceScalarToNumberOrError(t)),this.timesOp=(e,t)=>g(this.arithmeticHelper.multiply,this.arithmeticHelper.coerceScalarToNumberOrError(e),this.arithmeticHelper.coerceScalarToNumberOrError(t)),this.powerOp=(e,t)=>g(this.arithmeticHelper.pow,this.arithmeticHelper.coerceScalarToNumberOrError(e),this.arithmeticHelper.coerceScalarToNumberOrError(t)),this.divOp=(e,t)=>g(this.arithmeticHelper.divide,this.arithmeticHelper.coerceScalarToNumberOrError(e),this.arithmeticHelper.coerceScalarToNumberOrError(t)),this.unaryMinusOp=e=>m(this.arithmeticHelper.unaryMinus,this.arithmeticHelper.coerceScalarToNumberOrError(e)),this.percentOp=e=>m(this.arithmeticHelper.unaryPercent,this.arithmeticHelper.coerceScalarToNumberOrError(e)),this.unaryPlusOp=e=>this.arithmeticHelper.unaryPlus(e),this.functionRegistry.initializePlugins(this),this.criterionBuilder=new c.CriterionBuilder(e)}evaluateAst(e,t){let r=this.evaluateAstWithoutPostprocessing(e,t);if((0,d.isExtendedNumber)(r)){if((0,u.isNumberOverflow)((0,d.getRawValue)(r)))return new s.CellError(s.ErrorType.NUM,i.ErrorMessage.NaN);r=(0,d.cloneNumber)(r,(0,u.fixNegativeZero)((0,d.getRawValue)(r)))}return r instanceof p.SimpleRangeValue&&1===r.height()&&1===r.width()&&([[r]]=r.data),function(e,t){if(e instanceof s.CellError&&void 0!==t)return e.attachRootVertex(t);return e}(r,t.formulaVertex)}evaluateAstWithoutPostprocessing(e,t){switch(e.type){case a.AstNodeType.EMPTY:return d.EmptyValue;case a.AstNodeType.CELL_REFERENCE:{const r=e.reference.toSimpleCellAddress(t.formulaAddress);return(0,s.invalidSimpleCellAddress)(r)?new s.CellError(s.ErrorType.REF,i.ErrorMessage.BadRef):this.dependencyGraph.getCellValue(r)}case a.AstNodeType.NUMBER:case a.AstNodeType.STRING:return e.value;case a.AstNodeType.CONCATENATE_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.concatOp,r,n,t)}case a.AstNodeType.EQUALS_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.equalOp,r,n,t)}case a.AstNodeType.NOT_EQUAL_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.notEqualOp,r,n,t)}case a.AstNodeType.GREATER_THAN_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.greaterThanOp,r,n,t)}case a.AstNodeType.LESS_THAN_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.lessThanOp,r,n,t)}case a.AstNodeType.GREATER_THAN_OR_EQUAL_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.greaterThanOrEqualOp,r,n,t)}case a.AstNodeType.LESS_THAN_OR_EQUAL_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.lessThanOrEqualOp,r,n,t)}case a.AstNodeType.PLUS_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.plusOp,r,n,t)}case a.AstNodeType.MINUS_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.minusOp,r,n,t)}case a.AstNodeType.TIMES_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.timesOp,r,n,t)}case a.AstNodeType.POWER_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.powerOp,r,n,t)}case a.AstNodeType.DIV_OP:{const r=this.evaluateAst(e.left,t),n=this.evaluateAst(e.right,t);return this.binaryRangeWrapper(this.divOp,r,n,t)}case a.AstNodeType.PLUS_UNARY_OP:{const r=this.evaluateAst(e.value,t);return this.unaryRangeWrapper(this.unaryPlusOp,r,t)}case a.AstNodeType.MINUS_UNARY_OP:{const r=this.evaluateAst(e.value,t);return this.unaryRangeWrapper(this.unaryMinusOp,r,t)}case a.AstNodeType.PERCENT_OP:{const r=this.evaluateAst(e.value,t);return this.unaryRangeWrapper(this.percentOp,r,t)}case a.AstNodeType.FUNCTION_CALL:{if("valid"!==this.config.licenseKeyValidityState&&!l.FunctionRegistry.functionIsProtected(e.procedureName))return new s.CellError(s.ErrorType.LIC,i.ErrorMessage.LicenseKey(this.config.licenseKeyValidityState));const r=this.functionRegistry.getFunction(e.procedureName);return void 0!==r?r(e,new h.InterpreterState(t.formulaAddress,t.arraysFlag||this.functionRegistry.isArrayFunction(e.procedureName),t.formulaVertex)):new s.CellError(s.ErrorType.NAME,i.ErrorMessage.FunctionName(e.procedureName))}case a.AstNodeType.NAMED_EXPRESSION:{const r=this.namedExpressions.nearestNamedExpression(e.expressionName,t.formulaAddress.sheet);return r?this.dependencyGraph.getCellValue(r.address):new s.CellError(s.ErrorType.NAME,i.ErrorMessage.NamedExpressionName(e.expressionName))}case a.AstNodeType.CELL_RANGE:{if(!this.rangeSpansOneSheet(e))return new s.CellError(s.ErrorType.REF,i.ErrorMessage.RangeManySheets);const r=n.AbsoluteCellRange.fromCellRange(e,t.formulaAddress),a=this.dependencyGraph.getArray(r);if(a){const e=a.array;if(e instanceof o.NotComputedArray)throw new Error("Array should be already computed");if(e instanceof s.CellError)return e;if(e instanceof o.ArrayValue)return p.SimpleRangeValue.fromRange(e.raw(),r,this.dependencyGraph);throw new Error("Unknown array")}return p.SimpleRangeValue.onlyRange(r,this.dependencyGraph)}case a.AstNodeType.COLUMN_RANGE:{if(!this.rangeSpansOneSheet(e))return new s.CellError(s.ErrorType.REF,i.ErrorMessage.RangeManySheets);const r=n.AbsoluteColumnRange.fromColumnRange(e,t.formulaAddress);return p.SimpleRangeValue.onlyRange(r,this.dependencyGraph)}case a.AstNodeType.ROW_RANGE:{if(!this.rangeSpansOneSheet(e))return new s.CellError(s.ErrorType.REF,i.ErrorMessage.RangeManySheets);const r=n.AbsoluteRowRange.fromRowRangeAst(e,t.formulaAddress);return p.SimpleRangeValue.onlyRange(r,this.dependencyGraph)}case a.AstNodeType.PARENTHESIS:return this.evaluateAst(e.expression,t);case a.AstNodeType.ARRAY:{let r=void 0;const n=[];for(const o of e.args){let e=void 0;const a=[];for(const r of o){const n=(0,u.coerceToRange)(this.evaluateAst(r,t)),o=n.height();if(void 0===e)e=o,a.push(...n.data);else{if(e!==o)return new s.CellError(s.ErrorType.REF,i.ErrorMessage.SizeMismatch);for(let e=0;e<o;e++)a[e].push(...n.data[e])}}const c=a[0].length;if(void 0===r)r=c,n.push(...a);else{if(r!==c)return new s.CellError(s.ErrorType.REF,i.ErrorMessage.SizeMismatch);n.push(...a)}}return p.SimpleRangeValue.onlyValues(n)}case a.AstNodeType.ERROR_WITH_RAW_INPUT:case a.AstNodeType.ERROR:return e.error}}rangeSpansOneSheet(e){return e.start.sheet===e.end.sheet}unaryRangeWrapper(e,t,r){var n;if(t instanceof p.SimpleRangeValue&&!r.arraysFlag&&(t=null!==(n=(0,u.coerceRangeToScalar)(t,r))&&void 0!==n?n:new s.CellError(s.ErrorType.VALUE,i.ErrorMessage.ScalarExpected)),t instanceof s.CellError)return t;if(t instanceof p.SimpleRangeValue){const r=t.data.map(t=>t.map(e));return p.SimpleRangeValue.onlyValues(r)}return e(t)}binaryRangeWrapper(e,t,r,n){var o,a;if(t instanceof p.SimpleRangeValue&&!n.arraysFlag&&(t=null!==(o=(0,u.coerceRangeToScalar)(t,n))&&void 0!==o?o:new s.CellError(s.ErrorType.VALUE,i.ErrorMessage.ScalarExpected)),t instanceof s.CellError)return t;if(r instanceof p.SimpleRangeValue&&!n.arraysFlag&&(r=null!==(a=(0,u.coerceRangeToScalar)(r,n))&&void 0!==a?a:new s.CellError(s.ErrorType.VALUE,i.ErrorMessage.ScalarExpected)),r instanceof s.CellError)return r;if(t instanceof p.SimpleRangeValue||r instanceof p.SimpleRangeValue){if(!(t instanceof p.SimpleRangeValue)){if(r.isAdHoc()){const n=r.data;for(let r=0;r<n.length;r++)for(let o=0;o<n[0].length;o++)n[r][o]=e(t,n[r][o]);return p.SimpleRangeValue.onlyValues(n)}t=p.SimpleRangeValue.fromScalar(t)}if(!(r instanceof p.SimpleRangeValue)){if(t.isAdHoc()){const n=t.data;for(let t=0;t<n.length;t++)for(let o=0;o<n[0].length;o++)n[t][o]=e(n[t][o],r);return p.SimpleRangeValue.onlyValues(n)}r=p.SimpleRangeValue.fromScalar(r)}if(t.width()===r.width()&&t.height()===r.height()){if(t.isAdHoc()){const n=t.data,o=r.data;for(let t=0;t<n.length;t++)for(let r=0;r<n[0].length;r++)n[t][r]=e(n[t][r],o[t][r]);return p.SimpleRangeValue.onlyValues(n)}if(r.isAdHoc()){const n=t.data,o=r.data;for(let t=0;t<n.length;t++)for(let r=0;r<n[0].length;r++)o[t][r]=e(n[t][r],o[t][r]);return p.SimpleRangeValue.onlyValues(o)}}const n=Math.max(t.width(),r.width()),o=Math.max(t.height(),r.height()),i=Array(o);for(let e=0;e<o;e++)i[e]=Array(n);for(let a=0;a<o;a++){const o=1!==t.height()?a:0,u=1!==r.height()?a:0;for(let c=0;c<n;c++){const n=1!==t.width()?c:0,l=1!==r.width()?c:0;o<t.height()&&u<r.height()&&n<t.width()&&l<r.width()?i[a][c]=e(t.data[o][n],r.data[u][l]):i[a][c]=new s.CellError(s.ErrorType.NA)}}return p.SimpleRangeValue.onlyValues(i)}return e(t,r)}}},function(e,t,r){"use strict";t.__esModule=!0,t.buildCriterionLambda=t.buildCriterion=t.CriterionType=t.CriterionBuilder=void 0;var n,o=r(141);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.CriterionType=n,function(e){e.GREATER_THAN="GREATER_THAN",e.GREATER_THAN_OR_EQUAL="GREATER_THAN_OR_EQUAL",e.LESS_THAN="LESS_THAN",e.LESS_THAN_OR_EQUAL="LESS_THAN_OR_EQUAL",e.NOT_EQUAL="NOT_EQUAL",e.EQUAL="EQUAL"}(n||(t.CriterionType=n={}));const s=(e,t)=>({operator:e,value:t});t.buildCriterion=s;t.CriterionBuilder=class{constructor(e){var t,r,n,o;this.trueString=null!==(r=null===(t=e.translationPackage.getMaybeFunctionTranslation("TRUE"))||void 0===t?void 0:t.toLowerCase())&&void 0!==r?r:"true",this.falseString=null!==(o=null===(n=e.translationPackage.getMaybeFunctionTranslation("FALSE"))||void 0===n?void 0:n.toLowerCase())&&void 0!==o?o:"false"}fromCellValue(e,t){if("string"!=typeof e&&"boolean"!=typeof e&&"number"!=typeof e)return;const r=this.parseCriterion(e,t);return void 0!==r?{raw:e,lambda:a(r,t)}:void 0}parseCriterion(e,t){if("number"==typeof e||"boolean"==typeof e)return s(n.EQUAL,e);if("string"==typeof e){const r=i.exec(e);let a,u;r?(u=function(e){switch(e){case">":return n.GREATER_THAN;case">=":return n.GREATER_THAN_OR_EQUAL;case"<":return n.LESS_THAN;case"<=":return n.LESS_THAN_OR_EQUAL;case"<>":return n.NOT_EQUAL;case"=":return n.EQUAL;default:return}}(r[1]),a=r[2]):(u=n.EQUAL,a=e);const c=t.coerceToMaybeNumber(a),l=a.toLowerCase()===this.trueString||a.toLowerCase()!==this.falseString&&void 0;if(void 0===u)return;if(""===a)return s(u,null);if(void 0!==c)return s(u,(0,o.getRawValue)(c));if(u===n.EQUAL||u===n.NOT_EQUAL)return s(u,null!=l?l:a)}}};const i=/([<>=]+)(.*)/;const a=(e,t)=>{switch(e.operator){case n.GREATER_THAN:return"number"==typeof e.value?r=>"number"==typeof r&&t.floatCmp(r,e.value)>0:e=>!1;case n.GREATER_THAN_OR_EQUAL:return"number"==typeof e.value?r=>"number"==typeof r&&t.floatCmp(r,e.value)>=0:e=>!1;case n.LESS_THAN:return"number"==typeof e.value?r=>"number"==typeof r&&t.floatCmp(r,e.value)<0:e=>!1;case n.LESS_THAN_OR_EQUAL:return"number"==typeof e.value?r=>"number"==typeof r&&t.floatCmp(r,e.value)<=0:e=>!1;case n.EQUAL:return"number"==typeof e.value?r=>{if("number"==typeof r)return 0===t.floatCmp(r,e.value);if("string"==typeof r){if(""===r)return!1;const n=t.coerceToMaybeNumber(r);return void 0!==n&&0===t.floatCmp(n,e.value)}return!1}:"string"==typeof e.value?t.eqMatcherFunction(e.value):"boolean"==typeof e.value?t=>"boolean"==typeof t&&t===e.value:e=>e===o.EmptyValue;case n.NOT_EQUAL:return"number"==typeof e.value?r=>{if("number"==typeof r)return 0!==t.floatCmp(r,e.value);if("string"==typeof r){if(""===r)return!0;const n=t.coerceToMaybeNumber(r);return void 0===n||0!==t.floatCmp(n,e.value)}return!0}:"string"==typeof e.value?t.neqMatcherFunction(e.value):"boolean"==typeof e.value?t=>"boolean"!=typeof t||t!==e.value:e=>e!==o.EmptyValue}};t.buildCriterionLambda=a},function(e,t,r){"use strict";t.__esModule=!0,t.LazilyTransformingAstService=void 0,r(2);var n=r(204),o=r(143);t.LazilyTransformingAstService=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.stats=e,this.transformations=[]}version(){return this.transformations.length}addTransformation(e){return void 0!==this.combinedTransformer?this.combinedTransformer.add(e):this.transformations.push(e),this.version()}beginCombinedMode(e){this.combinedTransformer=new n.CombinedTransformer(e)}commitCombinedMode(){if(void 0===this.combinedTransformer)throw Error("Combined mode wasn't started");return this.transformations.push(this.combinedTransformer),this.combinedTransformer=void 0,this.version()}applyTransformations(e,t,r){this.stats.start(o.StatType.TRANSFORM_ASTS_POSTPONED);for(let n=r;n<this.transformations.length;n++){const r=this.transformations[n];r.isIrreversible()&&(this.undoRedo.storeDataForVersion(n,t,this.parser.computeHashFromAst(e)),this.parser.rememberNewAst(e));const[o,s]=r.transformSingleAst(e,t);e=o,t=s}const n=this.parser.rememberNewAst(e);return this.stats.end(o.StatType.TRANSFORM_ASTS_POSTPONED),[n,t,this.transformations.length]}*getTransformationsFrom(e,t){for(let r=e;r<this.transformations.length;r++){const e=this.transformations[r];t&&!t(e)||(yield e)}}}},function(e,t,r){"use strict";t.__esModule=!0,t.CombinedTransformer=void 0,r(2);t.CombinedTransformer=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.sheet=e,this.transformations=[]}add(e){this.transformations.push(e)}performEagerTransformations(e,t){this.transformations.forEach(r=>r.performEagerTransformations(e,t))}transformSingleAst(e,t){let[r,n]=[e,t];return this.transformations.forEach(e=>{[r,n]=e.transformSingleAst(r,n)}),[r,n]}isIrreversible(){return!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.buildColumnSearchStrategy=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e,t,r){return t.useColumnIndex?new o.ColumnIndex(e,t,r):new n.ColumnBinarySearch(e)};var n=r(206),o=r(209)},function(e,t,r){"use strict";t.__esModule=!0,t.ColumnBinarySearch=void 0;var n=r(207);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.AdvancedFind{constructor(e){super(e),this.dependencyGraph=e}add(e,t){}remove(e,t){}change(e,t,r){}applyChanges(e){}addColumns(e){}removeColumns(e){}removeSheet(e){}moveValues(e,t,r,n){}removeValues(e){}find(e,t,r){return this.basicFind(e,t,"row",r)}}t.ColumnBinarySearch=o},function(e,t,r){"use strict";t.__esModule=!0,t.AdvancedFind=void 0;var n=r(141),o=r(166),s=r(208);t.AdvancedFind=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.dependencyGraph=e}advancedFind(e,t){let r;const o=t.range;r=void 0===o?t.valuesFromTopLeftCorner():this.dependencyGraph.computeListOfValuesInRange(o);for(let t=0;t<r.length;t++)if(e((0,n.getRawValue)(r[t])))return t;return-1}basicFind(e,t,r,n){let{ordering:i,matchExactly:a}=n;const u="string"==typeof e?(0,o.forceNormalizeString)(e):e,c=t.range;return void 0===c?this.findNormalizedValue(u,t.valuesFromTopLeftCorner()):"none"===i?this.findNormalizedValue(u,this.dependencyGraph.computeListOfValuesInRange(c)):(0,s.findLastOccurrenceInOrderedRange)(u,c,{searchCoordinate:r,orderingDirection:i,matchExactly:a},this.dependencyGraph)}findNormalizedValue(e,t){return t.map(n.getRawValue).map(e=>"string"==typeof e?(0,o.forceNormalizeString)(e):e).indexOf(e)}}},function(e,t,r){"use strict";t.__esModule=!0,t.compare=i,t.findLastMatchingIndex=s,t.findLastOccurrenceInOrderedArray=function(e,t){let r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"asc";const n="asc"===r?r=>i(e,t[r])>=0:r=>-i(e,t[r])>=0;return s(n,0,t.length-1)},t.findLastOccurrenceInOrderedRange=function(e,t,r,a){let{searchCoordinate:u,orderingDirection:c,matchExactly:l}=r;const h=t.start[u],d="col"===u?t.effectiveEndColumn(a):t.effectiveEndRow(a),p="col"===u?e=>(0,o.getRawValue)(a.getCellValue((0,n.simpleCellAddress)(t.sheet,e,t.start.row))):e=>(0,o.getRawValue)(a.getCellValue((0,n.simpleCellAddress)(t.sheet,t.start.col,e))),m="asc"===c?(e,t)=>i(e,t):(e,t)=>-i(e,t),g=s(t=>m(e,p(t))>=0,h,d),f=p(g);if(-1===g||typeof f!=typeof e)return-1;if(l&&f!==e)return-1;return g-h};var n=r(74),o=r(141);function s(e,t,r){let n=t,o=r;for(;n<o;){const t=Math.ceil((n+o)/2);e(t)?n=t:o=t-1}return n===o&&e(n)?n:-1}function i(e,t){return typeof e==typeof t?e===o.EmptyValue?0:e<t?-1:e>t?1:0:e===o.EmptyValue?-1:t===o.EmptyValue?1:t instanceof n.CellError||"number"==typeof e&&"string"==typeof t||"number"==typeof e&&"boolean"==typeof t?-1:"string"==typeof e&&"number"==typeof t?1:"string"==typeof e&&"boolean"==typeof t?-1:1}},function(e,t,r){"use strict";t.__esModule=!0,t.ColumnIndex=void 0,t.findInOrderedArray=d;var n=r(74),o=r(188),s=r(192),i=r(166),a=r(141),u=r(140),c=r(143),l=r(206);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class h{constructor(e,t,r){this.dependencyGraph=e,this.config=t,this.stats=r,this.index=new Map,this.transformingService=this.dependencyGraph.lazilyTransformingAstService,this.binarySearchStrategy=new l.ColumnBinarySearch(e)}add(e,t){if(!(e===a.EmptyValue||e instanceof n.CellError))if(e instanceof u.SimpleRangeValue)for(const[r,n]of e.entriesFromTopLeftCorner(t))this.addSingleCellValue((0,a.getRawValue)(r),n);else this.addSingleCellValue(e,t)}remove(e,t){if(void 0!==e)if(e instanceof u.SimpleRangeValue)for(const[r,n]of e.entriesFromTopLeftCorner(t))this.removeSingleValue((0,a.getRawValue)(r),n);else this.removeSingleValue(e,t)}change(e,t,r){e!==t&&(this.remove(e,r),this.add(t,r))}applyChanges(e){for(const t of e)void 0!==t.oldValue&&this.change((0,a.getRawValue)(t.oldValue),(0,a.getRawValue)(t.value),t.address)}moveValues(e,t,r,o){for(const[s,i]of e){const e=(0,n.movedSimpleCellAddress)(i,o,t,r);this.remove(s,i),this.add(s,e)}}removeValues(e){for(const[t,r]of e)this.remove(t,r)}find(e,t,r){let{ordering:n,matchExactly:o}=r;const s=!0===o?"findFirst":"findLast",i=this.findUsingColumnIndex(e,t,s);return void 0!==i?i:this.binarySearchStrategy.find(e,t,{ordering:n,matchExactly:o})}findUsingColumnIndex(e,t,r){const n=t.range;if(void 0===n)return;this.ensureRecentData(n.sheet,n.start.col,e);const o=this.getColumnMap(n.sheet,n.start.col);if(!o)return-1;const s="string"==typeof e?(0,i.forceNormalizeString)(e):e,a=o.get(s);if(!a||!a.index||0===a.index.length)return;const u=h.findRowBelongingToRange(a,n,r);return void 0!==u?u-n.start.row:void 0}static findRowBelongingToRange(e,t,r){const n=t.start.row,o=t.end.row,s="findFirst"===r?d(n,e.index,"upperBound"):d(o,e.index,"lowerBound");if(-1===s)return;const i=e.index[s];return i>=n&&i<=o?i:void 0}advancedFind(e,t){return this.binarySearchStrategy.advancedFind(e,t)}addColumns(e){const t=this.index.get(e.sheet);t&&t.splice(e.columnStart,0,...Array(e.numberOfColumns))}removeColumns(e){const t=this.index.get(e.sheet);t&&t.splice(e.columnStart,e.numberOfColumns)}removeSheet(e){this.index.delete(e)}getColumnMap(e,t){this.index.has(e)||this.index.set(e,[]);const r=this.index.get(e);let n=r[t];return n||(n=new Map,r[t]=n),n}getValueIndex(e,t,r){const n=this.getColumnMap(e,t);let o=this.getColumnMap(e,t).get(r);return o||(o={version:this.transformingService.version(),index:[]},n.set(r,o)),o}ensureRecentData(e,t,r){const n=this.getValueIndex(e,t,r),i=this.transformingService.version();if(n.version===i)return;const a=this.transformingService.getTransformationsFrom(n.version,t=>t.sheet===e&&(t instanceof o.AddRowsTransformer||t instanceof s.RemoveRowsTransformer));for(const e of a)e instanceof o.AddRowsTransformer?this.addRows(t,e.rowsSpan,r):e instanceof s.RemoveRowsTransformer&&this.removeRows(t,e.rowsSpan,r);n.version=i}addSingleCellValue(e,t){this.stats.measure(c.StatType.BUILD_COLUMN_INDEX,()=>{this.ensureRecentData(t.sheet,t.col,e),"string"==typeof e&&(e=(0,i.forceNormalizeString)(e));const r=this.getValueIndex(t.sheet,t.col,e);h.addValue(r,t.row)})}removeSingleValue(e,t){this.stats.measure(c.StatType.BUILD_COLUMN_INDEX,()=>{this.ensureRecentData(t.sheet,t.col,e);const r=this.getColumnMap(t.sheet,t.col);"string"==typeof e&&(e=(0,i.forceNormalizeString)(e));const n=r.get(e);if(!n)return;const o=d(t.row,n.index);o>-1&&n.index.splice(o,1),0===n.index.length&&r.delete(e),0===r.size&&delete this.index.get(t.sheet)[t.col]})}addRows(e,t,r){const n=this.getValueIndex(t.sheet,e,r);h.shiftRows(n,t.rowStart,t.numberOfRows)}removeRows(e,t,r){const n=this.getValueIndex(t.sheet,e,r);h.removeRowsFromValues(n,t),h.shiftRows(n,t.rowEnd+1,-t.numberOfRows)}static addValue(e,t){const r=d(t,e.index,"lowerBound");e.index[r]===t||e.index.splice(r+1,0,t)}static removeRowsFromValues(e,t){const r=d(t.rowStart,e.index,"upperBound"),n=d(t.rowEnd,e.index,"lowerBound");r>-1&&n>-1&&r<=n&&e.index[r]<=t.rowEnd&&e.index.splice(r,n-r+1)}static shiftRows(e,t,r){const n=d(t,e.index,"upperBound");if(-1!==n)for(let t=n;t<e.index.length;++t)e.index[t]+=r}}function d(e,t){let r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"upperBound",n=0,o=t.length-1;for(;n<=o;){const r=Math.floor((n+o)/2);if(e>t[r])n=r+1;else{if(!(e<t[r]))return r;o=r-1}}const s="lowerBound"===r?o:n,i=s>=0&&s<=t.length;return i?s:-1}t.ColumnIndex=h},function(e,t,r){"use strict";t.__esModule=!0,t.NumberLiteralHelper=void 0;t.NumberLiteralHelper=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class{constructor(e){this.config=e;const t="."===this.config.thousandSeparator?"\\"+this.config.thousandSeparator:this.config.thousandSeparator,r="."===this.config.decimalSeparator?"\\"+this.config.decimalSeparator:this.config.decimalSeparator;this.numberPattern=new RegExp(`^([+-]?((${r}\\d+)|(\\d+(${t}\\d{3,})*(${r}\\d*)?)))(e[+-]?\\d+)?$`),this.allThousandSeparatorsRegex=new RegExp(""+t,"g")}numericStringToMaybeNumber(e){if(this.numberPattern.test(e)){const t=this.numericStringToNumber(e);if(isNaN(t))return;return t}}numericStringToNumber(e){const t=e.replace(this.allThousandSeparatorsRegex,"").replace(this.config.decimalSeparator,".");return Number(t)}}},function(e,t,r){"use strict";t.__esModule=!0,t.Serialization=void 0;var n=r(74),o=r(75),s=r(78);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i{constructor(e,t,r){this.dependencyGraph=e,this.unparser=t,this.exporter=r}getCellHyperlink(e){const t=this.dependencyGraph.getCell(e);if(t instanceof o.FormulaCellVertex){const e=t.getFormula(this.dependencyGraph.lazilyTransformingAstService);if("HYPERLINK"===e.procedureName)return e.hyperlink}}getCellFormula(e,t){const r=this.dependencyGraph.getCell(e);if(r instanceof o.FormulaCellVertex){const n=r.getFormula(this.dependencyGraph.lazilyTransformingAstService);return t=null!=t?t:e,this.unparser.unparse(n,t)}if(r instanceof o.ArrayVertex){const n=r.getAddress(this.dependencyGraph.lazilyTransformingAstService);if(n.row!==e.row||n.col!==e.col||n.sheet!==e.sheet)return;t=null!=t?t:e;const o=r.getFormula(this.dependencyGraph.lazilyTransformingAstService);if(void 0!==o)return this.unparser.unparse(o,t)}else if(r instanceof o.ParsingErrorVertex)return r.getFormula()}getCellSerialized(e,t){var r;return null!==(r=this.getCellFormula(e,t))&&void 0!==r?r:this.getRawValue(e)}getCellValue(e){return this.exporter.exportValue(this.dependencyGraph.getScalarValue(e))}getRawValue(e){return this.dependencyGraph.getRawValue(e)}getSheetValues(e){return this.genericSheetGetter(e,e=>this.getCellValue(e))}getSheetFormulas(e){return this.genericSheetGetter(e,e=>this.getCellFormula(e))}genericSheetGetter(e,t){const r=this.dependencyGraph.getSheetHeight(e),o=this.dependencyGraph.getSheetWidth(e),s=new Array(r);for(let i=0;i<r;i++){s[i]=new Array(o);for(let r=0;r<o;r++){const o=(0,n.simpleCellAddress)(e,r,i);s[i][r]=t(o)}for(let e=o-1;e>=0&&(null===s[i][e]||void 0===s[i][e]);e--)s[i].pop()}for(let e=r-1;e>=0&&0===s[e].length;e--)s.pop();return s}genericAllSheetsGetter(e){const t={};for(const r of this.dependencyGraph.sheetMapping.displayNames()){const n=this.dependencyGraph.sheetMapping.fetch(r);t[r]=e(n)}return t}getSheetSerialized(e){return this.genericSheetGetter(e,e=>this.getCellSerialized(e))}getAllSheetsValues(){return this.genericAllSheetsGetter(e=>this.getSheetValues(e))}getAllSheetsFormulas(){return this.genericAllSheetsGetter(e=>this.getSheetFormulas(e))}getAllSheetsSerialized(){return this.genericAllSheetsGetter(e=>this.getSheetSerialized(e))}getAllNamedExpressionsSerialized(){const e=[];let t=0;for(const r of this.dependencyGraph.sheetMapping.displayNames()){const n=this.dependencyGraph.sheetMapping.fetch(r);e[n]=t,t++}return this.dependencyGraph.namedExpressions.getAllNamedExpressions().map(t=>({name:t.expression.displayName,expression:this.getCellSerialized(t.expression.address),scope:void 0!==t.scope?e[t.scope]:void 0,options:t.expression.options}))}withNewConfig(e,t){const r=new s.Unparser(e,(0,s.buildLexerConfig)(e),this.dependencyGraph.sheetMapping.fetchDisplayName,t);return new i(this.dependencyGraph,r,this.exporter)}}t.Serialization=i},function(e,t,r){"use strict";function n(e){return()=>{throw new Error(`The "${e}" method cannot be called because this HyperFormula instance has been destroyed`)}}t.__esModule=!0,t.objectDestroy=
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function(e){for(const[t,r]of Object.entries(e))r instanceof Function?e[t]=n(r):delete e[t]}},function(e,t,r){"use strict";t.__esModule=!0,t.Events=t.Emitter=void 0;var n,o=r(214);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */t.Events=n,function(e){e.SheetAdded="sheetAdded",e.SheetRemoved="sheetRemoved",e.SheetRenamed="sheetRenamed",e.NamedExpressionAdded="namedExpressionAdded",e.NamedExpressionRemoved="namedExpressionRemoved",e.ValuesUpdated="valuesUpdated",e.EvaluationSuspended="evaluationSuspended",e.EvaluationResumed="evaluationResumed"}(n||(t.Events=n={}));class s extends o.TinyEmitter{emit(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];return super.emit(e,...r),this}}t.Emitter=s},function(e,t){function r(){}r.prototype={on:function(e,t,r){var n=this.e||(this.e={});return(n[e]||(n[e]=[])).push({fn:t,ctx:r}),this},once:function(e,t,r){var n=this;function o(){n.off(e,o),t.apply(r,arguments)}return o._=t,this.on(e,o,r)},emit:function(e){for(var t=[].slice.call(arguments,1),r=((this.e||(this.e={}))[e]||[]).slice(),n=0,o=r.length;n<o;n++)r[n].fn.apply(r[n].ctx,t);return this},off:function(e,t){var r=this.e||(this.e={}),n=r[e],o=[];if(n&&t)for(var s=0,i=n.length;s<i;s++)n[s].fn!==t&&n[s].fn._!==t&&o.push(n[s]);return o.length?r[e]=o:delete r[e],this}},e.exports=r,e.exports.TinyEmitter=r},function(e,t,r){"use strict";t.__esModule=!0,t.default=void 0;var n={errors:{CYCLE:"#CYCLE!",DIV_BY_ZERO:"#DIV/0!",ERROR:"#ERROR!",NA:"#N/A",NAME:"#NAME?",NUM:"#NUM!",REF:"#REF!",SPILL:"#SPILL!",VALUE:"#VALUE!"},functions:{FILTER:"FILTER",ADDRESS:"ADDRESS",ARRAY_CONSTRAIN:"ARRAY_CONSTRAIN",ARRAYFORMULA:"ARRAYFORMULA",ABS:"ABS",ACOS:"ACOS",ACOSH:"ACOSH",ACOT:"ACOT",ACOTH:"ACOTH",AND:"AND",ARABIC:"ARABIC",ASIN:"ASIN",ASINH:"ASINH",ATAN2:"ATAN2",ATAN:"ATAN",ATANH:"ATANH",AVERAGE:"AVERAGE",AVERAGEA:"AVERAGEA",AVERAGEIF:"AVERAGEIF",BASE:"BASE",BIN2DEC:"BIN2DEC",BIN2HEX:"BIN2HEX",BIN2OCT:"BIN2OCT",BITAND:"BITAND",BITLSHIFT:"BITLSHIFT",BITOR:"BITOR",BITRSHIFT:"BITRSHIFT",BITXOR:"BITXOR",CEILING:"CEILING",CHAR:"CHAR",CHOOSE:"CHOOSE",CLEAN:"CLEAN",CODE:"CODE",COLUMN:"COLUMN",COLUMNS:"COLUMNS",CONCATENATE:"CONCATENATE",CORREL:"CORREL",COS:"COS",COSH:"COSH",COT:"COT",COTH:"COTH",COUNT:"COUNT",COUNTA:"COUNTA",COUNTBLANK:"COUNTBLANK",COUNTIF:"COUNTIF",COUNTIFS:"COUNTIFS",COUNTUNIQUE:"COUNTUNIQUE",CSC:"CSC",CSCH:"CSCH",CUMIPMT:"CUMIPMT",CUMPRINC:"CUMPRINC",DATE:"DATE",DATEDIF:"DATEDIF",DATEVALUE:"DATEVALUE",DAY:"DAY",DAYS360:"DAYS360",DAYS:"DAYS",DB:"DB",DDB:"DDB",DEC2BIN:"DEC2BIN",DEC2HEX:"DEC2HEX",DEC2OCT:"DEC2OCT",DECIMAL:"DECIMAL",DEGREES:"DEGREES",DELTA:"DELTA",DOLLARDE:"DOLLARDE",DOLLARFR:"DOLLARFR",EDATE:"EDATE",EFFECT:"EFFECT",EOMONTH:"EOMONTH",ERF:"ERF",ERFC:"ERFC",EVEN:"EVEN",EXACT:"EXACT",EXP:"EXP",FALSE:"FALSE",FIND:"FIND",FORMULATEXT:"FORMULATEXT",FV:"FV",FVSCHEDULE:"FVSCHEDULE",HEX2BIN:"HEX2BIN",HEX2DEC:"HEX2DEC",HEX2OCT:"HEX2OCT",HLOOKUP:"HLOOKUP",HOUR:"HOUR",HYPERLINK:"HYPERLINK",IF:"IF",IFERROR:"IFERROR",IFNA:"IFNA",IFS:"IFS",INDEX:"INDEX",INT:"INT",INTERVAL:"INTERVAL",IPMT:"IPMT",ISBINARY:"ISBINARY",ISBLANK:"ISBLANK",ISERR:"ISERR",ISERROR:"ISERROR",ISEVEN:"ISEVEN",ISFORMULA:"ISFORMULA",ISLOGICAL:"ISLOGICAL",ISNA:"ISNA",ISNONTEXT:"ISNONTEXT",ISNUMBER:"ISNUMBER",ISODD:"ISODD",ISOWEEKNUM:"ISOWEEKNUM",ISPMT:"ISPMT",ISREF:"ISREF",ISTEXT:"ISTEXT",LEFT:"LEFT",LEN:"LEN",LN:"LN",LOG10:"LOG10",LOG:"LOG",LOWER:"LOWER",MATCH:"MATCH",MAX:"MAX",MAXA:"MAXA",MAXIFS:"MAXIFS",MAXPOOL:"MAXPOOL",MEDIAN:"MEDIAN",MEDIANPOOL:"MEDIANPOOL",MID:"MID",MIN:"MIN",MINA:"MINA",MINIFS:"MINIFS",MINUTE:"MINUTE",MIRR:"MIRR",MMULT:"MMULT",MOD:"MOD",MONTH:"MONTH",NA:"NA",NETWORKDAYS:"NETWORKDAYS","NETWORKDAYS.INTL":"NETWORKDAYS.INTL",NOMINAL:"NOMINAL",NOT:"NOT",NOW:"NOW",NPER:"NPER",NPV:"NPV",OCT2BIN:"OCT2BIN",OCT2DEC:"OCT2DEC",OCT2HEX:"OCT2HEX",ODD:"ODD",OFFSET:"OFFSET",OR:"OR",PI:"PI",PMT:"PMT",PDURATION:"PDURATION",PRODUCT:"PRODUCT",POWER:"POWER",PPMT:"PPMT",PROPER:"PROPER",PV:"PV",RADIANS:"RADIANS",RAND:"RAND",RATE:"RATE",REPLACE:"REPLACE",REPT:"REPT",RIGHT:"RIGHT",ROMAN:"ROMAN",ROUND:"ROUND",ROUNDDOWN:"ROUNDDOWN",ROUNDUP:"ROUNDUP",ROW:"ROW",ROWS:"ROWS",RRI:"RRI",SEARCH:"SEARCH",SEC:"SEC",SECH:"SECH",SECOND:"SECOND",SHEET:"SHEET",SHEETS:"SHEETS",SIN:"SIN",SINH:"SINH",SLN:"SLN",SPLIT:"SPLIT",SQRT:"SQRT",STDEVA:"STDEVA","STDEV.P":"STDEV.P",STDEVPA:"STDEVPA","STDEV.S":"STDEV.S",SUBSTITUTE:"SUBSTITUTE",SUBTOTAL:"SUBTOTAL",SUM:"SUM",SUMIF:"SUMIF",SUMIFS:"SUMIFS",SUMPRODUCT:"SUMPRODUCT",SUMSQ:"SUMSQ",SWITCH:"SWITCH",SYD:"SYD",T:"T",TAN:"TAN",TANH:"TANH",TBILLEQ:"TBILLEQ",TBILLPRICE:"TBILLPRICE",TBILLYIELD:"TBILLYIELD",TEXT:"TEXT",TIME:"TIME",TIMEVALUE:"TIMEVALUE",TODAY:"TODAY",TRANSPOSE:"TRANSPOSE",TRIM:"TRIM",TRUE:"TRUE",TRUNC:"TRUNC",UNICHAR:"UNICHAR",UNICODE:"UNICODE",UPPER:"UPPER",VARA:"VARA","VAR.P":"VAR.P",VARPA:"VARPA","VAR.S":"VAR.S",VLOOKUP:"VLOOKUP",WEEKDAY:"WEEKDAY",WEEKNUM:"WEEKNUM",WORKDAY:"WORKDAY","WORKDAY.INTL":"WORKDAY.INTL",XNPV:"XNPV",XOR:"XOR",YEAR:"YEAR",YEARFRAC:"YEARFRAC","HF.ADD":"HF.ADD","HF.CONCAT":"HF.CONCAT","HF.DIVIDE":"HF.DIVIDE","HF.EQ":"HF.EQ","HF.GT":"HF.GT","HF.GTE":"HF.GTE","HF.LT":"HF.LT","HF.LTE":"HF.LTE","HF.MINUS":"HF.MINUS","HF.MULTIPLY":"HF.MULTIPLY","HF.NE":"HF.NE","HF.POW":"HF.POW","HF.UMINUS":"HF.UMINUS","HF.UNARY_PERCENT":"HF.UNARY_PERCENT","HF.UPLUS":"HF.UPLUS",VARP:"VARP",VAR:"VAR",STDEVP:"STDEVP",STDEV:"STDEV","EXPON.DIST":"EXPON.DIST",FISHER:"FISHER",FISHERINV:"FISHERINV",GAMMA:"GAMMA","GAMMA.DIST":"GAMMA.DIST",GAMMALN:"GAMMALN","GAMMALN.PRECISE":"GAMMALN.PRECISE","GAMMA.INV":"GAMMA.INV",GAUSS:"GAUSS",FACT:"FACT",FACTDOUBLE:"FACTDOUBLE",COMBIN:"COMBIN",COMBINA:"COMBINA",GCD:"GCD",LCM:"LCM",MROUND:"MROUND",MULTINOMIAL:"MULTINOMIAL",QUOTIENT:"QUOTIENT",RANDBETWEEN:"RANDBETWEEN",SERIESSUM:"SERIESSUM",SIGN:"SIGN",SQRTPI:"SQRTPI",SUMX2MY2:"SUMX2MY2",SUMX2PY2:"SUMX2PY2",SUMXMY2:"SUMXMY2",EXPONDIST:"EXPONDIST",GAMMADIST:"GAMMADIST",GAMMAINV:"GAMMAINV","BETA.DIST":"BETA.DIST",BETADIST:"BETADIST","BETA.INV":"BETA.INV",BETAINV:"BETAINV","BINOM.DIST":"BINOM.DIST",BINOMDIST:"BINOMDIST","BINOM.INV":"BINOM.INV",BESSELI:"BESSELI",BESSELJ:"BESSELJ",BESSELK:"BESSELK",BESSELY:"BESSELY","CHISQ.DIST":"CHISQ.DIST","CHISQ.DIST.RT":"CHISQ.DIST.RT","CHISQ.INV":"CHISQ.INV","CHISQ.INV.RT":"CHISQ.INV.RT",CHIDIST:"CHIDIST",CHIINV:"CHIINV","F.DIST":"F.DIST","F.DIST.RT":"F.DIST.RT","F.INV":"F.INV","F.INV.RT":"F.INV.RT",FDIST:"FDIST",FINV:"FINV",WEIBULL:"WEIBULL","WEIBULL.DIST":"WEIBULL.DIST",POISSON:"POISSON","POISSON.DIST":"POISSON.DIST","HYPGEOM.DIST":"HYPGEOM.DIST",HYPGEOMDIST:"HYPGEOMDIST","T.DIST":"T.DIST","T.DIST.2T":"T.DIST.2T","T.DIST.RT":"T.DIST.RT","T.INV":"T.INV","T.INV.2T":"T.INV.2T",TDIST:"TDIST",TINV:"TINV",LOGINV:"LOGINV","LOGNORM.DIST":"LOGNORM.DIST","LOGNORM.INV":"LOGNORM.INV",LOGNORMDIST:"LOGNORMDIST","NORM.DIST":"NORM.DIST","NORM.INV":"NORM.INV","NORM.S.DIST":"NORM.S.DIST","NORM.S.INV":"NORM.S.INV",NORMDIST:"NORMDIST",NORMINV:"NORMINV",NORMSDIST:"NORMSDIST",NORMSINV:"NORMSINV",PHI:"PHI","NEGBINOM.DIST":"NEGBINOM.DIST",NEGBINOMDIST:"NEGBINOMDIST",COMPLEX:"COMPLEX",IMABS:"IMABS",IMAGINARY:"IMAGINARY",IMARGUMENT:"IMARGUMENT",IMCONJUGATE:"IMCONJUGATE",IMCOS:"IMCOS",IMCOSH:"IMCOSH",IMCOT:"IMCOT",IMCSC:"IMCSC",IMCSCH:"IMCSCH",IMDIV:"IMDIV",IMEXP:"IMEXP",IMLN:"IMLN",IMLOG10:"IMLOG10",IMLOG2:"IMLOG2",IMPOWER:"IMPOWER",IMPRODUCT:"IMPRODUCT",IMREAL:"IMREAL",IMSEC:"IMSEC",IMSECH:"IMSECH",IMSIN:"IMSIN",IMSINH:"IMSINH",IMSQRT:"IMSQRT",IMSUB:"IMSUB",IMSUM:"IMSUM",IMTAN:"IMTAN",LARGE:"LARGE",SMALL:"SMALL",AVEDEV:"AVEDEV",CONFIDENCE:"CONFIDENCE","CONFIDENCE.NORM":"CONFIDENCE.NORM","CONFIDENCE.T":"CONFIDENCE.T",DEVSQ:"DEVSQ",GEOMEAN:"GEOMEAN",HARMEAN:"HARMEAN",CRITBINOM:"CRITBINOM","COVARIANCE.P":"COVARIANCE.P","COVARIANCE.S":"COVARIANCE.S",COVAR:"COVAR",PEARSON:"PEARSON",RSQ:"RSQ",STANDARDIZE:"STANDARDIZE","Z.TEST":"Z.TEST",ZTEST:"ZTEST","F.TEST":"F.TEST",FTEST:"FTEST",STEYX:"STEYX",SLOPE:"SLOPE","CHISQ.TEST":"CHISQ.TEST",CHITEST:"CHITEST","T.TEST":"T.TEST",TTEST:"TTEST",SKEW:"SKEW","SKEW.P":"SKEW.P",WEIBULLDIST:"WEIBULLDIST",VARS:"VARS",TINV2T:"TINV2T",TDISTRT:"TDISTRT",TDIST2T:"TDIST2T",STDEVS:"STDEVS",FINVRT:"FINVRT",FDISTRT:"FDISTRT",CHIDISTRT:"CHIDISTRT",CHIINVRT:"CHIINVRT",COVARIANCEP:"COVARIANCEP",COVARIANCES:"COVARIANCES",LOGNORMINV:"LOGNORMINV",POISSONDIST:"POISSONDIST",SKEWP:"SKEWP","CEILING.MATH":"CEILING.MATH",FLOOR:"FLOOR","FLOOR.MATH":"FLOOR.MATH","CEILING.PRECISE":"CEILING.PRECISE","FLOOR.PRECISE":"FLOOR.PRECISE","ISO.CEILING":"ISO.CEILING"},langCode:"enGB",ui:{NEW_SHEET_PREFIX:"Sheet"}};t.default=n},function(e,t,r){"use strict";t.__esModule=!0;var n=r(165);t.FunctionPlugin=n.FunctionPlugin,t.FunctionArgumentType=n.FunctionArgumentType},function(e,t,r){"use strict";t.__esModule=!0,t.TrigonometryPlugin=t.TextPlugin=t.SumprodPlugin=t.StatisticalPlugin=t.StatisticalAggregationPlugin=t.SqrtPlugin=t.SimpleArithmerticPlugin=t.RoundingPlugin=t.RomanPlugin=t.RandomPlugin=t.RadixConversionPlugin=t.RadiansPlugin=t.PowerPlugin=t.NumericAggregationPlugin=t.ModuloPlugin=t.MedianPlugin=t.MatrixPlugin=t.MathPlugin=t.MathConstantsPlugin=t.LookupPlugin=t.LogarithmPlugin=t.IsOddPlugin=t.IsEvenPlugin=t.InformationPlugin=t.HyperlinkPlugin=t.FormulaTextPlugin=t.FinancialPlugin=t.ExpPlugin=t.DeltaPlugin=t.DegreesPlugin=t.DateTimePlugin=t.CountUniquePlugin=t.CountBlankPlugin=t.ConditionalAggregationPlugin=t.ComplexPlugin=t.CodePlugin=t.CharPlugin=t.BooleanPlugin=t.BitwiseLogicOperationsPlugin=t.BitShiftPlugin=t.ArrayPlugin=t.AddressPlugin=t.AbsPlugin=void 0;var n=r(218);t.AddressPlugin=n.AddressPlugin;var o=r(219);t.ArrayPlugin=o.ArrayPlugin;var s=r(220);t.AbsPlugin=s.AbsPlugin;var i=r(221);t.BitShiftPlugin=i.BitShiftPlugin;var a=r(222);t.BitwiseLogicOperationsPlugin=a.BitwiseLogicOperationsPlugin;var u=r(223);t.BooleanPlugin=u.BooleanPlugin;var c=r(224);t.CharPlugin=c.CharPlugin;var l=r(225);t.CodePlugin=l.CodePlugin;var h=r(226);t.CountBlankPlugin=h.CountBlankPlugin;var d=r(227);t.CountUniquePlugin=d.CountUniquePlugin;var p=r(228);t.DateTimePlugin=p.DateTimePlugin;var m=r(229);t.DegreesPlugin=m.DegreesPlugin;var g=r(230);t.DeltaPlugin=g.DeltaPlugin;var f=r(231);t.ExpPlugin=f.ExpPlugin;var E=r(232);t.FinancialPlugin=E.FinancialPlugin;var T=r(233);t.FormulaTextPlugin=T.FormulaTextPlugin;var y=r(234);t.HyperlinkPlugin=y.HyperlinkPlugin;var A=r(235);t.InformationPlugin=A.InformationPlugin;var R=r(236);t.IsEvenPlugin=R.IsEvenPlugin;var N=r(237);t.IsOddPlugin=N.IsOddPlugin;var S=r(238);t.LogarithmPlugin=S.LogarithmPlugin;var C=r(239);t.MathConstantsPlugin=C.MathConstantsPlugin;var v=r(240);t.MatrixPlugin=v.MatrixPlugin;var O=r(241);t.MedianPlugin=O.MedianPlugin;var M=r(242);t.ModuloPlugin=M.ModuloPlugin;var I=r(243);t.NumericAggregationPlugin=I.NumericAggregationPlugin;var w=r(244);t.PowerPlugin=w.PowerPlugin;var F=r(245);t.RadiansPlugin=F.RadiansPlugin;var b=r(246);t.RadixConversionPlugin=b.RadixConversionPlugin;var _=r(247);t.RandomPlugin=_.RandomPlugin;var L=r(248);t.RoundingPlugin=L.RoundingPlugin;var x=r(249);t.SqrtPlugin=x.SqrtPlugin;var P=r(250);t.ConditionalAggregationPlugin=P.ConditionalAggregationPlugin;var U=r(253);t.SumprodPlugin=U.SumprodPlugin;var V=r(254);t.TextPlugin=V.TextPlugin;var D=r(255);t.TrigonometryPlugin=D.TrigonometryPlugin;var B=r(256);t.LookupPlugin=B.LookupPlugin;var k=r(258);t.RomanPlugin=k.RomanPlugin;var G=r(259);t.SimpleArithmerticPlugin=G.SimpleArithmerticPlugin;var j=r(260);t.StatisticalPlugin=j.StatisticalPlugin;var W=r(263);t.MathPlugin=W.MathPlugin;var H=r(264);t.ComplexPlugin=H.ComplexPlugin;var z=r(265);t.StatisticalAggregationPlugin=z.StatisticalAggregationPlugin},function(e,t,r){"use strict";t.__esModule=!0,t.AddressPlugin=void 0;var n,o=r(79),s=r(165),i=r(74),a=r(132);!function(e){e[e.FullyAbsolute=1]="FullyAbsolute",e[e.RowAbsoluteColRelative=2]="RowAbsoluteColRelative",e[e.RowRelativeColAbsolute=3]="RowRelativeColAbsolute",e[e.FullyRelative=4]="FullyRelative"}(n||(n={}));class u extends s.FunctionPlugin{verifyAddressArguments(e,t,r,o){if(o){if(e<1||t<1)return new i.CellError(i.ErrorType.VALUE,a.ErrorMessage.LessThanOne)}else if(n.FullyAbsolute==r){if(e<1||t<1)return new i.CellError(i.ErrorType.VALUE,a.ErrorMessage.LessThanOne)}else if(n.RowAbsoluteColRelative==r){if(e<1)return new i.CellError(i.ErrorType.VALUE,a.ErrorMessage.LessThanOne)}else if(n.RowRelativeColAbsolute==r&&t<1)return new i.CellError(i.ErrorType.VALUE,a.ErrorMessage.LessThanOne)}address(e,t){return this.runFunction(e.args,t,this.metadata("ADDRESS"),(e,t,r,s,i)=>{const a=this.verifyAddressArguments(e,t,r,s);if(void 0!==a)return a;const u=(0,o.columnIndexToLabel)(t-1);let c="";null!=i&&(c=i+"!");const l=0==t?"C":`C[${t}]`,h=0==e?"R":`R[${e}]`;return n.FullyRelative==r?s?`${c}${u}${e}`:`${c}${h}${l}`:n.RowRelativeColAbsolute==r?s?`${c}$${u}${e}`:`${c}${h}C${t}`:n.RowAbsoluteColRelative==r?s?`${c}${u}$${e}`:`${c}R${e}${l}`:s?`${c}$${u}$${e}`:`${c}R${e}C${t}`})}}t.AddressPlugin=u,u.implementedFunctions={ADDRESS:{method:"address",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,optionalArg:!0,defaultValue:1,minValue:1,maxValue:4},{argumentType:s.FunctionArgumentType.BOOLEAN,optionalArg:!0,defaultValue:!0},{argumentType:s.FunctionArgumentType.STRING,optionalArg:!0}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.ArrayPlugin=void 0,r(2);var n=r(1),o=r(74),s=r(132),i=r(78),a=r(166),u=r(164),c=r(140),l=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class h extends l.FunctionPlugin{arrayformula(e,t){return this.runFunction(e.args,t,this.metadata("ARRAYFORMULA"),e=>e)}arrayformulaArraySize(e,t){if(1!==e.args.length)return n.ArraySize.error();const r=this.metadata("ARRAYFORMULA");return e.args.map(e=>{var n;return this.arraySizeForAst(e,new u.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))})[0]}arrayconstrain(e,t){return this.runFunction(e.args,t,this.metadata("ARRAY_CONSTRAIN"),(e,t,r)=>{t=Math.min(t,e.height()),r=Math.min(r,e.width());const n=e.data,o=[];for(let e=0;e<t;e++)o.push(n[e].slice(0,r));return c.SimpleRangeValue.onlyValues(o)})}arrayconstrainArraySize(e,t){if(3!==e.args.length)return n.ArraySize.error();const r=this.metadata("ARRAY_CONSTRAIN"),o=e.args.map(e=>{var n;return this.arraySizeForAst(e,new u.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))});let{height:s,width:a}=o[0];return e.args[1].type===i.AstNodeType.NUMBER&&(s=Math.min(s,e.args[1].value)),e.args[2].type===i.AstNodeType.NUMBER&&(a=Math.min(a,e.args[2].value)),s<1||a<1||!Number.isInteger(s)||!Number.isInteger(a)?n.ArraySize.error():new n.ArraySize(a,s)}filter(e,t){return this.runFunction(e.args,t,this.metadata("FILTER"),(function(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];for(const t of r)if(e.width()!==t.width()||e.height()!==t.height())return new o.CellError(o.ErrorType.NA,s.ErrorMessage.EqualLength);if(e.width()>1&&e.height()>1)return new o.CellError(o.ErrorType.NA,s.ErrorMessage.WrongDimension);const i=e.data,u=[];for(let t=0;t<e.height();t++){const n=[];for(let o=0;o<e.width();o++){let e=!0;for(const n of r){if(!0!==(0,a.coerceScalarToBoolean)(n.data[t][o])){e=!1;break}}e&&n.push(i[t][o])}n.length>0&&u.push(n)}return u.length>0?c.SimpleRangeValue.onlyValues(u):new o.CellError(o.ErrorType.NA,s.ErrorMessage.EmptyRange)}))}filterArraySize(e,t){if(e.args.length<=1)return n.ArraySize.error();const r=this.metadata("FILTER"),o=e.args.map(e=>{var n;return this.arraySizeForAst(e,new u.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))}),s=Math.max(...o.map(e=>e.width)),i=Math.max(...o.map(e=>e.height));return new n.ArraySize(s,i)}}t.ArrayPlugin=h,h.implementedFunctions={ARRAYFORMULA:{method:"arrayformula",arraySizeMethod:"arrayformulaArraySize",arrayFunction:!0,parameters:[{argumentType:l.FunctionArgumentType.ANY}]},ARRAY_CONSTRAIN:{method:"arrayconstrain",arraySizeMethod:"arrayconstrainArraySize",parameters:[{argumentType:l.FunctionArgumentType.RANGE},{argumentType:l.FunctionArgumentType.INTEGER,minValue:1},{argumentType:l.FunctionArgumentType.INTEGER,minValue:1}],vectorizationForbidden:!0},FILTER:{method:"filter",arraySizeMethod:"filterArraySize",arrayFunction:!0,parameters:[{argumentType:l.FunctionArgumentType.RANGE},{argumentType:l.FunctionArgumentType.RANGE}],repeatLastArgs:1}}},function(e,t,r){"use strict";t.__esModule=!0,t.AbsPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{abs(e,t){return this.runFunction(e.args,t,this.metadata("ABS"),Math.abs)}}t.AbsPlugin=o,o.implementedFunctions={ABS:{method:"abs",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.BitShiftPlugin=void 0;var n=r(74),o=r(132),s=r(165);class i extends s.FunctionPlugin{bitlshift(e,t){return this.runFunction(e.args,t,this.metadata("BITLSHIFT"),a)}bitrshift(e,t){return this.runFunction(e.args,t,this.metadata("BITRSHIFT"),u)}}function a(e,t){return t<0?u(e,-t):c(e*Math.pow(2,t))}function u(e,t){return t<0?a(e,-t):c(Math.floor(e/Math.pow(2,t)))}function c(e){return e>0xffffffffffff?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.BitshiftLong):e}t.BitShiftPlugin=i,i.implementedFunctions={BITLSHIFT:{method:"bitlshift",parameters:[{argumentType:s.FunctionArgumentType.INTEGER,minValue:0},{argumentType:s.FunctionArgumentType.INTEGER,minValue:-53,maxValue:53}]},BITRSHIFT:{method:"bitrshift",parameters:[{argumentType:s.FunctionArgumentType.INTEGER,minValue:0},{argumentType:s.FunctionArgumentType.INTEGER,minValue:-53,maxValue:53}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.BitwiseLogicOperationsPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{bitand(e,t){return this.runFunction(e.args,t,this.metadata("BITAND"),(e,t)=>e&t)}bitor(e,t){return this.runFunction(e.args,t,this.metadata("BITOR"),(e,t)=>e|t)}bitxor(e,t){return this.runFunction(e.args,t,this.metadata("BITXOR"),(e,t)=>e^t)}}t.BitwiseLogicOperationsPlugin=o,o.implementedFunctions={BITAND:{method:"bitand",parameters:[{argumentType:n.FunctionArgumentType.INTEGER,minValue:0},{argumentType:n.FunctionArgumentType.INTEGER,minValue:0}]},BITOR:{method:"bitor",parameters:[{argumentType:n.FunctionArgumentType.INTEGER,minValue:0},{argumentType:n.FunctionArgumentType.INTEGER,minValue:0}]},BITXOR:{method:"bitxor",parameters:[{argumentType:n.FunctionArgumentType.INTEGER,minValue:0},{argumentType:n.FunctionArgumentType.INTEGER,minValue:0}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.BooleanPlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{literalTrue(e,t){return this.runFunction(e.args,t,this.metadata("TRUE"),()=>!0)}literalFalse(e,t){return this.runFunction(e.args,t,this.metadata("FALSE"),()=>!1)}conditionalIf(e,t){return this.runFunction(e.args,t,this.metadata("IF"),(e,t,r)=>e?t:r)}ifs(e,t){return this.runFunction(e.args,t,this.metadata("IFS"),(function(){for(var e=arguments.length,t=new Array(e),r=0;r<e;r++)t[r]=arguments[r];for(let e=0;e<t.length;e+=2)if(t[e])return t[e+1];return new n.CellError(n.ErrorType.NA,o.ErrorMessage.NoConditionMet)}))}and(e,t){return this.runFunction(e.args,t,this.metadata("AND"),(function(){for(var e=arguments.length,t=new Array(e),r=0;r<e;r++)t[r]=arguments[r];return t.filter(e=>void 0!==e).every(e=>!!e)}))}or(e,t){return this.runFunction(e.args,t,this.metadata("OR"),(function(){for(var e=arguments.length,t=new Array(e),r=0;r<e;r++)t[r]=arguments[r];return t.filter(e=>void 0!==e).some(e=>e)}))}not(e,t){return this.runFunction(e.args,t,this.metadata("NOT"),e=>!e)}xor(e,t){return this.runFunction(e.args,t,this.metadata("XOR"),(function(){let e=0;for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return r.filter(e=>void 0!==e).forEach(t=>{t&&e++}),e%2==1}))}switch(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("SWITCH"),(function(e){for(var t=arguments.length,s=new Array(t>1?t-1:0),i=1;i<t;i++)s[i-1]=arguments[i];const a=s.length;let u=0;for(;u+1<a;u+=2)if(!(s[u]instanceof n.CellError)&&r.arithmeticHelper.eq(e,s[u]))return s[u+1];return u<a?s[u]:new n.CellError(n.ErrorType.NA,o.ErrorMessage.NoDefault)}))}iferror(e,t){return this.runFunction(e.args,t,this.metadata("IFERROR"),(e,t)=>e instanceof n.CellError?t:e)}ifna(e,t){return this.runFunction(e.args,t,this.metadata("IFNA"),(e,t)=>e instanceof n.CellError&&e.type===n.ErrorType.NA?t:e)}choose(e,t){return this.runFunction(e.args,t,this.metadata("CHOOSE"),(function(e){return e>(arguments.length<=1?0:arguments.length-1)?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.Selector):e-1+1<1||arguments.length<=e-1+1?void 0:arguments[e-1+1]}))}}t.BooleanPlugin=i,i.implementedFunctions={TRUE:{method:"literalTrue",parameters:[]},FALSE:{method:"literalFalse",parameters:[]},IF:{method:"conditionalIf",parameters:[{argumentType:s.FunctionArgumentType.BOOLEAN},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0},{argumentType:s.FunctionArgumentType.SCALAR,defaultValue:!1,passSubtype:!0}]},IFS:{method:"ifs",parameters:[{argumentType:s.FunctionArgumentType.BOOLEAN},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0}],repeatLastArgs:2},AND:{method:"and",parameters:[{argumentType:s.FunctionArgumentType.BOOLEAN}],repeatLastArgs:1,expandRanges:!0},OR:{method:"or",parameters:[{argumentType:s.FunctionArgumentType.BOOLEAN}],repeatLastArgs:1,expandRanges:!0},XOR:{method:"xor",parameters:[{argumentType:s.FunctionArgumentType.BOOLEAN}],repeatLastArgs:1,expandRanges:!0},NOT:{method:"not",parameters:[{argumentType:s.FunctionArgumentType.BOOLEAN}]},SWITCH:{method:"switch",parameters:[{argumentType:s.FunctionArgumentType.NOERROR},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0}],repeatLastArgs:1},IFERROR:{method:"iferror",parameters:[{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0}]},IFNA:{method:"ifna",parameters:[{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0}]},CHOOSE:{method:"choose",parameters:[{argumentType:s.FunctionArgumentType.INTEGER,minValue:1},{argumentType:s.FunctionArgumentType.SCALAR,passSubtype:!0}],repeatLastArgs:1}}},function(e,t,r){"use strict";t.__esModule=!0,t.CharPlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{char(e,t){return this.runFunction(e.args,t,this.metadata("CHAR"),e=>e<1||e>=256?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.CharacterCodeBounds):String.fromCharCode(Math.trunc(e)))}unichar(e,t){return this.runFunction(e.args,t,this.metadata("CHAR"),e=>e<1||e>=1114112?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.CharacterCodeBounds):String.fromCodePoint(Math.trunc(e)))}}t.CharPlugin=i,i.implementedFunctions={CHAR:{method:"char",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},UNICHAR:{method:"unichar",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.CodePlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{code(e,t){return this.runFunction(e.args,t,this.metadata("CODE"),e=>0===e.length?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.EmptyString):e.charCodeAt(0))}unicode(e,t){return this.runFunction(e.args,t,this.metadata("UNICODE"),e=>{var t;return null!==(t=e.codePointAt(0))&&void 0!==t?t:new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.EmptyString)})}}t.CodePlugin=i,i.implementedFunctions={CODE:{method:"code",parameters:[{argumentType:s.FunctionArgumentType.STRING}]},UNICODE:{method:"unicode",parameters:[{argumentType:s.FunctionArgumentType.STRING}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.CountBlankPlugin=void 0;var n=r(141),o=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.FunctionPlugin{countblank(e,t){return this.runFunction(e.args,t,this.metadata("COUNTBLANK"),(function(){let e=0;for(var t=arguments.length,r=new Array(t),o=0;o<t;o++)r[o]=arguments[o];return r.forEach(t=>{t===n.EmptyValue&&e++}),e}))}}t.CountBlankPlugin=s,s.implementedFunctions={COUNTBLANK:{method:"countblank",parameters:[{argumentType:o.FunctionArgumentType.SCALAR}],repeatLastArgs:1,expandRanges:!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.CountUniquePlugin=void 0;var n=r(74),o=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.FunctionPlugin{countunique(e,t){return this.runFunction(e.args,t,this.metadata("COUNTUNIQUE"),(function(){const e=new Set,t=new Set;for(var r=arguments.length,o=new Array(r),s=0;s<r;s++)o[s]=arguments[s];for(const r of o)r instanceof n.CellError?t.add(r.type):""!==r&&e.add(r);return e.size+t.size}))}}t.CountUniquePlugin=s,s.implementedFunctions={COUNTUNIQUE:{method:"countunique",parameters:[{argumentType:o.FunctionArgumentType.SCALAR}],repeatLastArgs:1,expandRanges:!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.DateTimePlugin=void 0,r(2);var n=r(74),o=r(173),s=r(132),i=r(175),a=r(141),u=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class c extends u.FunctionPlugin{constructor(){super(...arguments),this.isoweeknumCore=e=>{const t=Math.floor(this.dateTimeHelper.relativeNumberToAbsoluteNumber(e)),r=this.dateTimeHelper.numberToSimpleDate(e),n=this.dateTimeHelper.dateToNumber({year:r.year,month:1,day:1}),o=this.dateTimeHelper.relativeNumberToAbsoluteNumber(n),s=o+((4-o)%7+7)%7,i=Math.floor((t-1)/7)-Math.floor((s-1)/7)+1;return 0===i?this.isoweeknumCore(e-7)+1:i},this.days360Core=(e,t,r)=>{const n=this.dateTimeHelper.numberToSimpleDate(e),s=this.dateTimeHelper.numberToSimpleDate(t);let i,a;return r?(i=(0,o.toBasisEU)(n),a=(0,o.toBasisEU)(s)):[i,a]=this.dateTimeHelper.toBasisUS(n,s),360*(a.year-i.year)+30*(a.month-i.month)+a.day-i.day}}date(e,t){return this.runFunction(e.args,t,this.metadata("DATE"),(e,t,r)=>{const o=Math.trunc(r);let i=Math.trunc(t),a=Math.trunc(e);a<this.dateTimeHelper.getEpochYearZero()&&(a+=this.dateTimeHelper.getEpochYearZero());const u=Math.floor((i-1)/12);a+=u,i-=12*u;const c={year:a,month:i,day:1};if(this.dateTimeHelper.isValidDate(c)){let e=this.dateTimeHelper.dateToNumber(c)+(o-1);return e=this.dateTimeHelper.getWithinBounds(e),void 0===e?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.DateBounds):e}return new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.InvalidDate)})}time(e,t){return this.runFunction(e.args,t,this.metadata("TIME"),(e,t,r)=>{const i=(0,o.timeToNumber)({hours:Math.trunc(e),minutes:Math.trunc(t),seconds:Math.trunc(r)});return i<0?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.NegativeTime):i%1})}eomonth(e,t){return this.runFunction(e.args,t,this.metadata("EOMONTH"),(e,t)=>{const r=this.dateTimeHelper.numberToSimpleDate(e);let i=this.dateTimeHelper.dateToNumber(this.dateTimeHelper.endOfMonth((0,o.offsetMonth)(r,t)));return i=this.dateTimeHelper.getWithinBounds(i),void 0===i?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.DateBounds):i})}day(e,t){return this.runFunction(e.args,t,this.metadata("DAY"),e=>this.dateTimeHelper.numberToSimpleDate(e).day)}days(e,t){return this.runFunction(e.args,t,this.metadata("DAYS"),(e,t)=>Math.trunc(e)-Math.trunc(t))}month(e,t){return this.runFunction(e.args,t,this.metadata("MONTH"),e=>this.dateTimeHelper.numberToSimpleDate(e).month)}year(e,t){return this.runFunction(e.args,t,this.metadata("YEAR"),e=>this.dateTimeHelper.numberToSimpleDate(e).year)}hour(e,t){return this.runFunction(e.args,t,this.metadata("HOUR"),e=>(0,o.numberToSimpleTime)((0,o.roundToNearestSecond)(e)%1).hours)}minute(e,t){return this.runFunction(e.args,t,this.metadata("MINUTE"),e=>(0,o.numberToSimpleTime)((0,o.roundToNearestSecond)(e)%1).minutes)}second(e,t){return this.runFunction(e.args,t,this.metadata("SECOND"),e=>(0,o.numberToSimpleTime)((0,o.roundToNearestSecond)(e)%1).seconds)}text(e,t){return this.runFunction(e.args,t,this.metadata("TEXT"),(e,t)=>(0,i.format)(e,t,this.config,this.dateTimeHelper))}weekday(e,t){return this.runFunction(e.args,t,this.metadata("WEEKDAY"),(e,t)=>{const r=Math.floor(this.dateTimeHelper.relativeNumberToAbsoluteNumber(e));if(3===t)return(r-1)%7;const o=d.get(t);return void 0===o?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.BadMode):(r-o)%7+1})}weeknum(e,t){return this.runFunction(e.args,t,this.metadata("WEEKNUM"),(e,t)=>{const r=Math.floor(this.dateTimeHelper.relativeNumberToAbsoluteNumber(e)),o=this.dateTimeHelper.numberToSimpleDate(e),i=this.dateTimeHelper.dateToNumber({year:o.year,month:1,day:1}),a=this.dateTimeHelper.relativeNumberToAbsoluteNumber(i);if(21===t)return this.isoweeknumCore(e);const u=d.get(t);return void 0===u?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.BadMode):Math.floor((r-u)/7)-Math.floor((a-u)/7)+1})}isoweeknum(e,t){return this.runFunction(e.args,t,this.metadata("ISOWEEKNUM"),this.isoweeknumCore)}datevalue(e,t){return this.runFunction(e.args,t,this.metadata("DATEVALUE"),e=>{const{dateTime:t}=this.dateTimeHelper.parseDateTimeFromConfigFormats(e);return void 0===t?new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.IncorrectDateTime):(0,o.instanceOfSimpleDate)(t)?((0,o.instanceOfSimpleTime)(t)?Math.trunc((0,o.timeToNumber)(t)):0)+this.dateTimeHelper.dateToNumber(t):0})}timevalue(e,t){return this.runFunction(e.args,t,this.metadata("TIMEVALUE"),e=>{const t=this.dateTimeHelper.dateStringToDateNumber(e);return void 0===t?new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.IncorrectDateTime):(0,a.getRawValue)(t)%1})}now(e,t){return this.runFunction(e.args,t,this.metadata("NOW"),()=>{const e=new Date(Date.now());return(0,o.timeToNumber)({hours:e.getHours(),minutes:e.getMinutes(),seconds:e.getSeconds()})+this.dateTimeHelper.dateToNumber({year:e.getFullYear(),month:e.getMonth()+1,day:e.getDate()})})}today(e,t){return this.runFunction(e.args,t,this.metadata("TODAY"),()=>{const e=new Date(Date.now());return this.dateTimeHelper.dateToNumber({year:e.getFullYear(),month:e.getMonth()+1,day:e.getDate()})})}edate(e,t){return this.runFunction(e.args,t,this.metadata("EDATE"),(e,t)=>{const r=this.dateTimeHelper.numberToSimpleDate(e),i=(0,o.truncateDayInMonth)((0,o.offsetMonth)(r,t));let a=this.dateTimeHelper.dateToNumber(i);return a=this.dateTimeHelper.getWithinBounds(a),void 0===a?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.DateBounds):a})}datedif(e,t){return this.runFunction(e.args,t,this.metadata("DATEDIF"),(e,t,r)=>{if(e>t)return new n.CellError(n.ErrorType.NUM,s.ErrorMessage.StartEndDate);if("D"===r)return Math.floor(t)-Math.floor(e);const o=this.dateTimeHelper.numberToSimpleDate(e),i=this.dateTimeHelper.numberToSimpleDate(t);switch(r){case"M":return 12*(i.year-o.year)+(i.month-o.month)-(i.day<o.day?1:0);case"YM":return(i.month-o.month+12-(i.day<o.day?1:0))%12;case"Y":return i.month>o.month||i.month===o.month&&i.day>=o.day?i.year-o.year:i.year-o.year-1;case"MD":if(i.day>=o.day)return i.day-o.day;{const e=1===i.month?12:i.month-1,t=1===i.month?i.year-1:i.year;return this.dateTimeHelper.daysInMonth(t,e)+i.day-o.day}case"YD":return i.month>o.month||i.month===o.month&&i.day>=o.day?Math.floor(t)-this.dateTimeHelper.dateToNumber({year:i.year,month:o.month,day:o.day}):Math.floor(t)-Math.floor(e)-365*(i.year-o.year-1)-this.dateTimeHelper.leapYearsCount(i.year-1)+this.dateTimeHelper.leapYearsCount(o.year);default:return new n.CellError(n.ErrorType.NUM,s.ErrorMessage.BadMode)}})}days360(e,t){return this.runFunction(e.args,t,this.metadata("DAYS360"),this.days360Core)}yearfrac(e,t){return this.runFunction(e.args,t,this.metadata("YEARFRAC"),(e,t,r)=>{switch((e=Math.trunc(e))>(t=Math.trunc(t))&&([e,t]=[t,e]),r){case 0:return this.days360Core(e,t,!1)/360;case 1:return(t-e)/this.dateTimeHelper.yearLengthForBasis(this.dateTimeHelper.numberToSimpleDate(e),this.dateTimeHelper.numberToSimpleDate(t));case 2:return(t-e)/360;case 3:return(t-e)/365;case 4:return this.days360Core(e,t,!0)/360}throw new Error("Should not be reachable.")})}interval(e,t){return this.runFunction(e.args,t,this.metadata("INTERVAL"),e=>{const t=(e=Math.trunc(e))%60,r=(e=Math.trunc(e/60))%60,n=(e=Math.trunc(e/60))%24,o=(e=Math.trunc(e/24))%30,s=(e=Math.trunc(e/30))%12,i=Math.trunc(e/12);return"P"+(i>0?i+"Y":"")+(s>0?s+"M":"")+(o>0?o+"D":"")+"T"+(n>0?n+"H":"")+(r>0?r+"M":"")+(t>0?t+"S":"")})}networkdays(e,t){return this.runFunction(e.args,t,this.metadata("NETWORKDAYS"),(e,t,r)=>this.networkdayscore(e,t,1,r))}networkdaysintl(e,t){return this.runFunction(e.args,t,this.metadata("NETWORKDAYS.INTL"),(e,t,r,n)=>this.networkdayscore(e,t,r,n))}workday(e,t){return this.runFunction(e.args,t,this.metadata("WORKDAY"),(e,t,r)=>this.workdaycore(e,t,1,r))}workdayintl(e,t){return this.runFunction(e.args,t,this.metadata("WORKDAY.INTL"),(e,t,r,n)=>this.workdaycore(e,t,r,n))}networkdayscore(e,t,r,o){let s=1;(e=Math.trunc(e))>(t=Math.trunc(t))&&([e,t]=[t,e],s=-1);const i=h(r);if(i instanceof n.CellError)return i;const a=this.simpleRangeToFilteredHolidays(i,o);return a instanceof n.CellError?a:s*this.countWorkdays(e,t,i,a)}workdaycore(e,t,r,o){e=Math.trunc(e),t=Math.trunc(t);const s=h(r);if(s instanceof n.CellError)return s;const i=this.simpleRangeToFilteredHolidays(s,o);if(i instanceof n.CellError)return i;if(t>0){let r=1;for(;this.countWorkdays(e+1,e+r,s,i)<t;)r*=2;let n=1;for(;n+1<r;){const o=Math.trunc((n+r)/2);this.countWorkdays(e+1,e+o,s,i)<t?n=o:r=o}return e+r}if(t<0){t*=-1;let r=1;for(;this.countWorkdays(e-r,e-1,s,i)<t;)r*=2;let n=1;for(;n+1<r;){const o=Math.trunc((n+r)/2);this.countWorkdays(e-o,e-1,s,i)<t?n=o:r=o}return e-r}return e}countWorkdays(e,t,r,n){const o=Math.floor(this.dateTimeHelper.relativeNumberToAbsoluteNumber(t)),s=Math.floor(this.dateTimeHelper.relativeNumberToAbsoluteNumber(e));let i=0;for(let e=0;e<7;e++)"0"===r.charAt(e)&&(i+=Math.floor((o+6-e)/7),i-=Math.floor((s-1+6-e)/7));return i-=l(t+1,n)-l(e,n),i}simpleRangeToFilteredHolidays(e,t){var r;const o=null!==(r=null==t?void 0:t.valuesFromTopLeftCorner())&&void 0!==r?r:[];for(const e of o)if(e instanceof n.CellError)return e;const i=[];for(const e of o)if(e!==a.EmptyValue){if(!(0,a.isExtendedNumber)(e))return new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.WrongType);i.push(Math.trunc((0,a.getRawValue)(e)))}return[...new Set(i)].sort((e,t)=>e-t).filter(t=>{const r=(this.dateTimeHelper.relativeNumberToAbsoluteNumber(t)-1)%7;return"0"===e.charAt(r)})}}function l(e,t){if(0===t.length)return 0;if(e<=t[0])return 0;if(t[t.length-1]<e)return t.length;let r=0,n=t.length-1;for(;r+1<n;){const o=Math.floor((n+r)/2);t[o]>=e?n=o:r=o}return n}function h(e){var t;return"number"!=typeof e&&"string"!=typeof e?new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.WrongType):"string"==typeof e?7===e.length&&/^(0|1)*$/.test(e)&&"1111111"!==e?e:new n.CellError(n.ErrorType.NUM,s.ErrorMessage.WeekendString):null!==(t=p.get(e))&&void 0!==t?t:new n.CellError(n.ErrorType.NUM,s.ErrorMessage.BadMode)}t.DateTimePlugin=c,c.implementedFunctions={DATE:{method:"date",parameters:[{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.NUMBER}],returnNumberType:a.NumberType.NUMBER_DATE},TIME:{method:"time",parameters:[{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.NUMBER}],returnNumberType:a.NumberType.NUMBER_TIME},MONTH:{method:"month",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},YEAR:{method:"year",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},HOUR:{method:"hour",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},MINUTE:{method:"minute",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},SECOND:{method:"second",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},TEXT:{method:"text",parameters:[{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.STRING}]},EOMONTH:{method:"eomonth",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER}],returnNumberType:a.NumberType.NUMBER_DATE},DAY:{method:"day",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},DAYS:{method:"days",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},WEEKDAY:{method:"weekday",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,defaultValue:1}]},WEEKNUM:{method:"weeknum",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,defaultValue:1}]},ISOWEEKNUM:{method:"isoweeknum",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},DATEVALUE:{method:"datevalue",parameters:[{argumentType:u.FunctionArgumentType.STRING}],returnNumberType:a.NumberType.NUMBER_DATE},TIMEVALUE:{method:"timevalue",parameters:[{argumentType:u.FunctionArgumentType.STRING}],returnNumberType:a.NumberType.NUMBER_TIME},NOW:{method:"now",parameters:[],isVolatile:!0,returnNumberType:a.NumberType.NUMBER_DATETIME},TODAY:{method:"today",parameters:[],isVolatile:!0,returnNumberType:a.NumberType.NUMBER_DATE},EDATE:{method:"edate",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER}],returnNumberType:a.NumberType.NUMBER_DATE},DAYS360:{method:"days360",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.BOOLEAN,defaultValue:!1}]},DATEDIF:{method:"datedif",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.STRING}]},YEARFRAC:{method:"yearfrac",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.INTEGER,defaultValue:0,minValue:0,maxValue:4}]},INTERVAL:{method:"interval",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0}]},NETWORKDAYS:{method:"networkdays",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.RANGE,optionalArg:!0}]},"NETWORKDAYS.INTL":{method:"networkdaysintl",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NOERROR,defaultValue:1},{argumentType:u.FunctionArgumentType.RANGE,optionalArg:!0}]},WORKDAY:{method:"workday",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.RANGE,optionalArg:!0}]},"WORKDAY.INTL":{method:"workdayintl",parameters:[{argumentType:u.FunctionArgumentType.NUMBER,minValue:0},{argumentType:u.FunctionArgumentType.NUMBER},{argumentType:u.FunctionArgumentType.NOERROR,defaultValue:1},{argumentType:u.FunctionArgumentType.RANGE,optionalArg:!0}]}};const d=new Map([[1,0],[2,1],[11,1],[12,2],[13,3],[14,4],[15,5],[16,6],[17,0]]),p=new Map([[1,"0000011"],[2,"1000001"],[3,"1100000"],[4,"0110000"],[5,"0011000"],[6,"0001100"],[7,"0000110"],[11,"0000001"],[12,"1000000"],[13,"0100000"],[14,"0010000"],[15,"0001000"],[16,"0000100"],[17,"0000010"]])},function(e,t,r){"use strict";t.__esModule=!0,t.DegreesPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{degrees(e,t){return this.runFunction(e.args,t,this.metadata("DEGREES"),e=>e*(180/Math.PI))}}t.DegreesPlugin=o,o.implementedFunctions={DEGREES:{method:"degrees",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.DeltaPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{delta(e,t){return this.runFunction(e.args,t,this.metadata("DELTA"),(e,t)=>e===t?1:0)}}t.DeltaPlugin=o,o.implementedFunctions={DELTA:{method:"delta",parameters:[{argumentType:n.FunctionArgumentType.NUMBER},{argumentType:n.FunctionArgumentType.NUMBER,defaultValue:0}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.ExpPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{exp(e,t){return this.runFunction(e.args,t,this.metadata("EXP"),Math.exp)}}t.ExpPlugin=o,o.implementedFunctions={EXP:{method:"exp",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.FinancialPlugin=void 0,r(2);var n=r(74),o=r(132),s=r(141),i=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends i.FunctionPlugin{pmt(e,t){return this.runFunction(e.args,t,this.metadata("PMT"),u)}ipmt(e,t){return this.runFunction(e.args,t,this.metadata("IPMT"),c)}ppmt(e,t){return this.runFunction(e.args,t,this.metadata("PPMT"),h)}fv(e,t){return this.runFunction(e.args,t,this.metadata("FV"),l)}cumipmt(e,t){return this.runFunction(e.args,t,this.metadata("CUMIPMT"),(e,t,r,s,i,a)=>{if(s>i)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.EndStartPeriod);let u=0;for(let n=s;n<=i;n++)u+=c(e,n,t,r,0,a);return u})}cumprinc(e,t){return this.runFunction(e.args,t,this.metadata("CUMPRINC"),(e,t,r,s,i,a)=>{if(s>i)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.EndStartPeriod);let u=0;for(let n=s;n<=i;n++)u+=h(e,n,t,r,0,a);return u})}db(e,t){return this.runFunction(e.args,t,this.metadata("DB"),(e,t,r,s,i)=>{if(12===i&&s>r||s>r+1)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.PeriodLong);if(t>=e)return 0;const a=Math.round(1e3*(1-Math.pow(t/e,1/r)))/1e3,u=e*a*i/12;if(1===s)return u;let c=u;for(let t=0;t<s-2;t++)c+=(e-c)*a;return s===r+1?(e-c)*a*(12-i)/12:(e-c)*a})}ddb(e,t){return this.runFunction(e.args,t,this.metadata("DDB"),(e,t,r,o,s)=>{if(o>r)return new n.CellError(n.ErrorType.NUM);let i,a=s/r;a>=1?(a=1,i=1===o?e:0):i=e*Math.pow(1-a,o-1);const u=e*Math.pow(1-a,o);return Math.max(i-Math.max(t,u),0)})}dollarde(e,t){return this.runFunction(e.args,t,this.metadata("DOLLARDE"),(e,t)=>{if(t<1)return new n.CellError(n.ErrorType.DIV_BY_ZERO);for(t=Math.trunc(t);t>10;)t/=10;return Math.trunc(e)+10*(e-Math.trunc(e))/t})}dollarfr(e,t){return this.runFunction(e.args,t,this.metadata("DOLLARFR"),(e,t)=>{if(t<1)return new n.CellError(n.ErrorType.DIV_BY_ZERO);for(t=Math.trunc(t);t>10;)t/=10;return Math.trunc(e)+(e-Math.trunc(e))*t/10})}effect(e,t){return this.runFunction(e.args,t,this.metadata("EFFECT"),(e,t)=>(t=Math.trunc(t),Math.pow(1+e/t,t)-1))}ispmt(e,t){return this.runFunction(e.args,t,this.metadata("ISPMT"),(e,t,r,o)=>0===r?new n.CellError(n.ErrorType.DIV_BY_ZERO):o*e*(t/r-1))}nominal(e,t){return this.runFunction(e.args,t,this.metadata("NOMINAL"),(e,t)=>(t=Math.trunc(t),(Math.pow(e+1,1/t)-1)*t))}nper(e,t){return this.runFunction(e.args,t,this.metadata("NPER"),(e,t,r,o,s)=>0===e?0===t?new n.CellError(n.ErrorType.DIV_BY_ZERO):(-r-o)/t:(s&&(t*=1+e),Math.log((t-o*e)/(r*e+t))/Math.log(1+e)))}rate(e,t){return this.runFunction(e.args,t,this.metadata("RATE"),(e,t,r,o,s,i)=>{if(i<=-1)return new n.CellError(n.ErrorType.VALUE);let a=i;s=s?1:0;for(let i=0;i<50;i++){if(a<=-1)return new n.CellError(n.ErrorType.NUM);let i,u;if(Math.abs(a)<1e-7)i=r*(1+e*a)+t*(1+a*s)*e+o;else{const n=Math.pow(1+a,e);i=r*n+t*(1/a+s)*(n-1)+o}if(Math.abs(i)<1e-7)return a;if(Math.abs(a)<1e-7)u=r*e+t*s*e;else{const n=Math.pow(1+a,e),o=e*Math.pow(1+a,e-1);u=r*o+t*(1/a+s)*o+t*(-1/(a*a))*(n-1)}a-=i/u}return new n.CellError(n.ErrorType.NUM)})}pv(e,t){return this.runFunction(e.args,t,this.metadata("PV"),(e,t,r,o,s)=>(s=s?1:0,-1===e?0===t?new n.CellError(n.ErrorType.NUM):new n.CellError(n.ErrorType.DIV_BY_ZERO):0===e?-r*t-o:((1-Math.pow(1+e,t))*r*(1+e*s)/e-o)/Math.pow(1+e,t)))}rri(e,t){return this.runFunction(e.args,t,this.metadata("RRI"),(e,t,r)=>0===t||r<0&&t>0||r>0&&t<0?new n.CellError(n.ErrorType.NUM):Math.pow(r/t,1/e)-1)}sln(e,t){return this.runFunction(e.args,t,this.metadata("SLN"),(e,t,r)=>0===r?new n.CellError(n.ErrorType.DIV_BY_ZERO):(e-t)/r)}syd(e,t){return this.runFunction(e.args,t,this.metadata("SYD"),(e,t,r,o)=>o>r?new n.CellError(n.ErrorType.NUM):(e-t)*(r-o+1)*2/(r*(r+1)))}tbilleq(e,t){return this.runFunction(e.args,t,this.metadata("TBILLEQ"),(e,t,r)=>{if((e=Math.round(e))>=(t=Math.round(t)))return new n.CellError(n.ErrorType.NUM);const o=this.dateTimeHelper.numberToSimpleDate(e),s=this.dateTimeHelper.numberToSimpleDate(t);if(s.year>o.year+1||s.year===o.year+1&&(s.month>o.month||s.month===o.month&&s.day>o.day))return new n.CellError(n.ErrorType.NUM);const i=360-r*(t-e);return 0===i?0:i<0?new n.CellError(n.ErrorType.NUM):365*r/i})}tbillprice(e,t){return this.runFunction(e.args,t,this.metadata("TBILLPRICE"),(e,t,r)=>{if((e=Math.round(e))>=(t=Math.round(t)))return new n.CellError(n.ErrorType.NUM);const o=this.dateTimeHelper.numberToSimpleDate(e),s=this.dateTimeHelper.numberToSimpleDate(t);if(s.year>o.year+1||s.year===o.year+1&&(s.month>o.month||s.month===o.month&&s.day>o.day))return new n.CellError(n.ErrorType.NUM);const i=360-r*(t-e);return 0===i?0:i<0?new n.CellError(n.ErrorType.NUM):100*(1-r*(t-e)/360)})}tbillyield(e,t){return this.runFunction(e.args,t,this.metadata("TBILLYIELD"),(e,t,r)=>{if((e=Math.round(e))>=(t=Math.round(t)))return new n.CellError(n.ErrorType.NUM);const o=this.dateTimeHelper.numberToSimpleDate(e),s=this.dateTimeHelper.numberToSimpleDate(t);return s.year>o.year+1||s.year===o.year+1&&(s.month>o.month||s.month===o.month&&s.day>o.day)?new n.CellError(n.ErrorType.NUM):360*(100-r)/(r*(t-e))})}fvschedule(e,t){return this.runFunction(e.args,t,this.metadata("FVSCHEDULE"),(e,t)=>{const r=t.valuesFromTopLeftCorner();for(const e of r)if(e instanceof n.CellError)return e;for(const t of r)if((0,s.isExtendedNumber)(t))e*=1+(0,s.getRawValue)(t);else if(t!==s.EmptyValue)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NumberExpected);return e})}npv(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("NPV"),(function(e){for(var t=arguments.length,o=new Array(t>1?t-1:0),s=1;s<t;s++)o[s-1]=arguments[s];const i=r.arithmeticHelper.coerceNumbersExactRanges(o);return i instanceof n.CellError?i:d(e,i)}))}mirr(e,t){return this.runFunction(e.args,t,this.metadata("MIRR"),(e,t,r)=>{const o=this.arithmeticHelper.manyToExactNumbers(e.valuesFromTopLeftCorner());if(o instanceof n.CellError)return o;let s=!1,i=!1;const a=[],u=[];for(const e of o)e>0?(s=!0,a.push(e),u.push(0)):e<0?(i=!0,u.push(e),a.push(0)):(u.push(0),a.push(0));if(!s||!i)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const c=o.length,l=d(r,a);if(l instanceof n.CellError)return l;const h=d(t,u);return h instanceof n.CellError?h:Math.pow(-l*Math.pow(1+r,c)/h/(1+t),1/(c-1))-1})}pduration(e,t){return this.runFunction(e.args,t,this.metadata("PDURATION"),(e,t,r)=>(Math.log(r)-Math.log(t))/Math.log(1+e))}xnpv(e,t){return this.runFunction(e.args,t,this.metadata("XNPV"),(e,t,r)=>{const s=t.valuesFromTopLeftCorner();for(const e of s)if("number"!=typeof e)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NumberExpected);const i=s,a=r.valuesFromTopLeftCorner();for(const e of a)if("number"!=typeof e)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NumberExpected);const u=a;if(u.length!==i.length)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.EqualLength);const c=u.length;let l=0;if(u[0]<0)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);for(let t=0;t<c;t++){if(u[t]=Math.floor(u[t]),u[t]<u[0])return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);l+=i[t]/Math.pow(1+e,(u[t]-u[0])/365)}return l})}}function u(e,t,r,n,o){if(0===e)return(-r-n)/t;{const s=Math.pow(1+e,t);return(n*e+r*e*s)*(o?1/(1+e):1)/(1-s)}}function c(e,t,r,n,o,s){const i=u(e,r,n,o,s);return 1===t?e*(s?0:-n):e*(s?l(e,t-2,i,n,s)-i:l(e,t-1,i,n,s))}function l(e,t,r,n,o){if(0===e)return-n-r*t;{const s=Math.pow(1+e,t);return r*(o?1+e:1)*(1-s)/e-n*s}}function h(e,t,r,n,o,s){return u(e,r,n,o,s)-c(e,t,r,n,o,s)}function d(e,t){let r=0;for(let o=t.length-1;o>=0;o--){if(r+=t[o],-1===e){if(0===r)continue;return new n.CellError(n.ErrorType.DIV_BY_ZERO)}r/=1+e}return r}t.FinancialPlugin=a,a.implementedFunctions={PMT:{method:"pmt",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},IPMT:{method:"ipmt",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},PPMT:{method:"ppmt",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},FV:{method:"fv",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},CUMIPMT:{method:"cumipmt",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.INTEGER,minValue:1},{argumentType:i.FunctionArgumentType.INTEGER,minValue:1},{argumentType:i.FunctionArgumentType.INTEGER,minValue:0,maxValue:1}],returnNumberType:s.NumberType.NUMBER_CURRENCY},CUMPRINC:{method:"cumprinc",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.INTEGER,minValue:1},{argumentType:i.FunctionArgumentType.INTEGER,minValue:1},{argumentType:i.FunctionArgumentType.INTEGER,minValue:0,maxValue:1}],returnNumberType:s.NumberType.NUMBER_CURRENCY},DB:{method:"db",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.INTEGER,minValue:0},{argumentType:i.FunctionArgumentType.INTEGER,minValue:0},{argumentType:i.FunctionArgumentType.INTEGER,minValue:1,maxValue:12,defaultValue:12}],returnNumberType:s.NumberType.NUMBER_CURRENCY},DDB:{method:"ddb",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.INTEGER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0,defaultValue:2}],returnNumberType:s.NumberType.NUMBER_CURRENCY},DOLLARDE:{method:"dollarde",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0}]},DOLLARFR:{method:"dollarfr",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0}]},EFFECT:{method:"effect",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:1}],returnNumberType:s.NumberType.NUMBER_PERCENT},ISPMT:{method:"ispmt",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER}]},NOMINAL:{method:"nominal",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:1}],returnNumberType:s.NumberType.NUMBER_PERCENT},NPER:{method:"nper",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0}]},PV:{method:"pv",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},RATE:{method:"rate",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:i.FunctionArgumentType.NUMBER,defaultValue:.1}],returnNumberType:s.NumberType.NUMBER_PERCENT},RRI:{method:"rri",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER}],returnNumberType:s.NumberType.NUMBER_PERCENT},SLN:{method:"sln",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER}],returnNumberType:s.NumberType.NUMBER_CURRENCY},SYD:{method:"syd",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},TBILLEQ:{method:"tbilleq",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0}],returnNumberType:s.NumberType.NUMBER_PERCENT},TBILLPRICE:{method:"tbillprice",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0}],returnNumberType:s.NumberType.NUMBER_CURRENCY},TBILLYIELD:{method:"tbillyield",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0}],returnNumberType:s.NumberType.NUMBER_PERCENT},FVSCHEDULE:{method:"fvschedule",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.RANGE}],returnNumberType:s.NumberType.NUMBER_CURRENCY},NPV:{method:"npv",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.ANY}],repeatLastArgs:1,returnNumberType:s.NumberType.NUMBER_CURRENCY},MIRR:{method:"mirr",parameters:[{argumentType:i.FunctionArgumentType.RANGE},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER}],returnNumberType:s.NumberType.NUMBER_PERCENT},PDURATION:{method:"pduration",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:0}]},XNPV:{method:"xnpv",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,greaterThan:-1},{argumentType:i.FunctionArgumentType.RANGE},{argumentType:i.FunctionArgumentType.RANGE}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.FormulaTextPlugin=void 0;var n=r(74),o=r(132),s=r(216),i=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends s.FunctionPlugin{formulatext(e,t){return this.runFunctionWithReferenceArgument(e.args,t,this.metadata("FORMULATEXT"),()=>new n.CellError(n.ErrorType.NA,o.ErrorMessage.WrongArgNumber),e=>{var t;return null!==(t=this.serialization.getCellFormula(e))&&void 0!==t?t:new n.CellError(n.ErrorType.NA,o.ErrorMessage.Formula)})}}t.FormulaTextPlugin=a,a.implementedFunctions={FORMULATEXT:{method:"formulatext",parameters:[{argumentType:i.FunctionArgumentType.NOERROR}],doesNotNeedArgumentsToBeComputed:!0,isDependentOnSheetStructureChange:!0,vectorizationForbidden:!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.HyperlinkPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{hyperlink(e,t){return this.runFunction(e.args,t,this.metadata("HYPERLINK"),(t,r)=>(e.hyperlink=t,null!=r?r:t))}}t.HyperlinkPlugin=o,o.implementedFunctions={HYPERLINK:{method:"hyperlink",parameters:[{argumentType:n.FunctionArgumentType.STRING},{argumentType:n.FunctionArgumentType.STRING,optionalArg:!0}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.InformationPlugin=void 0;var n=r(74),o=r(151),s=r(132),i=r(78),a=r(141),u=r(140),c=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class l extends c.FunctionPlugin{isbinary(e,t){return this.runFunction(e.args,t,this.metadata("ISBINARY"),e=>/^[01]{1,10}$/.test(e))}iserr(e,t){return this.runFunction(e.args,t,this.metadata("ISERR"),e=>e instanceof n.CellError&&e.type!==n.ErrorType.NA)}iserror(e,t){return this.runFunction(e.args,t,this.metadata("ISERROR"),e=>e instanceof n.CellError)}isformula(e,t){return this.runFunctionWithReferenceArgument(e.args,t,this.metadata("ISFORMULA"),()=>new n.CellError(n.ErrorType.NA,s.ErrorMessage.WrongArgNumber),e=>this.dependencyGraph.addressMapping.getCell(e)instanceof o.FormulaVertex)}isblank(e,t){return this.runFunction(e.args,t,this.metadata("ISBLANK"),e=>e===a.EmptyValue)}isna(e,t){return this.runFunction(e.args,t,this.metadata("ISNA"),e=>e instanceof n.CellError&&e.type==n.ErrorType.NA)}isnumber(e,t){return this.runFunction(e.args,t,this.metadata("ISNUMBER"),a.isExtendedNumber)}islogical(e,t){return this.runFunction(e.args,t,this.metadata("ISLOGICAL"),e=>"boolean"==typeof e)}isref(e,t){return this.runFunction(e.args,t,this.metadata("ISREF"),e=>e instanceof n.CellError&&(e.type==n.ErrorType.REF||e.type==n.ErrorType.CYCLE))}istext(e,t){return this.runFunction(e.args,t,this.metadata("ISTEXT"),e=>"string"==typeof e)}isnontext(e,t){return this.runFunction(e.args,t,this.metadata("ISNONTEXT"),e=>!("string"==typeof e))}column(e,t){return this.runFunctionWithReferenceArgument(e.args,t,this.metadata("COLUMN"),()=>t.formulaAddress.col+1,e=>e.col+1)}columns(e,t){if(1!==e.args.length)return new n.CellError(n.ErrorType.NA,s.ErrorMessage.WrongArgNumber);if(e.args.some(e=>e.type===i.AstNodeType.EMPTY))return new n.CellError(n.ErrorType.NUM,s.ErrorMessage.EmptyArg);let r=e.args[0];for(;r.type===i.AstNodeType.PARENTHESIS;)r=r.expression;if(r.type===i.AstNodeType.CELL_RANGE||r.type===i.AstNodeType.COLUMN_RANGE)return r.end.col-r.start.col+1;if(r.type===i.AstNodeType.CELL_REFERENCE)return 1;if(r.type===i.AstNodeType.ROW_RANGE)return this.config.maxColumns;{const e=this.evaluateAst(r,t);return e instanceof u.SimpleRangeValue?e.width():e instanceof n.CellError?e:1}}row(e,t){return this.runFunctionWithReferenceArgument(e.args,t,this.metadata("ROW"),()=>t.formulaAddress.row+1,e=>e.row+1)}rows(e,t){if(1!==e.args.length)return new n.CellError(n.ErrorType.NA,s.ErrorMessage.WrongArgNumber);if(e.args.some(e=>e.type===i.AstNodeType.EMPTY))return new n.CellError(n.ErrorType.NUM,s.ErrorMessage.EmptyArg);let r=e.args[0];for(;r.type===i.AstNodeType.PARENTHESIS;)r=r.expression;if(r.type===i.AstNodeType.CELL_RANGE||r.type===i.AstNodeType.ROW_RANGE)return r.end.row-r.start.row+1;if(r.type===i.AstNodeType.CELL_REFERENCE)return 1;if(r.type===i.AstNodeType.COLUMN_RANGE)return this.config.maxRows;{const e=this.evaluateAst(r,t);return e instanceof u.SimpleRangeValue?e.height():e instanceof n.CellError?e:1}}index(e,t){return this.runFunction(e.args,t,this.metadata("INDEX"),(e,t,r)=>{var o,i,a,u,c,l;return r<1||t<1?new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.LessThanOne):r>e.width()||t>e.height()?new n.CellError(n.ErrorType.NUM,s.ErrorMessage.ValueLarge):null!==(l=null!==(a=null===(i=null===(o=null==e?void 0:e.data)||void 0===o?void 0:o[t-1])||void 0===i?void 0:i[r-1])&&void 0!==a?a:null===(c=null===(u=null==e?void 0:e.data)||void 0===u?void 0:u[0])||void 0===c?void 0:c[0])&&void 0!==l?l:new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.CellRangeExpected)})}na(e,t){return new n.CellError(n.ErrorType.NA)}sheet(e,t){return this.runFunctionWithReferenceArgument(e.args,t,this.metadata("SHEET"),()=>t.formulaAddress.sheet+1,e=>e.sheet+1,e=>{const t=this.dependencyGraph.sheetMapping.get(e);return void 0!==t?t+1:new n.CellError(n.ErrorType.NA,s.ErrorMessage.SheetRef)})}sheets(e,t){return this.runFunctionWithReferenceArgument(e.args,t,this.metadata("SHEETS"),()=>this.dependencyGraph.sheetMapping.numberOfSheets(),()=>1,()=>new n.CellError(n.ErrorType.VALUE,s.ErrorMessage.CellRefExpected))}}t.InformationPlugin=l,l.implementedFunctions={COLUMN:{method:"column",parameters:[{argumentType:c.FunctionArgumentType.NOERROR,optionalArg:!0}],isDependentOnSheetStructureChange:!0,doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0},COLUMNS:{method:"columns",parameters:[{argumentType:c.FunctionArgumentType.RANGE}],isDependentOnSheetStructureChange:!0,doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0},ISBINARY:{method:"isbinary",parameters:[{argumentType:c.FunctionArgumentType.STRING}]},ISERR:{method:"iserr",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISFORMULA:{method:"isformula",parameters:[{argumentType:c.FunctionArgumentType.NOERROR}],doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0},ISNA:{method:"isna",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISREF:{method:"isref",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}],vectorizationForbidden:!0},ISERROR:{method:"iserror",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISBLANK:{method:"isblank",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISNUMBER:{method:"isnumber",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISLOGICAL:{method:"islogical",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISTEXT:{method:"istext",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},ISNONTEXT:{method:"isnontext",parameters:[{argumentType:c.FunctionArgumentType.SCALAR}]},INDEX:{method:"index",parameters:[{argumentType:c.FunctionArgumentType.RANGE},{argumentType:c.FunctionArgumentType.NUMBER},{argumentType:c.FunctionArgumentType.NUMBER,defaultValue:1}]},NA:{method:"na",parameters:[]},ROW:{method:"row",parameters:[{argumentType:c.FunctionArgumentType.NOERROR,optionalArg:!0}],isDependentOnSheetStructureChange:!0,doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0},ROWS:{method:"rows",parameters:[{argumentType:c.FunctionArgumentType.RANGE}],isDependentOnSheetStructureChange:!0,doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0},SHEET:{method:"sheet",parameters:[{argumentType:c.FunctionArgumentType.STRING}],doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0},SHEETS:{method:"sheets",parameters:[{argumentType:c.FunctionArgumentType.STRING}],doesNotNeedArgumentsToBeComputed:!0,vectorizationForbidden:!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.IsEvenPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{iseven(e,t){return this.runFunction(e.args,t,this.metadata("ISEVEN"),e=>e%2==0)}}t.IsEvenPlugin=o,o.implementedFunctions={ISEVEN:{method:"iseven",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.IsOddPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{isodd(e,t){return this.runFunction(e.args,t,this.metadata("ISODD"),e=>e%2==1)}}t.IsOddPlugin=o,o.implementedFunctions={ISODD:{method:"isodd",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.LogarithmPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{log10(e,t){return this.runFunction(e.args,t,this.metadata("LOG10"),Math.log10)}log(e,t){return this.runFunction(e.args,t,this.metadata("LOG"),(e,t)=>Math.log(e)/Math.log(t))}ln(e,t){return this.runFunction(e.args,t,this.metadata("LN"),Math.log)}}t.LogarithmPlugin=o,o.implementedFunctions={LOG10:{method:"log10",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]},LOG:{method:"log",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:n.FunctionArgumentType.NUMBER,defaultValue:10,greaterThan:0}]},LN:{method:"ln",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.PI=t.MathConstantsPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */const o=parseFloat(Math.PI.toFixed(14));t.PI=o;class s extends n.FunctionPlugin{pi(e,t){return this.runFunction(e.args,t,this.metadata("PI"),()=>o)}sqrtpi(e,t){return this.runFunction(e.args,t,this.metadata("SQRTPI"),e=>Math.sqrt(o*e))}}t.MathConstantsPlugin=s,s.implementedFunctions={PI:{method:"pi",parameters:[]},SQRTPI:{method:"sqrtpi",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,minValue:0}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.MatrixPlugin=void 0,r(2);var n=r(1),o=r(74),s=r(132),i=r(78),a=r(164),u=r(140),c=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function l(e,t){return new n.ArraySize(t.width,e.height)}function h(e,t,r){return new n.ArraySize(1+(e.width-t)/r,1+(e.height-t)/r)}class d extends c.FunctionPlugin{mmult(e,t){return this.runFunction(e.args,t,this.metadata("MMULT"),(e,t)=>{if(!e.hasOnlyNumbers()||!t.hasOnlyNumbers())return new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.NumberRange);if(t.height()!==e.width())return new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.ArrayDimensions);const r=l(e.size,t.size),n=this.createKernel((function(e,t,r){let n=0;for(let o=0;o<r;++o)n+=e[this.thread.y][o]*t[o][this.thread.x];return n}),r)(e.rawNumbers(),t.rawNumbers(),e.width());return u.SimpleRangeValue.onlyNumbers(n)})}mmultArraySize(e,t){if(2!==e.args.length)return n.ArraySize.error();const r=this.metadata("MMULT"),o=e.args.map(e=>{var n;return this.arraySizeForAst(e,new a.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))}),[s,i]=o;return l(s,i)}maxpool(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("MAXPOOL"),(function(e,t){let n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:t;if(!e.hasOnlyNumbers())return new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.NumberRange);const i=h(e.size,t,n),a=r.createKernel((function(e,t,r){const n=this.thread.x*r,o=this.thread.y*r;let s=e[o][n];for(let r=0;r<t;r++)for(let i=0;i<t;i++)s=Math.max(s,e[o+r][n+i]);return s}),i)(e.rawNumbers(),t,n);return u.SimpleRangeValue.onlyNumbers(a)}))}medianpool(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("MEDIANPOOL"),(function(e,t){let n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:t;if(!e.hasOnlyNumbers())return new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.NumberRange);const i=h(e.size,t,n),a=r.createKernel((function(e,t,r){const n=this.thread.x*r,o=this.thread.y*r;let s=e[o][n];for(let r=0;r<t;r++)for(let i=0;i<t;i++)s=Math.max(s,e[o+r][n+i]);let i=e[o][n];for(let r=0;r<t;r++)for(let s=0;s<t;s++)i=Math.min(i,e[o+r][n+s]);const a=t*t;let u=i,c=s,l=42;for(let r=0;r<32;r++){const r=(u+c)/2;let s=0;for(let i=0;i<t;i++)for(let a=0;a<t;a++)e[o+i][n+a]>r&&s++;if(t%2==0){if(s===a/2){l=r;break}s>a/2?u=r:c=r}else{if(s===(a-1)/2){l=r;break}s>(a-1)/2?u=r:c=r}}return l}),i)(e.rawNumbers(),t,n);return u.SimpleRangeValue.onlyNumbers(a)}))}maxpoolArraySize(e,t){if(e.args.length<2||e.args.length>3)return n.ArraySize.error();const r=this.metadata("MAXPOOL"),o=e.args.map(e=>{var n;return this.arraySizeForAst(e,new a.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))})[0],s=e.args[1];let u;u=s.type===i.AstNodeType.NUMBER?s.value:1;let c=u;if(3===e.args.length){const t=e.args[2];c=t.type===i.AstNodeType.NUMBER?t.value:1}return u>o.width||u>o.height||c>u||(o.width-u)%c!=0||(o.height-u)%c!=0?n.ArraySize.error():h(o,u,c)}medianpoolArraySize(e,t){return this.maxpoolArraySize(e,t)}transpose(e,t){return this.runFunction(e.args,t,this.metadata("TRANSPOSE"),e=>{const t=e.rawData(),r=e.size,n=[];for(let e=0;e<r.width;++e){n[e]=[];for(let o=0;o<r.height;++o)n[e][o]=t[o][e]}return u.SimpleRangeValue.onlyValues(n)})}transposeArraySize(e,t){if(1!==e.args.length)return n.ArraySize.error();const r=this.metadata("MMULT"),o=e.args.map(e=>{var n;return this.arraySizeForAst(e,new a.InterpreterState(t.formulaAddress,t.arraysFlag||null!==(n=null==r?void 0:r.arrayFunction)&&void 0!==n&&n))}),[s]=o;return new n.ArraySize(s.height,s.width)}createKernel(e,t){return function(){const r=[];for(var n=arguments.length,o=new Array(n),s=0;s<n;s++)o[s]=arguments[s];for(let n=0;n<t.height;++n){r.push([]);for(let s=0;s<t.width;++s)r[n][s]=e.apply({thread:{x:s,y:n}},o)}return r}}}t.MatrixPlugin=d,d.implementedFunctions={MMULT:{method:"mmult",arraySizeMethod:"mmultArraySize",parameters:[{argumentType:c.FunctionArgumentType.RANGE},{argumentType:c.FunctionArgumentType.RANGE}],vectorizationForbidden:!0},TRANSPOSE:{method:"transpose",arraySizeMethod:"transposeArraySize",parameters:[{argumentType:c.FunctionArgumentType.RANGE}],vectorizationForbidden:!0},MAXPOOL:{method:"maxpool",arraySizeMethod:"maxpoolArraySize",parameters:[{argumentType:c.FunctionArgumentType.RANGE},{argumentType:c.FunctionArgumentType.NUMBER},{argumentType:c.FunctionArgumentType.NUMBER,optionalArg:!0}],vectorizationForbidden:!0},MEDIANPOOL:{method:"medianpool",arraySizeMethod:"medianpoolArraySize",parameters:[{argumentType:c.FunctionArgumentType.RANGE},{argumentType:c.FunctionArgumentType.NUMBER},{argumentType:c.FunctionArgumentType.NUMBER,optionalArg:!0}],vectorizationForbidden:!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.MedianPlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{median(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("MEDIAN"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const i=r.arithmeticHelper.coerceNumbersExactRanges(t);return i instanceof n.CellError?i:0===i.length?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.OneValue):(i.sort((e,t)=>e-t),i.length%2==0?(i[i.length/2-1]+i[i.length/2])/2:i[Math.floor(i.length/2)])}))}large(e,t){return this.runFunction(e.args,t,this.metadata("LARGE"),(e,t)=>{const r=this.arithmeticHelper.manyToExactNumbers(e.valuesFromTopLeftCorner());return r instanceof n.CellError?r:(r.sort((e,t)=>e-t),(t=Math.trunc(t))>r.length?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):r[r.length-t])})}small(e,t){return this.runFunction(e.args,t,this.metadata("SMALL"),(e,t)=>{const r=this.arithmeticHelper.manyToExactNumbers(e.valuesFromTopLeftCorner());return r instanceof n.CellError?r:(r.sort((e,t)=>e-t),(t=Math.trunc(t))>r.length?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):r[t-1])})}}t.MedianPlugin=i,i.implementedFunctions={MEDIAN:{method:"median",parameters:[{argumentType:s.FunctionArgumentType.ANY}],repeatLastArgs:1},LARGE:{method:"large",parameters:[{argumentType:s.FunctionArgumentType.RANGE},{argumentType:s.FunctionArgumentType.NUMBER,minValue:1}]},SMALL:{method:"small",parameters:[{argumentType:s.FunctionArgumentType.RANGE},{argumentType:s.FunctionArgumentType.NUMBER,minValue:1}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.ModuloPlugin=void 0;var n=r(74),o=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class s extends o.FunctionPlugin{mod(e,t){return this.runFunction(e.args,t,this.metadata("MOD"),(e,t)=>0===t?new n.CellError(n.ErrorType.DIV_BY_ZERO):e%t)}}t.ModuloPlugin=s,s.implementedFunctions={MOD:{method:"mod",parameters:[{argumentType:o.FunctionArgumentType.NUMBER},{argumentType:o.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.NumericAggregationPlugin=void 0,r(2);var n=r(73),o=r(74),s=r(132),i=r(148),a=r(78),u=r(166),c=r(141),l=r(140),h=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function d(e){return(0,c.isExtendedNumber)(e)&&!Number.isFinite((0,c.getRawValue)(e))?0:e}class p{constructor(e,t,r){this.sumsq=e,this.sum=t,this.count=r}static single(e){return new p(e*e,e,1)}compose(e){return new p(this.sumsq+e.sumsq,this.sum+e.sum,this.count+e.count)}averageValue(){return this.count>0?this.sum/this.count:void 0}varSValue(){return this.count>1?(this.sumsq-this.sum*this.sum/this.count)/(this.count-1):void 0}varPValue(){return this.count>0?(this.sumsq-this.sum*this.sum/this.count)/this.count:void 0}}p.empty=new p(0,0,0);class m extends h.FunctionPlugin{constructor(){super(...arguments),this.addWithEpsilonRaw=(e,t)=>this.arithmeticHelper.addWithEpsilonRaw(e,t)}sum(e,t){return this.doSum(e.args,t)}sumsq(e,t){return this.reduce(e.args,t,0,"SUMSQ",this.addWithEpsilonRaw,e=>Math.pow((0,c.getRawValue)(e),2),g)}max(e,t){return this.doMax(e.args,t)}maxa(e,t){return d(this.reduce(e.args,t,Number.NEGATIVE_INFINITY,"MAXA",(e,t)=>Math.max(e,t),c.getRawValue,f))}min(e,t){return this.doMin(e.args,t)}mina(e,t){return d(this.reduce(e.args,t,Number.POSITIVE_INFINITY,"MINA",(e,t)=>Math.min(e,t),c.getRawValue,f))}count(e,t){return this.doCount(e.args,t)}counta(e,t){return this.doCounta(e.args,t)}average(e,t){return this.doAverage(e.args,t)}averagea(e,t){var r;const n=this.reduce(e.args,t,p.empty,"_AGGREGATE_A",(e,t)=>e.compose(t),e=>p.single((0,c.getRawValue)(e)),f);return n instanceof o.CellError?n:null!==(r=n.averageValue())&&void 0!==r?r:new o.CellError(o.ErrorType.DIV_BY_ZERO)}vars(e,t){return this.doVarS(e.args,t)}varp(e,t){return this.doVarP(e.args,t)}vara(e,t){var r;const n=this.reduceAggregateA(e.args,t);return n instanceof o.CellError?n:null!==(r=n.varSValue())&&void 0!==r?r:new o.CellError(o.ErrorType.DIV_BY_ZERO)}varpa(e,t){var r;const n=this.reduceAggregateA(e.args,t);return n instanceof o.CellError?n:null!==(r=n.varPValue())&&void 0!==r?r:new o.CellError(o.ErrorType.DIV_BY_ZERO)}stdevs(e,t){return this.doStdevS(e.args,t)}stdevp(e,t){return this.doStdevP(e.args,t)}stdeva(e,t){const r=this.reduceAggregateA(e.args,t);if(r instanceof o.CellError)return r;{const e=r.varSValue();return void 0===e?new o.CellError(o.ErrorType.DIV_BY_ZERO):Math.sqrt(e)}}stdevpa(e,t){const r=this.reduceAggregateA(e.args,t);if(r instanceof o.CellError)return r;{const e=r.varPValue();return void 0===e?new o.CellError(o.ErrorType.DIV_BY_ZERO):Math.sqrt(e)}}product(e,t){return this.doProduct(e.args,t)}subtotal(e,t){if(e.args.length<2)return new o.CellError(o.ErrorType.NA,s.ErrorMessage.WrongArgNumber);const r=this.coerceToType(this.evaluateAst(e.args[0],t),{argumentType:h.FunctionArgumentType.NUMBER},t),n=e.args.slice(1);switch(r){case 1:case 101:return this.doAverage(n,t);case 2:case 102:return this.doCount(n,t);case 3:case 103:return this.doCounta(n,t);case 4:case 104:return this.doMax(n,t);case 5:case 105:return this.doMin(n,t);case 6:case 106:return this.doProduct(n,t);case 7:case 107:return this.doStdevS(n,t);case 8:case 108:return this.doStdevP(n,t);case 9:case 109:return this.doSum(n,t);case 10:case 110:return this.doVarS(n,t);case 11:case 111:return this.doVarP(n,t);default:return new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.BadMode)}}reduceAggregate(e,t){return this.reduce(e,t,p.empty,"_AGGREGATE",(e,t)=>e.compose(t),e=>p.single((0,c.getRawValue)(e)),g)}reduceAggregateA(e,t){return this.reduce(e,t,p.empty,"_AGGREGATE_A",(e,t)=>e.compose(t),e=>p.single((0,c.getRawValue)(e)),f)}doAverage(e,t){var r;const n=this.reduceAggregate(e,t);return n instanceof o.CellError?n:null!==(r=n.averageValue())&&void 0!==r?r:new o.CellError(o.ErrorType.DIV_BY_ZERO)}doVarS(e,t){var r;const n=this.reduceAggregate(e,t);return n instanceof o.CellError?n:null!==(r=n.varSValue())&&void 0!==r?r:new o.CellError(o.ErrorType.DIV_BY_ZERO)}doVarP(e,t){var r;const n=this.reduceAggregate(e,t);return n instanceof o.CellError?n:null!==(r=n.varPValue())&&void 0!==r?r:new o.CellError(o.ErrorType.DIV_BY_ZERO)}doStdevS(e,t){const r=this.reduceAggregate(e,t);if(r instanceof o.CellError)return r;{const e=r.varSValue();return void 0===e?new o.CellError(o.ErrorType.DIV_BY_ZERO):Math.sqrt(e)}}doStdevP(e,t){const r=this.reduceAggregate(e,t);if(r instanceof o.CellError)return r;{const e=r.varPValue();return void 0===e?new o.CellError(o.ErrorType.DIV_BY_ZERO):Math.sqrt(e)}}doCount(e,t){return this.reduce(e,t,0,"COUNT",(e,t)=>e+t,c.getRawValue,e=>(0,c.isExtendedNumber)(e)?1:0)}doCounta(e,t){return this.reduce(e,t,0,"COUNTA",(e,t)=>e+t,c.getRawValue,e=>e===c.EmptyValue?0:1)}doMax(e,t){return d(this.reduce(e,t,Number.NEGATIVE_INFINITY,"MAX",(e,t)=>Math.max(e,t),c.getRawValue,g))}doMin(e,t){return d(this.reduce(e,t,Number.POSITIVE_INFINITY,"MIN",(e,t)=>Math.min(e,t),c.getRawValue,g))}doSum(e,t){return this.reduce(e,t,0,"SUM",this.addWithEpsilonRaw,c.getRawValue,g)}doProduct(e,t){return this.reduce(e,t,1,"PRODUCT",(e,t)=>e*t,c.getRawValue,g)}reduce(e,t,r,n,i,u,c){return e.length<1?new o.CellError(o.ErrorType.NA,s.ErrorMessage.WrongArgNumber):e.reduce((e,s)=>{if(e instanceof o.CellError)return e;if(s.type===a.AstNodeType.CELL_RANGE||s.type===a.AstNodeType.COLUMN_RANGE||s.type===a.AstNodeType.ROW_RANGE){const a=this.evaluateRange(s,t,r,n,i,u,c);return a instanceof o.CellError?a:i(a,e)}let h;if(h=this.evaluateAst(s,t),h instanceof l.SimpleRangeValue){return Array.from(h.valuesFromTopLeftCorner()).map(c).filter(e=>void 0!==e).map(e=>e instanceof o.CellError?e:u(e)).reduce((e,t)=>e instanceof o.CellError?e:t instanceof o.CellError?t:i(e,t),e)}if(s.type===a.AstNodeType.CELL_REFERENCE){if(h=c(h),void 0===h)return e}else if(h=this.coerceScalarToNumberOrError(h),h=c(h),void 0===h)return e;return h instanceof o.CellError?h:i(e,u(h))},r)}evaluateRange(e,t,r,a,u,c,l){let h;try{h=n.AbsoluteCellRange.fromAst(e,t.formulaAddress)}catch(e){if(e instanceof i.SheetsNotEqual)return new o.CellError(o.ErrorType.REF,s.ErrorMessage.RangeManySheets);throw e}const d=this.dependencyGraph.getRange(h.start,h.end);if(void 0===d)throw new Error("Range does not exists in graph");let p=d.getFunctionValue(a);if(void 0===p){p=this.getRangeValues(a,h,d,c,l).reduce((e,t)=>e instanceof o.CellError?e:t instanceof o.CellError?t:u(e,t),r),d.setFunctionValue(a,p)}return p}getRangeValues(e,t,r,n,s){const i=[],{smallerRangeVertex:a,restRange:u}=this.dependencyGraph.rangeMapping.findSmallerRange(t);let c;if(void 0!==a&&this.dependencyGraph.existsEdge(a,r)){const t=a.getFunctionValue(e);if(void 0!==t)i.push(t);else for(const e of a.range.addresses(this.dependencyGraph)){const t=s(this.dependencyGraph.getScalarValue(e));t instanceof o.CellError?i.push(t):void 0!==t&&i.push(n(t))}c=u}else c=t;for(const e of c.addresses(this.dependencyGraph)){const t=s(this.dependencyGraph.getScalarValue(e));t instanceof o.CellError?i.push(t):void 0!==t&&i.push(n(t))}return i}}function g(e){return(0,c.isExtendedNumber)(e)||e instanceof o.CellError?e:void 0}function f(e){return"boolean"==typeof e?(0,u.coerceBooleanToNumber)(e):(0,c.isExtendedNumber)(e)||e instanceof o.CellError?e:"string"==typeof e?0:void 0}t.NumericAggregationPlugin=m,m.implementedFunctions={SUM:{method:"sum",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},SUMSQ:{method:"sumsq",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},MAX:{method:"max",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},MIN:{method:"min",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},MAXA:{method:"maxa",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},MINA:{method:"mina",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},COUNT:{method:"count",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},COUNTA:{method:"counta",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},AVERAGE:{method:"average",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},AVERAGEA:{method:"averagea",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},PRODUCT:{method:"product",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},"VAR.S":{method:"vars",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},"VAR.P":{method:"varp",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},VARA:{method:"vara",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},VARPA:{method:"varpa",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},"STDEV.S":{method:"stdevs",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},"STDEV.P":{method:"stdevp",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},STDEVA:{method:"stdeva",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},STDEVPA:{method:"stdevpa",parameters:[{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1},SUBTOTAL:{method:"subtotal",parameters:[{argumentType:h.FunctionArgumentType.NUMBER},{argumentType:h.FunctionArgumentType.ANY}],repeatLastArgs:1}},m.aliases={VAR:"VAR.S",VARP:"VAR.P",STDEV:"STDEV.S",STDEVP:"STDEV.P",VARS:"VAR.S",STDEVS:"STDEV.S"}},function(e,t,r){"use strict";t.__esModule=!0,t.PowerPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{power(e,t){return this.runFunction(e.args,t,this.metadata("POWER"),Math.pow)}}t.PowerPlugin=o,o.implementedFunctions={POWER:{method:"power",parameters:[{argumentType:n.FunctionArgumentType.NUMBER},{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.RadiansPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{radians(e,t){return this.runFunction(e.args,t,this.metadata("RADIANS"),e=>e*(Math.PI/180))}}t.RadiansPlugin=o,o.implementedFunctions={RADIANS:{method:"radians",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.RadixConversionPlugin=void 0;var n=r(74),o=r(132),s=r(175),i=r(165);class a extends i.FunctionPlugin{dec2bin(e,t){return this.runFunction(e.args,t,this.metadata("DEC2BIN"),(e,t)=>c(e,2,t))}dec2oct(e,t){return this.runFunction(e.args,t,this.metadata("DEC2OCT"),(e,t)=>c(e,8,t))}dec2hex(e,t){return this.runFunction(e.args,t,this.metadata("DEC2HEX"),(e,t)=>c(e,16,t))}bin2dec(e,t){return this.runFunction(e.args,t,this.metadata("BIN2DEC"),e=>{const t=u(e,2,10);return void 0===t?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotBinary):p(t,2)})}bin2oct(e,t){return this.runFunction(e.args,t,this.metadata("BIN2OCT"),(e,t)=>{const r=u(e,2,10);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotBinary):c(p(r,2),8,t)})}bin2hex(e,t){return this.runFunction(e.args,t,this.metadata("BIN2HEX"),(e,t)=>{const r=u(e,2,10);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotBinary):c(p(r,2),16,t)})}oct2dec(e,t){return this.runFunction(e.args,t,this.metadata("OCT2DEC"),e=>{const t=u(e,8,10);return void 0===t?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotOctal):p(t,8)})}oct2bin(e,t){return this.runFunction(e.args,t,this.metadata("OCT2BIN"),(e,t)=>{const r=u(e,8,10);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotOctal):c(p(r,8),2,t)})}oct2hex(e,t){return this.runFunction(e.args,t,this.metadata("OCT2HEX"),(e,t)=>{const r=u(e,8,10);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotOctal):c(p(r,8),16,t)})}hex2dec(e,t){return this.runFunction(e.args,t,this.metadata("HEX2DEC"),e=>{const t=u(e,16,10);return void 0===t?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotHex):p(t,16)})}hex2bin(e,t){return this.runFunction(e.args,t,this.metadata("HEX2BIN"),(e,t)=>{const r=u(e,16,10);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotHex):c(p(r,16),2,t)})}hex2oct(e,t){return this.runFunction(e.args,t,this.metadata("HEX2OCT"),(e,t)=>{const r=u(e,16,10);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotHex):c(p(r,16),8,t)})}base(e,t){return this.runFunction(e.args,t,this.metadata("BASE"),h)}decimal(e,t){return this.runFunction(e.args,t,this.metadata("DECIMAL"),(e,t)=>{const r=u(e,t,255);return void 0===r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NotHex):parseInt(r,t)})}}function u(e,t,r){const n="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".substr(0,t),o=new RegExp(`^[${n}]+$`);if(!(e.length>r)&&o.test(e))return e}function c(e,t,r){if(e>function(e){return-l(e)-1}(t))return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueBaseLarge);if(e<l(t))return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueBaseSmall);const i=d(e,t);return void 0===r||e<0?i:i.length>r?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueBaseLong):(0,s.padLeft)(i,r)}function l(e){return-Math.pow(e,10)/2}function h(e,t,r){const n=d(e,t);return void 0!==r&&r>n.length?(0,s.padLeft)(n,r):n}function d(e,t){return(e+(e<0?Math.pow(t,10):0)).toString(t).toUpperCase()}function p(e,t){const r=parseInt(e,t),n=Math.pow(t,10);return r>=n/2?r-n:r}t.RadixConversionPlugin=a,a.implementedFunctions={DEC2BIN:{method:"dec2bin",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:1,maxValue:10}]},DEC2OCT:{method:"dec2oct",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:1,maxValue:10}]},DEC2HEX:{method:"dec2hex",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:1,maxValue:10}]},BIN2DEC:{method:"bin2dec",parameters:[{argumentType:i.FunctionArgumentType.STRING}]},BIN2OCT:{method:"bin2oct",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:10}]},BIN2HEX:{method:"bin2hex",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:10}]},OCT2DEC:{method:"oct2dec",parameters:[{argumentType:i.FunctionArgumentType.STRING}]},OCT2BIN:{method:"oct2bin",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:10}]},OCT2HEX:{method:"oct2hex",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:10}]},HEX2DEC:{method:"hex2dec",parameters:[{argumentType:i.FunctionArgumentType.STRING}]},HEX2BIN:{method:"hex2bin",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:10}]},HEX2OCT:{method:"hex2oct",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:10}]},DECIMAL:{method:"decimal",parameters:[{argumentType:i.FunctionArgumentType.STRING},{argumentType:i.FunctionArgumentType.NUMBER,minValue:2,maxValue:36}]},BASE:{method:"base",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:0},{argumentType:i.FunctionArgumentType.NUMBER,minValue:2,maxValue:36},{argumentType:i.FunctionArgumentType.NUMBER,optionalArg:!0,minValue:0,maxValue:255}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.RandomPlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{rand(e,t){return this.runFunction(e.args,t,this.metadata("RAND"),Math.random)}randbetween(e,t){return this.runFunction(e.args,t,this.metadata("RANDBETWEEN"),(e,t)=>t<e?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.WrongOrder):((e=Math.ceil(e))===(t=Math.floor(t)+1)&&(t+=1),e+Math.floor(Math.random()*(t-e))))}}t.RandomPlugin=i,i.implementedFunctions={RAND:{method:"rand",parameters:[],isVolatile:!0},RANDBETWEEN:{method:"randbetween",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}],isVolatile:!0}}},function(e,t,r){"use strict";t.__esModule=!0,t.RoundingPlugin=void 0,t.findNextEvenNumber=a,t.findNextOddNumber=i;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function i(e){const t=Math.ceil(e);return t%2==1?t:t+1}function a(e){const t=Math.ceil(e);return t%2==0?t:t+1}class u extends s.FunctionPlugin{roundup(e,t){return this.runFunction(e.args,t,this.metadata("ROUNDDOWN"),(e,t)=>{const r=Math.pow(10,t);return e<0?-Math.ceil(-e*r)/r:Math.ceil(e*r)/r})}rounddown(e,t){return this.runFunction(e.args,t,this.metadata("ROUNDDOWN"),(e,t)=>{const r=Math.pow(10,t);return e<0?-Math.floor(-e*r)/r:Math.floor(e*r)/r})}round(e,t){return this.runFunction(e.args,t,this.metadata("ROUND"),(e,t)=>{const r=Math.pow(10,t);return e<0?-Math.round(-e*r)/r:Math.round(e*r)/r})}intFunc(e,t){return this.runFunction(e.args,t,this.metadata("INT"),e=>e<0?-Math.floor(-e):Math.floor(e))}even(e,t){return this.runFunction(e.args,t,this.metadata("EVEN"),e=>e<0?-a(-e):a(e))}odd(e,t){return this.runFunction(e.args,t,this.metadata("ODD"),e=>e<0?-i(-e):i(e))}ceilingmath(e,t){return this.runFunction(e.args,t,this.metadata("CEILING.MATH"),(e,t,r)=>0===t||0===e?0:(t=Math.abs(t),1===r&&e<0&&(t=-t),Math.ceil(e/t)*t))}ceiling(e,t){return this.runFunction(e.args,t,this.metadata("CEILING"),(e,t)=>0===e?0:0===t?new n.CellError(n.ErrorType.DIV_BY_ZERO):e>0&&t<0?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.DistinctSigns):Math.ceil(e/t)*t)}ceilingprecise(e,t){return this.runFunction(e.args,t,this.metadata("CEILING.PRECISE"),(e,t)=>0===t||0===e?0:(t=Math.abs(t),Math.ceil(e/t)*t))}floormath(e,t){return this.runFunction(e.args,t,this.metadata("FLOOR.MATH"),(e,t,r)=>0===t||0===e?0:(t=Math.abs(t),1===r&&e<0&&(t*=-1),Math.floor(e/t)*t))}floor(e,t){return this.runFunction(e.args,t,this.metadata("FLOOR"),(e,t)=>0===e?0:0===t?new n.CellError(n.ErrorType.DIV_BY_ZERO):e>0&&t<0?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.DistinctSigns):Math.floor(e/t)*t)}floorprecise(e,t){return this.runFunction(e.args,t,this.metadata("FLOOR.PRECISE"),(e,t)=>0===t||0===e?0:(t=Math.abs(t),Math.floor(e/t)*t))}}t.RoundingPlugin=u,u.implementedFunctions={ROUNDUP:{method:"roundup",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:0}]},ROUNDDOWN:{method:"rounddown",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:0}]},ROUND:{method:"round",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:0}]},INT:{method:"intFunc",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},EVEN:{method:"even",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ODD:{method:"odd",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},"CEILING.MATH":{method:"ceilingmath",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:0}]},CEILING:{method:"ceiling",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}]},"CEILING.PRECISE":{method:"ceilingprecise",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1}]},"FLOOR.MATH":{method:"floormath",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:0}]},FLOOR:{method:"floor",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}]},"FLOOR.PRECISE":{method:"floorprecise",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1}]}},u.aliases={"ISO.CEILING":"CEILING.PRECISE",TRUNC:"ROUNDDOWN"}},function(e,t,r){"use strict";t.__esModule=!0,t.SqrtPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{sqrt(e,t){return this.runFunction(e.args,t,this.metadata("SQRT"),Math.sqrt)}}t.SqrtPlugin=o,o.implementedFunctions={SQRT:{method:"sqrt",parameters:[{argumentType:n.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.ConditionalAggregationPlugin=void 0,r(2);var n=r(74),o=r(132),s=r(251),i=r(141),a=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class u{constructor(e,t){this.sum=e,this.count=t}static single(e){return new u(e,1)}compose(e){return new u(this.sum+e.sum,this.count+e.count)}averageValue(){return this.count>0?this.sum/this.count:void 0}}function c(e){return(0,i.isExtendedNumber)(e)&&!Number.isFinite((0,i.getRawValue)(e))?0:e}function l(e){return e instanceof n.CellError?e:(0,i.isExtendedNumber)(e)?(0,i.getRawValue)(e):void 0}u.empty=new u(0,0);class h extends a.FunctionPlugin{sumif(e,t){return this.runFunction(e.args,t,this.metadata("SUMIF"),(e,t,r)=>this.computeConditionalAggregationFunction(null!=r?r:e,[e,t],"SUMIF",0,(e,t)=>this.arithmeticHelper.nonstrictadd(e,t),l))}sumifs(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("SUMIFS"),(function(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),o=1;o<t;o++)n[o-1]=arguments[o];return r.computeConditionalAggregationFunction(e,n,"SUMIFS",0,(e,t)=>r.arithmeticHelper.nonstrictadd(e,t),l)}))}averageif(e,t){return this.runFunction(e.args,t,this.metadata("AVERAGEIF"),(e,t,r)=>{const o=this.computeConditionalAggregationFunction(null!=r?r:e,[e,t],"AVERAGEIF",u.empty,(e,t)=>e.compose(t),e=>(0,i.isExtendedNumber)(e)?u.single((0,i.getRawValue)(e)):u.empty);return o instanceof n.CellError?o:o.averageValue()||new n.CellError(n.ErrorType.DIV_BY_ZERO)})}countif(e,t){return this.runFunction(e.args,t,this.metadata("COUNTIF"),(e,t)=>this.computeConditionalAggregationFunction(e,[e,t],"COUNTIF",0,(e,t)=>e+t,()=>1))}countifs(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("COUNTIFS"),(function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];return r.computeConditionalAggregationFunction(t[0],t,"COUNTIFS",0,(e,t)=>e+t,()=>1)}))}minifs(e,t){var r=this;const n=(e,t)=>void 0===t||void 0===e?void 0===t?e:t:Math.min(e,t);return this.runFunction(e.args,t,this.metadata("MINIFS"),(function(e){for(var t=arguments.length,o=new Array(t>1?t-1:0),s=1;s<t;s++)o[s-1]=arguments[s];const i=r.computeConditionalAggregationFunction(e,o,"MINIFS",Number.POSITIVE_INFINITY,n,l);return c(i)}))}maxifs(e,t){var r=this;const n=(e,t)=>void 0===t||void 0===e?void 0===t?e:t:Math.max(e,t);return this.runFunction(e.args,t,this.metadata("MAXIFS"),(function(e){for(var t=arguments.length,o=new Array(t>1?t-1:0),s=1;s<t;s++)o[s-1]=arguments[s];const i=r.computeConditionalAggregationFunction(e,o,"MAXIFS",Number.NEGATIVE_INFINITY,n,l);return c(i)}))}computeConditionalAggregationFunction(e,t,r,i,a,u){const c=[];for(let e=0;e<t.length;e+=2){const r=t[e],i=this.interpreter.criterionBuilder.fromCellValue(t[e+1],this.arithmeticHelper);if(void 0===i)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.BadCriterion);c.push(new s.Condition(r,i))}return new s.CriterionFunctionCompute(this.interpreter,function(e){return t=>{const r=t.map(e=>`${e.conditionRange.range.sheet},${e.conditionRange.range.start.col},${e.conditionRange.range.start.row}`);return[e,...r].join(",")}}(r),i,a,u).compute(e,c)}}t.ConditionalAggregationPlugin=h,h.implementedFunctions={SUMIF:{method:"sumif",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR},{argumentType:a.FunctionArgumentType.RANGE,optionalArg:!0}]},COUNTIF:{method:"countif",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR}]},AVERAGEIF:{method:"averageif",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR},{argumentType:a.FunctionArgumentType.RANGE,optionalArg:!0}]},SUMIFS:{method:"sumifs",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR}],repeatLastArgs:2},COUNTIFS:{method:"countifs",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR}],repeatLastArgs:2},MINIFS:{method:"minifs",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR}],repeatLastArgs:2},MAXIFS:{method:"maxifs",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NOERROR}],repeatLastArgs:2}}},function(e,t,r){"use strict";t.__esModule=!0,t.CriterionFunctionCompute=t.Condition=void 0,r(2);var n=r(74),o=r(132),s=r(252),i=r(141);t.CriterionFunctionCompute=class{constructor(e,t,r,n,o){this.interpreter=e,this.cacheKey=t,this.reduceInitialValue=r,this.composeFunction=n,this.mapFunction=o,this.dependencyGraph=this.interpreter.dependencyGraph}compute(e,t){for(const r of t)if(!r.conditionRange.sameDimensionsAs(e))return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.EqualLength);const r=this.tryToGetRangeVertexForRangeValue(e),s=t.map(e=>this.tryToGetRangeVertexForRangeValue(e.conditionRange));if(r&&s.every(e=>void 0!==e)){const n=t.map(e=>e.criterionPackage.raw).join(","),o=this.findAlreadyComputedValueInCache(r,this.cacheKey(t),n);if(void 0!==o)return this.interpreter.stats.incrementCriterionFunctionFullCacheUsed(),o;const i=this.buildNewCriterionCache(this.cacheKey(t),t.map(e=>e.conditionRange.range),e.range);return i.has(n)||i.set(n,[this.evaluateRangeValue(e,t),t.map(e=>e.criterionPackage.lambda)]),r.setCriterionFunctionValues(this.cacheKey(t),i),s.forEach(e=>{void 0!==e&&e.addDependentCacheRange(r)}),i.get(n)[0]}return this.evaluateRangeValue(e,t)}tryToGetRangeVertexForRangeValue(e){const t=e.range;return void 0===t?void 0:this.dependencyGraph.getRange(t.start,t.end)}reduceFunction(e){let t=this.reduceInitialValue;for(const r of e)t=this.composeFunction(t,r);return t}findAlreadyComputedValueInCache(e,t,r){return e.getCriterionFunctionValue(t,r)}evaluateRangeValue(e,t){const r=t.map(e=>e.criterionPackage.lambda),n=Array.from(e.valuesFromTopLeftCorner()).map(this.mapFunction)[Symbol.iterator](),o=u(r,t.map(e=>e.conditionRange.iterateValuesFromTopLeftCorner()),n);return this.reduceFunction(o)}buildNewCriterionCache(e,t,r){const o=this.dependencyGraph.getRange(r.start,r.end),{smallerRangeVertex:s,restConditionRanges:i,restValuesRange:c}=((e,t,r)=>{if(r.end.row>r.start.row){const o=(0,n.simpleCellAddress)(r.end.sheet,r.end.col,r.end.row-1),s=e.getRange(r.start,o);if(void 0!==s)return{smallerRangeVertex:s,restValuesRange:r.withStart((0,n.simpleCellAddress)(r.start.sheet,r.start.col,r.end.row)),restConditionRanges:t.map(e=>e.withStart((0,n.simpleCellAddress)(e.start.sheet,e.start.col,e.end.row)))}}return{restValuesRange:r,restConditionRanges:t}})(this.dependencyGraph,t,r);let l;l=void 0!==s&&this.dependencyGraph.existsEdge(s,o)?s.getCriterionFunctionValues(e):new Map;const h=new Map;return l.forEach((e,t)=>{let[r,n]=e;const o=u(n,i.map(e=>a(this.dependencyGraph,e)),Array.from(a(this.dependencyGraph,c)).map(this.mapFunction)[Symbol.iterator]()),s=this.composeFunction(r,this.reduceFunction(o));this.interpreter.stats.incrementCriterionFunctionPartialCacheUsed(),h.set(t,[s,n])}),h}};function*a(e,t){for(const r of t.addresses(e))yield(0,i.getRawValue)(e.getScalarValue(r))}function*u(e,t,r){for(const n of r){const r=t.map(e=>(0,s.split)(e));if(!r.every(e=>Object.prototype.hasOwnProperty.call(e,"value")))return;c(r.map(e=>(0,i.getRawValue)(e.value)),e).every(e=>{let[t,r]=e;return r(t)})&&(yield n),t=r.map(e=>e.rest)}}function c(e,t){const r=[];for(let n=0;n<Math.min(e.length,t.length);n++)r.push([e[n],t[n]]);return r}t.Condition=class{constructor(e,t){this.conditionRange=e,this.criterionPackage=t}}},function(e,t,r){"use strict";
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
function*n(){}t.__esModule=!0,t.empty=n,t.first=function(e){const t=e[Symbol.iterator](),{done:r,value:n}=t.next();if(!r)return n;return},t.split=function(e){const t=e[Symbol.iterator](),{done:r,value:o}=t.next();return r?{rest:n()}:{value:o,rest:t}}},function(e,t,r){"use strict";t.__esModule=!0,t.SumprodPlugin=void 0;var n=r(74),o=r(132),s=r(141),i=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends i.FunctionPlugin{sumproduct(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("SUMPRODUCT"),(function(){for(var e=arguments.length,t=new Array(e),i=0;i<e;i++)t[i]=arguments[i];const a=t[0].width(),u=t[0].height();for(const e of t)if(e.width()!==a||e.height()!==u)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.EqualLength);let c=0;const l=t.map(e=>e.iterateValuesFromTopLeftCorner());for(let e=0;e<a*u;e++){let e=1;for(const t of l){const o=t.next().value;if(o instanceof n.CellError)return o;const i=r.coerceScalarToNumberOrError(o);(0,s.isExtendedNumber)(i)?e*=(0,s.getRawValue)(i):e=0}c+=e}return c}))}}t.SumprodPlugin=a,a.implementedFunctions={SUMPRODUCT:{method:"sumproduct",parameters:[{argumentType:i.FunctionArgumentType.RANGE}],repeatLastArgs:1}}},function(e,t,r){"use strict";t.__esModule=!0,t.TextPlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{concatenate(e,t){return this.runFunction(e.args,t,this.metadata("CONCATENATE"),(function(){return"".concat(...arguments)}))}split(e,t){return this.runFunction(e.args,t,this.metadata("SPLIT"),(e,t)=>{const r=e.split(" ");return t>=r.length||t<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.IndexBounds):r[t]})}len(e,t){return this.runFunction(e.args,t,this.metadata("LEN"),e=>e.length)}lower(e,t){return this.runFunction(e.args,t,this.metadata("LOWER"),e=>e.toLowerCase())}trim(e,t){return this.runFunction(e.args,t,this.metadata("TRIM"),e=>e.replace(/^ +| +$/g,"").replace(/ +/g," "))}proper(e,t){return this.runFunction(e.args,t,this.metadata("PROPER"),e=>e.replace(/\p{L}+/gu,e=>e.charAt(0).toUpperCase()+e.substring(1).toLowerCase()))}clean(e,t){return this.runFunction(e.args,t,this.metadata("CLEAN"),e=>e.replace(/[\u0000-\u001F]/g,""))}exact(e,t){return this.runFunction(e.args,t,this.metadata("EXACT"),(e,t)=>e===t)}rept(e,t){return this.runFunction(e.args,t,this.metadata("REPT"),(e,t)=>t<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NegativeCount):e.repeat(t))}right(e,t){return this.runFunction(e.args,t,this.metadata("RIGHT"),(e,t)=>t<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NegativeLength):0===t?"":e.slice(-t))}left(e,t){return this.runFunction(e.args,t,this.metadata("LEFT"),(e,t)=>t<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NegativeLength):e.slice(0,t))}mid(e,t){return this.runFunction(e.args,t,this.metadata("MID"),(e,t,r)=>t<1?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.LessThanOne):r<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NegativeLength):e.substring(t-1,t+r-1))}replace(e,t){return this.runFunction(e.args,t,this.metadata("REPLACE"),(e,t,r,s)=>t<1?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.LessThanOne):r<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.NegativeLength):e.substring(0,t-1)+s+e.substring(t+r-1))}search(e,t){return this.runFunction(e.args,t,this.metadata("SEARCH"),(e,t,r)=>{if(r<1||r>t.length)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.LengthBounds);const s=e.toLowerCase(),i=t.substring(r-1).toLowerCase(),a=this.arithmeticHelper.requiresRegex(s)?this.arithmeticHelper.searchString(s,i):i.indexOf(s);return a>-1?a+r:new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.PatternNotFound)})}substitute(e,t){return this.runFunction(e.args,t,this.metadata("SUBSTITUTE"),(e,t,r,s)=>{const i=new RegExp(t,"g");if(void 0===s)return e.replace(i,r);if(s<1)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.LessThanOne);let a,u=0;for(;null!==(a=i.exec(e));)if(s===++u)return e.substring(0,a.index)+r+e.substring(i.lastIndex);return e})}find(e,t){return this.runFunction(e.args,t,this.metadata("FIND"),(e,t,r)=>{if(r<1||r>t.length)return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.IndexBounds);const s=t.substring(r-1).indexOf(e)+r;return s>0?s:new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.PatternNotFound)})}t(e,t){return this.runFunction(e.args,t,this.metadata("T"),e=>e instanceof n.CellError||"string"==typeof e?e:"")}upper(e,t){return this.runFunction(e.args,t,this.metadata("UPPER"),e=>e.toUpperCase())}}t.TextPlugin=i,i.implementedFunctions={CONCATENATE:{method:"concatenate",parameters:[{argumentType:s.FunctionArgumentType.STRING}],repeatLastArgs:1,expandRanges:!0},EXACT:{method:"exact",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.STRING}]},SPLIT:{method:"split",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER}]},LEN:{method:"len",parameters:[{argumentType:s.FunctionArgumentType.STRING}]},LOWER:{method:"lower",parameters:[{argumentType:s.FunctionArgumentType.STRING}]},MID:{method:"mid",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}]},TRIM:{method:"trim",parameters:[{argumentType:s.FunctionArgumentType.STRING}]},T:{method:"t",parameters:[{argumentType:s.FunctionArgumentType.SCALAR}]},PROPER:{method:"proper",parameters:[{argumentType:s.FunctionArgumentType.STRING}]},CLEAN:{method:"clean",parameters:[{argumentType:s.FunctionArgumentType.STRING}]},REPT:{method:"rept",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER}]},RIGHT:{method:"right",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1}]},LEFT:{method:"left",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1}]},REPLACE:{method:"replace",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.STRING}]},SEARCH:{method:"search",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1}]},SUBSTITUTE:{method:"substitute",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER,optionalArg:!0}]},FIND:{method:"find",parameters:[{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.STRING},{argumentType:s.FunctionArgumentType.NUMBER,defaultValue:1}]},UPPER:{method:"upper",parameters:[{argumentType:s.FunctionArgumentType.STRING}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.TrigonometryPlugin=void 0;var n=r(74),o=r(132),s=r(165),i=r(239);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends s.FunctionPlugin{acos(e,t){return this.runFunction(e.args,t,this.metadata("ACOS"),Math.acos)}asin(e,t){return this.runFunction(e.args,t,this.metadata("ASIN"),Math.asin)}cos(e,t){return this.runFunction(e.args,t,this.metadata("COS"),Math.cos)}sin(e,t){return this.runFunction(e.args,t,this.metadata("SIN"),Math.sin)}tan(e,t){return this.runFunction(e.args,t,this.metadata("TAN"),Math.tan)}atan(e,t){return this.runFunction(e.args,t,this.metadata("ATAN"),Math.atan)}atan2(e,t){return this.runFunction(e.args,t,this.metadata("ATAN2"),(e,t)=>0===e&&0===t?new n.CellError(n.ErrorType.DIV_BY_ZERO):Math.atan2(t,e))}cot(e,t){return this.runFunction(e.args,t,this.metadata("COT"),e=>0===e?new n.CellError(n.ErrorType.DIV_BY_ZERO):1/Math.tan(e))}acot(e,t){return this.runFunction(e.args,t,this.metadata("ACOT"),e=>0===e?i.PI/2:Math.atan(1/e))}sec(e,t){return this.runFunction(e.args,t,this.metadata("SEC"),e=>1/Math.cos(e))}csc(e,t){return this.runFunction(e.args,t,this.metadata("CSC"),e=>0===e?new n.CellError(n.ErrorType.DIV_BY_ZERO):1/Math.sin(e))}sinh(e,t){return this.runFunction(e.args,t,this.metadata("SINH"),Math.sinh)}asinh(e,t){return this.runFunction(e.args,t,this.metadata("ASINH"),Math.asinh)}cosh(e,t){return this.runFunction(e.args,t,this.metadata("COSH"),Math.cosh)}acosh(e,t){return this.runFunction(e.args,t,this.metadata("ACOSH"),Math.acosh)}tanh(e,t){return this.runFunction(e.args,t,this.metadata("TANH"),Math.tanh)}atanh(e,t){return this.runFunction(e.args,t,this.metadata("ATANH"),Math.atanh)}coth(e,t){return this.runFunction(e.args,t,this.metadata("COTH"),e=>0===e?new n.CellError(n.ErrorType.DIV_BY_ZERO):1/Math.tanh(e))}acoth(e,t){return this.runFunction(e.args,t,this.metadata("ACOTH"),e=>0===e?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NonZero):Math.atanh(1/e))}sech(e,t){return this.runFunction(e.args,t,this.metadata("SECH"),e=>1/Math.cosh(e))}csch(e,t){return this.runFunction(e.args,t,this.metadata("CSCH"),e=>0===e?new n.CellError(n.ErrorType.DIV_BY_ZERO):1/Math.sinh(e))}}t.TrigonometryPlugin=a,a.implementedFunctions={ACOS:{method:"acos",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ASIN:{method:"asin",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},COS:{method:"cos",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},SIN:{method:"sin",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},TAN:{method:"tan",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ATAN:{method:"atan",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ATAN2:{method:"atan2",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}]},COT:{method:"cot",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},SEC:{method:"sec",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},CSC:{method:"csc",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},SINH:{method:"sinh",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},COSH:{method:"cosh",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},TANH:{method:"tanh",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},COTH:{method:"coth",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},SECH:{method:"sech",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},CSCH:{method:"csch",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ACOT:{method:"acot",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ASINH:{method:"asinh",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ACOSH:{method:"acosh",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ATANH:{method:"atanh",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},ACOTH:{method:"acoth",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.LookupPlugin=void 0;var n=r(73),o=r(74),s=r(132),i=r(257),a=r(143),u=r(166),c=r(140),l=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class h extends l.FunctionPlugin{constructor(){super(...arguments),this.rowSearch=new i.RowSearchStrategy(this.dependencyGraph)}vlookup(e,t){return this.runFunction(e.args,t,this.metadata("VLOOKUP"),(e,t,r,n)=>{const i=t.range;return void 0===i?new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.WrongType):r<1?new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.LessThanOne):r>i.width()?new o.CellError(o.ErrorType.REF,s.ErrorMessage.IndexLarge):this.doVlookup((0,u.zeroIfEmpty)(e),t,r-1,n)})}hlookup(e,t){return this.runFunction(e.args,t,this.metadata("HLOOKUP"),(e,t,r,n)=>{const i=t.range;return void 0===i?new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.WrongType):r<1?new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.LessThanOne):r>i.height()?new o.CellError(o.ErrorType.REF,s.ErrorMessage.IndexLarge):this.doHlookup((0,u.zeroIfEmpty)(e),t,r-1,n)})}match(e,t){return this.runFunction(e.args,t,this.metadata("MATCH"),(e,t,r)=>this.doMatch((0,u.zeroIfEmpty)(e),t,r))}searchInRange(e,t,r,n){if(!r&&"string"==typeof e&&this.arithmeticHelper.requiresRegex(e))return n.advancedFind(this.arithmeticHelper.eqMatcherFunction(e),t);{const o=r?{ordering:"asc"}:{ordering:"none",matchExactly:!0};return n.find(e,t,o)}}doVlookup(e,t,r,i){this.dependencyGraph.stats.start(a.StatType.VLOOKUP);const u=t.range;let l;l=void 0===u?c.SimpleRangeValue.onlyValues(t.data.map(e=>[e[0]])):c.SimpleRangeValue.onlyRange(n.AbsoluteCellRange.spanFrom(u.start,1,u.height()),this.dependencyGraph);const h=this.searchInRange(e,l,i,this.columnSearch);if(this.dependencyGraph.stats.end(a.StatType.VLOOKUP),-1===h)return new o.CellError(o.ErrorType.NA,s.ErrorMessage.ValueNotFound);let d;if(void 0===u)d=t.data[h][r];else{const e=(0,o.simpleCellAddress)(u.sheet,u.start.col+r,u.start.row+h);d=this.dependencyGraph.getCellValue(e)}return d instanceof c.SimpleRangeValue?new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.WrongType):d}doHlookup(e,t,r,i){const a=t.range;let u;u=void 0===a?c.SimpleRangeValue.onlyValues([t.data[0]]):c.SimpleRangeValue.onlyRange(n.AbsoluteCellRange.spanFrom(a.start,a.width(),1),this.dependencyGraph);const l=this.searchInRange(e,u,i,this.rowSearch);if(-1===l)return new o.CellError(o.ErrorType.NA,s.ErrorMessage.ValueNotFound);let h;if(void 0===a)h=t.data[r][l];else{const e=(0,o.simpleCellAddress)(a.sheet,a.start.col+l,a.start.row+r);h=this.dependencyGraph.getCellValue(e)}return h instanceof c.SimpleRangeValue?new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.WrongType):h}doMatch(e,t,r){if(![-1,0,1].includes(r))return new o.CellError(o.ErrorType.VALUE,s.ErrorMessage.BadMode);if(t.width()>1&&t.height()>1)return new o.CellError(o.ErrorType.NA);const n=0===r?{ordering:"none",matchExactly:!0}:{ordering:-1===r?"desc":"asc"},i=(1===t.width()?this.columnSearch:this.rowSearch).find(e,t,n);return-1===i?new o.CellError(o.ErrorType.NA,s.ErrorMessage.ValueNotFound):i+1}}t.LookupPlugin=h,h.implementedFunctions={VLOOKUP:{method:"vlookup",parameters:[{argumentType:l.FunctionArgumentType.NOERROR},{argumentType:l.FunctionArgumentType.RANGE},{argumentType:l.FunctionArgumentType.NUMBER},{argumentType:l.FunctionArgumentType.BOOLEAN,defaultValue:!0}]},HLOOKUP:{method:"hlookup",parameters:[{argumentType:l.FunctionArgumentType.NOERROR},{argumentType:l.FunctionArgumentType.RANGE},{argumentType:l.FunctionArgumentType.NUMBER},{argumentType:l.FunctionArgumentType.BOOLEAN,defaultValue:!0}]},MATCH:{method:"match",parameters:[{argumentType:l.FunctionArgumentType.NOERROR},{argumentType:l.FunctionArgumentType.RANGE},{argumentType:l.FunctionArgumentType.NUMBER,defaultValue:1}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.RowSearchStrategy=void 0;var n=r(207);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.AdvancedFind{constructor(e){super(e),this.dependencyGraph=e}find(e,t,r){return this.basicFind(e,t,"col",r)}}t.RowSearchStrategy=o},function(e,t,r){"use strict";t.__esModule=!0,t.RomanPlugin=void 0;var n=r(74),o=r(132),s=r(141),i=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends i.FunctionPlugin{roman(e,t){return this.runFunction(e.args,t,this.metadata("ROMAN"),(e,t)=>(e=Math.trunc(e),!1===t?t=4:!0===t&&(t=0),(t=(0,s.getRawValue)(this.coerceScalarToNumberOrError(t)))instanceof n.CellError?t:(t=Math.trunc(t))<0?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.ValueSmall):t>4?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.ValueLarge):function(e,t){const r={val:e%1e3,acc:"M".repeat(Math.floor(e/1e3))};4===t&&(c(r,"IM",999,1e3),c(r,"ID",499,500));t>=3&&(c(r,"VM",995,1e3),c(r,"VD",495,500));t>=2&&(c(r,"XM",990,1e3),c(r,"XD",490,500));t>=1&&(c(r,"LM",950,1e3),c(r,"LD",450,500));c(r,"CM",900,1e3),c(r,"CD",400,500),c(r,"D",500,900),r.acc+="C".repeat(Math.floor(r.val/100)),r.val%=100,t>=2&&(c(r,"IC",99,100),c(r,"IL",49,50));t>=1&&(c(r,"VC",95,100),c(r,"VL",45,50));return c(r,"XC",90,100),c(r,"XL",40,50),c(r,"L",50,90),r.acc+="X".repeat(Math.floor(r.val/10)),r.val%=10,c(r,"IX",9,10),c(r,"IV",4,5),c(r,"V",5,9),r.acc+="I".repeat(r.val),r.acc}(e,t)))}arabic(e,t){return this.runFunction(e.args,t,this.metadata("ARABIC"),e=>{let t=!1;if((e=e.trim().toUpperCase()).startsWith("-")&&(t=!0,""===(e=e.slice(1))))return new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.InvalidRoman);const r={input:e,acc:0};return u(r,{token:"MMM",val:3e3},{token:"MM",val:2e3},{token:"M",val:1e3}),u(r,{token:"IM",val:999},{token:"VM",val:995},{token:"XM",val:990},{token:"LM",val:950},{token:"CM",val:900}),u(r,{token:"D",val:500},{token:"ID",val:499},{token:"VD",val:495},{token:"XD",val:490},{token:"LD",val:450},{token:"CD",val:400}),u(r,{token:"CCC",val:300},{token:"CC",val:200},{token:"C",val:100}),u(r,{token:"IC",val:99},{token:"VC",val:95},{token:"XC",val:90}),u(r,{token:"L",val:50},{token:"IL",val:49},{token:"VL",val:45},{token:"XL",val:40}),u(r,{token:"XXX",val:30},{token:"XX",val:20},{token:"X",val:10}),u(r,{token:"IX",val:9}),u(r,{token:"V",val:5},{token:"IV",val:4}),u(r,{token:"III",val:3},{token:"II",val:2},{token:"I",val:1}),""!==r.input?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.InvalidRoman):t?-r.acc:r.acc})}}function u(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];for(const t of r)if(e.input.startsWith(t.token)){e.input=e.input.slice(t.token.length),e.acc+=t.val;break}}function c(e,t,r,n){e.val>=r&&e.val<n&&(e.val-=r,e.acc+=t)}t.RomanPlugin=a,a.implementedFunctions={ROMAN:{method:"roman",parameters:[{argumentType:i.FunctionArgumentType.NUMBER,minValue:1,lessThan:4e3},{argumentType:i.FunctionArgumentType.NOERROR,optionalArg:!0,defaultValue:0}]},ARABIC:{method:"arabic",parameters:[{argumentType:i.FunctionArgumentType.STRING}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.SimpleArithmerticPlugin=void 0;var n=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */class o extends n.FunctionPlugin{add(e,t){return this.runFunction(e.args,t,this.metadata("HF.ADD"),this.arithmeticHelper.addWithEpsilon)}concat(e,t){return this.runFunction(e.args,t,this.metadata("HF.CONCAT"),this.arithmeticHelper.concat)}divide(e,t){return this.runFunction(e.args,t,this.metadata("HF.DIVIDE"),this.arithmeticHelper.divide)}eq(e,t){return this.runFunction(e.args,t,this.metadata("HF.EQ"),this.arithmeticHelper.eq)}gt(e,t){return this.runFunction(e.args,t,this.metadata("HF.GT"),this.arithmeticHelper.gt)}gte(e,t){return this.runFunction(e.args,t,this.metadata("HF.GTE"),this.arithmeticHelper.geq)}lt(e,t){return this.runFunction(e.args,t,this.metadata("HF.LT"),this.arithmeticHelper.lt)}lte(e,t){return this.runFunction(e.args,t,this.metadata("HF.LTE"),this.arithmeticHelper.leq)}minus(e,t){return this.runFunction(e.args,t,this.metadata("HF.MINUS"),this.arithmeticHelper.subtract)}multiply(e,t){return this.runFunction(e.args,t,this.metadata("HF.MULTIPLY"),this.arithmeticHelper.multiply)}ne(e,t){return this.runFunction(e.args,t,this.metadata("HF.NE"),this.arithmeticHelper.neq)}pow(e,t){return this.runFunction(e.args,t,this.metadata("HF.POW"),this.arithmeticHelper.pow)}uminus(e,t){return this.runFunction(e.args,t,this.metadata("HF.UMINUS"),this.arithmeticHelper.unaryMinus)}upercent(e,t){return this.runFunction(e.args,t,this.metadata("HF.UNARY_PERCENT"),this.arithmeticHelper.unaryPercent)}uplus(e,t){return this.runFunction(e.args,t,this.metadata("HF.UPLUS"),this.arithmeticHelper.unaryPlus)}}t.SimpleArithmerticPlugin=o,o.implementedFunctions={"HF.ADD":{method:"add",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0},{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.CONCAT":{method:"concat",parameters:[{argumentType:n.FunctionArgumentType.STRING,passSubtype:!0},{argumentType:n.FunctionArgumentType.STRING,passSubtype:!0}]},"HF.DIVIDE":{method:"divide",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0},{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.EQ":{method:"eq",parameters:[{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0},{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0}]},"HF.GT":{method:"gt",parameters:[{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0},{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0}]},"HF.GTE":{method:"gte",parameters:[{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0},{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0}]},"HF.LT":{method:"lt",parameters:[{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0},{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0}]},"HF.LTE":{method:"lte",parameters:[{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0},{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0}]},"HF.MINUS":{method:"minus",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0},{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.MULTIPLY":{method:"multiply",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0},{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.NE":{method:"ne",parameters:[{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0},{argumentType:n.FunctionArgumentType.NOERROR,passSubtype:!0}]},"HF.POW":{method:"pow",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0},{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.UMINUS":{method:"uminus",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.UNARY_PERCENT":{method:"upercent",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]},"HF.UPLUS":{method:"uplus",parameters:[{argumentType:n.FunctionArgumentType.NUMBER,passSubtype:!0}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.StatisticalPlugin=void 0;var n=r(74),o=r(132),s=r(261),i=r(262),a=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class u extends a.FunctionPlugin{erf(e,t){return this.runFunction(e.args,t,this.metadata("ERF"),(e,t)=>void 0===t?(0,i.erf)(e):(0,i.erf)(t)-(0,i.erf)(e))}erfc(e,t){return this.runFunction(e.args,t,this.metadata("ERFC"),i.erfc)}expondist(e,t){return this.runFunction(e.args,t,this.metadata("EXPON.DIST"),(e,t,r)=>r?i.exponential.cdf(e,t):i.exponential.pdf(e,t))}fisher(e,t){return this.runFunction(e.args,t,this.metadata("FISHER"),e=>Math.log((1+e)/(1-e))/2)}fisherinv(e,t){return this.runFunction(e.args,t,this.metadata("FISHERINV"),e=>1-2/(Math.exp(2*e)+1))}gamma(e,t){return this.runFunction(e.args,t,this.metadata("GAMMA"),i.gammafn)}gammadist(e,t){return this.runFunction(e.args,t,this.metadata("GAMMA.DIST"),(e,t,r,n)=>n?i.gamma.cdf(e,t,r):i.gamma.pdf(e,t,r))}gammaln(e,t){return this.runFunction(e.args,t,this.metadata("GAMMALN"),i.gammaln)}gammainv(e,t){return this.runFunction(e.args,t,this.metadata("GAMMA.INV"),i.gamma.inv)}gauss(e,t){return this.runFunction(e.args,t,this.metadata("GAUSS"),e=>i.normal.cdf(e,0,1)-.5)}betadist(e,t){return this.runFunction(e.args,t,this.metadata("BETA.DIST"),(e,t,r,s,a,u)=>e<=a?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall):e>=u?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):(e=(e-a)/(u-a),s?i.beta.cdf(e,t,r):i.beta.pdf(e,t,r)))}betainv(e,t){return this.runFunction(e.args,t,this.metadata("BETA.INV"),(e,t,r,s,a)=>s>=a?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.WrongOrder):i.beta.inv(e,t,r)*(a-s)+s)}binomialdist(e,t){return this.runFunction(e.args,t,this.metadata("BINOM.DIST"),(e,t,r,s)=>e>t?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.WrongOrder):(e=Math.trunc(e),t=Math.trunc(t),s?i.binomial.cdf(e,t,r):i.binomial.pdf(e,t,r)))}binomialinv(e,t){return this.runFunction(e.args,t,this.metadata("BINOM.INV"),(e,t,r)=>{let n=-1,o=e=Math.trunc(e);for(;o>n+1;){const s=Math.trunc((n+o)/2);i.binomial.cdf(s,e,t)>=r?o=s:n=s}return o})}besselifn(e,t){return this.runFunction(e.args,t,this.metadata("BESSELI"),(e,t)=>(0,s.besseli)(e,Math.trunc(t)))}besseljfn(e,t){return this.runFunction(e.args,t,this.metadata("BESSELJ"),(e,t)=>(0,s.besselj)(e,Math.trunc(t)))}besselkfn(e,t){return this.runFunction(e.args,t,this.metadata("BESSELK"),(e,t)=>(0,s.besselk)(e,Math.trunc(t)))}besselyfn(e,t){return this.runFunction(e.args,t,this.metadata("BESSELY"),(e,t)=>(0,s.bessely)(e,Math.trunc(t)))}chisqdist(e,t){return this.runFunction(e.args,t,this.metadata("CHISQ.DIST"),(e,t,r)=>(t=Math.trunc(t),r?i.chisquare.cdf(e,t):i.chisquare.pdf(e,t)))}chisqdistrt(e,t){return this.runFunction(e.args,t,this.metadata("CHISQ.DIST.RT"),(e,t)=>1-i.chisquare.cdf(e,Math.trunc(t)))}chisqinv(e,t){return this.runFunction(e.args,t,this.metadata("CHISQ.INV"),(e,t)=>i.chisquare.inv(e,Math.trunc(t)))}chisqinvrt(e,t){return this.runFunction(e.args,t,this.metadata("CHISQ.INV.RT"),(e,t)=>i.chisquare.inv(1-e,Math.trunc(t)))}fdist(e,t){return this.runFunction(e.args,t,this.metadata("F.DIST"),(e,t,r,n)=>(t=Math.trunc(t),r=Math.trunc(r),n?i.centralF.cdf(e,t,r):i.centralF.pdf(e,t,r)))}fdistrt(e,t){return this.runFunction(e.args,t,this.metadata("F.DIST.RT"),(e,t,r)=>1-i.centralF.cdf(e,Math.trunc(t),Math.trunc(r)))}finv(e,t){return this.runFunction(e.args,t,this.metadata("F.INV"),(e,t,r)=>i.centralF.inv(e,Math.trunc(t),Math.trunc(r)))}finvrt(e,t){return this.runFunction(e.args,t,this.metadata("F.INV.RT"),(e,t,r)=>i.centralF.inv(1-e,Math.trunc(t),Math.trunc(r)))}weibulldist(e,t){return this.runFunction(e.args,t,this.metadata("WEIBULL.DIST"),(e,t,r,n)=>n?i.weibull.cdf(e,r,t):i.weibull.pdf(e,r,t))}poissondist(e,t){return this.runFunction(e.args,t,this.metadata("POISSON.DIST"),(e,t,r)=>(e=Math.trunc(e),r?i.poisson.cdf(e,t):i.poisson.pdf(e,t)))}hypgeomdist(e,t){return this.runFunction(e.args,t,this.metadata("HYPGEOM.DIST"),(e,t,r,s,a)=>e>t||e>r||t>s||r>s||e+s<r+t?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):(e=Math.trunc(e),t=Math.trunc(t),r=Math.trunc(r),s=Math.trunc(s),a?i.hypgeom.cdf(e,s,r,t):i.hypgeom.pdf(e,s,r,t)))}tdist(e,t){return this.runFunction(e.args,t,this.metadata("T.DIST"),(e,t,r)=>(t=Math.trunc(t),r?i.studentt.cdf(e,t):i.studentt.pdf(e,t)))}tdist2t(e,t){return this.runFunction(e.args,t,this.metadata("T.DIST.2T"),(e,t)=>2*(1-i.studentt.cdf(e,Math.trunc(t))))}tdistrt(e,t){return this.runFunction(e.args,t,this.metadata("T.DIST.RT"),(e,t)=>1-i.studentt.cdf(e,Math.trunc(t)))}tdistold(e,t){return this.runFunction(e.args,t,this.metadata("TDIST"),(e,t,r)=>r*(1-i.studentt.cdf(e,Math.trunc(t))))}tinv(e,t){return this.runFunction(e.args,t,this.metadata("T.INV"),(e,t)=>i.studentt.inv(e,Math.trunc(t)))}tinv2t(e,t){return this.runFunction(e.args,t,this.metadata("T.INV.2T"),(e,t)=>i.studentt.inv(1-e/2,Math.trunc(t)))}lognormdist(e,t){return this.runFunction(e.args,t,this.metadata("LOGNORM.DIST"),(e,t,r,n)=>n?i.lognormal.cdf(e,t,r):i.lognormal.pdf(e,t,r))}lognorminv(e,t){return this.runFunction(e.args,t,this.metadata("LOGNORM.INV"),(e,t,r)=>i.lognormal.inv(e,t,r))}normdist(e,t){return this.runFunction(e.args,t,this.metadata("NORM.DIST"),(e,t,r,n)=>n?i.normal.cdf(e,t,r):i.normal.pdf(e,t,r))}norminv(e,t){return this.runFunction(e.args,t,this.metadata("NORM.INV"),(e,t,r)=>i.normal.inv(e,t,r))}normsdist(e,t){return this.runFunction(e.args,t,this.metadata("NORM.S.DIST"),(e,t)=>t?i.normal.cdf(e,0,1):i.normal.pdf(e,0,1))}normsinv(e,t){return this.runFunction(e.args,t,this.metadata("NORM.S.INV"),e=>i.normal.inv(e,0,1))}phi(e,t){return this.runFunction(e.args,t,this.metadata("PHI"),e=>i.normal.pdf(e,0,1))}negbinomdist(e,t){return this.runFunction(e.args,t,this.metadata("NEGBINOM.DIST"),(e,t,r,n)=>(e=Math.trunc(e),t=Math.trunc(t),n?i.negbin.cdf(e,t,r):i.negbin.pdf(e,t,r)))}confidencenorm(e,t){return this.runFunction(e.args,t,this.metadata("CONFIDENCE.NORM"),(e,t,r)=>(0,i.normalci)(1,e,t,Math.trunc(r))[1]-1)}confidencet(e,t){return this.runFunction(e.args,t,this.metadata("CONFIDENCE.T"),(e,t,r)=>1===(r=Math.trunc(r))?new n.CellError(n.ErrorType.DIV_BY_ZERO):(0,i.tci)(1,e,t,r)[1]-1)}standardize(e,t){return this.runFunction(e.args,t,this.metadata("STANDARDIZE"),(e,t,r)=>(e-t)/r)}}t.StatisticalPlugin=u,u.implementedFunctions={ERF:{method:"erf",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,optionalArg:!0}]},ERFC:{method:"erfc",parameters:[{argumentType:a.FunctionArgumentType.NUMBER}]},"EXPON.DIST":{method:"expondist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},FISHER:{method:"fisher",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:-1,lessThan:1}]},FISHERINV:{method:"fisherinv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER}]},GAMMA:{method:"gamma",parameters:[{argumentType:a.FunctionArgumentType.NUMBER}]},"GAMMA.DIST":{method:"gammadist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},GAMMALN:{method:"gammaln",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0}]},"GAMMA.INV":{method:"gammainv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,lessThan:1},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0}]},GAUSS:{method:"gauss",parameters:[{argumentType:a.FunctionArgumentType.NUMBER}]},"BETA.DIST":{method:"betadist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN},{argumentType:a.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:a.FunctionArgumentType.NUMBER,defaultValue:1}]},"BETA.INV":{method:"betainv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,defaultValue:0},{argumentType:a.FunctionArgumentType.NUMBER,defaultValue:1}]},"BINOM.DIST":{method:"binomialdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"BINOM.INV":{method:"binomialinv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1}]},BESSELI:{method:"besselifn",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0}]},BESSELJ:{method:"besseljfn",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0}]},BESSELK:{method:"besselkfn",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0}]},BESSELY:{method:"besselyfn",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0}]},"CHISQ.DIST":{method:"chisqdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1,maxValue:1e10},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"CHISQ.DIST.RT":{method:"chisqdistrt",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1,maxValue:1e10}]},"CHISQ.INV":{method:"chisqinv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1,maxValue:1e10}]},"CHISQ.INV.RT":{method:"chisqinvrt",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"F.DIST":{method:"fdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"F.DIST.RT":{method:"fdistrt",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"F.INV":{method:"finv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"F.INV.RT":{method:"finvrt",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"WEIBULL.DIST":{method:"weibulldist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"POISSON.DIST":{method:"poissondist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"HYPGEOM.DIST":{method:"hypgeomdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"T.DIST":{method:"tdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"T.DIST.2T":{method:"tdist2t",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"T.DIST.RT":{method:"tdistrt",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},TDIST:{method:"tdistold",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.INTEGER,minValue:1,maxValue:2}]},"T.INV":{method:"tinv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"T.INV.2T":{method:"tinv2t",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,maxValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"LOGNORM.DIST":{method:"lognormdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"LOGNORM.INV":{method:"lognorminv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1},{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0}]},"NORM.DIST":{method:"normdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"NORM.INV":{method:"norminv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1},{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0}]},"NORM.S.DIST":{method:"normsdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"NORM.S.INV":{method:"normsinv",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1}]},PHI:{method:"phi",parameters:[{argumentType:a.FunctionArgumentType.NUMBER}]},"NEGBINOM.DIST":{method:"negbinomdist",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,minValue:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1},{argumentType:a.FunctionArgumentType.NUMBER,minValue:0,maxValue:1},{argumentType:a.FunctionArgumentType.BOOLEAN}]},"CONFIDENCE.NORM":{method:"confidencenorm",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},"CONFIDENCE.T":{method:"confidencet",parameters:[{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0,lessThan:1},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0},{argumentType:a.FunctionArgumentType.NUMBER,minValue:1}]},STANDARDIZE:{method:"standardize",parameters:[{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,greaterThan:0}]}},u.aliases={NEGBINOMDIST:"NEGBINOM.DIST",EXPONDIST:"EXPON.DIST",BETADIST:"BETA.DIST",NORMDIST:"NORM.DIST",NORMINV:"NORM.INV",NORMSDIST:"NORM.S.DIST",NORMSINV:"NORM.S.INV",LOGNORMDIST:"LOGNORM.DIST",LOGINV:"LOGNORM.INV",TINV:"T.INV.2T",HYPGEOMDIST:"HYPGEOM.DIST",POISSON:"POISSON.DIST",WEIBULL:"WEIBULL.DIST",FINV:"F.INV.RT",FDIST:"F.DIST.RT",CHIDIST:"CHISQ.DIST.RT",CHIINV:"CHISQ.INV.RT",GAMMADIST:"GAMMA.DIST","GAMMALN.PRECISE":"GAMMALN",GAMMAINV:"GAMMA.INV",BETAINV:"BETA.INV",BINOMDIST:"BINOM.DIST",CONFIDENCE:"CONFIDENCE.NORM",CRITBINOM:"BINOM.INV",WEIBULLDIST:"WEIBULL.DIST",TINV2T:"T.INV.2T",TDISTRT:"T.DIST.RT",TDIST2T:"T.DIST.2T",FINVRT:"F.INV.RT",FDISTRT:"F.DIST.RT",CHIDISTRT:"CHISQ.DIST.RT",CHIINVRT:"CHISQ.INV.RT",LOGNORMINV:"LOGNORM.INV",POISSONDIST:"POISSON.DIST"}},function(e,t,r){"use strict";t.__esModule=!0,t.bessely=t.besselk=t.besselj=t.besseli=void 0;
/**
 * @license
 bessel.js (C) 2013-present SheetJS -- http://sheetjs.com

 Apache License
 Version 2.0, January 2004
 http://www.apache.org/licenses/

 TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

 1. Definitions.

 "License" shall mean the terms and conditions for use, reproduction,
 and distribution as defined by Sections 1 through 9 of this document.

 "Licensor" shall mean the copyright owner or entity authorized by
 the copyright owner that is granting the License.

 "Legal Entity" shall mean the union of the acting entity and all
 other entities that control, are controlled by, or are under common
 control with that entity. For the purposes of this definition,
 "control" means (i) the power, direct or indirect, to cause the
 direction or management of such entity, whether by contract or
 otherwise, or (ii) ownership of fifty percent (50%) or more of the
 outstanding shares, or (iii) beneficial ownership of such entity.

 "You" (or "Your") shall mean an individual or Legal Entity
 exercising permissions granted by this License.

 "Source" form shall mean the preferred form for making modifications,
 including but not limited to software source code, documentation
 source, and configuration files.

 "Object" form shall mean any form resulting from mechanical
 transformation or translation of a Source form, including but
 not limited to compiled object code, generated documentation,
 and conversions to other media types.

 "Work" shall mean the work of authorship, whether in Source or
 Object form, made available under the License, as indicated by a
 copyright notice that is included in or attached to the work
 (an example is provided in the Appendix below).

 "Derivative Works" shall mean any work, whether in Source or Object
 form, that is based on (or derived from) the Work and for which the
 editorial revisions, annotations, elaborations, or other modifications
 represent, as a whole, an original work of authorship. For the purposes
 of this License, Derivative Works shall not include works that remain
 separable from, or merely link (or bind by name) to the interfaces of,
 the Work and Derivative Works thereof.

 "Contribution" shall mean any work of authorship, including
 the original version of the Work and any modifications or additions
 to that Work or Derivative Works thereof, that is intentionally
 submitted to Licensor for inclusion in the Work by the copyright owner
 or by an individual or Legal Entity authorized to submit on behalf of
 the copyright owner. For the purposes of this definition, "submitted"
 means any form of electronic, verbal, or written communication sent
 to the Licensor or its representatives, including but not limited to
 communication on electronic mailing lists, source code control systems,
 and issue tracking systems that are managed by, or on behalf of, the
 Licensor for the purpose of discussing and improving the Work, but
 excluding communication that is conspicuously marked or otherwise
 designated in writing by the copyright owner as "Not a Contribution."

 "Contributor" shall mean Licensor and any individual or Legal Entity
 on behalf of whom a Contribution has been received by Licensor and
 subsequently incorporated within the Work.

 2. Grant of Copyright License. Subject to the terms and conditions of
 this License, each Contributor hereby grants to You a perpetual,
 worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 copyright license to reproduce, prepare Derivative Works of,
 publicly display, publicly perform, sublicense, and distribute the
 Work and such Derivative Works in Source or Object form.

 3. Grant of Patent License. Subject to the terms and conditions of
 this License, each Contributor hereby grants to You a perpetual,
 worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 (except as stated in this section) patent license to make, have made,
 use, offer to sell, sell, import, and otherwise transfer the Work,
 where such license applies only to those patent claims licensable
 by such Contributor that are necessarily infringed by their
 Contribution(s) alone or by combination of their Contribution(s)
 with the Work to which such Contribution(s) was submitted. If You
 institute patent litigation against any entity (including a
 cross-claim or counterclaim in a lawsuit) alleging that the Work
 or a Contribution incorporated within the Work constitutes direct
 or contributory patent infringement, then any patent licenses
 granted to You under this License for that Work shall terminate
 as of the date such litigation is filed.

 4. Redistribution. You may reproduce and distribute copies of the
 Work or Derivative Works thereof in any medium, with or without
 modifications, and in Source or Object form, provided that You
 meet the following conditions:

 (a) You must give any other recipients of the Work or
 Derivative Works a copy of this License; and

 (b) You must cause any modified files to carry prominent notices
 stating that You changed the files; and

 (c) You must retain, in the Source form of any Derivative Works
 that You distribute, all copyright, patent, trademark, and
 attribution notices from the Source form of the Work,
 excluding those notices that do not pertain to any part of
 the Derivative Works; and

 (d) If the Work includes a "NOTICE" text file as part of its
 distribution, then any Derivative Works that You distribute must
 include a readable copy of the attribution notices contained
 within such NOTICE file, excluding those notices that do not
 pertain to any part of the Derivative Works, in at least one
 of the following places: within a NOTICE text file distributed
 as part of the Derivative Works; within the Source form or
 documentation, if provided along with the Derivative Works; or,
 within a display generated by the Derivative Works, if and
 wherever such third-party notices normally appear. The contents
 of the NOTICE file are for informational purposes only and
 do not modify the License. You may add Your own attribution
 notices within Derivative Works that You distribute, alongside
 or as an addendum to the NOTICE text from the Work, provided
 that such additional attribution notices cannot be construed
 as modifying the License.

 You may add Your own copyright statement to Your modifications and
 may provide additional or different license terms and conditions
 for use, reproduction, or distribution of Your modifications, or
 for any such Derivative Works as a whole, provided Your use,
 reproduction, and distribution of the Work otherwise complies with
 the conditions stated in this License.

 5. Submission of Contributions. Unless You explicitly state otherwise,
 any Contribution intentionally submitted for inclusion in the Work
 by You to the Licensor shall be under the terms and conditions of
 this License, without any additional terms or conditions.
 Notwithstanding the above, nothing herein shall supersede or modify
 the terms of any separate license agreement you may have executed
 with Licensor regarding such Contributions.

 6. Trademarks. This License does not grant permission to use the trade
 names, trademarks, service marks, or product names of the Licensor,
 except as required for reasonable and customary use in describing the
 origin of the Work and reproducing the content of the NOTICE file.

 7. Disclaimer of Warranty. Unless required by applicable law or
 agreed to in writing, Licensor provides the Work (and each
 Contributor provides its Contributions) on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 implied, including, without limitation, any warranties or conditions
 of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 PARTICULAR PURPOSE. You are solely responsible for determining the
 appropriateness of using or redistributing the Work and assume any
 risks associated with Your exercise of permissions under this License.

 8. Limitation of Liability. In no event and under no legal theory,
 whether in tort (including negligence), contract, or otherwise,
 unless required by applicable law (such as deliberate and grossly
 negligent acts) or agreed to in writing, shall any Contributor be
 liable to You for damages, including any direct, indirect, special,
 incidental, or consequential damages of any character arising as a
 result of this License or out of the use or inability to use the
 Work (including but not limited to damages for loss of goodwill,
 work stoppage, computer failure or malfunction, or any and all
 other commercial damages or losses), even if such Contributor
 has been advised of the possibility of such damages.

 9. Accepting Warranty or Additional Liability. While redistributing
 the Work or Derivative Works thereof, You may choose to offer,
 and charge a fee for, acceptance of support, warranty, indemnity,
 or other liability obligations and/or rights consistent with this
 License. However, in accepting such obligations, You may act only
 on Your own behalf and on Your sole responsibility, not on behalf
 of any other Contributor, and only if You agree to indemnify,
 defend, and hold each Contributor harmless for any liability
 incurred by, or claims asserted against, such Contributor by reason
 of your accepting any such warranty or additional liability.

 END OF TERMS AND CONDITIONS

 APPENDIX: How to apply the Apache License to your work.

 To apply the Apache License to your work, attach the following
 boilerplate notice, with the fields enclosed by brackets "{}"
 replaced with your own identifying information. (Don't include
 the brackets!)  The text should be enclosed in the appropriate
 comment syntax for the file format. We also recommend that a
 file or class name and description of purpose be included on the
 same "printed page" as the copyright notice for easier
 identification within third-party archives.

 Copyright (C) 2013-present   SheetJS LLC

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 */
var n=Math;function o(e,t){for(var r=0,n=0;r<e.length;++r)n=t*n+e[r];return n}function s(e,t,r,n,o){if(0===t)return r;if(1===t)return n;for(var s=2/e,i=n,a=1;a<t;++a)i=n*a*s+o*r,r=n,n=i;return i}function i(e,t,r,n,o){return function(r,i){if(n){if(0===r)return 1==n?-1/0:1/0;if(r<0)return NaN}return 0===i?e(r):1===i?t(r):i<0?NaN:s(r,i|=0,e(r),t(r),o)}}var a=function(){var e=.636619772,t=[57568490574,-13362590354,651619640.7,-11214424.18,77392.33017,-184.9052456].reverse(),r=[57568490411,1029532985,9494680.718,59272.64853,267.8532712,1].reverse(),i=[1,-.001098628627,2734510407e-14,-2073370639e-15,2.093887211e-7].reverse(),a=[-.01562499995,.0001430488765,-6911147651e-15,7.621095161e-7,-9.34935152e-8].reverse();function u(s){var u=0,c=0,l=0,h=s*s;if(s<8)u=(c=o(t,h))/(l=o(r,h));else{var d=s-.785398164;c=o(i,h=64/h),l=o(a,h),u=n.sqrt(e/s)*(n.cos(d)*c-n.sin(d)*l*8/s)}return u}var c=[72362614232,-7895059235,242396853.1,-2972611.439,15704.4826,-30.16036606].reverse(),l=[144725228442,2300535178,18583304.74,99447.43394,376.9991397,1].reverse(),h=[1,.00183105,-3516396496e-14,2457520174e-15,-2.40337019e-7].reverse(),d=[.04687499995,-.0002002690873,8449199096e-15,-8.8228987e-7,1.05787412e-7].reverse();function p(t){var r=0,s=0,i=0,a=t*t,u=n.abs(t)-2.356194491;return Math.abs(t)<8?r=(s=t*o(c,a))/(i=o(l,a)):(s=o(h,a=64/a),i=o(d,a),r=n.sqrt(e/n.abs(t))*(n.cos(u)*s-n.sin(u)*i*8/n.abs(t)),t<0&&(r=-r)),r}return function e(t,r){if(r=Math.round(r),!isFinite(t))return isNaN(t)?t:0;if(r<0)return(r%2?-1:1)*e(t,-r);if(t<0)return(r%2?-1:1)*e(-t,r);if(0===r)return u(t);if(1===r)return p(t);if(0===t)return 0;var o=0;if(t>r)o=s(t,r,u(t),p(t),-1);else{for(var i=!1,a=0,c=0,l=1,h=0,d=2/t,m=2*n.floor((r+n.floor(n.sqrt(40*r)))/2);m>0;m--)h=m*d*l-a,a=l,l=h,n.abs(l)>1e10&&(l*=1e-10,a*=1e-10,o*=1e-10,c*=1e-10),i&&(c+=l),i=!i,m==r&&(o=a);o/=c=2*c-l}return o}}();t.besselj=a;var u,c,l,h,d,p,m,g,f,E=(u=.636619772,c=[-2957821389,7062834065,-512359803.6,10879881.29,-86327.92757,228.4622733].reverse(),l=[40076544269,745249964.8,7189466.438,47447.2647,226.1030244,1].reverse(),h=[1,-.001098628627,2734510407e-14,-2073370639e-15,2.093887211e-7].reverse(),d=[-.01562499995,.0001430488765,-6911147651e-15,7.621095161e-7,-9.34945152e-8].reverse(),p=[-4900604943e3,127527439e4,-51534381390,734926455.1,-4237922.726,8511.937935].reverse(),m=[249958057e5,424441966400,3733650367,22459040.02,102042.605,354.9632885,1].reverse(),g=[1,.00183105,-3516396496e-14,2457520174e-15,-2.40337019e-7].reverse(),f=[.04687499995,-.0002002690873,8449199096e-15,-8.8228987e-7,1.05787412e-7].reverse(),i((function(e){var t=0,r=0,s=0,i=e*e,p=e-.785398164;return e<8?t=(r=o(c,i))/(s=o(l,i))+u*a(e,0)*n.log(e):(r=o(h,i=64/i),s=o(d,i),t=n.sqrt(u/e)*(n.sin(p)*r+n.cos(p)*s*8/e)),t}),(function(e){var t=0,r=0,s=0,i=e*e,c=e-2.356194491;return e<8?t=(r=e*o(p,i))/(s=o(m,i))+u*(a(e,1)*n.log(e)-1/e):(r=o(g,i=64/i),s=o(f,i),t=n.sqrt(u/e)*(n.sin(c)*r+n.cos(c)*s*8/e)),t}),0,1,-1));t.bessely=E;var T,y,A,R,N=(T=[1,3.5156229,3.0899424,1.2067492,.2659732,.0360768,.0045813].reverse(),y=[.39894228,.01328592,.00225319,-.00157565,.00916281,-.02057706,.02635537,-.01647633,.00392377].reverse(),A=[.5,.87890594,.51498869,.15084934,.02658733,.00301532,32411e-8].reverse(),R=[.39894228,-.03988024,-.00362018,.00163801,-.01031555,.02282967,-.02895312,.01787654,-.00420059].reverse(),function e(t,r){if(0===(r=Math.round(r)))return function(e){return e<=3.75?o(T,e*e/14.0625):n.exp(n.abs(e))/n.sqrt(n.abs(e))*o(y,3.75/n.abs(e))}(t);if(1===r)return function(e){return e<3.75?e*o(A,e*e/14.0625):(e<0?-1:1)*n.exp(n.abs(e))/n.sqrt(n.abs(e))*o(R,3.75/n.abs(e))}(t);if(r<0)return NaN;if(0===n.abs(t))return 0;if(t==1/0)return 1/0;var s,i=0,a=2/n.abs(t),u=0,c=1,l=0;for(s=2*n.round((r+n.round(n.sqrt(40*r)))/2);s>0;s--)l=s*a*c+u,u=c,c=l,n.abs(c)>1e10&&(c*=1e-10,u*=1e-10,i*=1e-10),s==r&&(i=u);return i*=e(t,0)/c,t<0&&r%2?-i:i});t.besseli=N;var S=function(){var e=[-.57721566,.4227842,.23069756,.0348859,.00262698,1075e-7,74e-7].reverse(),t=[1.25331414,-.07832358,.02189568,-.01062446,.00587872,-.0025154,53208e-8].reverse();var r=[1,.15443144,-.67278579,-.18156897,-.01919402,-.00110404,-4686e-8].reverse(),s=[1.25331414,.23498619,-.0365562,.01504268,-.00780353,.00325614,-68245e-8].reverse();return i((function(r){return r<=2?-n.log(r/2)*N(r,0)+o(e,r*r/4):n.exp(-r)/n.sqrt(r)*o(t,2/r)}),(function(e){return e<=2?n.log(e/2)*N(e,1)+1/e*o(r,e*e/4):n.exp(-e)/n.sqrt(e)*o(s,2/e)}),0,2,1)}();t.besselk=S},function(e,t,r){"use strict";
/**
 * @license
 Copyright (c) 2013 jStat

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
function n(e){const t=[-1.3026537197817094,.6419697923564902,.019476473204185836,-.00956151478680863,-.000946595344482036,.000366839497852761,42523324806907e-18,-20278578112534e-18,-1624290004647e-18,130365583558e-17,1.5626441722e-8,-8.5238095915e-8,6.529054439e-9,5.059343495e-9,-9.91364156e-10,-2.27365122e-10,96467911e-18,2394038e-18,-6886027e-18,894487e-18,313092e-18,-112708e-18,381e-18,7106e-18,-1523e-18,-94e-18,121e-18,-28e-18];let r,n,o,s,i=t.length-1,a=!1,u=0,c=0;if(0===e)return 0;for(e<0&&(e=-e,a=!0),r=2/(2+e),n=4*r-2;i>0;i--)o=u,u=n*u-c+t[i],c=o;return s=r*Math.exp(-e*e+.5*(t[0]+n*u)-c),a?s-1:1-s}function o(e){return 1-n(e)}function s(e){let t,r,n,s,i=0;if(e>=2)return-100;if(e<=0)return 100;for(s=e<1?e:2-e,n=Math.sqrt(-2*Math.log(s/2)),t=-.70711*((2.30753+.27061*n)/(1+n*(.99229+.04481*n))-n);i<2;i++)r=o(t)-s,t+=r/(1.1283791670955126*Math.exp(-t*t)-t*r);return e<1?t:-t}t.__esModule=!0,t.chisquare=t.centralF=t.binomial=t.beta=void 0,t.corrcoeff=function(e,t){return V(e,t)/U(e,1)/U(t,1)},t.covariance=V,t.erf=n,t.erfc=o,t.exponential=void 0,t.factorial=S,t.factorialln=N,t.gamma=void 0,t.gammafn=a,t.gammaln=c,t.geomean=function(e){return Math.pow(function(e){var t=1,r=e.length;for(;--r>=0;)t*=e[r];return t}(e),1/e.length)},t.lognormal=t.hypgeom=void 0,t.mean=L,t.normal=t.negbin=void 0,t.normalci=function(){var e,t=[].slice.call(arguments),r=new Array(2);e=4===t.length?Math.abs(d.inv(t[1]/2,0,1)*t[2]/Math.sqrt(t[3])):Math.abs(d.inv(t[1]/2,0,1)*U(t[2])/Math.sqrt(t[2].length));return r[0]=t[0]-e,r[1]=t[0]+e,r},t.poisson=void 0,t.stdev=U,t.studentt=void 0,t.sumsqerr=x,t.tci=function(){var e,t=[].slice.call(arguments),r=new Array(2);e=4===t.length?Math.abs(w.inv(t[1]/2,t[3]-1)*t[2]/Math.sqrt(t[3])):Math.abs(w.inv(t[1]/2,t[2].length-1)*U(t[2],!0)/Math.sqrt(t[2].length));return r[0]=t[0]-e,r[1]=t[0]+e,r},t.variance=P,t.weibull=void 0;const i={pdf:(e,t)=>e<0?0:t*Math.exp(-t*e),cdf:(e,t)=>e<0?0:1-Math.exp(-t*e)};function a(e){const t=[-1.716185138865495,24.76565080557592,-379.80425647094563,629.3311553128184,866.9662027904133,-31451.272968848367,-36144.413418691176,66456.14382024054],r=[-30.8402300119739,315.35062697960416,-1015.1563674902192,-3107.771671572311,22538.11842098015,4755.846277527881,-134659.9598649693,-115132.2596755535];let n,o,s,i,a=!1,u=0,c=0,l=0,h=e;if(e>171.6243769536076)return 1/0;if(h<=0){if(i=h%1,!i)return 1/0;a=(1&h?-1:1)*Math.PI/Math.sin(Math.PI*i),h=1-h}for(s=h,o=h<1?h++:(h-=u=(0|h)-1)-1,n=0;n<8;++n)l=(l+t[n])*o,c=c*o+r[n];if(i=l/c+1,s<h)i/=s;else if(s>h)for(n=0;n<u;++n)i*=h,h++;return a&&(i=a/i),i}t.exponential=i;const u={pdf:function(e,t,r){return e<0?0:0===e&&1===t?1/r:Math.exp((t-1)*Math.log(e)-e/r-c(t)-t*Math.log(r))},cdf:function(e,t,r){return e<0?0:l(t,e/r)},inv:function(e,t,r){return h(e,t)*r}};function c(e){let t=0;const r=[76.18009172947146,-86.50532032941678,24.01409824083091,-1.231739572450155,.001208650973866179,-5395239384953e-18];let n,o,s,i=1.000000000190015;for(s=(o=n=e)+5.5,s-=(n+.5)*Math.log(s);t<6;t++)i+=r[t]/++o;return Math.log(2.5066282746310007*i/n)-s}function l(e,t){const r=c(e);let n=e,o=1/e,s=o,i=t+1-e,a=1/1e-30,u=1/i,l=u,h=1;const d=-~(8.5*Math.log(e>=1?e:1/e)+.4*e+17);let p;if(t<0||e<=0)return NaN;if(t<e+1){for(;h<=d;h++)o+=s*=t/++n;return o*Math.exp(-t+e*Math.log(t)-r)}for(;h<=d;h++)p=-h*(h-e),i+=2,u=p*u+i,a=i+p/a,u=1/u,l*=u*a;return 1-l*Math.exp(-t+e*Math.log(t)-r)}function h(e,t){let r=0;const n=t-1,o=c(t);let s,i,a,u,h,d,p;if(e>=1)return Math.max(100,t+100*Math.sqrt(t));if(e<=0)return 0;for(t>1?(d=Math.log(n),p=Math.exp(n*(d-1)-o),h=e<.5?e:1-e,a=Math.sqrt(-2*Math.log(h)),s=(2.30753+.27061*a)/(1+a*(.99229+.04481*a))-a,e<.5&&(s=-s),s=Math.max(.001,t*Math.pow(1-1/(9*t)-s/(3*Math.sqrt(t)),3))):(a=1-t*(.253+.12*t),s=e<a?Math.pow(e/a,1/t):1-Math.log(1-(e-a)/(1-a)));r<12;r++){if(s<=0)return 0;if(i=l(t,s)-e,a=t>1?p*Math.exp(-(s-n)+n*(Math.log(s)-d)):Math.exp(-s+n*Math.log(s)-o),u=i/a,s-=a=u/(1-.5*Math.min(1,u*((t-1)/s-1))),s<=0&&(s=.5*(s+a)),Math.abs(a)<1e-8*s)break}return s}t.gamma=u;const d={pdf:function(e,t,r){return Math.exp(-.5*Math.log(2*Math.PI)-Math.log(r)-Math.pow(e-t,2)/(2*r*r))},cdf:function(e,t,r){return.5*(1+n((e-t)/Math.sqrt(2*r*r)))},inv:function(e,t,r){return-1.4142135623730951*r*s(2*e)+t}};t.normal=d;const p={pdf:function(e,t,r){return e>1||e<0?0:1==t&&1==r?1:t<512&&r<512?Math.pow(e,t-1)*Math.pow(1-e,r-1)/m(t,r):Math.exp((t-1)*Math.log(e)+(r-1)*Math.log(1-e)-g(t,r))},cdf:function(e,t,r){return e>1||e<0?+(e>1):E(e,t,r)},inv:function(e,t,r){return f(e,t,r)}};function m(e,t){if(!(e<=0||t<=0))return e+t>170?Math.exp(g(e,t)):a(e)*a(t)/a(e+t)}function g(e,t){return c(e)+c(t)-c(e+t)}function f(e,t,r){var n,o,s,i,a,u,l,h,d,p,m=t-1,g=r-1,f=0;if(e<=0)return 0;if(e>=1)return 1;for(t>=1&&r>=1?(s=e<.5?e:1-e,u=(2.30753+.27061*(i=Math.sqrt(-2*Math.log(s))))/(1+i*(.99229+.04481*i))-i,e<.5&&(u=-u),l=(u*u-3)/6,h=2/(1/(2*t-1)+1/(2*r-1)),d=u*Math.sqrt(l+h)/h-(1/(2*r-1)-1/(2*t-1))*(l+5/6-2/(3*h)),u=t/(t+r*Math.exp(2*d))):(n=Math.log(t/(t+r)),o=Math.log(r/(t+r)),u=e<(i=Math.exp(t*n)/t)/(d=i+(a=Math.exp(r*o)/r))?Math.pow(t*d*e,1/t):1-Math.pow(r*d*(1-e),1/r)),p=-c(t)-c(r)+c(t+r);f<10;f++){if(0===u||1===u)return u;if((u-=i=(a=(E(u,t,r)-e)/(i=Math.exp(m*Math.log(u)+g*Math.log(1-u)+p)))/(1-.5*Math.min(1,a*(m/u-g/(1-u)))))<=0&&(u=.5*(u+i)),u>=1&&(u=.5*(u+i+1)),Math.abs(i)<1e-8*u&&f>0)break}return u}function E(e,t,r){var n=0===e||1===e?0:Math.exp(c(t+r)-c(t)-c(r)+t*Math.log(e)+r*Math.log(1-e));return!(e<0||e>1)&&(e<(t+1)/(t+r+2)?n*T(e,t,r)/t:1-n*T(1-e,r,t)/r)}function T(e,t,r){var n,o,s,i,a=1,u=t+r,c=t+1,l=t-1,h=1,d=1-u*e/c;for(Math.abs(d)<1e-30&&(d=1e-30),i=d=1/d;a<=100&&(d=1+(o=a*(r-a)*e/((l+(n=2*a))*(t+n)))*d,Math.abs(d)<1e-30&&(d=1e-30),h=1+o/h,Math.abs(h)<1e-30&&(h=1e-30),i*=(d=1/d)*h,d=1+(o=-(t+a)*(u+a)*e/((t+n)*(c+n)))*d,Math.abs(d)<1e-30&&(d=1e-30),h=1+o/h,Math.abs(h)<1e-30&&(h=1e-30),i*=s=(d=1/d)*h,!(Math.abs(s-1)<3e-7));a++);return i}t.beta=p;const y={pdf:function(e,t,r){return 0===r||1===r?t*r===e?1:0:R(t,e)*Math.pow(r,e)*Math.pow(1-r,t-e)},cdf:function(e,t,r){var n;if(e<0)return 0;if(e>=t)return 1;if(r<0||r>1||t<=0)return NaN;var o=r,s=(e=Math.floor(e))+1,i=t-e,a=s+i,u=Math.exp(c(a)-c(i)-c(s)+s*Math.log(o)+i*Math.log(1-o));return n=o<(s+1)/(a+2)?u*A(o,s,i,1e-10):1-u*A(1-o,i,s,1e-10),Math.round(1/1e-10*(1-n))/(1/1e-10)}};function A(e,t,r,n){for(var o,s=0,i=1,a=1,u=1,c=0,l=0;Math.abs((a-l)/a)>n;)l=a,i=u+(o=-(t+c)*(t+r+c)*e/(t+2*c)/(t+2*c+1))*i,a=(s=a+o*s)+(o=(c+=1)*(r-c)*e/(t+2*c-1)/(t+2*c))*a,s/=u=i+o*u,i/=u,a/=u,u=1;return a/t}function R(e,t){return e>170||t>170?Math.exp(function(e,t){return N(e)-N(t)-N(e-t)}(e,t)):S(e)/S(t)/S(e-t)}function N(e){return e<0?NaN:c(e+1)}function S(e){return e<0?NaN:a(e+1)}t.binomial=y;const C={pdf:function(e,t){return e<0?0:0===e&&2===t?.5:Math.exp((t/2-1)*Math.log(e)-e/2-t/2*Math.log(2)-c(t/2))},cdf:function(e,t){return e<0?0:l(t/2,e/2)},inv:function(e,t){return 2*h(e,.5*t)}};t.chisquare=C;const v={pdf:function(e,t,r){return e<0?0:t<=2?0===e&&t<2?1/0:0===e&&2===t?1:1/m(t/2,r/2)*Math.pow(t/r,t/2)*Math.pow(e,t/2-1)*Math.pow(1+t/r*e,-(t+r)/2):t*(r/(r+e*t))/2*y.pdf((t-2)/2,(t+r-2)/2,t*e/(r+e*t))},cdf:function(e,t,r){return e<0?0:E(t*e/(t*e+r),t/2,r/2)},inv:function(e,t,r){return r/(t*(1/f(e,t/2,r/2)-1))}};t.centralF=v;const O={pdf:function(e,t,r){return e<0||t<0||r<0?0:r/t*Math.pow(e/t,r-1)*Math.exp(-Math.pow(e/t,r))},cdf:function(e,t,r){return e<0?0:1-Math.exp(-Math.pow(e/t,r))}};t.weibull=O;const M={pdf:function(e,t){return t<0||e%1!=0||e<0?0:Math.pow(t,e)*Math.exp(-t)/S(e)},cdf:function(e,t){var r=0;if(e<0)return 0;for(var n=0;r<=e;r++)n+=M.pdf(r,t);return n}};t.poisson=M;const I={pdf:function(e,t,r,n){if(e<0||e<r-(t-n))return 0;if(e>n||e>r)return 0;if(2*r>t)return 2*n>t?I.pdf(t-r-n+e,t,t-r,t-n):I.pdf(n-e,t,t-r,n);if(2*n>t)return I.pdf(r-e,t,r,t-n);if(r<n)return I.pdf(e,t,n,r);for(var o=1,s=0,i=0;i<e;i++){for(;o>1&&s<n;)o*=1-r/(t-s),s++;o*=(n-i)*(r-i)/((i+1)*(t-r-n+i+1))}for(;s<n;s++)o*=1-r/(t-s);return Math.min(1,Math.max(0,o))},cdf:function e(t,r,n,o){if(t<0||t<n-(r-o))return 0;if(t>=o||t>=n)return 1;if(2*n>r)return 2*o>r?e(r-n-o+t,r,r-n,r-o):1-I.cdf(o-t-1,r,r-n,o);if(2*o>r)return 1-I.cdf(n-t-1,r,n,r-o);if(n<o)return I.cdf(t,r,o,n);for(var s=1,i=1,a=0,u=0;u<t;u++){for(;s>1&&a<o;){var c=1-n/(r-a);i*=c,s*=c,a++}s+=i*=(o-u)*(n-u)/((u+1)*(r-n-o+u+1))}for(;a<o;a++)s*=1-n/(r-a);return Math.min(1,Math.max(0,s))}};t.hypgeom=I;const w={pdf:function(e,t){return t=t>1e100?1e100:t,1/(Math.sqrt(t)*m(.5,t/2))*Math.pow(1+e*e/t,-(t+1)/2)},cdf:function(e,t){var r=t/2;return E((e+Math.sqrt(e*e+t))/(2*Math.sqrt(e*e+t)),r,r)},inv:function(e,t){var r=f(2*Math.min(e,1-e),.5*t,.5);return r=Math.sqrt(t*(1-r)/r),e>.5?r:-r}};t.studentt=w;const F={pdf:function(e,t,r){return e<=0?0:Math.exp(-Math.log(e)-.5*Math.log(2*Math.PI)-Math.log(r)-Math.pow(Math.log(e)-t,2)/(2*r*r))},cdf:function(e,t,r){return e<0?0:.5+.5*n((Math.log(e)-t)/Math.sqrt(2*r*r))},inv:function(e,t,r){return Math.exp(-1.4142135623730951*r*s(2*e)+t)}};t.lognormal=F;const b={pdf:function(e,t,r){return e===e>>>0&&(e<0?0:R(e+t-1,t-1)*Math.pow(1-r,e)*Math.pow(r,t))},cdf:function(e,t,r){var n=0,o=0;if(e<0)return 0;for(;o<=e;o++)n+=b.pdf(o,t,r);return n}};function _(e){for(var t=0,r=e.length;--r>=0;)t+=e[r];return t}function L(e){return _(e)/e.length}function x(e){for(var t,r=L(e),n=0,o=e.length;--o>=0;)n+=(t=e[o]-r)*t;return n}function P(e,t){return x(e)/(e.length-(t?1:0))}function U(e,t){return Math.sqrt(P(e,t))}function V(e,t){var r,n=L(e),o=L(t),s=e.length,i=new Array(s);for(r=0;r<s;r++)i[r]=(e[r]-n)*(t[r]-o);return _(i)/(s-1)}t.negbin=b},function(e,t,r){"use strict";t.__esModule=!0,t.MathPlugin=void 0;var n=r(74),o=r(132),s=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class i extends s.FunctionPlugin{fact(e,t){return this.runFunction(e.args,t,this.metadata("FACT"),e=>{e=Math.trunc(e);let t=1;for(let r=1;r<=e;r++)t*=r;return t})}factdouble(e,t){return this.runFunction(e.args,t,this.metadata("FACTDOUBLE"),e=>{let t=1;for(let r=e=Math.trunc(e);r>=1;r-=2)t*=r;return t})}combin(e,t){return this.runFunction(e.args,t,this.metadata("COMBIN"),(e,t)=>t>e?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.WrongOrder):a(e=Math.trunc(e),t=Math.trunc(t)))}combina(e,t){return this.runFunction(e.args,t,this.metadata("COMBINA"),(e,t)=>(e=Math.trunc(e))+(t=Math.trunc(t))-1>=1030?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):0===e&&0===t?1:a(e+t-1,t))}gcd(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("GCD"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const i=r.arithmeticHelper.coerceNumbersCoerceRangesDropNulls(t);if(i instanceof n.CellError)return i;let a=0;for(const e of i){if(e<0)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);a=u(a,Math.trunc(e))}return a>Number.MAX_SAFE_INTEGER?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):a}))}lcm(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("LCM"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const i=r.arithmeticHelper.coerceNumbersCoerceRangesDropNulls(t);if(i instanceof n.CellError)return i;let a=1;for(const e of i){if(e<0)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);a=c(a,Math.trunc(e))}return a>Number.MAX_SAFE_INTEGER?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueLarge):a}))}mround(e,t){return this.runFunction(e.args,t,this.metadata("MROUND"),(e,t)=>0===t?0:e>0&&t<0||e<0&&t>0?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.DistinctSigns):Math.round(e/t)*t)}multinomial(e,t){return this.runFunction(e.args,t,this.metadata("MULTINOMIAL"),(function(){let e=0,t=1;for(var r=arguments.length,s=new Array(r),i=0;i<r;i++)s[i]=arguments[i];for(let r of s){if(r<0)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);r=Math.trunc(r);for(let n=1;n<=r;n++)t*=(e+n)/n;e+=r}return Math.round(t)}))}quotient(e,t){return this.runFunction(e.args,t,this.metadata("QUOTIENT"),(e,t)=>0===t?new n.CellError(n.ErrorType.DIV_BY_ZERO):Math.trunc(e/t))}seriessum(e,t){return this.runFunction(e.args,t,this.metadata("SERIESSUM"),(e,t,r,o)=>{const s=this.arithmeticHelper.manyToOnlyNumbersDropNulls(o.valuesFromTopLeftCorner());if(s instanceof n.CellError)return s;let i=0;s.reverse();for(const t of s)i*=Math.pow(e,r),i+=t;return i*Math.pow(e,t)})}sign(e,t){return this.runFunction(e.args,t,this.metadata("SIGN"),e=>e>0?1:e<0?-1:0)}sumx2my2(e,t){return this.runFunction(e.args,t,this.metadata("SUMX2MY2"),(e,t)=>{const r=e.valuesFromTopLeftCorner(),s=t.valuesFromTopLeftCorner();if(r.length!==s.length)return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const i=r.length;let a=0;for(let e=0;e<i;e++){const t=r[e],o=s[e];if(t instanceof n.CellError)return t;if(o instanceof n.CellError)return o;"number"==typeof t&&"number"==typeof o&&(a+=Math.pow(t,2)-Math.pow(o,2))}return a})}sumx2py2(e,t){return this.runFunction(e.args,t,this.metadata("SUMX2PY2"),(e,t)=>{const r=e.valuesFromTopLeftCorner(),s=t.valuesFromTopLeftCorner();if(r.length!==s.length)return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const i=r.length;let a=0;for(let e=0;e<i;e++){const t=r[e],o=s[e];if(t instanceof n.CellError)return t;if(o instanceof n.CellError)return o;"number"==typeof t&&"number"==typeof o&&(a+=Math.pow(t,2)+Math.pow(o,2))}return a})}sumxmy2(e,t){return this.runFunction(e.args,t,this.metadata("SUMXMY2"),(e,t)=>{const r=e.valuesFromTopLeftCorner(),s=t.valuesFromTopLeftCorner();if(r.length!==s.length)return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const i=r.length;let a=0;for(let e=0;e<i;e++){const t=r[e],o=s[e];if(t instanceof n.CellError)return t;if(o instanceof n.CellError)return o;"number"==typeof t&&"number"==typeof o&&(a+=Math.pow(t-o,2))}return a})}}function a(e,t){2*t>e&&(t=e-t);let r=1;for(let n=1;n<=t;n++)r*=(e-t+n)/n;return Math.round(r)}function u(e,t){for(e<t&&([e,t]=[t,e]);t>0;)[e,t]=[t,e%t];return e}function c(e,t){return 0===e||0===t?0:e*(t/u(e,t))}t.MathPlugin=i,i.implementedFunctions={FACT:{method:"fact",parameters:[{argumentType:s.FunctionArgumentType.NUMBER,minValue:0,maxValue:170}]},FACTDOUBLE:{method:"factdouble",parameters:[{argumentType:s.FunctionArgumentType.NUMBER,minValue:0,maxValue:288}]},COMBIN:{method:"combin",parameters:[{argumentType:s.FunctionArgumentType.NUMBER,minValue:0,lessThan:1030},{argumentType:s.FunctionArgumentType.NUMBER,minValue:0}]},COMBINA:{method:"combina",parameters:[{argumentType:s.FunctionArgumentType.NUMBER,minValue:0},{argumentType:s.FunctionArgumentType.NUMBER,minValue:0}]},GCD:{method:"gcd",parameters:[{argumentType:s.FunctionArgumentType.ANY}],repeatLastArgs:1},LCM:{method:"lcm",parameters:[{argumentType:s.FunctionArgumentType.ANY}],repeatLastArgs:1},MROUND:{method:"mround",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}]},MULTINOMIAL:{method:"multinomial",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}],repeatLastArgs:1,expandRanges:!0},QUOTIENT:{method:"quotient",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER}]},SERIESSUM:{method:"seriessum",parameters:[{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.NUMBER},{argumentType:s.FunctionArgumentType.RANGE}]},SIGN:{method:"sign",parameters:[{argumentType:s.FunctionArgumentType.NUMBER}]},SUMX2MY2:{method:"sumx2my2",parameters:[{argumentType:s.FunctionArgumentType.RANGE},{argumentType:s.FunctionArgumentType.RANGE}]},SUMX2PY2:{method:"sumx2py2",parameters:[{argumentType:s.FunctionArgumentType.RANGE},{argumentType:s.FunctionArgumentType.RANGE}]},SUMXMY2:{method:"sumxmy2",parameters:[{argumentType:s.FunctionArgumentType.RANGE},{argumentType:s.FunctionArgumentType.RANGE}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.ComplexPlugin=void 0;var n=r(74),o=r(132),s=r(166),i=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class a extends i.FunctionPlugin{complex(e,t){return this.runFunction(e.args,t,this.metadata("COMPLEX"),(e,t,r)=>"i"!==r&&"j"!==r?new n.CellError(n.ErrorType.VALUE,o.ErrorMessage.ShouldBeIorJ):(0,s.coerceComplexToString)([e,t],r))}imabs(e,t){return this.runFunction(e.args,t,this.metadata("IMABS"),f)}imaginary(e,t){return this.runFunction(e.args,t,this.metadata("IMAGINARY"),e=>{let[t,r]=e;return r})}imreal(e,t){return this.runFunction(e.args,t,this.metadata("IMREAL"),e=>{let[t,r]=e;return t})}imargument(e,t){return this.runFunction(e.args,t,this.metadata("IMARGUMENT"),e=>{let[t,r]=e;return 0===t&&0===r?new n.CellError(n.ErrorType.DIV_BY_ZERO):Math.atan2(r,t)})}imconjugate(e,t){return this.runFunction(e.args,t,this.metadata("IMCONJUGATE"),e=>{let[t,r]=e;return(0,s.coerceComplexToString)([t,-r])})}imcos(e,t){return this.runFunction(e.args,t,this.metadata("IMCOS"),e=>(0,s.coerceComplexToString)(h(e)))}imcosh(e,t){return this.runFunction(e.args,t,this.metadata("IMCOSH"),e=>(0,s.coerceComplexToString)(d(e)))}imcot(e,t){return this.runFunction(e.args,t,this.metadata("IMCOT"),e=>(0,s.coerceComplexToString)(l(h(e),p(e))))}imcsc(e,t){return this.runFunction(e.args,t,this.metadata("IMCSC"),e=>(0,s.coerceComplexToString)(l([1,0],p(e))))}imcsch(e,t){return this.runFunction(e.args,t,this.metadata("IMCSCH"),e=>(0,s.coerceComplexToString)(l([1,0],m(e))))}imsec(e,t){return this.runFunction(e.args,t,this.metadata("IMSEC"),e=>(0,s.coerceComplexToString)(l([1,0],h(e))))}imsech(e,t){return this.runFunction(e.args,t,this.metadata("IMSECH"),e=>(0,s.coerceComplexToString)(l([1,0],d(e))))}imsin(e,t){return this.runFunction(e.args,t,this.metadata("IMSIN"),e=>(0,s.coerceComplexToString)(p(e)))}imsinh(e,t){return this.runFunction(e.args,t,this.metadata("IMSINH"),e=>(0,s.coerceComplexToString)(m(e)))}imtan(e,t){return this.runFunction(e.args,t,this.metadata("IMTAN"),e=>(0,s.coerceComplexToString)(l(p(e),h(e))))}imdiv(e,t){return this.runFunction(e.args,t,this.metadata("IMDIV"),(e,t)=>(0,s.coerceComplexToString)(l(e,t)))}improduct(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("IMPRODUCT"),(function(){for(var e=arguments.length,t=new Array(e),o=0;o<e;o++)t[o]=arguments[o];const i=r.arithmeticHelper.coerceComplexExactRanges(t);if(i instanceof n.CellError)return i;let a=[1,0];for(const e of i)a=c(a,e);return(0,s.coerceComplexToString)(a)}))}imsum(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("IMSUM"),(function(){for(var e=arguments.length,t=new Array(e),o=0;o<e;o++)t[o]=arguments[o];const i=r.arithmeticHelper.coerceComplexExactRanges(t);if(i instanceof n.CellError)return i;let a=[0,0];for(const e of i)a=u(a,e);return(0,s.coerceComplexToString)(a)}))}imsub(e,t){return this.runFunction(e.args,t,this.metadata("IMSUB"),(e,t)=>(0,s.coerceComplexToString)(function(e,t){let[r,n]=e,[o,s]=t;return[r-o,n-s]}(e,t)))}imexp(e,t){return this.runFunction(e.args,t,this.metadata("IMEXP"),e=>(0,s.coerceComplexToString)(g(e)))}imln(e,t){return this.runFunction(e.args,t,this.metadata("IMLN"),e=>(0,s.coerceComplexToString)(E(e)))}imlog10(e,t){return this.runFunction(e.args,t,this.metadata("IMLOG10"),e=>{const[t,r]=E(e),n=Math.log(10);return(0,s.coerceComplexToString)([t/n,r/n])})}imlog2(e,t){return this.runFunction(e.args,t,this.metadata("IMLOG2"),e=>{const[t,r]=E(e),n=Math.log(2);return(0,s.coerceComplexToString)([t/n,r/n])})}impower(e,t){return this.runFunction(e.args,t,this.metadata("IMPOWER"),(e,t)=>(0,s.coerceComplexToString)(T(e,t)))}imsqrt(e,t){return this.runFunction(e.args,t,this.metadata("IMSQRT"),e=>(0,s.coerceComplexToString)(T(e,.5)))}}function u(e,t){let[r,n]=e,[o,s]=t;return[r+o,n+s]}function c(e,t){let[r,n]=e,[o,s]=t;return[r*o-n*s,r*s+o*n]}function l(e,t){let[r,n]=e,[o,s]=t;const i=Math.pow(o,2)+Math.pow(s,2),[a,u]=c([r,n],[o,-s]);return[a/i,u/i]}function h(e){let[t,r]=e;return[Math.cos(t)*Math.cosh(r),-Math.sin(t)*Math.sinh(r)]}function d(e){let[t,r]=e;return[Math.cosh(t)*Math.cos(r),Math.sinh(t)*Math.sin(r)]}function p(e){let[t,r]=e;return[Math.sin(t)*Math.cosh(r),Math.cos(t)*Math.sinh(r)]}function m(e){let[t,r]=e;return[Math.sinh(t)*Math.cos(r),Math.cosh(t)*Math.sin(r)]}function g(e){let[t,r]=e;return[Math.exp(t)*Math.cos(r),Math.exp(t)*Math.sin(r)]}function f(e){let[t,r]=e;return Math.sqrt(t*t+r*r)}function E(e){let[t,r]=e;return[Math.log(f([t,r])),Math.atan2(r,t)]}function T(e,t){const[r,n]=E(e);return g([t*r,t*n])}t.ComplexPlugin=a,a.implementedFunctions={COMPLEX:{method:"complex",parameters:[{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.NUMBER},{argumentType:i.FunctionArgumentType.STRING,defaultValue:"i"}]},IMABS:{method:"imabs",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMAGINARY:{method:"imaginary",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMREAL:{method:"imreal",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMARGUMENT:{method:"imargument",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMCONJUGATE:{method:"imconjugate",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMCOS:{method:"imcos",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMCOSH:{method:"imcosh",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMCOT:{method:"imcot",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMCSC:{method:"imcsc",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMCSCH:{method:"imcsch",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMSEC:{method:"imsec",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMSECH:{method:"imsech",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMSIN:{method:"imsin",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMSINH:{method:"imsinh",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMTAN:{method:"imtan",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMDIV:{method:"imdiv",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX},{argumentType:i.FunctionArgumentType.COMPLEX}]},IMPRODUCT:{method:"improduct",parameters:[{argumentType:i.FunctionArgumentType.ANY}],repeatLastArgs:1},IMSUM:{method:"imsum",parameters:[{argumentType:i.FunctionArgumentType.ANY}],repeatLastArgs:1},IMSUB:{method:"imsub",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX},{argumentType:i.FunctionArgumentType.COMPLEX}]},IMEXP:{method:"imexp",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMLN:{method:"imln",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMLOG10:{method:"imlog10",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMLOG2:{method:"imlog2",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]},IMPOWER:{method:"impower",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX},{argumentType:i.FunctionArgumentType.NUMBER}]},IMSQRT:{method:"imsqrt",parameters:[{argumentType:i.FunctionArgumentType.COMPLEX}]}}},function(e,t,r){"use strict";t.__esModule=!0,t.StatisticalAggregationPlugin=void 0,r(2);var n=r(74),o=r(132),s=r(141),i=r(262),a=r(165);
/**
 * @license
 * Copyright (c) 2023 Handsoncode. All rights reserved.
 */
class u extends a.FunctionPlugin{avedev(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("AVEDEV"),(function(){for(var e=arguments.length,t=new Array(e),o=0;o<e;o++)t[o]=arguments[o];const s=r.arithmeticHelper.coerceNumbersExactRanges(t);if(s instanceof n.CellError)return s;if(0===s.length)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const a=(0,i.mean)(s);return s.reduce((e,t)=>e+Math.abs(t-a),0)/s.length}))}devsq(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("DEVSQ"),(function(){for(var e=arguments.length,t=new Array(e),o=0;o<e;o++)t[o]=arguments[o];const s=r.arithmeticHelper.coerceNumbersExactRanges(t);return s instanceof n.CellError?s:0===s.length?0:(0,i.sumsqerr)(s)}))}geomean(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("GEOMEAN"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const a=r.arithmeticHelper.coerceNumbersExactRanges(t);if(a instanceof n.CellError)return a;if(0===a.length)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.OneValue);for(const e of a)if(e<=0)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);return(0,i.geomean)(a)}))}harmean(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("HARMEAN"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const i=r.arithmeticHelper.coerceNumbersExactRanges(t);if(i instanceof n.CellError)return i;if(0===i.length)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.OneValue);for(const e of i)if(e<=0)return new n.CellError(n.ErrorType.NUM,o.ErrorMessage.ValueSmall);return i.length/i.reduce((e,t)=>e+1/t,0)}))}correl(e,t){return this.runFunction(e.args,t,this.metadata("CORREL"),(e,t)=>{if(e.numberOfElements()!==t.numberOfElements())return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const r=c(e,t);if(r instanceof n.CellError)return r;return r[0].length<=1?new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues):(0,i.corrcoeff)(r[0],r[1])})}rsq(e,t){return this.runFunction(e.args,t,this.metadata("RSQ"),(e,t)=>{if(e.numberOfElements()!==t.numberOfElements())return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const r=c(e,t);if(r instanceof n.CellError)return r;return r[0].length<=1?new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues):Math.pow((0,i.corrcoeff)(r[0],r[1]),2)})}covariancep(e,t){return this.runFunction(e.args,t,this.metadata("COVARIANCE.P"),(e,t)=>{if(e.numberOfElements()!==t.numberOfElements())return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const r=c(e,t);if(r instanceof n.CellError)return r;const s=r[0].length;return s<1?new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.OneValue):1===s?0:(0,i.covariance)(r[0],r[1])*(s-1)/s})}covariances(e,t){return this.runFunction(e.args,t,this.metadata("COVARIANCE.S"),(e,t)=>{if(e.numberOfElements()!==t.numberOfElements())return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const r=c(e,t);if(r instanceof n.CellError)return r;return r[0].length<=1?new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues):(0,i.covariance)(r[0],r[1])})}ztest(e,t){return this.runFunction(e.args,t,this.metadata("Z.TEST"),(e,t,r)=>{const s=this.arithmeticHelper.manyToExactNumbers(e.valuesFromTopLeftCorner());if(s instanceof n.CellError)return s;const a=s.length;if(void 0===r){if(a<2)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues);r=(0,i.stdev)(s,!0)}return a<1?new n.CellError(n.ErrorType.NA,o.ErrorMessage.OneValue):0===r?new n.CellError(n.ErrorType.DIV_BY_ZERO):1-i.normal.cdf(((0,i.mean)(s)-t)/(r/Math.sqrt(a)),0,1)})}ftest(e,t){return this.runFunction(e.args,t,this.metadata("F.TEST"),(e,t)=>{const r=this.arithmeticHelper.manyToExactNumbers(e.valuesFromTopLeftCorner()),o=this.arithmeticHelper.manyToExactNumbers(t.valuesFromTopLeftCorner());if(r instanceof n.CellError)return r;if(o instanceof n.CellError)return o;if(r.length<=1||o.length<=1)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const s=(0,i.variance)(r,!0),a=(0,i.variance)(o,!0);if(0===s||0===a)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const u=s/a,c=i.centralF.cdf(u,r.length-1,o.length-1);return 2*Math.min(c,1-c)})}steyx(e,t){return this.runFunction(e.args,t,this.metadata("STEYX"),(e,t)=>{if(e.numberOfElements()!==t.numberOfElements())return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const r=c(e,t);if(r instanceof n.CellError)return r;const s=r[0].length;return s<=2?new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.ThreeValues):Math.sqrt(((0,i.sumsqerr)(r[0])-Math.pow((0,i.covariance)(r[0],r[1])*(s-1),2)/(0,i.sumsqerr)(r[1]))/(s-2))})}slope(e,t){return this.runFunction(e.args,t,this.metadata("SLOPE"),(e,t)=>{if(e.numberOfElements()!==t.numberOfElements())return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const r=c(e,t);if(r instanceof n.CellError)return r;const s=r[0].length;return s<=1?new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues):(0,i.covariance)(r[0],r[1])*(s-1)/(0,i.sumsqerr)(r[1])})}chisqtest(e,t){return this.runFunction(e.args,t,this.metadata("CHISQ.TEST"),(e,t)=>{const r=e.height(),s=e.width();if(t.height()!==r||t.width()!==s)return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);const a=c(e,t);if(a instanceof n.CellError)return a;if(a[0].length<=1)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues);let u=0;for(let e=0;e<a[0].length;e++){if(0===a[1][e])return new n.CellError(n.ErrorType.DIV_BY_ZERO);u+=Math.pow(a[0][e]-a[1][e],2)/a[1][e]}return u<0?new n.CellError(n.ErrorType.NUM,o.ErrorMessage.NaN):1-i.chisquare.cdf(u,r>1&&s>1?(r-1)*(s-1):r*s-1)})}ttest(e,t){return this.runFunction(e.args,t,this.metadata("T.TEST"),(e,t,r,s)=>{const a=this.arithmeticHelper.manyToExactNumbers(e.valuesFromTopLeftCorner()),u=this.arithmeticHelper.manyToExactNumbers(t.valuesFromTopLeftCorner());if(a instanceof n.CellError)return a;if(u instanceof n.CellError)return u;const c=a.length,l=u.length;if(1===s){if(l!==c)return new n.CellError(n.ErrorType.NA,o.ErrorMessage.EqualLength);if(c<=1)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues);const e=Array(c);for(let t=0;t<c;t++)e[t]=a[t]-u[t];const t=(0,i.stdev)(e,!0);if(0===t)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const s=Math.abs(Math.sqrt(c)*(0,i.mean)(e)/t);return r*(1-i.studentt.cdf(s,c-1))}if(2===s){if(c<=1||l<=1)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues);const e=((0,i.sumsqerr)(a)+(0,i.sumsqerr)(u))/(c+l-2);if(0===e)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const t=Math.abs(((0,i.mean)(a)-(0,i.mean)(u))/Math.sqrt(e*(1/c+1/l)));return r*(1-i.studentt.cdf(t,c+l-2))}{if(c<=1||l<=1)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.TwoValues);const e=(0,i.variance)(a,!0),t=(0,i.variance)(u,!0);if(0===e&&0===t)return new n.CellError(n.ErrorType.DIV_BY_ZERO);const s=Math.abs(((0,i.mean)(a)-(0,i.mean)(u))/Math.sqrt(e/c+t/l)),h=Math.pow(e/c+t/l,2)/(Math.pow(e/c,2)/(c-1)+Math.pow(t/l,2)/(l-1));return r*(1-i.studentt.cdf(s,h))}})}skew(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("SKEW"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const a=r.arithmeticHelper.coerceNumbersExactRanges(t);if(a instanceof n.CellError)return a;const u=a.length;if(u<3)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.ThreeValues);const c=(0,i.mean)(a),l=(0,i.stdev)(a,!0);return 0===l?new n.CellError(n.ErrorType.DIV_BY_ZERO):a.reduce((e,t)=>e+Math.pow((t-c)/l,3),0)*u/(u-1)/(u-2)}))}skewp(e,t){var r=this;return this.runFunction(e.args,t,this.metadata("SKEW.P"),(function(){for(var e=arguments.length,t=new Array(e),s=0;s<e;s++)t[s]=arguments[s];const a=r.arithmeticHelper.coerceNumbersExactRanges(t);if(a instanceof n.CellError)return a;const u=a.length;if(u<3)return new n.CellError(n.ErrorType.DIV_BY_ZERO,o.ErrorMessage.ThreeValues);const c=(0,i.mean)(a),l=(0,i.stdev)(a,!1);return 0===l?new n.CellError(n.ErrorType.DIV_BY_ZERO):a.reduce((e,t)=>e+Math.pow((t-c)/l,3),0)/u}))}}function c(e,t){const r=e.iterateValuesFromTopLeftCorner(),o=t.iterateValuesFromTopLeftCorner();let i,a;const u=[],c=[];for(;i=r.next(),a=o.next(),!i.done&&!a.done;){const e=i.value,t=a.value;if(e instanceof n.CellError)return e;if(t instanceof n.CellError)return t;(0,s.isExtendedNumber)(e)&&(0,s.isExtendedNumber)(t)&&(u.push((0,s.getRawValue)(e)),c.push((0,s.getRawValue)(t)))}return[u,c]}t.StatisticalAggregationPlugin=u,u.implementedFunctions={AVEDEV:{method:"avedev",parameters:[{argumentType:a.FunctionArgumentType.ANY}],repeatLastArgs:1},DEVSQ:{method:"devsq",parameters:[{argumentType:a.FunctionArgumentType.ANY}],repeatLastArgs:1},GEOMEAN:{method:"geomean",parameters:[{argumentType:a.FunctionArgumentType.ANY}],repeatLastArgs:1},HARMEAN:{method:"harmean",parameters:[{argumentType:a.FunctionArgumentType.ANY}],repeatLastArgs:1},CORREL:{method:"correl",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},RSQ:{method:"rsq",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},"COVARIANCE.P":{method:"covariancep",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},"COVARIANCE.S":{method:"covariances",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},"Z.TEST":{method:"ztest",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.NUMBER},{argumentType:a.FunctionArgumentType.NUMBER,optionalArg:!0}]},"F.TEST":{method:"ftest",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},STEYX:{method:"steyx",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},SLOPE:{method:"slope",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},"CHISQ.TEST":{method:"chisqtest",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE}]},"T.TEST":{method:"ttest",parameters:[{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.RANGE},{argumentType:a.FunctionArgumentType.INTEGER,minValue:1,maxValue:2},{argumentType:a.FunctionArgumentType.INTEGER,minValue:1,maxValue:3}]},SKEW:{method:"skew",parameters:[{argumentType:a.FunctionArgumentType.ANY}],repeatLastArgs:1},"SKEW.P":{method:"skewp",parameters:[{argumentType:a.FunctionArgumentType.ANY}],repeatLastArgs:1}},u.aliases={COVAR:"COVARIANCE.P",FTEST:"F.TEST",PEARSON:"CORREL",ZTEST:"Z.TEST",CHITEST:"CHISQ.TEST",TTEST:"T.TEST",COVARIANCEP:"COVARIANCE.P",COVARIANCES:"COVARIANCE.S",SKEWP:"SKEW.P"}}]).default}));